/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CActiveSink.h"
#include "CPassiveSource.h"
#include "danbug.h"

CActiveSink::CActiveSink(void)
{
	mSource = 0;
	mTerminate = 0;
	mConnectionType |= (CONN_ACTIVE | CONN_SINK);
}

CActiveSink::~CActiveSink(void)
{
	mSource = 0;
}

void
CActiveSink::SetPassiveSource(CConnection *s)
{
	mSource = dynamic_cast<CPassiveSource*>(s);
	if (!mSource)
		throw BadConnection("Bad connection",0);
}

void
CActiveSink::Connect(CConnection *c)
{
	SetPassiveSource(c);
}

void
CActiveSink::Run(void)
{
	assert(mSource);
	while ((!mTerminate) && ProcessData(mSource)) Yield();
}

void
CActiveSink::Initialize(void)
{
	mTerminate = 0;

	assert(mSource);

	GetStreamInfoFromSource();
	GetStreamFormatFromSource();
}

void
CActiveSink::GetStreamFormatFromSource(void)
{
	CStreamInfoObject &o = mSource->GetStreamInfoObject();

 dprintf("calling mStreamInfoObject.SetStreamFormat(2)");
	mStreamInfoObject.SetStreamFormat(o.GetStreamFormat(),
										o.GetStreamFormatSize());
}

void
CActiveSink::GetStreamInfoFromSource(void)
{
	CStreamInfoObject &o = mSource->GetStreamInfoObject();


	AVISTREAMINFO	i = o.GetStreamInfo();
	mStreamInfoObject.SetStreamInfo(i);
}

void
CActiveSink::Go(void)
{
	Start();
}

void
CActiveSink::Terminate(void)
{
	// terminate cant be called from the sink. Return 0 from
	// ProcessData instead
	assert(Self() != mThreadID);

	mTerminate = 1;
	Join();
}

void
CActiveSink::Wait(void)
{
	Join();
}





