/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CActiveSource.h"
#include "CPassiveSink.h"
#include "danbug.h"

CActiveSource::CActiveSource(void)
{
	mSink = 0;
	mTerminate = 0;
	mConnectionType |= (CONN_ACTIVE | CONN_SOURCE);
}

CActiveSource::~CActiveSource(void)
{
	mSink = 0;
}

void
CActiveSource::SetPassiveSink(CConnection *s)
{
	mSink = dynamic_cast<CPassiveSink*>(s);
	if (!mSink)
		throw BadConnection("Bad connection",0);
}

void
CActiveSource::Connect(CConnection *c)
{
	SetPassiveSink(c);
}

void
CActiveSource::Run(void)
{
	assert(mSink);
	while ((!mTerminate) && ProcessData(mSink)) Yield();
}

void
CActiveSource::Initialize(void)
{
 dprintf("CActiveSource::Initialize\n");
	mTerminate = 0;
	_SetInfo();

	assert(mSink);
	CStreamInfoObject &o = mSink->GetStreamInfoObject();

	AVISTREAMINFO i = mStreamInfoObject.GetStreamInfo();
	o.SetStreamInfo(i);
	o.SetStreamFormat(mStreamInfoObject.GetStreamFormat(),
						mStreamInfoObject.GetStreamFormatSize());
 dprintf("CActiveSource::Initialize done\n");
}


void
CActiveSource::Go(void)
{
	Start();
}

void
CActiveSource::Terminate(void)
{
	// we can't call this from ourself
	assert(Self() != mThreadID);

	mTerminate = 1;
	Join();
}

void
CActiveSource::Wait(void)
{
	Join();
}


