/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAutoLinker.h"

#include "CActiveSource.h"

#include "CPassiveSource.h"

#include "CActiveSink.h"

#include "CPassiveSink.h"



CAutoLinker::~CAutoLinker(void)

{

}





void

CAutoLinker::LinkComponents(ComponentList_t l)

{

	assert(!mMegaNodeList.size());

	assert(!mNodeList.size());

	assert(!mConnectionList.size());

	assert(!mComponentList.size());

	mMegaNodeList.erase(mMegaNodeList.begin(),mMegaNodeList.end());

	mNodeList.erase(mNodeList.begin(),mNodeList.end());

	mConnectionList.erase(mConnectionList.begin(),mConnectionList.end());

	mComponentList = l;

	

	// every component has a meganode

	MegaNode_t	mn;

	mMegaNodeList.insert(mMegaNodeList.begin(),mComponentList.size(),mn);



	vector<CComponent::ConnectionTypeList_t> component_connection_types;

	

	int x,x1,x2;

	int y1,y2;

	int cl_counter = 0; // connection list counter



	for (x = 0; x < mComponentList.size(); ++x)

		component_connection_types.push_back(mComponentList[x]->GetSupportedTypes());



	// generate and attach components

	for (x1 = 0; x1 < mComponentList.size(); ++x1)

		{

		for (x2 = x1+1;x2 < mComponentList.size(); ++x2)

			{

			for (y1 = 0; y1 < component_connection_types[x1].size(); ++y1)

				{

				for (y2 = 0; y2 < component_connection_types[x2].size(); ++y2)

					{

					CConnection::Connection_t	t1,t2;

					if (CompatibleConnections(

						t1 = component_connection_types[x1][y1],

						t2 = component_connection_types[x2][y2]))

						{

						CConnection *c1 = mComponentList[x1]->CreateConnection(t1);

						CConnection *c2 = mComponentList[x2]->CreateConnection(t2);

						c1->Connect(c2);

						c2->Connect(c1);



						mConnectionList.push_back(c1);

						mConnectionList.push_back(c2);



						Node_t		n1,n2;

						n1.mMegaNodeIndex = x1;

						n1.mMySink = n1.mMySource = -1;

						n2.mMegaNodeIndex = x2;

						n2.mMySink = n2.mMySource = -1;

						if (c1->GetConnectionType() & CONN_SOURCE)

							{

							n1.mMySink = cl_counter + 1;

							n2.mMySource = cl_counter;

							}

						else

							{

							n1.mMySource = cl_counter + 1;

							n2.mMySink = cl_counter;

							}

						mNodeList.push_back(n1);

						mNodeList.push_back(n2);



						MegaNode_t	mn1 = mMegaNodeList[x1];

						MegaNode_t	mn2 = mMegaNodeList[x2];

						mn1.mNodeIndexList.push_back(cl_counter);

						mn2.mNodeIndexList.push_back(cl_counter+1);

						mMegaNodeList[x1] = mn1;

						mMegaNodeList[x2] = mn2;

						cl_counter+=2;



						component_connection_types[x1][y1] = 0;

						component_connection_types[x2][y2] = 0;

						}

					}

				}

			}

		}



	// verify that graph is fully connected

	UnmarkMegaNodeList(mMegaNodeList);

	UnmarkNodeList(mNodeList);



	TraverseAndMarkGraph(mMegaNodeList,mNodeList);

	if (!AllNodesVisited(mNodeList))

			throw AutoLinkerError("Graph is not fully connected",0);

}





int

CAutoLinker::CompatibleConnections(CConnection::Connection_t t1,

								   CConnection::Connection_t t2)

{

	if (!t1 || !t2) return 0;



	CConnection::Connection_t t = (t1 ^ t2);

	CConnection::Connection_t st = t & (CONN_SEEKABLE | CONN_NEEDSSEEKABLE);



	if (st == CONN_NEEDSSEEKABLE)

		return 0;



	// ignore random access for next test

	t &= ~(CONN_SEEKABLE | CONN_NEEDSSEEKABLE);



	return (t == (CONN_ACTIVE | CONN_PASSIVE | CONN_SINK | CONN_SOURCE));

}



void

CAutoLinker::Wait(void)

{

	WaitAction wa;

	Traverse(mMegaNodeList,mNodeList,wa);

}



int

CAutoLinker::Configure(CWnd *wnd)

{

	ConfigureAction ca(wnd);

	return Traverse(mMegaNodeList,mNodeList,ca);

}



void

CAutoLinker::Initiate(void)

{

	InitializeAction ia;

	Traverse(mMegaNodeList,mNodeList,ia);



	GoAction ga;

	Traverse(mMegaNodeList,mNodeList,ga);

}



void

CAutoLinker::Terminate(void)

{

	TerminateAction ta;

	Traverse(mMegaNodeList,mNodeList,ta);

}



void

CAutoLinker::Cleanup(void)

{

	CleanupAction ca;

	Traverse(mMegaNodeList,mNodeList,ca);

}

