/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAviAReader.h"

#include "CPassiveSink.h"

#include "CGUIController.h"

#include <stack_aids.h>



CAviAReader::CAviAReader(CGUIController *gui)

: CComponent(gui)

{

	AVIFileInit();

	mFile = 0;

	mFileOpen = 0;

	mFileName = AfxGetApp()->GetProfileString("CAviAReader","FileName","");

}



void

CAviAReader::Initialize(void)

{

	assert(!mFile);

	assert(!mFileOpen);

	int err;

	if (err = AVIFileOpen(&mFile,mFileName,OF_READ,NULL))

		throw FileError("Error opening input file",err);

	mFileOpen = 1;

}



void

CAviAReader::Cleanup(void)

{

	AfxGetApp()->WriteProfileString("CAviAReader","FileName",mFileName);



	if (mFileOpen)

		AVIFileRelease(mFile);

	mFileOpen = 0;

	mFile = 0;

}



CAviAReader::~CAviAReader(void)

{

	ConnectionList_t::iterator i;

	for (i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		delete(*i);

	mProducedConnections.erase(mProducedConnections.begin(),mProducedConnections.end());



	AVIFileExit();

}



void	

CAviAReader::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	ConnectionList_t::iterator i;

	for(i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		{

		if ((*i) == c)

			{

			delete (*i);

			mProducedConnections.erase(i);

			break;

			}

		}

}





CComponent::ConnectionTypeList_t	

CAviAReader::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_AUDIO);

	return l;

}



nmr_string 

CAviAReader::GetName(void) const

{

	return "Avi Reader";

}



CConnection *

CAviAReader::CreateConnection(CConnection::Connection_t t)

{

	CConnection::Connection_t	minimum_requirements = (CONN_ACTIVE | CONN_SOURCE);

	

	if ((t & minimum_requirements) != minimum_requirements)

		return 0;

	

	CConnection *c = 0;



	if (t & CONN_AUDIO)

		c = new CAviAReaderAudioStream(this,0,mGUIController);

	if (t & CONN_VIDEO)

		c = new CAviAReaderVideoStream(this,0,mGUIController);

	

	if (c)

		mProducedConnections.push_back(c);

	return c;

}



PAVIFILE

CAviAReader::GetAviFile(void)

{

	return mFile;

}





///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////



CAviAReaderBaseStream::CAviAReaderBaseStream(CAviAReader *comp,

							int which,CGUIController *gui)

: CActiveSource()

{

	mComponent		= comp;

	mAviFile		= 0;

	mWhichStream	= which;

	mStreamOpen		= 0;

	mCurrentFrame	= 0;

	mGUIController	= gui;

}



CAviAReaderBaseStream::~CAviAReaderBaseStream(void)

{

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;

}



void

CAviAReaderBaseStream::Initialize(void)

{

	CActiveSource::Initialize();

	mCurrentFrame = 0;

}



void

CAviAReaderBaseStream::Cleanup(void)

{

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;



	CActiveSource::Cleanup();

}



void	

CAviAReaderBaseStream::_SetInfo(void)

{

	int err;



	assert(mAviStream);



	AVISTREAMINFO i;

	if (err = AVIStreamInfo(mAviStream,&i,sizeof(i)))

		throw StreamError("Error getting input stream info",err);

	mStreamInfoObject.SetStreamInfo(i);



	long siz;

	if (err = AVIStreamReadFormat(mAviStream,0,NULL,&siz))

		throw StreamError("Error getting input format size",err);



	char *f = new char[siz];

	stack_ArrayDeleter<char> sad(f);



	if (err = AVIStreamReadFormat(mAviStream,0,f,&siz))

		throw StreamError("Error getting input format",err);

	mStreamInfoObject.SetStreamFormat(f,siz);

}



/////////////////////////////////////////////////

//////////////////////////////////////////////////

//////////////////////////////////////////////////

int		

CAviAReaderVideoStream::ProcessData(CPassiveSink *s)

{

	long datasize;

	int err;

	if (err = AVIStreamRead(mAviStream,mCurrentFrame,1,NULL,0,&datasize,NULL))

		{

		s->DataComplete();

		return 0;

		}



	if (datasize > 0)

		{

		U32 keyframe = 

			(AVIStreamFindSample(mAviStream,mCurrentFrame,FIND_KEY | FIND_NEXT) == mCurrentFrame);



		CPassiveSinkDataReference *ref = s->GetDataReference(datasize);



		if (!ref)

			{

			s->DataComplete();

			return 0;

			}

		ref->SetFlags(keyframe);

		AVIStreamRead(mAviStream,mCurrentFrame,1,ref->GetWritePtr(),datasize,NULL,NULL);

		mCurrentFrame++;

		s->ReturnDataReference(ref);

#ifndef NDEBUG

		(mGUIController->Trace(this)) << "Reading video frame " 

			<< (mCurrentFrame - 1) << CMessageStream::show;

#endif

		return 1;

		}

	else

		{

		s->DataComplete();

		return 0;

		}

}





void

CAviAReaderVideoStream::Initialize(void)

{

	int err;



	mAviFile = mComponent->GetAviFile();

	assert(mAviFile);



	if (err = AVIFileGetStream(mAviFile,&mAviStream,streamtypeVIDEO,mWhichStream))

		throw StreamError("Error opening stream",err);

	mStreamOpen = 1;

	mCurrentFrame = 0;

	

	CAviAReaderBaseStream::Initialize();

}



int		

CAviAReaderAudioStream::ProcessData(CPassiveSink *s)

{



	long datasize,samples;

	int err;

	assert(mStreamOpen);



	if (err = AVIStreamRead(mAviStream,mCurrentFrame,AVISTREAMREAD_CONVENIENT,NULL,0,&datasize,&samples))

		{

		s->DataComplete();

		return 0;

		}



	if (datasize > 0)

		{

		CPassiveSinkDataReference *ref = s->GetDataReference(datasize);

		if (!ref)

			{

			s->DataComplete();

			return 0;

			}

		AVIStreamRead(mAviStream,mCurrentFrame,AVISTREAMREAD_CONVENIENT,ref->GetWritePtr(),datasize,NULL,NULL);

		mCurrentFrame += samples;

		s->ReturnDataReference(ref);

#ifndef NDEBUG

		(mGUIController->Trace(this)) << "Reading audio samples "

			<< (mCurrentFrame - samples) << " through " << mCurrentFrame

			<< CMessageStream::show;

#endif

		return 1;

		}

	else

		{

		s->DataComplete();

		return 0;

		}

}





void

CAviAReaderAudioStream::Initialize(void)

{

	int err;



	mAviFile = mComponent->GetAviFile();

	assert(mAviFile);



	if (err = AVIFileGetStream(mAviFile,&mAviStream,streamtypeAUDIO,mWhichStream))

		throw StreamError("Error opening stream",err);

	mStreamOpen = 1;

	mCurrentFrame = 0;

	

	CAviAReaderBaseStream::Initialize();

}

