/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CAVIAREADER_H_

#define CAVIAREADER_H_



#include "CComponent.h"

#include "CActiveSource.h"

#include <vector>
using namespace std;

#include <CError.h>

#include <windows.h>

#include <vfw.h>





class CAviAReader: public CComponent

{

protected:

	PAVIFILE	mFile;

	nmr_string	mFileName;

	int			mFileOpen;

	typedef vector<CConnection*> ConnectionList_t;

	ConnectionList_t	mProducedConnections;



public:

	CAviAReader(CGUIController *gui);

	~CAviAReader(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;

	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);



			PAVIFILE	GetAviFile(void);



	CERROR_SUBCLASS(FileError);

};



/////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////



class CAviAReaderBaseStream: public CActiveSource

{

protected:

	CGUIController	*mGUIController;

	PAVISTREAM		mAviStream;

	PAVIFILE		mAviFile;

	int				mWhichStream;

	int				mStreamOpen;

	int				mCurrentFrame;

	CAviAReader		*mComponent;



	CAviAReaderBaseStream(CAviAReader *comp,int which,CGUIController *gui);

	virtual	void	_SetInfo(void);



public:

	~CAviAReaderBaseStream(void);

	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);



	friend CAviAReader;

	CERROR_SUBCLASS(StreamError);

};



//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////



class CAviAReaderAudioStream: public CAviAReaderBaseStream

{

protected:

	CAviAReaderAudioStream(CAviAReader *comp,int which,CGUIController *gui)

		: CAviAReaderBaseStream(comp,which,gui)

	{ mConnectionType = (CONN_ACTIVE | CONN_SOURCE | CONN_AUDIO);}

	virtual	int		ProcessData(CPassiveSink *s);

	

public:

	virtual	void	Initialize(void);



	friend CAviAReader;

};



class CAviAReaderVideoStream: public CAviAReaderBaseStream

{

protected:

	CAviAReaderVideoStream(CAviAReader *comp,int which,CGUIController *gui)

		: CAviAReaderBaseStream(comp,which,gui) 

	{ mConnectionType = (CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO); }

	virtual	int		ProcessData(CPassiveSink *s);



public:

	virtual	void	Initialize(void);



	friend CAviAReader;

};



#endif