/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAviPWriter.h"

#include "AviUtils.h"

#include "CGUIController.h"





CAviPWriter::CAviPWriter(int interleave,

						 CGUIController *gui)

						 : CComponent(gui)

{

	AVIFileInit();

	mFileOpen = 0;

	mFile = 0;

	mInterleave = interleave;

	mFileName = AfxGetApp()->GetProfileString("CAviPWriter","FileName","");

	mRecalculateFramerate = AfxGetApp()->GetProfileInt("CAviPWriter","RecalculateFramerate",0);

}



void

CAviPWriter::Initialize(void)

{

	int err;



	remove(mFileName);

	if (err = AVIFileOpen(&mFile,mFileName,OF_CREATE | OF_WRITE,NULL))

		throw FileError("Error creating output file",err);

	mFileOpen = 1;

}



void

CAviPWriter::Cleanup(void)

{

	AfxGetApp()->WriteProfileString("CAviPWriter","FileName",mFileName);

	AfxGetApp()->WriteProfileInt("CAviPWriter","RecalculateFramerate",mRecalculateFramerate);



	if (mFileOpen)

		{

		AVIFileRelease(mFile);

		if (mRecalculateFramerate)

			RecalculateFramerate(mFileName);

		}

	mFileOpen = 0;

	mFile = 0;

}



CAviPWriter::~CAviPWriter(void)

{

	ConnectionList_t::iterator i;

	for (i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		delete (*i);

	mProducedConnections.erase(mProducedConnections.begin(),mProducedConnections.end());



	AVIFileExit();

}



void	

CAviPWriter::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	ConnectionList_t::iterator i;

	for(i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		{

		if ((*i) == c)

			{

			delete (*i);

			mProducedConnections.erase(i);

			break;

			}

		}

}



CComponent::ConnectionTypeList_t

CAviPWriter::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;



	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_AUDIO);



	return l;

}



nmr_string

CAviPWriter::GetName(void) const

{

	return "Avi Writer";

}



CConnection *

CAviPWriter::CreateConnection(CConnection::Connection_t t)

{

	CConnection::Connection_t	minimum_requirements = (CONN_SINK | CONN_PASSIVE);



	if ((t & minimum_requirements) != minimum_requirements)

		return 0;



	CConnection *c = 0;



	if (t & CONN_AUDIO)

		c =  new CAviPWriterAudioStream(this,mGUIController);

	if (t & CONN_VIDEO)

		c =  new CAviPWriterVideoStream(this,mGUIController);

	

	mProducedConnections.push_back(c);



	return c;

}





PAVIFILE

CAviPWriter::GetAviFile(void)

{

	return mFile;

}



/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////



CAviPWriterBaseStream::CAviPWriterBaseStream(CAviPWriter *comp,

											 CGUIController *gui)

{

	mGUIController = gui;

	mComponent = comp;

	mAviFile = 0;

	mStreamOpen = 0;

	mOutputFrameCounter = 0;

	mDataReferenceData = 0;

	mDataReferenceSize = 0;

}



CAviPWriterBaseStream::~CAviPWriterBaseStream(void)

{

	_CloseStream();

	delete [] mDataReferenceData;

}



void

CAviPWriterBaseStream::Initialize(void)

{

	CPassiveSink::Initialize();



	mTerminate = 0;

	mOutputFrameCounter = 0;

	assert(!mStreamOpen);

	mAviFile = mComponent->GetAviFile();

	assert(mAviFile);

	_OpenStream();

}



void

CAviPWriterBaseStream::Cleanup(void)

{

	_CloseStream();

}



void

CAviPWriterBaseStream::Terminate(void)

{

	mTerminate = 1;

}



void

CAviPWriterBaseStream::_OpenStream(void)

{

	int err;

	AVISTREAMINFO i = mStreamInfoObject.GetStreamInfo();



	if (err = AVIFileCreateStream(mAviFile,&mAviStream,&i))

		throw StreamError("Error creating output stream",err);



	const char *f	= mStreamInfoObject.GetStreamFormat();

	size_t s		= mStreamInfoObject.GetStreamFormatSize();



	if (err = AVIStreamSetFormat(mAviStream,0,(void *)f,s))

		throw StreamError("Error setting stream format",err);



	mStreamOpen = 1;

}



void

CAviPWriterBaseStream::_CloseStream(void)

{

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;

}



CPassiveSinkDataReference *

CAviPWriterBaseStream::GetDataReference(size_t s)

{

	if (mTerminate) return 0;

	if (mDataReferenceSize <= s)

		{

		delete [] mDataReferenceData;

		mDataReferenceSize = s;

		mDataReferenceData = new char[s];

		}

	mDataReference.Set(mDataReferenceData,s,0,0);

	return &mDataReference;

}



void	

CAviPWriterBaseStream::DataComplete(void)

{

	_CloseStream();

	mTerminate = 1;

}





/////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////



void

CAviPWriterVideoStream::_ProcessData(const CPassiveSinkDataReference *r)

{

	// write the data

	assert(r);

	if (!mStreamOpen) return;



	char		*data = r->GetWritePtr();

	long		len = r->GetSize();

	U32			key_frame = r->GetFlags();



#ifndef NDEBUG

	(mGUIController->Trace(this)) << "Writing video frame " 

		<< mOutputFrameCounter << CMessageStream::show;

#endif

	int err;

#if 1

	if (err = AVIStreamWrite(mAviStream,mOutputFrameCounter,1,(void *)data,len,

		key_frame ? AVIIF_KEYFRAME : 0,NULL,NULL))

		throw StreamError("Could not write to video stream",err);

#endif

	mOutputFrameCounter++;

}



void	

CAviPWriterVideoStream::ReturnDataReference(CPassiveSinkDataReference *r)

{

	_ProcessData(r);

}



int

CAviPWriterVideoStream::ConsumeDataNow(const CPassiveSinkDataReference *r)

{

	_ProcessData(r);

	return 1;

}



void

CAviPWriterAudioStream::_ProcessData(const CPassiveSinkDataReference *r)

{

	// write the data

 dprintf("CAviPWriterAudioStream::_ProcessData\n");
	assert(r);

	if (!mStreamOpen) return;



	char		*data = r->GetWritePtr();

	long		len = r->GetSize();



	WAVEFORMATEX	*w = (WAVEFORMATEX *)mStreamInfoObject.GetStreamFormat();

	int	num_samples;



	switch(w->wBitsPerSample)

		{

		case 3:

		case 4:

			// duck specific

			num_samples = len / 2048;

			break;

		default:

			num_samples = (len * 8) / (w->nChannels * w->wBitsPerSample);

			break;

		}



#ifndef NDEBUG

	(mGUIController->Trace(this)) << "Writing audio samples " 

		<< mOutputFrameCounter << " through " << (mOutputFrameCounter + num_samples)

		<< CMessageStream::show;

#endif

	int err;

#if 1

	if (err = AVIStreamWrite(mAviStream,mOutputFrameCounter,num_samples,(void *)data,len,0,NULL,NULL))

		throw StreamError("Could not write to audio stream",err);

#endif

	mOutputFrameCounter += num_samples;

}



void	

CAviPWriterAudioStream::ReturnDataReference(CPassiveSinkDataReference *r)

{

	_ProcessData(r);	

}



int

CAviPWriterAudioStream::ConsumeDataNow(const CPassiveSinkDataReference *r)

{

	_ProcessData(r);

	return 1;

}





///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////









