/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CAVIPWRITER_H_

#define CAVIPWRITER_H_



#include "CComponent.h"

#include "CPassiveSink.h"

#include	<vector.h>

#include	<CError.h>

#include	<windows.h>

#include	<vfw.h>







class CAviPWriter: public CComponent

{

protected:

	PAVIFILE	mFile;

	nmr_string	mFileName;

	int			mFileOpen;

	int			mInterleave;

	typedef vector<CConnection *> ConnectionList_t;

	ConnectionList_t mProducedConnections;



	int			mRecalculateFramerate;



public:

	CAviPWriter(int interleave,CGUIController *gui);

	virtual ~CAviPWriter(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;

	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

			PAVIFILE	GetAviFile(void);





	CERROR_SUBCLASS(FileError);

};



///////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////



class CAviPWriterBaseStream: public CPassiveSink

{

protected:

	PAVISTREAM	mAviStream;

	PAVIFILE	mAviFile;

	int			mStreamOpen;

	int			mOutputFrameCounter;

	CAviPWriter	*mComponent;

	CGUIController	*mGUIController;

	int			mTerminate;

	CPassiveSinkDataReference mDataReference;

	char		*mDataReferenceData;

	int			mDataReferenceSize;



	CAviPWriterBaseStream(CAviPWriter *comp,CGUIController *gui);

			void	_OpenStream(void);

			void	_CloseStream(void);



public:

	virtual ~CAviPWriterBaseStream(void);



	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

	virtual	void	Terminate(void);



	// get a place to put data

	virtual	CPassiveSinkDataReference *GetDataReference(size_t);

	virtual	void	DataComplete(void);



	CERROR_SUBCLASS(StreamError);

};







///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////



class CAviPWriterVideoStream: public CAviPWriterBaseStream

{

protected:

	CAviPWriterVideoStream(CAviPWriter *comp,CGUIController *gui) 

		: CAviPWriterBaseStream(comp,gui) 

	{ mConnectionType = (CONN_SINK | CONN_PASSIVE | CONN_VIDEO);}



			void	_ProcessData(const CPassiveSinkDataReference *r);

public:

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);



	friend CAviPWriter;

};



class CAviPWriterAudioStream: public CAviPWriterBaseStream

{

protected:

	CAviPWriterAudioStream(CAviPWriter *comp,CGUIController *gui) 

		: CAviPWriterBaseStream(comp,gui) 

	{ mConnectionType = (CONN_SINK | CONN_PASSIVE | CONN_AUDIO);}



			void	_ProcessData(const CPassiveSinkDataReference *r);

public:

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);

	friend CAviPWriter;

};



////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////





#endif

