/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAviReader.h"

#include "CGUIController.h"

#include <stack_aids.h>



CAviReader::CAviReader(CGUIController *gui)

				: CComponent(gui)

{

	AVIFileInit();

	mFile = 0;

	mFileOpen = 0;

	mFileName = AfxGetApp()->GetProfileString("CAviReader","FileName","");

}



void

CAviReader::Initialize(void)

{

	assert(!mFileOpen);

	assert(!mFile);



	int err;

	if (err = AVIFileOpen(&mFile,mFileName,OF_READ,NULL))

		throw FileError("Error opening input file",err);

	mFileOpen = 1;

}



void

CAviReader::Cleanup(void)

{

	AfxGetApp()->WriteProfileString("CAviReader","FileName",mFileName);

	if (mFileOpen)

		AVIFileRelease(mFile);

	mFileOpen = 0;

	mFile = 0;

}



CAviReader::~CAviReader(void)

{



	ConnectionList_t::iterator i;

	for (i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		delete (*i);

	mProducedConnections.erase(mProducedConnections.begin(),mProducedConnections.end());



	AVIFileExit();

}



void	

CAviReader::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	ConnectionList_t::iterator i;

	for(i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		{

		if ((*i) == c)

			{

			delete (*i);

			mProducedConnections.erase(i);

			break;

			}

		}

}



CComponent::ConnectionTypeList_t	

CAviReader::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_PASSIVE | CONN_SOURCE | CONN_SEEKABLE | CONN_VIDEO);

	l.push_back(CONN_PASSIVE | CONN_SOURCE | CONN_SEEKABLE | CONN_AUDIO);

	return l;

}



nmr_string 

CAviReader::GetName(void) const

{

	return "Avi Reader";

}



CConnection *

CAviReader::CreateConnection(CConnection::Connection_t t)

{

	CConnection::Connection_t	minimum_requirements = (CONN_PASSIVE | CONN_SOURCE);

	

	if ((t & minimum_requirements) != minimum_requirements)

		return 0;

	

	CConnection *c = 0;



	if (t & CONN_AUDIO)

		c = new CAviReaderAudioStream(this,0,mGUIController);

	if (t & CONN_VIDEO)

		c = new CAviReaderVideoStream(this,0,mGUIController);

	

	if (c)

		mProducedConnections.push_back(c);

	return c;

}



PAVIFILE

CAviReader::GetAviFile(void)

{

	return mFile;

}



/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////

CAviReaderBaseStream::CAviReaderBaseStream(CAviReader *comp,

							unsigned long kind,int which,CGUIController *gui)

{

	mComponent		= comp;

	mAviFile		= 0;

	mWhichStream	= which;

	mStreamOpen		= 0;

	mCurrentFrame	= -1;

	mGUIController	= gui;

	mStreamKind		= kind;

	mTerminate		= 0; 

}



void

CAviReaderBaseStream::SetupStreamInfo(void)

{

	AVISTREAMINFO	 i;

	int				err;



	assert(mAviStream);



	if (err = AVIStreamInfo(mAviStream,&i,sizeof(i)))

		throw StreamError("Error getting input stream info",err);

	mStreamInfoObject.SetStreamInfo(i);

}



void

CAviReaderBaseStream::SetupStreamFormat(void)

{

	int err;

	long siz;



	if (err = AVIStreamReadFormat(mAviStream,0,NULL,&siz))

		throw StreamError("Error getting input format size",err);



	char *f = new char[siz];

	stack_ArrayDeleter<char> sad(f);



	if (err = AVIStreamReadFormat(mAviStream,0,f,&siz))

		throw StreamError("Error getting input format",err);



	mStreamInfoObject.SetStreamFormat(f,siz);

}



void

CAviReaderBaseStream::Initialize(void)

{

	mTerminate = 0;

	mCurrentFrame = -1;

	mAviFile = mComponent->GetAviFile();

	assert(mAviFile);



	int err;

	if (err = AVIFileGetStream(mAviFile,&mAviStream,mStreamKind,mWhichStream))

		throw StreamError("Error opening stream",err);

	mStreamOpen = 1;

	mCurrentFrame = -1;

	CPassiveSource::Initialize();

}



void

CAviReaderBaseStream::Cleanup(void)

{

	CPassiveSource::Cleanup();

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;

}



void

CAviReaderBaseStream::Terminate(void)

{

	mTerminate = 1;

}



CAviReaderBaseStream::~CAviReaderBaseStream(void)

{

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;

}







void	

CAviReaderBaseStream::ReturnData(CPassiveSourceDataReference *r)

{

	delete r;

}





CPassiveSourceDataReference* 

CAviReaderVideoStream::GetData(int index,Direction_t dir)

{

	int frame;

	if (mTerminate) return 0;



	CPassiveSourceDataReference *ref = 0;



	mCurrentFrameLock.Lock();

	switch(dir)

		{

		case DIR_ABSOLUTE:

			frame = index;

			break;

		case DIR_FORWARD:

			frame = mCurrentFrame + index;

			break;

		case DIR_BACKWARD:

			frame = mCurrentFrame - index;

			break;

		}

	mCurrentFrameLock.Unlock();



	long datasize = 0;

	AVIStreamRead(mAviStream,frame,1,NULL,0,&datasize,NULL);

	if (datasize > 0)

		{

		mCurrentFrameLock.Lock();

		mCurrentFrame = frame;

		mCurrentFrameLock.Unlock();



		U32 keyframe = 

			(AVIStreamFindSample(mAviStream,frame,FIND_KEY | FIND_NEXT) == frame);



		char *data = new char[datasize];

		AVIStreamRead(mAviStream,frame,1,data,datasize,NULL,NULL);

		ref = new CPassiveSourceDataReference(data,datasize,keyframe,1);

#ifndef NDEBUG

		(mGUIController->Trace(this)) << "Reading video frame "

			<< frame << CMessageStream::show;

#endif

		}



	return ref;

}	





CPassiveSourceDataReference* 

CAviReaderAudioStream::GetData(int index,Direction_t dir)

{

	if (mTerminate) return 0;



	int frame;

	CPassiveSourceDataReference *ref = 0;



	mCurrentFrameLock.Lock();

	switch(dir)

		{

		case DIR_ABSOLUTE:

			assert(0); // this doesn't work yet

			frame = index;

			break;

		case DIR_FORWARD:

			assert(index == 1); // forward a frame at a time only

			frame = mCurrentFrame + index;

			break;

		case DIR_BACKWARD:

			assert(0); // this doesn't work yet

			frame = mCurrentFrame - index;

			break;

		}

	mCurrentFrameLock.Unlock();



	long datasize,samples;

	AVIStreamRead(mAviStream,frame,AVISTREAMREAD_CONVENIENT,NULL,0,&datasize,&samples);

	if (datasize > 0)

		{

		char *data = new char[datasize];

		AVIStreamRead(mAviStream,frame,AVISTREAMREAD_CONVENIENT,data,datasize,0,0);

		ref = new CPassiveSourceDataReference(data,datasize,0,1);

		

		frame += samples-1;

		mCurrentFrameLock.Lock();

		mCurrentFrame = frame;

		mCurrentFrameLock.Unlock();

#ifndef NDEBUG

		(mGUIController->Trace(this)) << "Reading audio samples "

			<< (mCurrentFrame - samples) << " through " << mCurrentFrame

			<< CMessageStream::show;

#endif

		}



	return ref;

}	



/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////











