/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CAVIREADERCOMPONENT_H_

#define CAVIREADERCOMPONENT_H_



#include	<nmr_string.h>

#include	<vector.h>

#include	<CCMutex.h>

#include	<CError.h>

#include	"CPassiveSource.h"

#include	"CComponent.h"



class CAviReader;

class CAviReaderBaseStream;

class CAviReaderDataReference;



class CAviReader: public CComponent

{

protected:

	PAVIFILE	mFile;

	nmr_string	mFileName;

	int			mFileOpen;

	typedef	 vector<CConnection *> ConnectionList_t;

	ConnectionList_t	mProducedConnections;



public:

	CAviReader(CGUIController *gui);

	~CAviReader(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;

	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);



	PAVIFILE	GetAviFile(void);



	CERROR_SUBCLASS(FileError);

};



////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////



class CAviReaderBaseStream: public CPassiveSource

{

protected:

	PAVISTREAM	mAviStream;

	int			mStreamOpen;

	int			mCurrentFrame;

	CCMutex		mCurrentFrameLock;

	CGUIController	*mGUIController;

	CAviReader		*mComponent;

	PAVIFILE		mAviFile;

	int				mWhichStream;

	unsigned long	mStreamKind;

	int				mTerminate;



	CAviReaderBaseStream(CAviReader *comp,

		unsigned long kind,int which,CGUIController *gui);



	virtual	void	SetupStreamInfo(void);

	virtual	void	SetupStreamFormat(void);

	

public:

	virtual ~CAviReaderBaseStream(void);



	virtual	void	ReturnData(CPassiveSourceDataReference *r);

	virtual	void		Initialize(void);

	virtual	void		Cleanup(void);

	virtual	void		Terminate(void);



	friend CAviReader;



	CERROR_SUBCLASS(StreamError);

};



class CAviReaderAudioStream: public CAviReaderBaseStream

{

protected:

	CAviReaderAudioStream(CAviReader *comp,int which,CGUIController *gui)

		: CAviReaderBaseStream(comp,streamtypeAUDIO,which,gui)

	{ mConnectionType = (CONN_PASSIVE | CONN_SOURCE | CONN_AUDIO | CONN_SEEKABLE); }

	

public:

	virtual	CPassiveSourceDataReference* GetData(int index,Direction_t dir);



	friend CAviReader;

};



class CAviReaderVideoStream: public CAviReaderBaseStream

{

protected:

	CAviReaderVideoStream(CAviReader *comp,int which,CGUIController *gui)

		: CAviReaderBaseStream(comp,streamtypeVIDEO,which,gui) 

	{ mConnectionType = (CONN_PASSIVE | CONN_SOURCE | CONN_VIDEO | CONN_SEEKABLE); }

	

public:

	virtual	CPassiveSourceDataReference* GetData(int index,Direction_t dir);



	friend CAviReader;

};

///////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////





#endif

