/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAviWriter.h"
#include "CPassiveSource.h"
#include "CGUIController.h"
#include "AviUtils.h"
#include <stack_aids.h>
#include "danbug.h"

//HANDLE h;


CAviWriter::CAviWriter(CGUIController *gui)
: CComponent(gui)
{
	AVIFileInit();
	//h = ::CreateFile("d:\\cap2.avi",GENERIC_WRITE,0,NULL,CREATE_ALWAYS,

	//	/* FILE_ATTRIBUTE_NORMAL */ FILE_FLAG_NO_BUFFERING ,NULL);



	mFileOpen = 0;
	mFile = 0;
	mInterleave = 0;
	mRecalculateFramerate = AfxGetApp()->GetProfileInt("CAviWriter","RecalculateFramerate",0);
	mFileName = AfxGetApp()->GetProfileString("CAviWriter","FileName","");
	mVideoStream = 0;
}

void
CAviWriter::Initialize(void)
{
	assert(!mFileOpen);
	assert(!mFile);

	int err;
	remove(mFileName);
	if (err = AVIFileOpen(&mFile,mFileName,OF_CREATE | OF_WRITE,NULL))
			throw FileError("Error creating output file",err);
	mFileOpen = 1;
	mVideoIsRunning = (mVideoStream ? 1 : 0);
}

void
CAviWriter::Cleanup(void)
{
	AfxGetApp()->WriteProfileString("CAviWriter","FileName",mFileName);
	AfxGetApp()->WriteProfileInt("CAviWriter","RecalculateFramerate",mRecalculateFramerate);

	if (mFileOpen)
		{
		assert(mFile);
		AVIFileRelease(mFile);
		if (mRecalculateFramerate)
			RecalculateFramerate(mFileName);
		}
	mFileOpen = 0;
	mFile = 0;
}

CAviWriter::~CAviWriter(void)
{
	//::CloseHandle(h);

	ConnectionList_t::iterator i;

	for (i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)
		delete (*i);
	mProducedConnections.erase(mProducedConnections.begin(),mProducedConnections.end());

	AVIFileExit();
}

void	
CAviWriter::ReturnConnection(CConnection *c,CConnection::Connection_t t)
{
	ConnectionList_t::iterator i;
	if (t & CONN_VIDEO)
		mVideoStream = 0;

	for(i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)
		{
		if ((*i) == c)
			{
			delete (*i);
			mProducedConnections.erase(i);
			break;
			}
		}
}

CComponent::ConnectionTypeList_t	
CAviWriter::GetSupportedTypes(void) const
{
	ConnectionTypeList_t	l;

	l.push_back(CONN_SINK | CONN_ACTIVE | CONN_VIDEO);
	l.push_back(CONN_SINK | CONN_ACTIVE | CONN_AUDIO);
	return l;
}

nmr_string 
CAviWriter::GetName(void) const
{
	return "Avi Writer";
}

CConnection *
CAviWriter::CreateConnection(CConnection::Connection_t t)
{
	CConnection::Connection_t	minimum_requirements = (CONN_SINK | CONN_ACTIVE);

	if ((t & minimum_requirements) != minimum_requirements)
		return 0;

	CConnection *c = 0;

	if (t & CONN_AUDIO)
		c =  new CAviWriterAudioStream(this,mGUIController);
	if (t & CONN_VIDEO)
		c =  mVideoStream = new CAviWriterVideoStream(this,mGUIController);
	
	if (c)
		mProducedConnections.push_back(c);
	
	return c;
}


PAVIFILE
CAviWriter::GetAviFile(void)
{
	return mFile;
}

int	
CAviWriter::IsVideoStreamRunning(void)
{
	return mVideoIsRunning;
}


void	
CAviWriter::VideoIsDone(void)
{
	mInterleaveMutex.Lock();
	mVideoIsRunning = 0;
	mInterleaveWait.Signal();
	mInterleaveMutex.Unlock();
}

int	
CAviWriter::VideoFramesWritten(void)
{
	assert(mVideoStream);
	return mVideoStream->GetFrameCount();
}




void	

CAviWriter::InterleaveLock(void)

{

	mInterleaveMutex.Lock();

}





void	

CAviWriter::InterleaveUnlock(void)

{

	mInterleaveMutex.Unlock();

}





void	

CAviWriter::InterleaveWait(void)

{

	mInterleaveWait.Wait(mInterleaveMutex);

}





void	

CAviWriter::InterleaveSignal(void)

{

	mInterleaveWait.Signal();

}



int

CAviWriter::GetInterleave(void)

{

	return mInterleave;

}



///////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////

CAviWriterBaseStream::CAviWriterBaseStream(CAviWriter *comp,CGUIController *gui)

{

	mComponent = comp;

	mGUIController = gui;

	mAviFile = 0;

	mStreamOpen = 0;

	mOutputFrameCounter = 0;

}



CAviWriterBaseStream::~CAviWriterBaseStream(void)

{

}



int

CAviWriterBaseStream::GetFrameCount(void)

{

	return mOutputFrameCounter;

}



void

CAviWriterBaseStream::Initialize(void)

{

	CActiveSink::Initialize();

	mOutputFrameCounter = 0;

	mAviFile = mComponent->GetAviFile();

	assert(mAviFile);



	int err;

	AVISTREAMINFO	i = mStreamInfoObject.GetStreamInfo();
	if (err = AVIFileCreateStream(mAviFile,&mAviStream,&i))

		throw StreamError("Error creating output stream",err);



	if (err = AVIStreamSetFormat(mAviStream,0,(void *)mStreamInfoObject.GetStreamFormat(),mStreamInfoObject.GetStreamFormatSize()))

		throw StreamError("Error setting stream format",err);



	mStreamOpen = 1;

}



void

CAviWriterBaseStream::Cleanup(void)

{

	if (mStreamOpen)

		AVIStreamRelease(mAviStream);

	mStreamOpen = 0;

}



///////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////



int

CAviWriterVideoStream::ProcessData(CPassiveSource *s)

{

	CPassiveSourceDataReference	*ref = s->GetData();



	if (!ref) 

		{

		mComponent->VideoIsDone();

		return 0;

		}



	const char *data;

	size_t		len;



	U32 key_frame;
	ref->RedeemData(data,len,key_frame);



#ifndef NDEBUG

	(mGUIController->Trace(this)) << "Writing video frame " << 

		mOutputFrameCounter << CMessageStream::show;

#endif

	int err;



#if 1

	if (err = AVIStreamWrite(mAviStream,mOutputFrameCounter,1,(void *)data,len,

		key_frame ? AVIIF_KEYFRAME : 0,NULL,NULL))

		{

		mComponent->VideoIsDone();

		return 0;

		}

#else

		{

			unsigned long newlen;

			if (len)

			::WriteFile(h,data,2048*18 /*len */,&newlen,NULL);

		}

#endif

	mOutputFrameCounter++;



	s->ReturnData(ref);

	mComponent->InterleaveLock();

	mComponent->InterleaveSignal();

	mComponent->InterleaveUnlock();

	return 1;

}



/////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////

void

CAviWriterAudioStream::Initialize(void)

{
	CAviWriterBaseStream::Initialize();

	mChunksWritten = 0;

}





int

CAviWriterAudioStream::ProcessData(CPassiveSource *s)

{

	mComponent->InterleaveLock();

	while (mComponent->IsVideoStreamRunning() && 

		((mChunksWritten * mComponent->GetInterleave()) >= mComponent->VideoFramesWritten()))

		{

		mComponent->InterleaveWait();

		}

	mComponent->InterleaveUnlock();



	CPassiveSourceDataReference	*ref = s->GetData();



	if (!ref) return 0;



	const char *data;

	size_t		len;

	int			num_samples;



	U32 flags;
	ref->RedeemData(data,len,flags);

	WAVEFORMATEX	*w = (WAVEFORMATEX *)mStreamInfoObject.GetStreamFormat();



	switch(w->wBitsPerSample)

		{

		case 3:

		case 4:

			// duck specific

			num_samples = len / 2048;

			break;

		default:

			num_samples = (len * 8) / (w->nChannels * w->wBitsPerSample);

			break;

		}

#ifndef NDEBUG

	(mGUIController->Trace(this)) << "Writing audio samples "

		<< mOutputFrameCounter << " through " << (mOutputFrameCounter + num_samples)

		<< CMessageStream::show;

#endif



#if 1

	if (AVIStreamWrite(mAviStream,mOutputFrameCounter,num_samples,(void *)data,len,0,NULL,NULL))

		return 0;

#endif

	mChunksWritten++;

	mOutputFrameCounter += num_samples;



	s->ReturnData(ref);

	return 1;

}







