/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CAVIWRITERCOMPONENT_H_
#define CAVIWRITERCOMPONENT_H_

#include	"CActiveSink.h"
#include	"CComponent.h"
#include	<CConditionVariable.h>
#include	<nmr_string.h>
#include	<vector.h>
#include	<CError.h>
#include	<windows.h>


class CAviWriter;
class CAviWriterBaseStream;
class CAviWriterAudioStream;
class CAviWriterVideoStream;

class CAviWriter: public CComponent
{
protected:
	PAVIFILE	mFile;
	nmr_string	mFileName;
	int			mFileOpen;
	typedef vector<CConnection *> ConnectionList_t;
	ConnectionList_t	mProducedConnections;

	int			mRecalculateFramerate;

	// interleave members
	int					mInterleave;
	int					mVideoIsRunning;
	CAviWriterVideoStream	*mVideoStream;
	CCMutex				mInterleaveMutex;
	CConditionVariable	mInterleaveWait;

protected:
	// called by streams
	virtual	int	IsVideoStreamRunning(void);
	virtual	void	VideoIsDone(void);
	virtual	int	VideoFramesWritten(void);
	virtual	void	InterleaveLock(void);
	virtual	void	InterleaveUnlock(void);
	virtual	void	InterleaveWait(void);
	virtual	void	InterleaveSignal(void);

public:
	CAviWriter(CGUIController *gui);
	~CAviWriter(void);

	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;
	virtual nmr_string GetName(void) const;
	virtual CConnection *CreateConnection(CConnection::Connection_t t);
	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);
	virtual	void	Configure(CWnd *wnd) = 0;
	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;
			PAVIFILE	GetAviFile(void);
	virtual	void	Initialize(void);
	virtual	void	Cleanup(void);

	virtual	int		GetInterleave(void);

	friend CAviWriterAudioStream;
	friend CAviWriterVideoStream;
	CERROR_SUBCLASS(FileError);
	CERROR_SUBCLASS(StreamError);
};

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

class CAviWriterBaseStream: public CActiveSink
{
protected:
	CAviWriter		*mComponent;
	CGUIController	*mGUIController;
	PAVISTREAM	mAviStream;
	PAVIFILE	mAviFile;
	int			mStreamOpen;
	int			mOutputFrameCounter;

	CAviWriterBaseStream(CAviWriter *comp,CGUIController *gui);

public:
	virtual ~CAviWriterBaseStream(void);
	virtual	void	Initialize(void);
	virtual	void	Cleanup(void);

	virtual	int		GetFrameCount(void);

	friend CAviWriter;
	CERROR_SUBCLASS(StreamError);
};



///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

class CAviWriterVideoStream: public CAviWriterBaseStream
{
	CAviWriterVideoStream(CAviWriter *comp,CGUIController *gui)
		: CAviWriterBaseStream(comp,gui) 
	{ mConnectionType |= CONN_VIDEO; }

	// returns true if processing should continue

	virtual	int		ProcessData(CPassiveSource *s);
	
public:
	friend CAviWriter;
	CERROR_SUBCLASS(StreamError);
};

class CAviWriterAudioStream: public CAviWriterBaseStream
{
	CAviWriterAudioStream(CAviWriter *comp,CGUIController *gui)
		: CAviWriterBaseStream(comp,gui) 
	{ mConnectionType |= CONN_AUDIO;}

	int	mChunksWritten;

	// returns true if processing should continue
	virtual	int		ProcessData(CPassiveSource *s);
	
public:
	virtual	void	Initialize(void);

	friend CAviWriter;
	CERROR_SUBCLASS(StreamError);
};

#endif
