/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	"CCircularBuffer.h"

#include	"CGUIController.h"
#include	"CCircularBufferComponent.h"

#include	<stack_aids.h>

#include	<safe_ostrstream.h>




CConnection::Connection_t	

CCircularBuffer::_source_GetConnectionType(void)

{

	return CCBSource::mConnectionType;

}



void			

CCircularBuffer::_source_Initialize(void)

{

	mLastFrameRead = -1;

	mConfig_PoolSize = mComponent->GetPoolSize(CCBSource::mConnectionType);

	mConfig_NumIndicies = mComponent->GetIndicies(CCBSource::mConnectionType);



	assert(mConfig_PoolSize);

	assert(mConfig_NumIndicies);



	if (mMemoryPool) delete [] mMemoryPool;



	mMemoryPool	= new char[mConfig_PoolSize];

	mEndOfPool 	= mMemoryPool + mConfig_PoolSize;

	mWritePtr 	= mMemoryPool;

	mBlockPtr 	= mMemoryPool;

	mDynamicEndOfPool = mMemoryPool;



	if (mIndicies) delete [] mIndicies;

	mIndicies = new CB_Index[mConfig_NumIndicies];

	mMaxIndex 			= mConfig_NumIndicies - 1;

	mMinIndex 			= 0;



	_Init();



	mIgnoreOverflow		= 0;

	CPassiveSource::Initialize();

}



void			

CCircularBuffer::_source_Go(void)

{

	CPassiveSource::Go();

}



void			

CCircularBuffer::_source_Terminate(void)

{

	stack_MutexLock sml(&mMutex);



	mTerminate = 1;

	mWaitingToWrite.Broadcast();

	mWaitingToRead.Broadcast();

	CPassiveSource::Terminate();

}





void			

CCircularBuffer::_source_Wait(void)

{

	CPassiveSource::Wait();

}



CConnection::Connection_t	

CCircularBuffer::_sink_GetConnectionType(void)

{

	return CCBSink::mConnectionType;

}



void			

CCircularBuffer::_sink_Initialize(void)

{

	CPassiveSink::Initialize();

}



void			

CCircularBuffer::_sink_Go(void)

{

	CPassiveSink::Go();

}



void			

CCircularBuffer::_sink_Terminate(void)

{

	stack_MutexLock sml(&mMutex);



	mTerminate = 1;

	mWaitingToWrite.Broadcast();

	mWaitingToRead.Broadcast();

	CPassiveSink::Terminate();

}



void			

CCircularBuffer::_sink_Wait(void)

{

	CPassiveSink::Wait();

}




//////////////////////////////////////

//////////////////////////////////////

void

CCircularBuffer::_Init(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	int num_indicies = mMaxIndex + 1;



	for (int x = 0; x < num_indicies; ++x)

		{

		mIndicies[x].mBusy = 0;

		mIndicies[x].mPtr = NULL;

		mIndicies[x].mFrameNumber = -10;

		}



	mCurrentReadIndex 	= -1;

	mCurrentWriteIndex	= 0;

	mChunksInBuffer = 0;

	mFrameCounter = 0;

	mDataComplete = 0;

	mTerminate = 0;

	mOutstandingReads = 0;

}


CCircularBuffer::CCircularBuffer(int vid_aud,CGUIController *gui,

								 CCircularBufferComponent *comp)
{

	mComponent = comp;

	mOutstandingReads = 0;

	mMemoryPool = 0;

	mIndicies = 0;



	mGUIController = gui;

	CCBSource::mConnectionType = (CONN_PASSIVE | CONN_SOURCE | CONN_SEEKABLE | vid_aud);

	CCBSink::mConnectionType = (CONN_SINK | CONN_PASSIVE | vid_aud);

	mMemoryPool = NULL;
	mIndicies = NULL;



	mConfig_PoolSize = 0;

	mConfig_NumIndicies = 0;

}


CCircularBuffer::~CCircularBuffer(void)
{
	delete [] mMemoryPool;
	mMemoryPool = NULL;

	delete [] mIndicies;
	mIndicies = NULL;
}





/////////////////////////////////////////////////////////////////////////////////

////////////////////////// PASSIVE SINK INTERFACE ///////////////////////////////

//////////////////////////////////////////////////////////////////////////////////



void
CCircularBuffer::Clear(void)
{
	mWriteLock.Lock();

	stack_MutexLock sml(&mMutex);



	assert(!mOutstandingReads);



	mWritePtr 	= mMemoryPool;
	mBlockPtr 	= mMemoryPool;
	mDynamicEndOfPool = mMemoryPool;

	_Init();



	mWriteLock.Unlock();



	mWaitingToWrite.Signal();

	mWaitingToRead.Broadcast();
}


int

CCircularBuffer::ConsumeDataNow(const CPassiveSinkDataReference *ref)

{

	CPassiveSinkDataReference *rtemp = GetDataReference(ref->GetSize());

	if (!rtemp) return 0;



	rtemp->SetFlags(ref->GetFlags());

	memcpy(rtemp->GetWritePtr(),ref->GetWritePtr(),ref->GetSize());

	ReturnDataReference(rtemp);

	return 1;

}



CPassiveSinkDataReference *

CCircularBuffer::GetDataReference(size_t siz)

{

	assert(mMemoryPool);

	assert(mIndicies);



	mWriteLock.Lock();

	stack_MutexLock sml(&mMutex);



	if (mTerminate || mDataComplete) return 0;



	int		blocked;



#ifdef	ALIGN_FACTOR

	siz = (siz + ALIGN_FACTOR) & ~((size_t)(ALIGN_FACTOR-1));

#endif



	// wait for our space

	do 

		{

		blocked = 0;



		if (mIndicies[mCurrentWriteIndex].mBusy)

			{

			blocked = 1;

			}

		else if (mBlockPtr > mWritePtr)

			{

			if ((mBlockPtr - mWritePtr - 1) < ((long)siz))

				blocked = 1;

			}

		else

			{

			if (mEndOfPool - mWritePtr < ((long)siz))

				{

				if (mBlockPtr == mMemoryPool)

					{

					blocked = 1;

					}

				else

					{

					mDynamicEndOfPool = mWritePtr;

					mWritePtr = mMemoryPool;

					if (mBlockPtr - mWritePtr - 1 < ((long)siz))

						blocked = 1;

					}

				}

			}



		if (blocked)

			{

			if (!mIgnoreOverflow)

				mWaitingToWrite.Wait(mMutex);

			}

		}

	while((!mIgnoreOverflow) && blocked && (!mTerminate));

	

	mSinkDataReference.SetWritePtr(mWritePtr);

	mSinkDataReference.SetSize(siz);

	assert(siz >= 0);

	// not really, but probably. If frames get huge you might have

	// to remove this test

	assert(siz < 1000000);

	return (blocked || mTerminate ? ((CPassiveSinkDataReference *)NULL) : &mSinkDataReference);

}




void	

CCircularBuffer::ReturnDataReference(CPassiveSinkDataReference *r)

{

	assert(mMemoryPool);

	assert(mIndicies);

	assert(r);

	assert(r == &mSinkDataReference);



	stack_MutexLock sml(&mMutex);



	size_t	siz		= r->GetSize();

	U32		flags	= r->GetFlags();



#ifdef	ALIGN_FACTOR

	mIndicies[mCurrentWriteIndex].mReportedLength = siz;

	siz = (siz + ALIGN_FACTOR) & ~((size_t)(ALIGN_FACTOR-1));

#endif

	mIndicies[mCurrentWriteIndex].mBusy = 1;

	mIndicies[mCurrentWriteIndex].mPtr = mWritePtr;

	mIndicies[mCurrentWriteIndex].mLen = siz;

	mIndicies[mCurrentWriteIndex].mFrameNumber = mFrameCounter++;

	mIndicies[mCurrentWriteIndex].mFlags = flags;



	mCurrentWriteIndex++;

	if (mCurrentWriteIndex > mMaxIndex)

		mCurrentWriteIndex = mMinIndex;



	mWritePtr += siz;



	if (mWritePtr > mDynamicEndOfPool)

		mDynamicEndOfPool = mWritePtr;



	mChunksInBuffer++;



	mWriteLock.Unlock();

	mWaitingToRead.Signal();

#ifndef NDEBUG

	(mGUIController->Trace(this)) <<

		((CCBSource::mConnectionType & CONN_AUDIO) ? "Audio" : "Video")

		<< " buffer " << GetPercentFull() << "% full" << CMessageStream::show;

#endif

}



void

CCircularBuffer::DataComplete(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	stack_MutexLock	smlw(&mWriteLock);

	stack_MutexLock sml(&mMutex);



	mDataComplete = 1;

	//mWaitingToWrite.Signal();

	mWaitingToRead.Signal();

}


//////////////////////////////////////////////////////////////////////////////////

////////////////PASSIVE SOURCE INTERFACE /////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////



void	

CCircularBuffer::SetupStreamInfo(void)

{

	CPassiveSource::mStreamInfoObject = CPassiveSink::mStreamInfoObject;

}



void	

CCircularBuffer::SetupStreamFormat(void)

{

	CPassiveSource::mStreamInfoObject = CPassiveSink::mStreamInfoObject;

}





int

CCircularBuffer::IsAllDataConsumed(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	if (!mDataComplete) return 0;

	return (!(mChunksInBuffer - mOutstandingReads));

}



CPassiveSourceDataReference* 

CCircularBuffer::GetData(int index,Direction_t dir)

{

	assert(mMemoryPool);

	assert(mIndicies);

	stack_MutexLock sml(&mMutex);



	int	iindex;

	int frame,goal_frame;

	iindex = index;



	CCircularBufferSourceDataReference *ref = 0;



	if (mTerminate) return 0;



	if (dir == ABSOLUTE)

		{

		// handle absolute as a special case

		do	

			{

			if (IsAllDataConsumed()) 

				{

				mWaitingToRead.Broadcast();

				return 0;

				}

			for (int x = mMinIndex; x <= mMaxIndex; ++x)

				{

				if (mIndicies[x].mFrameNumber == index)

					{

					if (mIndicies[x].mBusy)

						{

#ifdef	ALIGN_FACTOR

						assert(mIndicies[x].mReportedLength >= 0);

						size_t len = mIndicies[x].mReportedLength;

#else

						assert(mIndicies[x].mLen >= 0);

						size_t len = mIndicies[x].mLen;

#endif

						++mOutstandingReads;

						ref = new CCircularBufferSourceDataReference(x,

							mIndicies[x].mPtr,len,mIndicies[x].mFlags,0);

						}

					break;

					}

				}

			if (!ref)

				mWaitingToRead.Wait(mMutex);

			}

		while (!ref && !mTerminate);

		return ref;

		}

	else

		{

		// handle backward and forward



		// if just initialized mCurrentReadIndex will be -1 and we

		// can't ask for current frame

		do 

			{

			if (IsAllDataConsumed()) 

				{

				mWaitingToRead.Broadcast();

				return 0;

				}

			index = iindex;

			assert(mCurrentReadIndex >= 0 || index > 0);

			frame = mLastFrameRead;

			//(mCurrentReadIndex >= 0 ? mIndicies[mCurrentReadIndex].mFrameNumber

			//	: -1);

			goal_frame = (dir == DIR_BACKWARD ? frame - index : frame + index);

			int tmp = mCurrentReadIndex;

			if (dir == DIR_BACKWARD)

				{

				while (index--)

					if (--tmp < mMinIndex)

						tmp = mMaxIndex;

				}

			else

				{

				while(index--)

					if (++tmp > mMaxIndex)

						tmp = mMinIndex;				

				}



			if (mIndicies[tmp].mFrameNumber == goal_frame && mIndicies[tmp].mBusy)

				{

#ifdef	ALIGN_FACTOR

				size_t len = mIndicies[tmp].mReportedLength;

				assert(mIndicies[tmp].mReportedLength >= 0);

#else

				size_t len = mIndicies[tmp].mLen;

				assert(mIndicies[tmp].mLen >= 0);

#endif

				++mOutstandingReads;

				mCurrentReadIndex = tmp;

				ref = new CCircularBufferSourceDataReference(tmp,mIndicies[tmp].mPtr,len,

					mIndicies[tmp].mFlags,0);

				mLastFrameRead = goal_frame;

				}

			else

				{

				mWaitingToRead.Wait(mMutex);

				}

			}

		while (!ref && !mTerminate);

		}



	// if there are multiple entries at the wait, we need to release them all

	if (!ref) mWaitingToRead.Broadcast();

	return ref;

}




void	

CCircularBuffer::ReturnData(CPassiveSourceDataReference *r)

{

	assert(mMemoryPool);

	assert(mIndicies);

	assert(r);



	stack_MutexLock sml(&mMutex);



	CCircularBufferSourceDataReference *rr = 

		dynamic_cast<CCircularBufferSourceDataReference *>(r);

	assert(rr);

	int index = rr->GetIndex();



	if (mBlockPtr >= mDynamicEndOfPool && mBlockPtr != mWritePtr)

		mBlockPtr = mMemoryPool;



	if (mIndicies[index].mBusy)

		{

		mChunksInBuffer--;

		mIndicies[index].mBusy = 0;

		if (mIndicies[index].mPtr == mBlockPtr)

			{

			while (mIndicies[index].mBusy == 0 && 

												mIndicies[index].mPtr == mBlockPtr)

				{

				assert(mIndicies[index].mLen >= 0);

				mBlockPtr += mIndicies[index].mLen;

				mIndicies[index].mPtr = NULL;

				index++;

				if (index > mMaxIndex)

					index = mMinIndex;

				if (mBlockPtr >= mDynamicEndOfPool && mBlockPtr != mWritePtr)

					mBlockPtr = mMemoryPool;

				

				}

			mWaitingToWrite.Signal();



			}

		}



	--mOutstandingReads;

	delete rr;

#ifndef NDEBUG

	(mGUIController->Trace(this))

		<< ((CCBSource::mConnectionType & CONN_AUDIO) ? "Audio" : "Video")

		<< " buffer " << GetPercentFull() << "% full" << CMessageStream::show;

#endif

	//mWaitingToWrite.Broadcast();

	if (IsAllDataConsumed()) mWaitingToRead.Broadcast();

}


/////////////////////////// MISC /////////////////////////////////

///////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// ReadyToRoll
//
// We need to guess when this buffer is ready. We pass in a percent threshold
//
// INPUT:
//	percent	- percentage beyond which we are ready
//
// OUTPUT:
//	1 if ready else 0
/////////////////////////////////////////////////////////////////////////////
int
CCircularBuffer::ReadyToRoll(long percent)
{
	assert(mMemoryPool);

	assert(mIndicies);

	if ((mCurrentWriteIndex - mMinIndex) >= (((mMaxIndex - mMinIndex) * percent) / 100l))
		return 1;

	if ((mWritePtr - mMemoryPool) >= (((mEndOfPool - mMemoryPool) * percent) / 100l))
		return 1;

	return 0;
}



////////////////////////////////////////////////////////////////////
// SetOverflowIgnore
//
// set the value of the ignore overflow flag for writer thread
//
// INPUT:
//	val	 - new value for ignoreoverflow flag (1 or 0)
//////////////////////////////////////////////////////////////////////
void	
CCircularBuffer::SetOverflowIgnore(int val)
{
	stack_MutexLock sml(&mMutex);

	mIgnoreOverflow = val;
	mWaitingToWrite.Signal();
}



int

CCircularBuffer::GetPercentFull(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	stack_MutexLock sml(&mMutex);



	long	amount_remaining;





	if (mBlockPtr > mWritePtr)

		{

		amount_remaining = mBlockPtr - mWritePtr;

		}

	else

		{

		amount_remaining = mEndOfPool - mWritePtr;

		amount_remaining += mBlockPtr - mMemoryPool;

		}



	return (100 - ((amount_remaining * 100) / (mEndOfPool - mMemoryPool)));

}



size_t

CCircularBuffer::GetChunksInBuffer(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	stack_MutexLock sml(&mMutex);



	return mChunksInBuffer;

}



size_t

CCircularBuffer::GetBytesInBuffer(void)

{

	assert(mMemoryPool);

	assert(mIndicies);

	stack_MutexLock sml(&mMutex);



	long	amount_remaining;





	if (mBlockPtr > mWritePtr)

		{

		amount_remaining = mBlockPtr - mWritePtr;

		}

	else

		{

		amount_remaining = mEndOfPool - mWritePtr;

		amount_remaining += mBlockPtr - mMemoryPool;

		}



	return mEndOfPool - mMemoryPool - amount_remaining;

}










