/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CCIRCULARBUFFER_H_
#define CCIRCULARBUFFER_H_

#include	<CCMutex.h>
#include	<CConditionVariable.h>
#include	"CPassiveSource.h"

#include	"CPassiveSink.h"



class CGUIController;

class CCircularBufferComponent;



///////// Note: Supports multiple simultaneous readers but only 1 writer

// at a time. Multiple writers are safe but will be serialized.

//////////



class CCBSource: public CPassiveSource

{

protected:

	virtual Connection_t _source_GetConnectionType(void) = 0;

	virtual	void		 _source_Initialize(void) = 0;

	virtual	void		 _source_Go(void) = 0;

	virtual	void		 _source_Terminate(void) = 0;

	virtual	void		 _source_Wait(void) = 0;

public:

	virtual	Connection_t	GetConnectionType(void)

		{ return _source_GetConnectionType(); }

	virtual	void	Initialize(void)

		{
 dprintf("calling ::Initialize (which does beginxx??)");
			_source_Initialize(); 
		}

	virtual	void	Go(void) 

		{ _source_Go(); }

	virtual	void	Terminate(void)

		{ _source_Terminate(); }

	virtual	void	Wait(void)

		{ _source_Wait(); }



};



class CCBSink: public CPassiveSink

{

protected:

	virtual Connection_t _sink_GetConnectionType(void) = 0;

	virtual	void		 _sink_Initialize(void) = 0;

	virtual	void		 _sink_Go(void) = 0;

	virtual	void		 _sink_Terminate(void) = 0;

	virtual	void		 _sink_Wait(void) = 0;



public:

	// connection

	virtual	Connection_t	GetConnectionType(void)

				{ return _sink_GetConnectionType(); }

	virtual	void	Initialize(void)

				{ _sink_Initialize(); }

	virtual	void	Go(void) 

				{ _sink_Go(); }

	virtual	void	Terminate(void)

				{ _sink_Terminate(); }

	virtual	void	Wait(void)

				{ _sink_Wait(); }



};







#if ((defined IRIX) || (defined SOLARIS))
#define	ALIGN_FACTOR	4	// long word align
#endif


class CCircularBuffer;



class CCircularBufferSourceDataReference: public CPassiveSourceDataReference

{

private:

	int	mIndex;

	CCircularBufferSourceDataReference(void){}

public:

	CCircularBufferSourceDataReference(int index,char *data,size_t len,

		U32 flags,int ownsdata) : CPassiveSourceDataReference(data,len,flags,ownsdata) 

		{ mIndex = index; }

	int	GetIndex(void) const { return mIndex;}

};


//////////////



class CCircularBuffer: public CCBSource, public CCBSink
{
	CLASS_NOCOPY(CCircularBuffer)


private:

	void	_Init(void);


protected:

	typedef struct 
	{
		int	 mBusy;

		int	 mFrameNumber;
		char *mPtr;
		long mLen;
#ifdef	ALIGN_FACTOR
		long mReportedLength;
#endif

		U32	mFlags;
	}CB_Index;

	char	*mMemoryPool;
	char	*mEndOfPool;
	char	*mDynamicEndOfPool;
	char	*mWritePtr;

	char	*mBlockPtr;


	int		mMinIndex;
	int		mMaxIndex;
	int		mCurrentReadIndex;
	int		mCurrentWriteIndex;
	CB_Index	*mIndicies;


	CCMutex	mMutex;	//	This is the master lock
	CConditionVariable	mWaitingToWrite;
	CConditionVariable	mWaitingToRead;



	CCMutex	mWriteLock;	// this is to avoid multiple writers

	CPassiveSinkDataReference mSinkDataReference;

	int		mFrameCounter;

	int		mLastFrameRead;

	int		mDataComplete;

	int		mTerminate;

	int		mOutstandingReads;



	int		mIgnoreOverflow;
	int		mChunksInBuffer;



	int		IsAllDataConsumed(void);

	size_t				mConfig_PoolSize;

	size_t				mConfig_NumIndicies;

	CGUIController		*mGUIController;

	CCircularBufferComponent	*mComponent;


public:

	// really Connection type but MSVC++ has problems
	CCircularBuffer(int vid_aud,CGUIController *gui,CCircularBufferComponent *comp);
	~CCircularBuffer(void);


	virtual size_t	GetBytesInBuffer(void);

	virtual size_t	GetChunksInBuffer(void);

	virtual int		GetPercentFull(void);





	// Passive Sink

	virtual	void	Clear(void);

	virtual	CPassiveSinkDataReference *GetDataReference(size_t);

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);

	virtual	void	DataComplete(void);



	// Passive Source

	virtual CPassiveSourceDataReference* GetData(int index,Direction_t dir);

	virtual	void	ReturnData(CPassiveSourceDataReference *r);

	virtual	void	SetupStreamInfo(void);

	virtual	void	SetupStreamFormat(void);


protected:

	virtual	Connection_t	_source_GetConnectionType(void);

	virtual	void			_source_Initialize(void);

	virtual	void			_source_Go(void);

	virtual	void			_source_Terminate(void);

	virtual	void			_source_Wait(void);



	virtual	Connection_t	_sink_GetConnectionType(void);

	virtual	void			_sink_Initialize(void);

	virtual	void			_sink_Go(void);

	virtual	void			_sink_Terminate(void);

	virtual	void			_sink_Wait(void);



	int		ReadyToRoll(long percent);
	void	SetOverflowIgnore(int val);

};



#endif
