/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CCircularBufferComponent.h"

#include "CCircularBuffer.h"



CCircularBufferComponent::CCircularBufferComponent(CGUIController *gui)

: CComponent(gui)

{

	mAudioBuffer = 0;

	mVideoBuffer = 0;

	mAudioSinkGiven = mVideoSinkGiven = 0;

	mAudioSourceGiven = mVideoSourceGiven = 0;

	mAudioBuffer = new CCircularBuffer(CONN_AUDIO,gui,this);

	mVideoBuffer = new CCircularBuffer(CONN_VIDEO,gui,this);



	mAudioPoolSize = AfxGetApp()->GetProfileInt("CCB_Audio","PoolSize",5000 * 1024);

	mAudioIndicies = AfxGetApp()->GetProfileInt("CCB_Audio","Indicies",1000);

	mVideoPoolSize = AfxGetApp()->GetProfileInt("CCB_Video","PoolSize",5000 * 1024);

	mVideoIndicies = AfxGetApp()->GetProfileInt("CCB_Video","Indicies",1000);

}



CCircularBufferComponent::~CCircularBufferComponent(void)

{

	AfxGetApp()->WriteProfileInt("CCB_Audio","PoolSize",mAudioPoolSize);

	AfxGetApp()->WriteProfileInt("CCB_Audio","Indicies",mAudioIndicies);

	AfxGetApp()->WriteProfileInt("CCB_Video","PoolSize",mVideoPoolSize);

	AfxGetApp()->WriteProfileInt("CCB_Video","Indicies",mVideoIndicies);



	delete mAudioBuffer;

	mAudioBuffer = 0;

	delete mVideoBuffer;

	mVideoBuffer = 0;

}



void	

CCircularBufferComponent::Initialize(void)

{

}



void	

CCircularBufferComponent::Cleanup(void)

{

}





CComponent::ConnectionTypeList_t

CCircularBufferComponent::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;



	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

	l.push_back(CONN_PASSIVE | CONN_SOURCE | CONN_VIDEO | CONN_SEEKABLE);

	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_AUDIO);

	l.push_back(CONN_PASSIVE | CONN_SOURCE | CONN_AUDIO | CONN_SEEKABLE);



	return l;

}



nmr_string

CCircularBufferComponent::GetName(void) const

{

	return "Circular Buffer";

}



CConnection *

CCircularBufferComponent::CreateConnection(CConnection::Connection_t t)

{

	CConnection::Connection_t	minimum_requirements = (CONN_PASSIVE);



	if ((t & minimum_requirements) != minimum_requirements)

		return 0;



	if (t & CONN_AUDIO)

		{

		if (t & CONN_SINK)

			{

			if (mAudioSinkGiven) return 0;

			mAudioSinkGiven = 1;

			return 

				dynamic_cast<CConnection*>(

					dynamic_cast<CPassiveSink *>(mAudioBuffer));

			}

		if (mAudioSourceGiven) return 0;

		mAudioSourceGiven = 1;

		return 

			dynamic_cast<CConnection*>(

				dynamic_cast<CPassiveSource*>(mAudioBuffer));

		}

	if (t & CONN_VIDEO)

		{

		if (t & CONN_SINK)

			{

			if (mVideoSinkGiven) return 0;

			mVideoSinkGiven = 1;

			return 

				dynamic_cast<CConnection*>(

					dynamic_cast<CPassiveSink *>(mVideoBuffer));

			}

		if (mVideoSourceGiven) return 0;

		mVideoSourceGiven = 1;

		return 

			dynamic_cast<CConnection*>(

				dynamic_cast<CPassiveSource*>(mVideoBuffer));

		}



	return 0;

}





void

CCircularBufferComponent::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	CCircularBuffer *cb = dynamic_cast<CCircularBuffer *>(c);

	assert(cb);



	if (t & CONN_AUDIO)

		{

		if (t & CONN_SINK)

			{

			mAudioSinkGiven = 0;

			}

		else

			{

			mAudioSourceGiven = 0;

			}

		}

	else if (t & CONN_VIDEO)

		{

		if (t & CONN_SINK)

			{

			mVideoSinkGiven = 0;

			}

		else

			{

			mVideoSourceGiven = 0;

			}

		}

}



int	

CCircularBufferComponent::GetPoolSize(CConnection::Connection_t t)

{

	return ((t & CONN_AUDIO) ? mAudioPoolSize : mVideoPoolSize);

}



int	

CCircularBufferComponent::GetIndicies(CConnection::Connection_t t)

{

	return ((t & CONN_AUDIO) ? mAudioIndicies : mVideoIndicies);

}





