/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CCompressionCoordinator.h"

#include "CServerReceiver.h"

#include "CServerRequester.h"

#include "CCompressorThread.h"

#include "CDistributedCompressor.h"

#include "CDistributedGUIThread.h"

#include <milli_sleep.h>

#include <CLogger.h>

#include <safe_ostrstream.h>

#include <fstream.h>

#include <assert.h>





MD(CCompressionCoordinator,	Init,		CMOS_SYS_INIT_OK);

MD(CCompressionCoordinator,	INT_Error,	CMOS_SYS_INTERNAL_ERROR);

MD(CCompressionCoordinator,	IO_Error,	CMOS_SYS_IO_ERROR);

MD(CCompressionCoordinator,	Close,		CMOS_SYS_FIRST_USER_EVENT);

MD(CCompressionCoordinator,	ServerFound,CMOS_SYS_FIRST_USER_EVENT+1);

MD(CCompressionCoordinator,	Success,	CMOS_SYS_FIRST_USER_EVENT+2);

MD(CCompressionCoordinator,	Failure,	CMOS_SYS_FIRST_USER_EVENT+3);



void

CCompressionCoordinator::CB_Init(CMosMessage *message)

{

	TraceCallback("CB_Init");



	mServerReceiver = new CServerReceiver(this,mMyAddress,mMyUDPPort);

	mServerReceiver->Start();

	mServerRequester = new CServerRequester(mMyAddress,mMyUDPPort);

	mServerRequester->Start();



	mFramesCompleted = 0;

	mFramesInProgress = 0;



	assert(mGUI);

	mGUI->Message_Status("Compressing");

	mGUI->Message_ClearServers();



	mStartTime = 0;

	mTotalDataWritten = 0;

	mTotalFramesCompressed = 0;

}



#ifdef WIN32

void

CCompressionCoordinator::CB_ServerFound(CMosMessage *message)

#else

void

CCompressionCoordinator::CB_ServerFound(CMosMessageType<CServerDesignation> *message)

#endif

{

//	TraceCallback("CB_ServerFound");

	if (!mStartTime)

		mStartTime = timeGetTime();



#ifdef WIN32

	CServerDesignation d = *(((CMosMessageType<CServerDesignation> *)message) ->GetData());

#else

	CServerDesignation d = *(message->GetData());

#endif

	

	ServerDesignationList_t::iterator i = mServerList.find(d);

	if (i == mServerList.end())

		{

		safe_ostrstream o;

		o << ">>>Aquiring server " << d;

		gLogLog->SendMessage(o.str());

		mGUI->Message_AddServer(d);

		mServerList.insert(d);



		// create a new thread and connection

		CompressFrames(d);

		}

}



#ifdef WIN32

void

CCompressionCoordinator::CB_Success(CMosMessage *message)

#else

void

CCompressionCoordinator::CB_Success(CMosMessageType<U32> *message)

#endif

{

	TraceCallback("CB_Success");



#ifdef WIN32

	CCompressorThread	*ct = (CCompressorThread *)

		*((CMosMessageType<U32> *)message)->GetData();

#else

	CCompressorThread	*ct = (CCompressorThread *)*(message->GetData());

#endif



	int	first_frame = ct->GetFirstProcessedFrame();

	CServerDesignation d = ct->GetServerDesignation();

	ct->Join();



	if (first_frame == -1)

		{

		mServerList.erase(d);

		DeleteCompressorThread(ct);

		ct = 0;

		}

	else

		{

		CCompressorThread::TempFileNameList_t nlist = ct->GetTempFileNameList();

		int frame_count = nlist.size();

		mFramesCompleted += frame_count;

		mFramesInProgress -= frame_count;

		mDistributedCompressorStream->DataAvailable(nlist,first_frame);



		ct->DoNotDeleteTempFiles();

		ct->Join();



		mGUI->Message_FramesProcessed(mFramesCompleted);

		int delta_t = timeGetTime() - mStartTime;

		if (delta_t)

			{

			float f = ((float)mFramesCompleted * 1000) / ((float)delta_t);

			mGUI->Message_AvgFPS(f);

			}



		float	thread_fps		= ct->GetChunkFPS();

		int		thread_downlink = ct->GetChunkDownlink();

		mTotalDataWritten += ct->GetCompressedDataSize();

		mTotalFramesCompressed += ct->GetCompressedFrameCount();

		mGUI->Message_UpdateServer(d,0,thread_fps,thread_downlink);

		}



	//mFrameManager.Lock();

	int done = mFrameManager.Done();

	//mFrameManager.Unlock();



	if (done)

		SEND_MESS_Close();

	else if (ct)

		CompressFrames(d,ct);

}



#ifdef WIN32

void

CCompressionCoordinator::CB_Failure(CMosMessage *message)

#else

void

CCompressionCoordinator::CB_Failure(CMosMessageType<U32> *message)

#endif

{

	TraceCallback("CB_Failure");



#ifdef WIN32

	CCompressorThread	*ct = (CCompressorThread *)

		*((CMosMessageType<U32> *)message)->GetData();

#else

	CCompressorThread	*ct = (CCompressorThread *)*(message->GetData());

#endif



	//mFrameManager.Lock();

	CServerDesignation d = mFrameManager.RevertToRaw(ct);

	//mFrameManager.Unlock();





	mGUI->Message_UpdateServer(d,1,0,0);

	::Beep(3000,250);

	mServerList.erase(d);

	ct->Join();

	DeleteCompressorThread(ct);

}





void

CCompressionCoordinator::CB_Close(CMosMessage *message)

{

	TraceCallback("CB_Close");



	CompressorThreadList_t::iterator ii;

	for (ii = mCompressorThreadList.begin(); ii != mCompressorThreadList.end(); ++ii)

		{

		(*ii)->Terminate();

		(*ii)->Join();

		delete (*ii);

		}

	mCompressorThreadList.erase(mCompressorThreadList.begin(),mCompressorThreadList.end());

	mServerRequester->Terminate();

	mServerReceiver->Terminate();

	gTraceLog->SendMessage("klling requester");

	TERMINATE_JOIN_DELETE_CLEAR(mServerRequester);

	gTraceLog->SendMessage("killing receiver");

	TERMINATE_JOIN_DELETE_CLEAR(mServerReceiver);



	Terminate();

	mGUI->Message_Status("Done");

}



#define CCC	CCompressionCoordinator

#define	STATE_INITIAL		0

#define	STATE_OPEN			1

#define	STATE_CLOSE			2



CFsm::state_table_entry_t	CCompressionCoordinator::sStateTable[] = 

{

{STATE_INITIAL,		MESS_Init,			STATE_OPEN,		FCT(CCC::CB_Init)},



{STATE_OPEN,		MESS_ServerFound,	STATE_OPEN,		FCT(CCC::CB_ServerFound)},

{STATE_OPEN,		MESS_Success,		STATE_OPEN,		FCT(CCC::CB_Success)},

{STATE_OPEN,		MESS_Failure,		STATE_OPEN,		FCT(CCC::CB_Failure)},

{STATE_OPEN,		MESS_Close,			STATE_CLOSE,	FCT(CCC::CB_Close)},

{STATE_OPEN,		MESS_INT_Error,		STATE_CLOSE,	FCT(CCC::CB_Close)},

{STATE_OPEN,		MESS_IO_Error,		STATE_CLOSE,	FCT(CCC::CB_Close)}



};



CCompressionCoordinator::CCompressionCoordinator(COMPVARS *compvars,

		StateChangeMap_t &state_map,

		unsigned long ipaddr,int tcp_port,int udp_port,

		int chunk_size,

		int startframe,int endframe,CPassiveSource *source,

		CDistributedCompressorStream *dc,const char *input_format,

		int input_format_size,nmr_string temp_dir,CDistributedGUIThread *gui)

		: CFsmProtocol(200,0),mFrameManager(source)

{

	mGUI			= gui;

	mInputFormat	= input_format;

	mInputFormatSize= input_format_size;

	mTempDir		= temp_dir;

	mDistributedCompressorStream = dc;

	mPassiveSource	= source;

	mStateMap		= state_map;

	mServerReceiver = NULL;

	mServerRequester= NULL;

	mMyAddress		= ipaddr;

	mMyUDPPort		= udp_port;

	mMyTCPPort		= tcp_port;

	mCompVars		= compvars;

	mFrameCounterOffset = (startframe < 0  ? 0 : startframe);



	AttachStateTable(sStateTable,sizeof(sStateTable)/sizeof(state_table_entry_t));

	mChunkSize		= chunk_size;



#ifndef NDEBUG

	mTraceStream = new ofstream("TRACEFILE.TXT");

#endif

}



CCompressionCoordinator::~CCompressionCoordinator(void)

{

	CompressorThreadList_t::iterator ii;

	for (ii = mCompressorThreadList.begin(); ii != mCompressorThreadList.end(); ++ii)

		{

		(*ii)->Terminate();

		(*ii)->Join();

		delete (*ii);

		}

	TERMINATE_JOIN_DELETE_CLEAR(mServerRequester);

	TERMINATE_JOIN_DELETE_CLEAR(mServerReceiver);

#ifndef NDEBUG

	delete mTraceStream;

#endif

}





void

CCompressionCoordinator::CompressFrames(CServerDesignation d,

										CCompressorThread *ct)

{

	if (!ct)

		{

		ct = new CCompressorThread(this,d,

								&mFrameManager,mPassiveSource,

								mCompVars->fccHandler,mGUI,

								mCompVars->lQ,

								mInputFormat,mInputFormatSize,mTempDir);

		mCompressorThreadList.insert(ct);

		}

	ct->Start();

}





#if 0

void

CCompressionCoordinator::CompressFrames(CServerDesignation d,

										CCompressorThread *ct)

{

try {

	mFrameManager.Lock();



	// find the start of a block to compress

	int first_frame = mFrameManager.GetBlockStart();

	if (first_frame < 0)

		{

		mServerList.erase(d);

		DeleteCompressorThread(ct);

		mFrameManager.Unlock();

		return;

		}



	// get size for this chunk based on default chunk size and location of

	// state changes

	int chunk_size = mChunkSize;
	int start_of_next_block = FrameForNextStateBlock(first_frame);

	if (start_of_next_block != 0x7fffffff)
		chunk_size = start_of_next_block - first_frame;



	// the chunk size becomes the distance to the next state change clamped

	// to twice the size of a chunk

	if (chunk_size > (mChunkSize * 2)) chunk_size = mChunkSize;



	int last_frame = mFrameManager.ClampLastFrame(first_frame,first_frame + chunk_size - 1);



	mFramesInProgress += (last_frame - first_frame) + 1;



	DuckStateBlock_t dbs;

	dbs = StateBlockForFrame(first_frame);

	

#ifndef NDEBUG

	safe_ostrstream	oo;

	oo << "Block " << first_frame << " " << last_frame 

		<< " " << dbs.mStateFileName;

	gTraceLog->SendMessage(oo.str());

#endif



	if (!ct)

		{

		ct = new CCompressorThread(this,d,first_frame,last_frame,

								&mFrameManager,mPassiveSource,

								dbs.mDuckState,dbs.mDuckStateSize,

								mCompVars->fccHandler,mGUI,

								mCompVars->lQ,

								mInputFormat,mInputFormatSize,mTempDir);

		mCompressorThreadList.insert(ct);

		}

	else

		ct->ResetFrames(first_frame,last_frame,dbs.mDuckState,dbs.mDuckStateSize);

	

	mFrameManager.MarkInProgress(first_frame,last_frame,ct,d,timeGetTime());



	ct->Start();

	}

	catch(...)

	{

		mFrameManager.Unlock();

		throw;

	}

	mFrameManager.Unlock();

}

#endif



DuckStateBlock_t

CCompressionCoordinator::StateBlockForFrame(int frame)

{

	do

		{

		if (mStateMap.find(frame) != mStateMap.end())

			return mStateMap[frame];

		frame--;

		}

	while (frame >= 0);

	assert(0);

	throw CCompressionCoordinatorException("No state block for frame",frame);

	return mStateMap[0]; // just to shut up the compiler

}



int

CCompressionCoordinator::FrameForNextStateBlock(int current_frame)

{

	int	next_block = 0x7fffffff;



	StateChangeMap_t::iterator i;

	for (i = mStateMap.begin(); i != mStateMap.end(); ++i)

		{

		int	f = (*i).first;

		if (f > current_frame && f < next_block)

			next_block = f;

		}

	return next_block;

}



void	

CCompressionCoordinator::DeleteCompressorThread(CCompressorThread *ct)

{

	delete ct;

	mCompressorThreadList.erase(ct);

}



const char *

CCompressionCoordinator::Name(void)

{

	return "CCompressionCoordinator";

}



int

CCompressionCoordinator::GetChunkSize(void)

{

	return mChunkSize;

}



