/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CCOMPRESSIONCOORDINATOR_H_

#define CCOMPRESSIONCOORDINATOR_H_



#include	<CFsmProtocol.h>

#include	<nmr_string.h>

#include	<CError.h>

#include	<vfw.h>

#include	<set.h>

#include	<vector.h>

#include	<map.h>



#include "CCompressorThread.h"

#include "CServerDesignation.h"

#include "CFrameManager.h"



class CServerReceiver;

class CServerRequester;

class CDistrubutedGUI;

class CAviStreamPassiveSource;

class CDistributedCompressorStream;

class ofstream;



class DuckStateBlock_t

{

public:

	char		*mDuckState;

	int			mDuckStateSize;

	nmr_string	mStateFileName;

};



typedef map<int,DuckStateBlock_t,less<int> > StateChangeMap_t;



/////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////



class CCompressionCoordinator: public CFsmProtocol

{

//	CGUIController	*mGUIController;



#ifndef NDEBUG

	ofstream	*mTraceStream;

#endif

	StateChangeMap_t	mStateMap;



	int			mChunkSize;

	COMPVARS	*mCompVars;

	int			mFramesCompleted;

	int			mFramesInProgress;

	unsigned long	mMyAddress; //host format

	int				mMyTCPPort;

	int				mMyUDPPort;

	CServerReceiver	*mServerReceiver;

	CServerRequester *mServerRequester;

	int				mStartTime;

	double			mTotalDataWritten;

	double			mTotalFramesCompressed;

	int				mFrameCounterOffset;

	CPassiveSource	*mPassiveSource;

	CFrameManager	mFrameManager;

	CDistributedCompressorStream *mDistributedCompressorStream;

	const char		*mInputFormat;

	int				mInputFormatSize;

	nmr_string		mTempDir;

	CDistributedGUIThread	*mGUI;



	typedef set<CServerDesignation,less<CServerDesignation> > ServerDesignationList_t;

	ServerDesignationList_t	mServerList;



	typedef set<CCompressorThread*, less<CCompressorThread*> > CompressorThreadList_t;

	CompressorThreadList_t	mCompressorThreadList;



static state_table_entry_t	sStateTable[];



	void	CB_Init(CMosMessage *message);

	void	CB_Close(CMosMessage *message);

#ifdef WIN32

	void	CB_ServerFound(CMosMessage *message);

	void	CB_Success(CMosMessage *message);

	void	CB_Failure(CMosMessage *message);

#else

	void	CB_ServerFound(CMosMessageType<CServerDesignation> *message);

	void	CB_Success(CMosMessageType<U32> *message);

	void	CB_Failure(CMosMessageType<U32> *message);

#endif



	void	CompressFrames(CServerDesignation d,CCompressorThread *ct = NULL);

	void	DeleteCompressorThread(CCompressorThread *ct);

public:

	CCompressionCoordinator(COMPVARS *compvars,StateChangeMap_t &state_map,

							unsigned long ipaddr,

							int tcp_port,int udp_port,int chunk_size,

							int startframe,int endframe,

							CPassiveSource *source,

							CDistributedCompressorStream *dc,

							const char *input_format,int input_format_size,

							nmr_string temp_dir,CDistributedGUIThread *gui);

	~CCompressionCoordinator(void);

	virtual const char *Name(void);

	DuckStateBlock_t	StateBlockForFrame(int frame);

	int					FrameForNextStateBlock(int current_frame);

	int					GetChunkSize(void);

	MEP(Init);

	MEP(INT_Error);

	MEP(IO_Error);

	MEP(Close);

	MEP1(ServerFound,CServerDesignation);

	MEP1(Failure,U32);

	MEP1(Success,U32);



	CERROR_SUBCLASS(CCompressionCoordinatorException);



};





#endif