/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CCompressor.h"

#include <stack_aids.h>

#include "danbug.h"


CCompressor::CCompressor(CGUIController *gui)

: CComponent(gui)

{

	mCompressorStream = new CCompressorStream(this,gui);

	mCompressorMustBeFreed = 0;

	mCompressorMustBeClosed = 0;

	mHIC = 0;

}



CCompressor::~CCompressor(void)

{

	delete mCompressorStream;

	if (mCompressorMustBeFreed)

		{

		ICCompressorFree(&mCompVars);

		mCompressorMustBeFreed = 0;

		}

	if (mCompressorMustBeClosed)

		{

		ICClose(mCompVars.hic);

		mCompressorMustBeClosed = 0;

		}

}



void

CCompressor::Initialize(void)

{

}



void

CCompressor::Cleanup(void)

{

}





CComponent::ConnectionTypeList_t	

CCompressor::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

	return l;

}



nmr_string 

CCompressor::GetName(void) const

{

	return "Codec Compressor";

}



CConnection *

CCompressor::CreateConnection(CConnection::Connection_t t)

{

	if ((t & (CONN_VIDEO | CONN_PASSIVE | CONN_SINK)) == 

		(CONN_VIDEO | CONN_PASSIVE | CONN_SINK))

		return dynamic_cast<CPassiveSink*>(mCompressorStream);

	if ((t & (CONN_VIDEO | CONN_ACTIVE | CONN_SOURCE)) ==

		(CONN_VIDEO | CONN_ACTIVE | CONN_SOURCE))

		return dynamic_cast<CActiveSource*>(mCompressorStream);

	

	return 0;

}



void	

CCompressor::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	// nothing to be done

	

}



HIC

CCompressor::GetHIC(void)

{

	return mHIC;

}



unsigned long

CCompressor::GetFCCHandler(void)

{

	return mCompVars.fccHandler;

}



int

CCompressor::GetQuality(void)

{

	return mCompVars.lQ;

}



///////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////



CCompressorStream::CCompressorStream(CCompressor *comp,CGUIController *gui)

{

	mGUIController	= gui;

	mComponent		= comp;

	mHIC					= 0;

	mCompressorBeginCalled	= 0;

	mDataBuffer				= 0;

	mDataReference			= 0;

	mDataAvailable			= 0;

	mDataFinished			= 0;

	mOutputBuffer			= 0;

}





CCompressorStream::~CCompressorStream(void)

{

	delete [] mDataBuffer;

	delete [] mOutputBuffer;



	mDataBuffer = 0;

	mOutputBuffer = 0;



	delete mDataReference;

	mDataReference = 0;

}



/////////////



CConnection::Connection_t	

CCompressorStream::_source_GetConnectionType(void)

{

	return (CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

}



void			

CCompressorStream::_source_Initialize(void)

{

	mHIC = mComponent->GetHIC();

	mQuality = mComponent->GetQuality();

	mFCCHandler = mComponent->GetFCCHandler();



	assert(mHIC);

	CActiveSource::Initialize();



	mFrameCounter = 0;

	if (ICCompressBegin(mHIC,CPassiveSink::mStreamInfoObject.GetStreamFormat(),

							CActiveSource::mStreamInfoObject.GetStreamFormat()) != ICERR_OK)

		throw CompressorStreamError("Could not begin compression",0);

	mCompressorBeginCalled = 1;

}



void			

CCompressorStream::_source_Go(void)

{

	CActiveSource::Go();

}



void			

CCompressorStream::_source_Terminate(void)

{

	CActiveSource::Terminate();

}



void			

CCompressorStream::_source_Wait(void)

{

	CActiveSource::Wait();

}



void

CCompressorStream::_source_Cleanup(void)

{

	if (mCompressorBeginCalled)

		ICCompressEnd(mHIC);

	mCompressorBeginCalled = 0;



	CActiveSource::Cleanup();

}



// Active Source

void	

CCompressorStream::_SetInfo(void)

{

	delete [] mDataBuffer;

	BITMAPINFO *bmi =  (BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat();

	mInputSize = bmi->bmiHeader.biSizeImage;

	mDataBuffer = new char[mInputSize];

	delete mDataReference;

	mDataReference = new CPassiveSinkDataReference(mDataBuffer,mInputSize,0,0);



	AVISTREAMINFO i = CPassiveSink::mStreamInfoObject.GetStreamInfo();

	i.fccHandler = mFCCHandler;

	CActiveSource::mStreamInfoObject.SetStreamInfo(i);;



	BITMAPINFO	b;

	memset(&b,0,sizeof(BITMAPINFO));

	b.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

	int ic_err = ICCompressGetFormat(mHIC,CPassiveSink::mStreamInfoObject.GetStreamFormat(),&b);
	if (ic_err != ICERR_OK) {
		dprintf("ICERR = %d -- handle = %x",(int)ic_err,(int)mHIC);
		dprintf("format requested was at %p, here's the data:",CPassiveSink::mStreamInfoObject.GetStreamFormat());
		dprintf("size of BITMAPINFOHEADER: %d\n",sizeof(BITMAPINFOHEADER));
		dumpdata((unsigned char *)CPassiveSink::mStreamInfoObject.GetStreamFormat(), 3);
		throw CompressorStreamError("No valid output format for input",0);
	}
	dprintf("size of BITMAPINFOHEADER: %d\n",sizeof(BITMAPINFOHEADER));
	dumpdata((unsigned char *)CPassiveSink::mStreamInfoObject.GetStreamFormat(), 3);
	mOutputSize = b.bmiHeader.biSizeImage;


 dprintf("calling mStreamInfoObject.SetStreamFormat(1)");
	CActiveSource::mStreamInfoObject.SetStreamFormat((const char *)&b,sizeof(BITMAPINFOHEADER));



	if (mFCCHandler == MKFOURCC('T','R','2','0') ||

		mFCCHandler == MKFOURCC('t','r','2','0'))

		{

		int	bits_per_pixel = 0;

		int interlace = 0;

		if (mQuality >= 8000)

			{

			bits_per_pixel = 4;

			interlace = 1;

			}

		else if (mQuality >= 6000)

			{

			bits_per_pixel = 3;

			interlace = 1;

			}

		else if (mQuality >= 4000)

			{

			bits_per_pixel = 4;

			interlace = 2;

			}

		else if (mQuality >= 2000)

			{

			bits_per_pixel = 3;

			interlace = 2;

			}

		else

			{

			bits_per_pixel = 2;

			interlace = 2;

			}

		mOutputSize =  ((((((b.bmiHeader.biWidth/interlace)*b.bmiHeader.biHeight+

			((b.bmiHeader.biWidth/interlace)/4)*(b.bmiHeader.biHeight/4)*2)*

			bits_per_pixel)/8)+32));

;

		}

	delete [] mOutputBuffer;

	mOutputBuffer = new char[mOutputSize];

}



int

CCompressorStream::_ProcessData(CPassiveSink *s,const char *source,size_t siz)

{

	U32 avi_flags = 0;
 dprintf("CCompressorStream::_ProcessData");
 dtime();
	CPassiveSinkDataReference *ref = 0;



	if (siz == 0)
		{
		if (!(ref = s->GetDataReference(0)))
			{
			s->DataComplete();
			return 0;
			}	
		s->ReturnDataReference(ref);
		}

	// if we know we have realtime we can cheat
	else if (mFCCHandler == MKFOURCC('T','R','2','0') ||
		mFCCHandler == MKFOURCC('t','r','2','0'))
		{
		long datasize = mOutputSize;
		if (!(ref = s->GetDataReference(datasize)))
			{
			s->DataComplete();
			return 0;
			}	

		assert(ref->GetWritePtr());
		assert(source);

		if (ICCompress(mHIC,0,
			&(((BITMAPINFO *)CActiveSource::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			ref->GetWritePtr(), 
			&(((BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			(void *)source,
			0,&avi_flags,mFrameCounter,0,mQuality,
			&(((BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			NULL) != ICERR_OK)
				throw CompressorStreamError("Could not compress frame",mFrameCounter);

		ref->SetFlags(avi_flags);
		s->ReturnDataReference(ref);
		}
	else
		{
		assert(mOutputBuffer);
		ref = new CPassiveSinkDataReference(0,0,0,0);

		if (ICCompress(mHIC,0,
			&(((BITMAPINFO *)CActiveSource::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			mOutputBuffer, 
			&(((BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			(void *)source,
			0,&avi_flags,mFrameCounter,0,mQuality,
			&(((BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat())->bmiHeader),
			NULL) != ICERR_OK)
				throw CompressorStreamError("Could not compress frame",mFrameCounter);

		long datasize = 
			((BITMAPINFO *)CActiveSource::mStreamInfoObject.GetStreamFormat())->bmiHeader.biSizeImage;
		((BITMAPINFO *)CActiveSource::mStreamInfoObject.GetStreamFormat())->bmiHeader.biSizeImage = mOutputSize;

		ref->SetFlags(avi_flags);
		ref->SetWritePtr(mOutputBuffer);
		ref->SetSize(datasize);

 dprintf("   about to call ConsumeDataNow -- for video:\n");
		if (!s->ConsumeDataNow(ref))
			{
			s->DataComplete();
			delete ref;
			return 0;
			}

		delete ref;
		}

	mFrameCounter++;
	mDataAvailable = 0;
	mDataWait.Signal();
	return 1;
}





int		

CCompressorStream::ProcessData(CPassiveSink *s)

{

	stack_MutexLock sml(&mDataMutex);



	while (!mDataAvailable)

		{

		if (mDataFinished) 

			{

			s->DataComplete();

			return 0;

			}

		mDataWait.Wait(mDataMutex);

		}



	return _ProcessData(s,mDataBuffer,mInputSize);

}





//////////////



CConnection::Connection_t	

CCompressorStream::_sink_GetConnectionType(void)

{

	return (CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

}



void			

CCompressorStream::_sink_Initialize(void)

{

	mHIC = mComponent->GetHIC();

	mQuality = mComponent->GetQuality();

	mFCCHandler = mComponent->GetFCCHandler();



	CPassiveSink::Initialize();

	mDataAvailable = 0;

	mDataFinished = 0;

}



void			

CCompressorStream::_sink_Go(void)

{

	CPassiveSink::Go();

}



void			

CCompressorStream::_sink_Terminate(void)

{

	DataComplete();

	CPassiveSink::Terminate();

}



void			

CCompressorStream::_sink_Wait(void)

{

	CPassiveSink::Wait();

}



void

CCompressorStream::_sink_Cleanup(void)

{

	CPassiveSink::Cleanup();

}





int

CCompressorStream::ConsumeDataNow(const CPassiveSinkDataReference *r)

{

	stack_MutexLock sml(&mDataMutex);

	assert(r->GetSize() == mInputSize || r->GetSize() == 0);



	if (mDataFinished) return 0;



	while (mDataAvailable)

		mDataWait.Wait(mDataMutex);



	return _ProcessData(mSink,r->GetWritePtr(),r->GetSize());

}



CPassiveSinkDataReference *

CCompressorStream::GetDataReference(size_t s)

{

	assert(s == mInputSize);

	stack_MutexLock sml(&mDataMutex);



	if (mDataFinished) return 0;



	while (mDataAvailable)

		mDataWait.Wait(mDataMutex);



	return mDataReference;

}



void	

CCompressorStream::ReturnDataReference(CPassiveSinkDataReference *r)

{

	stack_MutexLock sml(&mDataMutex);

	

	if (!mDataFinished)

		mDataAvailable = 1;

	mDataWait.Signal();

}



void	

CCompressorStream::DataComplete(void)

{

	stack_MutexLock sml(&mDataMutex);

	mDataFinished = 1;

	mDataWait.Broadcast();

}





