/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CCOMPRESSOR_H_

#define CCOMPRESSOR_H_



#include "CComponent.h"

#include "CActiveSource.h"

#include "CPassiveSink.h"

#include <CCMutex.h>

#include <CConditionVariable.h>



class CCompressorStream;



class CCompressor: public CComponent

{

protected:

	CCompressorStream *mCompressorStream;

	COMPVARS			mCompVars;

	int					mCompressorMustBeFreed;

	int					mCompressorMustBeClosed;

	HIC					mHIC;



public:

	CCompressor(CGUIController *gui);

	virtual ~CCompressor(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void ReturnConnection(CConnection *c,CConnection::Connection_t t);



	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;



	HIC	GetHIC(void);

	unsigned long GetFCCHandler(void);

	int GetQuality(void);

};



/////////////////////////////////////////////////

///////////////////////////////////////////////////



class CCompSource: public CActiveSource

{

protected:

	virtual Connection_t _source_GetConnectionType(void) = 0;

	virtual	void		 _source_Initialize(void) = 0;

	virtual	void		 _source_Go(void) = 0;

	virtual	void		 _source_Terminate(void) = 0;

	virtual	void		 _source_Wait(void) = 0;

	virtual	void		 _source_Cleanup(void) = 0;

public:

	virtual	Connection_t	GetConnectionType(void)

		{ return _source_GetConnectionType(); }

	virtual	void	Initialize(void)

		{ _source_Initialize(); }

	virtual	void	Go(void) 

		{ _source_Go(); }

	virtual	void	Terminate(void)

		{ _source_Terminate(); }

	virtual	void	Wait(void)

		{ _source_Wait(); }

	virtual	void	Cleanup(void)

		{ _source_Cleanup(); }



};



class CCompSink: public CPassiveSink

{

protected:

	virtual Connection_t _sink_GetConnectionType(void) = 0;

	virtual	void		 _sink_Initialize(void) = 0;

	virtual	void		 _sink_Go(void) = 0;

	virtual	void		 _sink_Terminate(void) = 0;

	virtual	void		 _sink_Wait(void) = 0;

	virtual	void		 _sink_Cleanup(void) = 0;



public:

	// connection

	virtual	Connection_t	GetConnectionType(void)

				{ return _sink_GetConnectionType(); }

	virtual	void	Initialize(void)

				{ _sink_Initialize(); }

	virtual	void	Go(void) 

				{ _sink_Go(); }

	virtual	void	Terminate(void)

				{ _sink_Terminate(); }

	virtual	void	Wait(void)

				{ _sink_Wait(); }

	virtual	void	Cleanup(void)

				{ _sink_Cleanup(); }

};



/////////////////////////////

/////////////////////////////



class CCompressorStream: public CCompSink, public CCompSource

{

protected:

	CGUIController		*mGUIController;

	CCompressor			*mComponent;

	int					mCompressorBeginCalled;

	char				*mDataBuffer;

	CPassiveSinkDataReference	*mDataReference;

	int					mOutputSize,mInputSize;

	CCMutex				mDataMutex;

	CConditionVariable	mDataWait;

	int					mDataAvailable;

	int					mDataFinished;

	char				*mOutputBuffer;

	int					mFrameCounter;

	HIC					mHIC;

	int					mQuality;

	unsigned long		mFCCHandler;



	CCompressorStream(CCompressor *comp,CGUIController *gui);

	virtual ~CCompressorStream(void);



	virtual	Connection_t	_source_GetConnectionType(void);

	virtual	void			_source_Initialize(void);

	virtual	void			_source_Go(void);

	virtual	void			_source_Terminate(void);

	virtual	void			_source_Wait(void);

	virtual	void			_source_Cleanup(void);



	virtual	Connection_t	_sink_GetConnectionType(void);

	virtual	void			_sink_Initialize(void);

	virtual	void			_sink_Go(void);

	virtual	void			_sink_Terminate(void);

	virtual	void			_sink_Wait(void);

	virtual	void			_sink_Cleanup(void);



	// Passive Sink

	virtual	CPassiveSinkDataReference *GetDataReference(size_t);

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	void	DataComplete(void);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);



	// Active Source

	virtual	void	_SetInfo(void);

			int		_ProcessData(CPassiveSink *s,const char *data,size_t siz);

	virtual	int		ProcessData(CPassiveSink *s);



	friend CCompressor;



	CERROR_SUBCLASS(CompressorStreamError);

};





#endif

