/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CCompressorThread.h"

#include "TCPPacketTypes.h"

#include "CPassiveSource.h"

#include <CLogger.h>

#include <safe_ostrstream.h>

#include "CDistributedGUIThread.h"

#include "CCompressionCoordinator.h"

#include <stack_aids.h>



CCompressorThread::CCompressorThread(CCompressionCoordinator *supervisor,

						CServerDesignation d,

						CFrameManager *frame_manager,CPassiveSource *source,

						DWORD fcchandler,

						CDistributedGUIThread *gui,int quality,

						const char *stream_format,int stream_format_size,

						nmr_string tempdir)

{

	mStreamFormat		= stream_format;

	mStreamFormatSize	= stream_format_size;

	mPassiveSource		= source;

	mFrameManager		= frame_manager;

	mSupervisor			= supervisor;

	mServerDesignation	= d;

	mSocket				= NULL;

	mFccHandler			= fcchandler;

	mDeleteTempFiles	= 1;

	mAbort				= 0;

	mGUI				= gui;

	mQuality			= quality;

	mTempDir			= tempdir;

}



void

CCompressorThread::ResetFrames(int first_frame,int last_frame,char *duck_state,

							   int duck_state_size)

{

	mFirstFrame = first_frame;

	mLastFrame = last_frame;

	mDuckState = duck_state;

	mDuckStateSize = duck_state_size;

}



CCompressorThread::~CCompressorThread(void)

{

	stack_MutexLock sml(&mSocketMutex);



	try { delete mSocket;} catch(...) { }



	mSocket = NULL;



	if (mDeleteTempFiles)

		{

		// delete temp files

		TempFileNameList_t::iterator i;

		for (i = mTempFileNameList.begin(); i != mTempFileNameList.end(); ++i)

			remove(*i);

		}

}





int

CCompressorThread::GetFrameBlock(void)

{

try {

	mFrameManager->Lock();



	// find the start of a block to compress

	if ((mFirstFrame = mFrameManager->GetBlockStart()) < 0)

		{

		mFrameManager->Unlock();

		return 0;

		}



	// get size for this chunk based on default chunk size and location of

	// state changes

	int chunk_size = mSupervisor->GetChunkSize();

	int start_of_next_block = mSupervisor->FrameForNextStateBlock(mFirstFrame);

	if (start_of_next_block != 0x7fffffff)

		chunk_size = start_of_next_block - mFirstFrame;



	// the chunk size becomes the distance to the next state change clamped

	// to twice the size of a chunk

	if (chunk_size > (mSupervisor->GetChunkSize() * 2)) 

		chunk_size = mSupervisor->GetChunkSize();



	mLastFrame = mFrameManager->ClampLastFrame(mFirstFrame,mFirstFrame + chunk_size - 1);



	DuckStateBlock_t dbs;

	dbs = mSupervisor->StateBlockForFrame(mFirstFrame);

	

#ifndef NDEBUG

	safe_ostrstream	oo;

	oo << "Block " << mFirstFrame << " " << mLastFrame 

		<< " " << dbs.mStateFileName;

	gTraceLog->SendMessage(oo.str());

#endif



	mDuckState			= dbs.mDuckState;

	mDuckStateSize		= dbs.mDuckStateSize;



	mFrameManager->MarkInProgress(mFirstFrame,mLastFrame,this,mServerDesignation,timeGetTime());



	}

	catch(...)

	{

		mFrameManager->Unlock();

		throw;

	}

	mFrameManager->Unlock();

	return 1;

}



void

CCompressorThread::Run(void)

{

	if (!GetFrameBlock())

		{

		mFirstFrame = -1;

		mLastFrame = -1;

		mSupervisor->SEND_MESS_Success(new U32((U32)this));

		return;

		}



	int locked = 0;

	int failure = 0;

	char *frame_data = NULL;

	const char *input_data = NULL;

	I32 input_data_size = 0;

	int	framemanagerlocked = 0;



	safe_ostrstream oo;

	oo << "CompressorThread " << mServerDesignation << " is running";

	gTraceLog->SendMessage(oo.str());

	mCompressedDataSize = 0;

	mCompressedFrameCount = 0;



	mTempFileNameList.erase(mTempFileNameList.begin(),mTempFileNameList.end());



	try

		{

		mFrameStartTime = mDownlinkStartTime = timeGetTime();

		if (!mSocket)

			{

			// new connection

				{

				stack_MutexLock sml(&mSocketMutex);

				mSocket = new CTCPSocket;

				}

			mSocket->SetOption_Linger(1,60);

			mSocket->Connect(mServerDesignation.mAddress,mServerDesignation.mPort);

			gTraceLog->SendMessage("CCompressorThread connected");

			}



		I16 frame_count = mLastFrame - mFirstFrame + 1;

		long max_frame_size = MAX_FRAME_SIZE;

		I32 frame_size = 0;

		I32 x;

		frame_data = new char[max_frame_size];



		if (mAbort) throw AbortException("",0);

		// send quality and fourCC (long)

		(*mSocket) << mQuality << mFccHandler;



		// send the bitmap info stuff

		BITMAPINFO binfo;

		if (sizeof(binfo) < mStreamFormatSize)

			memcpy(&binfo,mStreamFormat,sizeof(binfo));

		else

			memcpy(&binfo,mStreamFormat,mStreamFormatSize);

		mSocket->Write((char *)&binfo,sizeof(binfo));



		// then send the duck state size (short)

		(*mSocket) << mDuckStateSize;



		// then send the duck state (variable)

		mSocket->Write((char *)mDuckState,mDuckStateSize);



		// now send the frame count (short)

		(*mSocket) << frame_count;

		mCompressedFrameCount = frame_count;



		for (x = mFirstFrame; x <= mLastFrame; ++x)

			{

			// load up the frame

			if (mAbort) throw AbortException("",0);



			char buf[100];

			sprintf(buf,"Transmitting frame %d",x);

			mGUI->Message_Activity(buf);

			gTraceLog->SendMessage(buf);



			//mFrameManager->Lock();

			//framemanagerlocked = 1;

			U32 flags;

			size_t sss;

			mFrameManager->GetDataReference(x)->

									RedeemData(input_data,sss,flags);

			input_data_size = sss;

			//mFrameManager->Unlock();

			//framemanagerlocked = 0;



			// send the frame number and frame size(long)

			(*mSocket) << x << ((I32)(input_data_size));



			// send the frame

			mSocket->Write((char *)(input_data),input_data_size);



			Yield();

			}



		mDownlinkEndTime = timeGetTime();



		// now get the results

		for (x = mFirstFrame; x <= mLastFrame; ++x)

			{

			// read frame number

			I32 tmp;

			U32 key_frame;



			char buf[100];

			sprintf(buf,"Receiving frame %d",x);

			mGUI->Message_Activity(buf);



			if (mAbort) throw AbortException("",0);

			(*mSocket) >> tmp >> key_frame >> frame_size;



			if (x != tmp) throw CompressException("Bad sequence order",0);

			if (frame_size > max_frame_size) throw CompressException("Frame too big",frame_size);



			// get the frame

			mSocket->Read((char *)frame_data,frame_size);

			mCompressedDataSize += frame_size;

			

			nmr_string filename = mTempDir;

			filename += "\\ctmp";

			filename += x;

			WriteTempFile(filename,key_frame,frame_size,frame_data);



			safe_ostrstream o;

			o << "Wrote data to temp file " << filename;

			gTraceLog->SendMessage(o.str());

			mTempFileNameList.push_back(filename);			

			Yield();

			}



		}

	catch(CError &err)

		{

		failure = 1;

		safe_ostrstream oo;

		oo << err;

		gErrorLog->SendMessage(oo.str());

		}

	catch(...)

		{

		failure = 1;

		gErrorLog->SendMessage("Unknown exception in CCompressorThread");

		}

	

	if (framemanagerlocked)

		mFrameManager->Unlock();

	delete [] frame_data;

	frame_data = NULL;

	mFrameEndTime = timeGetTime();



	if (failure)

		{

		// send failure message to supervisor

		mSupervisor->SEND_MESS_Failure(new U32((U32)this));

		}

	else

		{

		// send success message to supervisor

#ifndef NDEBUG

		{

		safe_ostrstream o;

		o << "Server " << mServerDesignation << " was succesfull, removing frames " 

			<< mFirstFrame << " through " << mLastFrame;

		gTraceLog->SendMessage(o.str());

		}

#endif

		//mFrameManager->Lock();

		mFrameManager->RemoveFinishedFrames(mFirstFrame,(mLastFrame - mFirstFrame) + 1,this);

		//mFrameManager->Unlock();

#ifndef NDEBUG

		{

		safe_ostrstream o;

		o << "Server " << mServerDesignation << " frames removed";

		gTraceLog->SendMessage(o.str());

		}

#endif

		mSupervisor->SEND_MESS_Success(new U32((U32)this));

		}

#ifndef NDEBUG

	{

	safe_ostrstream ooo;

	ooo << "CompressorThread " << mServerDesignation << " is exiting";

	gTraceLog->SendMessage(ooo.str());

	}

#endif

}



const char *

CCompressorThread::Name(void)

{

	return "CCompressorThread";

}



CCompressorThread::TempFileNameList_t	

CCompressorThread::GetTempFileNameList(void)

{

	return mTempFileNameList;

}



void

CCompressorThread::DoNotDeleteTempFiles(void)

{

	mDeleteTempFiles = 0;

}



void

CCompressorThread::WriteTempFile(nmr_string filename,U32 key_frame,

								 I32 frame_size,const char *frame_data)

{

	FILE *f = fopen(filename,"wb");

	if (!f) throw CompressException("Could not open temp file",0);

	if (fwrite(&key_frame,1,sizeof(key_frame),f) != sizeof(key_frame))

		throw CompressException("Error writing data to temp file",frame_size);

	if (fwrite(&frame_size,1,sizeof(frame_size),f) != sizeof(frame_size))

		throw CompressException("Error writing data to temp file",frame_size);

	if (fwrite(frame_data,1,frame_size,f) != frame_size)

		throw CompressException("Error writing data to temp file",frame_size);

	fclose(f);

}



float

CCompressorThread::GetChunkFPS(void)

{

	int delta_t = mFrameEndTime - mFrameStartTime;

	if (delta_t == 0) delta_t = 1;



	return ((float)((mLastFrame - mFirstFrame + 1) * 1000)) / delta_t;

}



int

CCompressorThread::GetChunkDownlink(void)

{

	return mDownlinkEndTime - mDownlinkStartTime;

}



long	

CCompressorThread::GetCompressedDataSize(void)

{

	return mCompressedDataSize;

}



long	

CCompressorThread::GetCompressedFrameCount(void)

{

	return mCompressedFrameCount;

}



int

CCompressorThread::GetFirstProcessedFrame(void)

{

	return mFirstFrame;

}



void

CCompressorThread::Terminate(void)

{

	mAbort = 1;

	stack_MutexLock sml(&mSocketMutex);

	mSocket->BreakBlockingIO(this);

}



CServerDesignation

CCompressorThread::GetServerDesignation(void)

{

	return mServerDesignation;

}

