/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CCOMPRESSORTHREAD_H_

#define CCOMPRESSORTHREAD_H_



#include <CCThread.h>

#include <CTCPSocket.h>

#include <CError.h>

#include <nmr_string.h>

#include <list.h>



#include "CServerDesignation.h"



class CCompressionCoordinator;

class CTCPSocket;

class CPassiveSource;

class CFrameManager;

class CDistributedGUIThread;



class CCompressorThread: public CCThread

{

public:

	typedef list<nmr_string>	TempFileNameList_t;

private:

	

	int						mAbort;

	CDistributedGUIThread			*mGUI;

	int						mDeleteTempFiles;

	CCompressionCoordinator	*mSupervisor;

	CServerDesignation		mServerDesignation;

	int						mFirstFrame;

	int						mLastFrame;

	CTCPSocket				*mSocket;

	CCMutex					mSocketMutex;

	char					*mDuckState;

	I16						mDuckStateSize;

	U32						mFccHandler;

	int						mFrameStartTime;

	int						mFrameEndTime;

	int						mDownlinkStartTime;

	int						mDownlinkEndTime;

	I32						mQuality;

	long					mCompressedDataSize;

	long					mCompressedFrameCount;

	CPassiveSource			*mPassiveSource;

	CFrameManager			*mFrameManager;

	const char				*mStreamFormat;

	int						mStreamFormatSize;

	nmr_string				mTempDir;



	TempFileNameList_t			mTempFileNameList;



	virtual void Run(void);

			void WriteTempFile(nmr_string filename,U32 key_frame,

								 I32 frame_size,const char *frame_data);

			int	GetFrameBlock(void);

public:

	CCompressorThread(CCompressionCoordinator *supervisor,

						CServerDesignation d,

						CFrameManager *frame_manager,CPassiveSource *passive_source,

						DWORD fcchandler,

						CDistributedGUIThread *gui,int quality,

						const char *stream_format,int stream_format_size,

						nmr_string tempdir);

	~CCompressorThread(void);



	virtual const char *Name(void);

	TempFileNameList_t	GetTempFileNameList(void);

	void	DoNotDeleteTempFiles(void);

	void	ResetFrames(int first_frame,int last_frame,char *duck_state,

							   int duck_state_size);

	float	GetChunkFPS(void);

	int		GetChunkDownlink(void);

	long	GetCompressedDataSize(void);

	long	GetCompressedFrameCount(void);

	int		GetFirstProcessedFrame(void);

	void	Terminate(void);

	CServerDesignation	GetServerDesignation(void);



	CERROR_SUBCLASS(CompressException);

	CERROR_SUBCLASS(AbortException);

};



#endif

