/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CDistributedCompressor.h"

#include "CDistributionSettingsDlog.h"

#include "CStateFilePromptDialog.h"

#include "CCompressionCoordinator.h"

#include "CGUIController.h"

#include "CDistributedGUIThread.h"

#include <stack_aids.h>

#include <ip_utils.h>



#define DEFAULT_TCPPORT	3579

#define DEFAULT_UDPPORT	3578





CDistributedCompressor::CDistributedCompressor(CGUIController *gui)

: CComponent(gui)

{

	mDistributedGUIThread = 0;

	mCompressorStream = new CDistributedCompressorStream(this,gui);

	mConfigurationNeeded = 1;



	char	hostname[1024];

	unsigned long addr = 0;

	if (gethostname(hostname,1024) != -1)

			addr = FindMyIPAddress(hostname);



	mTCPPort		= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"TCPPort",DEFAULT_TCPPORT);

	mUDPPort		= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"UDPPort",DEFAULT_UDPPORT);

	mIPAddr			= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"IPAddr",addr);

	mTmpDir			= AfxGetApp()->GetProfileString("CDistributedCompressor",

												"TmpDir","c:\\tmp");

	mChunkSize		= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"ChunkSize",0);

	mEndFrame		= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"EndFrame",-1);

	mStartFrame		= AfxGetApp()->GetProfileInt("CDistributedCompressor",

												"StartFrame",-1);

}



CDistributedCompressor::~CDistributedCompressor(void)

{

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","TCPPort",mTCPPort);

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","UDPPort",mUDPPort);

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","IPAddr",mIPAddr);

	AfxGetApp()->WriteProfileString("CDistributedCompressor","TmpDir",mTmpDir);

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","ChunkSize",mChunkSize);

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","EndFrame",mEndFrame);

	AfxGetApp()->WriteProfileInt("CDistributedCompressor","StartFrame",mStartFrame);



	delete mCompressorStream;

	ClearStateMap();

	if (mDistributedGUIThread)

		mDistributedGUIThread->DestroySelf();

	mDistributedGUIThread = 0;

}





void

CDistributedCompressor::Initialize(void)

{

	if (mDistributedGUIThread)

		mDistributedGUIThread->DestroySelf();

	mDistributedGUIThread = dynamic_cast<CDistributedGUIThread*>(

		AfxBeginThread(RUNTIME_CLASS(CDistributedGUIThread)));

	assert(mDistributedGUIThread);

	//mDistributedGUIThread = dynamic_cast<CDistributedGUIThread*>(

	//	RUNTIME_CLASS(CDistributedGUIThread)->CreateObject());

}



void

CDistributedCompressor::Cleanup(void)

{

	if (mDistributedGUIThread)

		mDistributedGUIThread->DestroySelf();



	mDistributedGUIThread = 0;

}





CComponent::ConnectionTypeList_t	

CDistributedCompressor::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_PASSIVE | CONN_SOURCE | CONN_VIDEO);

	l.push_back(CONN_ACTIVE | CONN_SINK | CONN_VIDEO);

	return l;

}



nmr_string 

CDistributedCompressor::GetName(void) const

{

	return "Distributed Compressor";

}



CConnection *

CDistributedCompressor::CreateConnection(CConnection::Connection_t t)

{

	if ((t & (CONN_VIDEO | CONN_ACTIVE | CONN_SINK)) == 

		(CONN_VIDEO | CONN_ACTIVE | CONN_SINK))

		return dynamic_cast<CActiveSink*>(mCompressorStream);

	if ((t & (CONN_VIDEO | CONN_PASSIVE | CONN_SOURCE)) ==

		(CONN_VIDEO | CONN_PASSIVE | CONN_SOURCE))

		return dynamic_cast<CPassiveSource*>(mCompressorStream);

	

	return 0;

}



void	

CDistributedCompressor::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	// nothing to be done

	

}





void

CDistributedCompressor::ClearStateMap(void)

{

	StateChangeMap_t::iterator i;

	for (i = mStateMap.begin(); i != mStateMap.end(); ++i)

		delete [] (*i).second.mDuckState;

		

	mStateMap.erase(mStateMap.begin(),mStateMap.end());

}



int

CDistributedCompressor::LoadStateData(void)

{

	CString fname;



	if (MessageBox(NULL,"Do you want to load a per frame state file?",

			"TrueMotion 2 special settings",MB_DEFBUTTON2 | MB_ICONQUESTION | MB_APPLMODAL | MB_YESNO) != IDYES)

			return 1;



	while (1)

		{

			{

			CStateFilePromptDialog fpd;

			if (fpd.DoModal() != IDOK) return 1;

			fname = fpd.mFilename;

			}



		struct stat buf;

		if (stat(fname,&buf) == -1)

			::MessageBox(NULL,"File not found","File not found" ,MB_OK | MB_ICONWARNING | MB_APPLMODAL);

		else

			break;

		}



	ClearStateMap();

	HIC hic = ICOpen(MKFOURCC('V','I','D','C'),mCompVars.fccHandler,ICMODE_COMPRESS);

	ifstream f(fname);

	while (f)

		{

		int frame_number;

		char	state_file[256];

		f >> frame_number >> state_file;



		if (!state_file[0] || (!f)) break;



		if(ICSendMessage(hic,DRV_USER + 0x100,(unsigned long)state_file,0) != ICERR_OK)

			{

			CString s = "Could not find file ";

			s += state_file;

			::MessageBox(NULL,s,"File not found",MB_OK | MB_ICONWARNING | MB_APPLMODAL);

			return 0;

			}



		DuckStateBlock_t dsb;

		dsb.mStateFileName = state_file;

		dsb.mDuckStateSize = 0;

		dsb.mDuckStateSize = ICGetState(hic,NULL,dsb.mDuckStateSize);

		dsb.mDuckState = new char[dsb.mDuckStateSize];

		ICGetState(hic,dsb.mDuckState,dsb.mDuckStateSize);

#if 0

		// This is commented out because our codec does not return the

		// correct error code here. It could, but videdit expects the

		// the wrong code and, to be compatible with it, we must err also.



		if (ICGetState(hic,dsb.mDuckState,dsb.mDuckStateSize) != ICERR_OK)

			{

			CString s = "Get state error on file ";

			s += state_file;

			::MessageBox(NULL,s,"File not found",MB_OK | MB_ICONWARNING | MB_APPLMODAL);

			return 0;

			}

#endif

		mStateMap[frame_number] = dsb;

		}

	ICClose(hic);

	if (mStateMap.size() == 0) return 0;

	return 1;

}



const COMPVARS*

CDistributedCompressor::GetCompvars(void)

{

	return &mCompVars;

}



///////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////



CDistributedCompressorStream::CDistributedCompressorStream(CDistributedCompressor *comp,CGUIController *gui)

{

	mGUIController		= gui;

	mComponent			= comp;

	mFrameSourceHasStopped = 0;

}





CDistributedCompressorStream::~CDistributedCompressorStream(void)

{

	stack_MutexLock sml(&mAvailableFramesMutex);

	AvailableFrames_t::iterator i;

	for (i = mAvailableFrames.begin(); i != mAvailableFrames.end(); ++i)

		remove((*i).second);

}



void			

CDistributedCompressorStream::DataAvailable(

				CCompressorThread::TempFileNameList_t l,int start_frame)

{

	stack_MutexLock sml(&mAvailableFramesMutex);



	CCompressorThread::TempFileNameList_t::iterator i;

	int x;

	for (i = l.begin(),x = start_frame; i != l.end(); ++i,++x)

		{

		assert(mAvailableFrames.find(x) == mAvailableFrames.end());

		mAvailableFrames[x] = (*i);

		}



	mAvailableFramesWait.Signal();

}



/////////////



CConnection::Connection_t	

CDistributedCompressorStream::_source_GetConnectionType(void)

{

	return (CONN_PASSIVE | CONN_SOURCE | CONN_VIDEO);

}



void			

CDistributedCompressorStream::_source_Initialize(void)

{

	CPassiveSource::Initialize();

	mNextFrame = 0;

}



void			

CDistributedCompressorStream::_source_Go(void)

{

	CPassiveSource::Go();

}



void			

CDistributedCompressorStream::_source_Terminate(void)

{

	CPassiveSource::Terminate();

}



void			

CDistributedCompressorStream::_source_Wait(void)

{

	CPassiveSource::Wait();

}



void

CDistributedCompressorStream::_source_Cleanup(void)

{

	CPassiveSource::Cleanup();

}



void

CDistributedCompressorStream::SetupStreamInfo(void)

{

}



void

CDistributedCompressorStream::SetupStreamFormat(void)

{

}



CPassiveSourceDataReference* 

CDistributedCompressorStream::GetData(int index,Direction_t dir)

{

	assert(index == 1 && dir == DIR_FORWARD);

	CDCDataReference *dr = NULL;



	stack_MutexLock sml(&mAvailableFramesMutex);



	while(mAvailableFrames.find(mNextFrame) == mAvailableFrames.end())

		{

		if (mFrameSourceHasStopped) break;

		mAvailableFramesWait.Wait(mAvailableFramesMutex);

		}

	if (mAvailableFrames.find(mNextFrame) != mAvailableFrames.end())

		{

		dr = new CDCDataReference(mNextFrame,mAvailableFrames[mNextFrame]);

		mNextFrame++;

		}

	return dr;

}



void		

CDistributedCompressorStream::ReturnData(CPassiveSourceDataReference *r)

{

	stack_MutexLock sml(&mAvailableFramesMutex);



	CDCDataReference *dr = dynamic_cast<CDCDataReference*>(r);

	assert(dr);

	assert(mAvailableFrames.find(dr->GetFrameNumber()) != mAvailableFrames.end());

	mAvailableFrames.erase(dr->GetFrameNumber());

	remove(dr->GetFileName());

	delete dr;

}





//////////////



CConnection::Connection_t	

CDistributedCompressorStream::_sink_GetConnectionType(void)

{

	return (CONN_ACTIVE | CONN_SINK | CONN_VIDEO);

}



void			

CDistributedCompressorStream::_sink_Initialize(void)

{

	// inherited method gets video format information from source

	CActiveSink::Initialize();



	mFrameSourceHasStopped = 0;

	// create the output format

	mCompressionCoordinator = 0;

	HIC hic = ICOpen(MKFOURCC('V','I','D','C'),

		mComponent->GetCompvars()->fccHandler,ICMODE_COMPRESS);



	BITMAPINFO	binfo_out;

	memset(&binfo_out,0,sizeof(binfo_out));

	binfo_out.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

	if (ICCompressGetFormat(hic,CActiveSink::mStreamInfoObject.GetStreamFormat(),

								&binfo_out) != ICERR_OK)

		throw DistributedCompressorStreamError("Cant compress format",0);



	CPassiveSource::mStreamInfoObject.SetStreamFormat((char *)&binfo_out,sizeof(BITMAPINFO));



	AVISTREAMINFO i = CActiveSink::mStreamInfoObject.GetStreamInfo();

	i.fccHandler = mComponent->GetCompvars()->fccHandler;

	CPassiveSource::mStreamInfoObject.SetStreamInfo(i);



	ICClose(hic);

}



void			

CDistributedCompressorStream::_sink_Go(void)

{

	CActiveSink::Go();

}



void			

CDistributedCompressorStream::_sink_Terminate(void)

{

//	DataComplete();

	mCompressionCoordinatorMutex.Lock();

	if (mCompressionCoordinator)

		mCompressionCoordinator->SEND_MESS_Close();

	mCompressionCoordinatorMutex.Unlock();

	CActiveSink::Terminate();

}



void			

CDistributedCompressorStream::_sink_Wait(void)

{

	CActiveSink::Wait();

}



void

CDistributedCompressorStream::_sink_Cleanup(void)

{

	CActiveSink::Cleanup();

}



int		

CDistributedCompressorStream::ProcessData(CPassiveSource *s)

{	

try {

	assert(!mCompressionCoordinator);



	int chunk_size = mComponent->mChunkSize;

	if (!chunk_size)

		{

		HIC hic = ICOpen(MKFOURCC('V','I','D','C'),

			mComponent->GetCompvars()->fccHandler,ICMODE_COMPRESS);

		int key_frame_rate = ICGetDefaultKeyFrameRate(hic);

		chunk_size = key_frame_rate * 4;

		ICClose(hic);

		}

	mCompressionCoordinator = new CCompressionCoordinator(&(mComponent->mCompVars),

		mComponent->mStateMap,mComponent->mIPAddr,mComponent->mTCPPort,

		mComponent->mUDPPort,chunk_size,mComponent->mStartFrame,

		mComponent->mEndFrame,s,this,CActiveSink::mStreamInfoObject.GetStreamFormat(),

		CActiveSink::mStreamInfoObject.GetStreamFormatSize(),

		(const char *)mComponent->mTmpDir,mComponent->mDistributedGUIThread);

	

	mCompressionCoordinator->Start();

	mCompressionCoordinator->Join();

	}

catch(...)

	{

	}

	stack_MutexLock	sml(&mCompressionCoordinatorMutex);



	mFrameSourceHasStopped = 1;

	mAvailableFramesWait.Signal();

	delete mCompressionCoordinator;

	mCompressionCoordinator = 0;

	return 0;

}



///////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////



CDCDataReference::CDCDataReference(int frame_number,nmr_string filename)

: CPassiveSourceDataReference(0,0,0,1)



{

	mFilename	= filename;

	mFrameNumber= frame_number;

	mData		= 0;

	mDataLen	= 0;

	FILE *f		= 0;

try {

	f = fopen(filename,"rb");

	I32 key_frame,frame_size;

	if (fread((char *)&key_frame,1,sizeof(I32),f) != sizeof(I32))

		throw CDCDataReferenceError("Could not read temp file",0);

	if (fread((char *)&frame_size,1,sizeof(I32),f) != sizeof(I32))

		throw CDCDataReferenceError("Could not read temp file",0);

	mDataLen= frame_size;

	mData = new char[frame_size];

	mFlags = key_frame;

	if (fread(mData,1,mDataLen,f) != mDataLen)

		throw CDCDataReferenceError("Could not read temp file",0);

	fclose(f);

	}

catch(...)

	{

	delete [] mData;

	if (f) fclose(f);

	throw;

	}

}



int	

CDCDataReference::GetFrameNumber(void)

{

	return mFrameNumber;

}



nmr_string 

CDCDataReference::GetFileName(void)

{

	return mFilename;

}





