/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CDISTRIBUTEDCOMPRESSOR_H_

#define CDISTRIBUTEDCOMPRESSOR_H_



#include "CComponent.h"

#include "CPassiveSource.h"

#include "CActiveSink.h"

#include "CCompressorThread.h"

#include <CCMutex.h>

#include <CConditionVariable.h>



#include "CCompressionCoordinator.h"



#include <map.h>



class CDistributedCompressorStream;

class CCompressionCoordinator;

class CDistributedGUIThread;



class CDistributedCompressor: public CComponent

{

protected:

	CDistributedCompressorStream *mCompressorStream;

	int	mConfigurationNeeded;

	int		mTCPPort;

	int		mUDPPort;

	unsigned long	mIPAddr;

	CString	mTmpDir;

	int		mChunkSize;

	int		mStartFrame;

	int		mEndFrame;

	int		mKeyFrameRate;

	StateChangeMap_t	mStateMap;

	COMPVARS	mCompVars;

	CDistributedGUIThread	*mDistributedGUIThread;



	void	ClearStateMap(void);

	int		LoadStateData(void);



public:

	CDistributedCompressor(CGUIController *gui);

	virtual ~CDistributedCompressor(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void ReturnConnection(CConnection *c,CConnection::Connection_t t);



	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;



	const COMPVARS*	GetCompvars(void);

	friend CDistributedCompressorStream;

};



/////////////////////////////////////////////////

///////////////////////////////////////////////////



class CDCompSource: public CPassiveSource

{

protected:

	virtual Connection_t _source_GetConnectionType(void) = 0;

	virtual	void		 _source_Initialize(void) = 0;

	virtual	void		 _source_Go(void) = 0;

	virtual	void		 _source_Terminate(void) = 0;

	virtual	void		 _source_Wait(void) = 0;

	virtual	void		 _source_Cleanup(void) = 0;

public:

	virtual	Connection_t	GetConnectionType(void)

		{ return _source_GetConnectionType(); }

	virtual	void	Initialize(void)

		{ _source_Initialize(); }

	virtual	void	Go(void) 

		{ _source_Go(); }

	virtual	void	Terminate(void)

		{ _source_Terminate(); }

	virtual	void	Wait(void)

		{ _source_Wait(); }

	virtual	void	Cleanup(void)

		{ _source_Cleanup(); }



};



class CDCompSink: public CActiveSink

{

protected:

	virtual Connection_t _sink_GetConnectionType(void) = 0;

	virtual	void		 _sink_Initialize(void) = 0;

	virtual	void		 _sink_Go(void) = 0;

	virtual	void		 _sink_Terminate(void) = 0;

	virtual	void		 _sink_Wait(void) = 0;

	virtual	void		 _sink_Cleanup(void) = 0;



public:

	// connection

	virtual	Connection_t	GetConnectionType(void)

				{ return _sink_GetConnectionType(); }

	virtual	void	Initialize(void)

				{ _sink_Initialize(); }

	virtual	void	Go(void) 

				{ _sink_Go(); }

	virtual	void	Terminate(void)

				{ _sink_Terminate(); }

	virtual	void	Wait(void)

				{ _sink_Wait(); }

	virtual	void	Cleanup(void)

				{ _sink_Cleanup(); }

};



/////////////////////////////

/////////////////////////////



class CDistributedCompressorStream: public CDCompSink, public CDCompSource

{

protected:

	CGUIController		*mGUIController;

	CDistributedCompressor			*mComponent;

	CCompressionCoordinator	*mCompressionCoordinator;

	CCMutex			mCompressionCoordinatorMutex;

	int				mNextFrame;

	int				mFrameSourceHasStopped;

	typedef map<int, nmr_string,less<int> > AvailableFrames_t;

	CCMutex				mAvailableFramesMutex;

	CConditionVariable	mAvailableFramesWait;

	AvailableFrames_t	mAvailableFrames;







	CDistributedCompressorStream(CDistributedCompressor *comp,CGUIController *gui);

	virtual ~CDistributedCompressorStream(void);



	virtual	Connection_t	_source_GetConnectionType(void);

	virtual	void			_source_Initialize(void);

	virtual	void			_source_Go(void);

	virtual	void			_source_Terminate(void);

	virtual	void			_source_Wait(void);

	virtual	void			_source_Cleanup(void);



	virtual	Connection_t	_sink_GetConnectionType(void);

	virtual	void			_sink_Initialize(void);

	virtual	void			_sink_Go(void);

	virtual	void			_sink_Terminate(void);

	virtual	void			_sink_Wait(void);

	virtual	void			_sink_Cleanup(void);



	// passive source

	virtual	void	SetupStreamInfo(void);

	virtual	void	SetupStreamFormat(void);

	virtual	CPassiveSourceDataReference* 

						GetData(int index,Direction_t dir);

	virtual	void		ReturnData(CPassiveSourceDataReference *r);





	// active sink

	virtual	int		ProcessData(CPassiveSource *s);



public:

	void			DataAvailable(CCompressorThread::TempFileNameList_t l,

											int start_frame);



	friend CDistributedCompressor;



	CERROR_SUBCLASS(DistributedCompressorStreamError);

};



class CDCDataReference: public CPassiveSourceDataReference

{

	nmr_string	mFilename;

	int			mFrameNumber;



public:

	CDCDataReference(int frame_number,nmr_string filename);

	int	GetFrameNumber(void);

	nmr_string GetFileName(void);



	CERROR_SUBCLASS(CDCDataReferenceError);

};



#endif

