/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CFrameManager.h"

#include "CPassiveSource.h"

#include <CLogger.h>

#include <safe_ostrstream.h>

#include <stack_aids.h>



CFrameManager::CFrameManager(CPassiveSource *source)

{

	mSource		= source;

	mFrameMax	= -1;

	mNoMoreData = 0;

#ifndef NDEBUG

	mLocked = 0;

#endif

}



CFrameManager::~CFrameManager(void)

{

	FrameStatusList_t::iterator i;



	gLogLog->SendMessage("Deleting tmp files");



	for (i = mFrameStatusList.begin(); i != mFrameStatusList.end(); ++i)

		{

		if ((*i).second.mTempFile != "")

			remove((*i).second.mTempFile);

		if ((*i).second.mDataReference)

			mSource->ReturnData((*i).second.mDataReference);

		}



	gLogLog->SendMessage("tmp files deleted");

}



void

CFrameManager::Lock(void)

{

	mLock.Lock();

	mAllocationLock.Lock();

#ifndef NDEBUG

	mLocked = 1;

	mWhoLocked = CCThread::Self();

#endif

}



void

CFrameManager::Unlock(void)

{

#ifndef NDEBUG

	mLocked = 0;

	mWhoLocked = 0;

#endif

	mAllocationLock.Unlock();

	mLock.Unlock();

}



int

CFrameManager::_AllocateNewFrame(void)

{

	assert(mLocked);

	assert(CCThread::Self() == mWhoLocked);



	if (mNoMoreData) return -1;

	

	int frame = mFrameMax + 1;

	CFrameStatus	fs;



#ifndef NDEBUG

	{

	safe_ostrstream o;

	o << "Trying to allocate frame " << frame;

	gTraceLog->SendMessage(o.str());

	}

#endif



	mAllocationLock.Unlock();

	fs.mDataReference = mSource->GetData();

	mAllocationLock.Lock();



	if (!fs.mDataReference) 

		{

		mNoMoreData = 1;

#ifndef NDEBUG

		{

		safe_ostrstream o;

		o << "FrameManager out of data " << frame;

		gTraceLog->SendMessage(o.str());

		}

#endif

		return -1;

		}

#ifndef NDEBUG

	{

	safe_ostrstream o;

	o << "FrameManager got data " << frame;

	gTraceLog->SendMessage(o.str());

	}

#endif

	mFrameStatusList[frame] = fs;

	mFrameMax++;

	return frame;

}



int

CFrameManager::GetBlockStart(void)

{

	FrameStatusList_t::iterator i;

	int	min_frame = 0x7fffffff;

	int found = 0;



	for (i = mFrameStatusList.begin(); i != mFrameStatusList.end(); ++i)

		{

		if (((*i).first < min_frame) && ((*i).second.mStatusFlag == CFrameStatus::Raw))

			{

			min_frame = (*i).first;

			found = 1;

			break;

			}

		}



	if (found)

		return min_frame;



	// not found, allocate new frame

	return _AllocateNewFrame();

}





int

CFrameManager::_ClampReusedFrames(int first_frame,int last_frame)

{

	assert(mFrameStatusList.find(first_frame) != mFrameStatusList.end());

	assert(mFrameStatusList[first_frame].mStatusFlag == CFrameStatus::Raw);

	

	int	new_last_frame = first_frame+1;

	while (1)

		{

		if (mFrameStatusList.find(new_last_frame) == mFrameStatusList.end()) break;

		if (mFrameStatusList[new_last_frame].mStatusFlag != CFrameStatus::Raw) break;

		if (new_last_frame > last_frame)  break;

		new_last_frame++;

		}

	return new_last_frame - 1;

}



int

CFrameManager::_ClampNewFrames(int first_frame,int last_frame)

{

	int new_last_frame = first_frame + 1;

	while (1)

		{

		assert(mFrameStatusList.find(new_last_frame) == mFrameStatusList.end());

		if (_AllocateNewFrame() < 0)		return new_last_frame - 1;

		if (new_last_frame == last_frame)	return last_frame;

		new_last_frame++;

		}

	return -1;

}



int

CFrameManager::ClampLastFrame(int first_frame,int last_frame)

{

	assert(mLocked);

	assert(CCThread::Self() == mWhoLocked);



	if (first_frame != mFrameMax)

		return _ClampReusedFrames(first_frame,last_frame);

	else

		return _ClampNewFrames(first_frame,last_frame);

}



void 

CFrameManager::MarkInProgress(int first_frame,int last_frame,CCompressorThread *ct,

							CServerDesignation d,long time_stamp)

{

	assert(mLocked);

	assert(CCThread::Self() == mWhoLocked);



	FrameStatusList_t::iterator i;

	int	x;

	for (x = first_frame; x <= last_frame; ++x)

		{

		assert(mFrameStatusList.find(x) != mFrameStatusList.end());

		assert(mFrameStatusList[x].mStatusFlag == CFrameStatus::Raw);

		mFrameStatusList[x].mStatusFlag			= CFrameStatus::InProgress;

		mFrameStatusList[x].mTimeStamp			= time_stamp;

		mFrameStatusList[x].mCompressorThread	= ct;

		mFrameStatusList[x].mServerDesignation	= d;

		}

}



CServerDesignation 

CFrameManager::RevertToRaw(CCompressorThread *ct)

{

	//assert(mLocked);

	//assert(CCThread::Self() == mWhoLocked);



	stack_MutexLock sml(&mAllocationLock);



	CServerDesignation d;

	int found_one = 0;



	FrameStatusList_t::iterator i;

	for (i = mFrameStatusList.begin(); i != mFrameStatusList.end(); ++i)

		{

		if ((*i).second.mCompressorThread == ct)

			{

			assert((*i).second.mStatusFlag == CFrameStatus::InProgress);

			(*i).second.mStatusFlag = CFrameStatus::Raw;

			(*i).second.mCompressorThread = NULL;

			d = (*i).second.mServerDesignation;

			found_one = 1;

			}

		}



	assert(found_one);

	return d;

}





CServerDesignation 

CFrameManager::RemoveFinishedFrames(int first_frame,int frame_count,CCompressorThread *ct)

{

	//assert(mLocked);

	//assert(CCThread::Self() == mWhoLocked);



	stack_MutexLock sml(&mAllocationLock);



	CServerDesignation d;



	int x;

	for (x = 0; x < frame_count; ++x)

		{

		assert(mFrameStatusList.find(x + first_frame) != mFrameStatusList.end());

		assert(mFrameStatusList[x+first_frame].mStatusFlag != CFrameStatus::Raw);

		assert(mFrameStatusList[x+first_frame].mCompressorThread == ct);



		mSource->ReturnData(mFrameStatusList[x+first_frame].mDataReference);

		d = mFrameStatusList[x+first_frame].mServerDesignation;

		mFrameStatusList.erase(x + first_frame);

		}

	return d;

}



CPassiveSourceDataReference* 

CFrameManager::GetDataReference(int frame)

{

	//assert(mLocked);

	//assert(CCThread::Self() == mWhoLocked);

	stack_MutexLock sml(&mAllocationLock);



	assert(mFrameStatusList.find(frame) != mFrameStatusList.end());

	assert(mFrameStatusList[frame].mStatusFlag == CFrameStatus::InProgress);

	return mFrameStatusList[frame].mDataReference;

}



int

CFrameManager::Done(void)

{

	//assert(mLocked);

	//assert(CCThread::Self() == mWhoLocked);



	stack_MutexLock sml(&mAllocationLock);



	return (mNoMoreData && (mFrameStatusList.size() == 0));

}



