/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CFRAMEMANAGER_H_

#define CFRAMEMANAGER_H_



#include "CServerDesignation.h"

#include <nmr_string.h>

#include <CCMutex.h>

#include <CCThread.h>

#include <map.h>



class CCompressorThread;

class CPassiveSourceDataReference;

class CPassiveSource;



class CFrameStatus

{

public:

	typedef enum { Raw,InProgress,Compressed} FrameStatusFlag_t;

	CServerDesignation	mServerDesignation;

	FrameStatusFlag_t	mStatusFlag;

	long				mTimeStamp;

	CCompressorThread	*mCompressorThread;

	nmr_string			mTempFile;

	CPassiveSourceDataReference	*mDataReference;



	CFrameStatus(void) 

		{ mStatusFlag = Raw; mCompressorThread = NULL; mTempFile="";}

};



class CFrameManager

{	

	typedef map<int,CFrameStatus,less<int> > FrameStatusList_t;

	FrameStatusList_t		mFrameStatusList;

	CPassiveSource			*mSource;

	int						mFrameMax;

	int						mNoMoreData;



	CCMutex					mLock;

	CCMutex					mAllocationLock;

	CCThread_ID_Type		mWhoLocked;

	int						mLocked;



	int	_ClampReusedFrames(int first_frame,int last_frame);

	int	_ClampNewFrames(int first_frame,int last_frame);

	int	_AllocateNewFrame(void);



public:

	CFrameManager(CPassiveSource *source);

	~CFrameManager(void);



	void	Lock(void);

	void	Unlock(void);



	int	GetBlockStart(void);

	int ClampLastFrame(int first_frame,int last_frame);

	void MarkInProgress(int first_frame,int last_frame,CCompressorThread *ct,

		CServerDesignation d,long time_stamp);

	CServerDesignation RevertToRaw(CCompressorThread *ct);

	CServerDesignation RemoveFinishedFrames(int first_frame,int frame_count,

											CCompressorThread *ct);

	CPassiveSourceDataReference* GetDataReference(int frame);

	int	Done(void);

};





#endif

