/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CGraphManager.h"

#include "CGUIController.h"

#include "CComponentFactory.h"

#include <stack_aids.h>

#include "danbug.h"




CGraphManager::CGraphManager(CGUIController *gui,CComponentFactory &fac)

					: mGraphRunner(this)

{

	fac.Generate(*this);



	mRunning = 0;

	mGUIController = gui;

}



CGraphManager::~CGraphManager(void)

{

	stack_MutexLock		sml(&mRunningMutex);

	while (mRunning)

		mRunningWait.Wait(mRunningMutex);



	MegaNodeList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)

		{

		delete (*i).mComponent;

		(*i).mComponent = 0;

		}

}



void

CGraphManager::AddComponentCreator(const char *name,CComponent::ComponentCreatorFunction_t ccf)

{

	mComponentCreatorMap[name] = ccf;

}



int

CGraphManager::IsRunning(void)

{

	stack_MutexLock sml(&mRunningMutex);

	return mRunning;

}



void

CGraphManager::AddComponent(const char *name)

{

	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return;



	assert(mComponentCreatorMap.find(name) != mComponentCreatorMap.end());

	MegaNode_t	mn;

	mn.mComponent = (*mComponentCreatorMap[name])(mGUIController);

	mComponentList.push_back(mn);

	mGUIController->AddComponent(mComponentList.size()-1);

}



int

CGraphManager::AddConnection(int c1,int c2,

	CConnection::Connection_t t1,CConnection::Connection_t t2)

{

	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return 0;



	if (c1 == c2) return 0;

	assert(c1 < mComponentList.size());

	assert(c2 < mComponentList.size());

	assert(ComponentCanProduceThisType(c1,t1));

	assert(ComponentCanProduceThisType(c2,t2));

	if (!ValidConnection(t1,t2)) return 0;

	assert(VerifyNoConnectionOfThisType(c1,t1));

	assert(VerifyNoConnectionOfThisType(c2,t2));



	CConnection *cc1 = mComponentList[c1].mComponent->CreateConnection(t1);

	CConnection *cc2 = mComponentList[c2].mComponent->CreateConnection(t2);

	int new_connection_index = mConnectionList.size();

	cc1->Connect(cc2);

	cc2->Connect(cc1);

	Node_t	node1,node2;

	node1.mConnection = cc1;

	node1.mMegaNodeIndex = c1;

	node1.mMySink = -1;

	node1.mMySource = -1;

	if (t1 & CONN_SOURCE)

		node1.mMySink = new_connection_index + 1;

	else

		node1.mMySource = new_connection_index + 1;



	node2.mConnection = cc2;

	node2.mMegaNodeIndex = c2;

	node2.mMySink = -1;

	node2.mMySource = -1;

	if (t2 & CONN_SOURCE)

		node2.mMySink = new_connection_index;

	else

		node2.mMySource = new_connection_index;



	mConnectionList.push_back(node1);

	mConnectionList.push_back(node2);



	mComponentList[c1].mNodeIndexList.push_back(new_connection_index);

	mComponentList[c2].mNodeIndexList.push_back(new_connection_index+1);



	return 1;

}





void

CGraphManager::RemoveConnection(int c1,int c2,

	CConnection::Connection_t t1,CConnection::Connection_t t2)

{

	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return;



	int cc1_index = -1;

	int cc2_index = -1;

	NodeList_t::iterator cc1_i;

	NodeList_t::iterator cc2_i;



	// find connection indicies

	int x;

	NodeList_t::iterator i;

	for (x = 0,i = mConnectionList.begin(); i != mConnectionList.end(); ++i,++x)

		{

		if ((*i).mMegaNodeIndex == c1 && (*i).mConnection->GetConnectionType() == t1)

			{

			assert(cc1_index == -1);

			cc1_index = x;

			cc1_i = i;

			}

		if ((*i).mMegaNodeIndex == c2 && (*i).mConnection->GetConnectionType() == t2)

			{

			assert(cc2_index == -1);

			cc2_index = x;

			cc2_i = i;

			}

		}	

	

	assert(cc1_index != -1);

	assert(cc2_index != -1);



	int cc1_meta_index = -1;

	int cc2_meta_index = -1;

	vector<int>::iterator cc1meta_i;

	vector<int>::iterator cc2meta_i;



	// now make sure the connection index exists inside the component

	for (x = 0,cc1meta_i = mComponentList[c1].mNodeIndexList.begin(); 

				x < mComponentList[c1].mNodeIndexList.size(); ++x,++cc1meta_i)

		{

		if (cc1_index == mComponentList[c1].mNodeIndexList[x])

			{

			cc1_meta_index = x;

			break;

			}

		}

	assert(cc1_meta_index != -1);



	for (x = 0,cc2meta_i = mComponentList[c2].mNodeIndexList.begin(); 

			x < mComponentList[c2].mNodeIndexList.size(); ++x,++cc2meta_i)

		{

		if (cc2_index == mComponentList[c2].mNodeIndexList[x])

			{

			cc2_meta_index = x;

			break;

			}

		}

	assert(cc2_meta_index != -1);



	// return the connection

	mComponentList[c1].mComponent->ReturnConnection(mConnectionList[cc1_index].mConnection,t1);

	mComponentList[c2].mComponent->ReturnConnection(mConnectionList[cc2_index].mConnection,t2);



	// delete from connection list

	if (cc1_index > cc2_index)

		{

		mConnectionList.erase(cc1_i);

		mConnectionList.erase(cc2_i);

		}

	else

		{

		mConnectionList.erase(cc2_i);

		mConnectionList.erase(cc1_i);

		}



	// delete from component connection list

	mComponentList[c1].mNodeIndexList.erase(cc1meta_i);

	mComponentList[c2].mNodeIndexList.erase(cc2meta_i);



	// update all node indexes

	MegaNodeList_t::iterator mni;

	for (mni = mComponentList.begin(); mni != mComponentList.end(); ++mni)

		{

		vector<int>::iterator vii;

		for (vii = (*mni).mNodeIndexList.begin(); vii != (*mni).mNodeIndexList.end(); ++vii)

			{

			int tmp = (*vii);



			if (tmp > cc1_index)

				(*vii)--;

			if (tmp > cc2_index)

				(*vii)--;

			}

		}



	NodeList_t::iterator ni;

	for (ni = mConnectionList.begin(); ni != mConnectionList.end(); ++ni)

		{

		int tmp = (*ni).mMySink;



		if (tmp > cc1_index)

			(*ni).mMySink--;

		if (tmp > cc2_index)

			(*ni).mMySink--;



		tmp = (*ni).mMySource;



		if (tmp > cc1_index)

			(*ni).mMySource--;

		if (tmp > cc2_index)

			(*ni).mMySource--;

		}

}





int

CGraphManager::VerifyNoConnectionOfThisType(int c,CConnection::Connection_t t)

{

	NodeList_t::const_iterator i;

	for (i = mConnectionList.begin(); i != mConnectionList.end(); ++i)

		{

		if ((*i).mMegaNodeIndex == c && (*i).mConnection->GetConnectionType() == t)

			return 0;

		}

	return 1;

}



const CComponent*

CGraphManager::GetComponent(int index)

{

	stack_MutexLock sml(&mRunningMutex);



	return mComponentList[index].mComponent;

}



void

CGraphManager::RemoveComponent(int index)

{

	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return;



	mRunningMutex.Unlock();

	RemoveComponentConnections(index);

	mRunningMutex.Lock();



	// delete the component

	MegaNodeList_t::iterator i;

	int x = 0;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i,++x)

		{

		if (x == index)

			{

			delete (*i).mComponent;

			mComponentList.erase(i);

			break;

			}

		}



	// update all meganode references

	NodeList_t::iterator ii;

	for (ii = mConnectionList.begin(); ii != mConnectionList.end(); ++ii)

		{

		assert((*ii).mMegaNodeIndex != index);

		if ((*ii).mMegaNodeIndex > index)

			(*ii).mMegaNodeIndex--;

		}

}



void

CGraphManager::RemoveComponentConnections(int index)

{

	while (mComponentList[index].mNodeIndexList.size())

		{

		int c1 = -1;

		int c2 = -1;

		int cc1 = -1;

		int cc2 = -1;

		CConnection::Connection_t	t1 = 0;

		CConnection::Connection_t	t2 = 0;



		c1 = index;



		cc1 = mComponentList[index].mNodeIndexList[0];

		assert(cc1 != -1);



		cc2 = mConnectionList[cc1].mMySink;

		if (cc2 == -1)

			cc2 = mConnectionList[cc1].mMySource;

		assert(cc2 != -1);



		t1 = mConnectionList[cc1].mConnection->GetConnectionType();

		t2 = mConnectionList[cc2].mConnection->GetConnectionType();

		c2 = mConnectionList[cc2].mMegaNodeIndex;

		

		assert(c1 != -1);

		assert(c2 != -1);

		assert(cc1 != -1);

		assert(cc2 != -1);

		assert(t1);

		assert(t2);

		RemoveConnection(c1,c2,t1,t2);

		}

}





int

CGraphManager::ComponentCanProduceThisType(int c,CConnection::Connection_t t)

{

	CComponent::ConnectionTypeList_t l = 

		mComponentList[c].mComponent->GetSupportedTypes();

	CComponent::ConnectionTypeList_t::iterator i;

	for (i = l.begin(); i != l.end(); ++i)

		if ((*i) == t) return 1;

	return 0;

}



int

CGraphManager::ValidConnection(CConnection::Connection_t t1,

							   CConnection::Connection_t t2)

{

	CConnection::Connection_t temp1,temp2;



	// check source-sink

	temp1 = t1;

	temp2 = t2;

	temp1 &= (CONN_SOURCE | CONN_SINK);

	temp2 &= (CONN_SOURCE | CONN_SINK);

	if ((temp1 ^ temp2) != (CONN_SOURCE | CONN_SINK)) return 0;



	// check active-passive

	temp1 = t1;

	temp2 = t2;

	temp1 &= (CONN_ACTIVE | CONN_PASSIVE);

	temp2 &= (CONN_ACTIVE | CONN_PASSIVE);

	if ((temp1 ^ temp2) != (CONN_ACTIVE | CONN_PASSIVE)) return 0;



	// video/audio

	temp1 = t1;

	temp2 = t2;

	if ((temp1 & CONN_VIDEO) && !(temp2 & CONN_VIDEO)) return 0;

	if ((temp1 & CONN_AUDIO) && !(temp2 & CONN_AUDIO)) return 0;



	if ((temp1 & CONN_NEEDSSEEKABLE) && !(temp2 & CONN_SEEKABLE)) return 0;

	if ((temp2 & CONN_NEEDSSEEKABLE) && !(temp1 & CONN_SEEKABLE)) return 0;



	return 1;

}



CGraphManager::ComponentNameList_t

CGraphManager::GetComponentNameList(void)

{

	ComponentNameList_t	l;

	ComponentCreatorMap_t::iterator i;

	for (i = mComponentCreatorMap.begin(); i != mComponentCreatorMap.end(); ++i)

		l.push_back((*i).first);

	

	return l;

}





/////////////////////////////////////

/////////////////////////////////////

int

CGraphManager::_ConfigureComponents(CWnd *wnd)

{

	MegaNodeList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)

		if (!(*i).mComponent->ConfigureIfNeeded(wnd))

			return 0;

	return 1;

}



int

CGraphManager::ConfigureComponents(void)

{

	CWnd *wnd = mGUIController->GetWindow();

	MegaNodeList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)

		(*i).mComponent->Configure(wnd);

	return 1;

}



void

CGraphManager::InitializeComponents(void)

{

	MegaNodeList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)
		(*i).mComponent->Initialize();
}



void

CGraphManager::CleanupComponents(void)

{

	MegaNodeList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)

		(*i).mComponent->Cleanup();

}



void

CGraphManager::Run(void)

{

 dprintf("CGraphManager::Run");
	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return;



	mGUIController->ClearMessages();



	// check for graph connectivity

	UnmarkMegaNodeList(mComponentList);

	UnmarkNodeList(mConnectionList);

	TraverseAndMarkGraph(mComponentList,mConnectionList);

	if (!AllNodesVisited(mConnectionList) || !AllMegaNodesVisited(mComponentList))

			throw CGraphManagerError("Graph is not fully connected",0);



	if (!_ConfigureComponents(mGUIController->GetWindow())) return;

	

	try 

		{
 dprintf("about to call InitializeComponents");
		InitializeComponents();
 dprintf("done called InitializeComponents");



		InitializeAction ia;
 dprintf("CGraphManager::Run A");

		Traverse(mComponentList,mConnectionList,ia);
 dprintf("CGraphManager::Run B");

		mNeedsCleanup = 1;

		mGraphRunner.Start();
 dprintf("CGraphManager::Run C");

		mRunning = 1;

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());

		mGUIController->ErrorDialog(buff);

		CleanupAction ca;

		Traverse(mComponentList,mConnectionList,ca);

		CleanupComponents();

		}

	catch(...)

		{

		mGUIController->ErrorDialog("Unknown Error");

		CleanupAction ca;

		Traverse(mComponentList,mConnectionList,ca);

		CleanupComponents();

		}

	

}



void

CGraphManager::Halt(void)

{

	stack_MutexLock sml(&mRunningMutex);

	if (!mRunning) return;



	try {

		stack_MutexLock sml2(&mCleanupMutex);

		if (mNeedsCleanup)

			{

			TerminateAction ta;

			Traverse(mComponentList,mConnectionList,ta);

			CleanupAction ca;

			Traverse(mComponentList,mConnectionList,ca);

			CleanupComponents();

			}		

		mNeedsCleanup = 0;

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());

		mGUIController->ErrorDialog(buff);

		}

	catch(...)

		{

		mGUIController->ErrorDialog("Unknown Error");

		}



	mGraphRunner.Join();

}



void

CGraphManager::CGraphRunnerThread::Run(void)

{

	try 

		{

		GoAction ga;

		Traverse(mGraphManager->mComponentList,mGraphManager->mConnectionList,ga);



		NodeList_t::iterator i;

		for (i = mGraphManager->mConnectionList.begin(); 

				i != mGraphManager->mConnectionList.end(); ++i)

			(*i).mConnection->Wait();



		stack_MutexLock sml2(&(mGraphManager->mCleanupMutex));

		if (mGraphManager->mNeedsCleanup)

			{

			CleanupAction ca;

			Traverse(mGraphManager->mComponentList,mGraphManager->mConnectionList,ca);

			mGraphManager->CleanupComponents();

			}	

		mGraphManager->mNeedsCleanup = 0;

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());

		mGraphManager->mGUIController->ErrorDialog(buff);

		}

	catch(...)

		{

		mGraphManager->mGUIController->ErrorDialog("Unknown Error");

		}



	try 

		{

		stack_MutexLock sml2(&(mGraphManager->mCleanupMutex));

		if (mGraphManager->mNeedsCleanup)

			{

			CleanupAction ca;

			Traverse(mGraphManager->mComponentList,mGraphManager->mConnectionList,ca);

			mGraphManager->CleanupComponents();

			}	

		mGraphManager->mNeedsCleanup = 0;

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());

		mGraphManager->mGUIController->ErrorDialog(buff);

		}

	catch(...)

		{

		mGraphManager->mGUIController->ErrorDialog("Unknown Error");

		}

	mGraphManager->mRunning = 0;

}



void	

CGraphManager::ConfigureComponent(int index,CWnd *wnd)

{

	stack_MutexLock sml(&mRunningMutex);

	if (mRunning) return;



	mComponentList[index].mComponent->Configure(wnd);

}



