/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CGRAPHMANAGER_H_

#define CGRAPHMANAGER_H_



#include "CComponent.h"

#include "NodeStuff.h"

#include <nmr_string.h>

#include <CCThread.h>

#include <CCMutex.h>

#include <CConditionVariable.h>

#include <CError.h>

#include <map>

#include <list>

#include <vector>
using namespace std;


class CGUIController;

class CComponentFactory;



class CGraphManager

{

protected:

	class CGraphRunnerThread: public CCThread

	{

		CGraphManager *mGraphManager;

	public:

		CGraphRunnerThread(CGraphManager *gm) 

		{ mGraphManager = gm;}

		virtual void Run(void);



		friend CGraphManager;

	};

	

	

	typedef map<nmr_string,CComponent::ComponentCreatorFunction_t, less<nmr_string> > ComponentCreatorMap_t;



	ComponentCreatorMap_t	mComponentCreatorMap;

	MegaNodeList_t			mComponentList;

	NodeList_t				mConnectionList;





	CCMutex				mRunningMutex;

	CConditionVariable	mRunningWait;

	int					mRunning;



	CGUIController		*mGUIController;



	CGraphRunnerThread	mGraphRunner;

	CCMutex				mCleanupMutex;

	int					mNeedsCleanup;



	int	ComponentCanProduceThisType(int c,CConnection::Connection_t t);

	int	ValidConnection(CConnection::Connection_t t1,

						CConnection::Connection_t t2);

	int	VerifyNoConnectionOfThisType(int c,CConnection::Connection_t t);

	void	InitializeComponents(void);

	void	CleanupComponents(void);

	int		_ConfigureComponents(CWnd *wnd);



public:

	typedef vector<nmr_string> ComponentNameList_t;



	CGraphManager(CGUIController *gui,CComponentFactory &f);

	~CGraphManager(void);



	void	AddComponentCreator(const char *name,CComponent::ComponentCreatorFunction_t ccf);

	void	AddComponent(const char *name);

	const CComponent*	GetComponent(int index);

	void	RemoveComponent(int index);

	void	RemoveComponentConnections(int index);

	void	RemoveConnection(int c1,int c2,

			CConnection::Connection_t t1,CConnection::Connection_t t2);

	int		AddConnection(int c1,int c2,

			CConnection::Connection_t t1,CConnection::Connection_t t2);

	ComponentNameList_t	GetComponentNameList(void);

	int		IsRunning(void);

	void	Run(void);

	void	Halt(void);



	void	ConfigureComponent(int index,CWnd *wnd);

	int		ConfigureComponents(void);



	CERROR_SUBCLASS(CGraphManagerError);



	friend CGraphRunnerThread;

};





#endif



