/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CIvaPWriter.h"
#include "CGUIController.h"

#include "ivaWriterFast.h"

#include <assert.h>

#include "danbug.h"
#include "..\dansoft\acmenc.h"

CIvaPWriter::CIvaPWriter(int interleave,

						 CGUIController *gui)

						 : CComponent(gui)

{

	mFileOpen = 0;

	mFile = 0;

	mInterleave = interleave;

	mFileName = AfxGetApp()->GetProfileString("CIvaPWriter","FileName","");

	mRecalculateFramerate = AfxGetApp()->GetProfileInt("CIvaPWriter","RecalculateFramerate",0);

}



void

CIvaPWriter::Initialize(void)

{

	assert(!mFile);

	remove(mFileName);



	ivaHeader out_header;

	memset(&out_header,0,sizeof(out_header));

	out_header.dwFlags = AVIFILEINFO_HASINDEX;



	// create the writer...

	mFile = new ivaWriterFast(out_header);

	if (!mFile->open(mFileName)) 

		throw FileError("Error creating output file",0);

	mFileOpen = 1;

}



void

CIvaPWriter::Cleanup(void)

{

	AfxGetApp()->WriteProfileString("CIvaPWriter","FileName",mFileName);

	AfxGetApp()->WriteProfileInt("CIvaPWriter","RecalculateFramerate",mRecalculateFramerate);



	if (mFileOpen)

		{

		mFile->close(mRecalculateFramerate);

		delete mFile;

		}

	mFileOpen = 0;

	mFile = 0;

}



CIvaPWriter::~CIvaPWriter(void)

{

	ConnectionList_t::iterator i;

	for (i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		delete (*i);

	mProducedConnections.erase(mProducedConnections.begin(),mProducedConnections.end());

}



void	

CIvaPWriter::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	ConnectionList_t::iterator i;

	for(i = mProducedConnections.begin(); i != mProducedConnections.end(); ++i)

		{

		if ((*i) == c)

			{

			delete (*i);

			mProducedConnections.erase(i);

			break;

			}

		}

}



CComponent::ConnectionTypeList_t

CIvaPWriter::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;



	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_AUDIO);



	return l;

}



nmr_string

CIvaPWriter::GetName(void) const

{

	return "Iva Writer";

}



CConnection *

CIvaPWriter::CreateConnection(CConnection::Connection_t t)

{

	CConnection::Connection_t	minimum_requirements = (CONN_SINK | CONN_PASSIVE);



	if ((t & minimum_requirements) != minimum_requirements)

		return 0;



	CConnection *c = 0;



	if (t & CONN_AUDIO)

	{ c =  new CIvaPWriterAudioStream(this,mGUIController);
 dprintf("connecting a CIvaPWriterAudioStream");
	}

	if (t & CONN_VIDEO)

		c =  new CIvaPWriterVideoStream(this,mGUIController);

	

	mProducedConnections.push_back(c);



	return c;

}





ivaWriter*

CIvaPWriter::GetIvaFile(void)

{

	return mFile;

}



/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////



CIvaPWriterBaseStream::CIvaPWriterBaseStream(CIvaPWriter *comp,

											 CGUIController *gui)

{

	mGUIController = gui;

	mComponent = comp;

	mIvaFile = 0;

	mStreamOpen = 0;

	mOutputFrameCounter = 0;

}



CIvaPWriterBaseStream::~CIvaPWriterBaseStream(void)

{

	_CloseStream();

}



void

CIvaPWriterBaseStream::Initialize(void)

{

	CPassiveSink::Initialize();



	mTerminate = 0;

	mOutputFrameCounter = 0;

	assert(!mStreamOpen);

	mIvaFile = mComponent->GetIvaFile();

	assert(mIvaFile);
 dprintf("about to call _OpenStream");
	_OpenStream();
 dprintf(" called _OpenStream");

}



void

CIvaPWriterBaseStream::Cleanup(void)

{

	_CloseStream();

}



void

CIvaPWriterBaseStream::Terminate(void)

{

	mTerminate = 1;

}



void

CIvaPWriterBaseStream::_OpenStream(void)

{
 dprintf("CIvaPWriterBaseStream::_OpenStream");

	AVISTREAMINFO i = mStreamInfoObject.GetStreamInfo();

	const char *f	= mStreamInfoObject.GetStreamFormat();

	size_t s		= mStreamInfoObject.GetStreamFormatSize();



	ivaStreamHeader	ii;
	ivaVideoHeader	vf;
	ivaAudioHeader	af;

	memset(&ii,0,sizeof(ii));

	ii.fccType			= i.fccType;
	ii.fccHandler		= i.fccHandler;
	ii.dwFlags			= i.dwFlags;
	ii.wPriority		= i.wPriority;
	ii.wLanguage		= i.wLanguage;
	ii.dwInitialFrames	= i.dwInitialFrames;
	ii.dwScale			= i.dwScale;
	ii.dwRate			= i.dwRate;
	ii.dwStart			= i.dwStart;
	ii.dwLength			= i.dwLength;
	ii.dwSuggestedBufferSize = i.dwSuggestedBufferSize;
	ii.dwQuality		= i.dwQuality;
	ii.dwSampleSize		= i.dwSampleSize;
	ii.rcFrame.left		= i.rcFrame.left;
	ii.rcFrame.right	= i.rcFrame.right;
	ii.rcFrame.top		= i.rcFrame.top;
	ii.rcFrame.bottom	= i.rcFrame.bottom;

 dprintf("::_OpenStream fccType = %x, sampleSize=%d, bufsize=%d, rate=%d, scale=%d",
				i.fccType,i.dwSampleSize,i.dwSuggestedBufferSize,i.dwRate,i.dwScale);
	if (mConnectionType & CONN_VIDEO)

		{

		memset(&vf,0,sizeof(vf));

		BITMAPINFO *bii = (BITMAPINFO *)f;



		vf.bmiHeader.biSize				= bii->bmiHeader.biSize;

		vf.bmiHeader.biWidth			= bii->bmiHeader.biWidth;

		vf.bmiHeader.biHeight			= bii->bmiHeader.biHeight;

		vf.bmiHeader.biPlanes			= bii->bmiHeader.biPlanes;

		vf.bmiHeader.biBitCount			= bii->bmiHeader.biBitCount;

		vf.bmiHeader.biCompression		= bii->bmiHeader.biCompression;

		vf.bmiHeader.biSizeImage		= bii->bmiHeader.biSizeImage;

		vf.bmiHeader.biXPelsPerMeter	= bii->bmiHeader.biXPelsPerMeter;

		vf.bmiHeader.biYPelsPerMeter	= bii->bmiHeader.biYPelsPerMeter;

		vf.bmiHeader.biClrUsed			= bii->bmiHeader.biClrUsed;

		vf.bmiHeader.biClrImportant		= bii->bmiHeader.biClrImportant;

		delete [] vf.bmiColors;

		vf.bmiColors = 0;

		if (bii->bmiHeader.biClrUsed)

			{

			vf.bmiColors = new pmsio_RGBQUAD[bii->bmiHeader.biClrUsed];

			for (int x = 0; x < bii->bmiHeader.biClrUsed; ++x)

				{

				vf.bmiColors[x].rgbBlue = bii->bmiColors[x].rgbBlue;

				vf.bmiColors[x].rgbGreen = bii->bmiColors[x].rgbGreen;

				vf.bmiColors[x].rgbRed = bii->bmiColors[x].rgbRed;

				vf.bmiColors[x].rgbReserved = bii->bmiColors[x].rgbReserved;

				}

			}

		mIvaStream = mIvaFile->create_stream(&ii,&vf);

		}

	else

		{
// ii.dwRate			= 11025;
		ii.dwRate			= 8000;											/// or else Vorb freaks in MP
 ii.dwSuggestedBufferSize = 8000;
 dprintf("Writing AUDIO header here... changed rate\n");
		memset(&af,0,sizeof(af));

///		af.wFormatTag		= ((WAVEFORMATEX *)f)->wFormatTag;
		af.wFormatTag		= ACMVORBIS;							///	kluge audio header munge
		af.nChannels		= ((WAVEFORMATEX *)f)->nChannels;
		af.nSamplesPerSec	= ((WAVEFORMATEX *)f)->nSamplesPerSec;
		af.nAvgBytesPerSec	= 8000;									///	another one
//		af.nAvgBytesPerSec	= 11025;								///	forced 2-bit CBR
		af.nBlockAlign		= 1;									///	and again!
		af.wBitsPerSample	= ((WAVEFORMATEX *)f)->wBitsPerSample;
		af.cbSize			= 8;									///	jeez...
///		af.mExtraData[0]	= 4;									///	dbm -- 2bit CBR means this = 4 (4 samp/byte)
/*		af.mExtraData[1]	= 0;
		af.mExtraData[2]	= 0;
		af.mExtraData[3]	= 0;
*/

 dprintf("            format: %x",af.wFormatTag);
 dprintf("         nChannels: %d",af.nChannels);
 dprintf("      Samp per sec: %d",af.nSamplesPerSec);
 dprintf("      bits/samp   : %d",af.wBitsPerSample);
 dprintf("        align     : %d",af.nBlockAlign);
 dprintf("           cbsize : %d",af.cbSize);

		delete [] af.mExtraData;

		af.mExtraData = 0;

		if (af.cbSize && af.wFormatTag != 1)

			{

			af.mExtraData = new char[af.cbSize];

			memcpy(af.mExtraData,((char *)f) + sizeof(WAVEFORMATEX),af.cbSize);
			af.mExtraData[0] = 4;									///	dbm -- set to 4 for CBR: 2 bits/samp, 4 samp/byte, get it?
			af.mExtraData[1] = 0;
			af.mExtraData[2] = 0;
			af.mExtraData[3] = 0;

 dprintf("        extraData : %d",af.mExtraData[0]);

			}



		mIvaStream = mIvaFile->create_stream(&ii,&af);

		}



	mStreamOpen = 1;

}



void

CIvaPWriterBaseStream::_CloseStream(void)

{

	//if (mStreamOpen)

	//	IvaStreamRelease(mIvaStream);

	mStreamOpen = 0;

}



CPassiveSinkDataReference *

CIvaPWriterBaseStream::GetDataReference(size_t s)

{

	if (mTerminate) return 0;

	ivaFrame *f = mIvaStream->write_getframe(s);

	mDataReference.SetFrame(f);

	mDataReference.Set((char *)f->buffer(),s,0,0);

	return &mDataReference;

}



void	

CIvaPWriterBaseStream::DataComplete(void)

{

	_CloseStream();

	mTerminate = 1;

}





/////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////



void

CIvaPWriterBaseStream::_ProcessData(const CPassiveSinkDataReference *r)

{

 dprintf("  CIvaPWriterBaseStream::_ProcessData");
	// write the data

	if (!mStreamOpen) return;

	assert(r);

	CPassiveSinkDataReference *rrr = const_cast<CPassiveSinkDataReference*>(r);

	assert(rrr);

	const CIvaPassiveSinkDataReference *rr = dynamic_cast<const CIvaPassiveSinkDataReference*>(rrr);

	assert(rr);



	ivaFrame *f = rr->GetFrame();

	assert(f);

	f->length() = rr->GetSize();

	if (!mIvaStream->write_returnframe(f,rr->GetFlags() ? AVIIF_KEYFRAME : 0,

		mConnectionType & CONN_VIDEO ? ffirTCC::dc() : ffirTCC::wb()))

		throw StreamError("Could not write to video stream",0);



#ifndef NDEBUG

	(mGUIController->Trace(this)) << "Writing video frame " 

		<< mOutputFrameCounter << CMessageStream::show;

#endif

	mOutputFrameCounter++;

}



void	

CIvaPWriterBaseStream::ReturnDataReference(CPassiveSinkDataReference *r)

{

	_ProcessData(r);

}



int

CIvaPWriterBaseStream::ConsumeDataNow(const CPassiveSinkDataReference *r)

{

 dprintf("  CIvaPWriterBaseStream::ConsumeDataNow");
 mConnectionType & CONN_VIDEO ? dprintf("    type=VIDEO") : dprintf("    type=AUDIO");
	ivaTrivialFrame tf((unsigned char *)r->GetWritePtr(),r->GetSize());

	tf.length() = r->GetSize();

	return mIvaStream->write(tf,r->GetFlags(),

		mConnectionType & CONN_VIDEO ? ffirTCC::dc() : ffirTCC::wb());

}





///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////









