/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CIVAPWRITER_H_
#define CIVAPWRITER_H_

#include "CComponent.h"

#include "CPassiveSink.h"

#include	<vector>
using namespace std;

#include	<CError.h>



class ivaWriter;

class ivaWriteStream;

class ivaFrame;





class CIvaPWriter: public CComponent

{

protected:

	ivaWriter		*mFile;

	nmr_string		mFileName;

	int				mFileOpen;

	int				mInterleave;

	typedef vector<CConnection *> ConnectionList_t;

	ConnectionList_t mProducedConnections;



	int			mRecalculateFramerate;



public:

	CIvaPWriter(int interleave,CGUIController *gui);

	virtual ~CIvaPWriter(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;

	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

			ivaWriter*	GetIvaFile(void);





	CERROR_SUBCLASS(FileError);

};



///////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////



class CIvaPassiveSinkDataReference: public CPassiveSinkDataReference

{

	ivaFrame	*mFrame;

public:



	CIvaPassiveSinkDataReference(void) {mFrame = 0;}

	ivaFrame*	GetFrame(void) const { return mFrame; }

	void		SetFrame(ivaFrame *f) { mFrame = f; }

};





class CIvaPWriterBaseStream: public CPassiveSink

{

protected:

	ivaWriteStream	*mIvaStream;

	ivaWriter		*mIvaFile;

	int			mStreamOpen;

	int			mOutputFrameCounter;

	CIvaPWriter	*mComponent;

	CGUIController	*mGUIController;

	int			mTerminate;

	CIvaPassiveSinkDataReference mDataReference;

	//char		*mDataReferenceData;

	//int			mDataReferenceSize;



	CIvaPWriterBaseStream(CIvaPWriter *comp,CGUIController *gui);

			void	_OpenStream(void);

			void	_CloseStream(void);



			void	_ProcessData(const CPassiveSinkDataReference *r);

public:

	virtual ~CIvaPWriterBaseStream(void);



	virtual	void	Initialize(void);

	virtual	void	Cleanup(void);

	virtual	void	Terminate(void);



	// get a place to put data

	virtual	CPassiveSinkDataReference *GetDataReference(size_t);

	virtual	void	DataComplete(void);

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);



	CERROR_SUBCLASS(StreamError);

};







///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////



class CIvaPWriterVideoStream: public CIvaPWriterBaseStream

{

protected:

	CIvaPWriterVideoStream(CIvaPWriter *comp,CGUIController *gui) 

		: CIvaPWriterBaseStream(comp,gui) 

	{ mConnectionType = (CONN_SINK | CONN_PASSIVE | CONN_VIDEO);}



public:



	friend CIvaPWriter;

};



class CIvaPWriterAudioStream: public CIvaPWriterBaseStream

{

protected:

	CIvaPWriterAudioStream(CIvaPWriter *comp,CGUIController *gui) 

		: CIvaPWriterBaseStream(comp,gui) 

	{ mConnectionType = (CONN_SINK | CONN_PASSIVE | CONN_AUDIO);}



public:

	friend CIvaPWriter;

};



////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////






#endif
