/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CMonitorThread.h"

#include "CAviReader.h"

#include "CAviWriter.h"

#include "CAviAReader.h"

#include "CAviPWriter.h"

#include "CVidcapSource.h"

#include "CCompressor.h"

#include "CCircularBufferComponent.h"

#include <stack_aids.h>



CMonitorThread::CMonitorThread(CWnd *main_window)

: mGUIController(main_window)

{

	mWindow = main_window;



	mAutoLinker = 0;

	mNeedsCleanup = 1;

	mCanCleanup = 0;

}



CMonitorThread::~CMonitorThread(void)

{

	Cleanup();

}



void

CMonitorThread::Cleanup(void)

{

	// terminate passives first

	stack_MutexLock sml(&mCleanupMutex);



	if (!mNeedsCleanup) return;



	if (mAutoLinker)

		mAutoLinker->Terminate();



	stack_MutexLock sml2(&mCanCleanupMutex);

	while (!mCanCleanup) mCanCleanupWait.Wait(mCanCleanupMutex);



	if (mAutoLinker)

		mAutoLinker->Cleanup();



	delete mAutoLinker;

	mAutoLinker = 0;



	CAutoLinker::ComponentList_t::iterator i;

	for (i = mComponentList.begin(); i != mComponentList.end(); ++i)

		delete (*i);



	mComponentList.erase(mComponentList.begin(),mComponentList.end());

	mNeedsCleanup = 0;

}





int

CMonitorThread::Prepare(void)

{

	assert(!mAutoLinker);

	int result = 0;

	try 

		{

		mAutoLinker = new CAutoLinker;



		//mComponentList.push_back(new CAviReader(&mGUIController));

		//mComponentList.push_back(new CAviWriter(0,&mGUIController));



		//mComponentList.push_back(new CAviAReader(&mGUIController));

		//mComponentList.push_back(new CAviPWriter(0,&mGUIController));



		//mComponentList.push_back(new CAviAReader(&mGUIController));

		//mComponentList.push_back(new CCircularBufferComponent(&mGUIController));

		//mComponentList.push_back(new CAviWriter(0,&mGUIController));



		//mComponentList.push_back(new CVidcapSource(&mGUIController));

		//mComponentList.push_back(new CCircularBufferComponent(&mGUIController));

		//mComponentList.push_back(new CAviWriter(0,&mGUIController));



		//mComponentList.push_back(new CAviAReader(&mGUIController));

		//mComponentList.push_back(new CCompressor(&mGUIController));

		//mComponentList.push_back(new CAviPWriter(0,&mGUIController));



		mComponentList.push_back(new CVidcapSource(&mGUIController));

		mComponentList.push_back(new CCompressor(&mGUIController));

		mComponentList.push_back(new CCircularBufferComponent(&mGUIController));

		mComponentList.push_back(new CAviWriter(0,&mGUIController));



		mAutoLinker->LinkComponents(mComponentList);



		result = mAutoLinker->Configure(mWindow);

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());



		::MessageBox(mWindow->m_hWnd,buff,"Error",MB_OK | MB_APPLMODAL);

		}

	catch(...)

		{

		::MessageBox(mWindow->m_hWnd,"Unknown Error","Error",MB_OK | MB_APPLMODAL);

		}

	if (!result) Cleanup();

	return result;

}



void

CMonitorThread::Run(void)

{

	try 

		{

		mAutoLinker->Initiate();

		mAutoLinker->Wait();

		stack_MutexLock sml(&mCanCleanupMutex);

		mCanCleanup = 1;

		mCanCleanupWait.Signal();

		::Beep(0,0);

		//::MessageBox(mWindow,"Done","Done",MB_OK | MB_APPLMODAL);

		}

	catch(CError &err)

		{

		char buff[1024];

		sprintf(buff,"%s %d",err.data(),err.code());



		::MessageBox(mWindow->m_hWnd,buff,"Error",MB_OK | MB_APPLMODAL);

		}

	catch(...)

		{

		::MessageBox(mWindow->m_hWnd,"Unknown Error","Error",MB_OK | MB_APPLMODAL);

		}

	Cleanup();

}



void

CMonitorThread::Terminate(void)

{

	Cleanup();

}

