/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CNeilCapGUIController.h"

#include "CGraphManager.h"

#include <stack_aids.h>



#define LINEHEIGHT	12

#define LINEWIDTH	300

#define NUM_LINES	3

#define DIAGNOSTIC_BOX_HEIGHT	(LINEHEIGHT * NUM_LINES)

#define DIAGNOSTIC_BOX_WIDTH	(LINEWIDTH)



#define GRAPH_BOTTOM	250

#define DIAGNOSTIC_TOP	253



#define VIDEO_HEIGHT	240

#define VIDEO_WIDTH		320

#define VIDEO_BOTTOM	(GRAPH_BOTTOM + VIDEO_HEIGHT)

#define VIDEO_RIGHT		(VIDEO_WIDTH + 2)

#define VIDEO_LEFT		0

#define VIDEO_TOP		253



#define DIAGNOSTIC_LEFT	(VIDEO_RIGHT + 3)



CNeilCapGUIController::CNeilCapGUIController(CWnd *window)

{

	mWnd = window;

	mFont.CreateFont(-12,0,0,0,FW_NORMAL,FALSE,FALSE,0,

		ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,

		DEFAULT_PITCH | FF_DONTCARE, "MS Sans Serif");

	mSmallFont.CreateFont(-10,0,0,0,FW_NORMAL,FALSE,FALSE,0,

		ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,

		DEFAULT_PITCH | FF_DONTCARE, "Times");



	mDarkBrush.CreateSolidBrush(::GetSysColor(COLOR_3DDKSHADOW));

	mMediumBrush.CreateSolidBrush(::GetSysColor(COLOR_3DSHADOW));

	mNormalBrush.CreateSolidBrush(::GetSysColor(COLOR_3DFACE));

	mLightBrush.CreateSolidBrush(::GetSysColor(COLOR_3DLIGHT));



	mDarkPen.CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DDKSHADOW));

	mMediumPen.CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DSHADOW));

	mNormalPen.CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DFACE));

	mLightPen.CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DLIGHT));

	

	mHiliteDarkPen.CreatePen(PS_SOLID,1,RGB(0,0,255));

	mHiliteMediumPen.CreatePen(PS_SOLID,1,RGB(100,100,255));

	mHiliteLightPen.CreatePen(PS_SOLID,1,RGB(200,200,255));



	mHiliteDarkBrush.CreateSolidBrush(RGB(0,0,255));

	mHiliteMediumBrush.CreateSolidBrush(RGB(150,150,255));

	mHiliteLightBrush.CreateSolidBrush(RGB(200,200,255));



	mComponentDragPen.CreatePen(PS_SOLID,4,::GetSysColor(COLOR_3DLIGHT));



	mSelectedComponent = -1;

	mGraphManager = 0;

	mDragComponent = -1;

	mPinDragComponent = -1;

}



CNeilCapGUIController::~CNeilCapGUIController(void)

{

	ClearMessages();

}



void

CNeilCapGUIController::SetGraphManager(CGraphManager *gm)

{

	mGraphManager = gm;

}



CSize

CNeilCapGUIController::MeasureText(const char *s,CFont *ff)

{

	CDC	*cdc = mWnd->GetDC();

	CFont *f = cdc->SelectObject(ff);

	CSize siz = cdc->GetTextExtent(s,strlen(s));

	cdc->SelectObject(f);

	mWnd->ReleaseDC(cdc);

	return siz;

}



CSize

CNeilCapGUIController::MeasureText(const char *s)

{

	return MeasureText(s,&mFont);

}



CSize

CNeilCapGUIController::MeasureTextSmall(const char *s)

{

	return MeasureText(s,&mSmallFont);

}



void

CNeilCapGUIController::DrawComponent(CGUIComponentInfo &i,CDC *dc,CBrush *dark,CBrush *medium,CBrush *light)

{

	const CComponent *component = mGraphManager->GetComponent(i.mComponent);

	assert(component);

	if (!component) return;



	RECT r;



	r = i.mBoundsRect; r.right -= 1;

	r.top += 2;

	r.left += 2;

	dc->FrameRect(&r,dark);

	

	r = i.mBoundsRect; r.right -= 1;

	r.top += 1;

	r.left += 1;

	r.bottom -= 1;

	r.right -= 1;

	dc->FrameRect(&r,medium);



	r = i.mBoundsRect; r.right -= 1;

	r.bottom -= 2;

	r.right -= 2;

	dc->FrameRect(&r,light);



	nmr_string s = component->GetName();

	CFont *f = dc->SelectObject(&mFont);

	dc->SetBkColor(::GetSysColor(COLOR_3DFACE));

	dc->TextOut(r.left+4,r.top+10,s,s.length());

	dc->SelectObject(f);



	// draw the pins

	int x;

	for (x = 0; x < i.mPins.size(); ++x)

		DrawPin(i.mPins[x],dc);

}



void

CNeilCapGUIController::DrawComponent(CGUIComponentInfo &i,CDC *dc)

{

	CBrush *dark,*medium,*light;

	if (i.mComponent != mSelectedComponent)

		{

		dark = &mMediumBrush;

		medium = &mNormalBrush;

		light = &mLightBrush;

		}

	else

		{

		dark = &mHiliteDarkBrush;

		medium = &mHiliteMediumBrush;

		light = &mHiliteLightBrush;

		}

	DrawComponent(i,dc,dark,medium,light);

}



void

CNeilCapGUIController::DrawComponent(CGUIComponentInfo &i)

{

	CDC	*dc = mWnd->GetDC();

	DrawComponent(i,dc);

	mWnd->ReleaseDC(dc);

}



void	

CNeilCapGUIController::Draw(CDC *dc)

{

	dc->Draw3dRect(0,GRAPH_BOTTOM,32000,3,

		::GetSysColor(COLOR_3DLIGHT),

		::GetSysColor(COLOR_3DSHADOW));



	dc->Draw3dRect(VIDEO_WIDTH,GRAPH_BOTTOM,3,32000,

		::GetSysColor(COLOR_3DLIGHT),

		::GetSysColor(COLOR_3DSHADOW));



	ComponentInfoList_t::iterator i;

	int x;

	for (x = 0,i = mComponentInfoList.begin(); i != mComponentInfoList.end(); ++i,++x)

		DrawComponent(*i,dc);

}



void

CNeilCapGUIController::DrawPinConnection(RECT &r1,RECT &r2,CDC *dc)

{

	CPen *p = dc->SelectObject(&mDarkPen);



	dc->MoveTo(

		r1.left + ((r1.right - r1.left) / 2),

		r1.top + ((r1.bottom - r1.top) / 2)

		);

	dc->LineTo(

		r2.left + ((r2.right - r2.left) / 2),

		r2.top + ((r2.bottom - r2.top) / 2)

		);

	dc->SelectObject(p);

}



void	

CNeilCapGUIController::DrawPin(CGUIPinInfo &pin,CDC *dc)

{

	if (pin.mKind & CONN_SOURCE)

		DrawOutputPin(pin,dc);

	else

		DrawInputPin(pin,dc);



	if (pin.mConnected)

		{

		int other_component = pin.mConnectionComponent;

		int other_pinindex = pin.mConnectionPinIndex;

		RECT r = mComponentInfoList[other_component].mPins[other_pinindex].mBoundsRect;



		DrawPinConnection(pin.mBoundsRect,r,dc);

		}

}



void	

CNeilCapGUIController::DrawInputPin(CGUIPinInfo &pin,CDC *dc)

{

	RECT	r = pin.mBoundsRect;



	CFont *f = dc->SelectObject(&mSmallFont);

	dc->SetBkColor(::GetSysColor(COLOR_3DFACE));

	CSize	siz = dc->GetTextExtent(pin.mLabel,pin.mLabel.length());

	dc->TextOut(r.right - siz.cx,r.top - siz.cy,pin.mLabel,pin.mLabel.length());

	dc->SelectObject(f);



	CPen *p = dc->SelectObject(&mLightPen);



	if (pin.mKind & CONN_ACTIVE)

		{

		dc->MoveTo(r.left,r.bottom - ((r.bottom - r.top) / 2));

		dc->LineTo(r.right,r.top);

		dc->SelectObject(&mDarkPen);

		dc->MoveTo(r.left,r.bottom - ((r.bottom - r.top) / 2));

		dc->LineTo(r.right,r.bottom);

		}

	else

		{

		dc->MoveTo(r.left,r.top);

		dc->LineTo(r.right,r.bottom - ((r.bottom - r.top) / 2));

		dc->SelectObject(&mDarkPen);

		dc->MoveTo(r.left,r.bottom);

		dc->LineTo(r.right,r.bottom - ((r.bottom - r.top) / 2));

		}



	dc->SelectObject(p);

}



void	

CNeilCapGUIController::DrawOutputPin(CGUIPinInfo &pin,CDC *dc)

{

	RECT	r = pin.mBoundsRect;



	CFont *f = dc->SelectObject(&mSmallFont);

	dc->SetBkColor(::GetSysColor(COLOR_3DFACE));

	CSize	siz = dc->GetTextExtent(pin.mLabel,pin.mLabel.length());

	dc->TextOut(r.left,r.top - siz.cy,pin.mLabel,pin.mLabel.length());

	dc->SelectObject(f);



	CPen *p = dc->SelectObject(&mLightPen);



	if (pin.mKind & CONN_ACTIVE)

		{

		dc->MoveTo(r.left,r.top);

		dc->LineTo(r.right,r.bottom - ((r.bottom - r.top) / 2));

		dc->SelectObject(&mDarkPen);

		dc->MoveTo(r.left,r.bottom);

		dc->LineTo(r.right,r.bottom - ((r.bottom - r.top) / 2));

		}

	else

		{

		dc->MoveTo(r.left,r.bottom - ((r.bottom - r.top) / 2));

		dc->LineTo(r.right,r.top);

		dc->SelectObject(&mDarkPen);

		dc->MoveTo(r.left,r.bottom - ((r.bottom - r.top) / 2));

		dc->LineTo(r.right,r.bottom);

		}

	dc->SelectObject(p);

}



void

CNeilCapGUIController::GetInitialXYForComponent(int &x,int &y)

{

	// for now, just stick it in the middle.

	// This will have to be made intelligent later

	ComponentInfoList_t::iterator i;

	int maxx = 0;

	int maxy = 0;

	for (i = mComponentInfoList.begin(); i != mComponentInfoList.end(); ++i)

		{

		if ((*i).mBoundsRect.right > maxx)

			maxx = (*i).mBoundsRect.right;

		if ((*i).mBoundsRect.bottom > maxy)

			maxy = (*i).mBoundsRect.bottom;

		}

	if (maxx > maxy)

		{

		x = 60;

		y = maxy + 20;

		}

	else

		{

		x = maxx + 60;

		y = 20;

		}

}





void

CNeilCapGUIController::CalculateAppropriateHeightAndWidthForComponent(

										int master_index,int &h,int &w)

{

	// for now, just make it square

	const CComponent *component = mGraphManager->GetComponent(master_index);

	nmr_string s = component->GetName();



	CSize siz = MeasureText(s);



	// minimum

	h = siz.cy + 20;

	w = siz.cx + 20;



	siz = MeasureTextSmall("WWW");

	int small_h = siz.cy;

	int small_w = siz.cx;



	CComponent::ConnectionTypeList_t t = component->GetSupportedTypes();

	int sink_count	= 0;

	int source_count = 0;

	CComponent::ConnectionTypeList_t::iterator i;

	for (i = t.begin(); i != t.end(); ++i)

		{

		if ((*i) & CONN_SINK)

			++sink_count;

		else if ((*i) & CONN_SOURCE)

			++source_count;

		}



	int vertical_pins = sink_count > source_count ? sink_count : source_count;

	int new_h = (small_h * 3 * vertical_pins) + small_h;

	if (new_h > h) h = new_h;

}



void

CNeilCapGUIController::SetupComponentPinVisuals(int master_index,CGUIComponentInfo &info)

{

	CSize siz = MeasureTextSmall("WWW");

	const CComponent *component = mGraphManager->GetComponent(master_index);

	CComponent::ConnectionTypeList_t t = component->GetSupportedTypes();

	CComponent::ConnectionTypeList_t::iterator i;

	int sink_count = 0;

	int source_count = 0;

	for (i = t.begin(); i != t.end(); ++i)

		{

		CGUIPinInfo pin;

		pin.mConnected = 0;

		pin.mKind = (*i);

		pin.mLabel = ((*i) & CONN_AUDIO ? "Aud" : "Vid");

		RECT r;

		if ((*i) & CONN_SINK)

			{

			r.top = info.mBoundsRect.top + (siz.cy * 2) + 

				(sink_count * (10 + siz.cy * 2));

			r.bottom  = r.top + 10;

			r.right = info.mBoundsRect.left;

			r.left = r.right -10;

			sink_count++;

			}

		else if ((*i) & CONN_SOURCE)

			{

			r.top = info.mBoundsRect.top + (siz.cy * 2) + 

				(source_count * (10 + siz.cy * 2));

			r.bottom  = r.top + 10;

			r.right = info.mBoundsRect.right + 10;

			r.left = r.right -10;

			source_count++;

			}

		pin.mBoundsRect = r;

		info.mPins.push_back(pin);

		}

}



void

CNeilCapGUIController::AddComponent(int master_index)

{

	assert(mGraphManager);



	int x,y,h,w;

	CGUIComponentInfo	info;



	GetInitialXYForComponent(x,y);

	CalculateAppropriateHeightAndWidthForComponent(master_index,h,w);



	info.mComponent = master_index;

	info.mBoundsRect.left = x;

	info.mBoundsRect.right = x + w;

	info.mBoundsRect.top = y;

	info.mBoundsRect.bottom = y + h;

	if (info.mBoundsRect.bottom > VIDEO_BOTTOM)

		{

		info.mBoundsRect.top -= (info.mBoundsRect.bottom - VIDEO_BOTTOM);

		info.mBoundsRect.bottom -= (info.mBoundsRect.bottom - VIDEO_BOTTOM);

		}

	SetupComponentPinVisuals(master_index,info);



	mComponentInfoList.push_back(info);

	mWnd->Invalidate();

}



////////////////////////////////////////////////

////////////////////////////////////////////////





void

CNeilCapGUIController::GetVideoBoxXY(int &x,int &y,int width,int height)

{

	x = VIDEO_LEFT;

	y = VIDEO_TOP;

}



void

CNeilCapGUIController::CreateNewOutputStream(void *who)

{

	int siz = mObjectOutputMap.size();

	int x = ((siz & 0x01) ? LINEWIDTH : 0);

	int y = DIAGNOSTIC_TOP + ((siz / 2) * (LINEHEIGHT*NUM_LINES));

	RECT	r;

	r.left = x + DIAGNOSTIC_LEFT;

	r.right = x + LINEWIDTH + DIAGNOSTIC_LEFT;

	r.top = y;

	r.bottom = (LINEHEIGHT * NUM_LINES);

	mObjectOutputMap[who] = new CNeilCapStream(r,mWnd,0);

}



void

CNeilCapGUIController::ClearMessages(void)

{

	stack_MutexLock sml(&mMutex);



	ObjectOutputMap_t::iterator i;

	for (i = mObjectOutputMap.begin(); i != mObjectOutputMap.end(); ++i)

		delete (*i).second;



	mObjectOutputMap.erase(mObjectOutputMap.begin(),

		mObjectOutputMap.end());

}



CNeilCapStream&

CNeilCapGUIController::GetOutputStream(void *who)

{

	stack_MutexLock sml(&mMutex);



	if (mObjectOutputMap.find(who) == mObjectOutputMap.end())

		CreateNewOutputStream(who);

	return *(mObjectOutputMap[who]);

}



#ifndef NDEBUG

CMessageStream &	

CNeilCapGUIController::Trace(void *who)

{

	CNeilCapStream &s = GetOutputStream(who);

	s.SetOffset(LINEHEIGHT);

	return s;

}

#endif



CMessageStream&	

CNeilCapGUIController::Log(void *who)

{

	CNeilCapStream &s = GetOutputStream(who);

	s.SetOffset(0);

	return s;

}



CMessageStream&	

CNeilCapGUIController::Error(void *who)

{	

	CNeilCapStream &s = GetOutputStream(who);

	s.SetOffset(LINEHEIGHT*2);

	return s;

}



/////////////////////////////////////////////

//////////////////////////////////////////////



void

CNeilCapGUIController::ChangeSelection(int i)

{

	if (i == mSelectedComponent) return;

	int old = mSelectedComponent;

	mSelectedComponent = i;

	if (old > -1)

		DrawComponent(mComponentInfoList[old]);

	if (mSelectedComponent > -1)

		DrawComponent(mComponentInfoList[mSelectedComponent]);

}



int

CNeilCapGUIController::GetSelectedComponent(void)

{

	return mSelectedComponent;

}



int

CNeilCapGUIController::FindComponentAtXY(int x,int y)

{

	for (int i = 0; i < mComponentInfoList.size(); ++i)

		{

		RECT r = mComponentInfoList[i].mBoundsRect;

		if (x >= r.left && x <= r.right && y >= r.top && y <= r.bottom)

			return i;

		}

	return -1;

}



int

CNeilCapGUIController::FindPinAtXY(int x,int y,int &component,

							CConnection::Connection_t &t,int &pin_index,RECT &pin_rect)

{

	for (int i = 0; i < mComponentInfoList.size(); ++i)

		{

		component = i;

		int iii;

		for (iii = 0; iii < mComponentInfoList[i].mPins.size(); ++iii)

			{

			t = mComponentInfoList[i].mPins[iii].mKind;

			pin_index = iii;

			RECT r = mComponentInfoList[i].mPins[iii].mBoundsRect;

			pin_rect = r;

			if (x >= r.left && x <= r.right && y >= r.top && y <= r.bottom)

				return 1;



			}

		}

	return 0;

}



void

CNeilCapGUIController::DrawDragRect(RECT &r)

{

	CDC	*dc = mWnd->GetDC();

	CPen	*p = dc->SelectObject(&mComponentDragPen);

	int	prev_draw_mode = dc->SetROP2(R2_XORPEN);

	dc->MoveTo(r.left,r.top);

	dc->LineTo(r.right,r.top);

	dc->LineTo(r.right,r.bottom);

	dc->LineTo(r.left,r.bottom);

	dc->LineTo(r.left,r.top);

	dc->SetROP2(prev_draw_mode);

	dc->SelectObject(p);

	mWnd->ReleaseDC(dc);

}



void

CNeilCapGUIController::OffsetComponent(int component,int xoffset,int yoffset)

{

	RECT r = mComponentInfoList[component].mBoundsRect;

	if (r.bottom + yoffset > GRAPH_BOTTOM)

		yoffset = GRAPH_BOTTOM - r.bottom;



	r.top += yoffset;

	r.bottom += yoffset;

	r.left += xoffset;

	r.right += xoffset;

	mComponentInfoList[component].mBoundsRect = r;



	int x;

	for (x = 0; x < mComponentInfoList[component].mPins.size(); ++x)

		{

		RECT r = mComponentInfoList[component].mPins[x].mBoundsRect;

		r.top += yoffset;

		r.bottom += yoffset;

		r.left += xoffset;

		r.right += xoffset;

		mComponentInfoList[component].mPins[x].mBoundsRect = r;

		}

}



void

CNeilCapGUIController::DeleteSelectedComponent(void)

{

	if (mSelectedComponent == -1) return;



	int	master_index;

	ComponentInfoList_t::iterator i;

	int x;

	for (x = 0,i = mComponentInfoList.begin(); x < mComponentInfoList.size(); ++x,++i)

		{

		if (x == mSelectedComponent)

			{

			master_index = (*i).mComponent;

			mComponentInfoList.erase(i);

			break;

			}

		}



	for (i = mComponentInfoList.begin(); i != mComponentInfoList.end(); ++i)

		{

		assert((*i).mComponent != master_index);

		if ((*i).mComponent > master_index)

			--(*i).mComponent;

		int iii;

		for (iii = 0; iii < (*i).mPins.size(); ++iii)

			{

			if ((*i).mPins[iii].mConnected &&

						(*i).mPins[iii].mConnectionComponent == master_index)

				(*i).mPins[iii].mConnected = 0;

			else if ((*i).mPins[iii].mConnected && 

							(*i).mPins[iii].mConnectionComponent > master_index)

				(*i).mPins[iii].mConnectionComponent--;

			}

		}



	mGraphManager->RemoveComponent(master_index);

	mSelectedComponent = -1;

	mWnd->Invalidate();

}



void	

CNeilCapGUIController::DrawPinDrag(RECT &base_rect,int x,int y)

{

	CDC	*dc = mWnd->GetDC();

	CPen	*p = dc->SelectObject(&mLightPen);

	int	prev_draw_mode = dc->SetROP2(R2_XORPEN);

	dc->MoveTo(

		base_rect.left + ((base_rect.right - base_rect.left) / 2),

		base_rect.top + ((base_rect.bottom - base_rect.top) / 2)

		);

	dc->LineTo(x,y);

	dc->SetROP2(prev_draw_mode);

	dc->SelectObject(p);

	mWnd->ReleaseDC(dc);

}





void

CNeilCapGUIController::BreakPinConnection(int component,CConnection::Connection_t t,

								   int pin_index,RECT pin_rect)

{

	assert(mGraphManager);



	if (mComponentInfoList[component].mPins[pin_index].mConnected)

		{

		mComponentInfoList[component].mPins[pin_index].mConnected = 0;

		int other_component = 

			mComponentInfoList[component].mPins[pin_index].mConnectionComponent;

		int other_pin_index = 

			mComponentInfoList[component].mPins[pin_index].mConnectionPinIndex;



		assert(mComponentInfoList[other_component].mPins[other_pin_index].mConnected);

		assert(mComponentInfoList[other_component].mPins[other_pin_index].mConnectionComponent == component);

		assert(mComponentInfoList[other_component].mPins[other_pin_index].mConnectionPinIndex == pin_index);

		

		mComponentInfoList[other_component].mPins[other_pin_index].mConnected = 0;

		

		mGraphManager->RemoveConnection(

			mComponentInfoList[component].mComponent,

			mComponentInfoList[other_component].mComponent,

			mComponentInfoList[component].mPins[pin_index].mKind,

			mComponentInfoList[other_component].mPins[other_pin_index].mKind);

		}

}



void	

CNeilCapGUIController::ConnectPins(int source_component,CConnection::Connection_t source_type,

						int source_pin_index,

						int dest_component,CConnection::Connection_t dest_type,

						int dest_pin_index)

{

	assert(mGraphManager);



	if (!mGraphManager->AddConnection(

		mComponentInfoList[source_component].mComponent,

		mComponentInfoList[dest_component].mComponent,

		source_type,

		dest_type)) return;



	assert(!mComponentInfoList[source_component].mPins[source_pin_index].mConnected);



	mComponentInfoList[source_component].mPins[source_pin_index].mConnected = 1;

	mComponentInfoList[source_component].mPins[source_pin_index].mConnectionComponent = dest_component;

	mComponentInfoList[source_component].mPins[source_pin_index].mConnectionType = dest_type;

	mComponentInfoList[source_component].mPins[source_pin_index].mConnectionPinIndex = dest_pin_index;



	mComponentInfoList[dest_component].mPins[dest_pin_index].mConnected = 1;

	mComponentInfoList[dest_component].mPins[dest_pin_index].mConnectionComponent = source_component;

	mComponentInfoList[dest_component].mPins[dest_pin_index].mConnectionType = source_type;

	mComponentInfoList[dest_component].mPins[dest_pin_index].mConnectionPinIndex = source_pin_index;



}



///////////////////////////

///////////////////////////





void

CNeilCapGUIController::SelectButtonDown(int x,int y)

{

	CConnection::Connection_t	t;

	int	pin_index;

	RECT pin_rect;



	int component = FindComponentAtXY(x,y);

	if (component != -1)

		ChangeSelection(component);

	else if (FindPinAtXY(x,y,component,t,pin_index,pin_rect))

		{

		BreakPinConnection(component,t,pin_index,pin_rect);

		mWnd->Invalidate();

		}

	else

		ChangeSelection(-1);

}



int	

CNeilCapGUIController::BeginDrag(int x,int y)

{

	int component;

	int pin_index;

	RECT pin_rect;

	CConnection::Connection_t t;



	mDragComponent = -1;

	mPinDragComponent = -1;



	mDragComponent = FindComponentAtXY(x,y);

	if (mDragComponent != -1)

		{

		mLastDragX = x;

		mLastDragY = y;

		mDragRect = mComponentInfoList[mDragComponent].mBoundsRect;

		DrawDragRect(mDragRect);

		return 1;

		}

	if (FindPinAtXY(x,y,component,t,pin_index,pin_rect))

		{

		mLastDragX = x;

		mLastDragY = y;

		mPinDragComponent = component;

		mPinDragType = t;

		mPinDragIndex = pin_index;

		mPinDragRect = pin_rect;

		DrawPinDrag(mPinDragRect,x,y);

		return 1;

		}

	return 0;

}



void	

CNeilCapGUIController::Drag(int x,int y)

{

	if (mDragComponent != -1)

		{

		DrawDragRect(mDragRect);

		int xdiff = x - mLastDragX;

		int ydiff = y - mLastDragY;

		mLastDragX = x;

		mLastDragY = y;

		mDragRect.left += xdiff;

		mDragRect.right += xdiff;

		mDragRect.top += ydiff;

		mDragRect.bottom += ydiff;

		DrawDragRect(mDragRect);

		}

	else if (mPinDragComponent != -1)

		{

		DrawPinDrag(mPinDragRect,mLastDragX,mLastDragY);

		mLastDragX = x;

		mLastDragY = y;

		DrawPinDrag(mPinDragRect,x,y);

		}

}



void	

CNeilCapGUIController::EndDrag(int x,int y)

{

	if (mDragComponent != -1)

		{

		DrawDragRect(mDragRect);

		int xdiff = mDragRect.left - mComponentInfoList[mDragComponent].mBoundsRect.left;

		int ydiff = mDragRect.top - mComponentInfoList[mDragComponent].mBoundsRect.top;

		OffsetComponent(mDragComponent,xdiff,ydiff);

		}

	else if (mPinDragComponent != -1)

		{

		DrawPinDrag(mPinDragRect,mLastDragX,mLastDragY);

		int component,pin_index;

		RECT pin_rect;

		CConnection::Connection_t t;

		if (FindPinAtXY(x,y,component,t,pin_index,pin_rect))

			{

			ConnectPins(mPinDragComponent,mPinDragType,mPinDragIndex,

				component,t,pin_index);

			}

		}



	mWnd->Invalidate();

	mDragComponent = -1;

	mPinDragComponent = -1;

}



void

CNeilCapGUIController::ErrorDialog(const char *message)

{

	::MessageBox(mWnd->m_hWnd,message,"Error",MB_OK | MB_APPLMODAL);

}



void

CNeilCapGUIController::ConfigureSelectedComponent(void)

{

	if (mSelectedComponent == -1) return;

	if (!mGraphManager) return;



	mGraphManager->ConfigureComponent(

		mComponentInfoList[mSelectedComponent].mComponent,mWnd);

}



CWnd *

CNeilCapGUIController::GetWindow(void)

{

	return mWnd;

}



