/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CNEILCAPGUICONTROLLER_H_

#define CNEILCAPGUICONTROLLER_H_



#include <MegaHeaders.h>

#include <nmr_string.h>

#include <CCMutex.h>

#include <map.h>

#include <vector.h>



#include "CGUIController.h"

#include "CConnection.h"

#include "CNeilCapStream.h"



class CGUIComponentInfo;

class CNeilCapGUIController;

class CUIPinInfo;

class CGraphManager;



class CGUIPinInfo

{

public:

	CGUIPinInfo(void){}

	~CGUIPinInfo(void){}

protected:



	RECT	mBoundsRect;

	nmr_string	mLabel;

	CConnection::Connection_t	mKind;



	int		mConnected;

	int		mConnectionComponent;

	CConnection::Connection_t		

			mConnectionType;

	int		mConnectionPinIndex;



	friend CNeilCapGUIController;

	friend CGUIComponentInfo;

};



class CGUIComponentInfo

{

public:

	CGUIComponentInfo(void){}

	~CGUIComponentInfo(void){}

protected:



	int		mComponent;	//component index

	RECT	mBoundsRect;

	vector<CGUIPinInfo>	mPins;



	friend CNeilCapGUIController;

};



class CNeilCapGUIController: public CGUIController

{

	CFont	mFont,mSmallFont;

	CBrush	mDarkBrush,mMediumBrush,mNormalBrush,mLightBrush;

	CPen	mDarkPen,mMediumPen,mNormalPen,mLightPen;

	CPen	mHiliteDarkPen,mHiliteMediumPen,mHiliteLightPen;

	CBrush	mHiliteDarkBrush,mHiliteMediumBrush,mHiliteLightBrush;

	CPen	mComponentDragPen;

	CCMutex	mMutex;

	CWnd	*mWnd;



	typedef map<void*,CNeilCapStream*,less<void*> > ObjectOutputMap_t;

	typedef vector<CGUIComponentInfo> ComponentInfoList_t;



	ObjectOutputMap_t	mObjectOutputMap;

	ComponentInfoList_t	mComponentInfoList;

	CGraphManager	*mGraphManager;



	int		mSelectedComponent;

	int		mLastDragX,mLastDragY;

	RECT	mDragRect;

	int		mDragComponent;

	int		mPinDragComponent;

	CConnection::Connection_t mPinDragType;

	int		mPinDragIndex;

	RECT	mPinDragRect;



	void	CreateNewOutputStream(void *who);

	CNeilCapStream	&GetOutputStream(void *who);

	void	GetInitialXYForComponent(int &x,int &y);

	void	CalculateAppropriateHeightAndWidthForComponent(int index,int &h,int &w);

	void	SetupComponentPinVisuals(int index,CGUIComponentInfo &info);

	CSize	MeasureText(const char *s,CFont *ff);

	CSize	MeasureText(const char *s);

	CSize	MeasureTextSmall(const char *s);

	void	DrawInputPin(CGUIPinInfo &pin,CDC *dc);

	void	DrawOutputPin(CGUIPinInfo &pin,CDC *dc);

	void	DrawPin(CGUIPinInfo &pin,CDC *dc);

	int		FindComponentAtXY(int x,int y);

	int		FindPinAtXY(int x,int y,int &component,CConnection::Connection_t &t,int &pin_index,RECT &pin_rect);

	void	ChangeSelection(int i);

	void	DrawComponent(CGUIComponentInfo &i,CDC *dc,CBrush *dark,CBrush *medium,CBrush *light);

	void	DrawComponent(CGUIComponentInfo &i,CDC *dc);

	void	DrawComponent(CGUIComponentInfo &i);

	void	DrawDragRect(RECT &r);

	void	OffsetComponent(int component,int xoffset,int yoffset);

	void	DrawPinDrag(RECT &base_rect,int x,int y);

	void	BreakPinConnection(int component,CConnection::Connection_t t,

							int pin_index,RECT pin_rect);

	void	ConnectPins(int source_component,CConnection::Connection_t source_type,

						int source_pin_index,

						int dest_component,CConnection::Connection_t dest_type,

						int dest_pin_index);

	void	DrawPinConnection(RECT &r1,RECT &r2,CDC *dc);





public:

	CNeilCapGUIController(CWnd *window);

	~CNeilCapGUIController(void);



	void	AddComponent(int index);

	void	SetGraphManager(CGraphManager *gm);

	void	SelectButtonDown(int x,int y);	

	int		BeginDrag(int x,int y);

	void	Drag(int x,int y);

	void	EndDrag(int x,int y);

	void	DeleteSelectedComponent(void);

	int		GetSelectedComponent(void);

	void	ConfigureSelectedComponent(void);

	

	CWnd	*GetWindow(void);

	void	GetVideoBoxXY(int &x,int &y,int width,int height);



	virtual	void	ClearMessages(void);

#ifndef NDEBUG

	CMessageStream	&Trace(void *who);

#endif

	CMessageStream	&Log(void *who);

	CMessageStream	&Error(void *who);



	void	ErrorDialog(const char *message);

	void	Draw(CDC *dc);

};





#endif

