/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CStreamInfoObject.h"
#include <stack_aids.h>
#include "danbug.h"

CStreamInfoObject::CStreamInfoObject(void)
{
	mStreamFormat = 0;
	Clear();
}

CStreamInfoObject::CStreamInfoObject(const CStreamInfoObject &o)
{
	mStreamInfo 		= o.mStreamInfo;
	mStreamFormatSize 	= o.mStreamFormatSize;	
	mStreamFormat		= new char[o.mStreamFormatSize];
	memcpy(mStreamFormat,o.mStreamFormat,mStreamFormatSize);
	mStreamFormatSet = 1;
	mStreamInfoSet = 1;
}

CStreamInfoObject &
CStreamInfoObject::operator=(const CStreamInfoObject &o)
{
	mStreamInfoMutex.Lock();
	mStreamInfo = o.mStreamInfo;
	mStreamInfoSet = 1;
	mStreamInfoWait.Signal();
	mStreamInfoMutex.Unlock();

	mStreamFormatMutex.Lock();
	mStreamFormatSize 	= o.mStreamFormatSize;	
	delete [] mStreamFormat;
	mStreamFormat		= new char[o.mStreamFormatSize];
	memcpy(mStreamFormat,o.mStreamFormat,mStreamFormatSize);
	mStreamFormatSet = 1;
	mStreamFormatWait.Signal();
	mStreamFormatMutex.Unlock();



	return (*this);
}


CStreamInfoObject::~CStreamInfoObject(void)
{
	Clear();
}

void
CStreamInfoObject::Clear(void)
{
	delete [] mStreamFormat;
	mStreamFormat = 0;
	mStreamFormatSize = 0;
	mStreamFormatSet = 0;
	mStreamInfoSet = 0;
}

void	
CStreamInfoObject::SetStreamInfo(AVISTREAMINFO &i)
{
	stack_MutexLock sml(&mStreamInfoMutex);

	mStreamInfo = i;
	mStreamInfoSet = 1;
	mStreamInfoWait.Signal();
}

void	
CStreamInfoObject::SetStreamFormat(const char *f,size_t s)
{
	stack_MutexLock sml(&mStreamFormatMutex);
 dprintf("setting format in CStreamInfoObject::SetStreamFormat; size=%d",s);
 dprintf("dump of format data:");
 dumpdata((unsigned char*)f,3);

	delete [] mStreamFormat;
	mStreamFormat = 0;
	mStreamFormatSize = s;
	if (s)
		{
		mStreamFormat = new char[mStreamFormatSize];
		memcpy(mStreamFormat,f,mStreamFormatSize);
		}
	mStreamFormatSet = 1;
	mStreamFormatWait.Signal();
}

AVISTREAMINFO	
CStreamInfoObject::GetStreamInfo(void)
{
	stack_MutexLock sml(&mStreamInfoMutex);

	assert(mStreamInfoSet);
	return mStreamInfo;
}


size_t			
CStreamInfoObject::GetStreamFormatSize(void)
{
	stack_MutexLock sml(&mStreamFormatMutex);

	assert(mStreamFormatSet);
	return mStreamFormatSize;
}

const char	*
CStreamInfoObject::GetStreamFormat(void)
{
	stack_MutexLock sml(&mStreamFormatMutex);

	assert(mStreamFormatSet);
	return mStreamFormat;
}

int				
CStreamInfoObject::IsStreamInfoSet(void)
{
	stack_MutexLock sml(&mStreamInfoMutex);

	return mStreamInfoSet;
}

int				
CStreamInfoObject::IsStreamFormatSet(void)
{
	stack_MutexLock sml(&mStreamFormatMutex);

	return mStreamFormatSet;
}

void			
CStreamInfoObject::WaitForStreamFormatSet(void)
{
	stack_MutexLock sml(&mStreamFormatMutex);

	while (!mStreamFormatSet)
		mStreamFormatWait.Wait(mStreamFormatMutex);
}

void			
CStreamInfoObject::WaitForStreamInfoSet(void)
{
	stack_MutexLock sml(&mStreamInfoMutex);

	while (!mStreamInfoSet)
		mStreamInfoWait.Wait(mStreamInfoMutex);
}



