/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CVidcapSource.h"
#include "CGUIController.h"
#include "CPassiveSink.h"
#include "CMessageStream.h"
#include <safe_ostrstream.h>
#include <stack_aids.h>
#include <vfw.h>
#include "danbug.h"
#include "..\dansoft\acmenc.h"

#define MAXVIDDRIVERS 10
#define CAPTUREBUFFERS 60 // 100



//int CVidcapStream::mState;
//CCMutex CVidcapStream::mStateMutex;
//CConditionVariable CVidcapStream::mStateWait;

void
CVidcapSource::AttachCapDriver(HWND wnd)

{
	int x,drivercount = 0;
	TCHAR	achDeviceName[80];
	TCHAR	achDeviceVersion[100];

	for (x = 0 ; x < MAXVIDDRIVERS ; x++) 
		{
		if (capGetDriverDescription(x,(LPTSTR)achDeviceName, 
			sizeof(achDeviceName)/ sizeof(TCHAR),(LPTSTR)achDeviceVersion, 
									sizeof(achDeviceVersion)/sizeof(TCHAR))) 
			{
			// There is such a driver in the "system.ini" file.
			if (drivercount++ == 0) 
				{
				// Only if no other driver is already connected
				if (capDriverConnect(wnd, x))
					{
#ifndef NDEBUG
					(mGUIController->Trace(this))
						<< "Capture driver connected" << &(CMessageStream::show);
#endif
					return;
					//gwDeviceIndex = x ;
					}
				}
			} // end of if (capGetDriverDesc..())
		}
	throw VidcapError("Could not attach capture driver",0);
}




CVidcapSource::CVidcapSource(CGUIController *gui)
				: CComponent(gui)
{
	try {
	SetDataIgnore(0);
	mIgnoreStatusMessages = 0;
	mIgnoreErrorMessages = 0;
	mStopCounter = -1;
	mAudioBytesProcessed = 0;
	mVideoBytesProcessed = 0;
	mVideoFramesProcessed = 0;
	mAudioConfigured = 0;
	mVideoConfigured = 0;
	mCaptureWindow = 0;
	mAudio = 0;
	mVideo = 0;
	mAudioUsed = 0;
	mVideoUsed = 0;
	mAudio = new CVidcapSourceAudio(this,gui);
	mVideo = new CVidcapSourceVideo(this,gui);
	mOperational = 0;
	mFramesToKeep = 1;
	mFramesToDrop = 0;
	mFramesDropped = 0;
	mFramesToRepeat = 0;
	mFramerateBasis = 29.97;



	mAudioFormat.wBitsPerSample = 

		AfxGetApp()->GetProfileInt("CVidcapSource","BitsPerSample",8);

	mAudioFormat.nSamplesPerSec = 

		AfxGetApp()->GetProfileInt("CVidcapSource","Frequency",22050);

	mAudioFormat.nChannels		=

		AfxGetApp()->GetProfileInt("CVidcapSource","Channels",1);



	mVideoWidth	= AfxGetApp()->GetProfileInt("CVidcapSource","VideoWidth",320);

	mVideoHeight= AfxGetApp()->GetProfileInt("CVidcapSource","VideoHeight",240);



	mFramesToKeep = AfxGetApp()->GetProfileInt("CVidcapSource","FramesToKeep",1);

	mFramesToDrop = AfxGetApp()->GetProfileInt("CVidcapSource","FramesToDrop",0);

	CString s = AfxGetApp()->GetProfileString("CVidcapSource","FramerateBasis","29.97");

	sscanf(s,"%f",&mFramerateBasis);



	int x,y;

	mGUIController->GetVideoBoxXY(x,y,mVideoWidth,mVideoHeight);

	mIgnoreErrorMessages = 1;

	mCaptureWindow = capCreateCaptureWindow("Capture Window",

		WS_CHILD | WS_VISIBLE,x,y,mVideoWidth,mVideoHeight,mGUIController->GetWindow()->m_hWnd,0);

	mIgnoreErrorMessages = 0;



	if (!mCaptureWindow)

		throw VidcapError("Could not create capture window",0);



	::SetProp(mCaptureWindow,"duck",(void *)this);



	if (!capSetCallbackOnStatus(mCaptureWindow,StatusCallback))

		throw VidcapError("Could not set status callback",0);



	if (!capSetCallbackOnError(mCaptureWindow,ErrorCallback))

		throw VidcapError("Could not set error callback",0);



	if (!capSetCallbackOnYield(mCaptureWindow,YieldCallback))

		throw VidcapError("Could not set yield callback",0);



//	AttachCapDriver(mCaptureWindow);

//	AdjustVideoSize();

//	capOverlay(mCaptureWindow,TRUE);

	}

catch(...)

	{

	_Cleanup();

	throw;

	}

}



void

CVidcapSource::VidcapInitialize(void)

{

	AttachCapDriver(mCaptureWindow);

	AdjustVideoSize();

	capOverlay(mCaptureWindow,TRUE);

}



void

CVidcapSource::_Cleanup(void)

{

	if (mCaptureWindow)

		{

		capSetCallbackOnError(mCaptureWindow,NULL);

		capSetCallbackOnStatus(mCaptureWindow,NULL);

		capSetCallbackOnVideoStream(mCaptureWindow,NULL);

		capSetCallbackOnWaveStream(mCaptureWindow,NULL);

		capSetCallbackOnYield(mCaptureWindow,NULL);

		::DestroyWindow(mCaptureWindow);

		mCaptureWindow = 0;

		}

	delete mAudio;

	delete mVideo;

}



CVidcapSource::~CVidcapSource(void)

{

	Stop();

	AfxGetApp()->WriteProfileInt("CVidcapSource","FramesToKeep",mFramesToKeep);

	AfxGetApp()->WriteProfileInt("CVidcapSource","FramesToDrop",mFramesToDrop);

	char buf[256];

	sprintf(buf,"%f",mFramerateBasis);

	CString s = buf;

	AfxGetApp()->WriteProfileString("CVidcapSource","FramerateBasis",s);

	AfxGetApp()->WriteProfileInt("CVidcapSource","BitsPerSample",

								mAudioFormat.wBitsPerSample);

	AfxGetApp()->WriteProfileInt("CVidcapSource","Frequency",

								mAudioFormat.nSamplesPerSec);

	AfxGetApp()->WriteProfileInt("CVidcapSource","Channels",

								mAudioFormat.nChannels);



	_Cleanup();

}



void	

CVidcapSource::ReturnConnection(CConnection *c,CConnection::Connection_t t)

{

	if (c == mAudio)

		{

		mAudioUsed = 0;

		if (mCaptureWindow)

			capSetCallbackOnWaveStream(mCaptureWindow,NULL);

		}

	else if (c == mVideo)

		{

		mVideoUsed = 0;

		if (mCaptureWindow)

			capSetCallbackOnVideoStream(mCaptureWindow,NULL);

		}

	else 

		throw VidcapError("Unknown connection returned",(long)c);

}



CComponent::ConnectionTypeList_t 

CVidcapSource::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_AUDIO);

	return l;

}



nmr_string 

CVidcapSource::GetName(void) const

{

	return "Vidcap Source";

}





CConnection *

CVidcapSource::CreateConnection(CConnection::Connection_t t)

{

	assert(mCaptureWindow);

	CConnection::Connection_t	minimum_requirements = (CONN_ACTIVE | CONN_SOURCE);

	

	if ((t & minimum_requirements) != minimum_requirements)

		return 0;

	

	if (t & CONN_AUDIO)

		{

		if (mAudioUsed) return 0;

		mAudioUsed = 1;

		if (!capSetCallbackOnWaveStream(mCaptureWindow,WaveCallback))

			throw VidcapError("Could not set audio callback",0);

		return mAudio;

		}

	if (t & CONN_VIDEO)

		{

		if (mVideoUsed) return 0;

		mVideoUsed = 1;

		if (!capSetCallbackOnVideoStream(mCaptureWindow,VideoCallback))

			throw VidcapError("Could not set video callback",0);

		return mVideo;

		}

	

	return 0;

}


void
CVidcapSource::LogFrames(int frames)
{
	if (frames < 100 || frames % 25 == 0)
//	(mGUIController->Log(this)) << frames << " frames captured, " <<
//		mFramesDropped << " frames dropped." << CMessageStream::show;
	(mGUIController->Log(this)) << frames << " frames captured, " <<
		mFramesDropped << " frames dropped." << &(CMessageStream::show);
}



void

CVidcapSource::FrameDropped(void)

{

	mFramesDropped++;

	mFramesToRepeat++;

}



int

CVidcapSource::GetFramesToRepeat(void)

{

	assert(mFramesToRepeat >= 0);

	return mFramesToRepeat;

}



void

CVidcapSource::DecrementFramesToRepeat(void)

{

	mFramesToRepeat--;

}



void

CVidcapSource::SetDataIgnore(int i)

{

	stack_MutexLock sml(&mDataIgnoreMutex);

	mDataIgnore = i;

}





void

CVidcapSource::Initialize(void)

{

	SetDataIgnore(0);

	mFramesDropped = 0;

	mFramesToRepeat = 0;

	//CVidcapStream::mState = (mVideoUsed ? 1 : 0);

}



void

CVidcapSource::Cleanup(void)

{

	stack_MutexLock sml(&mOperationalMutex);

	while (mOperational)

		mOperationalWait.Wait(mOperationalMutex);

}



void

CVidcapSource::AdjustVideoSize(void)

{

	BITMAPINFO bmi;

	capGetVideoFormat(mCaptureWindow,&bmi,sizeof(bmi));

	mVideoWidth = bmi.bmiHeader.biWidth;

	mVideoHeight = bmi.bmiHeader.biHeight;

	AfxGetApp()->WriteProfileInt("CVidcapSource","VideoWidth",mVideoWidth);

	AfxGetApp()->WriteProfileInt("CVidcapSource","VideoHeight",mVideoHeight);

	int x,y;

	mGUIController->GetVideoBoxXY(x,y,mVideoWidth,mVideoHeight);

	::MoveWindow(mCaptureWindow,x,y,mVideoWidth,mVideoHeight,TRUE);

}





////





void

CVidcapSource::ErrorMessage(int id,LPCSTR mess)

{

	if (mIgnoreErrorMessages) return;
 dprintf("CVidcapSource::ErrorMessage");

	if (id || *mess)
		(mGUIController->Error(this)) << mess << " (" << id << ")" << &(CMessageStream::show);

}



void

CVidcapSource::StatusMessage(int id,LPCSTR mess)

{

	if (!mIgnoreStatusMessages)
		(mGUIController->Log(this)) << mess << " (" << id << ")" << &(CMessageStream::show);

	if (id == IDS_CAP_BEGIN)

		{

		// intialize stuff

		mKeepFrameCounter = mFramesToKeep;

		mDropFrameCounter = mFramesToDrop;

		}

}





LRESULT CALLBACK ErrorCallback(HWND wind,int id, LPCSTR mess)

{

	CVidcapSource *vs = (CVidcapSource *)::GetProp(wind,"duck");

	assert(vs);
 dprintf("ErrorCallback");
	vs->ErrorMessage(id,mess);

	return 0;

}





LRESULT CALLBACK StatusCallback(HWND wind,int id,LPCSTR mess)

{

	CVidcapSource *vs = (CVidcapSource *)::GetProp(wind,"duck");

	assert(vs);

	vs->StatusMessage(id,mess);

	return 0;

}



LRESULT CALLBACK YieldCallback(HWND wnd)

{

	CVidcapSource *vs = (CVidcapSource *)::GetProp(wnd,"duck");

	assert(vs);

	return vs->ProcessYield();

}





void *previous_video_ptr = 0;



LRESULT CALLBACK VideoCallback(HWND wnd,void *vh)

{

	if (vh == previous_video_ptr) 

		return 0;

	previous_video_ptr = vh;

	CVidcapSource *vs = (CVidcapSource *)::GetProp(wnd,"duck");

	assert(vs);

	vs->ProcessVideo((LPVIDEOHDR)vh);

	return 0;

}



LRESULT CALLBACK WaveCallback(HWND wnd,void *vh)

{

	CVidcapSource *vs = (CVidcapSource *)::GetProp(wnd,"duck");

	assert(vs);

	vs->ProcessAudio((LPWAVEHDR)vh);

	return 0;

}



int

CVidcapSource::ProcessYield(void)

{

	//CCThread::Yield();

	stack_MutexLock sml(&mOperationalMutex);

	return mOperational;

}



void	

CVidcapSource::ProcessAudio(LPWAVEHDR wh)

{

	stack_MutexLock sml(&mDataIgnoreMutex);



	if (!mAudioUsed) return;

	if (mDataIgnore) return;

	mAudio->ProcessAudio(wh);

}







void	

CVidcapSource::ProcessVideo(LPVIDEOHDR vh)

{

	stack_MutexLock sml(&mDataIgnoreMutex);



	if (!mVideoUsed) return;

	if (mDataIgnore) return;



	if (mKeepFrameCounter)

		{

		mVideo->ProcessVideo(vh);

		--mKeepFrameCounter;

		}

	else if (mDropFrameCounter)

		{

		--mDropFrameCounter;

		}

	else

		{

		mKeepFrameCounter	= mFramesToKeep;

		mDropFrameCounter	=mFramesToDrop;

		mVideo->ProcessVideo(vh);

		--mKeepFrameCounter;

		}

}





void	

CVidcapSource::GetAudioInfo(AVISTREAMINFO &i)

{

	memset(&i,0,sizeof(i));

	i.fccType = streamtypeAUDIO; 

    i.dwScale = 1; 

    i.dwRate = mAudioFormat.nSamplesPerSec; 

    i.dwQuality = -1; 

    i.dwSampleSize = 1; 

}



void	

CVidcapSource::GetVideoInfo(AVISTREAMINFO &i)

{

	memset(&i,0,sizeof(i));

	i.fccType = streamtypeVIDEO; 

    i.dwScale = 1000000 * (mFramesToKeep + mFramesToDrop); 

    i.dwRate = (int)(mFramerateBasis * 1000000) * (mFramesToKeep);

    i.dwQuality = -1; 

}



size_t	

CVidcapSource::GetAudioFormatSize(void)

{

	assert(mCaptureWindow);

	return capGetAudioFormatSize(mCaptureWindow);

}



size_t	

CVidcapSource::GetVideoFormatSize(void)

{

	assert(mCaptureWindow);

	return capGetVideoFormatSize(mCaptureWindow);

}



void	

CVidcapSource::GetAudioFormat(char *f,size_t fs)

{

	capGetAudioFormat(mCaptureWindow,f,fs);

}



void	

CVidcapSource::GetVideoFormat(char *f,size_t fs)

{
 dprintf(" CVidcapSource::GetVideoFormat called");
 	capGetVideoFormat(mCaptureWindow,f,fs);

}



void

CVidcapSource::Stop(void)

{

	//_ShowStatus();

	stack_MutexLock sml(&mStopMutex);

	if (--mStopCounter > 0) return;

	if (mStopCounter < 0) return;
	mOperationalMutex.Lock();

	mOperational = 0;

	mOperationalMutex.Unlock();

	mOperationalWait.Signal();

	mCaptureThread.Join();

	if (mFramesDropped == 0)

		(mGUIController->Error(this)) << "Capture successful" << &(CMessageStream::show);

	else

		(mGUIController->Error(this)) << "Warning, " << mFramesDropped << " frames dropped."

			<< &(CMessageStream::show);

}



void

CVidcapSource::Start(void)

{

	stack_MutexLock sml(&mOperationalMutex);

	if (mOperational) return;

	assert(mCaptureWindow);

#ifdef USE_SKEW

	mSkewWaitFlag = 1;

#endif

	mAudioBytesProcessed = 0;
	mVideoBytesProcessed = 0;
	mVideoFramesProcessed = 0;
	SetDataIgnore(0);

	mStopCounter = 0;

	if (mVideoUsed) ++mStopCounter;

	if (mAudioUsed) ++mStopCounter;



	CAPTUREPARMS	parms;

	if (!capCaptureGetSetup(mCaptureWindow,&parms,sizeof(parms)))

		throw VidcapError("Could not get capture parameters",0);

	memset(&parms,0,sizeof(parms));

	parms.fCaptureAudio = mAudioUsed;

	parms.fAbortLeftMouse = FALSE;

	parms.fAbortRightMouse = FALSE;

	parms.vKeyAbort = FALSE;

	parms.fYield = TRUE;

	parms.wNumVideoRequested = CAPTUREBUFFERS;

	parms.dwRequestMicroSecPerFrame = 1000000 / 30;//29.97; //1000000 / 100;

	parms.wNumAudioRequested = 10; // ten is max was 15;



	//parms.dwAudioBufferSize = (mAudioFormat.wBitsPerSample *

	//	mAudioFormat.nSamplesPerSec * mAudioFormat.nChannels) / 8;
	parms.dwAudioBufferSize = 

		(mAudioFormat.nSamplesPerSec * mAudioFormat.nBlockAlign)

		/ mFramerateBasis;



	parms.dwAudioBufferSize =

		(parms.dwAudioBufferSize * (mFramesToKeep + mFramesToDrop)) / 

		mFramesToKeep;



	if (parms.dwAudioBufferSize % mAudioFormat.nBlockAlign)
		parms.dwAudioBufferSize += 

			mAudioFormat.nBlockAlign - 

			(parms.dwAudioBufferSize % mAudioFormat.nBlockAlign);

#if 0

	parms.dwAudioBufferSize = ((mAudioFormat.wBitsPerSample *

		mAudioFormat.nSamplesPerSec * mAudioFormat.nChannels) / 8)

		/ mFramerateBasis;


	parms.dwAudioBufferSize =

		(parms.dwAudioBufferSize * (mFramesToKeep + mFramesToDrop)) / 

		mFramesToKeep;

#endif

#if 0
	parms.dwAudioBufferSize = 1024;
	parms.dwAudioBufferSize = ((mAudioFormat.wBitsPerSample *

		mAudioFormat.nSamplesPerSec * mAudioFormat.nChannels) / 8)

		/ 2; //mFramerateBasis;


	parms.dwAudioBufferSize =

		(parms.dwAudioBufferSize * (mFramesToKeep + mFramesToDrop)) / 

		mFramesToKeep;


#endif


	if (!capCaptureSetSetup(mCaptureWindow,&parms,sizeof(parms)))

		throw VidcapError("Could not set capture parameters",0);



    mApparentFramerate = (mFramerateBasis * mFramesToKeep) / 

						(mFramesToKeep + mFramesToDrop);



	mOperational = 1;

	mCaptureThread.SetWindow(mCaptureWindow);

	mCaptureThread.Start();

	mOperationalWait.Signal();

}



void

CVidcapSource::_InternalStop(void)

{

	if (mAudio) mAudio->_End();

	if (mVideo) mVideo->_End();

	mOperationalMutex.Lock();

	mOperational = 0;

	mOperationalMutex.Unlock();

	mOperationalWait.Signal();

}



void

CVidcapSource::CCaptureThread::Run(void)

{

	if (!capCaptureSequenceNoFile(mCaptureWindow))

		::MessageBox(mCaptureWindow,"Could not start capture","Error",MB_OK | MB_APPLMODAL);

}



void

CVidcapSource::CCaptureThread::SetWindow(HWND capture_window)

{

	mCaptureWindow = capture_window;

}



void

CVidcapSource::_ShowStatus(void)

{

	CAPSTATUS	s;

	if (!capGetStatus(mCaptureWindow,&s,sizeof(s)))

		{

		mGUIController->ErrorDialog("CapStatus failed");

		return;

		}

	

	(mGUIController->Error(this)) << ((int)s.dwCurrentVideoFramesDropped)

					<< " frames dropped" << &(CMessageStream::show);

}




void
CVidcapSource::AddToAudioBytesProcessed(int b)
{
	mAudioBytesProcessed += b;
}

void
CVidcapSource::AddToVideoBytesProcessed(int b)
{
	mVideoBytesProcessed += b;
}

void
CVidcapSource::AddToVideoFramesProcessed(int f)
{
	mVideoFramesProcessed += f;
}

//////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////



CVidcapStream::CVidcapStream(CVidcapSource *c,CGUIController *gui)

{

	mGUIController = gui;

	mComponent = c;

	mCurrentFrame = 0;

	mLastFrameWritten = -1;

}





void

CVidcapStream::Initialize(void)

{

	CActiveSource::Initialize();

	mCurrentFrame = 0;

	mLastFrameWritten = -1;

}



int

CVidcapStream::ProcessData(CPassiveSink *s)

{

	mBogusMutex.Lock();

	while (!mTerminate)

		mBogusWait.Wait(mBogusMutex);



	mSink->DataComplete();

	return 0;

}



void

CVidcapStream::Go(void)

{

	CActiveSource::Go();

	mComponent->Start();

	

}



//////////

///////////

////////////



void

CVidcapSourceAudio::_SetInfo(void)

{

	AVISTREAMINFO i;

	mComponent->GetAudioInfo(i);

	mStreamInfoObject.SetStreamInfo(i);

	

	size_t s = mComponent->GetAudioFormatSize();

	char *f = new char[s];

	stack_ArrayDeleter<char> sad(f);



	mComponent->GetAudioFormat(f,s);

 dprintf("calling mStreamInfoObject.SetStreamFormat(3)");
	mStreamInfoObject.SetStreamFormat(f,s);

#ifdef CHUNK_AUDIO

	WAVEFORMATEX *we = (WAVEFORMATEX*)f;

	mAudioGlobSize = we->nAvgBytesPerSec / 4; //0x3000;
	mAudioGlobSize = 0x8000; //0x3000;								///	dbm -- CBR: use a big power of 2

	if (mAudioGlobSize % we->nBlockAlign)
		mAudioGlobSize += we->nBlockAlign - (mAudioGlobSize % we->nBlockAlign);



	delete [] mAudioGlobBuffer;

	mAudioGlobBuffer = new unsigned char[mAudioGlobSize];

	mAudioGlobPointer = mAudioGlobBuffer;

	mAudioGlobAmt = 0;

#endif

}



void

CVidcapSourceAudio::_End(void)

{

	//mStateMutex.Lock();

	//if (mState)

	//	{

	//	mState = -1;

	//	while (mState)

	//		mStateWait.Wait(mStateMutex);

	//	}

	//mStateMutex.Unlock();

	mComponent->SetDataIgnore(1);

	//mTerminate = 1;

	mBogusMutex.Lock();

	mBogusWait.Signal();

	mBogusMutex.Unlock();

	CActiveSource::Terminate();

	WAVEHDR wh;

	wh.lpData = 0;

	wh.dwBytesRecorded = 0;

	ProcessAudio(&wh);

}



void

CVidcapSourceAudio::Terminate(void)

{

	_End();

	mComponent->Stop();

}



void

CVidcapSourceAudio::SubmitData(long datasize,char *data)

{

	CPassiveSinkDataReference ref(data,datasize,0,0);



	if (datasize > 0)

		{

 dprintf("   about to call ConsumeDataNow -- for audio!\n");
		if (!mSink->ConsumeDataNow(&ref))

			{

			mSink->DataComplete();

			return;

			}

		mCurrentFrame += datasize;

#ifndef NDEBUG

		(mGUIController->Trace(this)) << "Reading Audio bytes "

			<< (mCurrentFrame - datasize) << " through " << mCurrentFrame

			<< &(CMessageStream::show);

#endif

		mComponent->AddToAudioBytesProcessed(datasize);

		return;

		}

	else

		{

		mSink->DataComplete();

		return;

		}

}


void	
CVidcapSourceAudio::ProcessAudio(LPWAVEHDR wh)

{
	int taken,compressed;
	stack_MutexLock sml(&mProcessAudioMutex);
 dprintf("CVidcapSourceAudio::ProcessAudio\n");


	char *data = wh->lpData;

	long datasize = wh->dwBytesRecorded;

// datasize >>= 1;							///	experiment -- submit half the audio!
 dprintf("datasize = %d\n",datasize);

#ifdef CHUNK_AUDIO
 dprintf("CHUNKing audio: chunk size = %d\n",mAudioGlobSize);

	if (!datasize) {

		if (mAudioGlobAmt) {
			 dprintf("about to call AcmCompress(1) with %d bytes",mAudioGlobAmt);
			AcmCompress((char*)mAudioGlobBuffer,mAudioGlobAmt,0,ENCODE_MAX_OUT,1,44100,taken,compressed);
///			SubmitData(mAudioGlobAmt,(char *)mAudioGlobBuffer);
			if(compressed > gAcmEncodeChunkCount) {
				SubmitData(compressed,AcmGetBuffer() );
				int overflow = compressed-ENCODE_CHUNK_SIZE;	///	could be negative!  Cool!
				gAcmEncodeChunkCount -= overflow;				///	subtract overflow from chunk count.
				if(gAcmEncodeChunkCount > ENCODE_CHUNK_SIZE) {	///	I don't think this is possible! JIC.
					gAcmEncodeChunkCount = ENCODE_CHUNK_SIZE;	///	reset - all better
				}
				dprintf("OOPS! gAcmEncodeChunkCount=%d, compressed=%d, overflow = %d",
					gAcmEncodeChunkCount,compressed,overflow);
			} else {
				SubmitData(gAcmEncodeChunkCount, AcmGetBuffer() );
				gAcmEncodeChunkCount = ENCODE_CHUNK_SIZE;		///	reset because we're OK now
			}
			mAudioGlobAmt = 0;
			mAudioGlobPointer = mAudioGlobBuffer;
		}
		SubmitData(0,0);
	} else {
		while (datasize) {
			long amt = datasize;
			if (amt + mAudioGlobAmt > mAudioGlobSize)
				amt = mAudioGlobSize - mAudioGlobAmt;
			memcpy(mAudioGlobPointer,data,amt);
			mAudioGlobPointer += amt;
			mAudioGlobAmt += amt;
			datasize -= amt;
			data += amt;
			if (mAudioGlobSize == mAudioGlobAmt)
				{
				 dprintf("OK fellas, blow some CHUNKS! %d bytes\n",mAudioGlobAmt);
				  dprintf("about to call AcmCompress(2) with %d bytes",mAudioGlobAmt);
				AcmCompress((char*)mAudioGlobBuffer,mAudioGlobAmt,0,ENCODE_MAX_OUT,1,44100,taken,compressed);
	///			SubmitData(mAudioGlobAmt,(char *)mAudioGlobBuffer);
				if(compressed > gAcmEncodeChunkCount) {
					SubmitData(compressed,AcmGetBuffer() );
					int overflow = compressed-ENCODE_CHUNK_SIZE;	///	could be negative!  Cool!
					gAcmEncodeChunkCount -= overflow;				///	subtract overflow from chunk count.
					if(gAcmEncodeChunkCount > ENCODE_CHUNK_SIZE) {	///	I don't think this is possible! JIC.
						gAcmEncodeChunkCount = ENCODE_CHUNK_SIZE;	///	reset - all better
					}
					dprintf("OOPS! gAcmEncodeChunkCount=%d, compressed=%d, overflow = %d",
						gAcmEncodeChunkCount,compressed,overflow);
				} else {
					SubmitData(gAcmEncodeChunkCount, AcmGetBuffer() );
					gAcmEncodeChunkCount = ENCODE_CHUNK_SIZE;		///	reset because we're OK now
				}
				mAudioGlobAmt = 0;
				mAudioGlobPointer = mAudioGlobBuffer;
			}
		}
	}
#else
	if (datasize)
		SubmitData(datasize,data);
	else
		SubmitData(0,0);
#endif

}


/////////////
////////////
////////////
CVidcapSourceVideo::CVidcapSourceVideo(CVidcapSource *c,CGUIController *gui)
		: CVidcapStream(c,gui), mBuffer(CAPTUREBUFFERS / 2)
{ 
	mConnectionType = (CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO); 
}

void
CVidcapSourceVideo::_SetInfo(void)
{
	AVISTREAMINFO	i;
	mComponent->GetVideoInfo(i);
	mStreamInfoObject.SetStreamInfo(i);

	size_t s = mComponent->GetVideoFormatSize();
	char *f = new char[s];
	stack_ArrayDeleter<char> sad(f);

	mComponent->GetVideoFormat(f,s);
 dprintf("calling mStreamInfoObject.SetStreamFormat(4)");
	mStreamInfoObject.SetStreamFormat(f,s);
 dprintf("leaving _SetInfo");
}

int		
CVidcapSourceVideo::ProcessData(CPassiveSink *s)
{
	//stack_MutexLock sml(&mStateMutex);
	//assert(mState);
	//if (mState == -1)
	//	{
	//	mState = 0;
	//	mStateWait.Broadcast();
	//	mBuffer.Clear();
	//	return 0;
	//	}

#ifdef USE_SKEW
	mComponent->mSkewWaitMutex.Lock();
	while (mComponent->mSkewWaitFlag)
		mComponent->mSkewWaitWait.Wait(mComponent->mSkewWaitMutex);
	mComponent->mSkewWaitMutex.Unlock();

#endif

	CVSVData	d = mBuffer.Get();
	CPassiveSinkDataReference ref((char *)d.mDataPtr,d.mDataLen,1,0);
	CPassiveSinkDataReference ref2(0,0,0,0);

	if (d.mDataLen == 0)
		{
		mSink->DataComplete();
		//mState = 0;
		mBuffer.Clear();
		return 0;
		}

	while (d.mFrame - 1 > mLastFrameWritten)
		{
		if (!mSink->ConsumeDataNow(&ref2))
			{
			mSink->DataComplete();
			//mState = 0;
			mBuffer.Clear();
			return 0;
			}
		mLastFrameWritten++;
		}
	if (!mSink->ConsumeDataNow(&ref))
		{
		mSink->DataComplete();
		//mState = 0;
		mBuffer.Clear();
		return 0;
		}
	mComponent->AddToVideoBytesProcessed(d.mDataLen);
	mComponent->AddToVideoFramesProcessed(1);
	mLastFrameWritten++;

	return 1;
}

void
CVidcapSourceVideo::Initialize(void)
{
	mBuffer.Clear();
	CVidcapStream::Initialize();
	//mStateMutex.Lock();
	//mState = 1;
	//mStateWait.Signal();
	//mStateMutex.Unlock();
}



void	
CVidcapSourceVideo::ProcessVideo(LPVIDEOHDR vh)
{
	void	*data = vh->lpData;
	long	datasize = vh->dwBytesUsed;

#ifdef USE_SKEW
	if (mComponent->mSkewWaitFlag && mCurrentFrame > (mComponent->mApparentFramerate * 0.75))
		{
		mComponent->mSkewWaitMutex.Lock();
		mComponent->mSkewWaitFlag = 0;
		mComponent->mSkewWaitMutex.Unlock();
		mComponent->mSkewWaitWait.Signal();
		}
#endif
	CVSVData	d;
	d.mDataPtr = (char *)data;
	d.mDataLen = datasize;
	d.mFrame = mCurrentFrame++;

	if (!mBuffer.PutCanFail(d))
		{
		mComponent->FrameDropped();
		mComponent->LogFrames(mCurrentFrame - 1);
		return;
		}
	mComponent->LogFrames(mCurrentFrame - 1);
}

void
CVidcapSourceVideo::_End(void)
{
	mComponent->SetDataIgnore(1);

#ifdef USE_SKEW
	mComponent->mSkewWaitMutex.Lock();
	mComponent->mSkewWaitFlag = 0;
	mComponent->mSkewWaitMutex.Unlock();
	mComponent->mSkewWaitWait.Signal();
#endif

	CVSVData	d;
	d.mDataLen = 0;
	d.mDataPtr = 0;
	d.mFrame = 0;
	mBuffer.Put(d);
	Join();
}


void
CVidcapSourceVideo::Terminate(void)
{
	_End();
	mComponent->Stop();
	assert(Self() != mThreadID);
//	CVidcapStream::Terminate();
}





ostream& operator<<(ostream &o,CVSVData &d)
{ o << d.mDataPtr << " " << d.mDataLen; return o; }

