/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CVIDCAPSOURCE_H_

#define CVIDCAPSOURCE_H_



#include "CComponent.h"

#include "CActiveSource.h"

#include "CError.h"

#include <CCMutex.h>

#include <CConditionVariable.h>

#include <CFifo.h>

#include <vector>
using namespace std;


#define USE_SKEW

#define CHUNK_AUDIO



class CVidcapSourceAudio;

class CVidcapSourceVideo;

class CVidcapStream;



class CVidcapSource: public CComponent

{

private:

		void	_Cleanup(void);



protected:

	class CCaptureThread: public CCThread

	{

	protected:

		HWND	mCaptureWindow;



		virtual void Run(void);

	

	public:

		void	SetWindow(HWND capture_window);

	};



	void	_InternalStop(void);

	int		mIgnoreStatusMessages;
	int		mIgnoreErrorMessages;

	int		mAudioBytesProcessed;
	int		mVideoBytesProcessed;
	int		mVideoFramesProcessed;
	int		mVideoWidth,mVideoHeight;

	int		mAudioConfigured;

	int		mVideoConfigured;

	int		mFramesDropped;

	int		mFramesToRepeat;

	int		mDataIgnore;

	CCMutex	mDataIgnoreMutex;

	int		mStopCounter;

	CCMutex	mStopMutex;



#ifdef USE_SKEW

	CCMutex				mSkewWaitMutex;

	int					mSkewWaitFlag;

	CConditionVariable	mSkewWaitWait;
#endif



	WAVEFORMATEX		mAudioFormat;



	CCaptureThread		mCaptureThread;

	CVidcapSourceAudio	*mAudio;

	CVidcapSourceVideo	*mVideo;

	int		mAudioUsed,mVideoUsed;

	CCMutex				mOperationalMutex;

	CConditionVariable	mOperationalWait;

	int					mOperational;



	HWND	mCaptureWindow;

	int		BasicConfigure(CWnd *wnd);

	void	AttachCapDriver(HWND wnd);



	int		mFramesToKeep;

	int		mFramesToDrop;

	int		mKeepFrameCounter;

	int		mDropFrameCounter;



	float	mFramerateBasis;

	float	mApparentFramerate;



	virtual	void	ErrorMessage(int id,LPCSTR mess);

	virtual	void	StatusMessage(int id,LPCSTR mess);

	void	ProcessAudio(LPWAVEHDR wh);

	void	ProcessVideo(LPVIDEOHDR vh);

	int		ProcessYield(void);

	void	GetAudioInfo(AVISTREAMINFO &i);

	void	GetVideoInfo(AVISTREAMINFO &i);

	size_t	GetAudioFormatSize(void);

	size_t	GetVideoFormatSize(void);

	void	GetAudioFormat(char *f,size_t fs);

	void	GetVideoFormat(char *f,size_t fs);

	virtual	void	Stop(void);

	virtual	void	Start(void);

	void	AdjustVideoSize(void);

	void	FrameDropped(void);

	int		GetFramesToRepeat(void);

	void	DecrementFramesToRepeat(void);

	virtual void	LogFrames(int frames);

	void	_ShowStatus(void);

	void	SetDataIgnore(int i);

	virtual	void	AddToAudioBytesProcessed(int b);
	virtual	void	AddToVideoBytesProcessed(int b);
	virtual	void	AddToVideoFramesProcessed(int f);




public:

	CVidcapSource(CGUIController *gui);

	virtual	~CVidcapSource(void);



	virtual ConnectionTypeList_t GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void	ReturnConnection(CConnection *c,CConnection::Connection_t t);

	virtual	void	Initialize(void);

	virtual	void	VidcapInitialize(void);

	virtual	void	Cleanup(void);

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;

	

	friend CVidcapSourceAudio;

	friend CVidcapSourceVideo;

	friend CVidcapStream;



	CERROR_SUBCLASS(VidcapError);



	friend LRESULT CALLBACK ErrorCallback(HWND wind,int id, LPCSTR mess);

	friend LRESULT CALLBACK StatusCallback(HWND wind,int id,LPCSTR mess);

	friend LRESULT CALLBACK YieldCallback(HWND wind);

	friend LRESULT CALLBACK VideoCallback(HWND wnd,void *vh);

	friend LRESULT CALLBACK WaveCallback(HWND wnd,void *vh);



};



////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////

class CVidcapStream: public CActiveSource

{

protected:

	// aud/vid terminate sync

//	static int				mState;

//	static CCMutex			mStateMutex;

//	static CConditionVariable	mStateWait;



	CVidcapSource	*mComponent;

	CGUIController	*mGUIController;

	int				mCurrentFrame;

	int				mLastFrameWritten;

	CCMutex			mBogusMutex;

	CConditionVariable	mBogusWait;



	CVidcapStream(CVidcapSource *c,CGUIController *gui);

	virtual	int		ProcessData(CPassiveSink *s);



public:

//	virtual	void	Terminate(void);

	virtual	void	Go(void);

	virtual	void	Initialize(void);



	friend CVidcapSource;



};





class CVidcapSourceAudio: public CVidcapStream

{

protected:

#ifdef CHUNK_AUDIO

	int		mAudioGlobSize;

	unsigned char *mAudioGlobBuffer;

	unsigned char *mAudioGlobPointer;

	int		mAudioGlobAmt;

#endif

	CCMutex	mProcessAudioMutex;



	CVidcapSourceAudio(CVidcapSource *c,CGUIController *gui)

		: CVidcapStream(c,gui)

	{ 

		mConnectionType = (CONN_ACTIVE | CONN_SOURCE | CONN_AUDIO);

#ifdef CHUNK_AUDIO

		mAudioGlobBuffer = 0;

#endif

	}

	virtual ~CVidcapSourceAudio(void)

	{ 

#ifdef CHUNK_AUDIO

		delete [] mAudioGlobBuffer;

#endif

	}



	virtual	void	_SetInfo(void);

			void	ProcessAudio(LPWAVEHDR wh);

			void SubmitData(long len,char *d);

			void	_End(void);

public:

	virtual	void	Terminate(void);

	friend CVidcapSource;



};





class CVSVData

{

public:

	char *mDataPtr;

	int mDataLen;

	int mFrame;

};

ostream& operator<<(ostream &o,CVSVData &d);





class CVidcapSourceVideo: public CVidcapStream

{



	CVidcapSourceVideo(CVidcapSource *c,CGUIController *gui);

	virtual	void	_SetInfo(void);

			void	ProcessVideo(LPVIDEOHDR vh);

	

	CFifo<CVSVData>	mBuffer;

	virtual	int		ProcessData(CPassiveSink *s);

			void	_End(void);



public:

	virtual	void	Initialize(void);

	virtual	void	Terminate(void);



	friend CVidcapSource;

};



////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////



#endif

