/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CVidcapSource_NC.h"

#include "CVidcapAudioDialog.h"

#include "CFramerateDialog.h"



CComponent *

CVidcapSource_NC::CreatorFunc(CGUIController *gui)

{

	return new CVidcapSource_NC(gui);

}



void

CVidcapSource_NC::Configure(CWnd *wnd)

{

	if (mVideoUsed || (!mVideoUsed && !mAudioUsed))

		{

		if (!_ConfigureVideo(wnd)) return;

		mVideoConfigured = 1;

		}

	if (mAudioUsed)

		{

		if (!_ConfigureAudio(wnd)) return;

		mAudioConfigured = 1;

		}

}



int

CVidcapSource_NC::ConfigureIfNeeded(CWnd *wnd)

{

	if (mVideoUsed)

		{

		if (!mVideoConfigured)

			if (!_ConfigureVideo(wnd)) return 0;

		mVideoConfigured = 1;

		}

	if (mAudioUsed)

		{

		if (!mAudioConfigured)

			if (!_ConfigureAudio(wnd)) return 0;

		mAudioConfigured = 1;

		}

	return 1;

}



int

CVidcapSource_NC::_ConfigureVideo(CWnd *wnd)

{

	assert(mCaptureWindow);



	if (!capDlgVideoDisplay(mCaptureWindow)) return 0;

	if (!capDlgVideoFormat(mCaptureWindow)) return 0;

	AdjustVideoSize();

	if (!capDlgVideoSource(mCaptureWindow)) return 0;



	CFramerateDialog dlog;

	dlog.mFramerateBasis= mFramerateBasis;

	dlog.mFramesToKeep	= mFramesToKeep;

	dlog.mFramesToDrop	= mFramesToDrop;

	if (dlog.DoModal() != IDOK) return 0;

	mFramerateBasis	= dlog.mFramerateBasis;

	mFramesToKeep	= dlog.mFramesToKeep;

	mFramesToDrop	= dlog.mFramesToDrop;



	return 1;

}



int

CVidcapSource_NC::_ConfigureAudio(CWnd *wnd)

{

	assert(mCaptureWindow);



	CVidcapAudioDialog dlog;

	dlog.mBits		= mAudioFormat.wBitsPerSample;

	dlog.mFrequency = mAudioFormat.nSamplesPerSec;

	dlog.mChannels	= mAudioFormat.nChannels;

	if (dlog.DoModal() != IDOK) return 0;



	mAudioFormat.wFormatTag		= 1;

	mAudioFormat.nChannels		= dlog.mChannels;

	mAudioFormat.nSamplesPerSec = dlog.mFrequency;

	mAudioFormat.nAvgBytesPerSec= 

		(dlog.mFrequency * dlog.mChannels * dlog.mBits) / 8;

	mAudioFormat.wBitsPerSample = dlog.mBits;

	mAudioFormat.nBlockAlign	= (dlog.mChannels * dlog.mBits) / 8;

	mAudioFormat.cbSize			= 0;



	if (!capSetAudioFormat(mCaptureWindow,&mAudioFormat,sizeof(mAudioFormat)))

		throw VidcapError("Could not set audio format",0);



	return 1;

}



