/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "NodeStuff.h"



void	UnmarkMegaNodeList(MegaNodeList_t &l)

{

	MegaNodeList_t::iterator i;

	for (i = l.begin(); i != l.end(); ++i)

		(*i).mMarked = 0;

}



void	UnmarkNodeList(NodeList_t &l)

{

	NodeList_t::iterator i;

	for (i = l.begin(); i != l.end(); ++i)

		(*i).mMarked = 0;

}





int		AllNodesVisited(const NodeList_t &l)

{

	NodeList_t::const_iterator i;

	for (i = l.begin(); i != l.end(); ++i)

		if (!((*i).mMarked)) 

			return 0;

	return 1;

}



int		AllMegaNodesVisited(const MegaNodeList_t &l)

{

	MegaNodeList_t::const_iterator i;

	for (i = l.begin(); i != l.end(); ++i)

		if (!((*i).mMarked)) 

			return 0;

	return 1;

}





static int		FindEndpointMegaNode(const MegaNodeList_t &l,const NodeList_t &nl)

{

	int	end_point = -1;



	for (int x = 0; x < l.size(); ++x)

		{

		end_point = x;

		for (int y = 0; y < l[x].mNodeIndexList.size(); ++y)

			{

			if (nl[l[x].mNodeIndexList[y]].mMySink != -1)

				{

				end_point = -1;

				break;

				}

			}

		if (end_point != -1) return end_point;

		}



	throw NodeError("Could not find graph endpoint",0);

	return end_point;

}



static int	_Traverse(MegaNodeList_t &ml,NodeList_t &nl,

					   int mega_node_index,NodeAction &action)

{

	int result = 1;

	//MegaNode_t	mn = ml[mega_node_index];



	if (ml[mega_node_index].mMarked) return 1;

	ml[mega_node_index].mMarked = 1;

	int x;



	for (x = 0; x < ml[mega_node_index].mNodeIndexList.size(); ++x)

		{

		if (!nl[ml[mega_node_index].mNodeIndexList[x]].mMarked)

			{

			int source = nl[ml[mega_node_index].mNodeIndexList[x]].mMySource;

			if (source != -1)

				{

				nl[ml[mega_node_index].mNodeIndexList[x]].mMarked = 1;

				result = _Traverse(ml,nl,nl[source].mMegaNodeIndex,action);

				if (result)

					result = action.Do(nl[ml[mega_node_index].mNodeIndexList[x]].mConnection);

				}

			}

		}



	for (x = 0; x < ml[mega_node_index].mNodeIndexList.size(); ++x)

		{

		int node_index = ml[mega_node_index].mNodeIndexList[x];

		if (!nl[node_index].mMarked)

			{

			nl[node_index].mMarked = 1;

			if (result)

				result = action.Do(nl[node_index].mConnection);

			}

		}



	return result;

}



int	Traverse(MegaNodeList_t &ml,NodeList_t &nl,NodeAction &action)

{

	UnmarkMegaNodeList(ml);

	UnmarkNodeList(nl);



	return _Traverse(ml,nl,FindEndpointMegaNode(ml,nl),action);

}







static void	_TraverseAndMarkGraph(MegaNodeList_t &ml,NodeList_t &nl,int mega_node_index)

{

	//MegaNode_t	mn = ml[mega_node_index];



	if (ml[mega_node_index].mMarked) return;

	ml[mega_node_index].mMarked = 1;



	for (int x = 0; x < ml[mega_node_index].mNodeIndexList.size(); ++x)

		{

		if (!nl[ml[mega_node_index].mNodeIndexList[x]].mMarked)

			{

			nl[ml[mega_node_index].mNodeIndexList[x]].mMarked = 1;

			int source = nl[ml[mega_node_index].mNodeIndexList[x]].mMySource;

			if (source != -1)

				_TraverseAndMarkGraph(ml,nl,nl[source].mMegaNodeIndex);

			}

		}

}



void	TraverseAndMarkGraph(MegaNodeList_t &ml,NodeList_t &nl)

{

	int mn = FindEndpointMegaNode(ml,nl);



	_TraverseAndMarkGraph(ml,nl,mn);

}

