/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef NODESTUFF_H_

#define NODESTUFF_H_



#include <MegaHeaders.h>

#include <CError.h>

#include <vector>
using namespace std;

#include "CConnection.h"



class CComponent;



class Node_t {

public:

	int	mMarked;

	int mMegaNodeIndex;

	int mMySink;

	int mMySource;

	CConnection	*mConnection;

};



class MegaNode_t {

public:

	int mMarked;

	CComponent *mComponent;

	vector<int>	mNodeIndexList;

};



typedef vector<Node_t> NodeList_t;

typedef vector<MegaNode_t> MegaNodeList_t;



class NodeAction

{

public:

	virtual int Do(CConnection *c) = 0;

};





class InitializeAction: public NodeAction

{

public:

	virtual int Do(CConnection *c)

		{

		c->Initialize();

		return 1;

		}

};



class GoAction: public NodeAction

{

public:

	virtual int Do(CConnection *c)

		{

		c->Go();

		return 1;

		}

};



class TerminateAction: public NodeAction

{

public:

	virtual int Do(CConnection *c)

		{

		c->Terminate();

		return 1;

		}

};



class CleanupAction: public NodeAction

{

public:

	virtual int Do(CConnection *c)

		{

		c->Cleanup();

		return 1;

		}

};





void	UnmarkMegaNodeList(MegaNodeList_t &l);

void	UnmarkNodeList(NodeList_t &l);

int		AllNodesVisited(const NodeList_t &l);

int		AllMegaNodesVisited(const MegaNodeList_t &l);

int		Traverse(MegaNodeList_t &ml,NodeList_t &nl,NodeAction &action);

void	TraverseAndMarkGraph(MegaNodeList_t &ml,NodeList_t &nl);





CERROR_SUBCLASS(NodeError);





#endif

