/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CAudioMixer.h"





CAudioMixer::CAudioMixer(int initial_line_in_volume,UINT mixer_id)

{

    MMRESULT            err;



	mMixerOK = 0;

	mMixerControl = 0;

	mMixer = 0;

	mMixerID = mixer_id;

	mLineInVolume = initial_line_in_volume;



    err = mixerGetDevCaps(mixer_id,&mMixerCaps, sizeof(mMixerCaps));

	if (err != MMSYSERR_NOERROR)

		{

		::MessageBox(NULL,"Could not find an audio mixer.\n Audio control will not be available.",

			"Audio Error",MB_APPLMODAL | MB_OK);

		return;

		}

		//throw CAudioMixerError("mixerGetDevCaps() failed",mixer_id);



    err = mixerOpen(&mMixer,mMixerID,0L,0L,CALLBACK_WINDOW);

    if (err != MMSYSERR_NOERROR)

		{

		::MessageBox(NULL,"Could not open an audio mixer.\n Audio control will not be available.",

			"Audio Error",MB_APPLMODAL | MB_OK);

		return;

		}

		//throw CAudioMixerError("mixerOpen() failed",mixer_id);



	FindLineIn();

}



CAudioMixer::~CAudioMixer(void)

{

	delete [] mMixerControl;



	if (mMixer)

		mixerClose(mMixer);

}



void

CAudioMixer::FindLineIn(void)

{

	UINT u,v,w,z,connections,controls;

	MIXERLINE			mixer_line; 

	MIXERLINECONTROLS	mixer_line_controls;

	MMRESULT	err;



	try {

	mMixerOK = 1;

    for (u = 0; u < mMixerCaps.cDestinations; u++)

		{

        mixer_line.cbStruct      = sizeof(mixer_line);

        mixer_line.dwDestination = u;



        err = mixerGetLineInfo((HMIXEROBJ)mMixer,&mixer_line,MIXER_GETLINEINFOF_DESTINATION);

        if (err != MMSYSERR_NOERROR)

			throw CAudioMixerError("mixerGetLineInfo() failed",err);



		if (mixer_line.dwComponentType == MIXERLINE_COMPONENTTYPE_DST_WAVEIN)

			{

			// this is the destination we're interested in. Now find the

			// line in source

			connections = mixer_line.cConnections;

			for (v = 0; v < connections; v++)

				{

				mixer_line.cbStruct      = sizeof(mixer_line);

				mixer_line.dwDestination = u;

				mixer_line.dwSource      = v;

				err = mixerGetLineInfo((HMIXEROBJ)mMixer,&mixer_line,MIXER_GETLINEINFOF_SOURCE);

				if (err != MMSYSERR_NOERROR)

					throw CAudioMixerError("mixerGetLineInfo() failed",err);

				if (strcmp(mixer_line.szName,"Line-In") == 0 || mixer_line.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_LINE)

					{

					// got it!!! Now found the appropriate control

					controls = mixer_line.cControls;

					//for (w = 0; w < controls;++w)

					//	{

					mMixerControl = new MIXERCONTROL[controls];

					mixer_line_controls.cbStruct = sizeof(mixer_line_controls);

					mixer_line_controls.dwLineID = mixer_line.dwLineID;

					mixer_line_controls.cbmxctrl  = sizeof(MIXERCONTROL);

					mixer_line_controls.pamxctrl  = mMixerControl;

					mixer_line_controls.cControls = controls;



					err = mixerGetLineControls((HMIXEROBJ)mMixer,&mixer_line_controls,MIXER_GETLINECONTROLSF_ALL);

					if (err != MMSYSERR_NOERROR)

						throw CAudioMixerError("mixerGetLineControls() failed",err);

					for (z = 0; z < mixer_line.cControls; ++z)

						{

						if (mMixerControl[z].dwControlType & MIXERCONTROL_CT_CLASS_FADER)

							{

							mMixerControlID = mMixerControl[z].dwControlID;

							mMixerControlMax = mMixerControl[z].Bounds.lMaximum;

							mMixerControlMin = mMixerControl[z].Bounds.lMinimum;

							return;

							}

						}

						

					//	}



					}

				}

			}

        }

	}

	catch(...)

	{

	}

	mMixerOK = 0;

	::MessageBox(NULL,"Could not find appropriate audio line and control.\n Audio control will not be available.",

		"Audio Error",MB_APPLMODAL | MB_OK);

	//throw CAudioMixerError("Could not find appropriate audio line and control",0);

}



void

CAudioMixer::SetLineInVolume(int v)

{

	MIXERCONTROLDETAILS	mixer_control_details;

	MIXERCONTROLDETAILS_UNSIGNED	u;

	MMRESULT	err;



	if (!mMixerOK) return;

	mixer_control_details.cbStruct		= sizeof(mixer_control_details);

	mixer_control_details.dwControlID	= mMixerControlID;

	mixer_control_details.cChannels		= 1;

	mixer_control_details.cMultipleItems= 0;

    mixer_control_details.cbDetails     = sizeof(u);

    mixer_control_details.paDetails     = &u;



    err = mixerGetControlDetails((HMIXEROBJ)mMixer,&mixer_control_details,0L);

    if (err != MMSYSERR_NOERROR)

		throw CAudioMixerError("mixerGetControlDetails() failed",err);



	u.dwValue = (((mMixerControlMax - mMixerControlMin) * v) / 100) + mMixerControlMin;

	

    err = mixerSetControlDetails((HMIXEROBJ)mMixer,&mixer_control_details,0L);

    if (err != MMSYSERR_NOERROR)

		throw CAudioMixerError("mixerSetControlDetails() failed",err);

}

