/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "stdafx.h"

#include "CCap.h"

#include "CStatusBarStream.h"

#include "CGraphManager.h"

#include "CVidcapSource_TC.h"

#include "CCompressor_TC.h"

#include "CIvaPWriter_TC.h"

#include "CRTCompressor_TC.h"



#include <vfw.h>

#include <assert.h>

#include "danbug.h"


extern int	gUseCodec;



#define MAXVIDDRIVERS 10



HWND

CCap::GetCaptureWindow(void)

{

	return mVidcapSource->GetCaptureWindow();

}



CCap::CCap(void)

{

	mVideo_SingleFieldInput = 0;

	mGraphManager	= 0;

	mVidcapSource	= 0;

	mCompressor		= 0;

	mRTCompressor	= 0;

	mWriter			= 0;

	if (gUseCodec)

		{
		dprintf("of course, we're using a codec");
		TRACE("Using Codec\n");

		}

	else

		{
		dprintf("in this case, no codec");
		TRACE("Not using codec\n");

		}

}



CCap::~CCap(void)

{

	delete mGraphManager;

}



BOOL	

CCap::GrabFrameNoStop(void)

{

	if (!mVidcapSource)

		return FALSE;

	return mVidcapSource->GrabFrameNoStop();

}



BOOL	

CCap::GrabFrame(void)

{

	if (!mVidcapSource)

		return FALSE;

	return mVidcapSource->GrabFrame();

}



BOOL	

CCap::FileSaveDib(const char *name)

{

	if (!mVidcapSource)

		return FALSE;

	return mVidcapSource->FileSaveDib(name);

}





void

CCap::SetFreeSpaceDrive(const char *d)

{

	if (mVidcapSource)

		mVidcapSource->SetFreeSpaceDrive(d);

}



int

CCap::BigImage(void)

{

	return (mVideo_Width > 160);

}



void

CCap::SetVideoParams(int width,int height,int fps,int quality,int single_field_input)

{

	assert(width == 320 || width == 160 || width==640);

	assert(height == 240 || height == 120 || height == 480);

	assert(fps == 30 || fps == 15 || fps == 10);

	assert(quality >= 0 && quality <= 4);



	mVideo_Width	= width;

	mVideo_Height	= height;

	mVideo_FPS		= fps;

	mVideo_Quality	= quality;

	mVideo_SingleFieldInput = single_field_input;

	

	if (mVidcapSource)

		mVidcapSource->SetVideoParams(width,height,fps,single_field_input);

}



void	

CCap::SetAudioParams(int sample_size,int channels,int frequency)

{
 dprintf("here's where audio params get set -- ");

	assert(sample_size == 8 || sample_size == 16);

	assert(channels == 1 || channels == 2);

	assert(frequency == 11025 || frequency == 22050 || frequency == 44100);

	mAudio_SampleSize = sample_size;

	mAudio_Channels = channels;

	mAudio_Frequency = frequency;

}



void

CCap::ConfigureCaptureDriverSource(void)

{

	if (mVidcapSource)

		mVidcapSource->ConfigureCaptureDriverSource();

}



void

CCap::ShowVideo(BOOL s)

{

	if (mVidcapSource)

		mVidcapSource->ShowVideo(s);

}



void

CCap::SetCaptureFile(const char *filename)

{

	mCaptureFile = filename;

}



void

CCap::Instantiate(HWND parent,CGUIController *g)

{
 dprintf("CCap::Instantiate");
	assert(!mGraphManager);



	mGraphManager = new CGraphManager(g,mTrueCapComponentFactory);

	mGraphManager->AddComponent("CVidcapSource_TC");

	if (gUseCodec)

		mGraphManager->AddComponent("CCompressor_TC");

	else

		mGraphManager->AddComponent("CRTCompressor_TC");

	mGraphManager->AddComponent("CIvaPWriter_TC");



	mVidcapSource	= dynamic_cast<CVidcapSource_TC *>(const_cast<CComponent*>(mGraphManager->GetComponent(0)));

	mVidcapSource->VidcapInitialize();

	if (gUseCodec)

		mCompressor		= dynamic_cast<CCompressor_TC *>(const_cast<CComponent*>(mGraphManager->GetComponent(1)));

	else

		mRTCompressor	= dynamic_cast<CRTCompressor_TC *>(const_cast<CComponent*>(mGraphManager->GetComponent(1)));

	mWriter			= dynamic_cast<CIvaPWriter_TC *>(const_cast<CComponent*>(mGraphManager->GetComponent(2)));



	assert(mVidcapSource);

	if (gUseCodec)

		{

		assert(mCompressor);

		}

	else

		{

		assert(mRTCompressor);

		}

	assert(mWriter);



	mVidcapSource->SetVideoParams(mVideo_Width,mVideo_Height,mVideo_FPS,mVideo_SingleFieldInput);



	if (!mGraphManager->AddConnection(0,1,

			CONN_SOURCE | CONN_ACTIVE | CONN_VIDEO,

			CONN_SINK | CONN_PASSIVE | CONN_VIDEO)

			) throw CCapException("Connection Failed",0);

	if (!mGraphManager->AddConnection(1,2,

			CONN_SOURCE | CONN_ACTIVE | CONN_VIDEO,

			CONN_SINK | CONN_PASSIVE | CONN_VIDEO)

			) throw CCapException("Connection Failed",2);

	if (!mGraphManager->AddConnection(0,2,

			CONN_SOURCE | CONN_ACTIVE | CONN_AUDIO,

			CONN_SINK | CONN_PASSIVE | CONN_AUDIO)

			) throw CCapException("Connection Failed",1);

}



void

CCap::DestroyWindow(void)

{

	delete mGraphManager;

	mGraphManager = 0;

	mVidcapSource = 0;

	mCompressor = 0;

	mRTCompressor = 0;

	mWriter = 0;

}



void

CCap::TempConfigure(void)

{

	if (mGraphManager)

		mGraphManager->ConfigureComponents();

}



void

CCap::Run(void)

{

	if (mGraphManager)

		{

		assert(mWriter);

		assert(mVidcapSource);

		if (gUseCodec)

			{

			dprintf("CCap::Run -- using mCompressor");
			assert(mCompressor);

			}

		else

			{
			dprintf("CCap::Run -- using mRTCompressor");
			assert(mRTCompressor);

			}



		remove(mCaptureFile);

		// configure components

		mWriter->SetFile(mCaptureFile);

		mVidcapSource->SetAudioParams(mAudio_SampleSize,mAudio_Channels,mAudio_Frequency);

		mVidcapSource->SetVideoParams(mVideo_Width,mVideo_Height,mVideo_FPS,mVideo_SingleFieldInput);

		mVidcapSource->SetQuality(mVideo_Quality);

		if (gUseCodec)

			mCompressor->SetQuality(mVideo_Quality);

		else

			mRTCompressor->SetQuality(mVideo_Quality);

		mGraphManager->Run();

		}

}



int

CCap::IsRunning(void)

{

	if (!mGraphManager) return 0;

	return mGraphManager->IsRunning();

}



void

CCap::Halt(void)

{

	if (mGraphManager)

		mGraphManager->Halt();

}



