/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CPlayerShell.h"

#include "TrueCapDoc.h"

#include "TrueCapView.h"

#include "CDuckFile.h"

#include "CDuckBuffer.h"

#include "CDuckWinblitScreen.h"

#include "CDuckImage.h"

#include "CDuckVideoStream.h"

#include "CDuckContext.h"

#include <stack_aids.h>

#include "danbug.h"


// 8 meg allows 2 gig playback with large frames. HFB shares the movie

// buffer with the index buffer



CPlayerShell::CPlayerShell(void)

	: mThreadData("",0,0,0,1,8 * 1024,0,0,0,640,480,0,this,0,0)	

{

	SetIsPlaying(0);

	mMPVEAvailable = 1;

}



CPlayerShell::~CPlayerShell(void)

{

	Stop();

}





void

CPlayerShell::Play(const char *filename,HWND mwnd,CGUIController *gui,

				   CColorSlider *slider,C3BitmapButton *playbutton)

{

	stack_MutexLock sml(&mMutex);

	if (mIsPlaying)

		{

		if (mPlayer.IsRewind())

			{

			mPlayer.Rewind(0);

			return;

			}

		else if (mPlayer.IsFastForward())

			{

			mPlayer.FastForward(0);

			return;

			}

		else if (mPlayer.IsPaused())

			{

			mPlayer.Pause(0);

			return;

			}

		else

			{

			return;

#if 0

			mPlayer.StopPlaying();

			mRunner.Join();

			mIsPlaying = 0;

#endif

			}

		}

	mIsPlaying = 1;

	mThreadData.mFilename = filename;

	mThreadData.mWindow = mwnd;

	mThreadData.mPlayer = &mPlayer;

	mThreadData.mWinblit = !gDirectDrawAvailable;

	mThreadData.mDirectSound = gDirectDrawAvailable;



	// direct sound seems to be fucked under NT

///	dbm -- commenting out these 2 lines under Neil's instructions.  Apparently
///	we NEED directsound now.

///	if (gNT)
///		mThreadData.mDirectSound = 0;

	///


	mThreadData.mGUIController = gui;

	mThreadData.mSlider = slider;

	mThreadData.mPlayButton = playbutton;

	mRunner.SetThreadData(&mThreadData);

	mRunner.Start();

}



void

CPlayerShell::GoFullScreen(int f)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.GoFullScreen(f);

}



void	

CPlayerShell::Stop(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.StopPlaying();

	while (mIsPlaying) mWait.Wait(mMutex);

	mRunner.Join();

}



void

CPlayerShell::AdvanceFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.AdvanceFrame();

}



void

CPlayerShell::RetreatFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.RetreatFrame();

}



void

CPlayerShell::AdvanceTime(int milliseconds)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.AdvanceTime(milliseconds);

}



void

CPlayerShell::RetreatTime(int milliseconds)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.RetreatTime(milliseconds);

}



void

CPlayerShell::DisplayCurrentFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.DisplayCurrentFrame();

}



void

CPlayerShell::SetAndDisplayFrame(int f)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.SetAndDisplayFrame(f);

}



int

CPlayerShell::GetCurrentFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return 0;

	return mPlayer.GetLastFrameDisplayed();

}



void

CPlayerShell::SkipCommercial(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	if (mPlayer.IsFastForward())

		{

		mPlayer.Pause(TRUE);

		int f = mPlayer.GetLastFrameDisplayed();

		mPlayer.RetreatTime(6000);

		mPlayer.Pause(FALSE);

		}

	else

		{

		mPlayer.FastForward(TRUE);

		}

}



void	

CPlayerShell::Rewind(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.Rewind(TRUE);

}



void	

CPlayerShell::FastForward(const char *filename,HWND wnd,CGUIController *gui,

						  CColorSlider *slider,C3BitmapButton *playbutton)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) 

		{

		mMutex.Unlock();

		mPlayer.StartInFFMode();

		try {Play(filename,wnd,gui,slider,playbutton);} catch(...) {}

		mMutex.Lock();

		}

	else

		{

		mPlayer.FastForward(TRUE);

		}

}





void	

CPlayerShell::StartPaused(const char *filename,HWND wnd,CGUIController *gui,

						  CColorSlider *slider,C3BitmapButton *playbutton)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) 

		{

		mMutex.Unlock();

		mPlayer.StartInPauseMode();

		try {Play(filename,wnd,gui,slider,playbutton);} catch(...) {}

		mMutex.Lock();

		}

	else

		{

		mPlayer.Pause(TRUE);

		}

}



void	

CPlayerShell::Pause(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return;

	mPlayer.Pause(!mPlayer.IsPaused());

}



bool	

CPlayerShell::IsRewind(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) 0;

	return mPlayer.IsRewind();

}



bool	

CPlayerShell::IsFastForward(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) 0;

	return mPlayer.IsFastForward();

}



bool	

CPlayerShell::IsPause(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) 0;

	return mPlayer.IsPaused();

}



bool	

CPlayerShell::IsPlaying(void)

{

	stack_MutexLock sml(&mMutex);

	return mIsPlaying;



	//if (!mIsPlaying) 0;

//	return mPlayer.IsPlaying();

}





void

CPlayerShell::CPlayerRunner::SetThreadData(CThreadData *data)

{

	mData = data;

}



int

CPlayerShell::GrabMPVE(void)

{

	mMPVEMutex.Lock();

	if (!mMPVEAvailable)  { mMPVEMutex.Unlock(); return 0;}

	return 1;

}



void

CPlayerShell::ReleaseMPVE(void)

{

	mMPVEAvailable = 1;

	mMPVEMutex.Unlock();

}



void

CPlayerShell::SetIsPlaying(int p)

{

	stack_MutexLock sml(&mMutex);

	mIsPlaying = p;

	mWait.Signal();

}



int

CPlayerShell::BigImage(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return 1; 

	else return mPlayer.BigImage();

}



void

CPlayerShell::CPlayerRunner::Run(void)

{

	int x,y,width = 320,height = 240;

	unsigned char *buffer = 0;

	long	buffersize = 320 * 240 * 3;

	if (mData->mPlayer->IsPlaying()) return;

	if (!mData->mShell->GrabMPVE()) return;

	try {

			{

			//CDuckContext context(1,2,1); uses one in player

			buffer = new unsigned char[320 * 240 * 3];

			CDuckBuffer	duck_buffer(8 * 1024 * 1024,0);

			CDuckFile	duck_file;
 dprintf("A");
			duck_file.OpenFile(mData->mFilename,duck_buffer);
 
 dprintf("B");

			CDuckVideoStream	duck_video_stream(duck_file,NULL,1);
 dprintf("C");

			duck_video_stream.ReadData(buffer,buffersize,DUK_ABSOLUTE,0);
 dprintf("D");

			CDuckImage	duck_image(buffer);
 dprintf("E");

			duck_image.GetDimensions(x,y,width,height);
 dprintf("F");

			delete [] buffer;
 dprintf("G");

			buffer = 0;
 dprintf("H");

			}

//		mData->mPlayer->Go(mData->mFilename,mData->mWindow,

//				width == 320 ? VIDEO_X_LARGE  : VIDEO_X_SMALL ,width == 320 ? VIDEO_Y_LARGE : VIDEO_Y_SMALL,

//		mData->mStretch,mData->mWinblit,mData->mBuffersize,0,0,0,mData->mLoopforever,

//		0,0,0,mData->mDirectSound,mData->mBlocksize,mData->mHRes,mData->mVRes,0,16);

		mData->mPlayer->Go(mData->mFilename,mData->mWindow,

				((320 - width)/2) + VIDEO_X_OFFSET,((240 - height)/2) + VIDEO_Y_OFFSET,

		mData->mStretch,mData->mWinblit,mData->mBuffersize,0,0,0,mData->mLoopforever,

		0,0,0,mData->mDirectSound,mData->mBlocksize,mData->mHRes,mData->mVRes,0,16,

		mData->mGUIController,mData->mSlider,mData->mPlayButton);

	}

	catch(CDuckStream::InitException &err)

	{

		::MessageBox(mData->mWindow,"File does not have video.","Playback Error",MB_OK | MB_ICONERROR);

	}

	catch(CError &err)

	{

		::MessageBox(mData->mWindow,err.data(),"Playback Error",MB_OK | MB_ICONERROR);

	}

	catch(CDuckFile::OpenException &err)

	{

		//::MessageBox(mData->mWindow,"Bad file","Playback Error",MB_OK | MB_ICONERROR);

	}

	catch(...)

	{
 dprintf("here's your error, boy");
		::MessageBox(mData->mWindow,"Unknown error 2","Playback Error",MB_OK | MB_ICONERROR);

	}

	delete [] buffer;

	mData->mShell->ReleaseMPVE();

	mData->mShell->SetIsPlaying(0);

	RECT r;

	r.left = VIDEO_X_OFFSET;

	r.top = VIDEO_Y_OFFSET;

	r.right = VIDEO_X_OFFSET + 320;

	r.bottom = VIDEO_Y_OFFSET + 240;

	::InvalidateRect(mData->mWindow,&r,FALSE);

}



int

CPlayerShell::IsPageFlipping(void)

{

	stack_MutexLock sml(&mMutex);

	if (!mIsPlaying) return 0;

	return mPlayer.IsPageFlipping();

}





