/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CPLAYERSHELL_H_

#define CPLAYERSHELL_H_



#include "CDuckPlayer.h"

#include "nmr_string.h"

#include "CCMutex.h"

#include "CColorSlider.h"

#include "C3BitmapButton.h"



class CGUIController;



class CPlayerShell

{

	class CThreadData

		{



		public:

			nmr_string	mFilename;

			HWND		mWindow;

			CDuckPlayer	*mPlayer;

			int			mStretch;

			int			mWinblit;

			int			mBuffersize;

			int			mLoopforever;

			int			mDirectSound;

			int			mBlocksize;

			int			mHRes;

			int			mVRes;

			CGUIController	*mGUIController;

			CPlayerShell	*mShell;

			CColorSlider		*mSlider;

			C3BitmapButton		*mPlayButton;



			CThreadData(const char *filename,HWND wnd,CDuckPlayer *player,

				int stretch,int winblit,int buffersize,int loopforever,

				int directsound,int blocksize,int hres,int vres,CGUIController *gui,

				CPlayerShell *shell,CColorSlider *slider,C3BitmapButton *playbutton) :

					mFilename(filename),mWindow(wnd),mPlayer(player),mStretch(stretch),

					mWinblit(winblit),mBuffersize(buffersize),mLoopforever(loopforever),

					mDirectSound(directsound),mBlocksize(blocksize),mHRes(hres),mVRes(vres),

					mGUIController(gui),mShell(shell),mSlider(slider),

					mPlayButton(playbutton){}

		};



	class CPlayerRunner: public CCThread

		{

			CThreadData *mData;

		public:

			void	SetThreadData(CThreadData *data);

			virtual void Run(void);

		};



	CDuckPlayer	mPlayer;

	CPlayerRunner	mRunner;

	int				mIsPlaying;

	CThreadData		mThreadData;

	CCMutex			mMPVEMutex;

	int				mMPVEAvailable;



public:

	CCMutex	mMutex;

	CConditionVariable mWait;



	CPlayerShell(void);

	~CPlayerShell(void);

	void	Play(const char *filename,HWND wnd,CGUIController *gui,CColorSlider *slider,

		C3BitmapButton *playbutton);

	//void	Play(void);

	void	SkipCommercial(void);

	void	Rewind(void);

	void	FastForward(const char *filename,HWND wnd,CGUIController *gui,CColorSlider *slider,C3BitmapButton *playbutton);

	void	StartPaused(const char *filename,HWND wnd,CGUIController *gui,CColorSlider *slider,C3BitmapButton *playbutton);

	void	Pause(void);

	void	Stop(void);

	bool	IsRewind(void);

	bool	IsFastForward(void);

	bool	IsPause(void);

	bool	IsPlaying(void);

	int		GrabMPVE(void);

	void	ReleaseMPVE(void);

	void	SetIsPlaying(int p);

	void	DisplayCurrentFrame(void);

	void	SetAndDisplayFrame(int f);

	int		GetCurrentFrame(void);

	int		BigImage(void);

	void	GoFullScreen(int f);

	int		IsPageFlipping(void);

	void	AdvanceFrame(void);

	void	RetreatFrame(void);

	void	AdvanceTime(int milliseconds);

	void	RetreatTime(int milliseconds);

};



#endif

