/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CRTCompressor.h"

#include <stack_aids.h>

#include <tmrtxx.h>

#include <tmrt.h>

#include "danbug.h"

#include "fourcc.h"

CRTCompressor::CRTCompressor(CGUIController *gui)

: CComponent(gui)

{

	mCompressorStream = new CRTCompressorStream(this,gui);

	mDataMustBeFreed = 0;

	mXXParam = 0;

}



CRTCompressor::~CRTCompressor(void)

{

	delete mCompressorStream;

	if (mDataMustBeFreed)

		{

		mDataMustBeFreed = 0;
 dprintf("CRTCompressor.cpp: CRTCompressor::~CRTCompressor calls tmrtxx_free");
		tmrtxx_free(mXXParam);

		}

	if (mXXParam)

		{

		LocalFree(mXXParam);

		mXXParam = 0;

		}

}



CComponent::ConnectionTypeList_t	

CRTCompressor::GetSupportedTypes(void) const

{

	CComponent::ConnectionTypeList_t l;

	l.push_back(CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

	l.push_back(CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

	return l;

}



nmr_string 

CRTCompressor::GetName(void) const

{

	return "RT Lib Compressor";

}



CConnection *

CRTCompressor::CreateConnection(CConnection::Connection_t t)

{

	if ((t & (CONN_VIDEO | CONN_PASSIVE | CONN_SINK)) == 

		(CONN_VIDEO | CONN_PASSIVE | CONN_SINK))

		return dynamic_cast<CPassiveSink*>(mCompressorStream);

	if ((t & (CONN_VIDEO | CONN_ACTIVE | CONN_SOURCE)) ==

		(CONN_VIDEO | CONN_ACTIVE | CONN_SOURCE))

		return dynamic_cast<CActiveSource*>(mCompressorStream);

	

	return 0;

}



///////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////



CRTCompressorStream::CRTCompressorStream(CRTCompressor *comp,CGUIController *gui)

{

	mGUIController	= gui;

	mComponent		= comp;

	mXXParam				= 0;

	mCompressorBeginCalled	= 0;

	mDataBuffer				= 0;

	mDataReference			= 0;

	mDataAvailable			= 0;

	mDataFinished			= 0;

	mOutputBuffer			= 0;

	mCompressorMode			= -1;

}





CRTCompressorStream::~CRTCompressorStream(void)

{

	delete [] mDataBuffer;

	delete [] mOutputBuffer;



	mDataBuffer = 0;

	mOutputBuffer = 0;



	delete mDataReference;

	mDataReference = 0;

}



/////////////



CConnection::Connection_t	

CRTCompressorStream::_source_GetConnectionType(void)

{

	return (CONN_ACTIVE | CONN_SOURCE | CONN_VIDEO);

}



void			

CRTCompressorStream::_source_Initialize(void)

{

	mXXParam	= mComponent->GetXXParam();

	mBitRate	= mComponent->GetBitRate();

	mInterlace	= mComponent->GetInterlace();

	mFCCHandler	=  *(unsigned long *)(char *)DEFAULT_FOURCC;


	assert(mXXParam);

	CActiveSource::Initialize();



	mFrameCounter = 0;

	BITMAPINFO	*bi = (BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat();

 dprintf("CRTCompressor.cpp: CRTCompressorStream::_source_Initialize calls tmrtxx_begin_xx");
	tmrtxx_begin_xx(mXXParam,bi->bmiHeader.biWidth,bi->bmiHeader.biHeight);

	mCompressorBeginCalled = 1;

}



void			

CRTCompressorStream::_source_Go(void)

{

	CActiveSource::Go();

}



void			

CRTCompressorStream::_source_Terminate(void)

{

	CActiveSource::Terminate();

}



void			

CRTCompressorStream::_source_Wait(void)

{

	CActiveSource::Wait();

}



void

CRTCompressorStream::_source_Cleanup(void)

{

	if (mCompressorBeginCalled)

 dprintf("CRTCompressor.cpp: CRTCompressorStream::_source_Cleanup calls tmrtxx_end_xx");
		tmrtxx_end_xx(mXXParam);

	mCompressorBeginCalled = 0;



	CActiveSource::Cleanup();

}



// Active Source

void	

CRTCompressorStream::_SetInfo(void)

{

	delete [] mDataBuffer;

	BITMAPINFO *bmi =  (BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat();

	mInputSize = bmi->bmiHeader.biSizeImage;

	mDataBuffer = new char[mInputSize];

	delete mDataReference;

	mDataReference = new CPassiveSinkDataReference(mDataBuffer,mInputSize,0,0);



	AVISTREAMINFO i = CPassiveSink::mStreamInfoObject.GetStreamInfo();

	i.fccHandler = mFCCHandler;

	CActiveSource::mStreamInfoObject.SetStreamInfo(i);;



	BITMAPINFO	*input_format = (BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat();

	

	if (input_format->bmiHeader.biCompression == *(unsigned long *)(char *)"YUY2")

	{
		mCompressorMode = TMRT_YUY2;
		dprintf("using mode YUY2");
	}
	else if (input_format->bmiHeader.biCompression == *(unsigned long *)(char *)"YVU9")

		mCompressorMode = TMRT_YUV9;

	else if (input_format->bmiHeader.biCompression == *(unsigned long *)(char *)"RGB ")

		mCompressorMode = TMRT_RGB555;

	else if (input_format->bmiHeader.biCompression == 0)

		mCompressorMode = TMRT_RGB555;

	else if (input_format->bmiHeader.biCompression == *(unsigned long *)(char *)" RGB")

		mCompressorMode = TMRT_RGB555;

	else if (input_format->bmiHeader.biCompression == *(unsigned long *)(char *)"    ")

		mCompressorMode = TMRT_RGB555;

	else throw RTCompressorStreamError("No valid output format for input (RT)",0);



	BITMAPINFO	output_format;

	memset(&output_format,0,sizeof(BITMAPINFO));

	output_format.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

	output_format.bmiHeader.biWidth = input_format->bmiHeader.biWidth; 

	output_format.bmiHeader.biHeight= input_format->bmiHeader.biHeight; 

	output_format.bmiHeader.biPlanes= input_format->bmiHeader.biPlanes; 

	output_format.bmiHeader.biBitCount = 24;

	output_format.bmiHeader.biCompression = mFCCHandler; 

	output_format.bmiHeader.biSizeImage = 0;

		//(input_format->bmiHeader.biWidth * input_format->bmiHeader.biHeight * 8) / 4;; 



 dprintf("calling mStreamInfoObject.SetStreamFormat(5)");
	CActiveSource::mStreamInfoObject.SetStreamFormat((const char *)&output_format,sizeof(BITMAPINFOHEADER));



	int interlace = mInterlace + 1;

	int bits_per_pixel = mBitRate;



	mOutputSize =  

		((((((output_format.bmiHeader.biWidth/interlace)* output_format.bmiHeader.biHeight+

		((output_format.bmiHeader.biWidth/interlace)/4)*(output_format.bmiHeader.biHeight/4)*2)*

		bits_per_pixel)/8)+32));



	delete [] mOutputBuffer;

	mOutputBuffer = new char[mOutputSize];

}



int

CRTCompressorStream::_ProcessData(CPassiveSink *s,const char *source,size_t siz)

{

	CPassiveSinkDataReference *ref = 0;



	if (siz == 0)

		{

		if (!(ref = s->GetDataReference(0)))

			{

			s->DataComplete();

			return 0;

			}	

		s->ReturnDataReference(ref);

		}

	else

		{

		long datasize = mOutputSize;

		if (!(ref = s->GetDataReference(datasize)))

			{

			s->DataComplete();

			return 0;

			}	

		assert(ref->GetWritePtr());

		assert(source);



		assert(mXXParam);

		assert(mCompressorMode >= 0);

		BITMAPINFO *binfo = (BITMAPINFO *)CPassiveSink::mStreamInfoObject.GetStreamFormat();

 dprintf("CRTCompressor.cpp: CRTCompressorStream::_ProcessData calls tmrtxx_xx");
		int comp_size = tmrtxx_xx(mXXParam,binfo->bmiHeader.biWidth,binfo->bmiHeader.biHeight,

			mCompressorMode,mBitRate,mInterlace,(char *)source,ref->GetWritePtr());



		if (comp_size <= 0)

			throw RTCompressorStreamError("Could not compress frame",mFrameCounter);

			

		ref->SetFlags(AVIIF_KEYFRAME);

		s->ReturnDataReference(ref);

		}

	mFrameCounter++;

	mDataAvailable = 0;

	mDataWait.Signal();

	return 1;

}





int		

CRTCompressorStream::ProcessData(CPassiveSink *s)

{

	stack_MutexLock sml(&mDataMutex);



	while (!mDataAvailable)

		{

		if (mDataFinished) 

			{

			s->DataComplete();

			return 0;

			}

		mDataWait.Wait(mDataMutex);

		}



	return _ProcessData(s,mDataBuffer,mInputSize);

}





//////////////



CConnection::Connection_t	

CRTCompressorStream::_sink_GetConnectionType(void)

{

	return (CONN_PASSIVE | CONN_SINK | CONN_VIDEO);

}



void			

CRTCompressorStream::_sink_Initialize(void)

{

	mXXParam = mComponent->GetXXParam();

	mBitRate = mComponent->GetBitRate();

	mInterlace = mComponent->GetInterlace();

	mFCCHandler	=  *(unsigned long *)(char *)DEFAULT_FOURCC;



	CPassiveSink::Initialize();

	mDataAvailable = 0;

	mDataFinished = 0;

}



void			

CRTCompressorStream::_sink_Go(void)

{

	CPassiveSink::Go();

}



void			

CRTCompressorStream::_sink_Terminate(void)

{

	DataComplete();

	CPassiveSink::Terminate();

}



void			

CRTCompressorStream::_sink_Wait(void)

{

	CPassiveSink::Wait();

}



void

CRTCompressorStream::_sink_Cleanup(void)

{

	CPassiveSink::Cleanup();

}





int

CRTCompressorStream::ConsumeDataNow(const CPassiveSinkDataReference *r)

{

	stack_MutexLock sml(&mDataMutex);

	assert(r->GetSize() == mInputSize || r->GetSize() == 0);



	if (mDataFinished) return 0;



	while (mDataAvailable)

		mDataWait.Wait(mDataMutex);



	return _ProcessData(mSink,r->GetWritePtr(),r->GetSize());

}



CPassiveSinkDataReference *

CRTCompressorStream::GetDataReference(size_t s)

{

	assert(s == mInputSize);

	stack_MutexLock sml(&mDataMutex);



	if (mDataFinished) return 0;



	while (mDataAvailable)

		mDataWait.Wait(mDataMutex);



	return mDataReference;

}



void	

CRTCompressorStream::ReturnDataReference(CPassiveSinkDataReference *r)

{

	stack_MutexLock sml(&mDataMutex);

	

	if (!mDataFinished)

		mDataAvailable = 1;

	mDataWait.Signal();

}



void	

CRTCompressorStream::DataComplete(void)

{

	stack_MutexLock sml(&mDataMutex);

	mDataFinished = 1;

	mDataWait.Broadcast();

}





