/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CRTCOMPRESSOR_H_

#define CRTCOMPRESSOR_H_



#include "CComponent.h"

#include "CActiveSource.h"

#include "CPassiveSink.h"

#include <CCMutex.h>

#include <CConditionVariable.h>



class CRTCompressorStream;



class CRTCompressor: public CComponent

{

protected:

	CRTCompressorStream *mCompressorStream;

	int					mDataMustBeFreed;

	HLOCAL				mXXParam;

	int					mBitRate;

	int					mInterlace;



public:

	CRTCompressor(CGUIController *gui);

	virtual ~CRTCompressor(void);



	virtual ConnectionTypeList_t	GetSupportedTypes(void) const;

	virtual nmr_string GetName(void) const;

	virtual CConnection *CreateConnection(CConnection::Connection_t t);

	virtual	void ReturnConnection(CConnection *c,CConnection::Connection_t t){}



	virtual	void	Initialize(void){}

	virtual	void	Cleanup(void){}

	virtual	void	Configure(CWnd *wnd) = 0;

	virtual	int		ConfigureIfNeeded(CWnd *wnd) = 0;



	HLOCAL	GetXXParam(void) { return mXXParam; }

	int		GetBitRate(void) { return mBitRate; }

	int		GetInterlace(void) { return mInterlace; }

};



/////////////////////////////////////////////////

///////////////////////////////////////////////////



class CRTCompSource: public CActiveSource

{

protected:

	virtual Connection_t _source_GetConnectionType(void) = 0;

	virtual	void		 _source_Initialize(void) = 0;

	virtual	void		 _source_Go(void) = 0;

	virtual	void		 _source_Terminate(void) = 0;

	virtual	void		 _source_Wait(void) = 0;

	virtual	void		 _source_Cleanup(void) = 0;

public:

	virtual	Connection_t	GetConnectionType(void)

		{ return _source_GetConnectionType(); }

	virtual	void	Initialize(void)

		{ _source_Initialize(); }

	virtual	void	Go(void) 

		{ _source_Go(); }

	virtual	void	Terminate(void)

		{ _source_Terminate(); }

	virtual	void	Wait(void)

		{ _source_Wait(); }

	virtual	void	Cleanup(void)

		{ _source_Cleanup(); }



};



class CRTCompSink: public CPassiveSink

{

protected:

	virtual Connection_t _sink_GetConnectionType(void) = 0;

	virtual	void		 _sink_Initialize(void) = 0;

	virtual	void		 _sink_Go(void) = 0;

	virtual	void		 _sink_Terminate(void) = 0;

	virtual	void		 _sink_Wait(void) = 0;

	virtual	void		 _sink_Cleanup(void) = 0;



public:

	// connection

	virtual	Connection_t	GetConnectionType(void)

				{ return _sink_GetConnectionType(); }

	virtual	void	Initialize(void)

				{ _sink_Initialize(); }

	virtual	void	Go(void) 

				{ _sink_Go(); }

	virtual	void	Terminate(void)

				{ _sink_Terminate(); }

	virtual	void	Wait(void)

				{ _sink_Wait(); }

	virtual	void	Cleanup(void)

				{ _sink_Cleanup(); }

};



/////////////////////////////

/////////////////////////////



class CRTCompressorStream: public CRTCompSink, public CRTCompSource

{

protected:

	CGUIController		*mGUIController;

	CRTCompressor			*mComponent;

	int					mCompressorBeginCalled;

	char				*mDataBuffer;

	CPassiveSinkDataReference	*mDataReference;

	int					mOutputSize,mInputSize;

	CCMutex				mDataMutex;

	CConditionVariable	mDataWait;

	int					mDataAvailable;

	int					mDataFinished;

	char				*mOutputBuffer;

	int					mFrameCounter;

	HLOCAL				mXXParam;

	int					mBitRate;

	int					mInterlace;

	int					mCompressorMode;

	unsigned long		mFCCHandler;



	CRTCompressorStream(CRTCompressor *comp,CGUIController *gui);

	virtual ~CRTCompressorStream(void);



	virtual	Connection_t	_source_GetConnectionType(void);

	virtual	void			_source_Initialize(void);

	virtual	void			_source_Go(void);

	virtual	void			_source_Terminate(void);

	virtual	void			_source_Wait(void);

	virtual	void			_source_Cleanup(void);



	virtual	Connection_t	_sink_GetConnectionType(void);

	virtual	void			_sink_Initialize(void);

	virtual	void			_sink_Go(void);

	virtual	void			_sink_Terminate(void);

	virtual	void			_sink_Wait(void);

	virtual	void			_sink_Cleanup(void);



	// Passive Sink

	virtual	CPassiveSinkDataReference *GetDataReference(size_t);

	virtual	void	ReturnDataReference(CPassiveSinkDataReference *r);

	virtual	void	DataComplete(void);

	virtual	int		ConsumeDataNow(const CPassiveSinkDataReference *r);



	// Active Source

	virtual	void	_SetInfo(void);

			int		_ProcessData(CPassiveSink *s,const char *data,size_t siz);

	virtual	int		ProcessData(CPassiveSink *s);



	friend CRTCompressor;



	CERROR_SUBCLASS(RTCompressorStreamError);

};





#endif

