/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// CRecordingEventDialog.cpp : implementation file

//



#include "stdafx.h"

#include "CRecordingEventDialog.h"

#include "CAudioSettingsDialog.h"

#include "CVideoSettingsDialog.h"

#include "IVideoTuner.h"

#include "nmr_string.h"

#include <assert.h>



#ifdef _DEBUG

#define new DEBUG_NEW

#undef THIS_FILE

static char THIS_FILE[] = __FILE__;

#endif



void ZeroSeconds(COleDateTime &d)

{

	d.SetDateTime(

		d.GetYear(),d.GetMonth(),d.GetDay(),d.GetHour(),d.GetMinute(),0

		);

}





CString gsQuality[] = 

{

"Highest",

"High",

"Moderate",

"Low",

"Lowest"

};



/////////////////////////////////////////////////////////////////////////////

// CRecordingEventDialog dialog





CRecordingEventDialog::CRecordingEventDialog(CWnd* pParent /*=NULL*/)

	: CDialog(CRecordingEventDialog::IDD, pParent)

{

	//{{AFX_DATA_INIT(CRecordingEventDialog)

	mChannel = 0;

	mDay = -1;

	mAppendDateAndTime = FALSE;

	mFilename = _T("");

	mAudioText = _T("");

	mVideoText = _T("");

	//}}AFX_DATA_INIT



	mEventTimeLimit = -1;



	mSupervisor = 0;

	mChannel = 2;

	mDate = COleDateTime::GetCurrentTime();

	mDay = mDate.GetDayOfWeek() - 1;

	mStartTime = COleDateTime::GetCurrentTime();

	mEndTime = COleDateTime::GetCurrentTime();

	mDateIsVisible = 1;

	ZeroSeconds(mDate);

	ZeroSeconds(mEndTime);

	ZeroSeconds(mStartTime);



	// default video settings

	mFPS = 15;

	mQuality = 1;

	mHeight = 240;

	mWidth = 320;

	mVideoSource = 	IVideoTuner::BaseBand_Composite;

	mBrightness = 128;

	mContrast = 128;

	mHue = 128;

	mSaturation = 128;



	// default audio settings changed 3/31/02 - dbm

	mSampleSize = 16;
	mChannels = 1;
	mFrequency = 44100;

/*	mSampleSize = 8;

	mChannels = 1;

	mFrequency = 22050;
*/
}





void CRecordingEventDialog::DoDataExchange(CDataExchange* pDX)

{

	CreateAudioText();

	CreateVideoText();



	CDialog::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(CRecordingEventDialog)

	DDX_Text(pDX, IDC_RE_CHANNEL, mChannel);

	DDX_CBIndex(pDX, IDC_RE_DAY, mDay);

	DDX_Control(pDX, IDC_RE_DATE, mcDate);

	DDX_Control(pDX, IDC_RE_ENDTIME, mcEndTime);

	DDX_Control(pDX, IDC_RE_STARTTIME, mcStartTime);

	DDX_Check(pDX, IDC_RE_APPENDDATETIME, mAppendDateAndTime);

	DDX_Text(pDX, IDC_RE_FILENAME, mFilename);

	DDX_Text(pDX, IDC_RE_AUDIOTEXT, mAudioText);

	DDX_Text(pDX, IDC_RE_VIDEOTEXT, mVideoText);

	//}}AFX_DATA_MAP



	if (pDX->m_bSaveAndValidate)

		{

		mDate = mcDate.GetTime().date;

		mEndTime = mcEndTime.GetTime().date;

		mStartTime = mcStartTime.GetTime().date;

		ZeroSeconds(mDate);

		ZeroSeconds(mEndTime);

		ZeroSeconds(mStartTime);

		}

	else

		{

		ZeroSeconds(mDate);

		ZeroSeconds(mEndTime);

		ZeroSeconds(mStartTime);



		VARIANT v;

		v = mcDate.GetTime();

		v.date = mDate;

		mcDate.SetTime(v);



		v = mcEndTime.GetTime();

		v.date = mEndTime;

		mcEndTime.SetTime(v);



		v = mcStartTime.GetTime();

		v.date = mStartTime;

		mcStartTime.SetTime(v);



		ShowDate(mDay < 7);

		CalculateDiskSpace();

		}

	CWnd *w = GetDlgItem(IDC_RE_CHANNEL);	

	if (w)

		w->EnableWindow(mVideoSource == IVideoTuner::Tuner_Cable ||

			mVideoSource == IVideoTuner::Tuner_Antenna);

}



double

CRecordingEventDialog::CalculateDiskSpace(void)

{

	CString	s = "Too large";

	double result = 0;



try {

	COleDateTimeSpan span = mEndTime - mStartTime;

	// make sure no numbers exceed 2 billion

	double seconds = span.GetTotalSeconds();

	if (seconds < 0) 

		{ 

		mEndTime += COleDateTimeSpan(1,0,0,0);

		span = mEndTime - mStartTime;

		seconds = span.GetTotalSeconds();

		if (seconds < 0)

			{

			s = "End time is earlier than start time."; 

			throw 0;

			}

		}

	if (seconds > 2000000000) throw 0;

	double audio_size = 

		((mSampleSize / 8) * mChannels * mFrequency) * seconds;

	if (audio_size > 2000000000) throw 0;



	assert(mQuality >= 0 && mQuality <= 4);

	int bit_rate = 0,interlace = 0;

	switch(mQuality)

		{

		case 0:

			bit_rate = 4;

			interlace = 0;

			break;

		case 1:

			bit_rate = 3;

			interlace = 0;

			break;

		case 2:

			bit_rate = 4;

			interlace = 1;

			break;

		case 3:

			bit_rate = 3;

			interlace = 1;

			break;

		case 4:

			bit_rate = 2;

			interlace = 1;

			break;

		}



	++interlace;



	double video_size =  

		((((((mWidth/interlace)* mHeight+

		((mWidth/interlace)/4)*(mHeight/4)*2)*bit_rate)/8)+32));



	video_size *= mFPS;

	video_size *= seconds;

	if (video_size > 2000000000) throw 0;

	if (video_size + audio_size > 2000000000) throw 0;

	double total_size = video_size + audio_size;

	s.Format("%lu KB required",(unsigned long)(total_size/1000)); 

	result = total_size;

	}

catch(...)

	{

	}	



	CWnd *w = GetDlgItem(IDC_RE_DISKSPACE);	

	assert(w);

	w->SetWindowText(s);

	return result;

}



void

CRecordingEventDialog::CreateAudioText(void)

{

	mAudioText.Format("%dkhz %d bit %s",mFrequency / 1000,mSampleSize,

		mChannels == 1 ? "Mono" : "Stereo");

}



void

CRecordingEventDialog::CreateVideoText(void)

{

	assert(mQuality >= 0 && mQuality <= 4);

	mVideoText.Format("%dx%d %dfps %s quality",mWidth,mHeight,mFPS,gsQuality[mQuality]);

}



void

CRecordingEventDialog::ShowDate(int show)

{

	CWnd	*date_text = GetDlgItem(IDC_RE_DATETEXT);

	CWnd	*date_ctrl = GetDlgItem(IDC_RE_DATE);

	assert(date_text);

	assert(date_ctrl);



	if ((mDateIsVisible && !show) ||

		(!mDateIsVisible && show))

		{

		date_text->EnableWindow(show);

		date_ctrl->EnableWindow(show);

		}

	mDateIsVisible = show;

}



int

CRecordingEventDialog::ValidateData(void)

{

	// round off seconds

	mEndTime.SetDateTime(

				mEndTime.GetYear(),

				mEndTime.GetMonth(), 

				mEndTime.GetDay(), 

				mEndTime.GetHour(), 

				mEndTime.GetMinute(),

				0);

	mStartTime.SetDateTime(

				mStartTime.GetYear(),

				mStartTime.GetMonth(), 

				mStartTime.GetDay(), 

				mStartTime.GetHour(), 

				mStartTime.GetMinute(),

				0);





	COleDateTimeSpan span = mEndTime - mStartTime;

	

	// error if end time is <= begin time

	if (span.GetTotalSeconds() <= 0)

		{

		mEndTime += COleDateTimeSpan(1,0,0,0);

		span = mEndTime - mStartTime;

		if (span.GetTotalSeconds() <= 0)

			{

			AfxMessageBox("Start time is after end time.");

			return 0;

			}

		}



	// error if beyond time limit

	assert(mEventTimeLimit > -1);

	if (mEventTimeLimit > 0)

		{

		if (span.GetTotalSeconds() > (mEventTimeLimit*60))

			{

			nmr_string s = "This version is limited to ";

			s += mEventTimeLimit;

			s += " minute events.";

			AfxMessageBox(s);

			return 0;

			}

		}

	// error if file is too big

	unsigned long space_required;

	if ((space_required = (unsigned long)CalculateDiskSpace()) == 0)

		{

		AfxMessageBox("Video file may be too large.");

///		return 0;				///	dbm

		}

	// add one percent overhead for index

	space_required += (unsigned long)(space_required * .01);



	// warn if not enough disk space

	DWORD	sectors_per_cluster;

	DWORD	bytes_per_sector;

	DWORD	number_of_free_clusters;

	DWORD	total_number_of_clusters;

	if (mFilename.GetLength() > 3 && isalpha(mFilename[0]) &&

		mFilename[1] == ':' && mFilename[2] == '\\')

		{

		char buffer[] = {mFilename[0],mFilename[1],mFilename[2],0};

		GetDiskFreeSpace(buffer,&sectors_per_cluster,&bytes_per_sector,

			&number_of_free_clusters,&total_number_of_clusters);

		}

	else

		{

		GetDiskFreeSpace(NULL,&sectors_per_cluster,&bytes_per_sector,

			&number_of_free_clusters,&total_number_of_clusters);

		}

	unsigned long space_available = number_of_free_clusters * sectors_per_cluster * bytes_per_sector;



	space_available /= 1000;

	space_required /= 1000;

	if (space_available <= space_required)

		{

		CString s;

		s.Format("The file will require %luk bytes and only %luk are available.\nDo it anyhow?",

			space_required,space_available);

		if (AfxMessageBox(s,MB_YESNO | MB_APPLMODAL) != IDYES)

			return 0;

		}



	// if date is in past, make sure it is next year.

	while (mDate < COleDateTime::GetCurrentTime())

		mDate.SetDateTime(mDate.GetYear()+1,

			mDate.GetMonth(), 

			mDate.GetDay(), 

			mDate.GetHour(), 

			mDate.GetMinute(),

			mDate.GetSecond());





	return 1;

}





BEGIN_MESSAGE_MAP(CRecordingEventDialog, CDialog)

	//{{AFX_MSG_MAP(CRecordingEventDialog)

	ON_CBN_SELCHANGE(IDC_RE_DAY, OnSelchangeReDay)

	ON_BN_CLICKED(IDC_RE_BROWSE, OnReBrowse)

	ON_BN_CLICKED(IDC_RE_AUDIO, OnReAudio)

	ON_BN_CLICKED(IDC_RE_VIDEO, OnReVideo)

	//}}AFX_MSG_MAP

END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////

// CRecordingEventDialog message handlers



void CRecordingEventDialog::OnSelchangeReDay() 

{

	// TODO: Add your control notification handler code here

	UpdateData(TRUE);



	ShowDate(mDay < 7);

	if (mDay < 7)

		{

		// sync up with date field

		COleDateTime	dd = COleDateTime::GetCurrentTime();

		COleDateTimeSpan	span(1,0,0,0);

		while (dd.GetDayOfWeek() != (mDay+1))

			dd = dd + span;

		mDate = dd;

		UpdateData(FALSE);

		}

}



void CRecordingEventDialog::OnReBrowse() 

{

	// TODO: Add your control notification handler code here

	UpdateData(TRUE); // get data from dialog

	CFileDialog dlog(FALSE,"vp3",mFilename);

	if (dlog.DoModal() == IDOK)

		{

		mFilename = dlog.GetPathName();

		UpdateData(FALSE);

		}

}



BEGIN_EVENTSINK_MAP(CRecordingEventDialog, CDialog)

    //{{AFX_EVENTSINK_MAP(CRecordingEventDialog)

	ON_EVENT(CRecordingEventDialog, IDC_RE_DATE, 1 /* Changed */, OnChangedReDate, VTS_DATE)

	ON_EVENT(CRecordingEventDialog, IDC_RE_STARTTIME, 1 /* Changed */, OnChangedReStarttime, VTS_DATE)

	ON_EVENT(CRecordingEventDialog, IDC_RE_ENDTIME, 1 /* Changed */, OnChangedReEndtime, VTS_DATE)

	//}}AFX_EVENTSINK_MAP

END_EVENTSINK_MAP()



void CRecordingEventDialog::OnChangedReDate(DATE datetime) 

{

	// sync up with day

	UpdateData(TRUE);

	mDay = mDate.GetDayOfWeek() - 1;

	UpdateData(FALSE);

}





void CRecordingEventDialog::OnChangedReStarttime(DATE datetime) 

{

	// TODO: Add your control notification handler code here

	UpdateData(TRUE);

	CalculateDiskSpace();

	

}



void CRecordingEventDialog::OnChangedReEndtime(DATE datetime) 

{

	// TODO: Add your control notification handler code here

	UpdateData(TRUE);

	CalculateDiskSpace();

}



void CRecordingEventDialog::OnOK() 

{

	// TODO: Add extra validation here

	UpdateData(TRUE);

	if (ValidateData())

		CDialog::OnOK();

}



void CRecordingEventDialog::OnReAudio() 

{

	// TODO: Add your control notification handler code here

	UpdateData(TRUE);

	CAudioSettingsDialog dlog;

	dlog.mVolume		= mVolume;

	dlog.mSampleSize = (mSampleSize == 8 ? 0 : 1);

	dlog.mChannels = (mChannels == 1 ? 0 : 1);

	dlog.mFrequency = (mFrequency == 11025 ? 0 : (mFrequency == 22050 ? 1 : 2));

	dlog.mSupervisor		= mSupervisor;

	if (dlog.DoModal() == IDOK)

		{

		mVolume		= dlog.mVolume;

		mFrequency	= (dlog.mFrequency == 0 ? 11025 : (dlog.mFrequency == 1 ? 22050 : 44100));

		mSampleSize = (dlog.mSampleSize == 0 ? 8 : 16);

		mChannels	= (dlog.mChannels == 0 ? 1 : 2);

		UpdateData(FALSE);

		}

	

}



void CRecordingEventDialog::OnReVideo() 

{

	UpdateData(TRUE);

	CVideoSettingsDialog dlog;

	dlog.mFPS		= (mFPS == 10 ? 0 : (mFPS == 15 ? 1 : 2));

	dlog.mQuality	= (mQuality - 4) * -1;

	dlog.mDimensions= (mWidth == 320 ? 1 : 0);

	dlog.mSupervisor	= mSupervisor;

	dlog.mHue		= mHue;

	dlog.mContrast	= mContrast;

	dlog.mSaturation= mSaturation;

	dlog.mBrightness= mBrightness;

	dlog.mVideoSource= mVideoSource;



	if (dlog.DoModal() == IDOK)

		{

		mFPS = (dlog.mFPS == 0 ? 10 : (dlog.mFPS == 1 ? 15 : 30));

		mQuality = dlog.mQuality;

		mQuality = (dlog.mQuality - 4) * -1;

		mWidth = (dlog.mDimensions == 0 ? 160 : 320);

		mHeight = (mWidth == 320 ? 240 : 120);

		mVideoSource= dlog.mVideoSource;

		mHue		= dlog.mHue;

		mContrast	= dlog.mContrast;

		mBrightness = dlog.mBrightness;

		mSaturation = dlog.mSaturation;

		UpdateData(FALSE);

		}

	// TODO: Add your control notification handler code here

	CWnd *w = GetDlgItem(IDC_RE_CHANNEL);	

	w->EnableWindow(mVideoSource == IVideoTuner::Tuner_Cable ||

		mVideoSource == IVideoTuner::Tuner_Antenna);

}

