/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// CTimedRecordDialog.cpp : implementation file

//



#include "stdafx.h"

#include "TrueCap.h"

#include "CTimedRecordDialog.h"

#include "MegaHeaders.h"

#include <assert.h>



#ifdef _DEBUG

#define new DEBUG_NEW

#undef THIS_FILE

static char THIS_FILE[] = __FILE__;

#endif



/////////////////////////////////////////////////////////////////////////////

// CTimedRecordDialog dialog





CTimedRecordDialog::CTimedRecordDialog(CWnd* pParent /*=NULL*/)

	: CDialog(CTimedRecordDialog::IDD, pParent)

{

	//{{AFX_DATA_INIT(CTimedRecordDialog)

	mTime = 0;

	//}}AFX_DATA_INIT

}





void CTimedRecordDialog::DoDataExchange(CDataExchange* pDX)

{

	CDialog::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(CTimedRecordDialog)

	DDX_Text(pDX, IDC_RECORDTIME, mTime);

	//}}AFX_DATA_MAP



	CalculateDiskSpace(mTime);

}





BEGIN_MESSAGE_MAP(CTimedRecordDialog, CDialog)

	//{{AFX_MSG_MAP(CTimedRecordDialog)

	ON_EN_UPDATE(IDC_RECORDTIME, OnUpdateRecordtime)

	//}}AFX_MSG_MAP

END_MESSAGE_MAP()



double

CTimedRecordDialog::CalculateDiskSpace(int minutes)

{

	CString	s = "Too large";

	double result = 0;



try {

	I64 seconds;

	seconds = ((I64)minutes) * 60.0;

	if (seconds > 2000000000) throw 0;

	I64 audio_size = 

		((mSampleSize / 8) * mChannels * mFrequency) * seconds;

	if (audio_size > 2000000000) throw 0;



	assert(mQuality >= 0 && mQuality <= 4);

	int bit_rate = 0,interlace = 0;

	switch(mQuality)

		{

		case 0:

			bit_rate = 4;

			interlace = 0;

			break;

		case 1:

			bit_rate = 3;

			interlace = 0;

			break;

		case 2:

			bit_rate = 4;

			interlace = 1;

			break;

		case 3:

			bit_rate = 3;

			interlace = 1;

			break;

		case 4:

			bit_rate = 2;

			interlace = 1;

			break;

		}



	++interlace;



	I64 video_size =  

		((((((mWidth/interlace)* mHeight+

		((mWidth/interlace)/4)*(mHeight/4)*2)*bit_rate)/8)+32));



	video_size *= mFPS;

	video_size *= seconds;

	if (video_size > 2000000000) throw 0;

	if (video_size + audio_size > 2000000000) throw 0;

	I64 total_size = video_size + audio_size;

	s.Format("%lu KB required",(unsigned long)(total_size/1000)); 

	result = total_size;

	}

catch(...)

	{

	result = 0;

	}	



	CWnd *w = GetDlgItem(IDC_TR_SIZE);	

	if (w)

		w->SetWindowText(s);

	return result;

}



int

CTimedRecordDialog::ValidateData(void)

{

	// error if file is too big

	unsigned long space_required;

	if ((space_required = (unsigned long)CalculateDiskSpace(mTime)) <= 0)

		{

		AfxMessageBox("Video file may be too large.");

///		return 0;

		}



	// add one percent overhead for index

	space_required += (unsigned long)(space_required * .01);



	// warn if not enough disk space

	DWORD	sectors_per_cluster;

	DWORD	bytes_per_sector;

	DWORD	number_of_free_clusters;

	DWORD	total_number_of_clusters;

	if (mFilename.GetLength() > 3 && isalpha(mFilename[0]) &&

		mFilename[1] == ':' && mFilename[2] == '\\')

		{

		char buffer[] = {mFilename[0],mFilename[1],mFilename[2],0};

		GetDiskFreeSpace(buffer,&sectors_per_cluster,&bytes_per_sector,

			&number_of_free_clusters,&total_number_of_clusters);

		}

	else

		{

		GetDiskFreeSpace(NULL,&sectors_per_cluster,&bytes_per_sector,

			&number_of_free_clusters,&total_number_of_clusters);

		}

	unsigned long space_available = number_of_free_clusters * sectors_per_cluster * bytes_per_sector;



	space_available /= 1000;

	space_required /= 1000;

	if (space_available <= space_required)

		{

		CString s;

		s.Format("The file will require %luk bytes and only %luk are available.\nDo it anyhow?",

			space_required,space_available);

		if (AfxMessageBox(s,MB_YESNO | MB_APPLMODAL) != IDYES)

			return 0;

		}



	return 1;

}





/////////////////////////////////////////////////////////////////////////////

// CTimedRecordDialog message handlers



void CTimedRecordDialog::OnUpdateRecordtime() 

{

	// TODO: Add your control notification handler code here

	CWnd *w = GetDlgItem(IDC_RECORDTIME);

	CString s;

	w->GetWindowText(s);

	int t;

	sscanf((const char *)s,"%d",&t);

	CalculateDiskSpace(t);

}



void CTimedRecordDialog::OnOK() 

{

	// TODO: Add extra validation here

	UpdateData(TRUE);

	if (ValidateData())

		CDialog::OnOK();

}

