/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CVidcapSource_TC.h"
#include "CGUIController.h"
#include "CMessageStream.h"
#include "stack_aids.h"
#include "danbug.h"
#include "..\dansoft\acmenc.h"

WNDPROC	CVidcapSource_TC::mOldWndproc = 0;



HWND

CVidcapSource_TC::GetCaptureWindow(void)

{

	return mCaptureWindow;

}



CComponent *

CVidcapSource_TC::CreatorFunc(CGUIController *gui)

{

	return new CVidcapSource_TC(gui);

}



BOOL	

CVidcapSource_TC::GrabFrameNoStop(void)

{

	return capGrabFrameNoStop(mCaptureWindow);

}



BOOL	

CVidcapSource_TC::GrabFrame(void)

{

	return capGrabFrame(mCaptureWindow);

}



BOOL	

CVidcapSource_TC::FileSaveDib(const char *name)

{

	return capFileSaveDIB(mCaptureWindow,name);

}



LRESULT CALLBACK mywndproc(HWND h, UINT i, WPARAM w, LPARAM l)

{

	LRESULT result =0;



	if (i == WM_KEYDOWN || i == WM_RBUTTONDOWN)

		{

		HWND parent = ::GetParent(h);

		if (parent)

			::PostMessage(parent,i,w,l);

		}

//	else if (i == WM_ACTIVATE)

//		{

//		}

	else

		{

		result = CallWindowProc(CVidcapSource_TC::mOldWndproc,

					h,i,w,l);

		}



	return result;

}



void

CVidcapSource_TC::Initialize(void)

{

	if (mSingleFieldInput)

		capOverlay(mCaptureWindow,FALSE);

	CVidcapSource::Initialize();

}



void

CVidcapSource_TC::Cleanup(void)

{

	CVidcapSource::Cleanup();

	capOverlay(mCaptureWindow,TRUE);

}



void

CVidcapSource_TC::ErrorMessage(int id,LPCSTR mess)

{

	if (mIgnoreErrorMessages) return;

	// kludge

	if (id == IDS_CAP_DRIVER_ERROR && 

		strcmp(mess,"Unsupported video format.") == 0)

		return;



	if (id == IDS_CAP_AVI_DRAWDIB_ERROR) 

		return;

	if (id || *mess)

		{

 dprintf("CVidcapSource_TC::ErrorMessage");
		(mGUIController->Error(this)) << mess << " (" << id << ")" << &(CMessageStream::show);

		}

}



void

CVidcapSource_TC::SetFreeSpaceDrive(const char *d)

{

	mFreeSpaceDrive = d;

}



void	

CVidcapSource_TC::ShowVideo(BOOL s)

{

	TRACE("ShowVideo %d\n",(int)s);

	stack_MutexLock sml(&mOperationalMutex);



	if(!mOperational || !mSingleFieldInput)

		capOverlay(mCaptureWindow,s);

	//::ShowWindow(mCaptureWindow, s ? SW_SHOW : SW_HIDE);

}



void

CVidcapSource_TC::SetQuality(int quality)

{

	assert(quality >= 0 && quality <= 4);

	int bits_per_pixel,interlace;



	switch(quality)

		{

		case 0:

			bits_per_pixel = 4;

			interlace = 1;

			break;

		case 1:

			bits_per_pixel = 3;

			interlace = 1;

			break;

		case 2:

			bits_per_pixel = 4;

			interlace = 2;

			break;

		case 3:

			bits_per_pixel = 3;

			interlace = 2;

			break;

		case 4:

			bits_per_pixel = 2;

			interlace = 2;

			break;

		}



	mVideoFrameSize =  ((((((mVideoWidth/interlace)*mVideoHeight+

			((mVideoWidth/interlace)/4)*(mVideoHeight/4)*2)*

			bits_per_pixel)/8)+32));

}



void

CVidcapSource_TC::LogFrames(int frames)

{

#if 0

	if (frames < 100 || frames % 25 == 0)

	(mGUIController->Log(this)) << frames << " frames captured, " <<

		mFramesDropped << " frames dropped."

		<< &(CMessageStream::show);

#endif

	long	tot = mAudioBytesProcessed + (mVideoFramesProcessed * mVideoFrameSize);

/*	if ( (mDiskSpaceRemaining - (double)tot) < 20000000.0 || tot > 2000000000)

		{

		_InternalStop();

		(mGUIController->Error(this)) 

			<< "Low disk space. Capture terminated\n" 

			<< mFramesDropped << " frames dropped."

			<< &(CMessageStream::show);

		}
*/
		

	if (frames % 3 == 0)

	{

		(mGUIController->Log(this)) 

			<< ((float)(((float)(timeGetTime() - mStartTime)) / 1000.0)) << " sec elapsed.\t"

			<< ((long)( (mDiskSpaceRemaining - (double)tot) / 1024.0)) << "k bytes free.\t"

			<< mFramesDropped << " frames dropped."

			<< &(CMessageStream::show);

	}

}



void

CVidcapSource_TC::Start(void)

{

	DWORD	sectors_per_cluster = 0;

	DWORD	bytes_per_sector = 0;

	DWORD	number_of_free_clusters = 0;

	DWORD	total_number_of_clusters = 0;

	BOOL	result;



	result = ::GetDiskFreeSpace(mFreeSpaceDrive,&sectors_per_cluster,

		&bytes_per_sector,&number_of_free_clusters,&total_number_of_clusters);

	assert(result);



	mDiskSpaceRemaining = (double)number_of_free_clusters * (double)sectors_per_cluster *

		(double)bytes_per_sector;



	mStartTime = timeGetTime();

	CVidcapSource::Start();

}



void

CVidcapSource_TC::ConfigureCaptureDriverSource(void)

{

	capDlgVideoSource(mCaptureWindow);

}



void

CVidcapSource_TC::SetVideoParams(int width,int height,int fps,int single_field_input)

{

	assert(width == 320 || width == 160 || width==640);

	assert(height == 240 || height == 120 || height == 480);

	assert(fps == 30 || fps == 15 || fps == 10);



	mFramerateBasis = (float)29.97;

	mFramesToKeep = 1;

	mFramesToDrop = (fps == 30 ? 0 : (fps == 15 ? 1 : 2));

	mSingleFieldInput = single_field_input;



	// must always do for Rockwell?

	//if (mVideoWidth != width || mVideoHeight != height)

		{

		mVideoWidth		= width;

		mVideoHeight	= height;



		_VideoColorSpace();

		}

}



#define VP31 1

int

CVidcapSource_TC::_VideoColorSpace(void)

{
	BITMAPINFO		v;
 dprintf("-->CVidcapSource_TC::_VideoColorSpace");

#if VP31

	v.bmiHeader.biSize			= sizeof(v.bmiHeader); 
	v.bmiHeader.biWidth			= mVideoWidth; 
	v.bmiHeader.biHeight		= mVideoHeight; 
	v.bmiHeader.biPlanes		= 1; 
	v.bmiHeader.biXPelsPerMeter	= 0; 
	v.bmiHeader.biYPelsPerMeter	= 0; 
	v.bmiHeader.biClrUsed		= 0; 
	v.bmiHeader.biClrImportant	= 0; 

	v.bmiHeader.biBitCount		= 12; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"I420"); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for I420");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v))) 
		{
 dprintf(" '--> I420 accepted");
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 12; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"YV12"); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for YV12");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v))) 
		{
 dprintf(" '--> YV12 accepted");
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 16; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"YUY2"); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for YUY2");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v))) 
		{
 dprintf(" '--> YUY2 accepted");
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 24; 
	v.bmiHeader.biCompression	= 0; // rgb 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for rgb24");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v)))
		{
 dprintf(" '--> rgb24 accepted");
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 32; 
	v.bmiHeader.biCompression	= 0; // rgb 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for rgb32");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v)))
		{
 dprintf(" '--> rgb32 accepted");
		AdjustVideoSize();
		return 1;
		}

#else
	v.bmiHeader.biSize			= sizeof(v.bmiHeader); 
	v.bmiHeader.biWidth			= mVideoWidth; 
	v.bmiHeader.biHeight		= mVideoHeight; 
	v.bmiHeader.biPlanes		= 1; 
	v.bmiHeader.biXPelsPerMeter	= 0; 
	v.bmiHeader.biYPelsPerMeter	= 0; 
	v.bmiHeader.biClrUsed		= 0; 
	v.bmiHeader.biClrImportant	= 0; 

	v.bmiHeader.biBitCount		= 9; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"YVU9"); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for YUV9");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v))) 
		{
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 16; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"YUY2"); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for YUY2");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v)))
		{
		AdjustVideoSize();
		return 1;
		}

	v.bmiHeader.biBitCount		= 16; 
	v.bmiHeader.biCompression	= 0; // rgb 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for rgb16");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v)))
		{
		AdjustVideoSize();
		return 1;
		}

	// john says it isn't so, but what the hell.

	v.bmiHeader.biBitCount		= 16; 
	v.bmiHeader.biCompression	= *((DWORD *)(char *)"RGB "); 
	v.bmiHeader.biSizeImage		= (mVideoWidth * mVideoHeight * v.bmiHeader.biBitCount) / 8; 

 dprintf("--> asking for rgb16 another way");
	if (capSetVideoFormat(mCaptureWindow,&v,sizeof(v)))
		{
		AdjustVideoSize();
		return 1;
		}
#endif

	return 0;
}


void

CVidcapSource_TC::SetAudioParams(int sample_size,int channels,int frequency)

{
 dprintf("here's also where audio params get set??? -- ");

	assert(sample_size == 8 || sample_size == 16);
	assert(channels == 1 || channels == 2);
	assert(frequency == 11025 || frequency == 22050 || frequency == 44100);



	mAudioFormat.wBitsPerSample = sample_size;
	mAudioFormat.nChannels = channels;
	mAudioFormat.nSamplesPerSec = frequency;
	mAudioFormat.wFormatTag		= 1; ///ACMVORBIS;
    mAudioFormat.nBlockAlign		= (mAudioFormat.nChannels * mAudioFormat.wBitsPerSample) / 8; 
    mAudioFormat.nAvgBytesPerSec	= mAudioFormat.nSamplesPerSec * mAudioFormat.nBlockAlign; 
    mAudioFormat.cbSize			= 0;
	
/*
	mAudioFormat.wFormatTag		= WAVE_FORMAT_PCM; 
    mAudioFormat.nBlockAlign		= (mAudioFormat.nChannels * mAudioFormat.wBitsPerSample) / 8; 
    mAudioFormat.nAvgBytesPerSec	= mAudioFormat.nSamplesPerSec * mAudioFormat.nBlockAlign; 
    mAudioFormat.cbSize			= 0; 
*/

	if (!capSetAudioFormat(mCaptureWindow,&mAudioFormat,sizeof(mAudioFormat)))
		throw VidcapError("Could not set audio format",0);
}



CVidcapSource_TC::CVidcapSource_TC(CGUIController *gui)

		: CVidcapSource(gui)

{

	mSingleFieldInput	= AfxGetApp()->GetProfileInt("CVidcapSource","SingleFieldInput",0);

}



CVidcapSource_TC::~CVidcapSource_TC(void)

{

	AfxGetApp()->WriteProfileInt("CVidcapSource","SingleFieldInput",mSingleFieldInput);

}



void

CVidcapSource_TC::VidcapInitialize(void)

{
 dprintf("CVidcapSource_TC::VidcapInitialize");

	CVidcapSource::VidcapInitialize();

	mIgnoreStatusMessages = 1;

	WAVEFORMATEX	a;



	a = mAudioFormat;

	// setup in mAudioFormat in CVidcapSource ctor
    //a.wBitsPerSample; 
    //a.nChannels; 
    //a.nSamplesPerSec; 

	a.wFormatTag		= WAVE_FORMAT_PCM; 
    a.nBlockAlign		= (a.nChannels * a.wBitsPerSample) / 8; 
    a.nAvgBytesPerSec	= a.nSamplesPerSec * a.nBlockAlign; 
    a.cbSize			= 0; 

	capSetAudioFormat(mCaptureWindow,&a,sizeof(a));

	mIgnoreErrorMessages = 1;

	if (mVideoWidth != 320 && mVideoWidth != 160 && mVideoWidth != 640)

		{

		mVideoWidth = 320;

		mVideoHeight = 240;

		}

	int res = _VideoColorSpace();

	mIgnoreErrorMessages = 0;

	if (!res)

		throw VidcapError("Could not find compatible video format",0);

	mOldWndproc = (WNDPROC)::GetWindowLong(mCaptureWindow,GWL_WNDPROC);

	::SetWindowLong(mCaptureWindow,GWL_WNDPROC,(long)&mywndproc);

    capPreviewRate(mCaptureWindow, 1000); // Slow preview when inactive

}



void

CVidcapSource_TC::Configure(CWnd *wnd)

{

	if (mVideoUsed || (!mVideoUsed && !mAudioUsed))

		{

		if (!_ConfigureVideo(wnd)) return;

		mVideoConfigured = 1;

		}

	if (mAudioUsed)

		{

		if (!_ConfigureAudio(wnd)) return;

		mAudioConfigured = 1;

		}

}



int

CVidcapSource_TC::ConfigureIfNeeded(CWnd *wnd)

{

	if (mVideoUsed)

		{

		if (!mVideoConfigured)

			if (!_ConfigureVideo(wnd)) return 0;

		mVideoConfigured = 1;

		}

	if (mAudioUsed)

		{

		if (!mAudioConfigured)

			if (!_ConfigureAudio(wnd)) return 0;

		mAudioConfigured = 1;

		}

	return 1;

}



int

CVidcapSource_TC::_ConfigureVideo(CWnd *wnd)

{

	assert(mCaptureWindow);



#if 0

	if (!capDlgVideoDisplay(mCaptureWindow)) return 0;

	if (!capDlgVideoFormat(mCaptureWindow)) return 0;

	AdjustVideoSize();

	if (!capDlgVideoSource(mCaptureWindow)) return 0;



	mFramerateBasis = 29.97;

	mFramesToKeep = 1;

	mFramesToDrop = 0;

#endif

	return 1;

}



int

CVidcapSource_TC::_ConfigureAudio(CWnd *wnd)

{



	return 1;

}



