/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <malloc.h>
#include <tmrtprv.h>
#include "tmrtxx.h"
#include "rtdeltas.h"
#include "xx.h"
#include "danbug.h"

typedef struct {
//	int savebits;
	int width;
	int height;
	int bits;
	int ntrp;
	char *tbuffer;
	int NumDeltasReallyUsed;
} XXPARAM;

U8 xxtbl_state = 0xFF;
U32 xxtbl[1024];
static I32 deltas[NUMDELTAS]=DELTAS_4BIT
static void make_header(U8 *hdr,long width,long height,long type,long bits,long finfo,long framesize);
static void build_tables(int deltasUsed);

//long    mrnvar1; //for XX_TIME_TEST

#define MAXWIDTH	1440	//720 x 2
U16	linebuf_xx[MAXWIDTH];
U8     rgb2y[0x8000];     // rgb to yvu converters
U8     rgb2v[0x8000];     // rgb to yvu converters
U8     rgb2u[0x8000];     // rgb to yvu converters

//static unsigned char tbuffer[ (320*240*9)/16 + 16];   /// 4-bit buffer for compress

/* Tim's matrix:

y= 0.298822 * r + .586815 * g + .114363 * b
v= .511545 * r - .428112 * g - .083434 * b
u= -.172486 * r - .338720 * g + .511206 * b

r = (int)((float)y + 1.370705 * (float)v - 191.45029);
g = (int)((float)y - 0.698001 * (float)v - 0.337633 * (float)u + 116.56116);
b = (int)((float)y + 1.732446 * (float)u - 237.7534);

*/

/* a couple notes about 'gamma fix' (dbm):
    right now, we just squash the input range to .75 then add 32 to
    normalize.  This is easy to do in assembler, and is being done in
    xx.asm.  We should use a lookup table and do a proper conversion,
    since we can do that on output (to RGB at least), but for some
    forgotten reason that was hard to do with YUV input, so I did this
    cheezy fix.  The squash has to occur to avoid too many outrange errors
    from occuring during hi-contrast material (that will slow down code),
    and because the results of these deltas are better with squash (tho
    would be better still with real gamma correction).
*/

#define GAMMA_FIX(x) x=(((x*3)/4) + 32)

//#define GAMMA_FIX(x) x=(((x*2)/3) + 42)
//#define GAMMA_FIX(x) { }
//#define GAMMA_FIX(x) x=(log10(x+50) * 200 - 300)

static U8 rgb_to_y(float r, float g, float b ) {

///    return ( (g*3)/4 + 25 ); // bullshit --replace with working functions

    int y;
    r += (float)191.45029;
    g -= (float)116.56116;
    b += (float)237.7534;
    y  = (int)(0.298822 * r + 0.586815 * g + 0.114363 * b);
    if(y<0) y=0;
    if(y>255) y=255;

/// scaling func to stop out-ranging -- fix with gamma correct:

    GAMMA_FIX(y);
    return y;
}

/* Gosh darn it, had to switch 'em again!  Those pesky V / U's */

static U8 rgb_to_u(float r, float g, float b ) {
///    return(128+((g-r)/2)); // bullshit --replace with working functions
    int v;
    r += (float)191.45029;
    g -= (float)116.56116;
    b += (float)237.7534;
    v  = (int)(0.511545 * r - 0.428112 * g - 0.083434 * b);
    if(v<0) v=0;
    if(v>255) v=255;
    return v;
}

static U8 rgb_to_v(float r, float g, float b ) {
///    return(128+((g-b)/2)); // bullshit --replace with working functions
    int u;
    r += (float)191.45029;
    g -= (float)116.56116;
    b += (float)237.7534;
    u  = (int)(-0.172486 * r - 0.338720 * g + 0.511206 * b);
    if(u<0) u=0;
    if(u>255) u=255;
    return u;
}


static void build_rgb2yvu() {          // build a table to convert rgb to y,v,&u
    I32 r,g,b,i;

    for(r=0;r<32;r++) {
        for(g=0;g<32;g++) {
            for(b=0;b<32;b++) {

                i=(r<<10)+(g<<5)+b;              // assume 555

                rgb2y[i]=rgb_to_y((float)(r<<3),(float)(g<<3),(float)(b<<3));
                rgb2v[i]=rgb_to_v((float)(r<<3),(float)(g<<3),(float)(b<<3));
                rgb2u[i]=rgb_to_u((float)(r<<3),(float)(g<<3),(float)(b<<3));
            }
		}
	}
}


static U32 encode(I32 diff,int deltasUsed) {
	I32 i,mini=0,err,min;

// d is in range -510 to +510

	min=1000000;
    for(i=deltasUsed-1;i>=0;i--) {
		err=abs(deltas[i]-diff);
        if(err<min) {
			min=err;
			mini=i;
/// DPF("  encode: new value %d\n",mini);
        }
	}
/// DPF("  encode: final value %d\n",mini);
	return(mini);
}

static I32 decode(U32 s) {

	return(deltas[s]);
}

static void build_tables(int deltasUsed) {
	I32 i,s,dvq,t;
 DPF("build_tables -- num=%d\n",deltasUsed);
	for(i=-511;i<512;i++) {				// for all possible dv's
		s=encode(i,deltasUsed);
		dvq=decode(s);
		t=(s<<28)+(dvq & 0xffff);
		xxtbl[i+512]=t;
	}
    build_rgb2yvu();
}

static void make_header(U8 *hdr,long width,long height,long type,long bits,long finfo,long framesize)
{
	int i,hdrSize;
	TR20FRAMEHEADER *fhdr = (TR20FRAMEHEADER *)hdr;

	fhdr->Type = (U8)type;
	fhdr->metaType = (U8)TMRT_METATYPE;
	fhdr->bits = (U8)bits;
	fhdr->Xsize = (U16)width;
	fhdr->Ysize = (U16)height;
	fhdr->Frameinfo = (U8)finfo;
	fhdr->FrameSize = framesize;

//	crypt it
	hdrSize = sizeof(TR20FRAMEHEADER);

	for (i = hdrSize; i > 1; i--)
		hdr[i-1] ^= hdr[i];

	hdrSize <<= 1;
	((TR20FRAMEHEADER *)hdr)->hdrSize = ((hdrSize | 1) << 4)|(hdrSize >> 4);
}

#if 0
static void conv16to8(U8 *p,int w,int h)
{
	int i,j;
	U16 *p16 = (U16 *)p;

	for(i=0; i<h; i++) {
		for(j=0; j<w; j++) {
            *p++ = ((*p16++ & 0x1F) << 3) +20 ;
		}
	}
}
#endif

/*
    dbm 10/96 -- pack up your troubles (2bit, 3bit, 4bit?) and return
    size of packed

    format = 2,3,4 (bits/pixel)

    Changed bitstream: now starts with 4-byte size (long, little endian)
*/


static long xx_pack(char *src, char *dest, long size, int format) {
    char *tdest;
    long theSize;

//#define TIME_TEST

#ifdef TIME_TEST
    static int countframe = 0;
    unsigned long   GAstart,GAstop;
    static unsigned long adder;
#endif
    if(format==4) {
         tdest=dest+4;
         theSize=size;
         pack4a_(src,tdest,theSize);  //mrn test or junky1
    } else if(format==2) {
        tdest=dest+4;
        theSize=size/2;
#ifdef TIME_TEST
        GKA_();
        GAstart = mrnvar1;
#endif
        pack2a_(src,tdest,size);

#ifdef TIME_TEST
        GKA_();
        GAstop = mrnvar1;
        adder += (GAstop - GAstart);
        countframe+=1;
#endif
        size /= 2;
    } else if(format==3) {
        tdest=dest+4;
        pack3a_(src,tdest,size);  
        size = (size * 6) / 8;
    } else {
        DPF ("xx_pack: nonsupported format\n");
    }

/// we need to encode size here for decompress!

    *(long *)dest = size;


#ifdef TIME_TEST
    if(countframe >= 100){
         FILE *fh;
         char foobar[128];
         sprintf(foobar,"Using *ASM* 2bit Time = %d avg per 100 frms\n",adder/countframe);
         fh = fopen("C:\\TMRN.OUT","a+");
         if(fh != NULL) {
             fputs(foobar,fh);
             fclose(fh);
         }
         adder = 0;
         countframe = 0;
    }//if
#endif


    return(size+4);
}


#ifdef CPACK
static long xx_pack_in_C(char *src, char *dest, long size, int format) {
    int i,t1,t2,t3,t4,t5,t6,t7,t8;
    char *tdest;

    if(format==4) {
        for(i=0;i<size;i++) {
            dest[i+4]=src[i];
        }
    } else if(format==2) {

        tdest=dest+4;
        for(i=0;i<size/2;i++) {
            t1=src[i*2] & 0x03;
            t2=src[i*2] >> 4;
            t3=src[i*2+1] & 0x03;
            t4=src[i*2+1] >> 4;
            *tdest++ = t1 + (t2<<2) + (t3<<4) + (t4<<6);
/// if(i==16) DPF("  at i=16, t's=%d %d %d %d, tdest=%x, tdest[-1]=%x, should be %x\n", t1,t2,t3,t4,tdest,*(tdest-1),t1 + (t2<<2) + (t3<<4) + (t4<<6));
        }
        size /= 2;

    } else if(format==3) {

        tdest=dest+4;
        for(i=0;i<size/4;i++) {

            t1=src[i*4] & 0x07;
            t2=src[i*4] >> 4;
            t3=src[i*4+1] & 0x07;
            t4=src[i*4+1] >> 4;
            t5=src[i*4+2] & 0x07;
            t6=src[i*4+2] >> 4;
            t7=src[i*4+3] & 0x07;
            t8=src[i*4+3] >> 4;

            *(long *)tdest =            /// note: little endian machines only!

                t1 + (t2<<3) + (t3<<6) + (t4<<9) +
                (t5<<12) + (t6<<15) + (t7<<18) + (t8<<21);

            tdest+=3;

        }
        size = (size * 6) / 8;

    } else {
        DPF ("xx_pack: nonsupported format\n");
    }
/// DPF("xx_pack size: %d src: %x dest: %x tdest: %x format: %d\n",size+4,src,dest,tdest,format);

/// we need to encode size here for decompress!

    *(long *)dest = size;

    return(size+4);
}
#endif

static void _setline(long width,long setval) {
DPF("TMRTXX_SETLINE %d %d",width,setval);
	clear_linebuf_xx_(width,setval);
}

int tmrtxx_xx(void *param,int w,int h,int format,int bits,int ntrp,char *src,char *dst)
{
	long why,whvu,ysize,vusize,packedsize;
	XXPARAM *rtparam = (XXPARAM *)param;
	long offset;
	char *tempbuffer;

//#define XX_TIME_TEST
#ifdef XX_TIME_TEST
    static int countframe = 0;
    unsigned long   GAstart,GAstop;
    static unsigned long adder;
#endif


//    if(rtparam->savebits != bits) {
    if(xxtbl_state != bits) {
		if(bits == 4) {
			INIT_DELTAS_4BIT
			rtparam->NumDeltasReallyUsed=16;
		} else if(bits == 3) {
			INIT_DELTAS_3BIT
			rtparam->NumDeltasReallyUsed=8;
		} else if(bits == 2) {
			INIT_DELTAS_2BIT
			rtparam->NumDeltasReallyUsed=4;
		} else rtparam->NumDeltasReallyUsed=0;
    
		build_tables(rtparam->NumDeltasReallyUsed);
//		rtparam->savebits = bits;
		xxtbl_state = bits;
	}

    if(ntrp) w = (w>>1);//&0xFFF0;
	if(bits == 4) tempbuffer = dst+sizeof(TR20FRAMEHEADER)+4;
	else tempbuffer = rtparam->tbuffer;

DPF("TMRTXX_XX %d %d %d %d %d %x %x",w,h,format,bits,ntrp,src,dst);

/// dbm -- use quality as a bitrate setting (0-10,000 = 2-4)

	why = w | (h<<16);
	whvu = (w>>2) | ((h>>2)<<16);
	ysize = (w*h)>>1;
	vusize = (w*h)>>5;


#ifdef XX_TIME_TEST
        GKA_();
        GAstart = mrnvar1;
#endif

///      conv16to8(hpInput,w,h);
	if(ntrp) _setline(w*4,0);
	else _setline(w*2,0);
	if(ntrp) offset = (h-1)*w*4;
	else offset = (h-1)*w*2;
	if(format == TMRT_RGB555) {
		if(ntrp) xxn_(src+offset,tempbuffer,why);
		else xx_(src+offset,tempbuffer,why);
	} else if(format == TMRT_YUY2) {
		if(ntrp) xxyuy2n_(src,tempbuffer,why);
		else xxyuy2_(src,tempbuffer,why);
	} else if(format == TMRT_YUV9) {
		if(ntrp) xxyvu9n_(src,tempbuffer,why);
		else xxyvu9_(src,tempbuffer,why);
	}

/// setup for vu -- clear line grey, send xxv/xxu same data as xx

   	if(ntrp) _setline(w*4,0x80);
   	else _setline(w*2,0x80);
	if(format == TMRT_RGB555) {
		if(ntrp) xxvn_(src+offset,tempbuffer+ysize,whvu);
        else xxv_(src+offset,tempbuffer+ysize,whvu);
	} else if(format == TMRT_YUY2) {
		if(ntrp) xxvyuy2n_(src,tempbuffer+ysize,whvu);
        else xxvyuy2_(src,tempbuffer+ysize,whvu);
	} else if(format == TMRT_YUV9) {
		if(ntrp) xxuyvu9n_(src+ysize*4+vusize*4,tempbuffer+ysize,whvu);
        else xxuyvu9_(src+ysize*2+vusize*2,tempbuffer+ysize,whvu);
	}

   	if(ntrp) _setline(w*4,0x80);
   	else _setline(w*2,0x80);
	if(format == TMRT_RGB555) {
		if(ntrp) xxun_(src+offset,tempbuffer+ysize+vusize,whvu);
		else xxu_(src+offset,tempbuffer+ysize+vusize,whvu);
	} else if(format == TMRT_YUY2) {
		if(ntrp) xxuyuy2n_(src,tempbuffer+ysize+vusize,whvu);
		else xxuyuy2_(src,tempbuffer+ysize+vusize,whvu);
	} else if(format == TMRT_YUV9) {
		if(ntrp) xxvyvu9n_(src+ysize*4,tempbuffer+ysize+vusize,whvu);
		else xxvyvu9_(src+ysize*2,tempbuffer+ysize+vusize,whvu);
	}

	if(bits == 4) {
		long *lp = (long *)(tempbuffer-4);
		
		*lp = packedsize = ysize+ vusize*2 + 4;
	} else {
		packedsize=xx_pack(tempbuffer,dst+sizeof(TR20FRAMEHEADER),ysize+vusize*2,bits);
	}


#ifdef XX_TIME_TEST
        GKA_();
        GAstop = mrnvar1;
        adder += (GAstop - GAstart);
        countframe+=1;
#endif

#ifdef XX_TIME_TEST
    if(countframe >= 100){
         FILE *fh;
         char foobar[128];
         sprintf(foobar,"New Version Comp Bits = %d NTRP = %d Time = %d kilo cycles; avg per 100 frms\n",bits,ntrp,adder/countframe);
         fh = fopen("C:\\RTxx.OUT","a+");
         if(fh != NULL) {
             fputs(foobar,fh);
             fclose(fh);
         }
         adder = 0;
         countframe = 0;
    }//if
#endif

	make_header(dst,ntrp?w*2:w,h,TIGER_BETA,bits,ntrp,sizeof(TR20FRAMEHEADER)+packedsize);
    	
	return sizeof(TR20FRAMEHEADER)+packedsize;
}

int tmrtxx_query_paramsize(void)
{
DPF("TMRTXX_QUERY_PARAMSIZE %d",sizeof(XXPARAM));
	return(sizeof(XXPARAM));
}

int tmrtxx_init(void *param)
{
	XXPARAM *rtparam = (XXPARAM *)param;
DPF("TMRTXX_INIT %x",param);

//	rtparam->savebits = -1;
	return 0;
}

int tmrtxx_begin_xx(void *param,int width,int height) {
	XXPARAM *rtparam = (XXPARAM *)param;

DPF("TMRTXX_BEGIN_XX %x %dx%d\n",param,width,height);

	rtparam->width = width;
	rtparam->height = height;

	//malloc enough space for 4-bit compression
	rtparam->tbuffer = malloc((width*height*9)/16 + 16);

	if(!rtparam->tbuffer) return 1;
	return 0;
}

void tmrtxx_end_xx(void *param) {
	XXPARAM *rtparam = (XXPARAM *)param;

	if(rtparam->tbuffer) free(rtparam->tbuffer);
	rtparam->tbuffer = NULL;
}

void tmrtxx_free(void *param)
{
//free the memory pointed at by rtparam
	XXPARAM *rtparam = (XXPARAM *)param;
DPF("TMRTXX_FREE %x",param);
}
