/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CTimeShiftEvent.h"
#include <assert.h>

#define OLE_MONDAY	2
#define OLE_FRIDAY	6


const char *gsDayConversion[] = 

{

	"Sun",

	"Mon",

	"Tue",

	"Wed",

	"Thu",

	"Fri",

	"Sat",

	"Mon-Fri",

	"every Sun",

	"every Mon",

	"every Tue",

	"every Wed",

	"every Thu",

	"every Fri",

	"every Sat"

};






CTimeShiftEvent::CTimeShiftEvent(void)
{
	mChannel = 2;
	mStartTime = COleDateTime::GetCurrentTime();
	mEndTime = COleDateTime::GetCurrentTime();
	mDate = COleDateTime::GetCurrentTime();
	mDay = (Day_t) (mDate.GetDayOfWeek() - 1);
	mAppendDateAndTime = false;						///	dbm 3/31/02 -- changed some defaults
	mAudioFrequency = 44100;
	mAudioBits = 16;
	mAudioChannels = 1;
	mVideoWidth = 320;
	mVideoHeight = 240;
	mFPS = 30;
	mVideoQuality = 0;
	mVideoSource = IVideoTuner::BaseBand_Composite;

	mBrightness = 128;

	mContrast = 128;

	mHue = 128;

	mSaturation = 128;

	mVolume = 50;





}

nmr_string	

CTimeShiftEvent::CreateCondensedString(void)

{

	nmr_string	result,filename,date,audio,video;

	char		buf[1024];



	filename = mFilename;

	if (filename.length() > 18)

		filename = ((const char *)mFilename) + mFilename.length() - 18;

	date = gsDayConversion[(int)mDay];

	if (mDay <= Saturday)

		date += mDate.Format(" %m/%d");



	audio = "["; audio += (mAudioFrequency / 1000); audio += "khz ";

	audio += mAudioBits; audio += "/";

	audio += (mAudioChannels == 1 ? "m" : "s");

	audio += "]";



	video = "["; video += mVideoWidth; video += "x"; video += mVideoHeight;

	video += " "; video += mFPS; video += "fps ";

	video += mVideoQuality; video += "q]";	



	// SCCS would interpret this as a keyword if it was a single constant

	nmr_string fs = "%I:%M";

	fs += "%p";



	sprintf(buf,"%3d %18s %s-%s %-9s %-12s %-22s",

		mChannel,

		(const char *)filename,

		(const char *)mStartTime.Format(fs),

		(const char *)mEndTime.Format(fs),

		(const char *)date,

		(const char *)audio,

		(const char *)video

		);



	result = buf;



	return result;

}



COleDateTime 
CTimeShiftEvent::NextActivationTime(void)
{
	COleDateTimeSpan	one_day(1,0,0,0);
	COleDateTime	result;
	COleDateTime	ct = COleDateTime::GetCurrentTime();

	if (mDay <= Saturday)
		{
		result.SetDateTime(
			mDate.GetYear(),mDate.GetMonth(),mDate.GetDay(),
			mStartTime.GetHour(),mStartTime.GetMinute(),mStartTime.GetSecond()
			);
		}
	else
		{
		result.SetDateTime(ct.GetYear(),ct.GetMonth(),ct.GetDay(),

			mStartTime.GetHour(),mStartTime.GetMinute(),mStartTime.GetSecond());
		if (result < ct)
			result += one_day;
		if (mDay == MonThruFri)
			{
			while (result.GetDayOfWeek() < OLE_MONDAY || 
										result.GetDayOfWeek() > OLE_FRIDAY)
				result += one_day;

			}
		else
			{
			while (result.GetDayOfWeek() != mDay - MonThruFri)
				result += one_day;
			}
		}

	return result;
}

COleDateTimeSpan 
CTimeShiftEvent::Duration(void)
{
	COleDateTimeSpan	one_day(1,0,0,0);

	COleDateTime 	e(
		mStartTime.GetYear(),mStartTime.GetMonth(),mStartTime.GetDay(),
		mEndTime.GetHour(),mEndTime.GetMinute(),mEndTime.GetSecond()
		);
	if (e < mStartTime)
		e += one_day;

	assert(e >= mStartTime);
	return (e - mStartTime);
}

int
CTimeShiftEvent::operator==(const CTimeShiftEvent &e)
{
	return (
		(mChannel 				== e.mChannel)			&&
		(mStartTime 			== e.mStartTime)		&&
		(mEndTime 				== e.mEndTime)			&&
		(mDay 					== e.mDay)				&&
		(mDate 					== e.mDate)				&&
		(mFilename 				== e.mFilename)			&&
		(mAppendDateAndTime 	== e.mAppendDateAndTime)&&
		(mAudioFrequency 		== e.mAudioFrequency)	&&
		(mAudioBits 			== e.mAudioBits)		&&
		(mAudioChannels 		== e.mAudioChannels)	&&
		(mVideoWidth 			== e.mVideoWidth)		&&
		(mVideoHeight 			== e.mVideoHeight)		&&
		(mFPS 					== e.mFPS)				&&
		(mVideoQuality 			== e.mVideoQuality)		&&
		(mVideoSource			== e.mVideoSource)		&&

		(mBrightness			== e.mBrightness)		&&

		(mContrast				== e.mContrast)			&&

		(mHue					== e.mHue)				&&

		(mSaturation			== e.mSaturation)		&&

		(mVolume				== e.mVolume)

		);
}


void OutputDate(ostream &o,const COleDateTime &d)

{

	o << d.GetYear() << endl;

	o << d.GetMonth() << endl;

	o << d.GetDay() << endl;

	o << d.GetHour() << endl;

	o << d.GetMinute() << endl;

	o << d.GetSecond() << endl;

}


void InputDate(istream &i,COleDateTime &d)

{

	int year,month,day,hour,minute,second;



	i >> year >> month >> day >> hour >> minute >> second;

	d.SetDateTime(year,month,day,hour,minute,second);

}



ostream& operator<<(ostream &o,const CTimeShiftEvent &e)
{
	o << e.mChannel << endl;

	OutputDate(o,e.mStartTime);

	OutputDate(o,e.mEndTime);
	o << ((int)e.mDay) << endl;
	OutputDate(o,e.mDate);

	o << e.mAppendDateAndTime << endl;
	o << e.mAudioFrequency << endl;
	o << e.mAudioBits << endl;
	o << e.mAudioChannels << endl;
	o << e.mVideoWidth << endl;
	o << e.mVideoHeight << endl;
	o << e.mFPS << endl;
	o << e.mVideoQuality << endl;
	o << ((int)e.mVideoSource) << endl;

	o << e.mBrightness << endl;

	o << e.mContrast << endl;

	o << e.mHue << endl;

	o << e.mSaturation << endl;

	o << e.mVolume << endl;

	o << e.mFilename;
	return o;
}

istream& operator>>(istream &i,CTimeShiftEvent &e)
{

	int ii;
	i >> e.mChannel;

	InputDate(i,e.mStartTime);

	InputDate(i,e.mEndTime);
	i >> ii;

	e.mDay = (CTimeShiftEvent::Day_t)ii;
	InputDate(i,e.mDate);

	i >> e.mAppendDateAndTime;
	i >> e.mAudioFrequency;
	i >> e.mAudioBits;
	i >> e.mAudioChannels;
	i >> e.mVideoWidth;
	i >> e.mVideoHeight;
	i >> e.mFPS;
	i >> e.mVideoQuality;
	i >> ii;

	e.mVideoSource = (IVideoTuner::VideoSource_t)ii;;

	i >> e.mBrightness;

	i >> e.mContrast;

	i >> e.mHue;

	i >> e.mSaturation;

	i >> e.mVolume;

	i >> e.mFilename; // eats end ofline

	i >> e.mFilename;
	return i;
}

