/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CTIMESHIFTMANAGER_H_
#define CTIMESHIFTMANAGER_H_

#include "CTimeShiftEvent.h"
#include "CCMutex.h"
#include <vector>
#include <iostream>
using namespace std;
#include <afxdisp.h>


class CTimeShiftManager
{
public:
	typedef vector<CTimeShiftEvent>	EventList_t;
	typedef enum { Nothing, Warning, StartCapture,StopCapture } Action_t;
private:
	CCMutex		mMutex;
	EventList_t	mEventList;
	int					mNextEventOfInterest;
	COleDateTime		mStartTimeOfInterest;
	COleDateTimeSpan	mDurationOfInterest;
	bool		mWarned;
	bool		mRunning;
	bool		mAdjustingList;

	void		FigureNextEventOfInterest(COleDateTime *current_time = 0);
	Action_t	_Query(CTimeShiftEvent &event,COleDateTime *current_time = 0,

					int warning_limit = 5);


public:
	CTimeShiftManager(void);

	Action_t	Query(CTimeShiftEvent &event,int warning_limit = 5);
	Action_t	Query(CTimeShiftEvent &event,COleDateTime current_time,

					int warning_limit = 5);

	void		BeginListAdjustment(void);
	void		EndListAdjustment(void);
	void		AddEvent(const CTimeShiftEvent &event);
	void		RemoveEvent(const CTimeShiftEvent &event);
	void		RemoveEvent(int index);

	void		RemoveOldEvents(COleDateTime *current_time = 0);

	void		Clear(void);
	EventList_t	GetEventList(void);
	bool		IsThereANextEvent(void);

	

	void		Reset(void);

friend ostream& operator<<(ostream &o,CTimeShiftManager &m);
friend istream& operator>>(istream &o,CTimeShiftManager &m);
};


#endif
