/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// TrueCapDoc.h : interface of the CTrueCapDoc class

//

/////////////////////////////////////////////////////////////////////////////

#include "CCap.h"

#include "CPlayerShell.h"

#include "CAudioMixer.h"

#include "IVideoTuner.h"

#include "IEventEditSupervisor.h"



class IScanProgressFeedback;

class CTrueCapGUIController;

class CTrueCapView;

class CMainFrame;

class CTimeShiftEvent;



#include <vector>
using namespace std;


extern int gDirectDrawAvailable;

extern int gNT;



class CTrueCapDoc : public CDocument, public IEventEditSupervisor

{

protected: // create from serialization only

	CTrueCapDoc();

	DECLARE_DYNCREATE(CTrueCapDoc)



// Attributes

public:



// Operations

public:

	typedef enum { Capture = 0, Playback = 1 } TrueCapMode_t;



// Overrides

	// ClassWizard generated virtual function overrides

	//{{AFX_VIRTUAL(CTrueCapDoc)

	public:

	virtual BOOL OnNewDocument();

	virtual void Serialize(CArchive& ar);

	//}}AFX_VIRTUAL



// Implementation

public:

	virtual ~CTrueCapDoc();

#ifdef _DEBUG

	virtual void AssertValid() const;

	virtual void Dump(CDumpContext& dc) const;

#endif



	void	ViewDestroying(void);

	CCap&	GetCaptureWindow(void);

	int		CreateCaptureWindow(void);

	void	DestroyCaptureWindow(void);

	void	ChangeVolume(int v);

	TrueCapMode_t	GetMode(void);

	void	SetCaptureMode(void);

	void	SetPlaybackMode(int firsttime = 0);

	void	SetView(CTrueCapView *v);



	void	Play(void);

	void	Stop(void);

	void	FastForward(void);

	void	Rewind(void);

	void	Pause(void);



	CString	GetPlaybackFileName(void);

	CString GetCaptureFileName(void);

	void	SetPlaybackFileName(const char *name);

	void	SetCaptureFileName(const char *name);

	int		GetCurrentDisplayFrame(void);

	int		GetCurrentFrame(void);

	int		IsPlaying(void);

	int		IsPaused(void);



	void	DoCapture(CString tempfilename="");

	int		IsCapturing(void);

	void	StopCapture(void);

	void	SetAudioConfiguration(int sample_size,int channels,int frequency,int volume);

	void	GetAudioConfiguration(int &sample_size,int &channels,int &frequency,int &volume);

	void	SetVideoConfiguration(int width,int height,int fps,int quality);

	void	GetVideoConfiguration(int &width,int &height,int &fps,int &quality);



	virtual void	ConfigureCaptureDriverSource(IVideoTuner::VideoSource_t &source,

		U8 &brightness,U8 &contrast,U8 &hue,U8 &saturation);

	virtual void	SetTunerState(IVideoTuner::VideoSource_t source,

		U8 brightness,U8 contrast,U8 hue,U8 saturation,IVideoTuner::Channel_t channel);

	virtual int		GetTunerState(IVideoTuner::VideoSource_t &source,

		U8 &brightness,U8 &contrast,U8 &hue,U8 &saturation,IVideoTuner::Channel_t &channel);

	virtual void	SetTunerState(IVideoTuner::VideoSource_t source,

		U8 brightness,U8 contrast,U8 hue,U8 saturation);

	virtual int		GetTunerState(IVideoTuner::VideoSource_t &source,

		U8 &brightness,U8 &contrast,U8 &hue,U8 &saturation);



	void	ShowVideo(BOOL s);

	int		GetCanCapture(void);

	void	RefreshFilename(void);



	void	WindowPosChanged(void);



	int		GrabMPVE(void);

	void	ReleaseMPVE(void);

	void	DisplayCurrentFrame(void);

	void	SetAndDisplayFrame(int f);

	int		GetNumVideoFrames(void);

	int		PrepareForSeek(void);



	int		BigImage(void);

	void	GoFullScreen(int f);

	int		IsPageFlipping(void);

	void	ShowStatusBar(int s);



	void	AdvanceFrame(void);

	void	RetreatFrame(void);

	void	AdvanceTime(int milliseconds);

	void	RetreatTime(int milliseconds);



	int		HasTuner(void);

	int		GetChannel(void);

	int		SetChannel(int channel);

	IVideoTuner	*GetVideoTuner(void);



	void	StopTimeShiftCapture(void);

	void	StartTimeShiftCapture(const CTimeShiftEvent &e);

	void	ReloadEvents(void);



	int			SetKeyString(CString s);

	CString		GetKeyString(void);



	IVideoTuner::ChannelList_t BuildScanList(IScanProgressFeedback &);

	void		ToggleCC(void);

	void		ToggleCCChannel(void);

	void		CaptureFrame(void);

	void		ToggleFullScreen(void);

	bool		IsFullScreenCapture(void);

	void		ConfigureCC(void);

	void		SkipCommercial(void);

	void		DoTimedRecord(void);

protected:

	CString				mKeyString;

	int					mDeferedEventLoad;

	CAudioMixer			mMixer;

	CCap				mCap;

	CTrueCapGUIController	*mGUI;

	CTrueCapView		*mView;

	CMainFrame			*mMainFrame;

	CPlayerShell		mPlayer;

	TrueCapMode_t		mMode;

	CString				mPlaybackFile;

	CString				mCaptureFile;

	CString				mPostCaptureFilename;

	int					mCurrentDisplayFrame;

	int					mCanCapture;

	int					mFirstModeSet;



public:

	int					mAudioCapture_SampleSize;

	int					mAudioCapture_Channels;

	int					mAudioCapture_Frequency;

	int					mAudioCapture_Volume;



	int					mVideoCapture_Width;

	int					mVideoCapture_Height;

	int					mVideoCapture_FPS;

	int					mVideoCapture_Quality;

protected:



	RECT				mWindowPosition;

	void				SendCaptureOptions(void);

	void				DDTest(void);



	IVideoTuner			*mVideoTuner;



	void				_HaltCapture(void);

	void				_StartCapture(void);



	int					mSingleFieldInput;

	bool				mTryingToCaptureFrame;



	bool				mFullScreen;

	int					mCCChannel;

	int					mCCOn;



// Generated message map functions

protected:

	//{{AFX_MSG(CTrueCapDoc)

	afx_msg void OnEditCopy();

	afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);

	afx_msg void OnOptionsInvertaudio();

	afx_msg void OnUpdateOptionsInvertaudio(CCmdUI* pCmdUI);

	afx_msg void OnOptionsHardwareSinglefieldinput();

	afx_msg void OnUpdateOptionsHardwareSinglefieldinput(CCmdUI* pCmdUI);

	afx_msg void OnOptionsClosedcaptionChannel1();

	afx_msg void OnUpdateOptionsClosedcaptionChannel1(CCmdUI* pCmdUI);

	afx_msg void OnOptionsClosedcaptionChannel2();

	afx_msg void OnUpdateOptionsClosedcaptionChannel2(CCmdUI* pCmdUI);

	afx_msg void OnOptionsClosedcaptionOff();

	afx_msg void OnUpdateOptionsClosedcaptionOff(CCmdUI* pCmdUI);

	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

};



/////////////////////////////////////////////////////////////////////////////

