/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// TrueCapView.cpp : implementation of the CTrueCapView class

//



#include "stdafx.h"

#include "TrueCap.h"

#include "TruecapGlobals.h"

#include "TrueCapDoc.h"

#include "TrueCapView.h"

#include "CDuckFile.h"

#include "CDuckBuffer.h"

#include "CDuckWinblitScreen.h"

#include "CDuckImage.h"

#include "CDuckVideoStream.h"

#include "CDuckContext.h"

#include "CAudioSettingsDialog.h"

#include "CVideoSettingsDialog.h"

#include "CEventListDialog.h"

#include "CRenameDialog.h"

#include "CChannelScanDialog.h"

#include "CUpgradeDialog.h"

#include "CTimedRecordDialog.h"

#include "CDialogNAG2.h"

#include "CDialogNAG3.h"

#include <vfw.h>

#include "danbug.h"


#ifdef _DEBUG

#define new DEBUG_NEW

#undef THIS_FILE

static char THIS_FILE[] = __FILE__;

#endif



#define DIGIT_LOCATION_0	102

#define DIGIT_LOCATION_1	113

#define DIGIT_LOCATION_2	124

#define DIGIT_LOCATION_Y	287

#define DIGIT_WIDTH			10

#define DIGIT_HEIGHT		14



int gIconic = 0;



/////////////////////////////////////////////////////////////////////////////

// CTrueCapView



IMPLEMENT_DYNCREATE(CTrueCapView, CView)



BEGIN_MESSAGE_MAP(CTrueCapView, CView)

	//{{AFX_MSG_MAP(CTrueCapView)

	ON_WM_DESTROY()

	ON_WM_KEYDOWN()

	ON_WM_CREATE()

	ON_COMMAND(ID_OPTIONS_AUDIO, OnOptionsAudio)

	ON_UPDATE_COMMAND_UI(ID_OPTIONS_AUDIO, OnUpdateOptionsAudio)

	ON_COMMAND(ID_OPTIONS_VIDEO, OnOptionsVideo)

	ON_UPDATE_COMMAND_UI(ID_OPTIONS_VIDEO, OnUpdateOptionsVideo)

	ON_COMMAND(ID_FILE_SETCAPTUREFILE, OnFileSetcapturefile)

	ON_UPDATE_COMMAND_UI(ID_FILE_SETCAPTUREFILE, OnUpdateFileSetcapturefile)

	ON_COMMAND(ID_FILE_SETPLAYBACKFILE, OnFileSetplaybackfile)

	ON_UPDATE_COMMAND_UI(ID_FILE_SETPLAYBACKFILE, OnUpdateFileSetplaybackfile)

	ON_COMMAND(ID_FILE_RENAMEFILE, OnFileRenamefile)

	ON_UPDATE_COMMAND_UI(ID_FILE_RENAMEFILE, OnUpdateFileRenamefile)

	ON_WM_ERASEBKGND()

	ON_WM_HSCROLL()

	ON_WM_SETCURSOR()

	ON_WM_TIMER()

	ON_COMMAND(ID_OPTIONS_EDITEVENTS, OnOptionsEditevents)

	ON_UPDATE_COMMAND_UI(ID_OPTIONS_EDITEVENTS, OnUpdateOptionsEditevents)

	ON_COMMAND(ID_OPTIONS_CHANNELSCAN, OnOptionsChannelscan)

	ON_UPDATE_COMMAND_UI(ID_OPTIONS_CHANNELSCAN, OnUpdateOptionsChannelscan)

	ON_COMMAND(ID_FILE_UPGRADE, OnFileUpgrade)

	ON_UPDATE_COMMAND_UI(ID_FILE_UPGRADE, OnUpdateFileUpgrade)

	ON_WM_DROPFILES()

	ON_WM_KEYUP()

	ON_COMMAND(ID_OPTIONS_TIMEDRECORD, OnOptionsTimedrecord)

	ON_UPDATE_COMMAND_UI(ID_OPTIONS_TIMEDRECORD, OnUpdateOptionsTimedrecord)

	ON_WM_RBUTTONDOWN()

	//}}AFX_MSG_MAP

	// Standard printing commands

	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)

	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)

	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)

END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////

// CTrueCapView construction/destruction



CTrueCapView::CTrueCapView()

:

	mPLAYBACK_PlayButton	(IDB_PL_PLAY_NM,	IDB_PL_PLAY_AC,		IDB_PL_PLAY_HL),

	mPLAYBACK_StopButton	(IDB_PL_STOP_NM,	IDB_PL_STOP_AC,		IDB_PL_STOP_HL),

	mPLAYBACK_VCRButton		(IDB_PL_ROCK_T,		IDB_PL_ROCK_1,		IDB_PL_ROCK_T),

	mPLAYBACK_SetFileButton	(IDB_PL_SETFILE_NM,	IDB_PL_SETFILE_AC,	IDB_PL_SETFILE_HL),

	mPLAYBACK_RenameButton	(IDB_PL_RENAME_NM,	IDB_PL_RENAME_AC,	IDB_PL_RENAME_HL),

	mPLAYBACK_FullscrnToggle(IDB_PL_WINDOW_AC,	IDB_PL_WINDOW_HL,	IDB_PL_WINDOW_AC),

	mPLAYBACK_SkipButton	(IDB_PL_SKIP_NM,	IDB_PL_SKIP_AC,		IDB_PL_SKIP_HL),

	mCAPTURE_AudioButton	(IDB_CP_AUDIO_NM,	IDB_CP_AUDIO_AC,	IDB_CP_AUDIO_HL),

	mCAPTURE_RecordButton	(IDB_CP_RECORD_NM,	IDB_CP_RECORD_AC,	IDB_CP_RECORD_HL),

	mCAPTURE_SetFileButton	(IDB_CP_SETFILE_NM,	IDB_CP_SETFILE_AC,	IDB_CP_SETFILE_HL),

	mCAPTURE_TVButton		(IDB_CP_ROCK_T,		IDB_CP_ROCK_1,		IDB_CP_ROCK_T),

	mCAPTURE_VideoButton	(IDB_CP_VIDEO_NM,	IDB_CP_VIDEO_AC,	IDB_CP_VIDEO_HL),

	mCAPTURE_StillButton	(IDB_CP_STILL_NM,	IDB_CP_STILL_AC,	IDB_CP_STILL_HL),

	mCAPTURE_FullscrnToggle (IDB_PL_WINDOW_AC,	IDB_PL_WINDOW_HL,	IDB_PL_WINDOW_AC),

	mCAPTURE_1				(IDB_CP_1_NM,		IDB_CP_1_AC,		IDB_CP_1_HL),

	mCAPTURE_2				(IDB_CP_2_NM,		IDB_CP_2_AC,		IDB_CP_2_HL),

	mCAPTURE_3				(IDB_CP_3_NM,		IDB_CP_3_AC,		IDB_CP_3_HL),

	mCAPTURE_4				(IDB_CP_4_NM,		IDB_CP_4_AC,		IDB_CP_4_HL),

	mCAPTURE_5				(IDB_CP_5_NM,		IDB_CP_5_AC,		IDB_CP_5_HL),

	mCAPTURE_6				(IDB_CP_6_NM,		IDB_CP_6_AC,		IDB_CP_6_HL),

	mCAPTURE_7				(IDB_CP_7_NM,		IDB_CP_7_AC,		IDB_CP_7_HL),

	mCAPTURE_8				(IDB_CP_8_NM,		IDB_CP_8_AC,		IDB_CP_8_HL),

	mCAPTURE_9				(IDB_CP_9_NM,		IDB_CP_9_AC,		IDB_CP_9_HL),

	mCAPTURE_0				(IDB_CP_0_NM,		IDB_CP_0_AC,		IDB_CP_0_HL),

	mCAPTURE_Up				(IDB_CP_UP_NM,		IDB_CP_UP_HL,		IDB_CP_UP_HL),

	mCAPTURE_Down			(IDB_CP_DOWN_NM,	IDB_CP_DOWN_HL,		IDB_CP_DOWN_HL),

	mCAPTURE_Enter			(IDB_CP_ENTER_NM,	IDB_CP_ENTER_AC,	IDB_CP_ENTER_HL)

{



	// TODO: add construction code here

	mTimedRecord = false;

	mShiftKey = false;

	mLastEffect = DROPEFFECT_NONE;

	mDropCache = 0;

	mCaptureFullScreen = false;

	mInhibitBlit = 1;

	mChannelColor = 1;

	mChannelTimerActive = 0;

	mPendingChannel = 0;

	char s[2049];

	::GetCurrentDirectory(2048,s);

	mDirectory = s;

	mLastPlaybackBitmap = &mPlaybackBackgroundSmall;

	mFirstUpdate = 1;

	mCaptureBackground.LoadBitmap(IDB_CP_BACKGROUND);

	mPlaybackBackground.LoadBitmap(IDB_PL_BACKGROUND);

	mCaptureBackgroundSmall.LoadBitmap(IDB_CP_BACKGROUND_SMALL);

	mPlaybackBackgroundSmall.LoadBitmap(IDB_PL_BACKGROUND_SMALL);

	mDigits[0].LoadBitmap(IDB_CP_CHAN0);

	mDigits[1].LoadBitmap(IDB_CP_CHAN1);

	mDigits[2].LoadBitmap(IDB_CP_CHAN2);

	mDigits[3].LoadBitmap(IDB_CP_CHAN3);

	mDigits[4].LoadBitmap(IDB_CP_CHAN4);

	mDigits[5].LoadBitmap(IDB_CP_CHAN5);

	mDigits[6].LoadBitmap(IDB_CP_CHAN6);

	mDigits[7].LoadBitmap(IDB_CP_CHAN7);

	mDigits[8].LoadBitmap(IDB_CP_CHAN8);

	mDigits[9].LoadBitmap(IDB_CP_CHAN9);

	mNoTunerBitmap.LoadBitmap(IDB_CP_NOTUNER);

	mLogoBitmap.LoadBitmap(IDB_LOGO);

	mCAPTURE_RecordButton.SetAltSet(IDB_CP_STOP_NM,IDB_CP_STOP_AC,IDB_CP_STOP_HL);

	mPLAYBACK_PlayButton.SetAltSet(IDB_PL_PAUSE_NM,IDB_PL_PAUSE_AC,IDB_PL_PAUSE_HL);

	mPLAYBACK_FullscrnToggle.SetAltSet(IDB_PL_FULL_AC,IDB_PL_FULL_HL,IDB_PL_FULL_AC);

	mPlayFullscreen = AfxGetApp()->GetProfileInt("DuckVision","FullscreenFlag",0);

	if (!gDirectDrawAvailable)

		mPlayFullscreen = 0;



	int siz = AfxGetApp()->GetProfileInt("DuckVision","ChannelListSize",0);

	for (int x = 0; x < siz; ++x)

		{

		char buf[32];

		sprintf(buf,"c%d",x);

		int chan = AfxGetApp()->GetProfileInt("DuckVision",buf,0);

		if (chan)

			mChannelList.push_back(chan);

		}



	LoadEvents();

}



void

CTrueCapView::LoadEvents(void)

{

	::LoadEvents(mDirectory + EVENTFILE,mTimeShiftManager);

}





CTrueCapView::~CTrueCapView()

{

	if (gNag)

		{

		CDialogNAG3 nag;

		nag.DoModal();

		}

	AfxGetApp()->WriteProfileInt("DuckVision","ChannelListSize",mChannelList.size());

	for (int x = 0; x < mChannelList.size(); ++x)

		{

		char buf[32];

		sprintf(buf,"c%d",x);

		AfxGetApp()->WriteProfileInt("DuckVision",buf,mChannelList[x]);

		}



	AfxGetApp()->WriteProfileInt("DuckVision","FullscreenFlag",mPlayFullscreen);

}







BOOL CTrueCapView::PreCreateWindow(CREATESTRUCT& cs)

{

	static char *window_name="_DuckVision_";

	// TODO: Modify the Window class or styles here by modifying

	//  the CREATESTRUCT cs

	cs.lpszName = window_name;

	//cs.style |= WS_CLIPCHILDREN | WS_CLIPSIBLINGS; //WS_CAPTION | WS_OVERLAPPED | WS_MINIMIZEBOX | WS_MAXIMIZEBOX; //0;//WS_OVERLAPPEDWINDOW;

	return CView::PreCreateWindow(cs);

}





/////////////////////////////////////////////////////////////////////////////

// CTrueCapView drawing



void

CTrueCapView::InvalidateChannels(void)

{

	if (mCaptureFullScreen) return;

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() == CTrueCapDoc::Playback || (!mPendingChannel && mChannelTimerActive))

		{

		mPendingChannel = 0;

		KillTimer(2);

		mChannelTimerActive = 0;

		}

	else if (mPendingChannel && !mChannelTimerActive)

		{

		mChannelColor = 1;

		SetTimer(2,500,0);

		mChannelTimerActive = 1;

		}

	CRect	r(DIGIT_LOCATION_0 - 5,DIGIT_LOCATION_Y - 5,

			DIGIT_LOCATION_2 + DIGIT_WIDTH + 5,DIGIT_LOCATION_Y + DIGIT_HEIGHT + 5);

	InvalidateRect(&r,TRUE);

	UpdateWindow();

}



void

CTrueCapView::_DrawChannel(int channel)

{

	static int	xslots[] = {DIGIT_LOCATION_0,DIGIT_LOCATION_1,DIGIT_LOCATION_2};

	int c		= channel;

	int	force	= 0;

	for (int x = 100,slot = 0; x > 0; x /= 10, ++slot)

		{

		int r = c / x;

		c -= (r * x);

		if (r > 9 || r < 0)

			r = 0;

		if (r || force || (x == 1))

			{

			assert((r >= 0) && (r <= 9));

			assert((slot >= 0) && (slot <= 2));

			force = 1;

			CBitmap	*tmp	= 0;

			CDC		*pDC	= GetDC();

			CDC		dcMem;

			dcMem.CreateCompatibleDC(pDC);

			tmp = dcMem.SelectObject(&(mDigits[r]));

			dcMem.SetMapMode(pDC->GetMapMode());

			pDC->BitBlt(xslots[slot],DIGIT_LOCATION_Y,DIGIT_WIDTH,DIGIT_HEIGHT,

				&dcMem,0,0,SRCCOPY);

			dcMem.SelectObject(tmp);

			ReleaseDC(pDC);

			}

		}

}



void

CTrueCapView::DrawChannel(void)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (!pDoc->HasTuner())

		{

		CBitmap	*tmp	= 0;

		CDC		*pDC	= GetDC();

		CDC		dcMem;

		dcMem.CreateCompatibleDC(pDC);

		tmp = dcMem.SelectObject(&mNoTunerBitmap);

		dcMem.SetMapMode(pDC->GetMapMode());

		pDC->BitBlt(DIGIT_LOCATION_0,DIGIT_LOCATION_Y,32,15,

				&dcMem,0,0,SRCCOPY);

		dcMem.SelectObject(tmp);

		ReleaseDC(pDC);

		}

	else if (mPendingChannel)

		{

		_DrawChannel(mPendingChannel);

		CDC		*pDC	= GetDC();

		CRect	r(DIGIT_LOCATION_0 - 1,DIGIT_LOCATION_Y - 3,

			DIGIT_LOCATION_2 + DIGIT_WIDTH + 1,DIGIT_LOCATION_Y + DIGIT_HEIGHT + 3);

		CBrush	b(mChannelColor ? RGB(255,0,0) : RGB(0,0,0));

		mChannelColor = !mChannelColor;

		pDC->FrameRect(&r,&b);

		ReleaseDC(pDC);

		}

	else

		{

		_DrawChannel(pDoc->GetChannel());

		}

}



void

CTrueCapView::BlitStaticLogo(void)

{

	CBitmap	*tmp = 0;

	CDC	*pDC = GetDC();

	CDC	 dcMem;

	dcMem.CreateCompatibleDC(pDC);

	RECT r;

	GetClientRect(&r);



	tmp = dcMem.SelectObject(&mLogoBitmap);

	dcMem.SetMapMode(pDC->GetMapMode());

	pDC->BitBlt(VIDEO_X_OFFSET,VIDEO_Y_OFFSET,320,240,

			&dcMem,0,0,SRCCOPY);

	dcMem.SelectObject(tmp);

	ReleaseDC(pDC);

}



void

CTrueCapView::DrawStaticFrame()

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if ((!mInhibitBlit) && (pDoc->GetMode() == CTrueCapDoc::Playback) && 

																pDoc->IsPaused())

		{

		pDoc->DisplayCurrentFrame();

		}

	else if (pDoc->GetMode() == CTrueCapDoc::Playback && !pDoc->IsPlaying())

		{

		BlitStaticLogo();

		}

}



void CTrueCapView::OnDraw(CDC* pDC)

{

	if (mCaptureFullScreen) return;



	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	

	// TODO: add draw code for native data here

	if (mFirstUpdate)

		{

		pDoc->SetView(this);

		mFirstUpdate = 0;

		// resize if not quite right

		RECT r;

		GetClientRect(&r);

		BITMAP bm;

		mPlaybackBackground.GetBitmap(&bm);

		int x = bm.bmWidth - (r.right - r.left);

		int y = bm.bmHeight - (r.bottom - r.top);

		if (x || y)

			{

			CWnd *w = GetParent();

			w->GetWindowRect(&r);

			r.right += x;

			r.bottom += y;

			w->MoveWindow(&r,TRUE);

			}

		if (pDoc->GetMode() == CTrueCapDoc::Capture)

			{

			pDoc->SetCaptureMode();

			//pDoc->SetCaptureFileName(pDoc->GetCaptureFileName()); // force status bar update

			//pDoc->CreateCaptureWindow();

			}

		else

			{

			pDoc->SetPlaybackMode(1);

			ReconfigureSlider();

			pDoc->PrepareForSeek();

			//pDoc->SetPlaybackFileName(pDoc->GetPlaybackFileName()); // force status bar update

			}

		Invalidate();

		mInhibitBlit = 0;

		}

	//pDC->FillSolidRect(CRect(0,0,451,350),255);



	if (pDoc->IsPageFlipping())

		{

		CFrameWnd *mf;

		CRect ClientRect;

		CDC *cdc;



		mf = GetParentFrame();

		cdc = mf->GetWindowDC();

		mf->GetWindowRect(&ClientRect);

		cdc->FillSolidRect(&ClientRect,RGB(0,0,0));

		mf->ReleaseDC(cdc);

		return;

		}





	{

		CBitmap	*tmp = 0;

		CDC	 dcMem;

		dcMem.CreateCompatibleDC(pDC);

		RECT r;

		GetClientRect(&r);



		if (pDoc->GetMode() == CTrueCapDoc::Playback)

			{

			int bi = pDoc->BigImage();

			switch(bi)

				{

				//case -1:

				//	tmp = dcMem.SelectObject(mLastPlaybackBitmap);

				//	break;

				case 0:

					tmp = dcMem.SelectObject(&mPlaybackBackgroundSmall);

					mLastPlaybackBitmap = &mPlaybackBackgroundSmall;

					break;

				case 1:

					tmp = dcMem.SelectObject(&mPlaybackBackground);

					mLastPlaybackBitmap = &mPlaybackBackground;

					break;

				}

			}

		else

			{

			tmp = dcMem.SelectObject(pDoc->BigImage() ? &mCaptureBackground

				: &mCaptureBackgroundSmall);

			}

		dcMem.SetMapMode(pDC->GetMapMode());

		pDC->BitBlt(r.left,r.top,r.right - r.left,r.bottom - r.top,

			&dcMem,0,0,SRCCOPY);

		dcMem.SelectObject(tmp);

	}

	DrawStaticFrame();

	mCAPTURE_RecordButton.UseAltSet(pDoc->IsCapturing());

	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

	if (pDoc->GetMode() == CTrueCapDoc::Capture)

		DrawChannel();

#ifndef ROCKWELL

	if (gDuckVisionLite)

		{

		// draw black square lower left

		RECT r;

		GetClientRect(&r);

		r.right = r.left + 5;

		r.top = r.bottom - 5;

		pDC->FillSolidRect(&r,0);

		}

#endif

}



/////////////////////////////////////////////////////////////////////////////

// CTrueCapView printing



BOOL CTrueCapView::OnPreparePrinting(CPrintInfo* pInfo)

{

	// default preparation

	return DoPreparePrinting(pInfo);

}



void CTrueCapView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)

{

	// TODO: add extra initialization before printing

}



void CTrueCapView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)

{

	// TODO: add cleanup after printing

}



/////////////////////////////////////////////////////////////////////////////

// CTrueCapView diagnostics



#ifdef _DEBUG

void CTrueCapView::AssertValid() const

{

	CView::AssertValid();

}



void CTrueCapView::Dump(CDumpContext& dc) const

{

	CView::Dump(dc);

}



CTrueCapDoc* CTrueCapView::GetDocument() // non-debug version is inline

{

	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTrueCapDoc)));

	return (CTrueCapDoc*)m_pDocument;

}

#endif //_DEBUG



/////////////////////////////////////////////////////////////////////////////

// CTrueCapView message handlers



void CTrueCapView::OnDestroy() 

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	pDoc->ViewDestroying();

	KillTimer(1);

	if (mChannelTimerActive)

		{

		KillTimer(2);

		mChannelTimerActive= 0;

		}

	CView::OnDestroy();

	

	// TODO: Add your message handler code here

	if (pDoc->GetMode() == CTrueCapDoc::Playback)

		pDoc->Stop();

	else

		pDoc->StopCapture();

	pDoc->DestroyCaptureWindow();

}



void

CTrueCapView::ClearPendingChannel(void)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	mPendingChannel = 0;

	InvalidateChannels();

}



void

CTrueCapView::ChannelDigit(int d)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (!pDoc->HasTuner()) return;



	mPendingChannel = (mPendingChannel * 10) + d;

	InvalidateChannels();

}



void

CTrueCapView::ChannelUp(void)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	int	 set = 0;



	if (!pDoc->HasTuner()) return;



	if (!mChannelList.size())

		{

		AfxMessageBox("The channel scan list is empty. Pick \"Channel Scan...\" from the Options menu.");

		return;

		}

	int c = pDoc->GetChannel();

	if (c)

		{

		for (int x = 0; x < mChannelList.size(); ++x)

			{

			if (c < mChannelList[x])

				{

				pDoc->SetChannel(mChannelList[x]);

				set = 1;

				break;

				}

			}

		}

	if (!set && mChannelList.size())

		pDoc->SetChannel(mChannelList[0]);

	mPendingChannel = 0;

	InvalidateChannels();

}



void

CTrueCapView::ChannelDown(void)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	int set = 0;



	if (!pDoc->HasTuner()) return;



	if (!mChannelList.size())

		{

		AfxMessageBox("The channel scan list is empty. Pick \"Channel Scan...\" from the Options menu.");

		return;

		}

	int c = pDoc->GetChannel();

	if (c)

		{

		for (int x = mChannelList.size() - 1; x >= 0; --x)

			{

			if (c > mChannelList[x])

				{

				set = 1;

				pDoc->SetChannel(mChannelList[x]);

				break;

				}

			}

		}

	if (!set && mChannelList.size())

		pDoc->SetChannel(mChannelList[mChannelList.size() - 1]);

	mPendingChannel = 0;

	InvalidateChannels();

}



void

CTrueCapView::ChannelEnter(void)

{

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (!pDoc->HasTuner()) return;

	if (mPendingChannel)

		{

		pDoc->SetChannel(mPendingChannel);

		mPendingChannel = 0;

		}	

	InvalidateChannels();

}



void CTrueCapView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 

{

	// TODO: Add your message handler code here and/or call default

	CTrueCapDoc* pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	SetFocus();


	dprintf("keydown dispatcher -- key=%d\n",(int)nChar);
	TRACE("Key %d\n",(int)nChar);

	if (nChar == 16)

		{

		mShiftKey = true;

		return;

		}

	if (pDoc->GetMode() == CTrueCapDoc::Capture)

		{

		switch(nChar)

			{

			case 70: // f 

				pDoc->CaptureFrame();

				break;

			case 27: // escape

				if (pDoc->IsFullScreenCapture())

					pDoc->ToggleFullScreen();

				break;

			case 8: //backspace

				ClearPendingChannel();

				break;

			case 13: // enter

				ChannelEnter();

				break;

			case 96: case 97: case 98: case 99: case 100: case 101:

			case 102: case 103: case 104: case 105:

			case '0': case '1': case '2': case '3': case '4': case '5':

			case '6': case '7': case '8': case '9':

				ChannelDigit(nChar - (nChar >= 96 ? 96 : '0'));

				break;

			case 33: // page up

			case 187:	// plus

			case 38:	// up arrow

			case 39:	// right arrow

			case 107:	// keypad plus

				ChannelUp();

				break;

			case 34: //page down

			case 189:  // minus

			case 37:	// left

			case 40:	// down

			case 109: // keypad minus

				ChannelDown();

				break;

			}

		return;

		}



	int	min = 0;

	int max = mPLAYBACK_Slider.GetRangeMax();

	int pos = pDoc->GetCurrentFrame();



	if (pDoc->GetMode() == CTrueCapDoc::Capture) return;



	switch(nChar)

		{

		case 38: // up

			if (mShiftKey)

				pDoc->SkipCommercial();

			else

				Do_Play(mPlayFullscreen);

			break;

		case 27:

		case 40: // down

			Do_Stop();

			break;

		case 37: // left arrow;

			pDoc->PrepareForSeek();

			mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

			if (--pos < min) 

				{

				pos = min;

				pDoc->SetAndDisplayFrame(pos);

				}

			else

				{

				if (mShiftKey)

					pDoc->RetreatTime(5000);

				else

					pDoc->RetreatFrame();

				}

			mPLAYBACK_Slider.SetPos(pos);

			break;

		case 39:

			pDoc->PrepareForSeek();

			mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

			if (++pos > max) 

				{

				pos = max;

				pDoc->SetAndDisplayFrame(pos);

				}

			else

				{

				if (mShiftKey)

					pDoc->AdvanceTime(5000);

				else

					pDoc->AdvanceFrame();

				}

			mPLAYBACK_Slider.SetPos(pos);

			break;

		default:

			CView::OnKeyDown(nChar, nRepCnt, nFlags);

			break;

		}

	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());	

}



int CTrueCapView::OnCreate(LPCREATESTRUCT lpCreateStruct) 

{

	if (CView::OnCreate(lpCreateStruct) == -1)

		return -1;

	



	// TODO: Add your specialized creation code here

	// left,top,right bottom  - CRECT

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	DWORD	playback_style = WS_CHILD | BS_OWNERDRAW;

	DWORD	capture_style = WS_CHILD | BS_OWNERDRAW;

	DWORD	slider_style = WS_CHILD | TBS_NOTICKS | TBS_BOTH;

	if (pDoc->GetMode() == CTrueCapDoc::Playback)

		{

		playback_style |= WS_VISIBLE;

		slider_style |= WS_VISIBLE;

		}

	else

		{

		capture_style |= WS_VISIBLE;

		}



	mPLAYBACK_Slider.Create(slider_style,CRect(CPoint(180,294),CSize(140,24)),this,45);



	mPLAYBACK_FullscrnToggle.Create

		("Screen",		playback_style,CRect(CPoint(17,283),CSize(60,52)),this,28);

	mPLAYBACK_FullscrnToggle.UseAltSet(mPlayFullscreen);



	mPLAYBACK_PlayButton.Create		

		("Play",		playback_style,CRect(CPoint(91,283),CSize(45,52)),this,17);

	mPLAYBACK_StopButton.Create		

		("Stop",		playback_style,CRect(CPoint(136,283),CSize(45,42)),this,18);

	mPLAYBACK_VCRButton.Create		

		("VCR",			playback_style,CRect(CPoint(362,289),CSize(77,47)),this,20);

	mPLAYBACK_SetFileButton.Create	

		("Set File",	playback_style,CRect(CPoint(379,80), CSize(45,42)),this,21);

	mPLAYBACK_RenameButton.Create	

		("Rename",		playback_style,CRect(CPoint(379,151),CSize(45,42)),this,22);

	mPLAYBACK_SkipButton.Create	

		("Skip",		playback_style,CRect(CPoint(379,208),CSize(45,42)),this,42);

	

	mPlaybackButtons.push_back(&mPLAYBACK_PlayButton);

	mPlaybackButtons.push_back(&mPLAYBACK_StopButton);

	mPlaybackButtons.push_back(&mPLAYBACK_VCRButton);

	mPlaybackButtons.push_back(&mPLAYBACK_SetFileButton);

	mPlaybackButtons.push_back(&mPLAYBACK_RenameButton);

	mPlaybackButtons.push_back(&mPLAYBACK_FullscrnToggle);

	mPlaybackButtons.push_back(&mPLAYBACK_SkipButton);





	mCAPTURE_FullscrnToggle.Create

		("Toggle",	capture_style,CRect(CPoint(17,283),CSize(60,52)),this,44);

	mCAPTURE_StillButton.Create

		("Still",	capture_style,CRect(CPoint(276,283),CSize(45,42)),this,43);

	mCAPTURE_AudioButton.Create

		("Audio",	capture_style,CRect(CPoint(379,151),CSize(45,42)),this,23);

	mCAPTURE_RecordButton.Create	

		("Record",	capture_style,CRect(CPoint(220,283),CSize(45,51)),this,24);

	mCAPTURE_SetFileButton.Create	

		("Set File",capture_style,CRect(CPoint(379,80), CSize(45,42)),this,25);

	mCAPTURE_TVButton.Create

		("TV",		capture_style,CRect(CPoint(362,289),CSize(77,47)),this,26);

	mCAPTURE_VideoButton.Create

		("Video",	capture_style,CRect(CPoint(379,211),CSize(45,42)),this,27);

	if (pDoc->HasTuner())

		{

		mCAPTURE_0.Create

			("0",		capture_style,CRect(CPoint(162,325),CSize(16,15)),this,29);

		mCAPTURE_1.Create

			("1",		capture_style,CRect(CPoint(146,280),CSize(16,15)),this,30);

		mCAPTURE_2.Create

			("2",		capture_style,CRect(CPoint(162,280),CSize(16,15)),this,31);

		mCAPTURE_3.Create

			("3",		capture_style,CRect(CPoint(178,280),CSize(16,15)),this,32);

		mCAPTURE_4.Create

			("4",		capture_style,CRect(CPoint(146,295),CSize(16,15)),this,33);

		mCAPTURE_5.Create

			("5",		capture_style,CRect(CPoint(162,295),CSize(16,15)),this,34);

		mCAPTURE_6.Create

			("6",		capture_style,CRect(CPoint(178,295),CSize(16,15)),this,35);

		mCAPTURE_7.Create

			("7",		capture_style,CRect(CPoint(146,310),CSize(16,15)),this,36);

		mCAPTURE_8.Create

			("8",		capture_style,CRect(CPoint(162,310),CSize(16,15)),this,37);

		mCAPTURE_9.Create

			("9",		capture_style,CRect(CPoint(178,310),CSize(16,15)),this,38);

		mCAPTURE_Up.Create

			("Up",		capture_style,CRect(CPoint(130,311),CSize(9,5)),this,39);

		mCAPTURE_Down.Create

			("Down",	capture_style,CRect(CPoint(130,319),CSize(9,5)),this,40);

		mCAPTURE_Enter.Create

			("Enter",	capture_style,CRect(CPoint(99,310),CSize(27,15)),this,41);

		}

	mCaptureButtons.push_back(&mCAPTURE_AudioButton);

	mCaptureButtons.push_back(&mCAPTURE_RecordButton);

	mCaptureButtons.push_back(&mCAPTURE_SetFileButton);

	mCaptureButtons.push_back(&mCAPTURE_TVButton);

	mCaptureButtons.push_back(&mCAPTURE_VideoButton);

	mCaptureButtons.push_back(&mCAPTURE_StillButton);

	mCaptureButtons.push_back(&mCAPTURE_FullscrnToggle);

	if (pDoc->HasTuner())

		{

		mCaptureButtons.push_back(&mCAPTURE_0);

		mCaptureButtons.push_back(&mCAPTURE_1);

		mCaptureButtons.push_back(&mCAPTURE_2);

		mCaptureButtons.push_back(&mCAPTURE_3);

		mCaptureButtons.push_back(&mCAPTURE_4);

		mCaptureButtons.push_back(&mCAPTURE_5);

		mCaptureButtons.push_back(&mCAPTURE_6);

		mCaptureButtons.push_back(&mCAPTURE_7);

		mCaptureButtons.push_back(&mCAPTURE_8);

		mCaptureButtons.push_back(&mCAPTURE_9);

		mCaptureButtons.push_back(&mCAPTURE_Up);

		mCaptureButtons.push_back(&mCAPTURE_Down);

		mCaptureButtons.push_back(&mCAPTURE_Enter);

		}



	HDC	device = ::GetDC(m_hWnd);

	int bitsperpixel = GetDeviceCaps(device, PLANES) * GetDeviceCaps(device, BITSPIXEL);

	::ReleaseDC(m_hWnd,device);

	if (bitsperpixel < 15 || bitsperpixel > 16)

		AfxMessageBox("Switch to 16 bit color for optimal performance.");



	mDropTarget.Register(this);

	//DragAcceptFiles(true);



	SetTimer(1,5000,0);

	return 0;

}



static char BASED_CODE szFilter[] = "VP3 Files (*.vp3)|*.vp3|All Files (*.*)|*.*||";



void

CTrueCapView::SetCaptureFile(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Capture) return;

	if (pDoc->IsCapturing()) return;



	CFileDialog	dlog(FALSE,"avi",pDoc->GetCaptureFileName(),

		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,szFilter);

	//pDoc->ShowVideo(FALSE);

	if (dlog.DoModal() == IDOK)

		{

		pDoc->SetCaptureFileName(dlog.GetPathName());

		}

	//pDoc->ShowVideo(TRUE);

}





void

CTrueCapView::SetPlaybackFile(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Playback) return;

	mInhibitBlit = 1;

	pDoc->Stop();

//	if (pDoc->IsPlaying()) return;



	CFileDialog	dlog(TRUE,"avi",pDoc->GetPlaybackFileName());

	if (dlog.DoModal() == IDOK)

		{

		pDoc->SetPlaybackFileName(dlog.GetPathName());

		}

	pDoc->PrepareForSeek();

	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

	//DrawStaticFrame();

	ReconfigureSlider();

	mInhibitBlit = 0;

	CRect	r(VIDEO_X_OFFSET,VIDEO_Y_OFFSET,VIDEO_X_OFFSET + 320,VIDEO_Y_OFFSET + 240);

	InvalidateRect(&r);

}



void

CTrueCapView::ReconfigureSlider(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	mPLAYBACK_Slider.SetRangeMin(0);

	mPLAYBACK_Slider.SetRangeMax(pDoc->GetNumVideoFrames());

	mPLAYBACK_Slider.SetPos(0);

	mPLAYBACK_Slider.SetPageSize(30);

}



void

CTrueCapView::HideCaptureButtons(void)

{

	vector<C3BitmapButton*>::iterator i;

	for (i = mCaptureButtons.begin(); i != mCaptureButtons.end(); ++i)

		(*i)->ShowWindow(SW_HIDE);

}

void

CTrueCapView::HidePlaybackButtons(void)

{

	vector<C3BitmapButton*>::iterator i;

	for (i = mPlaybackButtons.begin(); i != mPlaybackButtons.end(); ++i)

		(*i)->ShowWindow(SW_HIDE);

	mPLAYBACK_Slider.SetPos(0);

	mPLAYBACK_Slider.ShowWindow(SW_HIDE);

}

void

CTrueCapView::ShowPlaybackButtons(void)

{

	TRACE("ShowPlaybackButtons\n");

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	Invalidate();

	UpdateWindow();



	vector<C3BitmapButton*>::iterator i;

	for (i = mPlaybackButtons.begin(); i != mPlaybackButtons.end(); ++i)

		{

		(*i)->ShowWindow(SW_SHOW);

		(*i)->Invalidate();

		(*i)->UpdateWindow();

		}

	if (!pDoc->GetCanCapture())

		mPLAYBACK_VCRButton.ShowWindow(SW_HIDE);

	if (!gDirectDrawAvailable)

		mPLAYBACK_FullscrnToggle.ShowWindow(SW_HIDE);

	mPLAYBACK_Slider.ShowWindow(SW_SHOW);

	mPLAYBACK_Slider.SetPos(0);

	mPLAYBACK_Slider.Invalidate();

	mPLAYBACK_Slider.UpdateWindow();

}

void

CTrueCapView::ShowCaptureButtons(void)

{

	TRACE("ShowCaptureButtons\n");

	Invalidate();

	UpdateWindow();



	vector<C3BitmapButton*>::iterator i;

	for (i = mCaptureButtons.begin(); i != mCaptureButtons.end(); ++i)

		{

		(*i)->ShowWindow(SW_SHOW);

		(*i)->Invalidate();

		(*i)->UpdateWindow();

		}

}



void	

CTrueCapView::GetVideoBoxXY(int &x,int &y,int width,int height)

{

	if (width == 640 && height == 480)

		{

		x = 0;

		y = 0;

		}	

	else

		{

		x = ((320 - width) / 2) + VIDEO_X_OFFSET;

		y = ((240 - height) / 2) + VIDEO_Y_OFFSET;

		}

}



void

CTrueCapView::ConfigureVideo(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Capture) return;

	if (pDoc->IsCapturing()) return;



	int	width,height,fps,quality;

	pDoc->GetVideoConfiguration(width,height,fps,quality);

	CVideoSettingsDialog	dlog;

	dlog.mVideoSource = IVideoTuner::BaseBand_Composite;

	dlog.mBrightness = 128;

	dlog.mContrast = 128;

	dlog.mSaturation = 128;

	dlog.mHue = 128;

	IVideoTuner *t = pDoc->GetVideoTuner();

	if (t)

		{

		try {

		dlog.mVideoSource	= t->GetVideoSource();

		dlog.mBrightness	= t->GetBrightness();

		dlog.mContrast		= t->GetContrast();

		dlog.mSaturation	= t->GetSaturation();

		dlog.mHue			= t->GetHue();

		} catch(...){}

		}

	dlog.mFPS		= (fps == 10 ? 0 : (fps == 15 ? 1 : 2));

	dlog.mQuality	= (quality - 4) * -1;

	dlog.mDimensions= (width == 320 ? 1 : 0);

	dlog.mSupervisor	= pDoc;

	

	//pDoc->ShowVideo(FALSE);



	if (dlog.DoModal() == IDOK)

		{

		fps = (dlog.mFPS == 0 ? 10 : (dlog.mFPS == 1 ? 15 : 30));

		quality = (dlog.mQuality - 4) * -1;

		if (dlog.mDimensions == 1)

			{

			width = 320;

			height = 240;

			}

		else

			{

			width = 160;

			height = 120;

			}



		pDoc->SetVideoConfiguration(width,height,fps,quality);

		}

	else

		{

		int w,h,f,q;

		pDoc->GetVideoConfiguration(w,h,f,q);

		pDoc->SetVideoConfiguration(w,h,f,q);

		}

	//pDoc->ShowVideo(TRUE);

	RECT r;

	r.top = r.left = 0;

	r.right = 320 + VIDEO_X_OFFSET + 10;

	r.bottom = 240 + VIDEO_Y_OFFSET + 10;

	InvalidateRect(&r,FALSE);

}





void

CTrueCapView::ConfigureAudio(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Capture) return;

	if (pDoc->IsCapturing()) return;



	int sample_size,channels,frequency,volume;

	pDoc->GetAudioConfiguration(sample_size,channels,frequency,volume);

	CAudioSettingsDialog	dlog;

	dlog.mSupervisor = pDoc;

	dlog.mSampleSize = (sample_size == 8 ? 0 : 1);

	dlog.mChannels = (channels == 1 ? 0 : 1);

	dlog.mFrequency = (frequency == 11025 ? 0 : (frequency == 22050 ? 1 : 2));

	dlog.mMinVolume = 0;

	dlog.mMaxVolume = 100;

	dlog.mVolume =volume;

	

	//pDoc->ShowVideo(FALSE);



	if (dlog.DoModal() == IDOK)

		{

		sample_size = (dlog.mSampleSize == 0 ? 8 : 16);

		channels = (dlog.mChannels == 0 ? 1 : 2);

		frequency = (dlog.mFrequency == 0 ? 11025 : 

							(dlog.mFrequency == 1 ? 22050 : 44100));

		volume = dlog.mVolume;

		pDoc->SetAudioConfiguration(sample_size,channels,frequency,volume);

		}

	//pDoc->ShowVideo(TRUE);

}





void

CTrueCapView::RenameFile(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Playback) return;

	pDoc->Stop();



	CRenameDialog	dlog;



	dlog.mFrom = pDoc->GetPlaybackFileName();

	dlog.mTo = pDoc->GetPlaybackFileName();

	if (dlog.DoModal() == IDOK)

		{

		CFile::Rename(dlog.mFrom,dlog.mTo);

		pDoc->SetPlaybackFileName(dlog.mTo);

		}

	pDoc->PrepareForSeek();

	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

	DrawStaticFrame();

	ReconfigureSlider();

}



CColorSlider *

CTrueCapView::GetSlider(void)

{

	return &mPLAYBACK_Slider;

}



C3BitmapButton	*

CTrueCapView::GetPlayButton(void)

{

	return &mPLAYBACK_PlayButton;

}





void

CTrueCapView::Do_Stop(void)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	int pf = pDoc->IsPageFlipping();



	if (!pDoc->IsPaused())

		pDoc->Pause();

	

	ShowPlaybackButtons();

	pDoc->GoFullScreen(0);

	if (pf)

		pDoc->ShowStatusBar(SW_SHOW);



	pDoc->SetAndDisplayFrame(0);

	mPLAYBACK_Slider.SetPosMessage(0);

}



void

CTrueCapView::CleanupAfterStop(void)

{

	PostMessage(WM_USER,0,0);

}



void

CTrueCapView::Do_Play(int fullscreen)

{

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (fullscreen &&  !pDoc->IsPageFlipping())

		{

		pDoc->ShowStatusBar(SW_HIDE);

		HidePlaybackButtons();

		pDoc->GoFullScreen(1);

		}



	if (pDoc->IsPaused())

		{

		pDoc->Play();

		}

	else

		pDoc->Pause();

}



BOOL CTrueCapView::OnCommand(WPARAM wParam, LPARAM lParam) 

{

	// TODO: Add your specialized code here and/or call the base class

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);


	dprintf("main command dispatch - %d\n",(int)wParam);
	TRACE("View command %d\n",(int)wParam);

	switch(wParam)

		{

		case 15:

			pDoc->FastForward();

			break;

		case 16:

			pDoc->Rewind();

			break;

		case 17:

			Do_Play(mPlayFullscreen);

			break;

		case 18:

			Do_Stop();

			break;

		case 19:

			pDoc->Pause();

			break;

		case 20:

			pDoc->SetCaptureMode();

			break;

		case 21:

			SetPlaybackFile();

			break;

		case 22:

			RenameFile();

			break;

		case 42: 

			pDoc->SkipCommercial();

			break;

		case 23:

			ConfigureAudio();

			break;

		case 24:

			pDoc->DoCapture();

			break;

		case 25:

			SetCaptureFile();

			break;

		case 26:

			pDoc->SetPlaybackMode();

			break;

		case 27:

			ConfigureVideo();

			break;

		case 28:

			if (pDoc->IsPaused())

				{

				mPlayFullscreen = !mPlayFullscreen;

				mPLAYBACK_FullscrnToggle.UseAltSet(mPlayFullscreen);

				}

			break;

		case 29: case 30: case 31: case 32: case 33: case 34: case 35:

		case 36: case 37: case 38:

			ChannelDigit(wParam - 29);

			break;

		case 39:

			ChannelUp();

			break;

		case 40:

			ChannelDown();

			break;

		case 41:

			ChannelEnter();

			break;

		case 43:

			pDoc->CaptureFrame();

			break;

		case 44:

			pDoc->ToggleFullScreen();

			break;

		default:

			return CView::OnCommand(wParam, lParam);

		}



	mCAPTURE_RecordButton.UseAltSet(pDoc->IsCapturing());

	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

	return TRUE;

}



void

CTrueCapView::CaptureFullScreen(bool fullscreen)

{

	if (fullscreen)

		{

		HideCaptureButtons();

		CDC *dc = GetDC();

		RECT r;

		GetClientRect(&r);

		dc->FillSolidRect(&r,0);

		ReleaseDC(dc);

		}

	else

		{

		ShowCaptureButtons();

		}

	mCaptureFullScreen = fullscreen;

}





void CTrueCapView::OnOptionsAudio() 

{

	// TODO: Add your command handler code here

	ConfigureAudio();

}



void CTrueCapView::OnUpdateOptionsAudio(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	

	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Capture && !pDoc->IsCapturing());

}



void CTrueCapView::OnOptionsVideo() 

{

	// TODO: Add your command handler code here

	ConfigureVideo();

}



void CTrueCapView::OnUpdateOptionsVideo(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	

	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Capture && !pDoc->IsCapturing());

}



void CTrueCapView::OnFileSetcapturefile() 

{

	// TODO: Add your command handler code here

	SetCaptureFile();	

}



void CTrueCapView::OnUpdateFileSetcapturefile(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	

	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Capture && !pDoc->IsCapturing());

}



void CTrueCapView::OnFileSetplaybackfile() 

{

	// TODO: Add your command handler code here

	SetPlaybackFile();

}



void CTrueCapView::OnUpdateFileSetplaybackfile(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	

	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Playback);

}



void CTrueCapView::OnFileRenamefile() 

{

	// TODO: Add your command handler code here

	RenameFile();	

}



void CTrueCapView::OnUpdateFileRenamefile(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Playback);

}









void CTrueCapView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 

{

	// TODO: Add your specialized code here and/or call the base class

	

	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);

	TRACE("foo %d\n",(int)bActivate);//GetParentOwner()->IsIconic());

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);

	//pDoc->ShowVideo(bActivate);//!GetParentOwner()->IsIconic());

	gIconic = GetParentOwner()->IsIconic();

	TRACE("Iconic %d\n",gIconic);

	pDoc->ShowVideo(!GetParentOwner()->IsIconic());

}







BOOL CTrueCapView::OnEraseBkgnd(CDC* pDC) 

{

	// TODO: Add your message handler code here and/or call default

#if 0

	RECT r;

	GetClientRect(&r);

	pDC->FillSolidRect(&r,0x666666);

#endif

	return 1;

	//return CView::OnEraseBkgnd(pDC);

}





void CTrueCapView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 

{

	// TODO: Add your message handler code here and/or call default

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);





	TRACE("nPos %u nSBCode %d\n",(unsigned int)nPos,(int)nSBCode);

	if (nSBCode == SB_THUMBPOSITION || nSBCode == SB_THUMBTRACK)

		{

		pDoc->PrepareForSeek();

		pDoc->SetAndDisplayFrame(mPLAYBACK_Slider.GetPos(nPos));

		}

	else if (nSBCode == SB_PAGELEFT)

		{

		pDoc->PrepareForSeek();

		int f = pDoc->GetCurrentFrame();

		f -= 30;

		if (f < 0) f = 0;

		pDoc->SetAndDisplayFrame(f);	

		}

	else if (nSBCode == SB_PAGERIGHT)

		{

		pDoc->PrepareForSeek();

		int f = pDoc->GetCurrentFrame();

		f += 30;

		if (f > mPLAYBACK_Slider.GetRangeMax()) f = mPLAYBACK_Slider.GetRangeMax();

		pDoc->SetAndDisplayFrame(f);	

		}



	mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

	CView::OnHScroll(nSBCode, nPos, pScrollBar);

}



BOOL CTrueCapView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 

{

	// TODO: Add your message handler code here and/or call default

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->IsPageFlipping())

		{

		::SetCursor(NULL);

		return TRUE;

		}

	return CView::OnSetCursor(pWnd, nHitTest, message);

}



LRESULT CTrueCapView::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 

{

	// TODO: Add your specialized code here and/or call the base class

	//CTrueCapDoc *pDoc;



	if (message == WM_USER)

		{

		switch(wParam)

			{

			case 0:

				Do_Stop();

				return 0;

			}

		return 0;

		}

	else

		{

		return CView::WindowProc(message, wParam, lParam);

		}

}



void

CTrueCapView::CaptureMode(void)

{

}



void

CTrueCapView::PlaybackMode(void)

{

	mTimeShiftManager.Reset();

	mTimedRecord = false;

}



void

CTrueCapView::CaptureStopped(void)

{

	CWnd *w = AfxGetMainWnd();

	w->SetWindowPos(&wndNoTopMost,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);

	w->ModifyStyle(0,WS_SYSMENU);

	mTimedRecord = false;

	::InvalidateRect(NULL,NULL,FALSE);

}



void

CTrueCapView::CaptureStart(void)

{

	CWnd *w = AfxGetMainWnd();

	w->SetWindowPos(&wndTopMost,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);

	w->ModifyStyle(WS_SYSMENU,0);

}





void CTrueCapView::OnTimer(UINT nIDEvent) 

{

	// TODO: Add your message handler code here and/or call default

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	assert(nIDEvent >= 1 && nIDEvent <= 2);



	if (nIDEvent == 2)

		{

		InvalidateChannels();

		}

	else if (nIDEvent == 1)

		{

		if (mTimedRecord)

			{

			COleDateTime ct = COleDateTime::GetCurrentTime();

			if (ct - mStartTimedRecord > mTimedRecordLength)

				{

				mTimedRecord = false;

				pDoc->StopTimeShiftCapture();

				mCAPTURE_RecordButton.UseAltSet(FALSE);

				}

			}

		else

			{

			TRACE("Timer\n");

			COleDateTimeSpan 	five_seconds(0,0,0,5);

			COleDateTime	current_time = COleDateTime::GetCurrentTime();

			current_time += five_seconds;

			CTimeShiftEvent	e;

			switch(mTimeShiftManager.Query(e,current_time))

				{

				case CTimeShiftManager::Nothing:

					TRACE("Time shift action - nothing\n");

					break;

				case CTimeShiftManager::Warning:

					TRACE("Time shift action - warning\n");

					break;

				case CTimeShiftManager::StartCapture:

					TRACE("Time shift action - startcapture\n");

					pDoc->StartTimeShiftCapture(e);

					InvalidateChannels();

					mCAPTURE_RecordButton.UseAltSet(TRUE);

					break;

				case CTimeShiftManager::StopCapture:

					TRACE("Time shift action - stopcapture\n");

					pDoc->StopTimeShiftCapture();

					mCAPTURE_RecordButton.UseAltSet(FALSE);

					break;

				}

			}

		}

}



void CTrueCapView::OnOptionsEditevents() 

{

	// TODO: Add your command handler code here

	if (gNag)

		{

		CDialogNAG2 nag;

		nag.DoModal();

		}

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->IsCapturing()) return;



	CEventListDialog dlog;



	dlog.mSupervisor = pDoc;

	dlog.mEventLimit = gEventLimit;

	dlog.mEventTimeLimit = gEventTimeLimit;

	dlog.mList = mTimeShiftManager.GetEventList();

	if (dlog.DoModal() == IDOK)

		{

		mTimeShiftManager.BeginListAdjustment();

		mTimeShiftManager.Clear();

		CTimeShiftManager::EventList_t::iterator i;

		for (i = dlog.mList.begin(); i != dlog.mList.end(); ++i)

			mTimeShiftManager.AddEvent(*i);

		mTimeShiftManager.EndListAdjustment();

		CheckEventLimits(mTimeShiftManager);

		CheckEventTimeLimits(mTimeShiftManager);

		SaveEvents(mDirectory + EVENTFILE,mTimeShiftManager);

		::PostMessage(HWND_BROADCAST,gPrivateMessageLauncher,0,0);

		}

}



void CTrueCapView::OnUpdateOptionsEditevents(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	pCmdUI->Enable(!pDoc->IsCapturing());

	

}



void CTrueCapView::OnOptionsChannelscan() 

{

	// TODO: Add your command handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->IsCapturing()) return;

	if (pDoc->IsPlaying()) return;



	CChannelScanDialog dlog;

	dlog.mDocument = pDoc;

	dlog.mChannelList = mChannelList;

	if (dlog.DoModal() == IDOK)

		{

		mChannelList = dlog.mChannelList;

		}	

}



void CTrueCapView::OnUpdateOptionsChannelscan(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	pCmdUI->Enable((!pDoc->IsCapturing()) && (pDoc->GetMode() == CTrueCapDoc::Capture));

	

}







void CTrueCapView::OnFileUpgrade() 

{

	// TODO: Add your command handler code here

#ifndef ROCKWELL

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	CUpgradeDialog	dlog;

	dlog.mKeyString = pDoc->GetKeyString();

	if (dlog.DoModal() == IDOK)

		{

		if (!pDoc->SetKeyString(dlog.mKeyString))

			{

			AfxMessageBox("Key string is not valid");

			}

		else

			{

			MakeNormalVersion();

			::PostMessage(HWND_BROADCAST,gPrivateMessageLauncher,0,0);

			}

		Invalidate();

		}

#endif

}



void CTrueCapView::OnUpdateFileUpgrade(CCmdUI* pCmdUI) 

{

#ifndef ROCKWELL

	// TODO: Add your command update UI handler code here

	pCmdUI->Enable(gDuckVisionLite);

#endif

}





void

CTrueCapView::ConfirmOleData(COleDataObject *pDataObject)

{

	if (mDropCache) return;

	if (!pDataObject) return;

	mDropCache = pDataObject;

	mLastEffect = DROPEFFECT_NONE;

	if (!pDataObject->IsDataAvailable(CF_HDROP)) return;

	STGMEDIUM m;

	if (!pDataObject->GetData(CF_HDROP,&m)) return;

	if (m.tymed != TYMED_HGLOBAL) return;

	HDROP h = (HDROP)m.hGlobal;



	DragQueryFile(h,0,mDropFilename,DROP_FILENAME_SIZE);

	

	AVIFileInit();

	PAVIFILE	f;

	PAVISTREAM	s;

	if (AVIFileOpen(&f,mDropFilename,OF_READ,NULL)) 

		{

		AVIFileExit();

		if (m.pUnkForRelease == 0)

			GlobalFree(h);

		return;

		}

	if (AVIFileGetStream(f,&s,streamtypeVIDEO,0))

		{

		AVIFileRelease(f);

		AVIFileExit();

		if (m.pUnkForRelease == 0)

			GlobalFree(h);

		return;

		}

	AVISTREAMINFO	info;

	if (AVIStreamInfo(s,&info,sizeof(info)))

		{

		AVIStreamRelease(s);

		AVIFileRelease(f);

		AVIFileExit();

		if (m.pUnkForRelease == 0)

			GlobalFree(h);

		return;

		}



	AVIStreamRelease(s);

	AVIFileRelease(f);

	AVIFileExit();



	if (info.fccHandler == MKFOURCC('T','R','2','0') ||

		info.fccHandler == MKFOURCC('t','r','2','0') ||

		info.fccHandler == MKFOURCC('T','M','2','0') ||

		info.fccHandler == MKFOURCC('t','m','2','0') ||

		info.fccHandler == MKFOURCC('D','U','C','K') ||

		info.fccHandler == MKFOURCC('d','u','c','k'))

		mLastEffect = DROPEFFECT_COPY;

	if (info.rcFrame.right - info.rcFrame.left > 320 ||

		info.rcFrame.bottom - info.rcFrame.top > 240)

		mLastEffect = DROPEFFECT_NONE;



	if (m.pUnkForRelease == 0)

		GlobalFree(h);;

}



DROPEFFECT CTrueCapView::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 

{

	// TODO: Add your specialized code here and/or call the base class

	ConfirmOleData(pDataObject);

	return mLastEffect;



//	return CView::OnDragEnter(pDataObject, dwKeyState, point);

}



void CTrueCapView::OnDragLeave() 

{

	// TODO: Add your specialized code here and/or call the base class

	mDropCache = 0;

	mLastEffect = DROPEFFECT_NONE;

	CView::OnDragLeave();

}



DROPEFFECT CTrueCapView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 

{

	// TODO: Add your specialized code here and/or call the base class

	ConfirmOleData(pDataObject);

	return mLastEffect;

}



BOOL CTrueCapView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 

{

	// TODO: Add your specialized code here and/or call the base class

	ConfirmOleData(pDataObject);

	if (!mDropCache) return false;

	if (mDropCache && mLastEffect == DROPEFFECT_NONE) return false;

	mDropCache = 0;

	mLastEffect = DROPEFFECT_NONE;

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Playback)

		{

		pDoc->SetPlaybackFileName(mDropFilename);

		pDoc->SetPlaybackMode();

		}

	else

		{

		mInhibitBlit = 1;

		pDoc->Stop();

		pDoc->SetPlaybackFileName(mDropFilename);

		pDoc->PrepareForSeek();

		mPLAYBACK_PlayButton.UseAltSet(pDoc->IsPlaying() && !pDoc->IsPaused());

		//DrawStaticFrame();

		ReconfigureSlider();

		mInhibitBlit = 0;

		CRect	r(VIDEO_X_OFFSET,VIDEO_Y_OFFSET,VIDEO_X_OFFSET + 320,VIDEO_Y_OFFSET + 240);

		InvalidateRect(&r);

		}

	return TRUE;

}



void CTrueCapView::OnDropFiles(HDROP hDropInfo) 

{

	// TODO: Add your message handler code here and/or call default

	

	CView::OnDropFiles(hDropInfo);

}



void CTrueCapView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 

{

	// TODO: Add your message handler code here and/or call default

	if (nChar == 16)

		mShiftKey = false;

	TRACE("key up %d\n",(int)nChar);

	CView::OnKeyUp(nChar, nRepCnt, nFlags);

}



void CTrueCapView::OnOptionsTimedrecord() 

{

	// TODO: Add your command handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->GetMode() != CTrueCapDoc::Capture) return;

	if (pDoc->IsCapturing()) return;



	CTimedRecordDialog dlog;

	

	dlog.mTime = 15;

	dlog.mSampleSize	= pDoc->mAudioCapture_SampleSize;

	dlog.mChannels		= pDoc->mAudioCapture_Channels;

	dlog.mFrequency		= pDoc->mAudioCapture_Frequency;

	dlog.mWidth			= pDoc->mVideoCapture_Width;

	dlog.mHeight		= pDoc->mVideoCapture_Height;

	dlog.mFPS			= pDoc->mVideoCapture_FPS;

	dlog.mQuality		= pDoc->mVideoCapture_Quality;

	dlog.mFilename		= pDoc->GetCaptureFileName();



	if (dlog.DoModal() == IDOK)

		{

		mTimedRecord = true;

		mStartTimedRecord = COleDateTime::GetCurrentTime();

		mTimedRecordLength = COleDateTimeSpan(0,0,dlog.mTime,0);

		pDoc->DoTimedRecord();

		}

}



void CTrueCapView::OnUpdateOptionsTimedrecord(CCmdUI* pCmdUI) 

{

	// TODO: Add your command update UI handler code here

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	pCmdUI->Enable(pDoc->GetMode() == CTrueCapDoc::Capture && !pDoc->IsCapturing());

}



void CTrueCapView::OnRButtonDown(UINT nFlags, CPoint point) 

{

	// TODO: Add your message handler code here and/or call default

	CTrueCapDoc *pDoc = GetDocument();

	ASSERT_VALID(pDoc);



	if (pDoc->IsFullScreenCapture())

		pDoc->ToggleFullScreen();

	else

		CView::OnRButtonDown(nFlags, point);

}

