/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// TrueCapView.h : interface of the CTrueCapView class

//

/////////////////////////////////////////////////////////////////////////////

#include "C3BitmapButton.h"

#include "CColorSlider.h"

#include "CTimeShiftManager.h"

#include "IVideoTuner.h"

#include <afxole.h>

#include <vector>
using namespace std;


class CTrueCapDoc;

#define VIDEO_X_OFFSET	21

#define VIDEO_Y_OFFSET	19



#define DROP_FILENAME_SIZE 1024



class CTrueCapView : public CView

{

protected: // create from serialization only

	CTrueCapView();

	DECLARE_DYNCREATE(CTrueCapView)



	int					mPendingChannel;

	IVideoTuner::ChannelList_t			mChannelList;

	int					mFirstUpdate;

	int					mPlayFullscreen;

	CTimeShiftManager	mTimeShiftManager;

	nmr_string			mDirectory;

	int					mInhibitBlit;

	bool				mCaptureFullScreen;

	COleDropTarget		mDropTarget;

	COleDataObject		*mDropCache;

	DROPEFFECT			mLastEffect;

	char				mDropFilename[DROP_FILENAME_SIZE+1];

	bool				mShiftKey;



	bool				mTimedRecord;

	COleDateTime		mStartTimedRecord;

	COleDateTimeSpan	mTimedRecordLength;





	//int		BlitVideoFrame(const char *filename,int frame,

	//			int x,int y,int width,int height);

	void	BlitStaticLogo(void);

	void	SetPlaybackFile(void);

	void	SetCaptureFile(void);

	void	ConfigureAudio(void);

	void	ConfigureVideo(void);

	void	RenameFile(void);

	void	Do_Play(int fullscreen = 0);

	void	Do_Stop(void);

	void	ConfirmOleData(COleDataObject *pDataObject);



// Attributes

public:

	CTrueCapDoc* GetDocument();



// Operations

public:

	void	CaptureFullScreen(bool fullscreen);

	void	HideCaptureButtons(void);

	void	HidePlaybackButtons(void);

	void	ShowCaptureButtons(void);

	void	ShowPlaybackButtons(void);

	void	DrawStaticFrame(void);

	void	GetVideoBoxXY(int &x,int &y,int width,int height);

	void	ReconfigureSlider(void);

	CColorSlider *GetSlider(void);

	C3BitmapButton	*GetPlayButton(void);

	void	CleanupAfterStop(void);

	void	LoadEvents(void);

	void	CaptureMode(void);

	void	PlaybackMode(void);

	void	CaptureStopped(void);

	void	CaptureStart(void);



// Overrides

	// ClassWizard generated virtual function overrides

	//{{AFX_VIRTUAL(CTrueCapView)

	public:

	virtual void OnDraw(CDC* pDC);  // overridden to draw this view

	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

	virtual DROPEFFECT OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);

	virtual void OnDragLeave();

	virtual DROPEFFECT OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);

	virtual BOOL OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point);

	protected:

	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);

	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);

	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);

	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);

	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);

	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);

	//}}AFX_VIRTUAL



// Implementation

public:

	virtual ~CTrueCapView();

#ifdef _DEBUG

	virtual void AssertValid() const;

	virtual void Dump(CDumpContext& dc) const;

#endif



protected:

	vector<C3BitmapButton*>	mCaptureButtons;

	vector<C3BitmapButton*>	mPlaybackButtons;



	CColorSlider	mPLAYBACK_Slider;

	C3BitmapButton	mPLAYBACK_FullscrnToggle;

	C3BitmapButton	mPLAYBACK_PlayButton;

	C3BitmapButton	mPLAYBACK_StopButton;;

	C3BitmapButton	mPLAYBACK_VCRButton,mPLAYBACK_SetFileButton,mPLAYBACK_RenameButton;

	C3BitmapButton	mPLAYBACK_SkipButton;

	C3BitmapButton	mCAPTURE_AudioButton,mCAPTURE_RecordButton,mCAPTURE_SetFileButton;

	C3BitmapButton	mCAPTURE_TVButton,mCAPTURE_VideoButton;

	C3BitmapButton	mCAPTURE_StillButton;

	C3BitmapButton	mCAPTURE_1,mCAPTURE_2,mCAPTURE_3,mCAPTURE_4,mCAPTURE_5;

	C3BitmapButton	mCAPTURE_6,mCAPTURE_7,mCAPTURE_8,mCAPTURE_9,mCAPTURE_0;

	C3BitmapButton	mCAPTURE_Up,mCAPTURE_Down,mCAPTURE_Enter;

	C3BitmapButton	mCAPTURE_FullscrnToggle;



	CBitmap			mCaptureBackground,mCaptureBackgroundSmall;

	CBitmap			mPlaybackBackground,mPlaybackBackgroundSmall;

	CBitmap			*mLastPlaybackBitmap;

	CBitmap			mLogoBitmap;

	CBitmap			mDigits[10];

	CBitmap			mNoTunerBitmap;



	void	ClearPendingChannel(void);

	void	ChannelDigit(int d);

	void	ChannelUp(void);

	void	ChannelDown(void);

	void	ChannelEnter(void);

	void	DrawChannel(void);

	void	_DrawChannel(int channel);

	void	InvalidateChannels(void);

	int		mChannelColor;

	int		mChannelTimerActive;

	// Generated message map functions

protected:

	//{{AFX_MSG(CTrueCapView)

	afx_msg void OnDestroy();

	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);

	afx_msg void OnOptionsAudio();

	afx_msg void OnUpdateOptionsAudio(CCmdUI* pCmdUI);

	afx_msg void OnOptionsVideo();

	afx_msg void OnUpdateOptionsVideo(CCmdUI* pCmdUI);

	afx_msg void OnFileSetcapturefile();

	afx_msg void OnUpdateFileSetcapturefile(CCmdUI* pCmdUI);

	afx_msg void OnFileSetplaybackfile();

	afx_msg void OnUpdateFileSetplaybackfile(CCmdUI* pCmdUI);

	afx_msg void OnFileRenamefile();

	afx_msg void OnUpdateFileRenamefile(CCmdUI* pCmdUI);

	afx_msg BOOL OnEraseBkgnd(CDC* pDC);

	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);

	afx_msg void OnTimer(UINT nIDEvent);

	afx_msg void OnOptionsEditevents();

	afx_msg void OnUpdateOptionsEditevents(CCmdUI* pCmdUI);

	afx_msg void OnOptionsChannelscan();

	afx_msg void OnUpdateOptionsChannelscan(CCmdUI* pCmdUI);

	afx_msg void OnFileUpgrade();

	afx_msg void OnUpdateFileUpgrade(CCmdUI* pCmdUI);

	afx_msg void OnDropFiles(HDROP hDropInfo);

	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);

	afx_msg void OnOptionsTimedrecord();

	afx_msg void OnUpdateOptionsTimedrecord(CCmdUI* pCmdUI);

	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);

	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

};



#ifndef _DEBUG  // debug version in TrueCapView.cpp

inline CTrueCapDoc* CTrueCapView::GetDocument()

   { return (CTrueCapDoc*)m_pDocument; }

#endif



/////////////////////////////////////////////////////////////////////////////

