/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* Put up duck logo in lower R corner of video image.  David Silver 5/24/97 */

#include "ducklogo.h"

#include "duck_dxl.h"



enum BITDEPTH lastTableBD = DXRGBNULL;

unsigned char logoBlitTable[4640];



void duck_logo( unsigned char *dst, int width, int height, int pitch,

					   enum BITDEPTH format, int dci, enum BLITQUALITY bq)

{

	int i,j,jj;

	unsigned short int *srcw,*dstw;

	unsigned char *src;

	short int pitchw;

    unsigned char *tableDst;



	if(bq == DXBLIT_STRETCH || bq == DXBLIT_STRETCH_BRIGHT){

		pitch /= 2; /* convert down doubled screen pitch */

		width *= 2; /* convert up doubled width */

		height *= 2; /* convert up doubled height */

	}



	/* return if image is too small for logo */

	if( width < 48 || height < 48)

		return;



    /* if last bit depth used is not the same, then build a new table */

    if(lastTableBD != format) {

        tableDst = &logoBlitTable[0];



	    switch(format) {

		    case DXRGBNULL:

			    return;



		    case DXRGB32:

			    src = (char *)tmlogo24;

			    for(i = 0; i < 29; i++) {

				    for (j = 0,jj = 0; j < 40; j++,jj+=4) {

					    tableDst[jj] = tableDst[jj+1] = tableDst[jj+2] = src[j];

					    tableDst[jj+3] = 0;

				    }

				    src += 40;

				    tableDst += 40*4;

			    }

                lastTableBD = DXRGB32;

			    break;



		    case DXRGB24:

#if duckCOLOR_BORDER

              /* add color border */

			    for(i = 0; i < 29; i++) {

				    for (j = 0,jj = 0; j < 40; j++,jj+=3) {

				        tableDst[jj] = (unsigned char)128;

                        tableDst[jj+1] = (unsigned char)128;

                        tableDst[jj+2] = (unsigned char)256;

				    }

				    tableDst += 40*3;

			    }



                tableDst = &logoBlitTable[0];

		    tableDst += 40*3;

 			    src = (char *)tmlogo24;

		    src += 40;

			    for(i = 0; i < 27; i++) {

				    for (j = 1,jj = 3; j < 40-1; j++,jj+=3) {

				        tableDst[jj] = tableDst[jj+1] = tableDst[jj+2] = src[j] ;

				    }

				    src += 40;

				    tableDst += 40*3;

			    }

                lastTableBD = DXRGB24;

#else

 			    src = (char *)tmlogo24;

			    for(i = 0; i < 29; i++) {

				    for (j = 0,jj = 0; j < 40; j++,jj+=3) {

				        tableDst[jj] = tableDst[jj+1] = tableDst[jj+2] = src[j] ;

				    }

				    src += 40;

				    tableDst += 40*3;

			    }

                lastTableBD = DXRGB24;

#endif

  

			    break;



		    case DXRGB16_555:

			    srcw = (unsigned short *)tmlogo555;

                dstw = (unsigned short *)&logoBlitTable[0];

                lastTableBD = DXRGB16_555;



			    pitchw = pitch/2;

			    for(i = 0; i < 29; i++) {

				    for (j = 0; j < 40; j++) {

					    dstw[j] = srcw[j];

				    }

				    srcw += 40;

				    dstw += 40;

			    }

			    break;



		    case DXRGB16_565:

			    srcw = (unsigned short *)tmlogo565;

                dstw = (unsigned short *)&logoBlitTable[0];

                lastTableBD = DXRGB16_565;



			    pitchw = pitch/2;

			    for(i = 0; i < 29; i++) {

				    for (j = 0; j < 40; j++) {

					    dstw[j] = srcw[j];

				    }

				    srcw += 40;

				    dstw += 40;

			    }

			    break;



#if 0 /* for now */

		    case DXRGBYVYU:

			    src = (char *)tmlogo24;

			    dst += (width-44) * 2 + (height-33) * pitch;

			    for(i = 0; i < 29; i++) {

				    for (j = 0,jj = 0; j < 40; j++,jj+=2) {

					    dst[jj] = 128;

					    dst[jj+1] = src[j];

				    }

				    src += 40;

				    dst += pitch;

			    }

			    break;



		    case DXRGBYUY2:

			    src = (char *)tmlogo24;

			    dst += (width-44) * 2 + (height-33) * pitch;

			    for(i = 0; i < 29; i++) {

				    for (j = 0,jj = 0; j < 40; j++,jj+=2) {

					    dst[jj] = src[j];

					    dst[jj+1] = 128;

				    }

				    src += 40;

				    dst += pitch;

			    }

			    break;

#endif



		    default:

			    // ERR( "duck_logo::Unsupported output pixel format " << pxf)

			    return;

	    }

    } /* if currentTableOK */



    

	if(!dci) {

		dst += (height-1) * pitch;

		pitch = -pitch;

	}



    /* blit logo table */

    {

        unsigned long *longDst = (unsigned long *)dst;

        unsigned long *tableSrc = (unsigned long *)&logoBlitTable[0];

	    int longPitch = pitch/4;



        switch(format) {

		    case DXRGBNULL:

			    return;



	        case DXRGB32:

		        longDst += (width-44) + (height-33) * longPitch;

            

		        for(i = 0; i < 29; i++) {

			        for (j = 0; j < 40; j++) {

				        longDst[j] = tableSrc[j];

			        }

			        tableSrc += 40;

			        longDst += longPitch;

		        }

		        break;



		    case DXRGB24:

			    longDst += ((width-44)*3)/4 + (height-33)* (longPitch);

			    for(i = 0; i < 29; i++) {

				    for (j = 0; j < 30; j++) {

					    longDst[j] = tableSrc[j];

				    }

				    tableSrc += 30;

				    longDst += longPitch;

			    } 

			    break;



		    case DXRGB16_555:

			    srcw = (unsigned short *)&logoBlitTable[0];

                lastTableBD = DXRGB16_555;

			    dstw = (unsigned short *)dst;

			    pitchw = pitch/2;



			    dstw += (width-44) + (height-33) * pitchw;



			    for(i = 0; i < 29; i++) {

				    for (j = 0; j < 40; j++) {

					    dstw[j] = srcw[j];

				    }

				    srcw += 40;

				    dstw += pitchw;

			    }

			    break;



		    case DXRGB16_565:

			    srcw = (unsigned short *)&logoBlitTable[0];

                lastTableBD = DXRGB16_565;



			    dstw = (unsigned short *)dst;

			    pitchw = pitch/2;



			    dstw += (width-44) + (height-33) * pitchw;



			    for(i = 0; i < 29; i++) {

				    for (j = 0; j < 40; j++) {

					    dstw[j] = srcw[j];

				    }

				    srcw += 40;

				    dstw += pitchw;

			    }

			    break;

            default:

                break;

        }                    

    } /* blit logo table */

}

