/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* File: Mytypes.h

 * Purpose: Defines some types that either windows.h or vtoolsd.h don't define

 * Author: sasha

 * $Log: mytypes.h,v $
 * Revision 1.1  1999/08/27 02:52:32  nradisch
 * new
 *

 * Revision 2.5  1997/07/11 21:12:58  trantq

 * Add more changes to make Microsoft compatible.

 * Revision 2.3  1997/07/10 17:16:00  trantq

 * Make Grappler module compile under Microsoft.

 * Revision 2.2  1997/03/20 22:22:51  dtharp

 * Grafted WDM buffer management branch.

 * Revision 2.1.1.2  1997/03/20 22:22:10  dtharp

 * Tweaked wdm mytypes.h for vxd.

 * Revision 2.1  1996/06/10 20:13:28  ssm

 * MarkBeta1AsRev2

 * Revision 1.7  1996/03/11 21:19:14  sasha

 * Moved ErrorCode typedef into retcode.h

 * Revision 1.6  1996/02/08 23:43:08  sasha

 * Changed queue typedef

 * Revision 1.5  1996/01/02 19:11:04  sasha

 * Added ERROR_CODE_DEFINED macro

 * Revision 1.4  1995/12/08 22:29:14  sasha

 * Added couple empty lines

 * Revision 1.3  1995/10/16 23:29:30  sasha

 * First major check-in for testing

 * Revision 1.2  1995/09/30 22:13:26  sasha

 * Just to make Mike happy

 * Revision 1.1  1995/09/08 00:15:48  sasha

 * Initial revision

 */



#ifndef __MYTYPES_H

#define __MYTYPES_H



#ifdef __BORLANDC__ 			// Borland's class libraries - TT

	#ifndef __DEFS_H

	#include <classlib/defs.h>

	#endif



	#ifndef CLASSLIB_QUEUES_H

	#include <classlib/queues.h>

	#endif



	/* Type: VidBufQueue

 	 * Purpose: Queue of buffers to be sent to BtPisces

 	 */

	typedef TQueue<LPVOID> VidBufQueue;

	typedef TQueueIterator<LPVOID> VidBufQueueIter;



	#if (__BORLANDC__ < 0x500)  	// Check for earlier versions

	template<class T> inline bool ToBool(const T& t) {

   	return static_cast<bool>(t != 0);

	}

	#endif



#else	// Microsoft

	#include "msvccomp.h"

#endif  // Borland





#ifndef __RETCODE_H

#include "retcode.h"

#endif



#ifdef TESTING

    #define PVOID LPVOID

#else

    #ifdef VXD

    typedef PDWORD LPDWORD;

    typedef PBYTE  LPBYTE;

    typedef PWORD  LPWORD;

    typedef PVOID  LPVOID;

    #endif

#endif



#ifdef VXD

typedef struct _RGNDATAHEADER {

    DWORD   dwSize;

    DWORD   iType;

    DWORD   nCount;

    DWORD   nRgnSize;

    RECT    rcBound;

} RGNDATAHEADER, *PRGNDATAHEADER;



typedef struct _RGNDATA {

    RGNDATAHEADER   rdh;

    char            Buffer[1];

} RGNDATA, *PRGNDATA;

#endif





class TSize;

class TRect;





class  TPoint : public tagPOINT {

  public:

    // Constructors

    TPoint() {}

    TPoint(int _x, int _y) {x = _x; y = _y;}

    TPoint(const POINT far& point) {x = point.x; y = point.y;}

    TPoint(const TPoint far& point) {x = point.x; y = point.y;}

    TPoint(const SIZE far& size) {x = size.cx; y = size.cy;}

    TPoint( DWORD dw) {x = WORD(LOWORD(dw)); y = WORD(HIWORD(dw));}



    // Information functions/operators

    bool        operator ==(const TPoint& other) const;

    bool        operator !=(const TPoint& other) const;



    // Functions/binary-operators that return points or sizes

    TPoint      OffsetBy(int dx, int dy) const {return TPoint(x+dx, y+dy);}

    TPoint      operator +(const TSize& size) const;

    TSize       operator -(const TPoint& point) const;

    TPoint      operator -(const TSize& size) const;

    TPoint      operator -() const {return TPoint(-x, -y);}



    // Functions/assignement-operators that modify this point

    TPoint&     Offset(int dx, int dy);

    TPoint&     operator +=(const TSize& size);

    TPoint&     operator -=(const TSize& size);

};



//

// class TSize

// ----- -----

//

class  TSize : public tagSIZE {

  public:

    // Constructors

    TSize() {}

    TSize(int dx, int dy) {cx = dx; cy = dy;}

    TSize(const POINT far& point) {cx = point.x; cy = point.y;}

    TSize(const SIZE far& size) {cx = size.cx; cy = size.cy;}

    TSize(const TSize far& size) {cx = size.cx; cy = size.cy;}

    TSize(DWORD dw) {cx = LOWORD(dw); cy = HIWORD(dw);}



    // Information functions/operators

    bool        operator ==(const TSize& other) const;

    bool        operator !=(const TSize& other) const;



    // Functions/binary-operators that return sizes

    TSize      operator +(const TSize& size) const;

    TSize      operator -(const TSize& size) const;

    TSize      operator -() const {return TSize(-cx, -cy);}



    // Functions/assignement-operators that modify this size

    TSize&     operator +=(const TSize& size);

    TSize&     operator -=(const TSize& size);

};





class  TRect : public tagRECT {

  public:

    // Constructors

    TRect() {}

    TRect(const RECT far& rect);

    TRect(const TRect far& rect);

    TRect(int _left, int _top, int _right, int _bottom);

    TRect(const TPoint& upLeft, const TPoint& loRight);

    TRect(const TPoint& origin, const TSize& extent);



    // (re)Initializers

    void        SetNull();

    void        SetEmpty() {SetNull();}

    void        Set(int _left, int _top, int _right, int _bottom);



    // Type Conversion operators

    operator    const TPoint*() const {return (const TPoint*)this;}

    operator    TPoint*() {return (TPoint*)this;}



    // Testing functions

    bool        IsEmpty() const;

    bool        IsNull() const;

    bool        operator ==(const TRect& other) const;

    bool        operator !=(const TRect& other) const;



    // Information/access functions(const and non-const)

    const TPoint& TopLeft() const {return *(TPoint*)&left;}

    TPoint&     TopLeft() {return *(TPoint*)&left;}

    TPoint      TopRight() const {return TPoint(right, top);}

    TPoint      BottomLeft() const {return TPoint(left, bottom);}

    const TPoint& BottomRight() const {return *(TPoint*)&right;}

    TPoint&     BottomRight() {return *(TPoint*)&right;}

    int         Width() const {return right-left;}

    int         Height() const {return bottom-top;}

    TSize       Size() const {return TSize(Width(), Height());}

    long        Area() const {return long(Width())*long(Height());}



    bool        Contains(const TPoint& point) const;

    bool        Contains(const TRect& other) const;

    bool        Touches(const TRect& other) const;

    TRect       OffsetBy(int dx, int dy) const;

    TRect       operator +(const TSize& size) const;

    TRect       operator -(const TSize& size) const;

    TRect       InflatedBy(int dx, int dy) const;

    TRect       InflatedBy(const TSize& size) const;

    TRect       operator &(const TRect& other) const;

    TRect       operator |(const TRect& other) const;



    // Manipulation functions/operators

    TRect&      Offset(int dx, int dy);

    TRect&      operator +=(const TSize& delta);

    TRect&      operator -=(const TSize& delta);

    TRect&      Inflate(int dx, int dy);

    TRect&      Inflate(const TSize& delta);

    TRect&      operator &=(const TRect& other);

    TRect&      operator |=(const TRect& other);

};



//----------------------------------------------------------------------------

// Inlines

//----------------------------------------------------------------------------



inline bool TPoint::operator ==(const TPoint& other) const {  

	bool bRet = bool( other.x==x && other.y==y );

  return bRet;

}



inline bool TPoint::operator !=(const TPoint& other) const {

	bool bRet = bool( other.x!=x || other.y!=y );

  return bRet;

}



inline TPoint TPoint::operator +(const TSize& size) const {

  return TPoint(x+size.cx, y+size.cy);

}



inline TSize TPoint::operator -(const TPoint& point) const {

  return TSize(x-point.x, y-point.y);

}



inline TPoint TPoint::operator -(const TSize& size) const {

  return TPoint(x-size.cx, y-size.cy);

}



inline TPoint& TPoint::Offset(int dx, int dy) {

  x += dx;

  y += dy;

  return *this;

}



inline TPoint& TPoint::operator +=(const TSize& size) {

  x += size.cx;

  y += size.cy;

  return *this;

}



inline TPoint& TPoint::operator -=(const TSize& size) {

  x -= size.cx;

  y -= size.cy;

  return *this;

}





inline bool TSize::operator ==(const TSize& other) const {

  bool bRet = bool( other.cx==cx && other.cy==cy );

  return bRet;

}



inline bool TSize::operator !=(const TSize& other) const {

  bool bRet = bool( other.cx!=cx || other.cy!=cy );

  return bRet;

}



inline TSize TSize::operator +(const TSize& size) const {

  return TSize(cx+size.cx, cy+size.cy);

}



inline TSize TSize::operator -(const TSize& size) const {

  return TSize(cx-size.cx, cy-size.cy);

}



inline TSize& TSize::operator +=(const TSize& size) {

  cx += size.cx;

  cy += size.cy;

  return *this;

}



inline TSize& TSize::operator -=(const TSize& size) {

  cx -= size.cx;

  cy -= size.cy;

  return *this;

}







inline void TRect::SetNull() {

  left = top = right = bottom = 0;

}



inline void TRect::Set(int _left, int _top, int _right, int _bottom) {

  left = _left;

  top = _top;

  right = _right;

  bottom = _bottom;

}



inline TRect::TRect(const RECT far& rect) {

  *(RECT far*)this = rect;

}



inline TRect::TRect(const TRect far& rect) {

  *(RECT far*)this = *(RECT far*)&rect;

}



inline TRect::TRect(int _left, int _top, int _right, int _bottom) {

  Set(_left, _top, _right, _bottom);

}



inline TRect::TRect(const TPoint& topLeft, const TPoint& bottomRight) {

  Set(topLeft.x, topLeft.y, bottomRight.x, bottomRight.y);

}



inline TRect::TRect(const TPoint& origin, const TSize& extent) {

  Set(origin.x, origin.y, origin.x+extent.cx, origin.y+extent.cy);

}



inline bool TRect::IsEmpty() const {

  bool bRet = bool( left>=right || top>=bottom );

  return bRet;

}



inline bool TRect::IsNull() const {

  bool bRet = bool( !left && !right && !top && !bottom );

  return bRet;

}



inline bool TRect::operator ==(const TRect& other) const {

  bool bRet = bool( other.left==left && other.top==top && 

  					other.right==right && other.bottom==bottom );   

  return bRet;             

}



inline bool TRect::operator !=(const TRect& other) const { 

  bool bRet = bool( !(other==*this) );  

  return bRet;

}



inline bool TRect::Contains(const TPoint& point) const {

  bool bRet = bool( point.x >= left && point.x < right && 

  					point.y >= top  && point.y < bottom );

  return bRet;

             

}



inline bool TRect::Contains(const TRect& other) const {

  bool bRet = bool( other.left >= left && other.right <= right && 

  					other.top >= top   && other.bottom <= bottom );

  return bRet;

}



inline bool TRect::Touches(const TRect& other) const {

  bool bRet = bool( other.right > left && other.left < right && 

  					other.bottom > top && other.top < bottom );

  return bRet;

}



inline TRect TRect::OffsetBy(int dx, int dy) const {

  return TRect(left+dx, top+dy, right+dx, bottom+dy);

}



inline TRect TRect::operator +(const TSize& size) const {

  return OffsetBy(size.cx, size.cy);

}



inline TRect TRect::operator -(const TSize& size) const {

  return OffsetBy(-size.cx, -size.cy);

}



inline TRect TRect::InflatedBy(int dx, int dy) const {

  return TRect(left-dx, top-dy, right+dx, bottom+dy);

}



inline TRect TRect::InflatedBy(const TSize& size) const {

  return InflatedBy(size.cx, size.cy);

}





inline TRect& TRect::operator +=(const TSize& delta) {

  Offset(delta.cx, delta.cy);

  return *this;

}



inline TRect& TRect::operator -=(const TSize& delta) {

  return *this += -delta;

}



inline TRect& TRect::Inflate(const TSize& delta) {

  return Inflate(delta.cx, delta.cy);

}   





#endif

















