/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* Regutil.h

 * Purpose: Contains definition Windows registry functions wrapper class

 * Author: sasha

 * $Log: regutil.h,v $
 * Revision 1.1  1999/08/27 02:52:32  nradisch
 * new
 *

 * Revision 2.2  1997/07/10 00:20:08  trantq

 * Need to include string.h if compile with MSVC.

 * Revision 2.1  1996/06/10 20:13:29  ssm

 * MarkBeta1AsRev2

 * Revision 1.2  1995/09/30 22:12:58  sasha

 * Just to make Mike happy

 * Revision 1.1  1995/08/28 04:55:10  sasha

 * Initial revision

 */



#ifndef __REGUTIL_H

#define __REGUTIL_H



#ifndef __BORLANDC__ 	// Microsoft

#include <string.h>

#endif



#ifndef  __STDIO_H

#include <stdio.h>

#endif



#ifndef __SHELLAPI_H

#include <shellapi.h>

#endif



#ifndef __MYTYPES_H

#include "mytypes.h"

#endif



#ifndef __VIDDEFS_H

#include "viddefs.h"

#endif



/* Class: RegUtil

 * Purpose: Simplifies registry access

 */

class RegUtil

{

   private:

      HKEY hKey_;



   public:

      RegUtil( LPCSTR SubKeyName, HKEY hMainKey = HKEY_CLASSES_ROOT );

      ~RegUtil() { if ( hKey_ ) RegCloseKey( hKey_ ); }

      void SetValue( DWORD dwValue, LPCSTR szSubKey = "" );

      void SetValue( LPCSTR szValue, LPCSTR szSubKey = "" );

      ErrorCode GetValue( LPDWORD lpValue, LPCSTR szSubKey = "" );

      ErrorCode GetValue( LPSTR   lpValue, LPLONG lpByteCount, LPCSTR szSubKey = "" );

};



/* Constructor: RegUtil::RegUtil

 * Purpose: Create a RegUtil object

 * Input: szSubKeyName: LPCSTR - name of a subkey to create/open

 *   hMainKey: HKEY - handle of a main key. Defaults to HKEY_CLASSES_ROOT

 * Output: None

 */

inline RegUtil::RegUtil( LPCSTR szSubKeyName, HKEY hMainKey ) : hKey_( 0 )

{

   RegCreateKey( hMainKey, szSubKeyName, &hKey_ );

}



/* Method: RegUtil::SetValue

 * Purpose: Sets a value in the registry

 * Input:  dwValue: DWORD value to store

 *   lpszSubKey: LPCSTR - name of a subkey

 * Output: None

 */

inline void RegUtil::SetValue( DWORD dwValue, LPCSTR lpszSubKey )

{

   char str [25];

   sprintf( str, "%ld", dwValue );



   SetValue( str, lpszSubKey );

}



/* Method: RegUtil::SetValue

 * Purpose: Sets a value in the registry

 * Input:  szValue: LPCSTR - value to store

 *   lpszSubKey: LPCSTR - name of a subkey

 * Output:

 */

inline void RegUtil::SetValue( LPCSTR szValue, LPCSTR lpszSubKey )

{

   RegSetValue( hKey_,   /* handle of the main key         */

    lpszSubKey,            /* name of subkey                 */

    REG_SZ,              /* required                       */

    szValue,                 /* command to activate server     */

    strlen( szValue ) );     /* text string size               */

}



/* Method: RegUtil::GetValue

 * Purpose: Retrieves values from the registry

 * Input: lpValue: LPSTR - far pointer to a DWORD to recieve value

 *    lpByteCount: LPLONG - pointer to a variable that holds the size of the

 *    buffer pointed to by lpValue

 *    lpszSubKey: LPCSTR - sub key name

 * Output: ErrorCode: Success if value is found. Fail otherwise

 */

inline ErrorCode RegUtil::GetValue( LPSTR lpValue, LPLONG lpByteCount, LPCSTR lpszSubKey )

{

   return

   RegQueryValue( hKey_, lpszSubKey, lpValue, lpByteCount ) == ERROR_SUCCESS ?

   Success : Fail;

}



/* Method: RegUtil::GetValue

 * Purpose: Retrieves values from the registry

 * Input: lpValue: LPDWORD - far pointer to a DWORD to recieve value

 *    szSubKey: LPCSTR - sub key name

 *    lpByteCount: LPLONG - pointer to a variable that holds the size of the

 *    buffer pointed to by lpValue

 * Output: ErrorCode: Success if value is found. Fail otherwise

 */

inline ErrorCode RegUtil::GetValue( LPDWORD lpValue, LPCSTR lpszSubKey )

{

   const LONG BufLen = 11;

   char  ValueStr [BufLen];

   LONG  lBC = BufLen;

   if ( GetValue( ValueStr, &lBC, lpszSubKey ) == Success ) {

      *lpValue = atol( ValueStr );

      return Success;

   }

   return Fail;

}



#endif







