/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* File: Viddefs.h

 * Purpose: Contains different enums for the BtPisces capture chip

 * Copyright 1996 Brooktree Corporation

 * Author: sasha

 * $Log: viddefs.h,v $
 * Revision 1.1  1999/08/27 02:52:33  nradisch
 * new
 *

 * Revision 2.3  1997/05/02 21:31:04  dtharp

 * 848A Merge: from 2.1.1.4

 * Revision 2.2  1996/12/06 22:33:08  meierdw

 * Grafted 2.1.1 into trunk; NTSC-Japan support added.

 * Revision 2.1.1.2  1996/11/08 18:51:43  rudyl

 * Added NTSC-Japan, PAL-M & PAL-N formats.

 * Pisces+: added PAL_NC format.

 * Revision 2.1.1.1  1996/11/07 22:06:42  rudyl

 * Revision 2.1  1996/06/10 20:13:29  ssm

 * MarkBeta1AsRev2

 * Revision 1.9  1996/04/01 21:04:36  sasha

 * Added 'Con' to the connector enums

 * Revision 1.8  1996/04/01 20:53:38  sasha

 * Changed name back to Connector

 * Revision 1.7  1996/04/01 20:47:52  rudyl

 * Changed connector type to VideoSource (SVideo, Tuner, and Composite).

 * Revision 1.6  1996/03/29 17:32:00  rudyl

 * Changed Connector enum values back to Conn1=1, etc. Not mapping to mux.

 * Revision 1.5  1996/03/28 17:33:25  rudyl

 * Updated some and added more enums for the decoder.

 * Revision 1.4  1996/01/29 02:23:54  sasha

 * Changed enum values for the Connector to reflect real values put into a register

 * Revision 1.3  1995/09/30 14:10:20  sasha

 * First major check-in for testing

 * Revision 1.2  1995/09/30 06:10:20  sasha

 * Just to make Mike happy

 * Revision 1.1  1995/08/28 02:29:58  sasha

 * Initial revision

 */





/* CUT_HERE */



/* File: Viddefs.h */



#ifndef __VIDDEFS_H

#define __VIDDEFS_H



#if defined(VXD) && !defined(CLASSLIB_DEFS_H)

#include <classlib/defs.h>

#endif





/* Type: Connector

 * Purpose: Defines a video source

 */

typedef enum { ConCamera, ConSVideo, ConTuner, ConComposite } Connector;



/* Type: State

 * Purpose: used to define on-off operations

 */

typedef enum { Off, On } State;



/* Type: Level

 * Purpose: used to define a pin state

 */

typedef enum { Low, Hi } Level;



/* Type: Field

 * Purpose: defines fields

 */

typedef enum { VF_Both, VF_Even, VF_Odd } VidField;



/* Type: VideoFormat for BT848A

 * Purpose: Defines video format

 */

typedef enum {  VFormat_AutoDetect,

                VFormat_NTSC,

                VFormat_NTSC_J,

                VFormat_PAL_BDGHI,

                VFormat_PAL_M,

                VFormat_PAL_N,

                VFormat_SECAM,

                VFormat_PAL_NC } VideoFormat;



inline bool IsNTSCCompatible( VideoFormat vf )

{

   return ( ( vf == VFormat_NTSC   )   ||

            ( vf == VFormat_NTSC_J )   ||

            ( vf == VFormat_PAL_M  ) ) ? true : false;

}



// Note PAL-NC does not belong to either video format group



inline bool IsPALCompatible( VideoFormat vf )

{

   return ( ( vf == VFormat_PAL_BDGHI )   ||

            ( vf == VFormat_PAL_N     )   ||

            ( vf == VFormat_SECAM     ) ) ? true : false;

}



/* Type: LumaRange

 * Purpose: Used to define Luma Output Range

 */

typedef enum { LumaNormal, LumaFull } LumaRange;



/* Type: OutputRounding

 * Purpose: Controls the number of bits output

 */

typedef enum { RND_Normal, RND_6Luma4Chroma, RND_7Luma5Chroma } OutputRounding;



/* Type: ClampLevel

 * Purpose: Defines the clamp levels

 */

typedef enum { ClampLow, ClampMiddle, ClampNormal, ClampHi } ClampLevel;





/*

 * Type: Crystal

 * Purpose: Defines which crystal to use

 */

typedef enum { Crystal_XT0 = 1, Crystal_XT1, Crystal_AutoSelect } Crystal;





/*

 * Type: HoriFilter

 * Purpose: Defines horizontal low-pass filter

 */

typedef enum { HFilter_AutoFormat,

               HFilter_CIF,

               HFilter_QCIF,

               HFilter_ICON } HorizFilter;



/*

 * Type: CoringLevel

 * Purpose: Defines Luma coring level

 */

typedef enum { Coring_None,

               Coring_8,

               Coring_16,

               Coring_32 } CoringLevel;



/*

 * Type: ThreeState

 * Purpose: Defines output three-states for the OE pin

 */

typedef enum { TS_Timing_Data,

               TS_Data,

               TS_Timing_Data_Clock,

               TS_Clock_Data } ThreeState;



/*

 * Type: SCLoopGain

 * Purpose: Defines subcarrier loop gain

 */

typedef enum { SC_Normal, SC_DivBy8, SC_DivBy16, SC_DivBy32 } SCLoopGain;



/*

 * Type: ComparePt

 * Purpose: Defines the majority comparison point for the White Crush Up function

 */

typedef enum { CompPt_3Q, CompPt_2Q, CompPt_1Q, CompPt_Auto } ComparePt;



/* Type: TimeGenByte

 * Purpose: Defines available timing generator load bytes

 */

typedef enum { TGB_NTSC624, TGB_NTSC780, TGB_NTSC858, TGB_Limit } TimeGenByte;



/* Type: PLLFreq

 * Purpose: Defines supported synthesized frequencies

 */

typedef enum { NoPLL, KHz28636, KHz35469, KHz27000, KHz24545, KHz29500, KHz19636, PLL_Limit } PLLFreq;



/* Type: SignalFormat

 * Purpose: Defines video signal format

 */

typedef enum {  Signal_Analog,

                Signal_CCIR656,

                Signal_ByteStream,

                Signal_Reserved3,

                Signal_ExtHSYNC_VSYNC,

                Signal_ExtHSYNC_FIELD } SignalFormat;



/* Type: SyncVideoRef

 * Purpose: Defines synchrous video reference

 */

typedef enum {  VRef_alignedCb,

                VRef_alignedY0,

                VRef_alignedCr,

                VRef_alignedY1 } SyncVideoRef;



/* Type: DecoderInputClock

 * Purpose: Defines decoder input clock select

 */

typedef enum {  DecClk_XTAL,

                DecClk_PLL,

                DecClk_GPCLK,

                DecClk_invGPCLK } DecInputClk;



/* Type: GPCLKOutputClock

 * Purpose: Defines GPCLK output clock select

 */

typedef enum {  GPCLK_CLKx1,

                GPCLK_Xtal0,

                GPCLK_PLL,

                GPCLK_invPLL } GPCLKOutputClk;



/*

 * Type: PeakFilter

 * Purpose: Defines peaking filters

 */

typedef enum { PeakFilter_Min = 1,

               PeakFilter_Med,

               PeakFilter_Max } PeakFilter;



/////////////////////////////////////////////////////////////////////////////

// PLL register values to generate certain synthesize frequency

/////////////////////////////////////////////////////////////////////////////

//typedef struct tag_PLLPARMS

//{

//   BYTE f_lo;     // PLL_F_LO

//   BYTE f_hi;     // PLL_F_HI

//   BYTE i;        // PLL_I

//   State c;       // PLL_C

//   State x;       // PLL_X

//} PLLPARMS;



// video sizes related to each camera

typedef struct tag_VSIZEPARMS

{

   WORD HActive;              // number of active horizontol pixels (Clkx1_HACTIVE)

   WORD HDelay;               // number of pixels delayed before actual signal (Clkx1_HDELAY)

   WORD VActive;              // VActive register value

   WORD VDelay;               // VDelay register value

   WORD linesPerField;        // number of active lines in each field

} VSIZEPARMS;





#endif // __VIDDEFS

