/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// $Header: /export/raid/cvs_videodev/TrueCap/VideoTuners/Bt848V2_2/Source/Grappler/Interfac.cpp,v 1.1 1999/08/27 02:52:35 nradisch Exp $

// $Log: Interfac.cpp,v $
// Revision 1.1  1999/08/27 02:52:35  nradisch
// new
//

// Revision 2.7  1997/07/18 00:29:06  rudyl

// Define _BUILD_TVCTLDLL here instead of in makefile.

// Revision 2.6  1997/07/16 18:11:28  rudyl

// Use last 848 release as head rev. Incorporate Gemini changes later.

// Revision 2.2  1996/06/14 21:03:07  rudyl

// Fixed problem with not unable to change channel above 13 for cable.

// Since cable and antenna both connect to same connector, needs to remember which tuner source to tune to when change channel.

// Revision 2.1  1996/06/10 20:12:32  ssm

// MarkBeta1AsRev2

// Revision 1.13  1996/05/30 19:05:26  rudyl

// 0.54 - Due to removal of vxd related calls from Bt848api.h to vxdcall.h, added include "vxdcall.h" here.

// Revision 1.12  1996/05/22 18:28:42  rudyl

// Added call to VxD when channel is about to change.

// Revision 1.11  1996/05/22 00:45:21  rudyl

// Moved VxD calls from here to Bt848_32.dll in order to hide the calls.

// Revision 1.10  1996/05/10 23:15:47  rudyl

// Implement TV_ERR_... return values from Intel. Also added exported function GetTVErrString() to retrieve our specific error msg.

// Revision 1.9  1996/05/06 18:49:05  rudyl

// No changes; just added more comments into codes.

// Revision 1.8  1996/04/28 01:20:12  rudyl

// ver 0.50 - does not instantiate I2C and GPIO objects here. Just make API

// calls to BT848_32.DLL. Also returns error types to caller instead of just TRUE/FALSE result.

// Revision 1.7  1996/04/23 22:10:53  rudyl

// Removed codes inside SetAudio() and GetAudio() from GPIO to SetAudioSource() and GetAudioSource() here.

// Revision 1.6  1996/04/18 01:20:29  rudyl

// set audio to TV tuner when DLL starts and to mute when DLL ends

// Revision 1.5  1996/04/17 20:26:04  rudyl

// fixed no audio problem. Need to set GPOE to 0x0F (instead of 0x03). Don't know why.

// No changes in source here. Only in gpio.cpp.

// Revision 1.4  1996/04/16 22:00:00  rudyl

// Removed mapping function from here. Moved to decoder class.

// Revision 1.3  1996/04/16 18:18:33  rudyl

// Added range mapping from 0-255 to our own ranges for Brightness, Contrast, Saturation and Hue.

// Revision 1.2  1996/04/12 21:01:29  rudyl

// Renamed DLL from 848Tuner.dll to Grappler.dll.

// Added SetSaturation() and GetSaturation().

// Revision 1.1  1996/04/12 18:24:43  rudyl

// Initial revision



// CUT_HERE



/////////////////////////////////////////////////////////////////////////////

//

//  Copyright (c) 1996 Brooktree Corporation

//

//  Module:

//

//    Interfac.cpp

//

//  Abstract:

//

//    Contains all interface functions to the Bt848 Grappler DLL

//

/////////////////////////////////////////////////////////////////////////////



#include <windows.h>



#define _BUILD_TVCTLDLL    // all __TVCTLFUNC functions are exported



#include "tvctl.h"

#include "tverr.h"

#include "tuner.h"

#include "audio.h"

#include "bt848api.h"

#include "bt848fcn.h"





/////////////////////////////////////////////////////////////////////////////

// Parameter constants for brightness, contrast, saturation and hue

/////////////////////////////////////////////////////////////////////////////

const int paramMin = 0;       // min value of parameter

const int paramMax = 255;     // max value of parameter

const int paramDef = 128;     // def value of parameter





/////////////////////////////////////////////////////////////////////////////

// Following are the available TV control functions

/////////////////////////////////////////////////////////////////////////////

LONG SetAudioMute   ( ULONG );

LONG GetAudioMute   ( void );

LONG SetVideoSource ( ULONG );

LONG GetVideoSource ( void );

LONG SetChannel     ( ULONG );

LONG GetChannel     ( void );

LONG SetBrightness  ( ULONG );

LONG GetBrightness  ( void );

LONG SetContrast    ( ULONG );

LONG GetContrast    ( void );

LONG SetHue         ( ULONG );

LONG GetHue         ( void );

LONG SetSaturation  ( ULONG );

LONG GetSaturation  ( void );





/////////////////////////////////////////////////////////////////////////////

// Pointer to function Types

/////////////////////////////////////////////////////////////////////////////

typedef LONG (* tvSetFunc)( ULONG );

typedef LONG (* tvGetFunc)( void );





/////////////////////////////////////////////////////////////////////////////

// Create an array of ptrs to functions for the TV control functions

// Entries in these tables must be in the same order defined for TV Parameter in TVCtl.h

/////////////////////////////////////////////////////////////////////////////

tvSetFunc SetTVFunc[] = { NULL,              // 0x00

                          SetVideoSource,    // TV_PARM_SOURCE

                          SetChannel,        // TV_PARM_CHANNEL

                          SetBrightness,     // TV_PARM_BRIGHTNESS

                          SetContrast,       // TV_PARM_CONTRAST

                          SetHue,            // TV_PARM_HUE

                          SetSaturation,     // TV_PARM_SATURATION

                          NULL, NULL, NULL,  // 0x07 - 0x09

                          NULL, NULL, NULL,  // 0x0A - 0x0C

                          NULL, NULL, NULL,  // 0x0D - 0x0F

                          SetAudioMute };    // TV_PARM_MUTE



tvGetFunc GetTVFunc[] = { NULL,              // 0x00

                          GetVideoSource,    // TV_PARM_SOURCE

                          GetChannel,        // TV_PARM_CHANNEL

                          GetBrightness,     // TV_PARM_BRIGHTNESS

                          GetContrast,       // TV_PARM_CONTRAST

                          GetHue,            // TV_PARM_HUE

                          GetSaturation,     // TV_PARM_SATURATION

                          NULL, NULL, NULL,  // 0x07 - 0x09

                          NULL, NULL, NULL,  // 0x0A - 0x0C

                          NULL, NULL, NULL,  // 0x0D - 0x0F

                          GetAudioMute };    // TV_PARM_MUTE





/////////////////////////////////////////////////////////////////////////////

// Local Function Prototypes

/////////////////////////////////////////////////////////////////////////////

LONG SetAudioSource ( ULONG );

LONG GetAudioSource ( void );





/////////////////////////////////////////////////////////////////////////////

// Local variables to the DLL

/////////////////////////////////////////////////////////////////////////////

HINSTANCE    hInst;                          // Handle to instance

const  BYTE  TunerI2CAddress = 0xC2;         // Tuner address for Temic tuner

static Tuner theTuner( brandTemic,

                       formatNTSC,

                       TunerI2CAddress);     // create a Temic tuner

int          audioSource;                    // audio source

int          videoSource;                    // video source





//===========================================================================

// The only functions interfacing with the application level:

//    1) LONG SetTVParam( ULONG Parameter, ULONG Value )

//    2) LONG GetTVParam( ULONG Parameter )

//===========================================================================



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetTVParam( ULONG Parameter, ULONG Value )

// Purpose: Exported function allows other application to set parameter

// Input:   ULONG Parameter - parameter to be set

//          ULONG Value     - value to be set to

// Output:  None

// Return:  Returned value of function called, or TV_ERR_UNSUPPORTED_PARAM

/////////////////////////////////////////////////////////////////////////////

LONG __TVCTLFUNC SetTVParam( ULONG Parameter, ULONG Value )

{

   if ( Parameter >= sizeof( SetTVFunc ) / sizeof( *SetTVFunc ) )

      return ( TV_ERR_UNSUPPORTED_PARAM );



   if ( SetTVFunc[ Parameter ] == NULL )

      return ( TV_ERR_UNSUPPORTED_PARAM );



   return (* SetTVFunc[ Parameter ])( Value );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetTVParam( ULONG Parameter )

// Purpose: Exported function allows other application to get parameter

// Input:   ULONG Parameter - parameter to get value of

// Output:  None

// Return:  Returned value of function called, or TV_ERR_UNSUPPORTED_PARAM

/////////////////////////////////////////////////////////////////////////////

LONG __TVCTLFUNC GetTVParam( ULONG Parameter )

{

   if ( Parameter >= sizeof( GetTVFunc ) / sizeof( *GetTVFunc ) )

      return ( TV_ERR_UNSUPPORTED_PARAM );



   if ( GetTVFunc[ Parameter ] == NULL )

      return ( TV_ERR_UNSUPPORTED_PARAM );



   return (* GetTVFunc[ Parameter ])();

}





/////////////////////////////////////////////////////////////////////////////

// Method:  char* __TVCTLFUNC GetTVErrString( LONG ErrCode )

// Purpose: Exported function allows other application to get our specific

//          error message

// Input:   LONG ErrCode - error code

// Output:  None

// Return:  Pointer to char string with the error message

/////////////////////////////////////////////////////////////////////////////

char* __TVCTLFUNC GetTVErrString( LONG ErrCode )

{

   static char msg[ 255 ];

   if ( LoadString( hInst, ErrCode, msg, sizeof( msg ) ) == 0 )

      return NULL;

   else

      return msg;

}





//===========================================================================

// Following are the functions to perform the Set...() and Get...() operations

//===========================================================================



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetVideoSource ( ULONG source )

// Purpose: Set video source

// Input:   ULONG source - video source to set to

//                           TV_SOURCE_BASEBAND_COMPOSITE

//                           TV_SOURCE_BASEBAND_SVIDEO

//                           TV_SOURCE_BASEBAND_CAMERA

//                           TV_SOURCE_TUNER_CABLE

//                           TV_SOURCE_TUNER_ANTENNA

// Output:  None

// Return:  TV control error value

/////////////////////////////////////////////////////////////////////////////

LONG SetVideoSource ( ULONG source )

{

   // Determine which connector associate with TV source

   // Also determine which audio source to use associate with TV source

   Connector conn;

   int audio;

   switch ( source )

   {

	   case TV_SOURCE_BASEBAND_COMPOSITE:

      case TV_SOURCE_BASEBAND_CAMERA:

         conn = ConComposite;

         audio = AUDIO_SOURCE_EXTAUDIO;

         break;



	   case TV_SOURCE_BASEBAND_SVIDEO:

         conn = ConSVideo;

         audio = AUDIO_SOURCE_EXTAUDIO;

         break;



      case TV_SOURCE_TUNER_CABLE:

      case TV_SOURCE_TUNER_ANTENNA:

         conn = ConTuner;

         audio = AUDIO_SOURCE_TVTUNER;

         break;



      default:

   		return ( TV_ERR_BAD_VALUE );

   }



   // set audio source also when TV source changes

   if ( Bt848SetVideoSource( conn ) == Success )

   {

      // since both cable and antenna connect to tuner, need to remember

      // which video source we really want when we call GetVideoSource().

      // Similary for composite and camera.

      videoSource = source;

      return ( SetAudioSource( audio ) );

   }

   else

      return ( TV_ERR_DLL_ERROR );

}



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetVideoSource ( void )

// Purpose: Get video source

// Input:   None

// Output:  None

// Return:  LONG source - video source has been set to

//                           TV_SOURCE_BASEBAND_COMPOSITE

//                           TV_SOURCE_BASEBAND_SVIDEO

//                           TV_SOURCE_BASEBAND_CAMERA

//                           TV_SOURCE_TUNER_CABLE

//                           TV_SOURCE_TUNER_ANTENNA

// Note:    Video source set to Cable or Antenna returns TV_SOURCE_TUNER_CABLE

//          here since they both connect to same source.

//          Similarily, video source set to Composite or Camera returns

//          TV_SOURCE_BASEBAND_COMPOSITE.

/////////////////////////////////////////////////////////////////////////////

LONG GetVideoSource ( void )

{

   // Could return "videoSource" here without even asking DLL. But this is

   // the right way to do it.



   Connector conn;

   if ( Bt848GetVideoSource( &conn ) == Fail )

      return ( TV_ERR_DLL_ERROR );



   switch ( conn )

   {

	   case ConComposite:

         if ( ( videoSource == TV_SOURCE_BASEBAND_COMPOSITE ) ||

              ( videoSource == TV_SOURCE_BASEBAND_CAMERA ) )

            return ( videoSource );



	   case ConSVideo:

         return ( TV_SOURCE_BASEBAND_SVIDEO );



      case ConTuner:

         if ( ( videoSource == TV_SOURCE_TUNER_CABLE ) ||

              ( videoSource == TV_SOURCE_TUNER_ANTENNA ) )

            return ( videoSource );

   }



   return ( TV_ERR_DLL_ERROR );

}



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetChannel( ULONG channel )

// Purpose: Set channel

// Input:   ULONG channel - channel to set to

// Output:  None

// Return:  TV control error value

/////////////////////////////////////////////////////////////////////////////

LONG SetChannel( ULONG channel )

{

   LONG source = GetVideoSource();



   // only cable and antenna can change channel

   if ( ( source != TV_SOURCE_TUNER_CABLE ) &&

        ( source != TV_SOURCE_TUNER_ANTENNA ) )

      return ( TV_ERR_SOURCE_CANNOT_TUNE );

   else

      return ( theTuner.SetChannel( channel,

                                    ( source == TV_SOURCE_TUNER_ANTENNA ) ? modeTV : modeCable ) );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetChannel( void )

// Purpose: Get channel

// Input:   None

// Output:  None

// Return:  Channel number

/////////////////////////////////////////////////////////////////////////////

LONG GetChannel( void )

{

   LONG source = GetVideoSource();



   // only cable and antenna have channel number

   if ( ( source != TV_SOURCE_TUNER_CABLE ) &&

        ( source != TV_SOURCE_TUNER_ANTENNA ) )

      return ( TV_ERR_SOURCE_CANNOT_TUNE );

   else

      return ( (LONG)theTuner.GetChannel() );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetBrightness( ULONG brightness )

// Purpose: Set brightness

// Input:   ULONG brightness - brightness to set to

// Output:  None

// Return:  TV_ERR_SUCCESS if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG SetBrightness( ULONG brightness )

{

   // range checking

   if ( brightness > paramMax )

      return ( TV_ERR_BAD_VALUE );



   // request Bt848 DLL for action

   return ( Bt848SetBrightness( brightness ) == Success ? TV_ERR_SUCCESS

                                                        : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetBrightness( void )

// Purpose: Get brightness

// Input:   None

// Output:  None

// Return:  brightness value if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG GetBrightness( void )

{

   int brightness;

   return ( Bt848GetBrightness( &brightness ) == Success ? (LONG)brightness

                                                         : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetContrast( ULONG contrast )

// Purpose: Set contrast

// Input:   ULONG contrast - contrast to set to

// Output:  None

// Return:  TV_ERR_SUCCESS if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG SetContrast( ULONG contrast )

{

   // range checking

   if ( contrast > paramMax )

      return ( TV_ERR_BAD_VALUE );



   // request Bt848 DLL for action

   return ( Bt848SetContrast( contrast ) == Success ? TV_ERR_SUCCESS

                                                    : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetContrast( void )

// Purpose: Get contrast

// Input:   None

// Output:  None

// Return:  contrast value if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG GetContrast( void )

{

   int contrast;

   return ( Bt848GetContrast( &contrast ) == Success ? (LONG)contrast

                                                     : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetHue( ULONG hue )

// Purpose: Set hue

// Input:   ULONG hue - hue to set to

// Output:  None

// Return:  TV_ERR_SUCCESS if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG SetHue( ULONG hue )

{

   // range checking

   if ( hue > paramMax )

      return ( TV_ERR_BAD_VALUE );



   // request Bt848 DLL for action

   return ( Bt848SetHue( hue ) == Success ? TV_ERR_SUCCESS : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetHue( void )

// Purpose: Get hue

// Input:   None

// Output:  None

// Return:  hue value if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG GetHue( void )

{

   int hue;

   return ( Bt848GetHue( &hue ) == Success ? (LONG)hue : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetSaturation( ULONG sat )

// Purpose: Set saturation

// Input:   ULONG sat - saturation to set to

// Output:  None

// Return:  TV_ERR_SUCCESS if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG SetSaturation( ULONG sat )

{

   // range checking

   if ( sat > paramMax )

      return ( TV_ERR_BAD_VALUE );



   // request Bt848 DLL for action

   return ( Bt848SetSaturation( sat ) == Success ? TV_ERR_SUCCESS : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetSaturation( void )

// Purpose: Get saturation

// Input:   None

// Output:  None

// Return:  saturation value if success, else TV_ERR_DLL_ERROR

/////////////////////////////////////////////////////////////////////////////

LONG GetSaturation( void )

{

   int sat;

   return ( Bt848GetSaturation( &sat ) == Success ? (LONG)sat : TV_ERR_DLL_ERROR );

}





/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetAudioMute ( ULONG mute )

// Purpose: Mute or UnMute audio

// Input:   ULONG mute - 0 to mute; 1-255 to unmute

// Output:  None

// Return:  TV control error value

// Note:    Audio source is controlled by setting appropriate value to

//          bits[1:0] of GPDATA. In addition, GPOE should be set to 0x0F so

//          value set in GPDATA will be output to the pins.

/////////////////////////////////////////////////////////////////////////////

LONG SetAudioMute ( ULONG mute )

{

   // check parameter

   if ( mute > 255 )

      return ( TV_ERR_BAD_VALUE );



   // enables GPOE drivers for [1:0]

   // need to set GPOE to 0x0F in order to set audio source

   SetGPOE( 0x0000000FL );



   if ( mute == 0 )

   {  // try to mute

      if ( GetAudioSource() == AUDIO_SOURCE_MUTE )

         return ( TV_ERR_SUCCESS );    // already mute, returns success

      else

         return ( ( SetGPDATABits( 0, 1, AUDIO_SOURCE_MUTE, 0 ) == Success ) ?

                        TV_ERR_SUCCESS : TV_ERR_AUDIO_ERROR );

   }

   else  // unmute

      return ( ( SetGPDATABits( 0, 1, audioSource, 0 ) == Success ) ?

                        TV_ERR_SUCCESS : TV_ERR_AUDIO_ERROR );

}



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetAudioMute ( void )

// Purpose: Check if audio is Mute or UnMute

// Input:   None

// Output:  None

// Return:  0 if mute, 1 if not mute, or TV_ERR_AUDIO_ERROR if error

// Note:    Audio source is controlled by bits[1:0] of GPDATA. In order to

//          read from pins, GPIE should be set to 0x00.

/////////////////////////////////////////////////////////////////////////////

LONG GetAudioMute ( void )

{

   // read GPIE from pin (not register)

   SetGPIE( 0x00000000L );



   // ask GPDATA what the setting is

   DWORD value;

   return ( GetGPDATABits( 0, 1, &value, 0 ) == Success ? ( value == AUDIO_SOURCE_MUTE ) ? 0 : 1

                                                        : TV_ERR_AUDIO_ERROR );

}



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetAudioSource ( ULONG audio )

// Purpose: Set audio source

// Input:   ULONG audio - AUDIO_SOURCE_CDROM, AUDIO_SOURCE_EXTAUDIO,

//                        AUDIO_SOURCE_TVTUNER, AUDIO_SOURCE_MUTE    

// Output:  None

// Return:  TV control error value

// Note:    Audio source is controlled by setting appropriate value to

//          bits[1:0] of GPDATA. In addition, GPOE should be set to 0x0F so

//          value set in GPDATA will be output to the pins.

/////////////////////////////////////////////////////////////////////////////

LONG SetAudioSource ( ULONG audio )

{

   // check parameter

   if ( audio > AUDIO_SOURCE_MUTE )

      return ( TV_ERR_BAD_VALUE );



   // enables GPOE drivers for [1:0]

   // need to set GPOE to 0x0F in order to set audio source

   SetGPOE( 0x0000000FL );



   // if already mute, don't change audio source, just remember it since

   // changing audio source will enable audio

   if ( GetAudioSource() == AUDIO_SOURCE_MUTE ) 

   {

      audioSource = audio;

      return ( TV_ERR_SUCCESS );

   }

   else

   {  // not in mute, change audio source

      if ( SetGPDATABits( 0, 1, audio, 0 ) == Success )

      {

         audioSource = audio;

         return ( TV_ERR_SUCCESS );

      }

      else

         return ( TV_ERR_AUDIO_ERROR );

   }

}



/////////////////////////////////////////////////////////////////////////////

// Method:  LONG GetAudioSource ( void )

// Purpose: Get audio source

// Input:   None

// Output:  None

// Return:  audio source - AUDIO_SOURCE_CDROM, AUDIO_SOURCE_EXTAUDIO,

//                         AUDIO_SOURCE_TVTUNER, AUDIO_SOURCE_MUTE

// Note:    Audio source is controlled by bits[1:0] of GPDATA. In order to

//          read from pins, GPIE should be set to 0x00.

/////////////////////////////////////////////////////////////////////////////

LONG GetAudioSource ( void )

{

   // read GPIE from pin (not register)

   SetGPIE( 0x00000000L );



   // ask GPDATA what the setting is

   DWORD value;

   return ( GetGPDATABits( 0, 1, &value, 0 ) == Success ? value : TV_ERR_AUDIO_ERROR );

}





