/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// $Header: /export/raid/cvs_videodev/TrueCap/VideoTuners/Bt848V2_2/Source/Grappler/tuner.h,v 1.1 1999/08/27 02:52:41 nradisch Exp $

// $Log: tuner.h,v $
// Revision 1.1  1999/08/27 02:52:41  nradisch
// new
//

// Revision 2.4  1997/07/16 18:16:03  rudyl

// Use last 848 release as head rev. Incorporate Gemini changes later.

// Revision 2.1  1996/06/10 20:12:38  ssm

// MarkBeta1AsRev2

// Revision 1.5  1996/05/10 23:15:48  rudyl

// Implement TV_ERR_... return values from Intel. Also added exported function GetTVErrString() to retrieve our specific error msg.

// Revision 1.4  1996/05/06 18:49:06  rudyl

// No changes; just added more comments into codes.

// Revision 1.3  1996/04/28 01:20:14  rudyl

// ver 0.50 - does not instantiate I2C and GPIO objects here. Just make API

// calls to BT848_32.DLL. Also returns error types to caller instead of just TRUE/FALSE result.

// Revision 1.2  1996/04/12 21:01:11  rudyl

// Renamed DLL from 848Tuner.dll to Grappler.dll.

// Revision 1.1  1996/04/12 18:24:44  rudyl

// Initial revision



// CUT_HERE



/////////////////////////////////////////////////////////////////////////////

//

//  Copyright (c) 1996 Brooktree Corporation

//

//  Module:

//

//    Tuner.h

//

//  Abstract:

//

//    Bt848 Tuner class header file

//

/////////////////////////////////////////////////////////////////////////////



#ifndef __TUNER_H

#define __TUNER_H



#include "retcode.h"



/////////////////////////////////////////////////////////////////////////////

// Typedefs

/////////////////////////////////////////////////////////////////////////////

typedef WORD TunerCodes;

typedef TunerCodes* ptrTunerCodes;

typedef WORD ControlWord;





/////////////////////////////////////////////////////////////////////////////

// tuner brands

/////////////////////////////////////////////////////////////////////////////

enum { brandPhilips, brandTemic };





/////////////////////////////////////////////////////////////////////////////

// tuner has 2 different tuning modes: tv (ie. broadcast) or cable

/////////////////////////////////////////////////////////////////////////////

enum { modeTV, modeCable };





/////////////////////////////////////////////////////////////////////////////

// video formats

/////////////////////////////////////////////////////////////////////////////

enum { formatNTSC, formatPAL };





////////////////////////////////////////////////////////////////////

// tuner uses 3 different bands based on desired channel & mode

/////////////////////////////////////////////////////////////////////////////

enum { bandVHF_LOW, bandVHF_HIGH, bandUHF };





/////////////////////////////////////////////////////////////////////////////

// struct for associating a channel with a band

/////////////////////////////////////////////////////////////////////////////

typedef struct tagTunerBandStruct

{

   int minChannel;

   int maxChannel;

   int band;

} TunerBandStruct;

typedef TunerBandStruct* ptrTunerBandStruct;







/////////////////////////////////////////////////////////////////////////////

// Tuner Class

//

// Description:

//    This class encapsulates the tuner behavior of the Bt848. It provides

//    "SetChannel()" and "GetChannel()" to set and retrieve channel using the

//    tuner. To instantiate this class, the caller needs to supply 3 parameters

//    as defined in the constructor:

//       Tuner( int brand, int format, BYTE I2CAddress );

//          where brand can be either brandPhilips or brandTemic,

//                format can be either formatNTSC or formatPAL,

//                I2CAddress is the address of the tuner.

//    This class also incorporate Automatic Fine Tuning (AFT) when tuning a

//    channel. It will scan up to +/- 2 1/2 MHz from the predefined setting

//    in order to obtain optimum image of the channel.

//    

/////////////////////////////////////////////////////////////////////////////

class Tuner

{

   int  m_channel;      // which channel is set to

   int  m_format;       // video foramt (ntsc or pal)

   BYTE m_I2CAddress;   // I2C address



   // ptrs to lookup tables

   ptrTunerCodes	    m_ptrTunerCodeArray;

   ptrTunerBandStruct m_ptrTunerBandStructArray;

   ControlWord *      m_ptrControlWordArray;



   ////////////////////////////////////

   // Internal functions

   ////////////////////////////////////

   WORD      Tuner::GetDividerWord( int channel, int mode );

   void      Tuner::SetDividerWord( int channel, int mode, WORD value );

   WORD      Tuner::GetControlWord( int channel, int mode );

   LONG      Tuner::SetChannelAFT ( int channel, int mode );

   WORD      Tuner::MaxNumberOfChannel( int mode );



public:

   Tuner( int brand, int format, BYTE I2CAddress );

   ~Tuner();



   ////////////////////////////////////

   // Public member functions

   ////////////////////////////////////

   LONG      SetChannel( int channel, int mode );

   int       GetChannel( void );

};





#endif // __TUNER_H

