/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CGrapplerVideoTuner.h"
#include "IScanProgressFeedback.h"

#include "tvctl.h"

#define FIRST_CHANNEL	1
#define LAST_CHANNEL	125


#define TUNER_INTERNAL -1000000	// tuner internal error

#pragma optimize( "", off )

  


CGrapplerVideoTuner::CGrapplerVideoTuner(const char *appstring)
{

	mAppString = appstring;
	mSetGPIE = 0;

	mSetGPOE = 0;

	mGPIOIsInitOK = 0;

	mSetGPIOMODE = 0;

	mSetGPDATABits = 0;

	mGetGPDATABits = 0;

	mSetCCStatus = 0;

	mGetCCStatus = 0;

	mBT848 = 0;

	mInvertAudio = AfxGetApp()->GetProfileInt(appstring,"Grapp_InvertAudio",0);



	mLastChannel = 2;

	mLastSource = BaseBand_Composite;

	mLastBrightness = mLastContrast = mLastHue = mLastSaturation = 128;



	mHauppauge = 1;

	mGrappler = ::LoadLibrary("hcwtvctl.dll");
	if (!mGrappler)

		{

		mHauppauge = 0;
		mGrappler = ::LoadLibrary("Grappler.dll");

		if (!mGrappler)

			throw TunerException("Grappler not available",0);
		}

	if (mInvertAudio)

		SetupAudioInversion();

}

CGrapplerVideoTuner::~CGrapplerVideoTuner(void)
{
	if (mGrappler)
		{
		::FreeLibrary(mGrappler);
		mGrappler = 0;
		}

	if (mBT848)

		{

		::FreeLibrary(mBT848);

		mBT848 = 0;

		}
}


ErrorCode 

CGrapplerVideoTuner::_SetCCStatus(int status)

{

	assert(mSetCCStatus);

	return (*mSetCCStatus)(status);

}



int		  

CGrapplerVideoTuner::_GetCCStatus(void)

{

	assert(mGetCCStatus);

	return (*mGetCCStatus)();

}



bool	

CGrapplerVideoTuner::HasCC(void)

{

	// quick kludge for 2.5

	if (!mBT848)

		SetupAudioInversion();	// sets up bt848 stuff

	return (mBT848 && mSetCCStatus && mGetCCStatus ? true : false);

}



void	

CGrapplerVideoTuner::SetCC(int cc,int channel)

{

	if (!mBT848)

		SetupAudioInversion();	// sets up bt848 stuff

	if (mGetCCStatus && mSetCCStatus)

		{

		try {

		int c;

		if (cc)

			c = 0x8000 | channel;

		else

			c = 0;

		_SetCCStatus(c);

		}

		catch(...)

		{}

		}

}





bool	

CGrapplerVideoTuner::GetAudioInversion(void)

{

	return mInvertAudio;

}



void	

CGrapplerVideoTuner::SetAudioInversion(bool i)

{

	if (!mBT848)

		SetupAudioInversion();

	bool flip = ((i && !mInvertAudio) || (mInvertAudio && !i));

	mInvertAudio = TRUE;

	if (flip && mBT848)

		InvertAudio();

	mInvertAudio = i;

	AfxGetApp()->WriteProfileInt((const char *)mAppString,"Grapp_InvertAudio",mInvertAudio);

}




extern "C" {
typedef LONG 	(*grappler_set_t)(ULONG p,ULONG v);
typedef LONG 	(*grappler_get_t)(ULONG p);
typedef char* 	(*grappler_err_t)(LONG err);
};

int
CGrapplerVideoTuner::SetTVParam(unsigned parameter,unsigned value)
{
	if (!mGrappler)
		throw TunerException("Grappler not available",0);

	grappler_set_t proc = 
		(grappler_set_t)::GetProcAddress(mGrappler,"SetTVParam");
	

	LONG res;

	try {

		res = (*proc)(parameter,value);
		}

	catch(...)

		{

		// Hauppauge throws exceptions

		res = TUNER_INTERNAL;

		}
	return res;

	//ErrorCheck(res);
}

int
CGrapplerVideoTuner::GetTVParam(unsigned parameter)
{
	if (!mGrappler)
		throw TunerException("Grappler not available",0);

	grappler_get_t proc = 
		(grappler_get_t)::GetProcAddress(mGrappler,"GetTVParam");
	LONG res;

	try {

		res = (*proc)(parameter);
		}

	catch(...)

		{

		// Hauppauge throws exceptions

		res = TUNER_INTERNAL;

		}
	//ErrorCheck(res);
	return res;
}

void
CGrapplerVideoTuner::ErrorCheck(long err)
{
	if (err == TUNER_INTERNAL)

		{

		throw TunerException("Internal",err);

		}

	if (err < 0)
		{
		grappler_err_t proc = 
			(grappler_err_t)::GetProcAddress(mGrappler,"GetTVErrString");
		const char *errstring = 0;

		try {

			// try/catch because of hauppauge

			errstring = (*proc)(err);

			}

		catch(...) {}
		if (errstring)
			throw TunerException(errstring,err);
		else
			throw TunerException("",err);
		}
}

/////////////////////////////////////////////////////

int
CGrapplerVideoTuner::GetSourceCode(VideoSource_t source)
{
	int	 res;
	switch(source)
		{
		case BaseBand_Composite:
			res = TV_SOURCE_BASEBAND_COMPOSITE;
			break;
		case BaseBand_SVideo:
			res = TV_SOURCE_BASEBAND_SVIDEO;
			break;
		case BaseBand_Camera:
			res = TV_SOURCE_BASEBAND_CAMERA;
			break;
		case Tuner_Cable:
			res = TV_SOURCE_TUNER_CABLE;
			break;
		case Tuner_Antenna:
			res = TV_SOURCE_TUNER_ANTENNA;
			break;
		default:
			throw TunerException("Unknown video source",(int)source);
		}	

	return res;
}

void	
CGrapplerVideoTuner::SetVideoSource(VideoSource_t source)
{
	mLastSource = source;

	SetTVParam(TV_PARM_SOURCE,GetSourceCode(source));

	InvertAudio();
}

void	
CGrapplerVideoTuner::SetChannel(Channel_t channel)
{

	mLastChannel = channel;

	SetTVParam(TV_PARM_CHANNEL,channel);
}

void	
CGrapplerVideoTuner::SetBrightness(U8 brightness)
{
	mLastBrightness = brightness;

	SetTVParam(TV_PARM_BRIGHTNESS,brightness);
}

void	
CGrapplerVideoTuner::SetContrast(U8 contrast)
{
	mLastContrast = contrast;

	SetTVParam(TV_PARM_CONTRAST,contrast);
}

void	
CGrapplerVideoTuner::SetHue(U8 hue)
{
	mLastHue = hue;

	SetTVParam(TV_PARM_HUE,hue);
}

void	
CGrapplerVideoTuner::SetSaturation(U8 saturation)
{
	mLastSaturation = saturation;

	SetTVParam(TV_PARM_SATURATION,saturation);
}

/////////////////////////////////////////////

IVideoTuner::VideoSource_t
CGrapplerVideoTuner::ConvertSourceCode(int code)
{
	IVideoTuner::VideoSource_t	t;
	switch(code)
		{
		case TV_SOURCE_BASEBAND_COMPOSITE:
			t = BaseBand_Composite;
			break;
		case TV_SOURCE_BASEBAND_SVIDEO:
			t = BaseBand_SVideo;
			break;
		case TV_SOURCE_BASEBAND_CAMERA:
			t = BaseBand_Camera;
			break;
		case TV_SOURCE_TUNER_CABLE:
			t = Tuner_Cable;
			break;
		case TV_SOURCE_TUNER_ANTENNA:
			t = Tuner_Antenna;
			break;
		default:
			throw TunerException("Unknown source code",code);
		}
	return t;
}

IVideoTuner::VideoSource_t	
CGrapplerVideoTuner::GetVideoSource(void)
{

//try

//	{

//	LONG res = GetTVParam(TV_PARM_SOURCE);

//	mLastSource = ConvertSourceCode(res);

//	}

//catch(...){}



	return mLastSource;


}

IVideoTuner::Channel_t		
CGrapplerVideoTuner::GetChannel(void)
{

//	try 

//		{

//		mLastChannel = GetTVParam(TV_PARM_CHANNEL);

//		}

//	catch(...) {}



	return mLastChannel;

}

U8				
CGrapplerVideoTuner::GetBrightness(void)
{
//	try

//		{

//		mLastBrightness = GetTVParam(TV_PARM_BRIGHTNESS);

//		}

//	catch(...) 

//		{}



	return mLastBrightness;
}

U8				
CGrapplerVideoTuner::GetContrast(void)
{
//	try 

//		{

//		mLastContrast = GetTVParam(TV_PARM_CONTRAST);

//		}

//	catch(...)

//		{}

	return mLastContrast;
}

U8				
CGrapplerVideoTuner::GetHue(void)
{
//	try 

//		{

//		mLastHue = GetTVParam(TV_PARM_HUE);

//		}

//	catch(...)

//		{}

	return mLastHue;
}

U8				
CGrapplerVideoTuner::GetSaturation(void)
{

//	try 

//		{

//		mLastSaturation = GetTVParam(TV_PARM_SATURATION);

//		}

//	catch(...)

//		{}

	return mLastSaturation;
}

///////////////////////////////////////////

void	
CGrapplerVideoTuner::SetAudioMute(bool mute)
{

	SetTVParam(TV_PARM_MUTE,mute);

	InvertAudio();
}

bool	
CGrapplerVideoTuner::GetAudioMute(void)
{
	return (mInvertAudio ? !GetTVParam(TV_PARM_MUTE) : GetTVParam(TV_PARM_MUTE));
}


IVideoTuner::ChannelList_t	
CGrapplerVideoTuner::GetChannelsWithSignal(VideoSource_t source,

										   IScanProgressFeedback &feedback)
{
	IVideoTuner::ChannelList_t	l;
	SetVideoSource(source);

	for (IVideoTuner::Channel_t x = FIRST_CHANNEL; x <= LAST_CHANNEL; ++x)
		{

		feedback.Channel(x);
		if (feedback.Cancel())

			return l;

		try {
			if (SetTVParam(TV_PARM_CHANNEL,x) >= 0)

				l.push_back(x);
			}
		catch(...)

			{

			if (mHauppauge)

				{

				try {

					if (SetTVParam(TV_PARM_CHANNEL,x) >= 0)

						l.push_back(x);

					}

				catch(...){}

				}

			}

		}
	return l;
}

IVideoTuner::ChannelList_t	

CGrapplerVideoTuner::GetChannelsWithSignal(IScanProgressFeedback &feedback)

{

	IVideoTuner::ChannelList_t	l;



	for (IVideoTuner::Channel_t x = FIRST_CHANNEL; x <= LAST_CHANNEL; ++x)

		{

		feedback.Channel(x);

		if (feedback.Cancel())

			return l;



		try {

			if (SetTVParam(TV_PARM_CHANNEL,x) >= 0)

				l.push_back(x);

			}

		catch(...)

			{

			if (mHauppauge)

				{

				try {

					if (SetTVParam(TV_PARM_CHANNEL,x) >= 0)

						l.push_back(x);

					}

				catch(...){}

				}

			}

		}

	return l;

}



void
CGrapplerVideoTuner::SaveSettings(const char *appstring)
{
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Source",
													(int)GetVideoSource());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Channel",
													(int)GetChannel());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Brightness",
													(int)GetBrightness());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Contrast",
													(int)GetContrast());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Hue",
													(int)GetHue());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Saturation",
													(int)GetSaturation());
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_InvertAudio",mInvertAudio);


}

void
CGrapplerVideoTuner::LoadSettings(const char *appstring)
{

	try

	{
	SetVideoSource((VideoSource_t)
	AfxGetApp()->GetProfileInt(appstring,"Grapp_Source",(int)BaseBand_Composite));
	}

	catch(...){}

	

	try 

	{
	SetBrightness(
	AfxGetApp()->GetProfileInt(appstring,"Grapp_Brightness",128));
	}

	catch(...){}



	try

	{
	SetContrast(
	AfxGetApp()->GetProfileInt(appstring,"Grapp_Contrast",128));
	}

	catch(...) {}


	try

	{

	SetHue(
	AfxGetApp()->GetProfileInt(appstring,"Grapp_Hue",128));
	}

	catch(...){}


	try

	{

	SetSaturation(
	AfxGetApp()->GetProfileInt(appstring,"Grapp_Saturation",128));
	}

	catch(...){}



	try

	{

	SetChannel((Channel_t)

	AfxGetApp()->GetProfileInt(appstring,"Grapp_Channel",(int)9));

	}

	catch(...){}



}




void

CGrapplerVideoTuner::SetupAudioInversion(void)

{

	mBT848 = ::LoadLibrary("btvid_32.dll");

	if (mBT848 == NULL) mBT848 = ::LoadLibrary("bt848_32.dll");

	if (mBT848 == NULL) return;



	mGPIOIsInitOK = (GPIOIsInitOK_Ptr)::GetProcAddress(mBT848,"GPIOIsInitOK");

	if (mGPIOIsInitOK == NULL) return;



	mSetGPIOMODE = (SetGPIOMODE_Ptr)::GetProcAddress(mBT848,"SetGPIOMODE");

	if (mSetGPIOMODE == NULL) return;



	mSetGPDATABits = (SetGPDATABits_Ptr)::GetProcAddress(mBT848,"SetGPDATABits");

	if (mSetGPDATABits == NULL) return;



	mGetGPDATABits = (GetGPDATABits_Ptr)::GetProcAddress(mBT848,"GetGPDATABits");

	if (mGetGPDATABits == NULL) return;



	mSetGPIE = (SetGPIE_Ptr)::GetProcAddress(mBT848,"SetGPIE");

	if (mSetGPIE == NULL) return;



	mSetGPOE = (SetGPOE_Ptr)::GetProcAddress(mBT848,"SetGPOE");

	if (mSetGPOE == NULL) return;



	mSetCCStatus = (SetCCStatus_Ptr)::GetProcAddress(mBT848,"Bt848SetCCStatus");

	mGetCCStatus = (GetCCStatus_Ptr)::GetProcAddress(mBT848,"Bt848GetCCStatus");

#ifndef NDEBUG

	if (!mSetCCStatus || !mGetCCStatus)

		AfxMessageBox("no cc");

#endif

	// should have been done by grappler

    //if ( (*gGPIOIsInitOK)() == false )

	//{ cerr << "Could not initialize GPIO" << endl; ::FreeLibrary(bt848); return -1; }

    // set GPIO to normal mode

    //(*gSetGPIOMODE)( GPIO_NORMAL );

}



void

CGrapplerVideoTuner::InvertAudio(void)

{

	if (!mInvertAudio) return;

	LONG	r = __GetAudioSource();

	if (r == TV_ERR_AUDIO_ERROR) return;

//	switch(r)

//		{

//		case AUDIO_SOURCE_CDROM:

//			break;

//		case AUDIO_SOURCE_EXTAUDIO:

//			break;

//		case AUDIO_SOURCE_TVTUNER:

//			break;

//		case AUDIO_SOURCE_MUTE:

//			break;

//		}

	__SetAudioSource(r & 0x01 ? 0 : 1);

}





LONG 

CGrapplerVideoTuner::__GetAudioSource ( void )

{

   // read GPIE from pin (not register)

   if (!mSetGPIE) return TV_ERR_AUDIO_ERROR;

   if (!mGetGPDATABits) return TV_ERR_AUDIO_ERROR;



	(*mSetGPIE)( 0x00000000L );



   // ask GPDATA what the setting is



   DWORD value;

   return ( (*mGetGPDATABits)( 0, 1, &value, 0 ) == Success ? value : TV_ERR_AUDIO_ERROR );

}







/////////////////////////////////////////////////////////////////////////////

// Method:  LONG SetAudioSource ( ULONG audio )

// Purpose: Set audio source

// Input:   ULONG audio - AUDIO_SOURCE_CDROM, AUDIO_SOURCE_EXTAUDIO,

//                        AUDIO_SOURCE_TVTUNER, AUDIO_SOURCE_MUTE    

// Output:  None

// Return:  TV control error value

// Note:    Audio source is controlled by setting appropriate value to

//          bits[1:0] of GPDATA. In addition, GPOE should be set to 0x0F so

//          value set in GPDATA will be output to the pins.

/////////////////////////////////////////////////////////////////////////////

LONG 

CGrapplerVideoTuner::__SetAudioSource ( ULONG audio )

{

   // check parameter

	if (!mSetGPOE) return TV_ERR_AUDIO_ERROR;

	if (!mSetGPDATABits) return TV_ERR_AUDIO_ERROR;



	if ( audio > AUDIO_SOURCE_MUTE )

      return ( TV_ERR_BAD_VALUE );



   // enables GPOE drivers for [1:0]

   // need to set GPOE to 0x0F in order to set audio source

	(*mSetGPOE)( 0x0000000FL );



   // if already mute, don't change audio source, just remember it since

   // changing audio source will enable audio

  if ( (*mSetGPDATABits)( 0, 1, audio, 0 ) == Success )

      {

//         gsAudioSource = audio;

         return ( TV_ERR_SUCCESS );

      }

      else

         return ( TV_ERR_AUDIO_ERROR );

}



#pragma optimize( "", on ) 

