/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CHauppaugeNTVideoTuner.h"
#include "IScanProgressFeedback.h"



#define FIRST_CHANNEL	1

#define LAST_CHANNEL	125


CHauppaugeNTVideoTuner::CHauppaugeNTVideoTuner(const char *appstring)
{
try {

	mInitFunc	= 0;

	mFreeFunc	= 0;

	mGetFunc	= 0;

	mSetFunc	= 0;

	mDll	= 0;

	mAV		= 0;

	mDll	= ::LoadLibrary("hcwav.dll");

	if (!mDll)

		throw TunerException("Tuner not available",0);

	mInitFunc = (InitFunc_t)::GetProcAddress(mDll,"hcwAV_InitByDetail");

	mFreeFunc = (FreeFunc_t)::GetProcAddress(mDll,"hcwAV_Free");

	mGetFunc  = (GetFunc_t) ::GetProcAddress(mDll,"hcwAV_Get");

	mSetFunc  = (SetFunc_t) ::GetProcAddress(mDll,"hcwAV_Set");

	if ((!mInitFunc) || (!mFreeFunc) || (!mSetFunc) || (!mGetFunc))

		throw TunerException("Could not get Hauppauge functions",0);



	}

catch(...)

	{

	_Cleanup();

	throw;

	}

}


void

CHauppaugeNTVideoTuner::_InitCard(void)

{

	assert(mInitFunc);

	if (mAV) return;

	mAV = (*mInitFunc)( hcwHW_BT_CLASS_TVCard/*hcwHW_BT_UNDEFINED*/, hcwHW_FEATURE_UNSPECIFIED, hcwHW_GET_AVAILABLE, 0 );

	if (!mAV)

		throw TunerException("Could not init Hauppauge AV",0);



	AVInputQuery avinputquery = AVINPUT_COUNT;

	DWORD dTotalInputs = (*mGetFunc)(mAV, AVF_Input, &avinputquery);

			

	mSVideoIndex = -1;

	for (int j=0; (DWORD)j < dTotalInputs; j++) 

		{

		AVInputType tmp = (AVInputType)(*mGetFunc)(mAV, AVF_InputType, &j);

		if (tmp == AVInput_Composite)

			mCompositeIndex = j;

		else if (tmp == AVInput_Tuner)

			mTunerIndex = j;

		else mSVideoIndex = j;

        }

	if (mSVideoIndex == -1)

		mSVideoIndex = mCompositeIndex;

}



void

CHauppaugeNTVideoTuner::_Cleanup(void)

{

	if (mAV && mFreeFunc)

		(*mFreeFunc)(mAV);

	mAV = 0;



	if (mDll)

		::FreeLibrary(mDll);

	mDll = 0;

}


CHauppaugeNTVideoTuner::~CHauppaugeNTVideoTuner(void) 
{

	_Cleanup();

}

void	
CHauppaugeNTVideoTuner::SetVideoSource(VideoSource_t source)
{

	_InitCard();

	mLastSource = source;
	AVInputSource	t;

	switch(source)

		{

		case IVideoTuner::BaseBand_Composite:

			t = mCompositeIndex;

			break;

		case IVideoTuner::BaseBand_SVideo:

			t = mSVideoIndex;

			break;

		case IVideoTuner::BaseBand_Camera:

			t = mCompositeIndex;

			break;

		case IVideoTuner::Tuner_Cable:

			t = mTunerIndex;

			break;

		case IVideoTuner::Tuner_Antenna:

			t = mTunerIndex;

			break;

		}

	(*mSetFunc)(mAV,AVF_VideoSource,&t);

	(*mSetFunc)(mAV,AVF_AudioSource,&t);

}


AVError

CHauppaugeNTVideoTuner::_SetChannel(Channel_t channel,VideoSource_t source)

{

	AVTune	t;

	t.format = AVTune_Format_MN;

	t.tunemode = AVTune_Mode_Channel;

	t.options = 0;

	t.mode.channel.country = AVTune_Country_USA;

	t.mode.channel.source = (source == Tuner_Cable ? AVTune_Broadcast_Source_Cable : 

								AVTune_Broadcast_Source_Antenna);

	t.mode.channel.channel = channel;

	t.finetune = 0;

	return (AVError)(*mSetFunc)(mAV,AVF_Tune,&t);

}


void	
CHauppaugeNTVideoTuner::SetChannel(Channel_t channel)
{
	_InitCard();

	mLastChannel = channel;

	_SetChannel(channel,mLastSource);

}



void	
CHauppaugeNTVideoTuner::SetBrightness(U8 brightness)
{
	_InitCard();

	mLastBrightness = brightness;

	AVHSB	hsb;

	hsb.iHue = mLastHue;

	hsb.iSaturation = mLastSaturation;

	hsb.iBrightness = mLastBrightness;

	(*mSetFunc)(mAV,AVF_HSB,&hsb);

}

void	
CHauppaugeNTVideoTuner::SetContrast(U8 contrast)
{
	_InitCard();

	mLastContrast = contrast;

	AVContrast c = contrast;

	(*mSetFunc)(mAV,AVF_Contrast,&c);

}

void	
CHauppaugeNTVideoTuner::SetHue(U8 hue)
{
	_InitCard();

	mLastHue = hue;

	AVHSB	hsb;

	hsb.iHue = mLastHue;

	hsb.iSaturation = mLastSaturation;

	hsb.iBrightness = mLastBrightness;

	(*mSetFunc)(mAV,AVF_HSB,&hsb);

}

void	
CHauppaugeNTVideoTuner::SetSaturation(U8 saturation)
{
	_InitCard();

	mLastSaturation = saturation;

	AVHSB	hsb;

	hsb.iHue = mLastHue;

	hsb.iSaturation = mLastSaturation;

	hsb.iBrightness = mLastBrightness;

	(*mSetFunc)(mAV,AVF_HSB,&hsb);

}


IVideoTuner::VideoSource_t	
CHauppaugeNTVideoTuner::GetVideoSource(void)
{
	return mLastSource;

}

IVideoTuner::Channel_t		
CHauppaugeNTVideoTuner::GetChannel(void)
{

	return mLastChannel;


}

U8				
CHauppaugeNTVideoTuner::GetBrightness(void)
{

	return mLastBrightness;

}

U8				
CHauppaugeNTVideoTuner::GetContrast(void)
{

	return mLastContrast;

}

U8				
CHauppaugeNTVideoTuner::GetHue(void)
{

	return mLastHue;

}

U8				
CHauppaugeNTVideoTuner::GetSaturation(void)
{

	return mLastSaturation;

}

void	

CHauppaugeNTVideoTuner::SetAudioMute(bool mute)
{

	AVAudioMute	m;

	m = (mute ? AVAudioMute_No : AVAudioMute_Yes);
	(*mSetFunc)(mAV,AVF_AudioMute,&m);

}

bool	

CHauppaugeNTVideoTuner::GetAudioMute(void)
{

	return false;
}

void	

CHauppaugeNTVideoTuner::SaveSettings(const char *appstring)
{
	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Source",

													(int)GetVideoSource());

	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Channel",

													(int)GetChannel());

	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Brightness",

													(int)GetBrightness());

	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Contrast",

													(int)GetContrast());

	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Hue",

													(int)GetHue());

	AfxGetApp()->WriteProfileInt(appstring,"Grapp_Saturation",

													(int)GetSaturation());

}

void	

CHauppaugeNTVideoTuner::LoadSettings(const char *appstring)
{
	SetVideoSource((VideoSource_t)

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Source",(int)BaseBand_Composite));



	SetBrightness(

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Brightness",128));



	SetContrast(

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Contrast",128));



	SetHue(

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Hue",128));



	SetSaturation(

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Saturation",128));



	SetChannel((Channel_t)

				AfxGetApp()->GetProfileInt(appstring,"Grapp_Channel",(int)9));

}


IVideoTuner::ChannelList_t	

CHauppaugeNTVideoTuner::GetChannelsWithSignal(IVideoTuner::VideoSource_t source,
								IScanProgressFeedback &feedback)
{

	_InitCard();

	IVideoTuner::ChannelList_t	l;

	SetVideoSource(source);



	for (IVideoTuner::Channel_t x = FIRST_CHANNEL; x <= LAST_CHANNEL; ++x)

		{

		feedback.Channel(x);

		if (feedback.Cancel())

			return l;

		if (_SetChannel(x,source) == AVError_Success)

			{

			AVVideoSignalStatus status=AVVideoSignalStatus_Bad;

			if ((*mGetFunc)(mAV,AVF_VideoSignalStatus,&status) == AVError_Success)

				{

				if (status == AVVideoSignalStatus_Good)

					l.push_back(x);

				}

			}

		}

	return l;

}

IVideoTuner::ChannelList_t	

CHauppaugeNTVideoTuner::GetChannelsWithSignal(
								IScanProgressFeedback &feedback)
{
	_InitCard();

	IVideoTuner::ChannelList_t	l;



	for (IVideoTuner::Channel_t x = FIRST_CHANNEL; x <= LAST_CHANNEL; ++x)

		{

		feedback.Channel(x);

		if (feedback.Cancel())

			return l;

		if (_SetChannel(x,mLastSource) == AVError_Success)

			{

			AVVideoSignalStatus status=AVVideoSignalStatus_Bad;

			if ((*mGetFunc)(mAV,AVF_VideoSignalStatus,&status) == AVError_Success)

				{

				if (status == AVVideoSignalStatus_Good)

					l.push_back(x);

				}

			}

		}

	return l;

}


bool	

CHauppaugeNTVideoTuner::GetAudioInversion(void)
{

	return false;
}

void	

CHauppaugeNTVideoTuner::SetAudioInversion(bool i)
{
	//_InitCard();

}

bool	

CHauppaugeNTVideoTuner::HasCC(void)
{
	return false;

}

void	

CHauppaugeNTVideoTuner::SetCC(int cc,int channel)
{
	_InitCard();

}

