/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef IVIDEOTUNER_H_
#define IVIDEOTUNER_H_

#include "MegaHeaders.h"
#include "CError.h"
#include <vector>
using namespace std;

class IScanProgressFeedback;


class IVideoTuner
{
public:
	typedef U16 Channel_t;
	typedef vector<Channel_t> ChannelList_t;
	typedef enum { 
		BaseBand_Composite = 0,
		BaseBand_SVideo = 1,
		BaseBand_Camera = 2,
		Tuner_Cable = 3,
		Tuner_Antenna = 4 }VideoSource_t;


	virtual ~IVideoTuner(void) {}


	virtual	void	SetVideoSource(VideoSource_t source) = 0;
	virtual	void	SetChannel(Channel_t channel) = 0;
	virtual	void	SetBrightness(U8 brightness) = 0;
	virtual	void	SetContrast(U8 contrast) = 0;
	virtual	void	SetHue(U8 hue) = 0;
	virtual	void	SetSaturation(U8 saturation) = 0;

	virtual	VideoSource_t	GetVideoSource(void) = 0;
	virtual	Channel_t		GetChannel(void) = 0;
	virtual	U8				GetBrightness(void) = 0;
	virtual	U8				GetContrast(void) = 0;
	virtual	U8				GetHue(void) = 0;
	virtual	U8				GetSaturation(void) = 0;

	virtual	void	SetAudioMute(bool mute) = 0;
	virtual	bool	GetAudioMute(void) = 0;

	virtual	void	SaveSettings(const char *appstring) = 0;
	virtual	void	LoadSettings(const char *appstring) = 0;

	virtual	ChannelList_t	GetChannelsWithSignal(VideoSource_t source,

								IScanProgressFeedback &feedback) = 0;
	virtual	ChannelList_t	GetChannelsWithSignal(

								IScanProgressFeedback &feedback) = 0;



	virtual	bool	GetAudioInversion(void) = 0;

	virtual	void	SetAudioInversion(bool i) = 0;

	virtual bool	HasCC(void) = 0;

	virtual	void	SetCC(int cc,int channel) = 0;


	CERROR_SUBCLASS(TunerException);	
};



#endif
