/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef HCW_AV_H

#define HCW_AV_H



#include <Windows.h>

#include "hcwhwhdl.h"



/*

** AVHdl

** Handle used to control the Audio and/or Video for a specific device.

** Generally used to get more specific handles to particular inputs or controls of a particular device.

*/

typedef DWORD AVHdl;





/*

** AVFunction

** These are the functions exectued through hcwAV_Get and hcwAV_Set.

** Their use is described below.

*/

typedef enum {

	AVF_Handle,

	AVF_VideoSource,

	AVF_Tune,

	AVF_Input,

	AVF_Contrast,

	AVF_AudioSource,

	AVF_AudioPreferredMode,

	AVF_AudioVolume,

	AVF_GetMixerEntries,

	AVF_FreeMixerEntries,

	AVF_AudioMixerID,

	AVF_AudioBalance,

	AVF_AudioBass,

	AVF_AudioTreble,

	AVF_AudioSeparation,

	AVF_AudioSpatialEffect,

	AVF_AudioMute,

	AVF_AudioLoudness,

	AVF_AudioAvailableMode,

	AVF_AudioAutoMute,

	AVF_Select,

	AVF_HSB,

	AVF_RGB,

	AVF_InputType, 

	AVF_VideoSignalStatus,	// used only with hcwGet

	AVF_VideoFormat,

	AVF_Surface,				// used only with hcwget

	AVF_AudioSupportedMode,

	AVF_AudioActualMode,

	AVF_SupportedVideoFormat,

	AVF_SupportedTunerFormat,

	AVF_SupportedVideoMode,

	AVF_SelectedVideoMode,

	AVF_TunerMinMax,

	AVF_DriverInfo,

	AVF_TunerDefaultType,

	AVF_TunerStatusByte 

}	AVFunction;



/*

** AVF_Input, AVF_Input_Tuner, and AVF_Input_Composite #definitions

** AVINPUT_COUNT - return number of Inputs of the specified class

** AVINPUT_GROUP - return a handle which controls a group of Inputs simultaneously

*/

typedef enum {

	 AVINPUT_COUNT = -1,

	 AVINPUT_GROUP = -2

}AVInputQuery;

typedef AVInputQuery *AVInputQueryPtr;



/*

** TunerF_StatusByte

** Returns the tuner status byte.

**

*/

typedef struct {

	int		Locked;

	DWORD	StatusByte;

}AVTuner_StatusByte;

typedef AVTuner_StatusByte *AVTuner_StatusBytePtr;





/*

** AVF_GetMixerEntries uses this struct 

** to return the current mixer id, mixer count

** and a pointer to the list.

*/

#define AVAUDIO_MAX_MIXLIST_NAME_SIZE 16

typedef struct AVMixerEntryStruct {

    DWORD	dwID;

	char	name[AVAUDIO_MAX_MIXLIST_NAME_SIZE+1];

} AVMixerEntry, *AVMixerEntryPtr;





typedef struct {

	int		CurrentMixerID;

 	int		MixerCount;

	AVMixerEntryPtr	MixerListPtr;

}	AVMixerList;

typedef AVMixerList *AVMixerListPtr;

typedef AVMixerListPtr *AVMixerListHdl;



/*

** AVError

** These are error types returned by hcwAV_Get and hcwAV_Set.

*/

typedef enum {

	AVError_Success,

	AVError_Failure,

	AVError_Failure_InvalidHandle,

	AVError_Failure_HardwareNotSupported,

	AVError_Failure_InvalidParameter,

	AVError_Failure_InvalidOperation,

	AVError_Failure_InvalidChannel,

	AVError_Failure_BadOrMissingDLL

}	AVError;





/*

** AVContrast, AVContrastPtr

** Handle to Contrast details.

*/

typedef DWORD AVContrast;

typedef AVContrast *AVContrastPtr;



/*

** AVInputSource, AVInputSourcePtr

** Handle to Video Source details.

*/

typedef DWORD AVInputSource;

typedef AVInputSource *AVInputSourcePtr;



/*

** AVVideoSignalStatus

** 

*/

typedef enum {

	AVVideoSignalStatus_Good,

	AVVideoSignalStatus_Bad

}	AVVideoSignalStatus;



typedef AVVideoSignalStatus *AVVideoSignalStatusPtr;



/*

** AVGetOverlayFlag

** 

*/

typedef enum {

	AVSurface_Primary,

	AVSurface_Secondary

}	AVSurface;



typedef AVSurface *AVSurfacePtr;





/*

** AVHSB, AVHSBPtr

** Handle to HSB details.

*/

typedef struct {

	int iHue;

	int iSaturation;

	int iBrightness;

}	AVHSB;

typedef AVHSB *AVHSBPtr;





/*

** AVRGB, AVRGBPtr

** Handle to RGB details.

*/

typedef struct {

	int iRed;

 	int iGreen;

	int iBlue;

}	AVRGB;

typedef AVRGB *AVRGBPtr;





/*

** AVTune_Mode, AVTune_ModePtr

** Handle to Tuner Mode Settings

*/

typedef enum {

	AVTune_Mode_Frequency,

	AVTune_Mode_Channel,

	AVTune_Mode_FMFreq

}	AVTune_Mode;

typedef AVTune_Mode *AVTune_ModePtr;





/*

** AVTune_Frequency, AVTune_FrequencyPtr

** Handle to Tuner Frequency Settings

*/

typedef long AVTune_Frequency;

typedef AVTune_Frequency *AVTune_FrequencyPtr;





/*

** AVTune_Broadcast_Source, AVTune_Broadcast_SourcePtr

** Handle to Tuner Broadcast Source Settings

*/

typedef enum {

	AVTune_Broadcast_Source_Antenna,

	AVTune_Broadcast_Source_Cable,

	AVTune_Broadcast_Source_HRC

}	AVTune_Broadcast_Source;

typedef AVTune_Broadcast_Source *AVTune_Broadcast_SourcePtr;





/*

** AVTune_Country, AVTune_CountryPtr

** Handle to Tuner Country Settings

*/

typedef enum {

	AVTune_Country_Unknown      =0,

	AVTune_Country_USA			=1, 

	AVTune_Country_Canada		=2, 

	AVTune_Country_SouthAfrica	=27,

	AVTune_Country_Netherlands	=31,

	AVTune_Country_France		=33,

	AVTune_Country_Spain		=34,

	AVTune_Country_Portugal		=351,

	AVTune_Country_Ireland		=353,

	AVTune_Country_Finland		=358,

	AVTune_Country_Italy		=39,

	AVTune_Country_Switzerland	=41,

	AVTune_Country_Austria		=43,

	AVTune_Country_UK			=44,

	AVTune_Country_Denmark		=45,

	AVTune_Country_Sweden		=46,

	AVTune_Country_Norway		=47,

	AVTune_Country_Poland		=48,

	AVTune_Country_Germany		=49,

	AVTune_Country_Mexico		=52,

	AVTune_Country_Argentina	=54,

	AVTune_Country_Brazil		=55,

	AVTune_Country_Chile		=56,

	AVTune_Country_Australia	=61,

	AVTune_Country_Philippines	=63,

	AVTune_Country_Singapore	=65,

	AVTune_Country_Japan		=81,

	AVTune_Country_Korea		=82,

	AVTune_Country_HongKong		=852,

	AVTune_Country_China		=86,

	AVTune_Country_Taiwan		=886

}	AVTune_Country;

typedef AVTune_Country *AVTune_CountryPtr;





/*

** AVTune_Channel, AVTune_ChannelPtr

** Handle to Tuner Channel Settings

*/

typedef struct {

	AVTune_Country			country;

	AVTune_Broadcast_Source	source;

	int		channel;

}	AVTune_Channel;

typedef AVTune_Channel *AVTune_ChannelPtr;





/*

** AVTune_Format, AVTune_FormatPtr

** Handle to Tuner Format Settings

*/



#define AVTune_Format_NONE				0x00

#define AVTune_Format_OTHER				0x01

#define AVTune_Format_FM	            0x02 /* radio */

#define AVTune_Format_BG	            0x04

#define AVTune_Format_MN	            0x08

#define AVTune_Format_I		            0x10

#define AVTune_Format_L		            0x20 /* includes L' */

#define AVTune_Format_DK				0x40



typedef	DWORD AVTune_Format;

typedef AVTune_Format *AVTune_FormatPtr;





#define 	AVTuneOption_AFT			0x0001

#define		AVTuneOption_Search			0x0002

#define		AVTuneOption_TableUpdate	0x0004

#define		AVTuneOption_Notify			0x0008

#define		AVTuneOption_Async			0x0010

 



/*

** AVTune, AVTunePtr

** Handle to Tuner Settings

*/

typedef struct {

	AVTune_Format			format;

	AVTune_Mode				tunemode;

	DWORD					options;

	union {

		AVTune_Frequency	frequency;

		AVTune_Channel		channel;

	}						mode;

	

	int						finetune;

}	AVTune;

typedef AVTune *AVTunePtr;



/*

** AVInputType, AVInputTypeHdl

** indicate input type

*/

typedef enum {

	AVInput_Unknown,

	AVInput_Tuner,

	AVInput_Composite,

	AVInput_CompositeOrLuma,

	AVInput_FM,

	AVInput_SVideo

}	AVInputType;

typedef AVInputType *AVInputTypePtr;



typedef DWORD AVInputTypeVal;

typedef AVInputTypeVal *AVInputTypeValPtr;





/*

** AVAudioMute, AVAudioMutePtr

** Handle to Audio Mute Settings

*/

typedef enum {	

	AVAudioMute_Error,

	AVAudioMute_Yes,

	AVAudioMute_No,

	AVAudioMute_ForcedYes,				

	AVAudioMute_ForcedNo

}	AVAudioMute;				

typedef AVAudioMute *AVAudioMutePtr;



typedef enum {	

	AVAudioMode_Error,

	AVAudioMode_Mono,

	AVAudioMode_Stereo,

	AVAudioMode_SAP,

	AVAudioMode_MixedSAP,

	AVAudioMode_BiLingual

}	AVAudioMode;				

typedef AVAudioMode *AVAudioModePtr;



/*

** AVAudioVolume, AVAudioVolumePtr

** Handle to Audio Volume Settings

*/

typedef int AVAudioVolume;				//    0 .. 100 (0 Min .. 100 Max)

typedef AVAudioVolume *AVAudioVolumePtr;





/*

** AVAudioBalance, AVAudioBalancePtr

** Handle to Audio Balance Settings

*/

typedef int AVAudioBalance;				// -100 .. 100 (-100 Right Muted, 0 Left=Right, 100 Left Muted)

typedef AVAudioBalance *AVAudioBalancePtr;





/*

** AVAudioBass, AVAudioBassPtr

** Handle to Audio Bass Settings

*/

typedef int AVAudioBass;				// -100 .. 100 (-100 Min Bass, 0 Normal Bass, 100 Max Bass)

typedef AVAudioBass *AVAudioBassPtr;





/*

** AVAudioTreble, AVAudioTreblePtr

** Handle to Audio Treble Settings

*/

typedef int AVAudioTreble;				// -100 .. 100 (-100 Min Treble, 0 Normal Treble, 100 Max Treble)

typedef AVAudioTreble *AVAudioTreblePtr;





/*

** AVAudioSeparation, AVAudioSeparationPtr

** Handle to Audio Separation Settings

*/

typedef int AVAudioSeparation;			//    0 .. 100 (0 Mono .. 100 Stereo)

typedef AVAudioSeparation *AVAudioSeparationPtr;





/*

** AVAudioLoudness, AVAudioLoudnessPtr

** Handle to Audio Loudness Settings

*/

typedef enum {

	AVAudioLoudness_Error,

	AVAudioLoudness_Off,

	AVAudioLoudness_On

}	AVAudioLoudness;

typedef AVAudioLoudness *AVAudioLoudnessPtr;





/*

** AVAudioDualMono, AVAudioDualMonoPtr

** Handle to Audio "Dual Mono" Settings

** NOTE: Some hardware has two composite video inputs but only one set of stereo audio inputs which must be

** shared (manually switched).  Dual Mono "On" Mode allows you to use the single set of stereo audio inputs

** as two distinct mono inputs (one for each composite input).

*/

typedef enum {

	AVAudioDualMono_Error,

	AVAudioDualMono_Off,

	AVAudioDualMono_On

}	AVAudioDualMono;

typedef AVAudioDualMono *AVAudioDualMonoPtr;





/*

** AVAudioAutoMute, AVAudioAutoMutePtr

** Handle to Audio "Auto Mute" Settings

** NOTE: In AutoMute "On" mode, audio is automatically muted when the selected source does

** not have a valid Video Stream.

*/

typedef enum {

	AVAudioAutoMute_Error,

	AVAudioAutoMute_Off,

	AVAudioAutoMute_On

}	AVAudioAutoMute;

typedef AVAudioAutoMute *AVAudioAutoMutePtr;





typedef DWORD AVMixerID;

typedef AVMixerID *AVMixerIDPtr;







/*

** AVTunerAudioSignal, AVTunerAudioSignalPtr

** Handle to detected Tuner Audio Signal details.

*/

#define AVTUNERAUDIOSIGNAL_Mono				0x01    /* Simple, PlainJane, Mono */

#define AVTUNERAUDIOSIGNAL_Stereo			0x02    /* Stereo Detected */

#define AVTUNERAUDIOSIGNAL_SAP				0x04    /* SAP Detected */

#define AVTUNERAUDIOSIGNAL_Language_A		0x08    /* Bilingual or trilingual Detected */

#define AVTUNERAUDIOSIGNAL_Language_B		0x10 

#define AVTUNERAUDIOSIGNAL_Language_C		0x20



typedef DWORD AVTunerAudioSignal;

typedef AVTunerAudioSignal *AVTunerAudioSignalPtr;



/*

** AVVideoMode

** 

*/

#define AVVIDEOMODE_None					0x0000

#define AVVIDEOMODE_SVideo					0x0001

#define AVVIDEOMODE_BW						0x0002



typedef DWORD AVVideoModes;

typedef AVVideoModes *AVVideoModesPtr;





/*

** AVTunerMinMax, AVTunerMinMaxPtr

** Tuner Minimum / Maximum frequencey structur

** and structure handle.

*/

typedef long	AVTunerFrequency;

typedef DWORD	AVTunerChannelNo;

typedef AVTunerFrequency AVTunerMinFreq;

typedef AVTunerFrequency AVTunerMaxFreq;

typedef AVTunerChannelNo AVTunerMinChan;

typedef AVTunerChannelNo AVTunerMaxChan;



typedef struct {

	

	AVTune_Channel		channel;

	AVTunerMinFreq		minFreq;

	AVTunerMaxFreq		maxFreq;

	AVTunerMinChan		minChan;

	AVTunerMaxChan		maxChan;



}	AVTuneMinMax;

typedef AVTuneMinMax *AVTuneMinMaxPtr;









/*

** AVVideoFormat

** These are videoformats returned by hcwAV_Get and hcwAV_Set

*/

#define AVVideoFormat_NONE				0x0000

#define AVVideoFormatNTSC_M				0x0001

#define AVVideoFormatPAL_BGHIDK			0x0002

#define AVVideoFormatSECAM				0x0004

#define AVVideoFormatPAL_M				0x0008

#define AVVideoFormatPAL_N				0x0010

#define AVVideoFormatNTSC_443			0x0020

#define AVVideoFormatPAL_NCOMBO			0x0040

#define AVVideoFormatNTSC_Japan			0x0080

#define AVVideoFormatPAL_DK				0x0100



typedef DWORD  AVVideoFormat; 

typedef AVVideoFormat *AVVideoFormatPtr;





typedef struct {

	AVTune_Country			country;

	AVTune_Format			format;

	AVVideoFormat			videoformat;

	DWORD					options;

}AVTunerType;



typedef AVTunerType *AVTunerTypePtr;





/* 

** AVDriverInfo

** This structure is used to retrieve information about

** the driver

*/



typedef struct {

	DWORD dwDriverNameBuffSize;

	LPSTR lpstrDriverNameString;

} AVDriverInfo;

typedef AVDriverInfo *AVDriverInfoPtr;



/*

** hcwAV_Get, hcwAV_Set

** These routines do all the work.

** The dIdentifier parameter is typically a handle to the hardware or a specific input on a specific hardware.

** The AVFunction parameter specifies the function to be performed on the dIdentifier handle.

** The dQualifier parameter is typically a handle for passing parameters and return values.

** These routines all return a AVError (cast to a DWORD) to indicate Success or Failure.

*/

#ifdef __cplusplus

	extern "C" {

#endif



DWORD WINAPI hcwAV_Get( DWORD dDetail, AVFunction avFunc, PVOID dQualifier );

DWORD WINAPI hcwAV_Set( DWORD dDetail, AVFunction avFunc, PVOID dQualifier );

DWORD WINAPI hcwAV_InitByType ( WORD wBoardType, DWORD dMisc );

DWORD WINAPI hcwAV_Free ( DWORD hdl );

hcwAVHdl WINAPI hcwAV_InitByDetail( WORD wBoardType, DWORD dFeatures, WORD wGetType, DWORD dMisc );

hcwAVHdl WINAPI hcwAV_Init( hcwHW_Hdl hHW );



#ifdef __cplusplus

	}

#endif





#endif  //HCW_AV_H



