/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	WIN32
#include	"CDuckData.h"
#include	"CDuckAudioStream.h"
#include	"CDuckBuffer.h"
#include	"CDuckAudioSrc95.h"

#include	"CDuckAudioDest95.h"

#include "danbug.h"

CDuckAudioDest95::CDuckAudioDest95(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					int freq,int width16,int mono)
		:CDuckAudioDest()
{
	mSpeaker = NULL;
	mSrcFreq = SamplesPerSec;

	const uint bps( wave->wBitsPerSample);
	const ulong spb( wave->wSamplesPerBlock);

	audioSpeakerFactory *sf = audioSpeakerFactory::make();

	if(!mono) 
		{	// avoid frequency conversion

		sf->setFreq( mSrcFreq);			// 11025 allowed
		sf->setChan( audio::altStereo);
		sf->setWidth( audio::Sixteen);

		if( !(mSpeaker = sf->exact())) 
			{
			mono = 1; // failed, try mono
			}
		}
	if(mono) 
		{	// maybe width sb 8
		if (freq == 0)  
			freq = mSrcFreq;
		sf->setWidth( width16 ? audio::Sixteen : audio::Eight);
		sf->setFreq( freq);
		sf->setChan( (!mono) ? audio::altStereo : audio::Mono);
		if( !(mSpeaker = sf->exact()))
			{
			delete sf;
			throw CDuckAudioDest::AudioFormatException();
			}
		}
	delete sf;
	if(!mSpeaker)
		throw CDuckAudioDest::InitException();
}

CDuckAudioDest95::~CDuckAudioDest95(void)
{
	if (mSpeaker)
		delete mSpeaker;
	mSpeaker = NULL;
}

int
CDuckAudioDest95::ReadyForMore(void)
{
	return (mSpeaker ? mSpeaker->canAcceptMax(3000) >= 3000 : 0);

}

void
CDuckAudioDest95::Feed(CDuckAudioSrc &src)
{
CDuckAudioSrc95	*src95 = (CDuckAudioSrc95 *)&src;

 dprintf("CDuckAudioDest95::Feed\n");
	if (src95->CanProduce())
		mSpeaker->take(*((src95->Native())->Continue()));
}

void
CDuckAudioDest95::FeedSilence(CDuckAudioSrc &src)
{

	if (mSrcFreq)
		{
		audioNullSource s(mSrcFreq);
		mSpeaker->take(s);
		}
}

int
CDuckAudioDest95::GetFrequency(void)
{
	return (int)mSrcFreq;
}


void 
CDuckAudioDest95::StartPlaying(void) 
{            
	if (mSpeaker)
		mSpeaker->startPlaying();
}

void 
CDuckAudioDest95::StopPlaying(void) 
{ 
	if(mSpeaker) 
		mSpeaker->stopPlaying();
}

int 
CDuckAudioDest95::IsPlaying(void) 
{ 
	return mSpeaker ? mSpeaker->isPlaying() : 0;
}

void 
CDuckAudioDest95::PauseSpeaker(void) 
{ 
	if(mSpeaker) 
		mSpeaker->pause();
}

void 
CDuckAudioDest95::ResumeSpeaker(void) 
{ 
	if(mSpeaker) 
		mSpeaker->resume();
}

int 
CDuckAudioDest95::ElapsedAudioTime(void) 
{ 
	return mSpeaker ? mSpeaker->time() : 0;
}

int 
CDuckAudioDest95::AudioSkipped(void) 
{
	return (mSpeaker->samplesPlayed() > mSpeaker->samplesAccepted());
}

long 
CDuckAudioDest95::SamplesPlayed(void) 
{ 
	return mSpeaker->samplesPlayed(); 
}

long 
CDuckAudioDest95::SamplesAccepted(void) 
{ 
	return mSpeaker->samplesAccepted(); 
}
#endif
