/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	WIN32

#include	<MegaHeaders.h>
#include	"CDuckAudioSrcDirect.h"
#include	"CDuckAudioDestDirect.h"
#include	<vfw.h>
#include	"CDuckData.h"
#include	"CDuckAudioStream.h"
#include	"CDuckBuffer.h"
#include	<dsound.h>

#include "danbug.h"

#define TWOBUFFERS	1
// NT will not write to primary buffer

// constant from directX
#define DSBCAPS_GLOBALFOCUS         0x00008000 


CDuckAudioDestDirect::CDuckAudioDestDirect(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					HWND window)
		:CDuckAudioDest()
{
try 
	{
	int				err;
	PCMWAVEFORMAT	pcmwf;
    HRESULT			hr;
    DSBUFFERDESC	dsbdesc;

	CoInitialize(NULL);
	mSrcFreq = SamplesPerSec;
	mDirectSound = NULL;
	mAudioPrimary = NULL;
	mAudioBuffer = NULL;

	while (::GetParent(window)) 
		window = ::GetParent(window);
	
    if(CoCreateInstance(CLSID_DirectSound,NULL,CLSCTX_ALL,
            IID_IDirectSound,(void **)&mDirectSound) != DS_OK)
		{
		::MessageBox(window,"DirectSound not available",
					"Audio Error",MB_OK | MB_ICONERROR);		
		throw CDuckAudioDest::InitException();
		}

    if ((hr=mDirectSound->Initialize(NULL)) != DS_OK)
		{


		::MessageBox(window,"DirectSound could not initialize.",
					"Audio Error",MB_OK | MB_ICONERROR);		
		throw CDuckAudioDest::InitException();
		}

#ifdef	TWOBUFFERS
	err = mDirectSound->SetCooperativeLevel(window,DSSCL_PRIORITY);//NORMAL);//EXCLUSIVE);//PRIORITY);//NORMAL);
#else
	err = mDirectSound->SetCooperativeLevel(window,DSSCL_WRITEPRIMARY);//PRIORITY);//NORMAL);//EXCLUSIVE);//PRIORITY);//NORMAL);
#endif
	if (err != DS_OK)
		throw CDuckAudioDest::InitException();

	// Set up wave format structure.
    memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));

	switch(wave->wFormatTag)
		{
		case VOXWARE_FORMAT_TAG: case 0x0062: case 0x0061: case 0xFE0D: case 0xFE0C: case 0:case 1:

		switch (wave->wBitsPerSample)
			{
			case 3:
#ifndef USE_DUCK
				::MessageBox(window,"Unsupported Audio format(1). Try disabling DirectSound",
					"Audio Error",MB_OK | MB_ICONERROR);
				throw CDuckAudioDest::AudioFormatException();
#endif
				pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
				pcmwf.wf.nChannels = 2;
				pcmwf.wf.nSamplesPerSec = wave->nSamplesPerSec;
				pcmwf.wf.nBlockAlign = 4;
				pcmwf.wf.nAvgBytesPerSec = 
						pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
				pcmwf.wBitsPerSample = 16;
				break;
			case 4:
#ifndef	USE_DUCK
				::MessageBox(window,"Unsupported Audio format(2). Try disabling DirectSound",
					"Audio Error",MB_OK | MB_ICONERROR);
				throw CDuckAudioDest::AudioFormatException();
#endif
				pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
				pcmwf.wf.nChannels = wave->nChannels;
				pcmwf.wf.nSamplesPerSec = 44100;
				pcmwf.wf.nBlockAlign = 2 * wave->nChannels;
				pcmwf.wf.nAvgBytesPerSec = 
						pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
				pcmwf.wBitsPerSample = 16;
				break;
			case 8: case 16:
				pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
				pcmwf.wf.nChannels = wave->nChannels;
				pcmwf.wf.nSamplesPerSec = wave->nSamplesPerSec;
				pcmwf.wf.nBlockAlign = wave->nChannels * (wave->wBitsPerSample / 8);
				pcmwf.wf.nAvgBytesPerSec = 
					pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
				pcmwf.wBitsPerSample = wave->wBitsPerSample;
				break;
			default:
				::MessageBox(window,"Unsupported Audio format(3). Try disabling DirectSound",
					"Audio Error",MB_OK | MB_ICONERROR);
				throw CDuckAudioDest::AudioFormatException();
				break;
			}
			break; // duck or voxware format tag
		default:
#ifdef	USE_ACM
			MMRESULT mmr;
			DWORD cbwfx;
			LPWAVEFORMATEX	out_format;
			char			*bogus_out_format;
			if (mmr = acmMetrics(NULL, ACM_METRIC_MAX_SIZE_FORMAT, &cbwfx))
				{
				throw CDuckAudioDest::InitException();
				}

			bogus_out_format = new char[cbwfx];
			out_format = (LPWAVEFORMATEX)bogus_out_format;
			memset(out_format,0,cbwfx);
			memset(&pcmwf,0,sizeof(pcmwf));

			out_format->wFormatTag = WAVE_FORMAT_PCM;
			if (mmr = acmFormatSuggest(NULL,(LPWAVEFORMATEX)wave,out_format,cbwfx,
				0/*ACM_FORMATSUGGESTF_WFORMATTAG*/))
				{
				::MessageBox(window,"No audio conversion.","Audio Error",MB_OK | MB_ICONERROR);
				delete [] bogus_out_format;
				throw CDuckAudioDest::AudioFormatException();
				}

						
			pcmwf.wf.wFormatTag		= WAVE_FORMAT_PCM;//wave->wFormatTag;
			pcmwf.wf.nChannels		= wave->nChannels;
			pcmwf.wf.nSamplesPerSec = wave->nSamplesPerSec;
			pcmwf.wBitsPerSample	= 16;//wave->wBitsPerSample;
			pcmwf.wf.nBlockAlign	= 2 * wave->nChannels;//wave->nBlockAlign;
			pcmwf.wf.nAvgBytesPerSec= wave->nSamplesPerSec * wave->nChannels * 2;
										//wave->nAvgBytesPerSec;
#else
			::MessageBox(window,"Unsupported Audio format(4). Try disabling DirectSound",
				"Audio Error",MB_OK | MB_ICONERROR);
			throw CDuckAudioDest::AudioFormatException();
#endif
			break;
		}
    // Set up DSBUFFERDESC structure.
#ifdef TWOBUFFERS
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    //dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT; // Need default controls 
	dsbdesc.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME |DSBCAPS_CTRLFREQUENCY;


	dsbdesc.dwFlags |= DSBCAPS_GLOBALFOCUS;




    dsbdesc.dwBufferBytes = mBufferSize = 2 * pcmwf.wf.nAvgBytesPerSec; // 2 second buffer.
    dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
    // Create buffer.
    hr = mDirectSound->CreateSoundBuffer(&dsbdesc,&mAudioBuffer, NULL);
	if (hr != DS_OK)
		throw CDuckAudioDest::InitException();
#endif

    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
	dsbdesc.dwBufferBytes = 0;
    dsbdesc.lpwfxFormat = NULL;
	hr = mDirectSound->CreateSoundBuffer(&dsbdesc,&mAudioPrimary,NULL);
	if (hr != DS_OK)
		throw CDuckAudioDest::InitException();
	hr = mAudioPrimary->SetFormat((WAVEFORMATEX *)&pcmwf);
	if (hr != DS_OK)
		throw CDuckAudioDest::InitException();

#ifndef	TWOBUFFERS
	{
	DSBCAPS dsbcaps;
	dsbcaps.dwSize = sizeof(DSBCAPS);
	mAudioPrimary->GetCaps(&dsbcaps);
	mBufferSize = dsbcaps.dwBufferBytes;
	mAudioBuffer = mAudioPrimary;
	}
#endif

	mLastWriteIndex = -1;
	mBytesConsumed	= 0;
	mBlockAlign		= pcmwf.wf.nBlockAlign;
	mSamplesPerSec	= pcmwf.wf.nSamplesPerSec;
	}
catch(...)
	{
	if (mDirectSound)
		mDirectSound->Release();
	mDirectSound = NULL;
	throw;
	}
	
}

CDuckAudioDestDirect::~CDuckAudioDestDirect(void)
{
#ifdef TWOBUFFERS
	if (mAudioBuffer)
		mAudioBuffer->Release();
#endif
	mAudioBuffer = NULL;

	if (mAudioPrimary)
		mAudioPrimary->Release();
	mAudioPrimary = NULL;
	if (mDirectSound)
		mDirectSound->Release();
	mDirectSound = NULL;
	CoUninitialize();

}



int
CDuckAudioDestDirect::ReadyForMore(void)
{
	// cant really tell with direct sound???
	DWORD	current_play_cursor,current_write_cursor;
	mAudioBuffer->GetCurrentPosition(&current_play_cursor,&current_write_cursor);
	
	return (mLastWriteIndex == -1 ||
		(mLastWriteIndex != current_play_cursor)
		);

}

void
CDuckAudioDestDirect::Feed(CDuckAudioSrc &src)
{
char	*ptr1,*ptr2;
DWORD	siz1,siz2,a1 = 0,a2 = 0;
int		err;
DWORD	current_play_cursor,current_write_cursor;
mAudioBuffer->GetCurrentPosition(&current_play_cursor,&current_write_cursor);

CDuckAudioSrcDirect	*srcDirect = (CDuckAudioSrcDirect *)&src;

 dprintf("CDuckAudioDestDirect::Feed\n");
	if (mLastWriteIndex == -1)
		mLastWriteIndex = 0;

	if (srcDirect->CanProduce())
		{
		int	dataleft = srcDirect->GetDataLeft();
		const char *dataptr = srcDirect->GetDataPtr();
		int		space_in_buffer;

		if (current_play_cursor <= mLastWriteIndex)
			{
			space_in_buffer = mBufferSize - (mLastWriteIndex - current_play_cursor);
			}
		else
			{
			space_in_buffer = current_play_cursor - mLastWriteIndex;
			}

		if (space_in_buffer < dataleft)
			dataleft = space_in_buffer;

		if (dataleft)
			{

			err = mAudioBuffer->Lock(mLastWriteIndex,dataleft,
				(void**)&ptr1,&siz1,(void**)&ptr2,&siz2,0);//DSBLOCK_FROMWRITECURSOR);
			if (err == DS_OK)
				{
				if (ptr1)
					{
					a1 = (siz1 > dataleft ? dataleft : siz1);
					memcpy(ptr1,dataptr,a1);
					dataptr += a1;
					dataleft -= a1;
					}
				if (ptr2)
					{
					a2 = (siz2 > dataleft ? dataleft : siz2);
					memcpy(ptr2,dataptr,a2);
					dataptr += a2;
					dataleft -= a2;
					}
				}
			srcDirect->DecreaseDataPtr(a1+a2);
			err = mAudioBuffer->Unlock(ptr1,a1,ptr2,a2);
			mLastWriteIndex += (a1 + a2);
			mBytesConsumed += (a1 + a2);
			if (mLastWriteIndex >= mBufferSize)
				mLastWriteIndex -= mBufferSize;
			}		
		}
 dprintf("CDuckAudioDestDirect::Feed exit\n");
}

void
CDuckAudioDestDirect::FeedSilence(CDuckAudioSrc &src)
{
	char	*ptr1,*ptr2;
	DWORD	siz1,siz2;
	DWORD	current_play_cursor,current_write_cursor;
	mAudioBuffer->GetCurrentPosition(&current_play_cursor,&current_write_cursor);
	int		err;
	int		space_in_buffer;

	if (current_play_cursor <= mLastWriteIndex)
		space_in_buffer = mBufferSize - (mLastWriteIndex - current_play_cursor);
	else
		space_in_buffer = current_play_cursor - mLastWriteIndex;

	if (space_in_buffer)
		{
		err = mAudioBuffer->Lock(mLastWriteIndex,space_in_buffer,
			(void**)&ptr1,&siz1,(void**)&ptr2,&siz2,0);//DSBLOCK_FROMWRITECURSOR);
		if (err == DS_OK)
			{
			if (ptr1)
				memset(ptr1,0,siz1);
			if (ptr2)
				memset(ptr2,0,siz2);
			}
		err = mAudioBuffer->Unlock(ptr1,siz1,ptr2,siz2);
		mLastWriteIndex += (siz1 + siz2);
		mBytesConsumed += (siz1 + siz2);
		if (mLastWriteIndex >= mBufferSize)
			mLastWriteIndex -= mBufferSize;
		}		
}

int
CDuckAudioDestDirect::GetFrequency(void)
{
	DWORD	freq;

	mAudioBuffer->GetFrequency(&freq);
	return freq;
}



void 
CDuckAudioDestDirect::StartPlaying(void) 
{            
	mAudioBuffer->SetCurrentPosition(0);

	mAudioBuffer->Play(0,0,DSBPLAY_LOOPING);
}

void 
CDuckAudioDestDirect::StopPlaying(void) 
{ 
	char	*ptr1,*ptr2;
	DWORD	siz1,siz2;
	int		err;

#if 0

	err = mAudioBuffer->Lock(mLastWriteIndex,mBufferSize,
		&ptr1,&siz1,&ptr2,&siz2,0);//DSBLOCK_FROMWRITECURSOR);
	if (err == DS_OK)
		{
		if (ptr1)
			memset(ptr1,0,siz1);
		if (ptr2)
			memset(ptr2,0,siz2);
		}
	err = mAudioBuffer->Unlock(ptr1,siz1,ptr2,siz2);
#endif


	mAudioBuffer->Stop();
	mLastWriteIndex = -1;
	mBytesConsumed = 0;
}

int 
CDuckAudioDestDirect::ElapsedAudioTime(void) 
{ 
	// take bytes consumed and subtract the amount not yet played
	_int64	bytesplayed;

	DWORD	current_play_cursor,current_write_cursor;
	mAudioBuffer->GetCurrentPosition(&current_play_cursor,&current_write_cursor);

	if (current_play_cursor < mLastWriteIndex)
		bytesplayed = mBytesConsumed - (mLastWriteIndex - current_play_cursor);
	else
		bytesplayed = mBytesConsumed - (mBufferSize - (current_play_cursor - mLastWriteIndex));

	// now convert the bytes into time
	return (1000 * bytesplayed) / (mBlockAlign * mSamplesPerSec);
}

int		
CDuckAudioDestDirect::AudioSkipped(void)
{
	return 0;	// cant be calculated
}

#endif
