/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	__POWERPC__
#include	"CDuckAudioDestMAC.h"
#include	"CDuckAudioSrcMAC.h"
#include	"CDuckData.h"
#include	"CDuckAudioStream.h"
#include	"CDuckBuffer.h"
#include	<system_ticks.h>


CDuckAudioDestMAC::CDuckAudioDestMAC(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					int freq,int width16,int mono)
		:CDuckAudioDest()
{
	mSpeaker = NULL;
	mSrcFreq = SamplesPerSec;

	const uint bps( wave->wBitsPerSample);
	const ulong spb( wave->wSamplesPerBlock);

	audioSpeakerFactory *sf = audioSpeakerFactory::make();

	if(!mono) 
		{	// avoid frequency conversion

		sf->setFreq( mSrcFreq);			// 11025 allowed
		sf->setChan( audio::altStereo);
		sf->setWidth( audio::Sixteen);

		//if( !(mSpeaker = sf->usual())) 
		if( !(mSpeaker = sf->exact())) 
			{
			mono = 1; // failed, try mono
			}
		}
	if(mono) 
		{	// maybe width sb 8
		if (freq == 0)  
			freq = mSrcFreq;
		sf->setWidth( width16 ? audio::Sixteen : audio::Eight);
		sf->setFreq( freq);
		sf->setChan( (!mono) ? audio::altStereo : audio::Mono);
		//if( !(mSpeaker = sf->usual()))
		if( !(mSpeaker = sf->exact()))
			{
			delete sf;
			throw CDuckAudioDest::InitException();
			}
		}
	delete sf;
	if(!mSpeaker)
		throw CDuckAudioDest::InitException();
}

CDuckAudioDestMAC::~CDuckAudioDestMAC(void)
{
	if (mSpeaker)
		delete mSpeaker;
	mSpeaker = NULL;
}

int
CDuckAudioDestMAC::ReadyForMore(void)
{
	return (mSpeaker ? mSpeaker->canAcceptMax(3000) >= 3000 : 0);

}

void
CDuckAudioDestMAC::Feed(CDuckAudioSrc &src)
{
CDuckAudioSrcMAC	*srcMAC = (CDuckAudioSrcMAC *)&src;

	if (srcMAC->CanProduce())
		mSpeaker->take(*((srcMAC->Native())->Continue()));
}

void
CDuckAudioDestMAC::FeedSilence(CDuckAudioSrc &src)
{

	if (mSrcFreq)
		{
		audioNullSource s(mSrcFreq);
		mSpeaker->take(s);
		}
}

int
CDuckAudioDestMAC::GetFrequency(void)
{
	return (int)mSrcFreq;
}


void 
CDuckAudioDestMAC::StartPlaying(void) 
{            
	if (mSpeaker)
		mSpeaker->startPlaying();
	m8kTime = timeGetTime();
}

void 
CDuckAudioDestMAC::StopPlaying(void) 
{ 
	if(mSpeaker) 
		mSpeaker->stopPlaying();
}

int 
CDuckAudioDestMAC::IsPlaying(void) 
{ 
	return mSpeaker ? mSpeaker->isPlaying() : 0;
}

void 
CDuckAudioDestMAC::PauseSpeaker(void) 
{ 
	if(mSpeaker) 
		mSpeaker->pause();
}

void 
CDuckAudioDestMAC::ResumeSpeaker(void) 
{ 
	if(mSpeaker) 
		mSpeaker->resume();
}

int 
CDuckAudioDestMAC::ElapsedAudioTime(void) 
{ 
// tim's stuff only seems to work for compressed (dk3) audio
	//if (mSrcFreq == 8000)
		return timeGetTime() - m8kTime;

	//return mSpeaker ? mSpeaker->time() : 0;
}

int 
CDuckAudioDestMAC::AudioSkipped(void) 
{
	return (mSpeaker->samplesPlayed() > mSpeaker->samplesAccepted());
}

long 
CDuckAudioDestMAC::SamplesPlayed(void) 
{ 
	return mSpeaker->samplesPlayed(); 
}

long 
CDuckAudioDestMAC::SamplesAccepted(void) 
{ 
	return mSpeaker->samplesAccepted(); 
}
#endif
