/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	IRIX
#include	"CDuckAudioDestSGI.h"
#include	"CDuckAudioSrcUnix.h"
#include	"CDuckData.h"
#include	"CDuckAudioStream.h"
#include	"CDuckBuffer.h"
#include	<milli_sleep.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<errno.h>

CDuckAudioDestSGI::CDuckAudioDestSGI(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format)
		:CDuckAudioDest()
{
	mBytesConsumed = 0;
	mAudioConfig = NULL;
	mAudioPort = NULL;
	mAudioConfig = ALnewconfig();
	mNumChannels = NumChannels;

	if (ALsetchannels(mAudioConfig,NumChannels) == -1)
		throw InitException();

	if (ALsetsampfmt(mAudioConfig,AL_SAMPFMT_TWOSCOMP) == -1)
		throw InitException();

	if (ALsetqueuesize(mAudioConfig,SamplesPerSec * 2) == -1)
		throw InitException();

	mSrcFreq = SamplesPerSec;
	int bits_per_sample;
	mBytesPerSample = 2;
	switch(wave->wBitsPerSample)
		{
		case 3: case 4:
			bits_per_sample = 16;
			break;
		case 8:
			mBytesPerSample = 1;
			bits_per_sample = 8;
			break;
		case 16:
			bits_per_sample = 16;
			break;
		default:
			throw AudioFormatException();
		}

	if (ALsetwidth(mAudioConfig,
			wave->wBitsPerSample == 8 ? AL_SAMPLE_8 : AL_SAMPLE_16) == -1)
		throw AudioFormatException();

	mAudioPort = ALopenport("duck_audio","w",mAudioConfig);
	if (mAudioPort == (ALport) 0)
		{
		int err = oserror();
		switch(err)
			{
			case AL_BAD_NO_PORTS:
				fprintf(stderr,"System is out of audio ports\n");
				break;
			case AL_BAD_DEVICE_ACCESS:
				fprintf(stderr,"Could not access audio port\n");
				break;
			case AL_BAD_OUT_OF_MEM:
				fprintf(stderr,"audio out of memory\n");
				break;
			default:
				fprintf(stderr,"Unknown audio error\n");
			}
		throw InitException();
		}

	long pvbuf[2];
	pvbuf[0] = AL_OUTPUT_RATE;
	pvbuf[1] = SamplesPerSec;
	if (ALsetparams(AL_DEFAULT_DEVICE,pvbuf,2) < 0)
		{
		fprintf(stderr,"Could not set sample rate %d\n",SamplesPerSec);
		throw InitException();
		}

}

CDuckAudioDestSGI::~CDuckAudioDestSGI(void)
{
	ALfreeconfig(mAudioConfig);
	ALcloseport(mAudioPort);
}

int
CDuckAudioDestSGI::ReadyForMore(void)
{
	return ALgetfillable(mAudioPort);
}

void
CDuckAudioDestSGI::Feed(CDuckAudioSrc &src)
{
	CDuckAudioSrcUnix	*sgi_src = (CDuckAudioSrcUnix *)(&src);
	if (sgi_src->CanProduce())
		{
		int	dataleft 		= sgi_src->GetDataLeft();
		const char *dataptr = sgi_src->GetDataPtr();

		int space_left = ALgetfillable(mAudioPort) * mBytesPerSample;
		if (space_left > dataleft)
			space_left = dataleft;

		ALwritesamps(mAudioPort,(void *)dataptr,space_left / mBytesPerSample);
		sgi_src->DecreaseDataPtr(space_left);
		mBytesConsumed += space_left;
		}
}

void
CDuckAudioDestSGI::FeedSilence(CDuckAudioSrc &src)
{
	char	mem[10000];
	memset(mem,0,10000);
	int space_left = ALgetfillable(mAudioPort) * mBytesPerSample;
	if (space_left > 10000)
		space_left = 10000;
	mBytesConsumed += space_left;
	ALwritesamps(mAudioPort,(void *)mem,space_left / mBytesPerSample);	
}

int
CDuckAudioDestSGI::GetFrequency(void)
{
	return mSrcFreq;
}



void 
CDuckAudioDestSGI::StartPlaying(void) 
{            
}

void 
CDuckAudioDestSGI::StopPlaying(void) 
{ 
	alDiscardFrames(mAudioPort,1000000);
	while (ALgetfilled(mAudioPort) != 0) milli_sleep(100);
	mBytesConsumed = 0;
}

int 
CDuckAudioDestSGI::ElapsedAudioTime(void) 
{ 
	// convert bytes consumed to samples
	long long	samples_consumed = mBytesConsumed / mBytesPerSample;

	// subtract unplayed samples in queue
	samples_consumed -= ALgetfilled(mAudioPort);

	// convert samples to milliseconds
	return (samples_consumed *1000) / (mSrcFreq * mNumChannels);
}

int 
CDuckAudioDestSGI::AudioSkipped(void) 
{
	return 0;
}

#endif
