/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	SOLARIS
#include	<MegaHeaders.h>
#include	"CDuckAudioDestSUN.h"
#include	"CDuckAudioSrcUnix.h"
#include	"CDuckData.h"
#include	"CDuckAudioStream.h"
#include	"CDuckBuffer.h"
#include	<sys/types.h>
#include	<stropts.h>
#include	<sys/conf.h>
 
CDuckAudioDestSUN::CDuckAudioDestSUN(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format)
		:CDuckAudioDest()
{
	mReadyForMore = 1;
	mAudioAccumulator = 0;
	mAudioTimeMultiplier = 1;
	mAudioDevice = -1;
	mBytesConsumed = 0;
	mNumChannels = NumChannels;


	mSrcFreq = SamplesPerSec;
	int bits_per_sample;
	mBytesPerSample = 2;
	switch(wave->wBitsPerSample)
		{
		case 3: case 4:
			bits_per_sample = 16;
			break;
		case 8:
			mBytesPerSample = 1;
			bits_per_sample = 8;
			break;
		case 16:
			bits_per_sample = 16;
			break;
		default:
			throw AudioFormatException();
		}

	mAudioDevice = -1;
	mAudioDevice = open("/dev/audio",O_WRONLY | O_NONBLOCK);
	if (mAudioDevice == -1) throw InitException();
	AUDIO_INITINFO(&mAudioInfo);
	
	// 22 must set to 11 for 8 bit... why????
	// for 16 bit we use normal data rate
	mAudioInfo.play.sample_rate = mSrcFreq;
	mAudioInfo.play.channels = mNumChannels;
	mAudioInfo.play.precision = bits_per_sample;
	if (bits_per_sample == 8)
		{
		mAudioInfo.play.sample_rate /= 2; // SUN bug
		mAudioTimeMultiplier = 2;
		}
	mAudioInfo.play.encoding = AUDIO_ENCODING_LINEAR;
	// not supported
	//mAudioInfo.play.buffer_size = 
	//	(mSrcFreq * mNumChannels * bits_per_sample) / 8;
	if (ioctl(mAudioDevice,AUDIO_SETINFO,&mAudioInfo) == -1) 
		{
		close(mAudioDevice);
		mAudioDevice = -1;
		throw AudioFormatException();
		}

}

CDuckAudioDestSUN::~CDuckAudioDestSUN(void)
{
	if (mAudioDevice != -1)
		close(mAudioDevice);
	mAudioDevice = -1;
}

int
CDuckAudioDestSUN::ReadyForMore(void)
{
	int res = mReadyForMore;
	mReadyForMore = 1;
	return res;
}

void
CDuckAudioDestSUN::Feed(CDuckAudioSrc &src)
{
	CDuckAudioSrcUnix	*sun_src = (CDuckAudioSrcUnix *)(&src);
	if (sun_src->CanProduce())
		{
		int amount_written;
		int	dataleft 		= sun_src->GetDataLeft();
		const char *dataptr = sun_src->GetDataPtr();

		amount_written = write(mAudioDevice,(void *)dataptr,dataleft);
		if (amount_written == -1 && errno != EAGAIN)
			throw IOException();
		if (amount_written > 0)
			{
			sun_src->DecreaseDataPtr(amount_written);
			mBytesConsumed += amount_written;
			mReadyForMore = 1;
			}
		else 
			{
			mReadyForMore = 0;
			}
		}
}

void
CDuckAudioDestSUN::FeedSilence(CDuckAudioSrc &src)
{
	char	mem[10000];
	memset(mem,0,10000);
	mBytesConsumed += write(mAudioDevice,(void *)mem,10000);
}

int
CDuckAudioDestSUN::GetFrequency(void)
{
	return mSrcFreq;
}



void 
CDuckAudioDestSUN::StartPlaying(void) 
{            
}

void 
CDuckAudioDestSUN::StopPlaying(void) 
{
	if (ioctl(mAudioDevice,I_FLUSH,FLUSHW) == -1)
		throw IOException(); 
	if (ioctl(mAudioDevice,AUDIO_GETINFO,&mAudioInfo) == -1)
		throw IOException();
	
	mAudioAccumulator = mAudioInfo.play.samples;
}

int 
CDuckAudioDestSUN::ElapsedAudioTime(void) 
{ 
	if (ioctl(mAudioDevice,AUDIO_GETINFO,&mAudioInfo) == -1)
		throw IOException();
	
	long long samples = mAudioInfo.play.samples;
	samples -= mAudioAccumulator;
	long long millisecs = 1000;

	int result = mAudioTimeMultiplier *
				((samples * millisecs) / mSrcFreq);
	//cout << result << endl;
	return result;
}

int 
CDuckAudioDestSUN::AudioSkipped(void) 
{
	if (ioctl(mAudioDevice,AUDIO_GETINFO,&mAudioInfo) == -1)
		throw IOException();

	return mAudioInfo.play.error;
}

#endif
