/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	WIN32

#ifdef	_AFXDLL
#include	<afxwin.h>
#else
#include	<windows.h>
#endif


#include	<MegaHeaders.h>
#include	<string.h>
#include	"CDuckAudioSrc95.h"
#include	"CDuckAudioDest.h"
#include	"CDuckData.h"
#include	"audioAVI.h"

#ifdef	USE_VOXWARE

// voxware startup kludge routine
#ifdef	VOXWARE_KLUDGE
extern VOXWARE_RETCODE GetDecodeHeader(unsigned char * lpHeaderBuffer, short wHeaderBufferSize,
                                unsigned long dwCodecID,short *new_size);
#endif

#define	DECOMPRESS_SIZE	64000

void 
CDuckAudioSrc95::InitializeVoxware(const char *first_data_chunk,int len)
{
	int err;
	short vox_size;
	mDecompressBuffer = new char[DECOMPRESS_SIZE];

#ifdef	VOXWARE_KLUDGE
	unsigned char HeaderBuffer[512];
    GetDecodeHeader(HeaderBuffer, 512, VOXWARE_AUDIO_CODEC_08,&vox_size);//VOXWARE_CODEC_AC_8K);
#endif

	memset(&mVoxwareData,0,sizeof(mVoxwareData));
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
#ifdef	VOXWARE_KLUDGE
	mVoxwareData.dwInputType = TYPE_RAW_VOX_BITSTREAM;
	mVoxwareData.lpInputLocation = HeaderBuffer;
	mVoxwareData.dwInputSize = vox_size;
#else
	mVoxwareData.dwInputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_VOX;
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputSize = len;
#endif
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputPosition = 0;
	mVoxwareData.dwOutputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_PCM;
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.dwOutputSamplingRate = 8000;
	mVoxwareData.wOutputBytesPerSample = VOXWARE_SAMPLE_SIZE_16BIT;
	mVoxwareData.lpCallbackFunc = NULL;
	mVoxwareData.fWarpedLengthFactor = 1.0;
	mVoxwareData.bUseAbsolutePitchFlag = USE_RELATIVE_PITCH;
	mVoxwareData.bUseAbsoluteGainFlag = USE_RELATIVE_GAIN;
	mVoxwareData.lpVoiceFont = NO_VOICE_FONT;
	mVoxwareData.dwCompressionSettings = VOXWARE_AUDIO_CODEC_08;//VOXWARE_CODEC_RT_8K;

	if ((err = DLLdcmpVoxInit(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();

	mVoxwareUp = 1;

#ifdef	VOXWARE_KLUDGE
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#else
	mVoxwareData.dwInputSize = len - mVoxwareData.dwInputPosition;
	mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#endif



#ifdef	DEBUG_VOXWARE

//		TraceBuffer((const char *)mVoxwareData.lpInputLocation,mVoxwareData.dwInputSize);

#endif


	if ((err = DLLdcmpVoxConvert(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();


#ifdef	DEBUG_VOXWARE

		TraceBuffer((const char *)mDecompressBuffer,mVoxwareData.dwOutputPosition);

#endif

}

void 
CDuckAudioSrc95::CleanupVoxware(void)
{
	if (mVoxwareUp)

		{

		mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
		DLLdcmpVoxFree(&mVoxwareData);
		}

	delete [] mDecompressBuffer;
}

void
CDuckAudioSrc95::VoxwareDecompress(const char *data,int len)
{
	if (!mGotFirstDataChunk)
		{
		InitializeVoxware(data,len);
		}
	else
		{
		mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
		mVoxwareData.lpOutputLocation = mDecompressBuffer;
		mVoxwareData.lpInputLocation = (void *)data;
		mVoxwareData.dwInputSize = len;
		mVoxwareData.dwInputPosition = 0;
		mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
		mVoxwareData.dwOutputPosition = 0;


#ifdef	DEBUG_VOXWARE

//		TraceBuffer(data, len);

#endif

		

		if (DLLdcmpVoxConvert(&mVoxwareData) != VOX_NO_ERROR)
			throw DecompressException();
#ifdef	DEBUG_VOXWARE

		TraceBuffer((const char *)mDecompressBuffer,mVoxwareData.dwOutputPosition);

#endif

		}
	mGotFirstDataChunk = 1;
}

#endif




///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
CDuckAudioSrc95::CDuckAudioSrc95(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					CDuckAudioDest &dest,HWND window)
					: CDuckAudioSrc()
{
#ifdef	USE_VOXWARE

#ifdef DEBUG_VOXWARE

	mTraceFileInitialized = FALSE;
#endif

	mVoxwareUp = 0;

	mGotFirstDataChunk = 0;
	mDecompressBuffer = NULL;
	mUseVoxware = (wave->wBitsPerSample == 16 && wave->wFormatTag == VOXWARE_FORMAT_TAG);
	if (mUseVoxware && !VoxwareDLLCtor())

		{

		::MessageBox(window,"Cannot process voxware audio format",

			"Audio Error",MB_OK | MB_ICONERROR);

		throw AudioFormatException();

		}

	if (mUseVoxware)	// voxware 64 bytes -> 1024 bytes
		{
		wave->wSamplesPerBlock = 512;
		mAviFact = NULL;
		mSrcFreq = dest.GetFrequency();

		const uint bps( wave->wBitsPerSample);
		const ulong spb( wave->wSamplesPerBlock);

		mAviFact = makeAVIaudioFactory( NumChannels,SamplesPerSec,
			1,bps,spb);
		}
	else {
#endif
	
	
	mAviFact = NULL;
	mSrcFreq = dest.GetFrequency();

	const uint bps( wave->wBitsPerSample);
	const ulong spb( wave->wSamplesPerBlock);

	mAviFact = makeAVIaudioFactory( NumChannels,SamplesPerSec,
		(Format == NEWDK3_FORMAT_TAG ? OLDDK3_FORMAT_TAG : Format),bps,spb);

#ifdef USE_VOXWARE
	}
#endif
}


CDuckAudioSrc95::~CDuckAudioSrc95(void)
{
	if (mAviFact)
		delete mAviFact;
	mAviFact = NULL;
	mSrcFreq = 0;
#ifdef	USE_VOXWARE
	if (mUseVoxware)
		CleanupVoxware();
	VoxwareDLLDtor();

#endif
}


void
CDuckAudioSrc95::Feed(CDuckData &data)
{
#ifdef	USE_VOXWARE
	if (mUseVoxware)
		{
		VoxwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
		mAviFact->newBlock( (const uchar *)mDecompressBuffer,
				mVoxwareData.dwOutputPosition);
		}
	else
#endif
	mAviFact->newBlock( (const uchar *)data.GetRawData(),
		data.GetRawDataLength());
}


int
CDuckAudioSrc95::ReadyForMore(void)
{
	return 	(mAviFact->Continue() == NULL ? 1 : 0);
}

void 
CDuckAudioSrc95::Clear(void) 
{ 
	if( mAviFact) 
		mAviFact->invalidateBlock();
}

audioBlockFactory	*
CDuckAudioSrc95::Native(void)
{
	return mAviFact;
}

int		
CDuckAudioSrc95::CanProduce(void)
{
	audioSource	*as = mAviFact->Continue();
	return (as ? as->canProduce() : 0);
}

unsigned long 
CDuckAudioSrc95::Skip(unsigned long samples)
{
	audioSource *as = mAviFact->Continue();
	return (as ? as->skip(samples) : 0);
}



#ifdef	DEBUG_VOXWARE

BOOL CDuckAudioSrc95::TraceBuffer(const char* data, int len)	

{																

	// Make sure we have valid input							 

	ASSERT(data != NULL);// && data[0] != 0);						

	ASSERT(len > 0);											

																

	// Open the trace file										

	UINT uOpenFlags = CFile::modeWrite		|					

		              CFile::shareExclusive |					

		              CFile::typeBinary;						

																

	if (mTraceFileInitialized)									

		uOpenFlags |= CFile::modeCreate | CFile::modeNoTruncate;

	else														

		uOpenFlags |= CFile::modeCreate;						

										

	mTraceFileInitialized = TRUE;

	if (!mTraceFile.Open("d:\\Voxware.trc", uOpenFlags, NULL))		

	{															

		AfxMessageBox("Unable to open Voxware trace file", MB_OK);

																

		return FALSE;											

	}															

																

	// Position to end of the file								

	try															

	{															

		mTraceFile.SeekToEnd();								

	}															

	catch(CFileException* e)									

	{															

		e->Delete();											

		AfxMessageBox("Error positioning in trace file", MB_OK);

		return FALSE;											

	}															

																

	// Write buffer to trace file and close the file            

	try															

	{															

		mTraceFile.Write(data, len);							

	}															

	catch(CFileException* e)									

	{															

		e->Delete();														

		mTraceFile.Close();										

		AfxMessageBox("Error writing to trace file", MB_OK);	

		return FALSE;											

	}															

																

	mTraceFile.Close();											

}																

#endif

#endif
