/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	WIN32

#include	<MegaHeaders.h>
#include	"CDuckAudioSrcDirect.h"
#include	<vfw.h>
#include	"CDuckAudioDest.h"
#include	"CDuckData.h"
#include	<assert.h>
#include	<..\dansoft\acmenc.h>

#include "danbug.h"

#define	MAX_OUTPUT_SIZE 400000
#define MAX_SOURCE_SIZE 16384

unsigned char gAcmSrcBuffer[MAX_SOURCE_SIZE];

CDuckAudioSrcDirect::CDuckAudioSrcDirect(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					CDuckAudioDest &dest,HWND window)
					: CDuckAudioSrc()
{
	mInputDataPtr = NULL;
	mInputDataLeft = 0;
	mDecompressBuffer = NULL;
	mWindow = window;

#ifdef	USE_ACM
	mUseAcm = 0;
 dprintf("USE_ACM defined\n");
#else
 dprintf("USE_ACM undefined\n");
#endif
#ifdef	USE_VOXWARE

	mUseVoxware = 0;

	mVoxwareUp = 0;
#endif
#ifdef	USE_DUCK
	mUseDuckware = 0;
	mDuckAudioDest = NULL;
	mDuckAudioSrc = NULL;
	mDuckWaveform = *wave;
#endif

	mFirstAudioChunk = 1;
	int allocate_decompress_buffer = 0;

	switch(wave->wFormatTag)
		{
		///	dbm -- adding 0x6751 (vorbis) NO!
//		case VOXWARE_FORMAT_TAG: case 0x0062: case 0x0061: case 0xFE0D: case 0xFE0C: case 0:case 1: case 0x6751:
		case VOXWARE_FORMAT_TAG: case 0x0062: case 0x0061: case 0xFE0D: case 0xFE0C: case 0:case 1:
		// Duck, Voxware, or uncompressed
			switch(mBitsPerSample = wave->wBitsPerSample)
				{
				case 3:
					mBlockAlign = 4;
					mSamplesPerSec = 44100;
					break;
				case 4:
					mBlockAlign = 2 * wave->nChannels;
					mSamplesPerSec = 44100;
					break;
				default:
					mBlockAlign = wave->nChannels * (wave->wBitsPerSample / 8);
					mSamplesPerSec = wave->nSamplesPerSec;
					dprintf("---BlockAlign=%d, SamplesPerSec=%d\n",mBlockAlign,mSamplesPerSec);
					break;
				}

#ifdef	USE_VOXWARE
			mUseVoxware = (mBitsPerSample == 16 && wave->wFormatTag == VOXWARE_FORMAT_TAG);
			if (mUseVoxware && !VoxwareDLLCtor())

				{

				::MessageBox(window,"Cannot process voxware audio format",

					"Audio Error",MB_OK | MB_ICONERROR);

				throw AudioFormatException();

				}

			if (!allocate_decompress_buffer)
				allocate_decompress_buffer = mUseVoxware;
			if (mUseVoxware)	// voxware 64 bytes -> 1024 bytes
				wave->wSamplesPerBlock = 512;	
#endif
#ifdef	USE_DUCK
			mUseDuckware = mBitsPerSample < 8;
			if (!allocate_decompress_buffer)
				allocate_decompress_buffer = mUseDuckware;
#endif
			break;

#ifdef USE_ACM
		case ACMVORBIS:									///	vorbis ACM
			dbgout("case default\n");
			dprintf("mBlockAlign=%d\n",mBlockAlign);
//|
			mBlockAlign=4;								///	note this kluge -- should be initalized by stream!!!
														///	hard-coded for Vorbis
			mUseAcm = 1;
			allocate_decompress_buffer = 1;
			break;
 dprintf("CDuckAudioSrcDirect: mUseAcm=%d\n",mUseAcm);
#endif
		default:
			::MessageBox(window,"Cannot process audio format",
					"Audio Error",MB_OK | MB_ICONERROR);
			throw AudioFormatException();
			break;
		}

	if (allocate_decompress_buffer)
		{
		mDecompressBuffer = new char[MAX_OUTPUT_SIZE];	// max output size
		memset(mDecompressBuffer,0,MAX_OUTPUT_SIZE);
		}
}


CDuckAudioSrcDirect::~CDuckAudioSrcDirect(void)
{
	if (mDecompressBuffer)
		delete [] mDecompressBuffer;
	mDecompressBuffer = NULL;
#ifdef	USE_DUCK
	if (mUseDuckware)
		CleanupDuckware();
#endif
#ifdef USE_VOXWARE
	if (mUseVoxware)
		CleanupVoxware();
	VoxwareDLLDtor();

#endif
#ifdef	USE_ACM
	if (mUseAcm)
		CleanupAcm();
#endif
}

#ifdef USE_ACM

void
CDuckAudioSrcDirect::InitializeAcm(const char *first_data_chunk,int len,
			DKWAVEFORM *wave,int num_channels,int samples_per_sec,HWND window)
{
///	 we need to: acmstreamprepareheader, check if DKWAVEFORM is big enough, acmstreamconvert, think about
///	block alignment & chunkage....

 dprintf("InitializeAcm -- len=%d\n",len);
	MMRESULT result;
	WAVEFORMATEX	out_wave;

	if(!wave->nBlockAlign) wave->nBlockAlign=4;	///	dbm-PrepareHeader crashes if =0 -- 
												///	it seems to be zero in some streams -- 
												///	I assume 1 makes more sense. What do you think?
												///	VDub has it at 4!
	wave->cbSize =8;							///	why?  Good question.  Is HFB getting the extra stuff?

	wave->nAvgBytesPerSec = 8000;				///	force this -- 11025 for seeking, but Vorbis gets confused

 dprintf("CDuckAudioSrcDirect::InitializeAcm -- src:");
 dprintf("                                      wFormatTag:     %x @ %d",wave->wFormatTag,(char*)&wave->wFormatTag-(char*)wave);
 dprintf("                                      nChannels:      %d @ %d",wave->nChannels,(char*)&wave->nChannels-(char*)wave);
 dprintf("                                      nSamplesPerSec: %d @ %d",wave->nSamplesPerSec,(char*)&wave->nSamplesPerSec-(char*)wave);
 dprintf("                                      nAvgBytesPerSec %d @ %d",wave->nAvgBytesPerSec,(char*)&wave->nAvgBytesPerSec-(char*)wave);
 dprintf("                                      nBlockAlign:    %d @ %d",wave->nBlockAlign,(char*)&wave->nBlockAlign-(char*)wave);
 dprintf("                                      wBitsPerSample: %d @ %d",wave->wBitsPerSample,(char*)&wave->wBitsPerSample-(char*)wave);
 dprintf("                                      cbSize:         %d @ %d",wave->cbSize,(char*)&wave->cbSize-(char*)wave);
 dprintf("                                      extrashort1:    %x ",wave->extrashort1);
 dprintf("                                      extrashort2:    %x ",wave->extrashort2);
 dprintf("                                      extrashort3:    %x ",wave->extrashort3);
 dprintf("                                      extra 1:        %x ",((long*)&wave->wBitsPerSample)[1]);
 dprintf("                                      extra 2:        %x ",((long*)&wave->wBitsPerSample)[2]);
 dprintf("                                      SIZE:           %d",sizeof(*wave));

	out_wave.wFormatTag = WAVE_FORMAT_PCM;
	out_wave.nChannels = wave->nChannels;
	out_wave.nSamplesPerSec = wave->nSamplesPerSec;
	out_wave.wBitsPerSample = 16;
	out_wave.nBlockAlign = 2 * out_wave.nChannels;
	out_wave.nAvgBytesPerSec = out_wave.nBlockAlign * out_wave.nSamplesPerSec;
	out_wave.cbSize = 0;

 dprintf("CDuckAudioSrcDirect::InitializeAcm -- dst:");
 dprintf("                                      wFormatTag:     %x",out_wave.wFormatTag);
 dprintf("                                      nChannels:      %d",out_wave.nChannels);
 dprintf("                                      nSamplesPerSec: %d",out_wave.nSamplesPerSec);
 dprintf("                                      nAvgBytesPerSec %d",out_wave.nAvgBytesPerSec);
 dprintf("                                      nBlockAlign:    %d",out_wave.nBlockAlign);
 dprintf("                                      wBitsPerSample: %d",out_wave.wBitsPerSample);
 dprintf("                                      cbSize:         %d",out_wave.cbSize);

	///	casting DKWAVEFORM-->WAVEFORMATEX??? is this valid?  YES
	result = acmStreamOpen(&mAcmStream,NULL,(LPWAVEFORMATEX)wave,&out_wave,				
		NULL,NULL/*reinterpret_cast<unsigned long>(window)*/,NULL,0);//CALLBACK_WINDOW);

 dprintf("CDuckAudioSrcDirect::InitializeAcm -- A\n");
	if (result)
		throw DecompressException();
 dprintf("acmStreamOpen gives us this stream handle: %x\n",mAcmStream);

	unsigned long ssout;
	result = acmStreamSize(mAcmStream,len, &ssout, ACM_STREAMSIZEF_DESTINATION);
 if(result) dprintf("error codes: invalf=%d, invalh=%d, invalp=%d, nomem=%d, notposs=%d\n",MMSYSERR_INVALFLAG,MMSYSERR_INVALHANDLE,MMSYSERR_INVALPARAM,MMSYSERR_NOMEM,ACMERR_NOTPOSSIBLE);
 dprintf("recommended output size given input = %d is %d (result=%d)\n",len,ssout,result);
	result = acmStreamSize(mAcmStream,MAX_OUTPUT_SIZE, &ssout, ACM_STREAMSIZEF_SOURCE);
 dprintf("recommended source buf size given dest size = %d is %d (result=%d)\n",MAX_OUTPUT_SIZE,ssout,result);

	mAcmStreamHeader.cbStruct = sizeof(mAcmStreamHeader);
	mAcmStreamHeader.fdwStatus = 0;
	mAcmStreamHeader.dwUser = 0;
	mAcmStreamHeader.pbSrc = gAcmSrcBuffer;					///	ACM requires src in same place all the time!
	mAcmStreamHeader.cbSrcLength = MAX_SOURCE_SIZE;
	mAcmStreamHeader.pbDst = (unsigned char *)mDecompressBuffer;
	mAcmStreamHeader.cbDstLength = MAX_OUTPUT_SIZE;

 dprintf("acmStreamPrep: insize=%d, outsize=%d\n",mAcmStreamHeader.cbSrcLength,mAcmStreamHeader.cbDstLength);		///	UNK: -- crashing here

	int r=acmStreamPrepareHeader(mAcmStream,&mAcmStreamHeader,0);				///	gotta do whatcha gotta do
 dprintf("after prep, fdwStatus = %x\n",mAcmStreamHeader.fdwStatus);
 dprintf("CDuckAudioSrcDirect::InitializeAcm -- acmStreamPrepareHeader returns %d\n",r);
 if(r) dprintf("error codes: invalf=%d, invalh=%d, invalp=%d, nomem=%d, notposs=%d\n",MMSYSERR_INVALFLAG,MMSYSERR_INVALHANDLE,MMSYSERR_INVALPARAM,MMSYSERR_NOMEM,ACMERR_NOTPOSSIBLE);
	if(r) throw DecompressException();

	for(int x=0; x<len; x++) {
		gAcmSrcBuffer[x] = first_data_chunk[x];
	}
 dprintf("  compressed data in stream:");
 dumpdata((unsigned char*)gAcmSrcBuffer,3);
 mAcmStreamHeader.cbSrcLength=len;

//]	mAcmStreamHeader.cbDstLength = len;						/// should be in!!!
 dprintf("len = %d, mAcmStreamHeader.cbDstLength = %d\n",len,mAcmStreamHeader.cbDstLength);
 dprintf("   mAcmStreamHeader.cbStruct  = %d",mAcmStreamHeader.cbStruct   );
 dprintf("                    fdwStatus = %x",mAcmStreamHeader.fdwStatus  );
 dprintf("                    dwUser    = %d",mAcmStreamHeader.dwUser     );
 dprintf("                    cbSrcLength=%d",mAcmStreamHeader.cbSrcLength);
 dprintf("                    cbDstLength=%d",mAcmStreamHeader.cbDstLength);

	r=acmStreamConvert(mAcmStream, &mAcmStreamHeader, ACM_STREAMCONVERTF_BLOCKALIGN | ACM_STREAMCONVERTF_START);
	if(r) throw DecompressException();

 dprintf("----------acmStreamConvert returns %d; src used=%d, dest used=%d\n",
			r,mAcmStreamHeader.cbSrcLengthUsed,mAcmStreamHeader.cbDstLengthUsed);
	mInputDataLeft = mAcmStreamHeader.cbDstLengthUsed;
	mInputDataPtr = mDecompressBuffer;
 dprintf("CDuckAudioSrcDirect::InitializeAcm done\n");
 	wave->nAvgBytesPerSec = 11025;				///	force this -- 11025 for seeking, but Vorbis gets confused

}

void
CDuckAudioSrcDirect::CleanupAcm(void)
{
	acmStreamUnprepareHeader(mAcmStream,&mAcmStreamHeader,0);				///	gotta do whatcha gotta do
	acmStreamClose(mAcmStream, 0);
}

void
CDuckAudioSrcDirect::AcmDecompress(const char *data,int len)
{

///	note len = length of source data (STUPID!)
 dbgout("===AcmDecompress...\n");
 dprintf("data is at %p",data);
	int x;
	assert(mDecompressBuffer);
	assert(MAX_SOURCE_SIZE >= len);

	for(x=0; x<len; x++) {
		gAcmSrcBuffer[x] = data[x];
	}

 dprintf("  -compressed data in stream:");
 dumpdata((unsigned char*)gAcmSrcBuffer,3);
	mAcmStreamHeader.cbSrcLength=len;
	mAcmStreamHeader.cbDstLength=MAX_OUTPUT_SIZE;

 dprintf("   mAcmStreamHeader.cbStruct  = %d",mAcmStreamHeader.cbStruct   );
 dprintf("                    fdwStatus = %x",mAcmStreamHeader.fdwStatus  );
 dprintf("                    dwUser    = %d",mAcmStreamHeader.dwUser     );
 dprintf("                    cbSrcLength=%d",mAcmStreamHeader.cbSrcLength);
 dprintf("                    cbDstLength=%d",mAcmStreamHeader.cbDstLength);

 dprintf(" handle=%x",mAcmStream);
	int r=acmStreamConvert(mAcmStream, &mAcmStreamHeader, ACM_STREAMCONVERTF_BLOCKALIGN);
 dprintf("==========acmStreamConvert returns %d; src used=%d, dest used=%d\n",
			r,mAcmStreamHeader.cbSrcLengthUsed,mAcmStreamHeader.cbDstLengthUsed);
 if(r) dprintf("error codes: busy=%d, uprep=%d, invalf=%d, invalh=%d, invalp=%d\n",ACMERR_BUSY,ACMERR_UNPREPARED,MMSYSERR_INVALFLAG,MMSYSERR_INVALHANDLE,MMSYSERR_INVALPARAM);
//	if(r) throw DecompressException();
	mInputDataLeft = mAcmStreamHeader.cbDstLengthUsed;
	mInputDataPtr = mDecompressBuffer;
 dprintf("===AcmDecompress returns with mInputDataLeft=%d, mInputDataPtr=%p\n",mInputDataLeft,mInputDataPtr);
}

#endif

#ifdef	USE_DUCK

#ifdef	EXPLORER
int	audio_is_up = 0;
#endif

void
CDuckAudioSrcDirect::InitializeDuckware(const char *first_data_chunk,int len,
			DKWAVEFORM *wave,int num_channels,int samples_per_sec)
{
	assert(mDecompressBuffer);

	if (mDuckAudioDest || mDuckAudioSrc)
		CleanupDuckware();

#ifdef	EXPLORER
	if (!audio_is_up)
		DXL_InitAudio(20,20);
	audio_is_up = 1;
#else
	DXL_InitAudio(1,1);
#endif
	

	DKWAVEFORM w1;

	memcpy(&w1,wave,sizeof(DKWAVEFORM));

	if (w1.wFormatTag == NEWDK3_FORMAT_TAG)

		w1.wFormatTag = OLDDK3_FORMAT_TAG;
	mDuckAudioSrc = DXL_CreateXAudioSrc(&w1,(unsigned char *)first_data_chunk,len);
	mDuckAudioDest = DXL_CreateAudioDst(mDecompressBuffer,NULL,MAX_OUTPUT_SIZE,
		16,num_channels,samples_per_sec);

	if (!mDuckAudioSrc || !mDuckAudioDest)
		throw DecompressException();
	int gotten = DXL_dxAudio(mDuckAudioSrc,mDuckAudioDest,50000);
	mInputDataPtr = mDecompressBuffer;
	mInputDataLeft = gotten * mBlockAlign;

}

void
CDuckAudioSrcDirect::CleanupDuckware(void)
{
	if (mDuckAudioDest)
		DXL_DestroyAudioDst(mDuckAudioDest);
	mDuckAudioDest = NULL;

	if (mDuckAudioSrc)
		DXL_DestroyXAudioSrc(mDuckAudioSrc);
	mDuckAudioSrc = NULL;

#ifndef	EXPLORER
	DXL_ExitAudio();
#endif

}

void
CDuckAudioSrcDirect::DuckwareDecompress(const char *data,int len)
{
	int	gotten;

	DXL_AlterAudioDst(mDuckAudioDest,(unsigned char *)mDecompressBuffer,
		NULL,0,0,0,0);
	DXL_AlterXAudioData(mDuckAudioSrc,(unsigned char *)data,len);
	//mDuckAudioDest->addrL = (unsigned char *)mDecompressBuffer;
	//mDuckAudioDest->index = 0;
	gotten = DXL_dxAudio(mDuckAudioSrc,mDuckAudioDest,50000);
	mInputDataLeft = gotten * mBlockAlign;
	mInputDataPtr = mDecompressBuffer;
}

#endif


#ifdef	USE_VOXWARE

// voxware startup kludge routine
#ifdef	VOXWARE_KLUDGE
extern VOXWARE_RETCODE GetDecodeHeader(unsigned char * lpHeaderBuffer, short wHeaderBufferSize,
                                unsigned long dwCodecID,short *new_size);
#endif

void 
CDuckAudioSrcDirect::InitializeVoxware(const char *first_data_chunk,int len)
{
	int err;
	short vox_size;
	assert(mDecompressBuffer);

#ifdef	VOXWARE_KLUDGE
	unsigned char HeaderBuffer[512];
    GetDecodeHeader(HeaderBuffer, 512, VOXWARE_AUDIO_CODEC_08,&vox_size);//VOXWARE_CODEC_AC_8K);
#endif

	memset(&mVoxwareData,0,sizeof(mVoxwareData));
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
#ifdef	VOXWARE_KLUDGE
	mVoxwareData.dwInputType = TYPE_RAW_VOX_BITSTREAM;
	mVoxwareData.lpInputLocation = HeaderBuffer;
	mVoxwareData.dwInputSize = vox_size;
#else
	mVoxwareData.dwInputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_VOX;
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputSize = len;
#endif
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputPosition = 0;
	mVoxwareData.dwOutputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_PCM;
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.dwOutputSamplingRate = 8000;
	mVoxwareData.wOutputBytesPerSample = VOXWARE_SAMPLE_SIZE_16BIT;
	mVoxwareData.lpCallbackFunc = NULL;
	mVoxwareData.fWarpedLengthFactor = 1.0;
	mVoxwareData.bUseAbsolutePitchFlag = USE_RELATIVE_PITCH;
	mVoxwareData.bUseAbsoluteGainFlag = USE_RELATIVE_GAIN;
	mVoxwareData.lpVoiceFont = NO_VOICE_FONT;
	mVoxwareData.dwCompressionSettings = VOXWARE_AUDIO_CODEC_08;//VOXWARE_CODEC_RT_8K;

	if ((err = DLLdcmpVoxInit(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();

	mVoxwareUp = 1;

#ifdef	VOXWARE_KLUDGE
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#else
	mVoxwareData.dwInputSize = len - mVoxwareData.dwInputPosition;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#endif
	if ((err = DLLdcmpVoxConvert(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();
	mInputDataLeft = mVoxwareData.dwOutputPosition;
	mInputDataPtr = mDecompressBuffer;
}

void 
CDuckAudioSrcDirect::CleanupVoxware(void)
{
	if (mVoxwareUp)

		{

		mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
		DLLdcmpVoxFree(&mVoxwareData);
		}

}

void
CDuckAudioSrcDirect::VoxwareDecompress(const char *data,int len)
{
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.lpInputLocation = (void *)data;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
	if (DLLdcmpVoxConvert(&mVoxwareData) != VOX_NO_ERROR)
		throw DecompressException();
	mInputDataLeft = mVoxwareData.dwOutputPosition;
	mInputDataPtr = mDecompressBuffer;
}

#endif

const char *
CDuckAudioSrcDirect::GetDataPtr(void)
{
	return mInputDataPtr;
}

int			
CDuckAudioSrcDirect::GetDataLeft(void)
{
	return mInputDataLeft;
}

void		
CDuckAudioSrcDirect::DecreaseDataPtr(int howmuch)
{
	mInputDataPtr += howmuch;
	mInputDataLeft -= howmuch;
}



void
CDuckAudioSrcDirect::Feed(CDuckData &data)
{
	char* dat;
	unsigned int len;
	int offset=4;

 dprintf("CDuckAudioSrcDirect::Feed -- mDataLen=%d\n",data.GetRawDataLength());
	if (mDecompressBuffer)
		{
		// cant decompress yet!1
#ifdef	USE_ACM
		if (mUseAcm) {
			dat = (char*)data.GetRawData();
			len = *((unsigned int*)dat);
															dprintf(" dat=%p, len=%d\n",dat,len);
			if(len > data.GetRawDataLength()) {				///	probably raw ogg stream without length header
															///	(made outside of VpV??)
				len = data.GetRawDataLength();
				offset = 0;
				dprintf("old style file -- no length header");
			}
															dprintf("  ::Feed A\n");
			if (mFirstAudioChunk) {
///				InitializeAcm((const char *)data.GetRawData(),data.GetRawDataLength(),
///					&mDuckWaveform,mDuckWaveform.nChannels,mSamplesPerSec,mWindow);
															dprintf("  ::Feed B\n");
				InitializeAcm(dat+offset, len,
					&mDuckWaveform,mDuckWaveform.nChannels,mSamplesPerSec,mWindow);
															dprintf("  ::Feed C\n");
			} else {
///				AcmDecompress((const char *)data.GetRawData(),data.GetRawDataLength());	///	old way (before length
															dprintf("  ::Feed D\n");
				AcmDecompress(dat+offset,len);
															dprintf("  ::Feed E\n");
			}
		} else
#endif
#ifdef	USE_DUCK
		if (mUseDuckware) 
			{
			if (mFirstAudioChunk)
				InitializeDuckware((const char *)data.GetRawData(),data.GetRawDataLength(),
					&mDuckWaveform,mDuckWaveform.nChannels,mSamplesPerSec);
			else
				DuckwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
			}
		else
#endif

#ifdef	USE_VOXWARE
		if (mUseVoxware)
			{
			if (mFirstAudioChunk)
				InitializeVoxware((const char *)data.GetRawData(),data.GetRawDataLength());
			else
				VoxwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
			}
		else
#endif
			{
															dprintf("  ::Feed F2\n");
			assert(0);										///	dbm note this was being called!
															///	crashed when I compiled debug mode
			}
		}
	else
		{
		assert(!mInputDataLeft);
		mInputDataLeft = data.GetRawDataLength();
		mInputDataPtr = (const char *)data.GetRawData();
		}
	mFirstAudioChunk = 0;
															dprintf("  ::Feed G\n");
}


int
CDuckAudioSrcDirect::ReadyForMore(void)
{
	return (mInputDataLeft == 0);
}

void 
CDuckAudioSrcDirect::Clear(void) 
{ 
	mInputDataLeft = 0;
}

int		
CDuckAudioSrcDirect::CanProduce(void)
{
 dprintf("==CDuckAudioSrcDirect::CanProduce -- mInputDataLeft = %d\n",mInputDataLeft);
	return mInputDataLeft;
//	return (mInputDataLeft / mBlockAlign);
}

unsigned long 
CDuckAudioSrcDirect::Skip(unsigned long samples)
{
 dprintf("==CDuckAudioSrcDirect::Skip -- mBlockAlign = %d, skipping %d\n",mBlockAlign,samples);
	int bytes_to_skip = samples * mBlockAlign;

	if (mInputDataLeft < bytes_to_skip)
		bytes_to_skip = mInputDataLeft;
	DecreaseDataPtr(bytes_to_skip);
 dprintf("==CDuckAudioSrcDirect::Skip done (%d bytes), returning %d\n",bytes_to_skip,bytes_to_skip / mBlockAlign);

	return (bytes_to_skip / mBlockAlign);
}

#endif
