/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	__POWERPC__

#include	<string.h>
#include	<MegaHeaders.h>
#include	"CDuckAudioSrcMAC.h"
#include	"CDuckAudioDest.h"
#include	"CDuckData.h"
#include	"audioavi.h"

#ifdef	USE_VOXWARE

#ifdef VOXWARE_WIN32
gaseous anomoloy captain
#endif

// voxware startup kludge routine
#ifdef	VOXWARE_KLUDGE
extern VOXWARE_RETCODE GetDecodeHeader(unsigned char * lpHeaderBuffer, short wHeaderBufferSize,
                                unsigned long dwCodecID,short *new_size);
#endif

#define	DECOMPRESS_SIZE	64000

void invert_data(char *d,int len)
{
	assert(!(len & 1));
	for (int x = 0; x < len; x+=2)
		{
		char t = d[x];
		d[x] = d[x+1];
		d[x+1] = t;
		}
}
void 
CDuckAudioSrcMAC::InitializeVoxware(const char *first_data_chunk,int len)
{
	int err;
	short vox_size;
	mDecompressBuffer = new char[DECOMPRESS_SIZE];

#ifdef	VOXWARE_KLUDGE
	unsigned char HeaderBuffer[512];
    GetDecodeHeader(HeaderBuffer, 512, VOXWARE_AUDIO_CODEC_08,&vox_size);
#endif

	memset(&mVoxwareData,0,sizeof(mVoxwareData));
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
#ifdef	VOXWARE_KLUDGE
	mVoxwareData.dwInputType = TYPE_RAW_VOX_BITSTREAM;
	mVoxwareData.lpInputLocation = HeaderBuffer;
	mVoxwareData.dwInputSize = vox_size;
#else
	mVoxwareData.dwInputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_VOX;
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputSize = len;
#endif
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputPosition = 0;
	mVoxwareData.dwOutputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_PCM;
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.dwOutputSamplingRate = 8000;
	mVoxwareData.wOutputBytesPerSample = VOXWARE_SAMPLE_SIZE_16BIT;
	mVoxwareData.lpCallbackFunc = NULL;
	mVoxwareData.fWarpedLengthFactor = 1.0;
	mVoxwareData.bUseAbsolutePitchFlag = USE_RELATIVE_PITCH;
	mVoxwareData.bUseAbsoluteGainFlag = USE_RELATIVE_GAIN;
	mVoxwareData.lpVoiceFont = NO_VOICE_FONT;
	mVoxwareData.dwCompressionSettings = VOXWARE_AUDIO_CODEC_08;

	if ((err = dcmpVoxInit(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();

#ifdef	VOXWARE_KLUDGE
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#else
	mVoxwareData.dwInputSize = len - mVoxwareData.dwInputPosition;
	mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#endif

#ifdef	DEBUG_VOXWARE
//		TraceBuffer((const char *)mVoxwareData.lpInputLocation,mVoxwareData.dwInputSize);
#endif

	if ((err = dcmpVoxConvert(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();

#ifdef	DEBUG_VOXWARE
		TraceBuffer((const char *)mDecompressBuffer,mVoxwareData.dwOutputPosition);
#endif
	invert_data(mDecompressBuffer,mVoxwareData.dwOutputPosition);
}

void 
CDuckAudioSrcMAC::CleanupVoxware(void)
{
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
	dcmpVoxFree(&mVoxwareData);
	delete [] mDecompressBuffer;
}

void
CDuckAudioSrcMAC::VoxwareDecompress(const char *data,int len)
{
	if (!mGotFirstDataChunk)
		{
		InitializeVoxware(data,len);
		}
	else
		{
		mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
		mVoxwareData.lpOutputLocation = mDecompressBuffer;
		mVoxwareData.lpInputLocation = (void *)data;
		mVoxwareData.dwInputSize = len;
		mVoxwareData.dwInputPosition = 0;
		mVoxwareData.dwOutputSize = DECOMPRESS_SIZE;
		mVoxwareData.dwOutputPosition = 0;

#ifdef	DEBUG_VOXWARE
//		TraceBuffer(data, len);
#endif
		
		if (dcmpVoxConvert(&mVoxwareData) != VOX_NO_ERROR)
			throw DecompressException();
#ifdef	DEBUG_VOXWARE
		TraceBuffer((const char *)mDecompressBuffer,mVoxwareData.dwOutputPosition);
#endif
		}
	mGotFirstDataChunk = 1;
	invert_data(mDecompressBuffer,mVoxwareData.dwOutputPosition);
}

#endif




///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
CDuckAudioSrcMAC::CDuckAudioSrcMAC(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					CDuckAudioDest &dest)
					: CDuckAudioSrc()
{
#ifdef	USE_VOXWARE
#ifdef DEBUG_VOXWARE
	mTraceFileInitialized = FALSE;
#endif
	mGotFirstDataChunk = 0;
	mDecompressBuffer = NULL;
	mUseVoxware = (wave->wBitsPerSample == 16 && wave->wFormatTag == VOXWARE_FORMAT_TAG);
	if (mUseVoxware)	// voxware 64 bytes -> 1024 bytes
		{
		wave->wSamplesPerBlock = 512;
		mAviFact = NULL;
		mSrcFreq = dest.GetFrequency();

		const uint bps( wave->wBitsPerSample);
		const ulong spb( wave->wSamplesPerBlock);

		mAviFact = makeAVIaudioFactory( NumChannels,SamplesPerSec,
			1,bps,spb);
		}
	else {
#endif
	
	
	mAviFact = NULL;
	mSrcFreq = dest.GetFrequency();

	const uint bps( wave->wBitsPerSample);
	const ulong spb( wave->wSamplesPerBlock);

	mAviFact = makeAVIaudioFactory( NumChannels,SamplesPerSec,
		(Format == NEWDK3_FORMAT_TAG ? OLDDK3_FORMAT_TAG : Format),bps,spb);

#ifdef USE_VOXWARE
	}
#endif
}


CDuckAudioSrcMAC::~CDuckAudioSrcMAC(void)
{
	if (mAviFact)
		delete mAviFact;
	mAviFact = NULL;
	mSrcFreq = 0;
#ifdef	USE_VOXWARE
	if (mUseVoxware)
		CleanupVoxware();
#endif
}


void
CDuckAudioSrcMAC::Feed(CDuckData &data)
{
#ifdef	USE_VOXWARE
	if (mUseVoxware)
		{
		VoxwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
		mAviFact->newBlock( (const uchar *)mDecompressBuffer,
				mVoxwareData.dwOutputPosition);
		}
	else
#endif
	mAviFact->newBlock( (const uchar *)data.GetRawData(),
		data.GetRawDataLength());
}


int
CDuckAudioSrcMAC::ReadyForMore(void)
{
	return 	(mAviFact->Continue() == NULL ? 1 : 0);
}

void 
CDuckAudioSrcMAC::Clear(void) 
{ 
	if( mAviFact) 
		mAviFact->invalidateBlock();
}

audioBlockFactory	*
CDuckAudioSrcMAC::Native(void)
{
	return mAviFact;
}

int		
CDuckAudioSrcMAC::CanProduce(void)
{
	audioSource	*as = mAviFact->Continue();
	return (as ? as->canProduce() : 0);
}

unsigned long 
CDuckAudioSrcMAC::Skip(unsigned long samples)
{
	audioSource *as = mAviFact->Continue();
	return (as ? as->skip(samples) : 0);
}

#ifdef	DEBUG_VOXWARE

BOOL CDuckAudioSrcMAC::TraceBuffer(const char* data, int len)	
{																
	// Make sure we have valid input							 
	ASSERT(data != NULL);// && data[0] != 0);						
	ASSERT(len > 0);											
																
	// Open the trace file										
	UINT uOpenFlags = CFile::modeWrite		|					
		              CFile::shareExclusive |					
		              CFile::typeBinary;						
																
	if (mTraceFileInitialized)									
		uOpenFlags |= CFile::modeCreate | CFile::modeNoTruncate;
	else														
		uOpenFlags |= CFile::modeCreate;						
										
	mTraceFileInitialized = TRUE;
	if (!mTraceFile.Open("d:\\Voxware.trc", uOpenFlags, NULL))		
	{															
		AfxMessageBox("Unable to open Voxware trace file", MB_OK);
																
		return FALSE;											
	}															
																
	// Position to end of the file								
	try															
	{															
		mTraceFile.SeekToEnd();								
	}															
	catch(CFileException* e)									
	{															
		e->Delete();											
		AfxMessageBox("Error positioning in trace file", MB_OK);
		return FALSE;											
	}															
																
	// Write buffer to trace file and close the file            
	try															
	{															
		mTraceFile.Write(data, len);							
	}															
	catch(CFileException* e)									
	{															
		e->Delete();														
		mTraceFile.Close();										
		AfxMessageBox("Error writing to trace file", MB_OK);	
		return FALSE;											
	}															
																
	mTraceFile.Close();											
}																
#endif

#endif
