/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if ((defined	IRIX) || (defined SOLARIS) || (defined LINUX))
#include	<MegaHeaders.h>
#include	"CDuckAudioSrcUnix.h"
#include	"CDuckAudioDest.h"
#include	"CDuckData.h"
#include	<string.h>
#include	<assert.h>
#include	<stdio.h>

#include	<dxl_audi.h>

#define	MAX_OUTPUT_SIZE	(44100 * 2 * 2)

CDuckAudioSrcUnix::CDuckAudioSrcUnix(DKWAVEFORM *wave,int NumChannels,
					int SamplesPerSec,int BytesPerSec,int Format,
					CDuckAudioDest &dest)
					: CDuckAudioSrc()
{
	mInputDataPtr = NULL;
	mInputDataLeft = 0;
	mDecompressBuffer = NULL;
#ifdef USE_VOXWARE
	mUseVoxware = 0;
#endif
#ifdef	USE_DUCK
	mUseDuckware = 0;
	mDuckAudioDest = NULL;
	mDuckAudioSrc = NULL;
	mDuckWaveform = *wave;
#endif

	mFirstAudioChunk = 1;
	int allocate_decompress_buffer = 0;

	switch(wave->wFormatTag)
		{
		case VOXWARE_FORMAT_TAG: case 0x0062: case 0x0061: case 0xFE0D: case 0xFE0C: case 0:case 1:
		// Duck, Voxware, or uncompressed
			switch(mBitsPerSample = wave->wBitsPerSample)
				{
				case 3:
					mBlockAlign = 4;
					mSamplesPerSec = 44100;
					break;
				case 4:
					mBlockAlign = 2 * wave->nChannels;
					mSamplesPerSec = 44100;
					break;
				default:
					mBlockAlign = wave->nChannels * (wave->wBitsPerSample / 8);
					mSamplesPerSec = wave->nSamplesPerSec;
					break;
				}
#ifdef	USE_VOXWARE
			mUseVoxware = (mBitsPerSample == 16 && wave->wFormatTag == VOXWARE_FORMAT_TAG);
			if (!allocate_decompress_buffer)
				allocate_decompress_buffer = mUseVoxware;
			if (mUseVoxware)	// voxware 64 bytes -> 1024 bytes
				wave->wSamplesPerBlock = 512;	
#endif

#ifdef	USE_DUCK
			mUseDuckware = mBitsPerSample < 8;
			if (!allocate_decompress_buffer)
				allocate_decompress_buffer = mUseDuckware;
#endif
			break;
		default:
			fprintf(stderr,"Cannot process audio format\n");
//			::MessageBox(window,"Cannot process audio format",
//					"Audio Error",MB_OK | MB_ICONERROR);
			throw AudioFormatException();
		}

	if (allocate_decompress_buffer)
		{
		mDecompressBuffer = new char[MAX_OUTPUT_SIZE];	// max output size
		memset(mDecompressBuffer,0,MAX_OUTPUT_SIZE);
		}
}


CDuckAudioSrcUnix::~CDuckAudioSrcUnix(void)
{
	if (mDecompressBuffer)
		delete [] mDecompressBuffer;
	mDecompressBuffer = NULL;
#ifdef	USE_DUCK
	if (mUseDuckware)
		CleanupDuckware();
#endif
#ifdef USE_VOXWARE
	if (mUseVoxware)
		CleanupVoxware();
#endif
}
#ifdef	USE_VOXWARE

// voxware startup kludge routine
#ifdef	VOXWARE_KLUDGE
extern VOXWARE_RETCODE GetDecodeHeader(unsigned char * lpHeaderBuffer, short wHeaderBufferSize,
                                unsigned long dwCodecID,short *new_size);
#endif

void 
CDuckAudioSrcUnix::InitializeVoxware(const char *first_data_chunk,int len)
{
	int err;
	short vox_size;
	assert(mDecompressBuffer);

#ifdef	VOXWARE_KLUDGE
	unsigned char HeaderBuffer[512];
	unsigned char *hbp = &(HeaderBuffer[0]);
	int hbp_len = 512;
	while (((long)hbp) % 4) { hbp++; hbp_len--; }
    GetDecodeHeader(hbp, hbp_len, VOXWARE_AUDIO_CODEC_08,&vox_size);//VOXWARE_CODEC_AC_8K);
#endif

	memset(&mVoxwareData,0,sizeof(mVoxwareData));
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
#ifdef	VOXWARE_KLUDGE
	mVoxwareData.dwInputType = TYPE_RAW_VOX_BITSTREAM;
	mVoxwareData.lpInputLocation = hbp;
	mVoxwareData.dwInputSize = vox_size;
#else
	mVoxwareData.dwInputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_VOX;
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputSize = len;
#endif
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputPosition = 0;
	mVoxwareData.dwOutputType = VOXTYPE_LOCATION_BUFFER | VOXTYPE_FORMAT_RAW | 
								VOXTYPE_QUALIFIER_PCM;
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.dwOutputSamplingRate = 8000;
	mVoxwareData.wOutputBytesPerSample = VOXWARE_SAMPLE_SIZE_16BIT;
	mVoxwareData.lpCallbackFunc = NULL;
	mVoxwareData.fWarpedLengthFactor = 1.0;
	mVoxwareData.bUseAbsolutePitchFlag = USE_RELATIVE_PITCH;
	mVoxwareData.bUseAbsoluteGainFlag = USE_RELATIVE_GAIN;
	mVoxwareData.lpVoiceFont = NO_VOICE_FONT;
	mVoxwareData.dwCompressionSettings = VOXWARE_AUDIO_CODEC_08;//VOXWARE_CODEC_RT_8K;

	if ((err = dcmpVoxInit(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();

#ifdef	VOXWARE_KLUDGE
	mVoxwareData.lpInputLocation = (void *)first_data_chunk;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#else
	mVoxwareData.dwInputSize = len - mVoxwareData.dwInputPosition;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
#endif
	if ((err = dcmpVoxConvert(&mVoxwareData)) != VOX_NO_ERROR)
		throw DecompressException();
	mInputDataLeft = mVoxwareData.dwOutputPosition;
	mInputDataPtr = mDecompressBuffer;
}

void 
CDuckAudioSrcUnix::CleanupVoxware(void)
{
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
	dcmpVoxFree(&mVoxwareData);
}

void
CDuckAudioSrcUnix::VoxwareDecompress(const char *data,int len)
{
	mVoxwareData.wSizeOfVoxwareData = sizeof(VOXWARE_DATA);
	mVoxwareData.lpOutputLocation = mDecompressBuffer;
	mVoxwareData.lpInputLocation = (void *)data;
	mVoxwareData.dwInputSize = len;
	mVoxwareData.dwInputPosition = 0;
	mVoxwareData.dwOutputSize = MAX_OUTPUT_SIZE;
	mVoxwareData.dwOutputPosition = 0;
	if (dcmpVoxConvert(&mVoxwareData) != VOX_NO_ERROR)
		throw DecompressException();
	mInputDataLeft = mVoxwareData.dwOutputPosition;
	mInputDataPtr = mDecompressBuffer;
}

#endif


#ifdef	USE_DUCK

void
CDuckAudioSrcUnix::InitializeDuckware(const char *first_data_chunk,int len,
			DKWAVEFORM *wave,int num_channels,int samples_per_sec)
{
	assert(mDecompressBuffer);

	if (mDuckAudioDest || mDuckAudioSrc)
		CleanupDuckware();

	DXL_InitAudio(1,1);
	
	DKWAVEFORM w1;
	memcpy(&w1,wave,sizeof(DKWAVEFORM));
	if (w1.wFormatTag == NEWDK3_FORMAT_TAG)
		w1.wFormatTag = OLDDK3_FORMAT_TAG;
	mDuckAudioSrc = DXL_CreateXAudioSrc(&w1,(unsigned char *)first_data_chunk,len);
	mDuckAudioDest = DXL_CreateAudioDst(mDecompressBuffer,NULL,MAX_OUTPUT_SIZE,
		16,num_channels,samples_per_sec);

	if (!mDuckAudioSrc || !mDuckAudioDest)
		throw DecompressException();
	int gotten = DXL_dxAudio(mDuckAudioSrc,mDuckAudioDest,50000);
	mInputDataPtr = mDecompressBuffer;
	mInputDataLeft = gotten * mBlockAlign;

}

void
CDuckAudioSrcUnix::CleanupDuckware(void)
{
	if (mDuckAudioDest)
		DXL_DestroyAudioDst(mDuckAudioDest);
	mDuckAudioDest = NULL;

	if (mDuckAudioSrc)
		DXL_DestroyXAudioSrc(mDuckAudioSrc);
	mDuckAudioSrc = NULL;

	DXL_ExitAudio();
}

void
CDuckAudioSrcUnix::DuckwareDecompress(const char *data,int len)
{
	int	gotten;

	DXL_AlterAudioDst(mDuckAudioDest,(unsigned char *)mDecompressBuffer,
		NULL,0,0,0,0);
	DXL_AlterXAudioData(mDuckAudioSrc,(unsigned char *)data,len);
	//mDuckAudioDest->addrL = (unsigned char *)mDecompressBuffer;
	//mDuckAudioDest->index = 0;
	gotten = DXL_dxAudio(mDuckAudioSrc,mDuckAudioDest,50000);
	mInputDataLeft = gotten * mBlockAlign;
	mInputDataPtr = mDecompressBuffer;
}

#endif



const char *
CDuckAudioSrcUnix::GetDataPtr(void)
{
	return mInputDataPtr;
}

int			
CDuckAudioSrcUnix::GetDataLeft(void)
{
	return mInputDataLeft;
}

void		
CDuckAudioSrcUnix::DecreaseDataPtr(int howmuch)
{
	mInputDataPtr += howmuch;
	mInputDataLeft -= howmuch;
}



void
CDuckAudioSrcUnix::Feed(CDuckData &data)
{
	if (mDecompressBuffer)
		{
		// cant decompress yet!1
#ifdef	USE_DUCK
		if (mUseDuckware) 
			{
			if (mFirstAudioChunk)
				InitializeDuckware((const char *)data.GetRawData(),data.GetRawDataLength(),
					&mDuckWaveform,mDuckWaveform.nChannels,mSamplesPerSec);
			else
				DuckwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
			}
		else
#endif
#ifdef	USE_VOXWARE
		if (mUseVoxware)
			{
			if (mFirstAudioChunk)
				InitializeVoxware((const char *)data.GetRawData(),data.GetRawDataLength());
			else
				VoxwareDecompress((const char *)data.GetRawData(),data.GetRawDataLength());
			}
		else
#endif

			{
			assert(0);
			}
		}
	else
		{
		assert(!mInputDataLeft);
		mInputDataLeft = data.GetRawDataLength();
		mInputDataPtr = (const char *)data.GetRawData();

		if (mBitsPerSample == 16)
			{
			// PC and irix range are the same for 16 bit, but the
			// byte order is wrong
			unsigned char *p = (unsigned char *)mInputDataPtr;
			for (int x = 0; x < mInputDataLeft/2; ++x,p+=2)
				{
				short	s;
				unsigned char c;
				c = *p;
				*p = *(p+1);
				*(p+1) = c;

				s = *((short *)p);
				}
			}
		if (mBitsPerSample == 8)
			{
			// PC range is 0 -> 255 while irix range is -128 -> 127
			// we must subtract
			char *p = (char *)mInputDataPtr;
			for (int x = 0; x < mInputDataLeft; ++x,++p)
				{
				short	s;
				s = *((unsigned char *)p);
				s -= 128;
				*p = (char)s;
				}
			}
		}
	mFirstAudioChunk = 0;
}


int
CDuckAudioSrcUnix::ReadyForMore(void)
{
	return (mInputDataLeft == 0);
}

void 
CDuckAudioSrcUnix::Clear(void) 
{ 
	mInputDataLeft = 0;
}

int		
CDuckAudioSrcUnix::CanProduce(void)
{
	return (mInputDataLeft / mBlockAlign);
}

unsigned long 
CDuckAudioSrcUnix::Skip(unsigned long samples)
{
	int bytes_to_skip = samples * mBlockAlign;

	if (mInputDataLeft < bytes_to_skip)
		bytes_to_skip = mInputDataLeft;
	DecreaseDataPtr(bytes_to_skip);

	return (bytes_to_skip / mBlockAlign);
}

#endif
