/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef WIN32

#ifdef	_AFXDLL
#include <afxwin.h>
#else
#include <windows.h>
#endif

#endif

#include	<CDuckBasePlayer.h>
#include	<CDuckAudioSrc.h>
#include	<CDuckAudioDest.h>
#include	<CDuckAudioStream.h>
#include	<CDuckData.h>
#include	<CDuckFile.h>
#include	<CDuckBuffer.h>
#include	<CDuckImage.h>
#include	<CDuckScreen.h>
#include	<CDuckVideoStream.h>
#ifdef NETHFB
#include	<CAviCapture.h>
#endif

CDuckBasePlayer::CDuckBasePlayer(void)
{
	mStaticVideoBuffer = NULL;
	mAudioSrc		= NULL;
	mAudioDest		= NULL;
	mAudioData		= NULL;
	mAudioStream	= NULL;
	mFile			= NULL;
	mBuffer			= NULL;
	mVideoStream	= NULL;
	mVideoData		= NULL;
	mImage			= NULL;
	mScreen			= NULL;
	mRetainScreenObject = 0;

	mCapturing				= 0;
#ifdef NETHFB
	mAVICapture				= NULL;
#endif
	mFirstCaptureFrame		= -1;
	mAudioBytesProcessed	= 0;
	mFramesPerSecond		= 0;
	mFPSfloat				= 0.0;
	mStaticVideoBuffer = new unsigned char[MAX_STATIC_FRAME_SIZE];

}

void
CDuckBasePlayer::AudioChunkReleased(void)
{
	// intentionally left empty
}

void
CDuckBasePlayer::RetainScreenObject(int retain)
{
	mRetainScreenObject = retain;
}

void
CDuckBasePlayer::Cleanup(void)
{
	if (mAudioStream)
		{
		if (mAudioDest)
			mAudioDest->StopPlaying();
		if (mAudioSrc)
			mAudioSrc->Clear();
		}

	if (!mRetainScreenObject)
		{
		if (mScreen) mScreen->AsyncCleanup();
		delete mScreen;
		mScreen = NULL;
		}
	mRetainScreenObject = 0;

	try {delete mAudioDest; } catch(...){}
	mAudioDest = NULL;

	try {delete mAudioSrc; } catch(...){}
	mAudioSrc = NULL;

	try {delete mImage;} catch(...){}
	mImage = NULL;

	try {delete mVideoStream;} catch(...){}
	mVideoStream = NULL;

	try {delete mAudioStream;} catch(...){}
	mAudioStream = NULL;

	try {delete mAudioData;} catch(...){}
	mAudioData = NULL;

	try {delete mVideoData;} catch(...){}
	mVideoData = NULL;

	if (mFile)
		{
		try {
		mFile->CloseFile();
		delete mFile;
		}catch(...){}
		}
	mFile = NULL;

	try {delete mBuffer;} catch(...){}
	mBuffer = NULL;

}

CDuckBasePlayer::~CDuckBasePlayer(void)
{
	Cleanup();

#ifdef NETHFB
	delete mAVICapture;
	mAVICapture = NULL;
#endif

	delete [] mStaticVideoBuffer;
	mStaticVideoBuffer = NULL;
}

int		
CDuckBasePlayer::IsCapturing(void)
{
	return mCapturing;
}

void
CDuckBasePlayer::LoadAudio(int ignore_data)
{
	if (!mAudioStream) return;

	if (ignore_data)
		{	
		mAudioData->GetStreamingData(*mAudioStream,DUK_FORWARD,1);
		if (*mAudioData == CDuckData::Ok)
			{
			mAudioData->ReleaseStreamingData(*mBuffer);
			AudioChunkReleased();
			}
		return;
		}
	
	if (mAudioSrc->ReadyForMore())
		{
		CDuckData::GetResult	res;

		res = *mAudioData;

		if (res == CDuckData::Ok)
			{
			mAudioData->ReleaseStreamingData(*mBuffer);
			AudioChunkReleased();
			}
		if (res == CDuckData::Ok ||
			res == CDuckData::NothingYet)
			{
			mAudioData->GetStreamingData(*mAudioStream,DUK_FORWARD,1);
#ifdef NETHFB
			if (mCapturing && *mAudioData == CDuckData::Ok)
				{
				if (mFirstCaptureFrame == -1)
					CalculateFirstCaptureFrame();
				mAVICapture->AVIWriteAudio(
					mAudioData->GetRawData(),mAudioData->GetRawDataLength());
				}
#endif
			}
		if (*mAudioData == CDuckData::Ok)
			{
			mAudioBytesProcessed += mAudioData->GetRawDataLength();
			mAudioSrc->Feed(*mAudioData);
			}
		}
	if (mAudioDest->ReadyForMore())
		{
		if (*mAudioData == CDuckData::Eof)
			mAudioDest->FeedSilence(*mAudioSrc);
		else
			mAudioDest->Feed(*mAudioSrc);
		}
}


void
CDuckBasePlayer::CalculateFirstCaptureFrame(void)
{
	if (mAudioStream)
		{
		int	channels;
		int frequency;
		int avg_bytes_per_sec;
		int	format_tag;
		DKWAVEFORM	*waveform = 
			mAudioStream->GetInfo(channels,frequency,avg_bytes_per_sec,format_tag);

		switch(waveform->wBitsPerSample)
			{
			case 3:
			case 4:
				mFirstCaptureFrame = (mAudioBytesProcessed / 2048);
				// first capture has number of blocks
				mFirstCaptureFrame *= waveform->wSamplesPerBlock;
				// first capture has number of samples
				mFirstCaptureFrame = (int)((mFPSfloat * mFirstCaptureFrame) / frequency); 
				break;
			case 16:
				if (format_tag == VOXWARE_FORMAT_TAG)
					{
					mFirstCaptureFrame = (mAudioBytesProcessed / 64);
					// first capture has number of blocks
					mFirstCaptureFrame *= waveform->wSamplesPerBlock;
					// first capture has number of samples
					mFirstCaptureFrame = (int)((mFPSfloat * mFirstCaptureFrame) / frequency); 
					break;
					}
				// else fall through
			default:
				// calculate absolute samples to date
				mFirstCaptureFrame = (mAudioBytesProcessed / channels) / 
					(waveform->wBitsPerSample / 8);
				// time is mFirstCaptureFrame / frequency
				mFirstCaptureFrame = (int)((mFPSfloat * mFirstCaptureFrame) / frequency); 
				break;
			}
		}
	else
		{
		mFirstCaptureFrame = 0;
		}
}
