/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	__POWERPC__
#include	"CDuckMACDirectScreen.h"
#include	"CDuckImage.h"
#include	"CDuckCompositeBuffer.h"
#include	<assert.h>
#include	<string.h>
#include	<UGWorld.h>
#include	<UDrawingState.h>
#include	<LPane.h>
#include	<LView.h>

CDuckMACDirectScreen::CDuckMACDirectScreen(HWND wnd)
{
	mPane = reinterpret_cast<LPane*>(wnd);
	GDHandle gdh = GetMainDevice();
	PixMapHandle pm = (**gdh).gdPMap;
	switch(mPixelSize = (**pm).pixelSize)
		{
		case 8:
			mBitDepth = DXRGB8;
			mPitchBytes = 1;
			break;
		case 16:
			mBitDepth = DXRGB16;
			mPitchBytes = 2;
			break;
		case 24:
			mBitDepth = DXRGB24;
			mPitchBytes = 3;
			break;
		case 32:
			mBitDepth = DXRGB32;
			mPitchBytes = 4;
			break;
		default:
			ParamText("\pUnsupported screen depth","\p","\p","\p");
			Alert(5001,0);
			throw InitException();
			break;
		}

#if 0		
	if ((**pm).pixelSize != 16)
		{
		ParamText("\pSet screen to 16 bit to use direct drawing","\p","\p","\p");
		Alert(5001,0);
		
		throw InitException();
		}
#endif
}

CDuckMACDirectScreen::~CDuckMACDirectScreen(void)
{
}


void	
CDuckMACDirectScreen::Instantiate(int xstart,int ystart,int width,int height,int stretch)
{
	GrafPtr gpr;
	::GetPort(&gpr);
	StColorPortState state(gpr);
	mX = xstart;
	mY = ystart;
	
	mMovieWidth = width * (stretch ? 2 : 1);
	mMovieHeight = height * (stretch ? 2 : 1);
	mStretchFlag = stretch;
	
	Rect	bounds;
	bounds.top = 0; bounds.left = 0; 
	bounds.right = mMovieWidth; bounds.bottom = mMovieHeight;
	
	GDHandle gdh = GetMainDevice();
	PixMapHandle pm = (**gdh).gdPMap;
	::LockPixels(pm);
	Ptr screen = ::GetPixBaseAddr(pm);
	mPitch = (**pm).rowBytes & 0x7fff;
	mScreen = DXL_CreateVScreen((unsigned char *)screen,mBitDepth,mPitch,mMovieHeight);
	
	if (!mScreen)
		{
		::UnlockPixels(pm);
		throw VScreenException();
		}
		
	if (mStretchFlag)
		{
		Point	p = {0,0};

		Rect frame; 
		mPane->FocusDraw();
		mPane->CalcLocalFrameRect(frame);
		::LocalToGlobal(&p);
		frame.left = frame.left + ((frame.right - frame.left - mMovieWidth) / 2);
		frame.top = frame.top + ((frame.bottom - frame.top - mMovieHeight) / 2);
		frame.right = frame.left + mMovieWidth;
		frame.bottom = frame.top + mMovieHeight;
		OffsetRect(&frame,p.h,p.v);
		
		Blackout((unsigned char *)screen,frame.left,frame.top,mMovieWidth,mMovieHeight,mPixelSize,
		mPitch,mMovieWidth,mMovieHeight);
		}
	::UnlockPixels(pm);
	LView *v = dynamic_cast<LView*>(mPane);
	if (v)v->OutOfFocus(nil);
}

void	
CDuckMACDirectScreen::Blit(CDuckCompositeBuffer &cbuffer,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;
}


void	
CDuckMACDirectScreen::Blit(CDuckImage &image,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;

	GrafPtr gpr;
	::GetPort(&gpr);
	StColorPortState state(gpr);

	mPane->FocusDraw();
	
	Point	p = {0,0};
	::LocalToGlobal(&p);

	Rect frame;
	mPane->CalcLocalFrameRect(frame);
	frame.left = frame.left + ((frame.right - frame.left - mMovieWidth) / 2);
	frame.top = frame.top + ((frame.bottom - frame.top - mMovieHeight) / 2);
	frame.right = frame.left + mMovieWidth;
	frame.bottom = frame.top + mMovieHeight;
	OffsetRect(&frame,p.h,p.v);
	
	GDHandle gdh = GetMainDevice();
	PixMapHandle pm = (**gdh).gdPMap;

	assert(pm);
	::LockPixels(pm);
	Ptr screen = ::GetPixBaseAddr(pm);
	
	screen += (frame.top * mPitch);
	screen += (frame.left * mPitchBytes);
	DXL_AlterVScreen(mScreen,(unsigned char *)screen,mBitDepth,mPitch,mMovieHeight);
	DXL_dxImageToVScreen(image.Native(),mScreen);
	::UnlockPixels(pm);
	LView *v = dynamic_cast<LView*>(mPane);
	if (v)v->OutOfFocus(nil);
}

#endif


