/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	__POWERPC__
#include	"CDuckMACScreen.h"
#include	"CDuckImage.h"
#include	"CDuckCompositeBuffer.h"
#include	<assert.h>
#include	<string.h>
#include	<UGWorld.h>
#include	<UDrawingState.h>
#include	<LPane.h>
#include	<LView.h>

CDuckMACScreen::CDuckMACScreen(HWND wnd)
{
	mPane = reinterpret_cast<LPane*>(wnd);
	mGWorld = 0;
	GDHandle gdh = GetMainDevice();
	PixMapHandle pm = (**gdh).gdPMap;
	switch(mPixelSize = (**pm).pixelSize)
		{
		case 8:
			mBitDepth = DXRGB8;
			break;
		case 16:
			mBitDepth = DXRGB16;
			break;
		case 24:
			mBitDepth = DXRGB24;
			break;
		case 32:
			mBitDepth = DXRGB32;
			break;
		default:
			ParamText("\pUnsupported screen depth","\p","\p","\p");
			Alert(5001,0);
			throw InitException();
			break;
		}
}

CDuckMACScreen::~CDuckMACScreen(void)
{
	delete mGWorld;
}


void	
CDuckMACScreen::Instantiate(int xstart,int ystart,int width,int height,int stretch)
{
	mX = xstart;
	mY = ystart;
	GrafPtr gpr;
	::GetPort(&gpr);
	StColorPortState state(gpr);
	
	mMovieWidth = width * (stretch ? 2 : 1);
	mMovieHeight = height * (stretch ? 2 : 1);
	mStretchFlag = stretch;
	
	Rect	bounds;
	bounds.top = 0; bounds.left = 0; 
	bounds.right = mMovieWidth; bounds.bottom = mMovieHeight;
	
	mGWorld = new LGWorld(bounds,mPixelSize);
	GWorldPtr gw = mGWorld->GetMacGWorld();
	PixMapHandle pm = ::GetGWorldPixMap(gw);
	assert(pm);
	::LockPixels(pm);
	Ptr screen = ::GetPixBaseAddr(pm);
	
	mPitch = (**pm).rowBytes & 0x7fff;
	mScreen = DXL_CreateVScreen((unsigned char *)screen,mBitDepth,mPitch,mMovieHeight);
	
	if (!mScreen)
		{
		::UnlockPixels(pm);
		throw VScreenException();
		}
	if (mStretchFlag)
		memset(screen,0,mMovieHeight * mPitch);
		
	::UnlockPixels(pm);
}

void	
CDuckMACScreen::Blit(CDuckCompositeBuffer &cbuffer,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;
}


void	
CDuckMACScreen::Blit(CDuckImage &image,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;

	GrafPtr gpr;
	::GetPort(&gpr);
	StColorPortState state(gpr);

	mPane->FocusDraw();
	GWorldPtr gw = mGWorld->GetMacGWorld();
	PixMapHandle pm = ::GetGWorldPixMap(gw);
	assert(pm);
	::LockPixels(pm);
	Ptr screen = ::GetPixBaseAddr(pm);
	
	DXL_AlterVScreen(mScreen,(unsigned char *)screen,mBitDepth,mPitch,mMovieHeight);
	DXL_dxImageToVScreen(image.Native(),mScreen);
	::UnlockPixels(pm);
	
	Rect frame;
	mPane->CalcLocalFrameRect(frame);
	frame.left = frame.left + ((frame.right - frame.left - mMovieWidth) / 2);
	frame.top = frame.top + ((frame.bottom - frame.top - mMovieHeight) / 2);
	frame.right = frame.left + mMovieWidth;
	frame.bottom = frame.top + mMovieHeight;
	
	::ForeColor(blackColor);
	::BackColor(whiteColor);
	mGWorld->CopyImage(mPane->GetMacPort(),frame);
	LView *v = dynamic_cast<LView*>(mPane);
	if (v)v->OutOfFocus(nil);
}

#endif


