/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CDuckPlayer.h"
#ifdef	WIN32
#define	INITGUID
#include <objbase.h>
#include <initguid.h>
#endif
#include "memory.h"

#include "CGUIController.h"
//#include "CDuckAudioSrc95.h"
//#include "CDuckAudioDest95.h"
#include "CDuckAudioSrcDirect.h"
#include "CDuckAudioDestDirect.h"
#include "CDuckAudioDestSGI.h"
#include "CDuckAudioDestSUN.h"
#include "CDuckAudioSrcMAC.h"
#include "CDuckAudioDestMAC.h"
#include "CDuckAudioSrcUnix.h"
#include "CDuckVideoStream.h"
#include "CDuckAudioStream.h"
#include "CDuckData.h"
#include "CDuckImage.h"
#include "CDuckSGIScreen.h"
#include "CDuckXILScreen.h"
//#include "CDuckSGITextureScreen.h"
#include "CDuckUnixScreen.h"
#include "CDuckMACScreen.h"
#include "CDuckMACDirectScreen.h"
//#include "CDuckSGIDigitalMediaScreen.h"
//#include "CDuckSGIDMBufferScreen.h"
#include "CDuckWinblitScreen.h"
#include "CDuckDirectXScreen.h"
#include "CDuckFlipScreen.h"
#include "CDuckCompositeBuffer.h"
#include "CDuckBuffer.h"
#include "CDuckContext.h"
#include "CDuckFile.h"
#ifdef NETHFB
#include "CAviCapture.h"
#endif
#include <CallbackData.h>
#include <system_ticks.h>
#include <milli_sleep.h>
#include <assert.h>

#include <stack_aids.h>

#include <duck_mem.h>

#include <CTrueCapGUIController.h>

#include "danbug.h"


#define _WAIT 32514 // temp kludge

#define _NORMAL 32512

extern int gNT;


#ifdef SOLARIS
//#define SINGLETHREADPLAYBACK
#endif

#ifndef	WIN32
#define	WAVEFORMATEX	DKWAVEFORM
#endif

#define	VTR_PAUSE	1
#define VTR_FF		2
#define	VTR_REWIND	4


extern "C" {
	static void	CDP_DucknetCallback(int code,const void *data,const char *message,void *passback);
};

extern int			g___ReportBufferUpdates;


CDuckPlayer::CDuckPlayer(void)
{
	g___ReportBufferUpdates = 1;
	mFillBufferThread.SetPlayer(this);
	mCBuffer = NULL;
	mContext= NULL;

	mMovieWidth = 320;

	mBlockSize = 0;
	mPlaying = 0;
	mKeepPlaying = 0;
	mVTRControl = 0;

	mWarpFlag = 0;
	mWarpHeight = 0;
	mWarpWidth = 0;

	mIsPageFlipping = 0;
	mMulticasting = 0;

	mCanCapture = 0;

	mFillBufferState = Killed; //Halted
	mContext= new CDuckContext(4,8,4);
	mPlayLoopInterrupt = 0;

	mFFStartMode = 0;

	mPauseStartMode = 0;

	mInterruptable = 0;
}

void
CDuckPlayer::Cleanup(void)
{
	// ORDER HERE IS CRUCIAL!!! PARTICULARLY
	// IF THE NET LAYER IS TO SHUTDOWN CORRECTLY
	DestroyFillBufferThread();
		
	mIsPageFlipping = 0;

	try {delete mCBuffer;}catch(...){}
	mCBuffer = NULL;

	CDuckBasePlayer::Cleanup();


}

CDuckPlayer::~CDuckPlayer(void)
{
	DestroyFillBufferThread();

	try {delete mContext;}catch(...){}
	mContext = NULL;
}

int
CDuckPlayer::IsMulticasting(void)
{
	return mMulticasting;
}


void FormatTime(char *buf,float fseconds)

{

	static char *h[2] = {"hour","hours"};

	static char *m[2] = {"minute","minutes"};

	static char *s[2] = {"second","seconds"};



	int	seconds,minutes,hours;

	hours = fseconds / (60 * 60);

	fseconds -= (hours * (60 * 60));

	minutes = fseconds / 60;

	fseconds -= minutes * 60;

	seconds = fseconds;

	if (hours)

		sprintf(buf,"%d %s %d %s %d %s",hours,h[hours == 1 ? 0 : 1],minutes,m[minutes == 1 ? 0 : 1],seconds,s[seconds == 1 ? 0 : 1]);

	else if (minutes)

		sprintf(buf,"%d %s %d %s",minutes,m[minutes == 1 ? 0 : 1],seconds,s[seconds == 1 ? 0 : 1]);

	else 

		sprintf(buf,"%d %s",seconds,s[seconds == 1 ? 0 : 1]);

}




void

CDuckPlayer::ShowPositionAndTime(void)

{

	if (mIsPageFlipping) return;

	if (!mVideoStream) return;

	char buf[128];

	char buf2[128];

	float fseconds = ((float)mLastFrameDisplayed) / mFPSfloat;

	FormatTime(buf,fseconds);

	long	total_frames = mVideoStream->GetStreamLength();

	fseconds = ((float)total_frames) / mFPSfloat;

	FormatTime(buf2,fseconds);

	(mGUIController->Log(this)) << "Frame " << mLastFrameDisplayed 

		<< "/" << total_frames << "\t\t" << buf << "/" << buf2 << &(CMessageStream::show);

}



void

CDuckPlayer::ConstructScreen(void)

{

	mIsPageFlipping = 0;

	HDC	device = ::GetDC(mWindow);

	int bpp = GetDeviceCaps(device, PLANES) * GetDeviceCaps(device, BITSPIXEL);

	::ReleaseDC(mWindow,device);

	if ((bpp < 15 || bpp > 16) && (mWinblit == 0)) mWinblit = 1;

	if (mWinblit == 1)

		{

		int rescode;

		mScreen = new CDuckWinblitScreen(mWindow,rescode,0);

		if (rescode)

			InvokeCallback(NCMDUCK_CODE_BADSCREENCONFIG,

				0,NCMDUCK_STRING_BADSCREENCONFIG);

		}

	else if (mWinblit >= 2)

		{

		mIsPageFlipping = 1;

		mScreen = new CDuckFlipScreen(mWindow,mWinblit - 2,mHScreenRes,

				mVScreenRes,mVidFreq,mBPP);

		}

	else

		{

		mScreen = new CDuckDirectXScreen(mWindow,0);

		}

}





#ifdef NETHFB
float
CDuckPlayer::Go(const char* filename,HWND view,int xoffset,int yoffset,
				int stretch,int winblit,int buffersize,
				NetworkStatusCallback_t func,void *passback,
				MovieEventCallback_t func2,void *passback2,
				int loopforever,int netbuffersize,int netbufferwait,
				int videospeedtest,int directsound,int blocksize,
				int hscreenres,int vscreenres,int vid_frequency,int bitsperpixel)
#else
float
CDuckPlayer::Go(const char* filename,HWND view,int xoffset,int yoffset,
				int stretch,int winblit,int buffersize,
				void *passback,
				MovieEventCallback_t func2,void *passback2,
				int loopforever,int netbuffersize,int netbufferwait,
				int videospeedtest,int directsound,int blocksize,
				int hscreenres,int vscreenres,int vid_frequency,int bitsperpixel,

				CGUIController *gui,CColorSlider *slider,C3BitmapButton *playbackbutton)
#endif

{
int				created_screen_object;
int				lost_iterations,dropped_frames;
		mReportFramesAndIterations = 1;
		mLocked = 0;

try {
	if (!gNT)

		::SetSystemCursor(::LoadCursor(NULL,IDC_WAIT),_NORMAL);

	mMutex.Lock();

	if (mPlaying) { mMutex.Unlock(); return 0.0; }

	mPlaying = 1;

	mInterruptable = 0;

	mMutex.Unlock();

	do {

		{
		char *c1 = strchr(filename,'@');
		char *c2 = strrchr(filename,'@');
		mMulticasting = 0;
		if (c1 && c2 && (c1 != c2))
			mMulticasting = 1;
		}


	mStretch = stretch;

	mWinblit = winblit;

	mHScreenRes = hscreenres;

	mVScreenRes = vscreenres;

	mVidFreq = vid_frequency;

	mBPP = bitsperpixel;

	mGUIController = gui;
	int	video_frame_required = -1;
	int	elapsed_time;
	int audio_samples_till_start;
	int NumChannels,SamplesPerSec,BytesPerSec,Format;
	//int direct_audio = 1;
	DKWAVEFORM	*wave;
	CDuckData::GetResult	ares,vres;

	mBlockSize = blocksize;
#ifdef NETHFB
	mCallback = func;
#endif
	mCallback2 = func2;
	mPassback = passback;
	mPassback2 = passback2;
	mWindow = view;
	mXOffset = xoffset;
	mYOffset = yoffset;
	mLastFrameDecompressed = -1;
	mLastFrameDisplayed = -1;
	//mPlaying = 1;
	mKeepPlaying = 1;
	mVTRControl = 0;
	mAudioTimeVTRAccumulator = 0;
	mAudioBytesProcessed = 0;
	mVideoSpeedTest = videospeedtest;
	mDirectSound = directsound;

	mBuffer = new CDuckBuffer(buffersize * 1024,0);
	mFile	= new CDuckFile;

#ifdef NETHFB
	mFile->OpenFile2(filename,*mBuffer,CDP_DucknetCallback,this);

	if (netbuffersize)
		mBuffer->NET_SetNetBufferTime(netbuffersize);

	mBuffer->NET_SetNetPrebufferPercentWait(netbufferwait);
	mNetworkMode = mBuffer->NET_NetworkMode();
#else

	mNetworkMode = 0;

	mFile->OpenFile(filename,*mBuffer);
#endif

	created_screen_object = !mScreen;
#ifdef	WIN32
		
		if (!mScreen)
			{

			ConstructScreen();



			}
#endif	
#ifdef	IRIX
		if (!mScreen)
			mScreen = new CDuckSGIScreen(view);
#endif
#ifdef	__POWERPC__
		if (!mScreen)
			{
			if (mWinblit == 0)
				mScreen = new CDuckMACScreen(view);
			else
				mScreen = new CDuckMACDirectScreen(view);
			}
#endif
#if ((defined	SOLARIS) || (defined LINUX))
		if (!mScreen)
//			mScreen = new CDuckUnixScreen(view);
			mScreen = new CDuckXILScreen(view);
#endif
	
	
#ifdef WIN32
	milli_sleep(1000); // lets give windows a chance to breath
#endif

#ifdef SINGLETHREADPLAYBACK
	mBuffer->InitBuffer(0,-1,*mFile);	// frame, preload
#else
	if (mNetworkMode)
		{
		mBuffer->InitBuffer(0,-1,*mFile);
		}
	else
		{
		CreateFillBufferThread();

		mFillBufferStartFrame = 0;
		mFillBufferMutex.Lock();
		mFillBufferState = Start;
		mFillBufferWait.Signal();
		mFillBufferMutex.Unlock();
		while (mFillBufferState == Start) milli_sleep(10);
		}
#endif

	try 
		{
		mVideoStream = new CDuckVideoStream(*mFile,NULL,1);
		}
	catch(...)
		{
		// This is ok. Stream will throw exception if it doesn't exist
		// and the handle will still be NULL
		}
	try
		{
		mAudioStream = new CDuckAudioStream(*mFile,NULL,1);
		}
	catch(...)
		{
		// This is ok. Stream will throw exception if it doesn't exist
		// and the handle will still be NULL
		}
	
	if (mAudioStream) {
		mAudioData = new CDuckData;
		
 dprintf("just created mAudioData; mDataLen=%d",mAudioData->GetRawDataLength());
	}

	if (mVideoStream)
		{
		mVideoData = new CDuckData;


		mVideoData->GetStreamingData(*mVideoStream,DUK_FORWARD,1);
		if (*mVideoData != CDuckData::Ok)  throw 0;

		// build video
		mImage = new CDuckImage(*mVideoData);

		// build compositing buffer
		// not used in this example
		//mCBuffer = new CDuckCompositeBuffer(*mImage);

		//mCBuffer->Composite(*mImage);
		// build screen
		int x,y,width,height;
		mImage->GetDimensions(x,y,width,height);
		mMovieWidth = width;
		mMovieHeight = height;
		if (width > 320 || height > 240)

			throw ImageError("Video dimensions must be no greater than 320x240.",0);



#if 0	///	dbm

		if (mVideoStream->GetFCC() != (*(int *)"TR20") && 

			mVideoStream->GetFCC() != (*(int *)"tr20"))

			throw ImageError("File is not TrueMotion RT.",0);

#endif


		if (created_screen_object)
			mScreen->Instantiate(mXOffset,mYOffset,mMovieWidth,mMovieHeight,mStretch); 
		else
			mScreen->Reset();
		
		mScreen->SetBlitQuality(*mImage,mStretch & 0x000f);

		if (mAudioStream)
			mVideoStream->GetFrameRates(*mAudioStream,mFramesPerSecond,
				audio_samples_till_start);
		else
			mVideoStream->GetFrameRates(mFramesPerSecond,
													audio_samples_till_start);
		mFPSfloat = ((float)mFramesPerSecond.Numerator()) / 
					((float)mFramesPerSecond.Denominator());

		mScreen->NullBlit(*mImage,0);
		mLastFrameDecompressed = 0;
		mLastFrameDisplayed = 0;

		mVideoData->ReleaseStreamingData(*mBuffer);
		FillBufferSignal();
		}


	// build audio
	if (mAudioStream && (!mVideoSpeedTest))
		{
		wave = mAudioStream->GetInfo(NumChannels,SamplesPerSec,
				BytesPerSec,Format);

#ifdef	IRIX
		mAudioDest = new CDuckAudioDestSGI(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format);
		mAudioSrc = new CDuckAudioSrcUnix(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,*mAudioDest);
#endif
#ifdef	SOLARIS
		mAudioDest = new CDuckAudioDestSUN(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format);
		mAudioSrc = new CDuckAudioSrcUnix(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,*mAudioDest);
#endif
#ifdef	__POWERPC__
		mAudioDest = new CDuckAudioDestMAC(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,0,0,0);
		mAudioSrc = new CDuckAudioSrcMAC(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,*mAudioDest);
#endif
#ifdef WIN32
		if (mDirectSound)
			{
			mAudioDest = new CDuckAudioDestDirect(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,view);
			mAudioSrc = new CDuckAudioSrcDirect(wave,NumChannels,SamplesPerSec,
								BytesPerSec,Format,*mAudioDest,view);
			}
		else
			{exit(-1);
			//mAudioDest = new CDuckAudioDest95(wave,NumChannels,SamplesPerSec,
			//					BytesPerSec,Format,0,0,0);
			//mAudioSrc = new CDuckAudioSrc95(wave,NumChannels,SamplesPerSec,
			//					BytesPerSec,Format,*mAudioDest,view);
			}
#endif

		// prime the audio pump
		if (!mFFStartMode && !mPauseStartMode)

			{

			if (!mVideoSpeedTest)
				mAudioBytesProcessed += mAudioDest->Preload(*mBuffer,*mAudioSrc,*mAudioStream,*mAudioData);

	#ifdef SINGLETHREADPLAYBACK
			if (!mNetworkMode)
				mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
	#endif
			// start playing
			if (!mVideoSpeedTest)
				mAudioDest->StartPlaying();
			else
				{
				mVideoOnlyElapsedTime = 0;
				mLastVideoOnlyTime = timeGetTime();
				}
			}

		else

			{

			mLastFrameDisplayed = 0;

			mVTRControl = (mPauseStartMode ? VTR_PAUSE : VTR_FF);

			InterruptPlayback();

			RECT r;

			r.left = 0;

			r.top = 0;

			r.right = 320 + xoffset + 20;

			r.bottom = 240 + yoffset + 20;

			//r.left = xoffset;

			//r.top = yoffset;

			//r.bottom = mMovieHeight;

			//r.right = mMovieWidth;

			::InvalidateRect(view,&r,FALSE);

			}

	}

	mFFStartMode = 0;

	mPauseStartMode = 0;
	if (mAudioData)

		ares = *mAudioData;

	else
		ares = CDuckData::Eof;
	if (mVideoData)
		vres = *mVideoData;
	else
		vres = CDuckData::Eof;


	CanCaptureLock();
	mCanCapture = 1;
	CanCaptureUnlock();
	CanCaptureSignal();
	
	mVideoSpeedTestStartTime = timeGetTime();
	lost_iterations = 0;
	dropped_frames = 0;
	if (!gNT)

		::SetSystemCursor(::LoadCursor(NULL,IDC_ARROW),_NORMAL);

	if (!mLocked)

		mMutex.Lock();

	mLocked = 1;

	mInterruptable = 1;

	mInterruptWait.Signal();

	mLocked = 0;

	mMutex.Unlock();

	while (
		
		((ares == CDuckData::Ok ||
		ares == CDuckData::NothingYet) 		
		||		
		(vres == CDuckData::Ok ||
		vres == CDuckData::NothingYet))

		)
		{


		ShowPositionAndTime();

		if (!IsPaused())

			{

			if (slider && mKeepPlaying)

				slider->SetPosMessage(mLastFrameDisplayed);

			}



		mMutex.Lock();
		mLocked = 1;
		if (!mKeepPlaying)
			{
			mMutex.Unlock();
			mLocked = 0;
			break;
			}
		
		if (!mVTRControl)
			{

#ifdef SINGLETHREADPLAYBACK
			if (!mNetworkMode)
				mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
#endif
			LoadAudio(mVideoSpeedTest);

			if (mVideoSpeedTest)
				{
				if (video_frame_required <= mLastFrameDisplayed)
					video_frame_required++;
				}
			else
				{
				elapsed_time = ElapsedTime();
				video_frame_required = VideoFrameRequired(elapsed_time);
				}

			if (mVideoStream)
				{
				int ready_frame = mScreen->ReadyFrame();
				if (ready_frame >= 0 && mVideoSpeedTest)
					{
					mLastFrameDisplayed = ready_frame;
					mScreen->Flip();
					}
				else if (video_frame_required >= ready_frame && ready_frame >= 0)
					{
					mLastFrameDisplayed = ready_frame;
					mScreen->Flip();
					}

				CCThread::Yield();
				if (!mScreen->BuffersAvailable())
					lost_iterations++;

				// while here instead of if?
				if  (mScreen->BuffersAvailable())//&& !mPlayLoopInterrupt)
					{
#ifdef SINGLETHREADPLAYBACK
					if (!mNetworkMode)
						mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
#endif
					// if I put this back in, I must also remember to Unlock before breaking
					// if (!mKeepPlaying) break;
					if (!mNetworkMode)
						{
						int percent_full = mBuffer->GetPercentFull();
						if (percent_full > 100 || percent_full < 0)
							percent_full = 0;
						InvokeCallback(NCMDUCK_CODE_DISKBUFFERUPDATE,
							(void *)percent_full,
							NCMDUCK_STRING_DISKBUFFERUPDATE);
						}
					MovieEventCheck();
					mVideoData->GetStreamingData(*mVideoStream,DUK_FORWARD,1);
					LoadAudio(mVideoSpeedTest);

					if (*mVideoData == CDuckData::Ok)
						{

						mLastFrameDecompressed++;

#ifdef NETHFB
						if (mCapturing && 
							mLastFrameDecompressed >= mFirstCaptureFrame 
							&& mFirstCaptureFrame >= 0)

							mAVICapture->AVIWriteVideo(
								mVideoData->GetRawData(),
								mVideoData->GetRawDataLength(),
								mImage->GetType() == DXL_INTRAFRAME ? 1 : 0);
#endif

						if (mImage->Update(*mVideoData) >= 0)
							{
							if (video_frame_required <= mLastFrameDecompressed) 
								mScreen->Blit(*mImage,mLastFrameDecompressed,mWarpFlag,mWarpHeight,mWarpWidth);
							else 
								{
								mScreen->NullBlit(*mImage,mLastFrameDecompressed);
								dropped_frames++;
								}
							}

						else

							{

#ifdef WIN32

							TRACE("Image error\n");

#endif

							}
						mVideoData->ReleaseStreamingData(*mBuffer);
#ifdef SINGLETHREADPLAYBACK
						if (!mNetworkMode)
							mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
#else
						FillBufferSignal();
#endif
						}
					else
						{
#ifdef SINGLETHREADPLAYBACK
						if (!mNetworkMode)
							mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
#endif
						TRACE("Duck data error %d\n",(int)(*mVideoData));

						}
					//////////////////
					LoadAudio(mVideoSpeedTest);
					CCThread::Yield();
					/////////////////
					} // while BuffersAvailable()
					VideoSleepTest();
				} // if videoStream
			} // if !vtrcontrol
		else if (mVTRControl & VTR_FF)
			{
			DoFastForward();
			// so we recover correctly when theres no audio
			video_frame_required = mLastFrameDisplayed;
			}
		else if (mVTRControl & VTR_REWIND)
			{
			DoRewind();
			// so we recover correctly when theres no audio
			video_frame_required = mLastFrameDisplayed;
			}
		else if (mVTRControl & VTR_PAUSE)
			{
			mMutex.Unlock();

			milli_sleep(1000);

			mMutex.Lock();
			}

		mMutex.Unlock();

		mLocked = 0;
		if (mAudioData)
			ares = *mAudioData;
		else
			ares = CDuckData::Eof;
		if (mVideoData)
			vres = *mVideoData;
		else
			vres = CDuckData::Eof;

		if (ares == CDuckData::Eof && vres == CDuckData::Eof)

			{

			Pause(1);

			SetAndDisplayFrame(0); // back into pause mode

			ares = CDuckData::Ok;

			vres = CDuckData::Ok;

			if (mAudioData) mAudioData->Reset();

			if (mVideoData) mVideoData->Reset();

			if (slider) slider->SetPosMessage(0);

			if (playbackbutton) playbackbutton->UseAltSet(0);

			ShowPositionAndTime();

			if (mIsPageFlipping)

				((CTrueCapGUIController *)mGUIController)->CleanupAfterStop();

			}
		} // while
	if (!mLocked)

		{

		mMutex.Lock();

		mInterruptable = 0;

		mMutex.Unlock();

		}

	else

		{

		mInterruptable = 0;

		}

	if (!mVideoStream)
		milli_sleep(3000); // let audio drain
	CanCaptureLock();
	mCanCapture = 0;
	CanCaptureUnlock();
	CanCaptureSignal();
	StopCapture();
	RetainScreenObject(loopforever && mKeepPlaying && mWinblit>= 2);
	Cleanup();
	}
	while (loopforever && mKeepPlaying);

	}

// ********** Error Handling ************* //
#ifdef WIN32

catch(ImageError &err)

	{

	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,err.data());

	}
catch(CDuckDirectXScreen::DDException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"DirectDraw error.");
	}
catch(CDuckDirectXScreen::UnlockException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"DirectDraw Unlock error.");
	}
catch(CDuckDirectXScreen::UninitializedException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"Uninitialized DirectDraw error.");
	}
catch(CDuckFlipScreen::DDException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"DirectDraw error.");
	}
catch(CDuckFlipScreen::UnlockException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"DirectDraw Unlock error.");
	}
catch(CDuckFlipScreen::UninitializedException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"Uninitialized DirectDraw error.");
	}
catch(CDuckFlipScreen::BackbufferMismatch &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"Backbuffer Mismatch.");
	}
#endif
catch(CDuckAudioSrc::InitException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICAUDIOERROR,
		"Audio source init  error");
	}
catch(CDuckAudioDest::InitException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICAUDIOERROR,
		"Audio dest init  error");
	}
catch(CDuckCompositeBuffer::InitException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,
		"Composite initialization error");
	}
catch(CDuckCompositeBuffer::CompositeException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,
		"Compositing Error");
	}
catch(CDuckImage::InitException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,
		"Image initialization error");
	}
catch(CDuckStream::InitException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,
		"Stream initialization error");
	}
catch(CDuckAudioSrc::AudioFormatException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_BADAUDIOFORMAT,
		NCMDUCK_STRING_BADAUDIOFORMAT);
	}
catch(CDuckAudioDest::AudioFormatException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_BADAUDIOFORMAT,
		NCMDUCK_STRING_BADAUDIOFORMAT);
	}
#ifdef SOLARIS
catch(CDuckAudioDestSUN::IOException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_GENERICAUDIOERROR,
		NCMDUCK_STRING_GENERICAUDIOERROR);
	}
#endif
catch(CDuckAudioSrc::DecompressException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_AUDIODECOMPRESSERROR,
		NCMDUCK_STRING_AUDIODECOMPRESSERROR);
	}
catch(CDuckScreen::VScreenException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FATALGRAPHICSERROR,
		"VScreen init failure");
	}
catch(CDuckFile::OpenException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FILENOTFOUND,
		NCMDUCK_STRING_FILENOTFOUND);
	}
catch(CDuckFile::FindException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FILENOTFOUND,
		NCMDUCK_STRING_FILENOTFOUND);
	}
catch(CDuckFile::ParseException &)
	{
	BasicErrorHandler(NCMDUCK_CODE_FILEFORMATBAD,
		NCMDUCK_STRING_FILEFORMATBAD);
	}
catch(...)
	{
	dprintf("undefined error here -- hmm...");
	BasicErrorHandler(NCMDUCK_CODE_GENERICERROR,
		"Unknown error 0x001");
	}
// ***************************** //

	mMutex.Lock();
	mPlaying = 0;

	mInterruptable = 1;

	mInterruptWait.Signal();
	CanCaptureSignal();

	mVTRControl = 0;
	mMutex.Unlock();

	if (mVideoSpeedTest)
		{
		mVideoSpeedTestStartTime = timeGetTime() - mVideoSpeedTestStartTime;
		float fps = ((float)(mLastFrameDisplayed * 1000)) / mVideoSpeedTestStartTime;
#ifndef	IRIX
		char buf[256];
		sprintf(buf,"%f fps",fps);
		InvokeCallback(NCMDUCK_CODE_MISC,0,buf);
#endif
		return fps;
		}
	else if (mReportFramesAndIterations)
		{
		char buf[256];
		sprintf(buf,"%d lost iterations %d dropped frames",lost_iterations,dropped_frames);
		InvokeCallback(NCMDUCK_CODE_MISC,0,buf);
		}
	::InvalidateRect(view,NULL,FALSE);

	(mGUIController->Log(this)) << filename << &(CMessageStream::show);

	if (!gNT)

		::SetSystemCursor(::LoadCursor(NULL,IDC_ARROW),_NORMAL);

	mMovieWidth = 320;

	return 0.0;
}

void
CDuckPlayer::BasicErrorHandler(int code,const char *message)
{

	if (!gNT)

		::SetSystemCursor(::LoadCursor(NULL,IDC_ARROW),_NORMAL);
	if (mLocked) mMutex.Unlock();
	mLocked = 0;
	mReportFramesAndIterations = 0;
	InvokeCallback(code,0,message);
	::MessageBox(mWindow,message,"Playback Error",MB_OK | MB_ICONERROR);

	Cleanup();
}

int
CDuckPlayer::ElapsedTime(void)
{
	int elapsed_time = 0;

	if (mAudioDest)
		{
		elapsed_time = mAudioTimeVTRAccumulator + 
								mAudioDest->ElapsedAudioTime();
		}
	else
		{
		int	tmp = timeGetTime();
		mVideoOnlyElapsedTime += tmp - mLastVideoOnlyTime;
		mLastVideoOnlyTime = tmp;
		elapsed_time = mVideoOnlyElapsedTime;
		}
	return elapsed_time;
}

int
CDuckPlayer::VideoFrameRequired(int elapsed_time)
{
	return (int)(((elapsed_time * mFPSfloat) / 1000));
}

int
CDuckPlayer::FrameTime(int frame)
{
	return (int)( (((float)frame) * 1000.0) / mFPSfloat);
}


int

CDuckPlayer::GetLastFrameDisplayed(void)

{

	stack_MutexLock sml(&mMutex);

	return mLastFrameDisplayed;

}



void
CDuckPlayer::VideoSleepTest(void)
{
	if (!mVideoSpeedTest && !mScreen->BuffersAvailable())
		{
		int e_time = ElapsedTime();
		int rf = mScreen->ReadyFrame();
		int f = rf - VideoFrameRequired(e_time);
		if (f > 0)
			{
			int ft = FrameTime(rf);
			int sleep_time = (ft - e_time);
			if (sleep_time > 50) sleep_time = 50;
			if (sleep_time >= 15) // arbitrary
				{
				milli_sleep(sleep_time/2);
				}
			}
		}
}

void
CDuckPlayer::AudioChunkReleased(void)
{
	FillBufferSignal();
}

int
CDuckPlayer::IsPlaying(void)
{
	return mPlaying;
}




void

CDuckPlayer::AdvanceFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;

	mLastFrameDisplayed++;

	mLastFrameDecompressed = mLastFrameDisplayed;

	long	max_len = MAX_STATIC_FRAME_SIZE;



	if (!mVideoStream) return;



	//if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_FORWARD,0) >= 0)

	if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_ABSOLUTE,mLastFrameDisplayed) >= 0)

		{

		mImage->Update(mStaticVideoBuffer);

		//mCBuffer->Composite(*mImage);

		mScreen->BlitNow(*mImage,mLastFrameDisplayed,mWarpFlag,mWarpHeight,mWarpWidth);

		}

}



void

CDuckPlayer::AdvanceTime(int milliseconds)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;

	int frames = (int)((mFPSfloat * milliseconds) / 1000);



	long	total_frames = mVideoStream->GetStreamLength();



	if (mLastFrameDisplayed + frames >= total_frames)

		mLastFrameDisplayed = total_frames - 1;

	else

		mLastFrameDisplayed += frames;



	mLastFrameDecompressed = mLastFrameDisplayed;

	long	max_len = MAX_STATIC_FRAME_SIZE;



	if (!mVideoStream) return;



	//if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_FORWARD,0) >= 0)

	if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_ABSOLUTE,mLastFrameDisplayed) >= 0)

		{

		mImage->Update(mStaticVideoBuffer);

		//mCBuffer->Composite(*mImage);

		mScreen->BlitNow(*mImage,mLastFrameDisplayed,mWarpFlag,mWarpHeight,mWarpWidth);

		}

}



void

CDuckPlayer::RetreatTime(int milliseconds)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;



	int frames = (int)((mFPSfloat * milliseconds) / 1000);



	if (frames > mLastFrameDisplayed)

		frames = mLastFrameDisplayed;

	mLastFrameDisplayed -= frames;

	mLastFrameDecompressed = mLastFrameDisplayed;

	long	max_len = MAX_STATIC_FRAME_SIZE;



	if (!mVideoStream) return;

	//if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_BACKWARD,1) >= 0)

	if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_ABSOLUTE,mLastFrameDisplayed) >= 0)

		{

		mImage->Update(mStaticVideoBuffer);

		//mCBuffer->Composite(*mImage);

		mScreen->BlitNow(*mImage,mLastFrameDisplayed,mWarpFlag,mWarpHeight,mWarpWidth);

		}

}



void

CDuckPlayer::RetreatFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;

	mLastFrameDisplayed--;

	mLastFrameDecompressed = mLastFrameDisplayed;

	long	max_len = MAX_STATIC_FRAME_SIZE;



	if (!mVideoStream) return;

	//if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_BACKWARD,1) >= 0)

	if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_ABSOLUTE,mLastFrameDisplayed) >= 0)

		{

		mImage->Update(mStaticVideoBuffer);

		//mCBuffer->Composite(*mImage);

		mScreen->BlitNow(*mImage,mLastFrameDisplayed,mWarpFlag,mWarpHeight,mWarpWidth);

		}

}



void

CDuckPlayer::DisplayCurrentFrame(void)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;

	while (!mInterruptable)

		mInterruptWait.Wait(mMutex);

	mLastFrameDecompressed = mLastFrameDisplayed;

	PutFrameOnScreen(mLastFrameDecompressed);

}



void

CDuckPlayer::SetAndDisplayFrame(int f)

{

	stack_MutexLock sml(&mMutex);

	if (!IsPaused()) return;

	while (!mInterruptable)

		mInterruptWait.Wait(mMutex);

	mLastFrameDecompressed = mLastFrameDisplayed = f;

	PutFrameOnScreen(mLastFrameDecompressed);

	ShowPositionAndTime();

}






void
CDuckPlayer::StopPlaying(void)
{
	mKeepPlaying = 0;
	mPlayLoopInterrupt = 1;
	mMutex.Lock();
	while (mPlaying) 
		{
		mPlayLoopInterrupt = 0;
		mMutex.Unlock();
		milli_sleep(100);
		mPlayLoopInterrupt = 1;
		mMutex.Lock();
		}
	
	mPlayLoopInterrupt = 0;
	mMutex.Unlock();
}

void
CDuckPlayer::InterruptPlayback(void)
{

#ifndef SINGLETHREADPLAYBACK
	if (!mNetworkMode)
		{
		mFillBufferMutex.Lock();
		mFillBufferState = Stop;
		mFillBufferWait.Signal();
		mFillBufferMutex.Unlock();
		while (mFillBufferState != Halted) CCThread::Yield();//Sleep(0);
		}
#endif

	if (mAudioStream && !mVideoSpeedTest)
		{
		mAudioDest->StopPlaying();
		mAudioSrc->Clear();
		}
	
	if (mVideoStream)
		{
		if (mScreen) mScreen->VTRReset();

		if (*mVideoData == CDuckData::Ok)
			mVideoData->ReleaseStreamingData(*mBuffer);

		long			single_len = MAX_STATIC_FRAME_SIZE;


		mLastFrameDecompressed = mLastFrameDisplayed;

		//while (mLastFrameDecompressed > mLastFrameDisplayed)

		//	{

		//	mVideoStream->ReadData(mStaticVideoBuffer,single_len,DUK_BACKWARD,1);

		//	mLastFrameDecompressed--;

		//	}

		

		// we need to do this next step to halt the server under DuckNet.
		// It's bogus, I know, but a real server halt would require extending
		// the API which we aren't doing write now. If you're not using
		// DuckNet this next statement will have no meaningful effect


		//if (mNetworkMode)

		//	mVideoStream->ReadData(mStaticVideoBuffer,single_len,DUK_ABSOLUTE,mLastFrameDisplayed);
		}
}

int
CDuckPlayer::RestorePlayback(int frame)
{
 dprintf("CDuckPlayer::RestorePlayback");
	if (mVideoStream)
		{
 dprintf("  RestorePlayback A");
		if (!mVideoStream->Seek(frame,*mImage))
			{
 dprintf("  RestorePlayback B");
			frame = 0;

			if (!mVideoStream->Seek(frame,*mImage))

				{

 dprintf("  RestorePlayback C");
				mKeepPlaying = 0;

				return frame;

				}

//			mKeepPlaying = 0;
//			return;
			}
		}
 dprintf("  RestorePlayback D");
	try {

		if (!gNT)

			::SetSystemCursor(::LoadCursor(NULL,IDC_WAIT),_NORMAL);


#ifdef SINGLETHREADPLAYBACK
	mBuffer->InitBuffer(frame+1,-1,*mFile);
 dprintf("  RestorePlayback E");
#else
	if (mNetworkMode)
		{
 dprintf("  RestorePlayback F");
		mBuffer->InitBuffer(frame+1,-1,*mFile);
		}
	else
		{
 dprintf("  RestorePlayback G");
		mFillBufferStartFrame = frame+1;
		mFillBufferMutex.Lock();
		mFillBufferState = Start;
		mFillBufferWait.Signal();
		mFillBufferMutex.Unlock();
 dprintf("  RestorePlayback G2");
		while (mFillBufferState == Start) milli_sleep(10);
		}
#endif
			
 dprintf("  RestorePlayback G2a");
	// reset the streams
	if (mVideoStream)
		{
 dprintf("  RestorePlayback G2b");
		if (mScreen) mScreen->VTRReset();
 dprintf("  RestorePlayback G3");
		mVideoStream->Reset();
 dprintf("  RestorePlayback G4");
		mVideoData->Reset();
 dprintf("  RestorePlayback G5");
		}
	if (mAudioStream && !mVideoSpeedTest)
		{
 dprintf("  RestorePlayback H");
		mAudioStream->Reset();
 dprintf("  RestorePlayback I");
		mAudioData->Reset();
 dprintf("  RestorePlayback J");

		// load up the audio
		//mAudioDest->Preload(*mAudioSrc,*mAudioStream,*mAudioData);
		mAudioBytesProcessed = 0;
		mFirstCaptureFrame = -1;
 dprintf("about to call Resync -- mAudioData->mDataLen=%d",mAudioData->GetRawDataLength());
		mAudioBytesProcessed = 
			mAudioDest->Resync(*mBuffer,*mAudioSrc,*mAudioStream,*mAudioData,
								frame+1,mFPSfloat);

#ifdef SINGLETHREADPLAYBACK
		if (!mNetworkMode)
			mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
#endif
		mAudioTimeVTRAccumulator = (long)( ((float)((frame+1) * 1000))
			/ mFPSfloat);
		//if (!mVideoSpeedTest)
		mAudioDest->StartPlaying();
		if (!gNT)

			::SetSystemCursor(::LoadCursor(NULL,IDC_ARROW),_NORMAL);

		}

	}	
	catch(...)

		{

		if (!gNT)

			::SetSystemCursor(::LoadCursor(NULL,IDC_ARROW),_NORMAL);

		throw;

		}
	return frame;

}


int		
CDuckPlayer::IsPaused(void)
{
	 return (mVTRControl & VTR_PAUSE ? 1 : 0);
}

void	
CDuckPlayer::Pause(int p)
{
	if (!mVideoStream) return;
	mPlayLoopInterrupt = 1;
	mMutex.Lock();
	if (mPlaying)
		{
		if (p)
			{
			//if (mVTRControl == 0)
			//	{
				// only pause if nothing is in effect
				mVTRControl = VTR_PAUSE;
				InterruptPlayback();
				//}
			}
		else
			{
			if (mVTRControl & VTR_PAUSE)
				{
				mVTRControl &= (~VTR_PAUSE);
				mLastFrameDecompressed = mLastFrameDisplayed;
				mLastFrameDecompressed = mLastFrameDisplayed = RestorePlayback(mLastFrameDisplayed);
				}
			}
		}
	mPlayLoopInterrupt = 0;
	mMutex.Unlock();
}

void	
CDuckPlayer::FastForward(int f)
{
	if (!mVideoStream) return;
	mPlayLoopInterrupt = 1;
	mMutex.Lock();

	if (mPlaying)
		{
		if (f)
			{
			//if (mVTRControl == 0)
			//	{
				// only ff if nothing is in effect
				mVTRControl = VTR_FF;
				InterruptPlayback();
			//	}
			}
		else
			{
			if (mVTRControl & VTR_FF)
				{
				mVTRControl &= (~VTR_FF);
				mLastFrameDecompressed = mLastFrameDisplayed;
				mLastFrameDecompressed = mLastFrameDisplayed = RestorePlayback(mLastFrameDisplayed);

				}
			}
		}
	mPlayLoopInterrupt = 0;
	mMutex.Unlock();
}

int		
CDuckPlayer::IsFastForward(void)
{
	return (mVTRControl & VTR_FF ? 1 : 0);
}


void	
CDuckPlayer::Rewind(int r)
{
	if (!mVideoStream) return;
	mPlayLoopInterrupt = 1;
	mMutex.Lock();

	if (mPlaying)
		{
		if (r)
			{
			//if (mVTRControl == 0)
			//	{
				// only ff if nothing is in effect
				mVTRControl = VTR_REWIND;
				InterruptPlayback();
			//	}
			}
		else
			{
			if (mVTRControl & VTR_REWIND)
				{
				mVTRControl &= (~VTR_REWIND);
				mLastFrameDecompressed = mLastFrameDisplayed;
				mLastFrameDecompressed = mLastFrameDisplayed = RestorePlayback(mLastFrameDisplayed);
				}

			}
		}
	mPlayLoopInterrupt = 0;
	mMutex.Unlock();
}

int		
CDuckPlayer::IsRewind(void)
{
	return (mVTRControl & VTR_REWIND ? 1 : 0);
}

void
CDuckPlayer::PutFrameOnScreen(int frame)
{
	long	max_len = MAX_STATIC_FRAME_SIZE;

	if (!mVideoStream) return;

	if (mVideoStream->ReadData(mStaticVideoBuffer,max_len,DUK_ABSOLUTE,frame) >= 0)
		{
		mImage->Update(mStaticVideoBuffer);
		//mCBuffer->Composite(*mImage);
		mScreen->BlitNow(*mImage,frame,mWarpFlag,mWarpHeight,mWarpWidth);
		}
}

void
CDuckPlayer::DoFastForward(void)
{
	int		x;
	int		ftn;
	int		last;

	if (!mVideoStream) return;
	last = mLastFrameDisplayed;

	for (x = 0; (x < (mFPSfloat/3)) && (mLastFrameDisplayed >= 0); ++x)
		{
		ftn = mVideoStream->FramesToNextKeyFrame(mLastFrameDisplayed);
		if (ftn > 0)
			mLastFrameDisplayed += ftn;
		}
	// this is a server, not a hard disk, so don't razz it
	if ((mLastFrameDisplayed >= 0) && (last != mLastFrameDisplayed))
		{
		PutFrameOnScreen(mLastFrameDisplayed);
		}

	else

		{

		CCThread::Yield();

		}
	mLastFrameDecompressed = mLastFrameDisplayed;
}

void
CDuckPlayer::DoRewind(void)
{
	int		x;
	int	prev = 0;
	int last;

	if (!mVideoStream) return;
	last = mLastFrameDisplayed;

	for (x = 0; (x < (mFPSfloat/3)) && (prev >= 0); ++x)
		{
		// -1 because of HFB bug with key frames. We just keep
		// getting ourself instead of the PREVIOUS ONE
		prev = mVideoStream->PreviousKeyFrame(mLastFrameDisplayed-1);
		if (prev >= 0)
			mLastFrameDisplayed = prev;
		}
	if (mLastFrameDisplayed >= 0 && (last != mLastFrameDisplayed))
		{
		PutFrameOnScreen(mLastFrameDisplayed);
		}

	else

		{

		CCThread::Yield();

		}
	mLastFrameDecompressed = mLastFrameDisplayed;
}

void
CDuckPlayer::ClearWarp(void)
{
	mMutex.Lock();
	mWarpFlag = 0;
	mMutex.Unlock();
}

void
CDuckPlayer::SetWarp(int height,int width)
{
	mMutex.Lock();

	mWarpFlag = 1;
	mWarpHeight = height;
	mWarpWidth = width;

	mMutex.Unlock();
}

int
CDuckPlayer::IsPageFlipping(void)
{
	return mIsPageFlipping;
}

void
CDuckPlayer::InvokeCallback(int code,const void *data,const char *message)
{
#ifdef NETHFB
		if (mCallback)
			(*mCallback)(code,data,message,mPassback);
#endif
}

void	
CDP_DucknetCallback(int code,const void *data,const char *message,void *passback)
{
	CDuckPlayer	*obj;

	obj = (CDuckPlayer *)passback;
	
	obj->InvokeCallback(code,data,message);
}

void
CDuckPlayer::MovieEventCheck(void)
{
#ifdef NETHFB
	if (!mBuffer->NET_NetworkMode())
		return;

	if (mLastFrameDisplayed == -1)
		return;

	int	event_frame;
	while (((event_frame = mBuffer->NET_NextMovieEventFrame()) < mLastFrameDisplayed)
		&& (event_frame != -1))
		mBuffer->NET_AdvanceMovieEvent();

	if (mLastFrameDisplayed == event_frame)
		{
		if (mCallback2)
			(*mCallback2)(event_frame,mBuffer->NET_NextMovieEventData1(),
				mBuffer->NET_NextMovieEventData2(),mPassback2);
		mBuffer->NET_AdvanceMovieEvent();
		}
#else
	return;
#endif
}


void
CDuckPlayer::CanCaptureLock(void)
{
	mCanCaptureMutex.Lock();
}

void
CDuckPlayer::CanCaptureUnlock(void)
{
	mCanCaptureMutex.Unlock();
}

void
CDuckPlayer::CanCaptureWait(void)
{
	mCanCaptureCondition.Wait(mCanCaptureMutex);
}

void
CDuckPlayer::CanCaptureSignal(void)
{
	mCanCaptureCondition.Signal();
}

void	
CDuckPlayer::StartCapture(const char *filename)
{
 dprintf("well it gets called, but it's def'd out!");
#ifdef NETHFB
	// GetImageType() doesn't work for TM2. We could look in the index

	// but I currently don't have time to implement this. Besides, in 

	// the next version TM2.1 this should be fixed
	if (!mVideoStream) return;

	// however, let's do it anyhow since inter-frame files never
	// really capture correctly
//	unsigned long tfcc = mVideoStream->GetFCC();
//	if ((tfcc == *(long *)(char *)"TM20") ||
//		(tfcc == *(long *)(char *)"tm20")) return;

	int use_video,use_audio;
	DWORD	vid_fourcc;
	nmr_fraction<I32>		framerate;
	int		width;
	int		height;
	int		format_tag;
	int		frequency;
	int		channels;
	int		avg_bytes_per_sec;
	WAVEFORMATEX	*wf = NULL;
	int		got_good_frame = 0;

	do 
		{
		CanCaptureLock();
		while (!mCanCapture && mPlaying) CanCaptureWait();
		if (!mPlaying)
			{
			CanCaptureUnlock();
			mMutex.Unlock();
			return;
			}
		if (mAVICapture)
			StopCapture();

		mMutex.Lock();
		use_video = mVideoStream ? 1 : 0;
		use_audio = mAudioStream ? 1 : 0;
		got_good_frame = 1;
		if (use_video)
			{
			int x,y;
			mImage->GetDimensions(x,y,width,height);
			if (width != mMovieWidth || height != mMovieHeight)
				{
				got_good_frame = 0;
				CanCaptureUnlock();
				mMutex.Unlock();
				milli_sleep(100);	// wait a bit then try again
				}
			}
		}
	while (!got_good_frame);
	
	wf = NULL;

	if (use_video)
		{
		framerate = mFramesPerSecond;
		vid_fourcc = mVideoStream->GetFCC();
		mFirstCaptureFrame = -1;

		}
	if (use_audio)
		{
		DKWAVEFORM	*waveform = 
			mAudioStream->GetInfo(channels,frequency,avg_bytes_per_sec,format_tag);

		wf = (WAVEFORMATEX *)duck_malloc(sizeof(DKWAVEFORM),DMEM_GENERAL);
		memcpy(wf,waveform,sizeof(DKWAVEFORM));
		}

 dprintf("initializing capture..");
	mAVICapture = new CAviCapture(use_video,vid_fourcc,framerate,width,height,
				use_audio,wf);
	
	
	
	
	mAVICapture->OpenCaptureFile(filename);
	mCapturing = 1;
	CanCaptureUnlock();
	mMutex.Unlock();
 dprintf("capture complete");
#endif
}

void	
CDuckPlayer::StopCapture(void)
{
#ifdef NETHFB
	if (mAVICapture)
		{
		mAVICapture->CloseCaptureFile();
		delete mAVICapture;
		}
	mAVICapture = NULL;
	mCapturing = 0;
#endif
}

void
CDuckPlayer::FillBufferSignal(void)
{
	// only for use in normal playing mode 
	// we TryLock because we don't want to hang around if the

	// i/o thread has the lock and is reading
	if (mFillBufferMutex.TryLock())
		{
		if (mBuffer->GetPercentFull() <= 75 && !mFillBufferEOF)
			mFillBufferWait.Signal();
		mFillBufferMutex.Unlock();
		}
}

void
CDuckPlayer::DoFillBufferThread(void)
{
#ifdef SINGLETHREADPLAYBACK
	assert(0);
#endif
 dprintf("  DoFillBufferThread");
	if (mNetworkMode) {assert(0);}

	int err;
	while (mFillBufferState != Exit)
		{
		switch(mFillBufferState)
			{
			case Stop:
				mFillBufferState = Halted;
				break;
			case Start:
 dprintf("    DoFillBufferThread A");
				mFillBufferMutex.Lock();
				mFillBufferEOF = 0;
 dprintf("    DoFillBufferThread B");
				mBuffer->InitBuffer(mFillBufferStartFrame,-1,*mFile);	// frame, preload
 dprintf("    DoFillBufferThread C");
				mFillBufferMutex.Unlock();
				mFillBufferState = Running;
				break;
			case Exit:
				break;
			case Running:
				mFillBufferMutex.Lock();
				while ((mFillBufferState == Running) && (mBuffer->GetPercentFull() >= 75))
					mFillBufferWait.Wait(mFillBufferMutex);
				if (mFillBufferState == Running)
					{
					err = mBuffer->FillBuffer(mBlockSize,HFB_IGNORE_COUNT);
					if (err <= 0)
						{
						mFillBufferEOF = 1;
						mFillBufferState = Stop;
						}
					}
				mFillBufferMutex.Unlock();
				CCThread::Yield();//Sleep(0);
				break;
			case Halted:
				milli_sleep(250);//Sleep(250);
				break;
			case Killed:
				assert(0);
				break;
			}
	if (!mNetworkMode)
		{
		int percent_full = mBuffer->GetPercentFull();
		// at ends of file we get ridiculous numbers
		if (percent_full < 0 || percent_full > 100) percent_full = 0;
		InvokeCallback(NCMDUCK_CODE_DISKBUFFERUPDATE,
				(void *)percent_full,NCMDUCK_STRING_DISKBUFFERUPDATE);
		}
	}

	mFillBufferState = Killed;
}


void
CDuckPlayer::CFillBufferThread::Run(void)
{
#ifdef SINGLETHREADPLAYBACK
	assert(0);
#endif
	mPlayer->DoFillBufferThread();
}

void	
CDuckPlayer::CreateFillBufferThread(void)
{
#ifdef SINGLETHREADPLAYBACK
	assert(0);
#endif
	if (mNetworkMode) {assert(0);}
	if (mFillBufferState != Killed)
		DestroyFillBufferThread();
	mFillBufferState = Stop;
	mFillBufferThread.Start();

}

void	
CDuckPlayer::DestroyFillBufferThread(void)
{
	if (mFillBufferState != Killed)
		{
#ifdef SINGLETHREADPLAYBACK
		assert(0);
#endif
		if (mNetworkMode) {assert(0);}
		mFillBufferMutex.Lock();
		mFillBufferState = Exit;
		mFillBufferWait.Signal();
		mFillBufferMutex.Unlock();
		mFillBufferThread.Join();
		//while (mFillBufferState != Killed) milli_sleep(100);
		}

}

void

CDuckPlayer::StartInFFMode(void)

{

	mFFStartMode = 1;

}



void

CDuckPlayer::StartInPauseMode(void)

{

	mPauseStartMode = 1;

}



int

CDuckPlayer::BigImage(void)

{

	return (mMovieWidth > 160 ? 1 : 0);

}


void

CDuckPlayer::GoFullScreen(int f)

{

	stack_MutexLock sml(&mMutex);



	if (!f && !mIsPageFlipping) return;

	if (f && mIsPageFlipping) return;



	delete mAudioDest;

	delete mAudioSrc;

	mAudioDest = 0;

	mAudioSrc = 0;

	if (mScreen) mScreen->AsyncCleanup();

	delete mScreen;

	mScreen = NULL;



	mWinblit = (f ? 4 : 0);

	mStretch = (f ? 1 : 0);

	ConstructScreen();

	mScreen->Instantiate(mXOffset,mYOffset,mMovieWidth,mMovieHeight,mStretch); 

	mScreen->SetBlitQuality(*mImage,mStretch & 0x000f);

	mScreen->NullBlit(*mImage,0);



	if (mAudioStream && (!mVideoSpeedTest))

		{

		int NumChannels,SamplesPerSec,BytesPerSec,Format;

		DKWAVEFORM	*wave;

		wave = mAudioStream->GetInfo(NumChannels,SamplesPerSec,

				BytesPerSec,Format);

#ifdef WIN32

		if (mDirectSound)

			{

			mAudioDest = new CDuckAudioDestDirect(wave,NumChannels,SamplesPerSec,

								BytesPerSec,Format,mWindow);

			mAudioSrc = new CDuckAudioSrcDirect(wave,NumChannels,SamplesPerSec,

								BytesPerSec,Format,*mAudioDest,mWindow);

			}

		else

			{
			exit(-1);
			//mAudioDest = new CDuckAudioDest95(wave,NumChannels,SamplesPerSec,

			//					BytesPerSec,Format,0,0,0);

			//mAudioSrc = new CDuckAudioSrc95(wave,NumChannels,SamplesPerSec,

			//					BytesPerSec,Format,*mAudioDest,mWindow);

			}

#endif

	}

}

