/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CDUCKPLAYER_H_
#define CDUCKPLAYER_H_

#ifdef	WIN32
#ifdef	_AFXDLL
#define VC_EXTRALEAN
#include <afxwin.h>
#else
#include <windows.h>
#endif
#endif


#include "CColorSlider.h"

#include "C3BitmapButton.h"


#include <MegaHeaders.h>
#include <HWNDDefinition.h>


#include <duck_hfb.h>
#ifdef NETHFB
#include <network.h>
#endif

#include <CDuckBasePlayer.h>
#include <CCMutex.h>
#include <CConditionVariable.h>
#include <CCThread.h>

class CDuckContext;
class CDuckCompositeBuffer;
class CGUIController;


typedef void (*MovieEventCallback_t)(int frame,const char *m1,const char *m2,void *passback);


class CDuckPlayer: public CDuckBasePlayer
{
protected:
	int	mFFStartMode,mPauseStartMode;



	class CFillBufferThread: public CCThread
		{
		protected:
			CDuckPlayer	*mPlayer;
			virtual void Run(void);
		public:
			void SetPlayer(CDuckPlayer *player) {mPlayer = player;};
			CFillBufferThread(void){};
			~CFillBufferThread(void){};
		};

	typedef enum {Stop,Start,Exit,Running,Halted,Killed } FillBufferState;

	int					mStretch;

	int					mWinblit;

	int					mHScreenRes;

	int					mVScreenRes;

	int					mVidFreq;

	int					mBPP;

	CGUIController		*mGUIController;

	int					mLocked;
	int					mReportFramesAndIterations;
	int					mPlayLoopInterrupt;

	int					mCanCapture;
	CCMutex				mCanCaptureMutex;
	CConditionVariable	mCanCaptureCondition;

	FillBufferState		mFillBufferState;
	CCMutex				mFillBufferMutex;
	CConditionVariable	mFillBufferWait;
	CFillBufferThread	mFillBufferThread;
	int					mFillBufferStartFrame;
	int					mFillBufferEOF;

	int				mBlockSize;
	int				mMulticasting;
	int				mDirectSound;
	CDuckCompositeBuffer	*mCBuffer;
	CDuckContext	*mContext;
	int					mLastFrameDisplayed;
	int					mLastFrameDecompressed;
	int					mAudioTimeVTRAccumulator;
	int					mMovieWidth,mMovieHeight;
	int					mVideoSpeedTest;
	long				mVideoSpeedTestStartTime;

	long				mLastVideoOnlyTime;
	long				mVideoOnlyElapsedTime;

	// odd sizes
	int					mWarpFlag;
	int					mWarpHeight;
	int					mWarpWidth;

	int					mXOffset;
	int					mYOffset;
	HWND				mWindow;


	int	mIsPageFlipping;

#ifdef NETHFB
	NetworkStatusCallback_t	mCallback;
#endif

	void					*mPassback;
	MovieEventCallback_t	mCallback2;
	void					*mPassback2;

	int mPlaying;
	int mKeepPlaying;
	CCMutex	mMutex;
	CConditionVariable	mInterruptWait;

	int	mInterruptable;
	int	mVTRControl;

	int mNetworkMode;

	virtual void Cleanup(void);
	int		RestorePlayback(int frame);
	void	InterruptPlayback(void);
	void	DoFastForward(void);
	void	DoRewind(void);
	void	PutFrameOnScreen(int frame);
	void	MovieEventCheck(void);

	void	CanCaptureLock(void);
	void	CanCaptureUnlock(void);
	void	CanCaptureWait(void);
	void	CanCaptureSignal(void);

	int		ElapsedTime(void);
	int		VideoFrameRequired(int elapsedTime);
	int		FrameTime(int frame);
	void	VideoSleepTest(void);
	void	AudioChunkReleased(void);
	void	FillBufferSignal(void);
	void	BasicErrorHandler(int code,const char *message);
	void	ShowPositionAndTime(void);

	void	ConstructScreen(void);


public:
	CDuckPlayer(void);
#ifdef NETHFB
	float	Go(const char* filename,HWND wind,int xoffset,int yoffset,
		int stretch,int winblit,int buffersize,NetworkStatusCallback_t func,
		void *passback,MovieEventCallback_t func2,void *passback2,int loopforever,
		int netbuffersize,int netbufferwait,int videospeedtest,int directsound,
		int blocksize,int hres,int vres,int vid_frequency,int bitsperpixel);
#else
	float	Go(const char* filename,HWND wind,int xoffset,int yoffset,
		int stretch,int winblit,int buffersize,
		void *passback,MovieEventCallback_t func2,void *passback2,int loopforever,
		int netbuffersize,int netbufferwait,int videospeedtest,int directsound,
		int blocksize,int hres,int vres,int vid_frequency,int bitsperpixel,

		CGUIController *gui,CColorSlider *slider,C3BitmapButton *playbackbutton);
#endif
	~CDuckPlayer(void);
	int	IsPlaying(void);
	void StopPlaying(void);


	int		IsPaused(void);
	void	Pause(int p);
	void	FastForward(int f);
	int		IsFastForward(void);
	void	Rewind(int r);
	int		IsRewind(void);
	int		IsPageFlipping(void);


	void	ClearWarp(void);
	void	SetWarp(int height,int width);

	void	InvokeCallback(int code,const void *data,const char *message);
	int		IsMulticasting(void);

	void	StartCapture(const char *filename);
	void	StopCapture(void);

	void	CreateFillBufferThread(void);
	void	DestroyFillBufferThread(void);
	void	DoFillBufferThread(void);


	void	StartInFFMode(void);

	void	StartInPauseMode(void);

	void	DisplayCurrentFrame(void);

	void	SetAndDisplayFrame(int f);

	int		GetLastFrameDisplayed(void);

	int		BigImage(void);

	void	GoFullScreen(int f);



	void	AdvanceFrame(void);

	void	RetreatFrame(void);

	void	RetreatTime(int milliseconds);

	void	AdvanceTime(int milliseconds);

	CERROR_SUBCLASS(ImageError);

};


#endif
