/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	IRIX
#include	"CDuckSGIScreen.h"
#include	"CDuckImage.h"
#include	"CDuckCompositeBuffer.h"
#include	<milli_sleep.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<stropts.h>
#include	<gl.h>
#include <GL/GLwDrawA.h> 

#define DOUBLE_BUFFERED
//#define NOBLIT
//#define NODECOMPRESS
#define	BYTES_PER_PIXEL	4


extern int gsMenuComingDownHack;

CDuckSGIScreen::CDuckSGIScreen(HWND wnd)
{
	mWindow = wnd;
	mVideoBuffer = NULL;
	mBlitDone = 1;
}

CDuckSGIScreen::~CDuckSGIScreen(void)
{
	int res = ioctl(mWindow,I_FLUSH,FLUSHRW);		
#ifndef	 NDEBUG
	if (res == -1)
		fprintf(stderr,"ioctl error %d\n",errno);
#endif

	if (mVideoBuffer)
		delete [] mVideoBuffer;
	mVideoBuffer = NULL;
}

void
CDuckSGIScreen::SetWidgetDimensions(int h,int w)
{
	mWidgetHeight = h;
	mWidgetWidth = w;
}


void	
CDuckSGIScreen::Instantiate(int xstart,int ystart,int width,int height,int stretch)
{

	// the sgi blit will stretch horizontally for us. The low level
    // dxv blit will not do the stretch
	//mMovieWidth = width * (stretch ? 2 : 1);
	mMovieWidth = width;
	mMovieHeight = height * (stretch ? 2 : 1);
	mStretchFlag = stretch;

	// the second nibble of stretch has a further multiplier
	int mult = (stretch & 0x000000f0) >> 4;
	if (mult == 0) mult = 1;

	mMovieHeightForPlacement = mMovieHeight * mult;
	mMovieWidthForPlacement = mMovieWidth * (stretch ? 2 : 1) * mult;
	mSGIHorZoom = (stretch ? 2 : 1) * mult;
	mSGIVerZoom = 1 * mult;

	int	video_buffer_size = (mMovieWidth*mMovieHeight*BYTES_PER_PIXEL); 
	mVideoBufferSize = video_buffer_size;

	mVideoBuffer = new unsigned char[video_buffer_size];
	memset(mVideoBuffer,0x00,video_buffer_size);

	mScreen = DXL_CreateVScreen(mVideoBuffer+video_buffer_size - 
						(mMovieWidth*BYTES_PER_PIXEL),DXRGB32,
						mMovieWidth*-BYTES_PER_PIXEL,mMovieHeight);
	if (mScreen == NULL)
		throw VScreenException();

	DXL_AlterVScreenView(mScreen,0,0,10000,10000);
	DXL_AlterVScreenClip(mScreen,0,0,10000,10000);
	mFirstBlit = 1;
	mBlitDone = 1;

}

void	
CDuckSGIScreen::Blit(CDuckCompositeBuffer &cbuffer,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;
}


void	
CDuckSGIScreen::Blit(CDuckImage &image,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	/* This is here to speed menu response. The main loop and the
       play loop sort of deadlock when a menu comes down. The wait
       interval timeout below handles it ok, but the 1 second delay
       can be too slow. This just speeds things up a bit
    */

	if (gsMenuComingDownHack)
		{
		DXL_dxImageToVScreen(image.Native(),NULL);
		mLastBlitFrame = -1;
		return;
		}

try {

	mVideoBufferMutex.Lock();
	while (!mBlitDone)
		mVideoBufferCondition.WaitInterval(mVideoBufferMutex,1);
#ifndef	NODECOMPRESS
	DXL_dxImageToVScreen(image.Native(),mScreen);
#endif
	mBlitDone = 0;
	mLastBlitFrame = frame_number;
	mVideoBufferMutex.Unlock();
#ifdef TR20_KLUDGE
	if (mTR20)
		TR20_Convert((unsigned short *)mKludgeBuffer,
					(unsigned long *)(mVideoBuffer + mVideoBufferSize
					-(mMovieWidth*BYTES_PER_PIXEL)),mMovieWidth,
					mMovieHeight,mStretchFlag);
#endif

	SendBlitMessage();
	}
catch(...) // wait timed out
	{
	// timeout causes mutex reaquisition
	DXL_dxImageToVScreen(image.Native(),NULL);
	mLastBlitFrame = -1;
	mVideoBufferMutex.Unlock();
	}
}



void
CDuckSGIScreen::SendBlitMessage(void)
{
	CDuckSGIScreen	*tmp;

	tmp = this;
	write(mWindow,&tmp,sizeof(tmp));
}

void
CDuckSGIScreen::Blit(Widget w)
{
	if (gsMenuComingDownHack)
		{
		mBlitDone = 1;
		mVideoBufferCondition.Signal();
		return;
		}

#ifndef	NOBLIT
#ifdef DOUBLE_BUFFERED
	if (mFirstBlit) glDrawBuffer(GL_BACK);
#endif
#endif
	mVideoBufferMutex.Lock();

#ifndef	NOBLIT
	float x,y;
	x = (((float)mMovieWidthForPlacement) / mWidgetWidth) / -1;
	y = (((float)mMovieHeightForPlacement) / mWidgetHeight) / -1;
	if (x < -1) x = -1;
	if (y < -1) y = -1;
	if (mFirstBlit || (mLastX != x) || (mLastY != y))
		glRasterPos2f(x,y);
	mLastX = x;
	mLastY = y;

	if (mFirstBlit)
		glPixelZoom(mSGIHorZoom,mSGIVerZoom);

	glDrawPixels(mMovieWidth,mMovieHeight,
						GL_RGBA,GL_UNSIGNED_BYTE,mVideoBuffer);
	mFirstBlit = 0;
//	glDrawPixels(mMovieWidth,mMovieHeight,
//						GL_YCRCB_422_SGIX,GL_UNSIGNED_BYTE,mVideoBuffer);

#endif
	mBlitDone = 1;
	mVideoBufferMutex.Unlock();
	mVideoBufferCondition.Signal();

#ifndef	NOBLIT
#ifdef DOUBLE_BUFFERED
	glXSwapBuffers(XtDisplay(w),glXGetCurrentDrawable());
#endif
#endif
}

void
CDuckSGIScreen::Reset(void)
{
	mFirstBlit = 1;
}

#endif
