/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	"CDuckScreen.h"
#include	"BuildClutFromPalette.h"
#include	"CDuckImage.h"
#include	"danbug.h"

CDuckScreen::CDuckScreen(void)
{
	mScreen = NULL;
	mLastBlitFrame = -1;
}

CDuckScreen::~CDuckScreen(void)
{
	if (mScreen)
		DXL_DestroyVScreen(mScreen);
	mScreen = NULL;
}

void
CDuckScreen::SetBlitQuality(CDuckImage &image,int stretch_flag)
{
	switch(stretch_flag)
		{
		case 0:
			DXL_SetVScreenBlitQuality(mScreen,DXBLIT_SAME);
			break;
		case 1:
			DXL_SetVScreenBlitQuality(mScreen,DXBLIT_STRETCH);
			break;
		case 2:
			DXL_SetVScreenBlitQuality(mScreen,DXBLIT_STRETCH_BRIGHT);
			break;

		}
}

void
CDuckScreen::NullBlit(CDuckImage &image,int frame_number)
{
	DXL_dxImageToVScreen(image.Native(),NULL);
	mLastBlitFrame = -1;
}

#ifdef WIN32
void
CDuckScreen::BuildClut(HDC device,DXL_VSCREEN_HANDLE vs)
{
///	dbm
 dprintf("unsupported call: CDuckScreen::BuildClut(HDC device,DXL_VSCREEN_HANDLE vs)");
 /*
	unsigned char *clutbuf1 = NULL;
	unsigned char *clutbuf2 = NULL;
	HRESULT	result;
	WORD *clut = NULL;
	unsigned char *pal = NULL;

try
	{
	#define BCFP_PALSIZE 255	// Since I'm just lookin
					// I can use the whole palette, not just
					// 236. I don't use 256 becaue the first
					// entry in a windows palette is special
	#define BCFP_CLUTSIZE  32768	// 15 bit clut


	PALETTEENTRY pentry[BCFP_PALSIZE];
	pal = new unsigned char[BCFP_PALSIZE*3];
	clut = new WORD[BCFP_CLUTSIZE];
	clutbuf1 = new unsigned char[BCFP_CLUTSIZE];
	clutbuf2 = new unsigned char[BCFP_CLUTSIZE];

	result = ::GetSystemPaletteEntries(device,1,BCFP_PALSIZE,pentry);

	// copy for ease of use
	for (int x = 0; x < BCFP_PALSIZE; ++x)
	{
		pal[(x*3)+0] = pentry[x].peRed;
		pal[(x*3)+1] = pentry[x].peGreen;
		pal[(x*3)+2] = pentry[x].peBlue;
	}

	BuildClutFromPalette(clut,BCFP_CLUTSIZE,pal,BCFP_PALSIZE,0);

	for (x = 0; x < BCFP_CLUTSIZE; ++x)
	{
		clutbuf1[x] = (unsigned char)((clut[x] & 0xff00) >> 8);
		clutbuf2[x] = (unsigned char)(clut[x] & 0x00ff);
	}

	DXL_SetVScreenCLUTs(vs,clutbuf1,clutbuf2,0);
	
	if (clutbuf1) delete clutbuf1;
	clutbuf1 = NULL;
	if (clutbuf2) delete clutbuf2;
	clutbuf2 = NULL;
	if (clut) delete clut;
	clut = NULL;
	if (pal) delete pal;
	pal = NULL;
	}
catch(...)
	{
	if (clutbuf1) delete clutbuf1;
	clutbuf1 = NULL;
	if (clutbuf2) delete clutbuf2;
	clutbuf2 = NULL;
	if (clut) delete clut;
	clut = NULL;
	if (pal) delete pal;
	pal = NULL;
	throw;
	}
*/
}
#endif


void 
CDuckScreen::Blackout(unsigned char *p,int x,int y,int width,int height,
					 int depth,int pitch,int s_width,int s_height)
{
	unsigned char *pp;
	unsigned short *ppp;

	width *= 2;
	height *= 2;

	if (width + x > s_width)
		width = s_width -x;
	if (height + y > s_height)
		height = s_height -y ;

	int	mult = (depth == 8 ? 1 :(depth == 24 ? 3 : 2));
	p += y*pitch;
	p += x*mult;

	if (mult == 1 || mult == 3)
	{
	for (int i = 0; i < height; ++i)
		{
		pp = p;
		for (int ii = 0; ii < width; ++ii)
			{
			*(pp++) = 0;
			if (mult == 3)
				{
				*(pp++) = 0;
				*(pp++) = 0;
				}
			}
		p += pitch;
		}
	}
	else
	{
	for (int i = 0; i < height; ++i)
		{
		ppp = (unsigned short *)p;
		for (int ii = 0; ii < width; ++ii)
			{
			*(ppp++) = 0;
			}
		p += pitch;
		}

	}

}

int		
CDuckScreen::ReadyFrame(void)
{
	return mLastBlitFrame;
}

void	
CDuckScreen::Flip(void)
{
	// default does nothing
	mLastBlitFrame = -1;
}

int		
CDuckScreen::BuffersAvailable(void)
{
	// default, we always do
	return (mLastBlitFrame == -1);
}

void	
CDuckScreen::BlitNow(CDuckImage &image,int frame_number,
		int warpflag,int warpwidth,int warpheight)
{
	Blit(image,frame_number,warpflag,warpwidth,warpheight);
}

#if 0
void	
CDuckScreen::BlitNow(CDuckCompositeBuffer &cbuffer,int frame_number,
		int warpflag,int warpwidth,int warpheight)
{
	Blit(cbuffer,frame_number,warpflag,warpwidth,warpheight);
}
#endif

void
CDuckScreen::Reset(void)
{
	mLastBlitFrame = -1;
}

void
CDuckScreen::VTRReset(void)
{
	mLastBlitFrame = -1;
}


// this is used for sun's xil stuff
void
CDuckScreen::AsyncCleanup(void)
{
}
