/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if ((defined SOLARIS) || (defined LINUX))
#include	<MegaHeaders.h>
#include	"CDuckUnixScreen.h"
#include	"CDuckImage.h"
#include	"CDuckCompositeBuffer.h"

//#define NOBLIT
//#define NODECOMPRESS
#define	BYTES_PER_PIXEL	4



extern int gsMenuComingDownHack;
int t,counter = 0;

CDuckUnixScreen::CDuckUnixScreen(HWND wnd)
{
	mXImage[0] = 0;
	mXImage[1] = 0;
	mWindow = wnd;
	mVideoBuffer[0] = NULL;
	mVideoBuffer[1] = NULL;
	mBlitDone = 1;
	mWhichImage = 0;
}

CDuckUnixScreen::~CDuckUnixScreen(void)
{
#ifdef LINUX
	int res = ioctl(mWindow.mPipe,FDFLUSH);
#else
	int res = ioctl(mWindow.mPipe,I_FLUSH,FLUSHRW);		
#endif
#ifndef	 NDEBUG
	if (res == -1)
		fprintf(stderr,"ioctl error\n");
#endif

	// XDestroyImage free's this
//	if (mVideoBuffer)
//		delete [] mVideoBuffer;
//	mVideoBuffer = NULL;
	if (mXImage[0])
		XDestroyImage(mXImage[0]);
	mXImage[0] = 0;
	if (mXImage[1])
		XDestroyImage(mXImage[1]);
	mXImage[1] = 0;
}

void	
CDuckUnixScreen::Instantiate(int xstart,int ystart,int width,int height,int stretch)
{

	mMovieWidth = width * (stretch ? 2 : 1);
	mMovieHeight = height * (stretch ? 2 : 1);
	mStretchFlag = stretch;

	mMovieHeightForPlacement = mMovieHeight;
	mMovieWidthForPlacement = mMovieWidth;

	int	video_buffer_size = (mMovieWidth*mMovieHeight*BYTES_PER_PIXEL); 
	// must do malloc because XDestroyImage calls free()
	mVideoBuffer[0] = (unsigned char *)malloc(video_buffer_size);
	mVideoBuffer[1] = (unsigned char *)malloc(video_buffer_size);
	memset(mVideoBuffer[0],0x00,video_buffer_size);
	memset(mVideoBuffer[1],0x00,video_buffer_size);

	mScreen = DXL_CreateVScreen(mVideoBuffer[0],DXRGB16,
						mMovieWidth*BYTES_PER_PIXEL,mMovieHeight);

	if (mScreen == NULL)
		throw VScreenException();

	DXL_AlterVScreenView(mScreen,0,0,10000,10000);
	DXL_AlterVScreenClip(mScreen,0,0,10000,10000);

	int cnt;
	int *foo = XListDepths(mWindow.mDisplay,DefaultScreen(mWindow.mDisplay),
				&cnt);
	mXImage[0] = XCreateImage(mWindow.mDisplay,mWindow.mVisual,24,ZPixmap,0,
			(char *)mVideoBuffer[0],mMovieWidth,mMovieHeight,
			32,mMovieWidth * BYTES_PER_PIXEL);
	mXImage[1] = XCreateImage(mWindow.mDisplay,mWindow.mVisual,24,ZPixmap,0,
			(char *)mVideoBuffer[1],mMovieWidth,mMovieHeight,
			32,mMovieWidth * BYTES_PER_PIXEL);
	if (!mXImage[0] || !mXImage[1])
		throw InitException();

	mWhichImage = 0;
	mFirstBlit = 1;
	mBlitDone = 1;
}

void	
CDuckUnixScreen::Blit(CDuckCompositeBuffer &cbuffer,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;
}


// image swapping is commented out because of logo flashing
// at startup. Probably a NULL frame problem
void	
CDuckUnixScreen::Blit(CDuckImage &image,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	/* This is here to speed menu response. The main loop and the
       play loop sort of deadlock when a menu comes down. The wait
       interval timeout below handles it ok, but the 1 second delay
       can be too slow. This just speeds things up a bit
    */

	//t = timeGetTime();

	if (gsMenuComingDownHack)
		{
#ifndef NODECOMPRESS
		DXL_AlterVScreen(mScreen,(unsigned char *)mVideoBuffer[mWhichImage],
			DXRGB16,mMovieWidth*BYTES_PER_PIXEL,mMovieHeight);
		DXL_dxImageToVScreen(image.Native(),NULL);
#endif
		//mWhichImage ^= 1;
		mLastBlitFrame = -1;
		return;
		}

try {

	mVideoBufferMutex.Lock();
	while (!mBlitDone)
		{
		mVideoBufferCondition.WaitInterval(mVideoBufferMutex,2);
		}
#ifndef	NODECOMPRESS
	DXL_AlterVScreen(mScreen,(unsigned char *)mVideoBuffer[mWhichImage],
			DXRGB16,mMovieWidth*BYTES_PER_PIXEL,mMovieHeight);
	DXL_dxImageToVScreen(image.Native(),mScreen);
#endif
//	mWhichImage ^= 1;
	mBlitDone = 0;
	mVideoBufferMutex.Unlock();
	mLastBlitFrame = frame_number;
	SendBlitMessage();
	}
catch(...) // wait timed out
	{
	// timeout causes mutex reaquisition
#ifndef NODECOMPRESS
	DXL_AlterVScreen(mScreen,(unsigned char *)mVideoBuffer[mWhichImage],
			DXRGB16,mMovieWidth*BYTES_PER_PIXEL,mMovieHeight);
	DXL_dxImageToVScreen(image.Native(),NULL);
#endif
//	mWhichImage ^= 1;
	mLastBlitFrame = -1;
	mVideoBufferMutex.Unlock();
	}
}



void
CDuckUnixScreen::SendBlitMessage(void)
{
	CDuckUnixScreen	*tmp;

	tmp = this;
	int amt = write(mWindow.mPipe,&tmp,sizeof(tmp));
}

void
CDuckUnixScreen::Blit(void)
{
	if (gsMenuComingDownHack)
		{
		mBlitDone = 1;
		mVideoBufferCondition.Signal();
		return;
		}

	mVideoBufferMutex.Lock();

#ifndef	NOBLIT

	int x = 0;
	int y = 0;
	Dimension h,w;
	XtVaGetValues(mWindow.mWidget,XmNwidth,&w,XmNheight,&h,NULL);
	x = (w - mMovieWidthForPlacement) / 2;
	y = (h - mMovieHeightForPlacement) / 2;
	if (!gsMenuComingDownHack)
		{
		XPutImage(mWindow.mDisplay,XtWindow(mWindow.mWidget),
		(*(mWindow.mGC))(mWindow.mRef),
		mXImage[mWhichImage],0,0,x,y,mMovieWidth,mMovieHeight);
		XFlush(mWindow.mDisplay);
		}
#endif
	mFirstBlit = 0;
	mBlitDone = 1;
	mVideoBufferMutex.Unlock();
	mVideoBufferCondition.Signal();

}

#endif
