/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	"CDuckVideoStream.h"
#include	"CDuckAudioStream.h"
#include	"CDuckImage.h"
#include	"danbug.h"

int gForceRate=0;
int gForceScale=0;

CDuckVideoStream::CDuckVideoStream(CDuckFile &file,const char *name,int which)
: CDuckStream(file,name,which,DUK_VIDSTREAM)
{
	mSeekBuffer = new unsigned char[MAX_STATIC_FRAME_SIZE];
}

CDuckVideoStream::~CDuckVideoStream(void)
{
	delete [] mSeekBuffer;
}

int
CDuckVideoStream::IsFrameKeyFrame(int frame)
{
	int	flags;

	flags = HFB_GetIndexFlags(mStreamHandle,HFB_FRAMENUM,frame);
	return (flags & HFB_DATA_KEYFRAME ? 1 : 0);
}

void
CDuckVideoStream::GetFrameRates(nmr_fraction<I32> &FramesPerSecond,
											int &AudioSamplesTillStart)
{
	int frames_per_second;

	HFB_GetFrameRates(mStreamHandle,NULL,
		&frames_per_second,&AudioSamplesTillStart);

	long rate,scale;
	HFB_GetStreamRateAndScale(mStreamHandle,&rate,&scale);
	nmr_fraction<I32> f(rate,scale);
	FramesPerSecond =f;
}

void
CDuckVideoStream::GetFrameRates(CDuckAudioStream &audio_stream,
					nmr_fraction<I32> &FramesPerSecond,
													int &AudioSamplesTillStart)
{
	int frames_per_second;

	HFB_GetFrameRates(mStreamHandle,audio_stream.Native(),
		&frames_per_second,&AudioSamplesTillStart);

	long rate,scale;
	HFB_GetStreamRateAndScale(mStreamHandle,&rate,&scale);

// rate = 14985;									///	dbm -- rate/scale kluge 15 fps
// rate = 29970;									///	dbm -- rate/scale kluge 30 fps
// rate = 2000;										///	for StingRay 2fps
// scale = 1000;
	if(gForceRate) rate = gForceRate;
	if(gForceScale) scale = gForceScale;			///	dbm -- for testing purposes  & stuff
	nmr_fraction<I32> f(rate,scale);
	FramesPerSecond = f;
}

int
CDuckVideoStream::ReadData(unsigned char *framedata,
						   long &frame_len,dukDirect direction,int count)
{
	return HFB_ReadData(mStreamHandle,framedata,&frame_len,direction,count);
}

int 
CDuckVideoStream::Seek(int frame,CDuckImage &image)
{
	int				prev_frame;
	long			single_len = MAX_STATIC_FRAME_SIZE;

	// in an interframe file, may have keyframe way before this
	if ((prev_frame = HFB_PreviousKeyFrame(mStreamHandle,frame)) < 0) 
		return 0;

 dprintf("CDuckVideoStream::Seek: suddenly seeking susan?\n");
	while (prev_frame <= frame)
		{
		single_len = MAX_STATIC_FRAME_SIZE;
		ReadData(mSeekBuffer,single_len,DUK_ABSOLUTE,prev_frame);
		image.UpdateAndDecompress(mSeekBuffer);
		//if (!DXL_IsXImageKeyFrame(image.Native())) exit(0);
		prev_frame++;
		}

 dprintf("::Seek done\n");
	return 1;
}

int
CDuckVideoStream::FramesToNextKeyFrame(int cur_frame)
{
	int		junk;

	long	chunk = HFB_FrameToChunk(mStreamHandle,cur_frame);		
	int		ftn = HFB_FramestoNextKeyFrame(mStreamHandle,chunk,&junk);

	return ftn;
}

int
CDuckVideoStream::PreviousKeyFrame(int cur_frame)
{
	return HFB_PreviousKeyFrame(mStreamHandle,cur_frame);
}

int
CDuckVideoStream::FrameToChunk(int frame)
{
	return HFB_FrameToChunk(mStreamHandle,frame);
}
