/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if (defined SOLARIS)
#include	<MegaHeaders.h>
#include	"CDuckXILScreen.h"
#include	"CDuckImage.h"
#include	"CDuckCompositeBuffer.h"
#include	<Xm/Xm.h>

//#define NOBLIT
//#define NODECOMPRESS
#define	BYTES_PER_PIXEL	3

extern CCMutex	gX11WaitingMutex;
extern int		gX11WaitingForEvent;

extern int gsMenuComingDownHack;
//int t,counter = 0;

CDuckXILScreen::CDuckXILScreen(HWND wnd)
{
	mWindow = wnd;
	mBlitDone = 1;
	mXilState = NULL;
	mXilDisplay = NULL;
	mXilBuffer = NULL;
	mCleanup = 0;
	mXilChild = NULL;
}

CDuckXILScreen::~CDuckXILScreen(void)
{
	int res = ioctl(mWindow.mPipe,I_FLUSH,FLUSHRW);		
#ifndef	 NDEBUG
	if (res == -1)
		fprintf(stderr,"ioctl error\n");
#endif
	Cleanup();
}

void
CDuckXILScreen::AsyncCleanup(void)
{
	gX11WaitingMutex.Lock();
	int res = ioctl(mWindow.mPipe,I_FLUSH,FLUSHRW);		
#ifndef	 NDEBUG
	if (res == -1)
		fprintf(stderr,"ioctl error\n");
#endif
	if (gX11WaitingForEvent)
		{
		mCleanup = 1;
		mCleanupMutex.Lock();
		SendBlitMessage();
		gX11WaitingMutex.Unlock();
		while (mCleanup) mCleanupWait.Wait(mCleanupMutex);
		}
	else
		{
		Cleanup();
		gX11WaitingMutex.Unlock();
		}
}

void
CDuckXILScreen::Cleanup(void)
{
	if (mXilDisplay)
		xil_destroy(mXilDisplay);
	mXilDisplay = NULL;

	if (mXilBuffer)
		xil_destroy(mXilBuffer);
	mXilBuffer = NULL;

	if (mXilState)
		xil_close(mXilState);
	mXilState = NULL;
	mCleanup = 0;
	mCleanupWait.Signal();
}

void	
CDuckXILScreen::Instantiate(int xstart,int ystart,int width,int height,int stretch)
{
	mCleanup = 0;
	mMovieWidth = width * (stretch ? 2 : 1);
	mMovieHeight = height * (stretch ? 2 : 1);
	mStretchFlag = stretch;

	mMovieHeightForPlacement = mMovieHeight;
	mMovieWidthForPlacement = mMovieWidth;

	mFirstBlit = 1;
	mBlitDone = 1;

	mScreen = DXL_CreateVScreen(0,DXRGB24,0,0);
}


void
CDuckXILScreen::_Instantiate(void)
{
	mXilState = xil_open();
	if (!mXilState)
		throw InitException();

	mXilDisplay = xil_create_from_window(mXilState,mWindow.mDisplay,
													XtWindow(mWindow.mWidget));
	if (!mXilDisplay)
		throw InitException();

	mXilBuffer = xil_create(mXilState,mMovieWidth,mMovieHeight,3,XIL_BYTE);
	if (!mXilBuffer)
		throw InitException();
	XilColorspace cspace = xil_colorspace_get_by_name(mXilState,"rgblinear");
	xil_set_colorspace(mXilBuffer,cspace);
	xil_set_colorspace(mXilDisplay,cspace);

	XilMemoryStorage storage;
	xil_export(mXilBuffer);
	xil_get_memory_storage(mXilBuffer,&storage);

	int	video_buffer_size = (mMovieWidth*mMovieHeight*BYTES_PER_PIXEL); 

	DXL_AlterVScreen(mScreen,storage.byte.data,DXRGB24,
									mMovieWidth*BYTES_PER_PIXEL,mMovieHeight);
	memset(storage.byte.data,0,mMovieWidth*BYTES_PER_PIXEL*mMovieHeight);

	if (mScreen == NULL)
		throw VScreenException();

}

void	
CDuckXILScreen::Blit(CDuckCompositeBuffer &cbuffer,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	mLastBlitFrame = frame_number;
}


void	
CDuckXILScreen::Blit(CDuckImage &image,int frame_number,
						 int warp,int warpheight,int warpwidth)
{
	/* This is here to speed menu response. The main loop and the
       play loop sort of deadlock when a menu comes down. The wait
       interval timeout below handles it ok, but the 1 second delay
       can be too slow. This just speeds things up a bit
    */

	if (mFirstBlit)
		{
		mVideoBufferMutex.Lock();
		mBlitDone = 0;
		DXL_dxImageToVScreen(image.Native(),NULL);
		mLastBlitFrame = frame_number;
		mVideoBufferMutex.Unlock();
		SendBlitMessage();
		return;
		}

	//t = timeGetTime();

	if (gsMenuComingDownHack)
		{
		DXL_dxImageToVScreen(image.Native(),NULL);
		mLastBlitFrame = -1;
		return;
		}

try {

	mVideoBufferMutex.Lock();
	while (!mBlitDone)
		{
		mVideoBufferCondition.WaitInterval(mVideoBufferMutex,2);
		}
#ifndef	NODECOMPRESS
	DXL_dxImageToVScreen(image.Native(),mScreen);
#endif
	mBlitDone = 0;
	mVideoBufferMutex.Unlock();
	mLastBlitFrame = frame_number;
	SendBlitMessage();
	}
catch(...) // wait timed out
	{
	// timeout causes mutex reaquisition
	DXL_dxImageToVScreen(image.Native(),NULL);
	mLastBlitFrame = -1;
	mVideoBufferMutex.Unlock();
	}
}



void
CDuckXILScreen::SendBlitMessage(void)
{
	CDuckXILScreen	*tmp;

	tmp = this;
	int amt = write(mWindow.mPipe,&tmp,sizeof(tmp));
}

void
CDuckXILScreen::Blit(void)
{
	if (mFirstBlit)
		{
		mVideoBufferMutex.Lock();
		mFirstBlit = 0;
		_Instantiate();
		mBlitDone =1;
		mVideoBufferCondition.Signal();
		mVideoBufferMutex.Unlock();
		return;
		}

	if (mCleanup)
		{
		Cleanup();
		return;
		}

	if (gsMenuComingDownHack)
		{
		mBlitDone = 1;
		mVideoBufferCondition.Signal();
		return;
		}
	
	mVideoBufferMutex.Lock();

#ifndef	NOBLIT

	int x,y;
	Dimension h,w;
	XtVaGetValues(mWindow.mWidget,XmNwidth,&w,XmNheight,&h,NULL);
	x = (w - mMovieWidthForPlacement) / 2;
	y = (h - mMovieHeightForPlacement) / 2;
	if (!gsMenuComingDownHack)
		{
		mXilChild = xil_create_child(mXilDisplay,x,y,
			mMovieWidthForPlacement,mMovieHeightForPlacement,0,3);
#if 1
		xil_copy(mXilBuffer,mXilChild);
#else
		// this is totally pointless. It makes the image spin.
		// I like to turn it on and watch it sometimes. Kinda funny
		static float rot = 0.0;
		xil_set_origin(mXilBuffer,mMovieWidthForPlacement / 2,
						mMovieHeightForPlacement / 2);
		xil_set_origin(mXilChild,mMovieWidthForPlacement / 2,
						mMovieHeightForPlacement / 2);
		xil_rotate(mXilBuffer,mXilChild,"general",rot += 0.1);

#endif
		xil_destroy(mXilChild);
		}
	mFirstBlit = 0;
#endif
	mBlitDone = 1;
	mVideoBufferMutex.Unlock();
	mVideoBufferCondition.Signal();

}

void
CDuckXILScreen::Reset(void)
{
	mFirstBlit = 1;
	CDuckScreen::Reset();
}

#endif
