/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CPlayerContext.h>
#include	<CPlayerUIContext.h>
#include	<stdio.h>

CPlayerContext::CPlayerContext(CPlayerUIContext *ui,int ownsui)
{
	mUI = ui;
	mOwnsUI = ownsui;
	mFilename = NULL;
	mLastNetspec = NULL;
	mPlayer = new CDuckPlayer;
	mDiskBufferSize = 1024;
	mDiskBufferReadSize = 64000;
	mNetBufferSize = 3000;
	mNetBufferPreload = 75;
	mStretch = 0;
	mBlitter = 0;
	mLastCaptureName = NULL;
	mLastDirectoryAddress = NULL;
	mLastDirectoryPort = NULL;
	mVideoSpeedTest = 0;
	mLoopForever = 0;
	mShowMessages = 0;
}

void	
CPlayerContext::PlayLogo(const char *filename,HWND wnd)
{
	if (IsPlaying())
		Stop();

	mFilename = filename;
	int lastDiskBufferSize 		= mDiskBufferSize;
	int lastDiskBufferReadSize 	= mDiskBufferReadSize;
	int lastStretch 			= mStretch;
	int lastBlitter 			= mBlitter;
	int lastVideoSpeedTest 		= mVideoSpeedTest;
	int lastLoopForever			= mLoopForever;
	int lastShowMessages		= mShowMessages;
	
	mDiskBufferSize = 1024;
	mDiskBufferReadSize = 0;
	mStretch = 0;
	mBlitter = 0;
	mVideoSpeedTest = 0;
	mLoopForever = 0;
	mShowMessages = 0;
	_LaunchPlayer(filename,wnd);
	
	mDiskBufferSize 	= lastDiskBufferSize;
	mDiskBufferReadSize = lastDiskBufferReadSize;
	mStretch 			= lastStretch;
	mBlitter 			= lastBlitter;
	mVideoSpeedTest 	= lastVideoSpeedTest;
	mLoopForever		= lastLoopForever;
	mShowMessages		= lastShowMessages;
}

CPlayerContext::~CPlayerContext(void)
{
	if (mUI && mOwnsUI)
		delete mUI;
	mUI = NULL;

	if (mPlayer)
		delete mPlayer;
	mPlayer = NULL;
}


///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

int
CPlayerContext::CanPlay(void)
{
	if (!mPlayer) return 0;
	if (!mPlayer->IsPlaying() && !mFilename) return 0;
	if (!mPlayer->IsPlaying() && mFilename) return 1;
	if (mPlayer->IsRewind() || mPlayer->IsFastForward() ||
			mPlayer->IsPaused())
		return 1;
	return 0;
}


int
CPlayerContext::IsPlaying(void)
{
	if (mPlayer)
		return mPlayer->IsPlaying();
	else
		return 0;
}

int
CPlayerContext::CanRewind(void)
{
	if (!mPlayer) return 0;
	if (mPlayer->IsMulticasting()) return 0;
	if (!(mPlayer->IsPlaying()) || 
			mPlayer->IsFastForward() || 
			mPlayer->IsPaused()) return 0;
	return 1;
}

int
CPlayerContext::CanFastforward(void)
{
	if (!mPlayer) return 0;
	if (mPlayer->IsMulticasting()) return 0;
	if (!(mPlayer->IsPlaying()) ||
			mPlayer->IsRewind() ||
			mPlayer->IsPaused()) return 0;
	return 1;
}

int
CPlayerContext::CanPause(void)
{
	if (!mPlayer) return 0;
	if (mPlayer->IsMulticasting()) return 0;
	if (!(mPlayer->IsPlaying()) || 
			mPlayer->IsRewind() ||
			mPlayer->IsFastForward())
		return 0;
	return 1;
}

void
CPlayerContext::Pause(void)
{
	if (mPlayer)
		mPlayer->Pause(!mPlayer->IsPaused());
}

void
CPlayerContext::Fastforward(void)
{
	if (mPlayer)
		mPlayer->FastForward(!mPlayer->IsFastForward());
}

void
CPlayerContext::NetworkDirectory(HWND wnd)
{
	if (IsPlaying())
		Stop();

	nmr_string netspec = "";
	nmr_string new_directory_address = "";
	nmr_string new_directory_port = "";

	netspec = mUI->PostNetworkDirectoryDialog(
								mLastDirectoryAddress,mLastDirectoryPort,
							new_directory_address,new_directory_port);
	if (netspec != "")
		{
		mLastNetspec = netspec;
		_LaunchPlayer(mLastNetspec,wnd);
		}
	if (new_directory_address != "")
		mLastDirectoryAddress = new_directory_address;

	if (new_directory_port != "")
		mLastDirectoryPort = new_directory_port;
}

void
CPlayerContext::OpenNetwork(HWND wnd)
{
	if (IsPlaying())
		Stop();

	nmr_string netspec;

	netspec = mUI->PostOpenNetworkDialog(mLastNetspec);
	if (netspec != "")
		{
		mLastNetspec = netspec;
		_LaunchPlayer(mLastNetspec,wnd);
		}
}

void
CPlayerContext::Play(HWND wnd)
{
	if (!mFilename) return;

	if (IsPlaying())
		Stop();

	_LaunchPlayer(mFilename,wnd);
}


void
CPlayerContext::Rewind(void)
{
	if (mPlayer)
		mPlayer->Rewind(!mPlayer->IsRewind());
}

void
CPlayerContext::Stop(void)
{
	if (mPlayer)
		mPlayer->StopPlaying();
}

void	
CPlayerContext::OpenFile(const char *filename,HWND wnd)
{
	if (IsPlaying())
		Stop();

	mFilename = filename;
	_LaunchPlayer(filename,wnd);
}

void
CPlayerContext::BuffersConfig(void)
{
int	nb_size,nb_preload,db_size,db_read;

	db_size 	= mDiskBufferSize;
	db_read 	= mDiskBufferReadSize;
	nb_size 	= mNetBufferSize;
	nb_preload	= mNetBufferPreload;
	if (mUI->PostBuffersConfigDialog(db_size,db_read,nb_size,nb_preload))
		{
		mNetBufferSize 		= nb_size;
		mNetBufferPreload 	= nb_preload;
		mDiskBufferSize 	= db_size;
		mDiskBufferReadSize	= db_read;
		}
}

void
CPlayerContext::BlittingConfig(void)
{
	int	stretch,blitter;

	stretch = mStretch;
	blitter = mBlitter;
	if (mUI->PostBlittingConfigDialog(stretch,blitter))
		{
		mStretch = stretch;
		mBlitter = blitter;
		}
}

int
CPlayerContext::IsCapturing(void)
{
	if (mPlayer && mPlayer->IsPlaying() && mPlayer->IsCapturing())
		return 1;
	return 0;
}


void
CPlayerContext::StartCapture(void)
{
	nmr_string name;

	name = mUI->PostStartCaptureDialog(mLastCaptureName);
	if (name != "")
		{
		mLastCaptureName = name;
		mPlayer->StartCapture(name);
		}
}

void
CPlayerContext::StopCapture(void)
{
	mPlayer->StopCapture();
}

void
CPlayerContext::SetVideoSpeedTest(int v)
{
	mVideoSpeedTest = v;
}

void
CPlayerContext::SetLoopForever(int v)
{
	mLoopForever = v;
}

void
CPlayerContext::SetShowMessages(int v)
{
	mShowMessages = v;
}

int
CPlayerContext::GetVideoSpeedTest(void)
{
	return mVideoSpeedTest;
}

int
CPlayerContext::GetLoopForever(void)
{
	return mLoopForever;
}

int
CPlayerContext::GetShowMessages(void)
{
	return mShowMessages;
}










