/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<MegaHeaders.h>
#include 	<NeilHack.h>
#include	<duck_mem.h>
#include	"danbug.h"

NEIL_FUNCTION_TABLE	gNeilFunctionTable;

#if (defined WIN32)
extern "C" {
int e_open(const char *name);
int e_read(int f,char *buf,size_t siz);
void e_close(int f);
long e_seek(int f,long offset,int whence);
};

int e_open(const char *name)
{
	HANDLE h;
	h  = ::CreateFile(name,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,
			/* FILE_ATTRIBUTE_NORMAL | */ FILE_FLAG_NO_BUFFERING,NULL);
	if (h == INVALID_HANDLE_VALUE)
		return -1;
	else
		return ((int)h);
}

void e_close(int f)
{
	::CloseHandle((HANDLE)f);
}

int e_read(int f,char *buf,size_t siz)
{
	DWORD	bytes_read;

	if (buf == NULL)
		{
		char *b2 = new char[siz];

		if (::ReadFile((HANDLE)f,buf,siz,&bytes_read,NULL) == TRUE)
			{
			delete [] b2;

			return bytes_read;
			}
		else
			{
			delete [] b2;

			return -1;
			}
		
		}
	else
		{
		if (::ReadFile((HANDLE)f,buf,siz,&bytes_read,NULL) == TRUE)
			return bytes_read;
		else
			return -1;
		}
}

long e_seek(int f,long offset,int whence)
{
	switch(whence)
		{
		case SEEK_SET:
			whence = FILE_BEGIN;
			break;
		case SEEK_CUR:
			whence = FILE_CURRENT;
			break;
		case SEEK_END:
			whence = FILE_END;
			break;
		}
	return ::SetFilePointer((HANDLE)f,offset,NULL,whence);
}
#endif

int normal_open(const char *name)
{
    int f;                                       
    char filename[80];

    if(name[strlen(name)-4] != '.') {   /*no extension, try .AVI and .DUK*/
        sprintf(filename,"%s.AVI",name);
#ifdef	WIN32
        f = open(filename,O_BINARY|O_RDONLY);
#else
		f = open(filename,O_RDONLY);
#endif
        if(f < 0) {
            sprintf(filename,"%s.DUK",name);
#ifdef	WIN32
            f = open(filename,O_BINARY|O_RDONLY);
#else
			f = open(filename,O_RDONLY);
#endif
            if(f < 0) strcpy(filename,name);
        }
        return(f);
    }
    strcpy(filename,name);
#ifdef	WIN32
    return(open(filename,O_BINARY|O_RDONLY));
#else
	return(open(filename,O_RDONLY));
#endif
}


#ifdef	WIN32
void *buff_addr = 0;
int buff_siz = 0;

void duck_free(void *blk)
{
	unsigned long bf = (unsigned long)buff_addr;


	if (blk == (void *)(bf & 0xfffffc00))
		{
		::VirtualUnlock(buff_addr,buff_siz);
		::VirtualFree(buff_addr,0,MEM_RELEASE);
		}
	else

		{

		char *c = (char *)blk;

		delete [] c;

		}
}

void *duck_malloc(size_t size,enum tmemtype fred)
{
	if (fred == DMEM_CDBUFF)
		{
		buff_siz = size + 2048;

		buff_addr = ::VirtualAlloc(NULL,buff_siz,MEM_COMMIT,
											PAGE_NOCACHE | PAGE_READWRITE);
		if (buff_addr)
			::VirtualLock(buff_addr,buff_siz);
		//buff_siz = size;

		if (buff_addr)

			memset(buff_addr,0,buff_siz);
		return (void *)(((unsigned long)buff_addr) & 0xfffffc00);
		}
	else
		{

		char *c = new char[size];

		if (c) memset(c,0,size);

		return c;

		}

}
void *duck_calloc(size_t n, size_t size, enum tmemtype fred)
{
	char *c = new char[n * size];

	if(c) memset(c,0,n*size);

	return c;

}
#else
extern "C"{

void duck_free(void *blk)
{
	char *c = (char *)blk;
	delete [] c;
}
void *duck_malloc(size_t size,enum tmemtype fred)
{
	char *c = new char[size];
	if (c)
		memset(c,0,size);
	return c;
}
void *duck_calloc(size_t n, size_t size, enum tmemtype fred)
{
	char *c = new char[size * n];
	if (c)
		memset(c,0,size * n);
	return c;
}
};

#endif

extern "C"{

int duck_open(const char *a)
{
	return NEIL_OPEN_PROC(a);
}
void duck_close(int h)
{
	NEIL_CLOSE_PROC(h);
}
int duck_read(int h,void *buf,long amt)
{
	return NEIL_READ_PROC(h,buf,amt);
}
long duck_seek(int h,long amt,int org)
{
 dprintf("duck_seek (lseek) %d, %d, %d",h, amt, org);
	int r=NEIL_SEEK_PROC(h,amt,org);
 dprintf("returns %d",r);
	return r;
}
};

// NOTE Windows NT can't do unbuffered I/O


void Neil_Init(void)
{
#if ((defined WIN32))
	OSVERSIONINFO	vi;

	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if (!GetVersionEx(&vi) || vi.dwPlatformId == VER_PLATFORM_WIN32_NT)

		{

		gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)normal_open;

		gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)close;

		gNeilFunctionTable[NEIL_READ_INDEX] = (void *)read;

		gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)lseek;

		}

	else

		{

		gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)e_open;
		gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)e_close;
		gNeilFunctionTable[NEIL_READ_INDEX] = (void *)e_read;
		gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)e_seek;
		}

#else
	gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)normal_open;
	gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)close;
	gNeilFunctionTable[NEIL_READ_INDEX] = (void *)read;
	gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)lseek;
#endif
}

void Neil_InstallIOProcs(NEIL_OPEN_PROC_TYPE openproc,NEIL_CLOSE_PROC_TYPE closeproc,
						 NEIL_READ_PROC_TYPE readproc,NEIL_SEEK_PROC_TYPE seekproc)
{
	gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)openproc;
	gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)closeproc;
	gNeilFunctionTable[NEIL_READ_INDEX] = (void *)readproc;
	gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)seekproc;
}

void Neil_DefaultIOProcs(void)
{
#if ((defined WIN32))
	OSVERSIONINFO	vi;

	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if (!GetVersionEx(&vi) || vi.dwPlatformId == VER_PLATFORM_WIN32_NT)

		{

		gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)normal_open;

		gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)close;

		gNeilFunctionTable[NEIL_READ_INDEX] = (void *)read;

		gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)lseek;

		}

	else

		{

		gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)e_open;

		gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)e_close;

		gNeilFunctionTable[NEIL_READ_INDEX] = (void *)e_read;

		gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)e_seek;

		}

#else
	gNeilFunctionTable[NEIL_OPEN_INDEX] = (void *)normal_open;
	gNeilFunctionTable[NEIL_CLOSE_INDEX] = (void *)close;
	gNeilFunctionTable[NEIL_READ_INDEX] = (void *)read;
	gNeilFunctionTable[NEIL_SEEK_INDEX] = (void *)lseek;
#endif
}

void Neil_Exit(void)
{
}

