/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>//
#include <windows.h>
#include <vfw.h>
#include "danbug.h"
#include "acmenc.h"

HACMSTREAM		eAcmStream;	
ACMSTREAMHEADER	eAcmStreamHeader;
unsigned char	eAcmSrcBuffer[ENCODE_MAX_SRC];
int				acmEncodeInitialized=0;
int				gAcmEncodeChunkCount=ENCODE_CHUNK_SIZE;
char			gCompressBuffer[ENCODE_MAX_OUT];

typedef struct tDKWAVEFORM
{
	unsigned short	wFormatTag;        /* format type */
	unsigned short	nChannels;         /* number of channels (i.e. mono, stereo...) */
	unsigned long   nSamplesPerSec;    /* sample rate */
	unsigned long	nAvgBytesPerSec;   /* for buffer estimation */
	unsigned short	nBlockAlign;       /* block size of data */
	unsigned short	wBitsPerSample;    /* Number of bits per sample of mono data */
	unsigned short	cbSize;            /* The count in bytes of the size of
									extra information (after cbSize) */
	unsigned short	wSamplesPerBlock;
	unsigned short	extrashort1;
	unsigned short	extrashort2;		///	dbm -- attempting to support vorbis, which has 8 extra bytes
	unsigned short	extrashort3;

} DKWAVEFORM;

int
InitializeAcmEncode(const char *audiosrc,int srclen, char *audiodest, int destlen,
			int numchan, int samplerate, int &srcused, int &destused)
{
/// returns 0 if OK

	MMRESULT result;
	WAVEFORMATEX	srcformat;
	DKWAVEFORM		destformat;

	srcformat.wFormatTag = WAVE_FORMAT_PCM;								///	=1 btw
	srcformat.nChannels = numchan;
	srcformat.nSamplesPerSec = samplerate;
	srcformat.nBlockAlign = 2 * numchan;
	srcformat.nAvgBytesPerSec = samplerate * srcformat.nBlockAlign;		///	righto
	srcformat.wBitsPerSample = 16;
	srcformat.cbSize = 0;												///	irrelephant?

	destformat.wFormatTag = ACMVORBIS;
	destformat.nChannels = numchan;
	destformat.nSamplesPerSec = samplerate;
	destformat.wBitsPerSample = 16;
	destformat.nBlockAlign = 1;								///2 * destformat.nChannels;
	destformat.nAvgBytesPerSec = 8000;						///	= 128 kbits/sec, 44 khz stereo -- is this filled in?
	destformat.cbSize = 8;									///	filled in?
	destformat.extrashort1 = 0;									///	filled in?
	destformat.extrashort2 = 0;									///	filled in?
	destformat.extrashort3 = 0;									///	filled in?

	result = acmStreamOpen(&eAcmStream, NULL, &srcformat, (WAVEFORMATEX*)&destformat, NULL,NULL,NULL,0);

	if (result) {
		dprintf("I been a bad boy, mommy\n");
		return 1; /// throw DecompressException();
	}
	dprintf("acmStreamOpen gives us this stream handle: %x\n",eAcmStream);
	dprintf("cbSize comes back %d\n",destformat.cbSize);

	eAcmStreamHeader.cbStruct = sizeof(eAcmStreamHeader);
	eAcmStreamHeader.fdwStatus = 0;
	eAcmStreamHeader.dwUser = 0;
	eAcmStreamHeader.pbSrc = eAcmSrcBuffer;					///	ACM requires src in same place all the time!
	eAcmStreamHeader.cbSrcLength = ENCODE_MAX_SRC;
	eAcmStreamHeader.pbDst = (unsigned char *)gCompressBuffer+4;
	eAcmStreamHeader.cbDstLength = ENCODE_MAX_OUT;

	int r=acmStreamPrepareHeader(eAcmStream,&eAcmStreamHeader,0);				///	gotta do whatcha gotta do
 dprintf("after prep, fdwStatus = %x\n",eAcmStreamHeader.fdwStatus);
 dprintf("cbSize comes back %d\n",destformat.cbSize);
 dprintf("InitializeAcmEncode -- acmStreamPrepareHeader returns %d\n",r);
 if(r) dprintf("error codes: invalf=%d, invalh=%d, invalp=%d, nomem=%d, notposs=%d\n",MMSYSERR_INVALFLAG,MMSYSERR_INVALHANDLE,MMSYSERR_INVALPARAM,MMSYSERR_NOMEM,ACMERR_NOTPOSSIBLE);
	if(r) return 2; /// if(r) throw DecompressException();

	if(srclen>ENCODE_MAX_SRC) { 
		dprintf("badd!!!  too much source\n");
		return 3;
	}
	for(int x=0; x<srclen; x++) {
		eAcmSrcBuffer[x] = audiosrc[x];
	}
 dprintf("copied %d bytes into src buffer\n",srclen);
	eAcmStreamHeader.cbSrcLength = srclen;						/// should be in!!!
	r=acmStreamConvert(eAcmStream, &eAcmStreamHeader, ACM_STREAMCONVERTF_BLOCKALIGN | ACM_STREAMCONVERTF_START);
	if(r) return 4; /// if(r) throw DecompressException();

 dprintf("----------acmStreamConvert returns %d; src used=%d, dest used=%d\n\n",
			r,eAcmStreamHeader.cbSrcLengthUsed,eAcmStreamHeader.cbDstLengthUsed);
 dprintf("  compressed data encoded:");
 dumpdata((unsigned char*)gCompressBuffer+4,3);
	srcused = eAcmStreamHeader.cbSrcLengthUsed;
	destused = eAcmStreamHeader.cbDstLengthUsed;
 dprintf("InitializeAcmEncode done\n");
 dprintf("destformat extra 1=%x, 2=%x, 3=%x\n",destformat.extrashort1,destformat.extrashort2,destformat.extrashort3);

///	copy from prepped buffer to requested output buf:
	if(audiodest) {
		for(x=0; x<destused; x++) {
			audiodest[x] = gCompressBuffer[x+4];
		}
	}
	*((int*)gCompressBuffer) = destused;										///	for CBR, buffer needs a count
	destused += 4;															///	factor in the count itself
	return 0;
}

void
CleanupAcmEncode(void)
{
	acmStreamUnprepareHeader(eAcmStream,&eAcmStreamHeader,0);				///	gotta do whatcha gotta do
	acmStreamClose(eAcmStream, 0);
	acmEncodeInitialized=0;
	gAcmEncodeChunkCount=ENCODE_CHUNK_SIZE;
}

int
AcmCompress(const char *audiosrc, int srclen, char *audiodest, int destlen, 
			int numchan, int samplerate, int &srcused, int &destused)
{
	if(!acmEncodeInitialized) {
		acmEncodeInitialized++;
		InitializeAcmEncode(audiosrc, srclen, audiodest, destlen, numchan, samplerate, srcused, destused);
		return 0;
	}
	if(srclen>ENCODE_MAX_SRC) { 
		dprintf("badd!!!  too much source\n");
		return 1;
	}
	for(int x=0; x<srclen; x++) {
		eAcmSrcBuffer[x] = audiosrc[x];
	}
	dprintf("copied %d bytes into src buffer\n",srclen);
	eAcmStreamHeader.cbSrcLength = srclen;						/// should be in!!!
	int r=acmStreamConvert(eAcmStream, &eAcmStreamHeader, ACM_STREAMCONVERTF_BLOCKALIGN);
	if(r) return 2; /// if(r) throw DecompressException();

	dprintf("----------acmStreamConvert returns %d; src used=%d, dest used=%d\n\n",
			r,eAcmStreamHeader.cbSrcLengthUsed,eAcmStreamHeader.cbDstLengthUsed);
 dprintf("  -compressed data encoded:");
 dumpdata((unsigned char*)gCompressBuffer+4,3);
	srcused = eAcmStreamHeader.cbSrcLengthUsed;
	destused = eAcmStreamHeader.cbDstLengthUsed;
	dprintf("AcmCompress done\n");

///	copy from prepped buffer to requested output buf:
	if(audiodest) {
		for(x=0; x<destused; x++) {
			audiodest[x] = gCompressBuffer[x+4];
		}
	}
	*((int*)gCompressBuffer) = destused;										///	for CBR, buffer needs a count
	destused += 4;															///	factor in the count itself
	return 0;
}

char* AcmGetBuffer() {
	return gCompressBuffer;
}