/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CActiveMovieStat.h>
#include	<CTCPSocket.h>

CActiveMovieStat::CActiveMovieStat(void)
{
}

CActiveMovieStat::CActiveMovieStat(U32 inet_addr,nmr_string filename,
				U32 time_active,U32 thread_id,U16 mode)
{
	mInetAddr 	= inet_addr;
	mTimeActive = time_active;
	mThreadID 	= thread_id;
	mMode 		= mode;
	mFilename 	= filename;
}

CActiveMovieStat::CActiveMovieStat(const CActiveMovieStat &am)
{
	mInetAddr	= am.mInetAddr;
	mTimeActive	= am.mTimeActive;
	mThreadID	= am.mThreadID;
	mMode		= am.mMode;
	mFilename	= am.mFilename;
}

CActiveMovieStat&
CActiveMovieStat::operator=(const CActiveMovieStat &am)
{
	mInetAddr	= am.mInetAddr;
	mTimeActive	= am.mTimeActive;
	mThreadID	= am.mThreadID;
	mMode		= am.mMode;
	mFilename	= am.mFilename;
	return (*this);
}

int	
CActiveMovieStat::operator==(const CActiveMovieStat& s) const
{
	return (mThreadID == s.mThreadID);
}

int 
CActiveMovieStat::operator!=(const CActiveMovieStat &s ) const
{
	return (mThreadID != s.mThreadID);
}

int 
CActiveMovieStat::operator<(const CActiveMovieStat &s ) const
{
	return (mThreadID < s.mThreadID);
}

int 
CActiveMovieStat::operator>(const CActiveMovieStat &s ) const
{
	return (mThreadID > s.mThreadID);
}


void
CActiveMovieStat::Write(CTCPSocket &socket)
{
	socket << mInetAddr << mTimeActive << mThreadID
			<< mMode << mFilename;
}

void
CActiveMovieStat::Read(CTCPSocket &socket)
{
	socket >> mInetAddr >> mTimeActive >> mThreadID
			>> mMode >> mFilename;
}

CTCPSocket& operator<<(CTCPSocket &socket,CActiveMovieStat &am)
{
	am.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CActiveMovieStat &am)
{
	am.Read(socket);
	return socket;
}

size_t
CActiveMovieStat::CalculateSize(void)
{
	return (sizeof(u_long) * 3) + (sizeof(u_short) * 2) + 
		mFilename.length();
}

const char *
CActiveMovieStat::Name(void)
{
	return "CActiveMovieStat";
}

void
CActiveMovieStat::Dump(ostream &o,const char *indent)
{
	o << indent << "CActiveMovieStat: " << endl;
	o << indent << "   mInetAddr: " << ((mInetAddr >> 24) & 0xff) << "."
		<< ((mInetAddr >> 16) & 0xff) << "."
		<< ((mInetAddr >> 8) & 0xff) << "."
		<< ((mInetAddr) & 0xff) << endl;
		
	o << indent << "   mFilename: " << mFilename << endl;
	o << indent << "   mTimeActive: " << mTimeActive << endl;
	o << indent << "   mThreadID: " << mThreadID << endl;
	o << indent << "   mMode: " << mMode << endl;
}

ostream&
operator<<(ostream &o,CActiveMovieStat &s)
{ 
	s.Dump(o,"");
	return o;
}

ostream&
operator<<(ostream &o,CActiveMovieStat *s)
{ 
	if (s)
		s->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}


U32	
CActiveMovieStat::GetInetAddr(void)
{
	return mInetAddr;
}

nmr_string 
CActiveMovieStat::GetFilename(void)
{
	return mFilename;
}

U32 
CActiveMovieStat::GetTimeActive(void)
{
	return mTimeActive;
}

U32	
CActiveMovieStat::GetThreadID(void)
{
	return mThreadID;
}

U16	
CActiveMovieStat::GetMode(void)
{
	return mMode;
}
