/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CAudioInfo.h>
#ifdef NETHFB
#include	<CTCPSocket.h>
#include	<UDPPacketDefinition.h>
#endif

#ifndef USE_VOXWARE
#define	USE_VOXWARE	1
#endif

#include "danbug.h"

CAudioInfo::CAudioInfo(void)
{
#ifdef NETHFB
	assert(sizeof(UDPAudioInfoBlock_t) == 60);
#endif
}

CAudioInfo::~CAudioInfo(void)
{
}

double
CAudioInfo::GetTimingFactor(void)
{
	if (mWaveInfo.wFormatTag != 1)
		{
		// compressed. We deal in samples per second
		return (double)mWaveInfo.nSamplesPerSec;
		}
	else
		{
		// uncompressed we deal in bytes per second
		return (double)mBytesPerSec;
		}
}

nmr_fraction<I32>
CAudioInfo::GetTimingFactorAsFraction(void)
{
	if (mWaveInfo.wFormatTag != 1)
		{
		// compressed. We deal in samples per second
		return nmr_fraction<I32>(mWaveInfo.nSamplesPerSec);
		}
	else
		{
		// uncompressed we deal in bytes per second
		return nmr_fraction<I32>(mBytesPerSec);
		}
}

double
CAudioInfo::ChunkTimeSize(long len)
{
	double	len2 = (double)len;

	if (mWaveInfo.wFormatTag != 1)
		{
		// compressed, we deal in samples per second
		return (len2 * ((double)mWaveInfo.wSamplesPerBlock)) / 
			((double)mBytesPerBlock);
		}
	else
		{
		// uncompressed we deal in bytes per second
		return len2;
		}
}

unsigned long
CAudioInfo::GetFCCHandler(void)
{
	return mFCCHandler;
}

///	dbm -- hard-code for Vorbis CBR for now (2 bits)
int
CAudioInfo::DataPosToBytes(int data_pos)
{
	int result = 0;
	switch(mWaveInfo.wBitsPerSample)
		{
		case 4:
		case 3:
			result =(int)( (((U64)data_pos) * ((U64)mBytesPerBlock)) / ((U64)mWaveInfo.wSamplesPerBlock));
			break;
		default:
			if (mWaveInfo.wFormatTag == VOXWARE_FORMAT_TAG && mWaveInfo.wBitsPerSample == 16)
				result = (int)((((U64)data_pos) * ((U64)mBytesPerBlock)) / ((U64)mWaveInfo.wSamplesPerBlock));
			else
				result = data_pos * (mWaveInfo.wBitsPerSample / 8);
			break;
		}

	result = data_pos/4;							///	each byte = 4 samples (easy!)
 dprintf(">>>CAudioInfo::DataPosToBytes called with %d, returns %d",data_pos,result);
	return result;
}

void
CAudioInfo::ExtractFromStream(HFB_STREAM_HANDLE primary_audio,
									HFB_STREAM_HANDLE strm,int s_num)
{
	unsigned long fcc;
	HFB_GetStreamFCCs(strm,NULL,&mFCCHandler);
 dprintf("==calling MFP_GetAudioInfo.  BTW, USE_VOXWARE = %d\n",USE_VOXWARE);
	DKWAVEFORM	*wv = MFP_GetAudioInfo(strm,&(mNumChannels),
				&(mSamplesPerSec),&(mBytesPerSec),
				&(mFormat));
 dprintf("==done calling MFP_GetAudioInfo\n");

	mBytesPerBlock = HFB_GetStreamSampleSize(strm); //strm->parent->sampleSize;
 dprintf("==CAudioInfo::ExtractFromStream A\n");

	if (wv)
		{
		mWaveInfo.wFormatTag 		= wv->wFormatTag;
		mWaveInfo.nChannels			= wv->nChannels;
		mWaveInfo.nSamplesPerSec 	= wv->nSamplesPerSec;
		mWaveInfo.nAvgBytesPerSec 	= wv->nAvgBytesPerSec;
		mWaveInfo.nBlockAlign 		= wv->nBlockAlign;
		mWaveInfo.wBitsPerSample 	= wv->wBitsPerSample;
		mWaveInfo.cbSize 			= wv->cbSize;
		mWaveInfo.wSamplesPerBlock 	= wv->wSamplesPerBlock;
		}

	// special adjustment for Voxware
#ifdef	USE_VOXWARE
	if (mFormat == VOXWARE_FORMAT_TAG && wv->wBitsPerSample == 16)
		wv->wSamplesPerBlock = mWaveInfo.wSamplesPerBlock = 512;
#endif
 dprintf("==exiting CAudioInfo::ExtractFromStream\n");
}

#ifdef NETHFB
void	
CAudioInfo::FillInUDPData(UDPAudioInfoBlock_t *block)
{
	block->mType 			= htons(AUDIOINFOBLOCK_TYPE);
	block->mNumChannels 	= htonl(mNumChannels);
	block->mSamplesPerSec 	= htonl(mSamplesPerSec);
	block->mBytesPerSec 	= htonl(mBytesPerSec);
	block->mFormat			= htonl(mFormat);
	block->mBytesPerBlock 	= htonl(mBytesPerBlock);
	block->wi_mFormatTag 		= htonl(mWaveInfo.wFormatTag);
	block->wi_mChannels 		= htonl(mWaveInfo.nChannels);
	block->wi_mSamplesPerSec 	= htonl(mWaveInfo.nSamplesPerSec);
	block->wi_mAvgBytesPerSec 	= htonl(mWaveInfo.nAvgBytesPerSec);
	block->wi_mBlockAlign 		= htonl(mWaveInfo.nBlockAlign);
	block->wi_mBitsPerSample 	= htonl(mWaveInfo.wBitsPerSample);
	block->wi_mCbSize			= htonl(mWaveInfo.cbSize);
	block->wi_mSamplesPerBlock 	= htonl(mWaveInfo.wSamplesPerBlock);
	block->mFCCHandler			= htonl(mFCCHandler);
}

CAudioInfo *
CAudioInfo::CreateFromUDPData(const UDPAudioInfoBlock_t *block,int siz)
{
	if (siz < sizeof(UDPAudioInfoBlock_t))
		return NULL;
	if (ntohs(block->mType) != AUDIOINFOBLOCK_TYPE)
		return NULL;

	CAudioInfo	*info = new CAudioInfo;

	info->mNumChannels	= ntohl(block->mNumChannels);
	info->mSamplesPerSec= ntohl(block->mSamplesPerSec);
	info->mBytesPerSec	= ntohl(block->mBytesPerSec);
	info->mFormat		= ntohl(block->mFormat);
	info->mBytesPerBlock= ntohl(block->mBytesPerBlock);
	info->mWaveInfo.wFormatTag		= ntohl(block->wi_mFormatTag);
	info->mWaveInfo.nChannels		= ntohl(block->wi_mChannels);
	info->mWaveInfo.nSamplesPerSec	= ntohl(block->wi_mSamplesPerSec);
	info->mWaveInfo.nAvgBytesPerSec	= ntohl(block->wi_mAvgBytesPerSec);
	info->mWaveInfo.nBlockAlign		= ntohl(block->wi_mBlockAlign);
	info->mWaveInfo.wBitsPerSample	= ntohl(block->wi_mBitsPerSample);
	info->mWaveInfo.cbSize			= ntohl(block->wi_mCbSize);
	info->mWaveInfo.wSamplesPerBlock= ntohl(block->wi_mSamplesPerBlock);
	info->mFCCHandler				= ntohl(block->mFCCHandler);
	return info;
}


void
CAudioInfo::Read(CTCPSocket &socket)
{
	u_long	data[14];

	socket.Read((char *)data,sizeof(data));

	mNumChannels 	= ntohl(data[0]);
	mSamplesPerSec 	= ntohl(data[1]);
	mBytesPerSec 	= ntohl(data[2]);
	mFormat 		= ntohl(data[3]);
	mBytesPerBlock	= ntohl(data[4]);
	//////

	mWaveInfo.wFormatTag 		= ntohl(data[5]);
	mWaveInfo.nChannels 		= ntohl(data[6]);
	mWaveInfo.nSamplesPerSec	= ntohl(data[7]);
	mWaveInfo.nAvgBytesPerSec	= ntohl(data[8]);
	mWaveInfo.nBlockAlign 		= ntohl(data[9]);
	mWaveInfo.wBitsPerSample	= ntohl(data[10]);
	mWaveInfo.cbSize 			= ntohl(data[11]);
	mWaveInfo.wSamplesPerBlock	= ntohl(data[12]);
	mFCCHandler					= ntohl(data[13]);
}

size_t
CAudioInfo::CalculateSize(void)
{
	return (sizeof(u_long) * 13);
}

void
CAudioInfo::Write(CTCPSocket &socket)
{
	u_long	data[14];

	data[0] = htonl(mNumChannels);
	data[1] = htonl(mSamplesPerSec);
	data[2] = htonl(mBytesPerSec);
	data[3] = htonl(mFormat);
	data[4] = htonl(mBytesPerBlock);
	//////

	data[5] = htonl(mWaveInfo.wFormatTag);
	data[6] = htonl(mWaveInfo.nChannels);
	data[7] = htonl(mWaveInfo.nSamplesPerSec);
	data[8] = htonl(mWaveInfo.nAvgBytesPerSec);
	data[9] = htonl(mWaveInfo.nBlockAlign);
	data[10] = htonl(mWaveInfo.wBitsPerSample);
	data[11] = htonl(mWaveInfo.cbSize);
	data[12] = htonl(mWaveInfo.wSamplesPerBlock);
	data[13] = htonl(mFCCHandler);

	socket.Write((char *)data,sizeof(data));
}
#endif

const char *
CAudioInfo::Name(void)
{
	return "CAudioInfo";
}

void
CAudioInfo::Dump(ostream& o,const char *indent)
{
	o << indent << Name() << ": " << endl;
	o << indent <<	"   mNumChannels: " << mNumChannels << endl;
	o << indent <<	"   mSamplesPerSec: " << mSamplesPerSec << endl;
	o << indent <<	"   mBytesPerSec: " << mBytesPerSec << endl;
	o << indent <<	"   mFormat: " << "mFormat" << endl;
	o << indent <<	"   mBytesPerBlock: " << mBytesPerBlock << endl;
	o << indent <<	"   mWaveInfo.wFormatTag: " << 
									mWaveInfo.wFormatTag << endl;
	o << indent <<	"   mWaveInfo.nChannels: " << 
									mWaveInfo.nChannels << endl;
	o << indent <<	"   mWaveInfo.nSamplesPerSec: " << 
									mWaveInfo.nSamplesPerSec << endl;
	o << indent <<	"   mWaveInfo.nAvgBytesPerSec: " << 
									mWaveInfo.nAvgBytesPerSec << endl;
	o << indent <<	"   mWaveInfo.nBlockAlign: " << 
									mWaveInfo.nBlockAlign << endl;
	o << indent <<	"   mWaveInfo.wBitsPerSample: " << 
									mWaveInfo.wBitsPerSample << endl;
	o << indent <<	"   mWaveInfo.cbSize: " << 
									mWaveInfo.cbSize << endl;
	o << indent <<	"   mWaveInfo.wSamplesPerBlock: " <<
									mWaveInfo.wSamplesPerBlock << endl;
	o << indent <<	"   mFCCHandler: " << 
									mFCCHandler << endl;
}

ostream& operator<<(ostream&o,CAudioInfo *ai)
{
	if (ai)
		ai->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

ostream& operator<<(ostream&o,CAudioInfo &ai)
{
	ai.Dump(o,"");
	return o;
}

#ifdef NETHFB
CTCPSocket& operator<<(CTCPSocket &socket,CAudioInfo &ai)
{
	ai.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CAudioInfo &ai)
{
	ai.Read(socket);
	return socket;
}

size_t CalculateTCPSize(CAudioInfo &ai)
{
	return ai.CalculateSize();
}
#endif



