/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CCIRCULARBUFFER_H_
#define CCIRCULARBUFFER_H_

// HFB style circular buffer simulation for network client

#include	<CCMutex.h>
#include	<CConditionVariable.h>

#if ((defined IRIX) || (defined SOLARIS))
#define	ALIGN_FACTOR	4	// long word align
#endif

class CCircularBuffer
{
	CLASS_NOCOPY(CCircularBuffer)

protected:

	typedef struct 
	{
		int	 mBusy;
		char *mPtr;
		long mLen;
#ifdef	ALIGN_FACTOR
		long mReportedLength;
#endif
	}CB_Index;

	char	*mMemoryPool;
	char	*mEndOfPool;
	char	*mDynamicEndOfPool;

	int		mMinIndex;
	int		mMaxIndex;
	int		mCurrentReadIndex;
	int		mCurrentWriteIndex;
	CB_Index	*mIndicies;

	char	*mWritePtr;
	char	*mBlockPtr;

	CCMutex	mMutex;
	CConditionVariable	mWaitingToWrite;
	CConditionVariable	mWaitingToRead;

	int		mIgnoreOverflow;
	int		mChunksInBuffer;

	//int		mFirstAccess;

public:
	CCircularBuffer(size_t pool_size,int num_indicies);
	~CCircularBuffer(void);

	char *WaitForWriteSpace(size_t siz);

	void	AdvanceWriteMark(size_t siz);

	int	FetchIndex(int direction,int val);

	void	GetDataFromIndex(int index,char **data,long *len);
	void	FreeIndex(int index);

	void	Clear(void);

	int		ReadyToRoll(long percent);
	int		GetPercentFull(void);
	long	GetBytesInBuffer(void);
	int		GetChunksInBuffer(void);

	void	SetOverflowIgnore(int val);

};


#endif
