/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CConfigEntry.h>
#include	<CTCPSocket.h>
#include	<stack_aids.h>

CConfigEntry::CConfigEntry(void)
{
}

CConfigEntry::CConfigEntry(unsigned long server_address,int server_port)
{
	mServerAddress	= server_address;
	mServerPort 	= server_port;
}

CConfigEntry::CConfigEntry(const CConfigEntry &ce)
{
	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS; ++x)
		mFields[x] = ce.mFields[x];
	mServerAddress = ce.mServerAddress;
	mServerPort = ce.mServerPort;
}

CConfigEntry::~CConfigEntry(void)
{
}

void
CConfigEntry::SetAddressAndPort(unsigned long addr,int port)
{
	mServerAddress = addr;
	mServerPort = port;
}

nmr_string
CConfigEntry::GetField(ConfigField which) const
{
	return mFields[which];
}

void
CConfigEntry::SetField(ConfigField which,nmr_string &d)
{
	mFields[which] = d;
}

int
CConfigEntry::IntegrityCheck(nmr_string &rootpath) const
{
	nmr_string	buf;
	ifstream	f;

	// display name
	if (mFields[eDisplayName][0] == '\0') return 0;

	// path
	if (mFields[ePathName][0] != '<') // not vidcap
		{
		buf = rootpath + mFields[ePathName];
		f.open(buf);
		if (!f)	return 0;
		f.close();	
		}

	// event file
	if (mFields[eEventFile][0] == '\0') return 0;
	if (mFields[eEventFile][0] != '@')
		{
		buf = rootpath + mFields[eEventFile];
		f.open(buf);
		if (!f) return 0;
		f.close();
		}

	if (mFields[eMulticastAddress][0] == '\0') return 0;
	if (mFields[eMulticastAddress].length() > 15) return 0;
	if (mFields[eMulticastAddress][0] != '@')
		if (SOCKET_inet_addr(mFields[eMulticastAddress]) == -1) return 0;

	if (SOCKET_inet_addr(mFields[eServerAddress]) == -1) return 0;

	if (!isdigit(mFields[eTTL][0]) && !mFields[eTTL][0] == '@') return 0;
	return 1;
}


int
CConfigEntry::LastReadOk(void) const
{
	return (mFields[0][0] != '\0');
}

void	
CConfigEntry::Read(CTCPSocket &s)
{
	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS; ++x)
		s >> mFields[x];
}

void		
CConfigEntry::Write(CTCPSocket &s)
{
	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS; ++x)
		s << mFields[x];
}

CTCPSocket& operator<<(CTCPSocket &socket,CConfigEntry &ce)
{
	ce.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CConfigEntry &ce)
{
	ce.Read(socket);
	return socket;
}

size_t	
CConfigEntry::CalculateSize(void) const
{
	size_t	siz = 0;

	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS; ++x)
		siz += mFields[x].length() + 1;

	return siz;
}

const char *
CConfigEntry::Name(void) const
{
	return "CConfigEntry";
}

void
CConfigEntry::Dump(ostream &o,const char *indent)
{
	o << indent << "CConfigEntry: " << endl;
	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS; ++x)
		{
		nmr_string	nmr;
		switch(x)
			{
			case eDisplayName:
				nmr = "eDisplayName";
				break;
			case ePathName:
				nmr = "ePathName";
				break;
			case eEventFile:
				nmr = "eEventFile";
				break;
			case eMulticastAddress:
				nmr = "eMulticastAddress";
				break;
			case eTTL:
				nmr = "eTTL";
				break;
			case eServerAddress:
				nmr = "eServerAddress";
				break;
			case eServerPort:
				nmr = "eServerPort";
				break;
			}
		o << indent << "   " << nmr << " : " << mFields[x] << endl;
		}
}

ostream&	
operator<<(ostream &o,CConfigEntry &ce)
{
	ce.Dump(o,"");
	return o;
}

ostream&	
operator<<(ostream &o,CConfigEntry *ce)
{
	if (ce)
		ce->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

istream&	
operator>>(istream &i,CConfigEntry &ce)
{
	char	buf[2048];
	char	*ptr,*placeholder;

	ce.mFields[0] = "";
	do 
		{
		i.getline(buf,2047,'\n');
		}
	while (i && (buf[0] == '#' || !buf[0]));
	if (!i) return i;
	
	ptr = strtok_r(buf," \t\n",&placeholder);
	for (int x = 0; x < CCONFIGENTRY_NUMFIELDS && ptr; ++x)
		{
		ce.mFields[x] = ptr;
		ptr = strtok_r(NULL," \t\n",&placeholder);
		}

	if (ce.mFields[CConfigEntry::eTTL][0] == '@')
		{
		ce.mFields[CConfigEntry::eTTL] = "1";
		}

	if (ce.mFields[CConfigEntry::eServerAddress][0] == '@')
		{
		char sbuf[2048];
		sprintf(sbuf,"%d.%d.%d.%d",
			(ce.mServerAddress >> 24) & 0xff,
			(ce.mServerAddress >> 16) & 0xff,
			(ce.mServerAddress >> 8) & 0xff,
			(ce.mServerAddress & 0xff));

		ce.mFields[CConfigEntry::eServerAddress] = sbuf;
		}

	if (ce.mFields[CConfigEntry::eServerPort][0] == '@')
		{
		char sbuf[2048];
		sprintf(sbuf,"%d",ce.mServerPort);	
		ce.mFields[CConfigEntry::eServerPort] = sbuf;
		}

	return i;
}
