/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CConfiguration.h>
#include	<CTCPSocket.h>
#include	<CLogger.h>
#include	<safe_ostrstream.h>

CConfiguration::CConfiguration(void)
{
	mServerAddress = 0;
	mServerPort = 0;
}

CConfiguration::CConfiguration(unsigned long server_address,int server_port)
{
	mServerAddress 	= server_address;
	mServerPort		= server_port;
}

CConfiguration::CConfiguration(const CConfiguration &c)
	:mList(c.mList)
{
	mServerAddress 	= c.mServerAddress;
	mServerPort		= c.mServerPort;
}

CConfiguration::~CConfiguration(void)
{
}

void
CConfiguration::SetAddressAndPort(unsigned long addr,int port)
{
	mServerAddress = addr;
	mServerPort = port;
}

void
CConfiguration::Clear(void)
{
	ClearList();
}

int
CConfiguration::NumEntries(void)
{
	return mList.size();
}

void
CConfiguration::ClearList(void)
{
	int	siz = mList.size();

	for (int x = 0; x < siz; ++x)
		mList.pop_back();
}

////////////////////////////////////////////////////////////////////////
// DeleteEntryByField
//
// Delete an entry from the configuration based on the field type and
// data. Return true if an entry was actually deleted
//
// INPUT:
//	f		- Field type
//	data	- data for match
//
// OUTPUT:
//	1 if entry was deleted else 0.
///////////////////////////////////////////////////////////////////////
int		
CConfiguration::DeleteEntryByField(CConfigEntry::ConfigField f,
															nmr_string data)
{
	int result	 = 0;
	CEList_t::iterator	i;
	for (i = mList.begin(); i != mList.end(); ++i)
		{
		if ((*i).GetField(f) == data)
			{
			result = 1;
			mList.erase(i);
			break;
			}
		}

	return result;
}


const CConfigEntry *
CConfiguration::GetNextEntry(int x)
{
	if (x)
		++mGetNextIterator;
	else
		mGetNextIterator = mList.begin();
	
	if (mGetNextIterator == mList.end())
		return NULL;

	return &(*mGetNextIterator);
}

void
CConfiguration::Write(CTCPSocket &socket)
{
	U16 siz = (u_short)mList.size();

	socket << siz;
	
	CEList_t::iterator i;

	for (i = mList.begin(); i != mList.end(); ++i)
		(*i).Write(socket);
}

void	
CConfiguration::Read(CTCPSocket &socket)
{
	U16	siz;

	socket >> siz;

	ClearList();

	for (int x = 0; x < siz; ++x)
		{
		CConfigEntry	c(mServerAddress,mServerPort);
		c.Read(socket);
		mList.push_back(c);
		}
}

CTCPSocket& operator<<(CTCPSocket &socket,CConfiguration &c)
{
	c.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CConfiguration &c)
{
	c.Read(socket);
	return socket;
}

size_t CalculateTCPSize(CConfiguration &c)
{
	return c.CalculateSize();
}

size_t	
CConfiguration::CalculateSize(void)
{
	size_t	siz = 0;
	CEList_t::iterator i;

	for (i = mList.begin(); i != mList.end(); ++i)
		siz += (*i).CalculateSize();

	return siz;
}

const char *
CConfiguration::Name(void) const
{
	return "CConfiguration";
}

void
CConfiguration::Dump(ostream &o,const char *indent)
{
	o << indent << "CConfiguration: " << endl;

	CEList_t::iterator i;
	for (i = mList.begin(); i != mList.end(); ++i)
		o << indent << "   " << (*i) << endl;
}


ostream&
operator<<(ostream &o,CConfiguration &c)
{
	c.Dump(o,"");
	return o;
}

ostream&
operator<<(ostream &o,CConfiguration *c)
{
	if (c)
		c->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

istream&
operator>>(istream &i,CConfiguration &c)
{
	CConfigEntry	ce(c.mServerAddress,c.mServerPort);
	int	version;

	i >> version;

	while (i)
		{
		i >> ce;
		if (ce.LastReadOk())
			c.mList.push_back(ce);
		else
			break;
		}
	return i;
}

int
CConfiguration::IntegrityCheck(nmr_string rootpath)
{
	CEList_t::iterator	i;

	for (i = mList.begin(); i != mList.end(); ++i)
		if (!(*i).IntegrityCheck(rootpath)) 
			{
			safe_ostrstream	o;
			o << "Error in config file " << (*i);
			gErrorLog->SendMessage(o.str());
			return 0;
			}

	return 1;
}



