/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CCONFIGURATION_H_
#define CCONFIGURATION_H_

#include	<MegaHeaders.h>
#include	<list.h>
#include	<CConfigEntry.h>
#include	<nmr_string.h>

#ifdef	NEEDS_STL_ALLOCATOR
typedef list<CConfigEntry, allocator<CConfigEntry> > CEList_t;
#else
typedef list<CConfigEntry>	CEList_t;
#endif

class CTCPSocket;

class CConfiguration
{
protected:
	CEList_t::iterator	mGetNextIterator;
	CEList_t			mList;

	unsigned long		mServerAddress;
	int					mServerPort;

	void	ClearList(void);
	
public:
				CConfiguration(void);
				CConfiguration(unsigned long server_address,int server_port);
				CConfiguration(const CConfiguration &c);
	virtual 	~CConfiguration(void);

	void					SetAddressAndPort(unsigned long addr,int port);
	void					Write(CTCPSocket &socket);
	void					Read(CTCPSocket &socket);
	size_t					CalculateSize(void);
	const CConfigEntry *	GetNextEntry(int x);
	int						NumEntries(void);
	void					Clear(void);
	int						DeleteEntryByField(CConfigEntry::ConfigField,
																nmr_string);

	int	IntegrityCheck(nmr_string rootpath);

	virtual const char *Name(void) const;
	virtual	void		Dump(ostream &o,const char *indent);

	friend ostream &operator<<(ostream &o,CConfiguration &c);
	friend ostream &operator<<(ostream &o,CConfiguration *c);
	friend istream &operator>>(istream &o,CConfiguration &c);
};

CTCPSocket& operator<<(CTCPSocket &socket,CConfiguration &c);
CTCPSocket& operator>>(CTCPSocket &socket,CConfiguration &c);
size_t	CalculateTCPSize(CConfiguration &c);

#endif
