/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CEventEntry.h>
#include	<CTCPSocket.h>
#include	<CUDPSocket.h>
#include	<CLogger.h>
#include	<stack_aids.h>

CEventEntry::CEventEntry(void)
{
	assert(sizeof(UDPEventEntryBlock_t) == 520);
	mFrame = 0;
}

CEventEntry::CEventEntry(I32 frame,nmr_string action,nmr_string target)
{
	assert(sizeof(UDPEventEntryBlock_t) == 520);
	mFrame = frame;

	SetAction(action);
	SetTarget(target);
}

CEventEntry::CEventEntry(const CEventEntry &ee)
{
	assert(sizeof(UDPEventEntryBlock_t) == 520);
	mFrame = ee.mFrame;

	SetAction(ee.mAction);
	SetTarget(ee.mTarget);
}

CEventEntry::~CEventEntry(void)
{
}

nmr_string
CEventEntry::GetAction(void) const
{
	return mAction;
}

void
CEventEntry::SetAction(nmr_string d)
{
	mAction = d;
}

nmr_string
CEventEntry::GetTarget(void) const
{
	return mTarget;
}

void
CEventEntry::SetTarget(nmr_string t)
{
	mTarget = t;
}

I32
CEventEntry::GetFrame(void) const
{
	return mFrame;
}

void
CEventEntry::SetFrame(I32 f)
{
	mFrame = f;
}

int
CEventEntry::IntegrityCheck(void) const
{
	return (mAction[0] != '\0' && mFrame >= 0 && mTarget[0] != '\0');
}

const char *
CEventEntry::Name(void)
{
	return "CEventEntry";
}

void
CEventEntry::Dump(ostream &o,const char *indent)
{
	o << indent << "CEventEntry: " << endl;
	o << indent << "   mFrame: " << mFrame << endl;
	o << indent << "   mAction: " << mAction << endl;
	o << indent << "   mTarget: " << mTarget << endl;
}


ostream&	
operator<<(ostream &o,CEventEntry &ee)
{
	ee.Dump(o,"");
	return o;
}

ostream&	
operator<<(ostream &o,CEventEntry *ee)
{
	if (ee)
		ee->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

istream&	
operator>>(istream &i,CEventEntry &ee)
{
	char	buf[2048];
	char	*ptr,*placeholder;

	ee.mAction = "";
	ee.mTarget = "";
	ee.mFrame = 0;
	do 
		{
		i.getline(buf,2047,'\n');
		}
	while (i && (buf[0] == '#' || !buf[0]));
	if (!i) return i;
	
	ptr = strtok_r(buf,"`\n",&placeholder);
	if (ptr) sscanf(ptr,"%d",&ee.mFrame);

	ptr = strtok_r(NULL,"`\n",&placeholder);
	if (ptr) 
		{
		ee.mAction = ptr;
		ptr = strtok_r(NULL,"`\n",&placeholder);
		}
	if (ptr)
		{
		ptr = strtok_r(NULL,"`\n",&placeholder);
		if (ptr) 
			{
			ee.mTarget = ptr;
			}
		}

	return i;
}

int
CEventEntry::LastReadOk(void) const
{
	return (mAction[0] != '\0' && mTarget[0] != '\0');
}

void	
CEventEntry::Read(CTCPSocket &s)
{
	s >> mFrame >> mAction >> mTarget;
}

void		
CEventEntry::Write(CTCPSocket &s) const
{
	s << mFrame << mAction << mTarget;
}

CTCPSocket& operator<<(CTCPSocket &socket,CEventEntry &ee)
{
	ee.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CEventEntry &ee)
{
	ee.Read(socket);
	return socket;
}

size_t CalculateTCPSize(CEventEntry &ee)
{
	return ee.CalculateSize();
}

size_t	
CEventEntry::CalculateSize(void) const
{
	return (sizeof(u_long) + (sizeof(u_short) * 2) + mAction.length()
			+ mTarget.length());
}



void	
CEventEntry::FillInUDPData(UDPEventEntryBlock_t *block)
{
	block->mType = htons(ERRORBLOCK_TYPE);

	if (!mAction || !mTarget)
		{
		gErrorLog->SendMessage("Event has null data");
		return;
		}
	int	action_len = mAction.length();
	int target_len = mTarget.length();
	if (action_len > 255 || target_len > 255)
		{
		gErrorLog->
		SendMessage("Event Entry in multicast is greater than 255 chars");
		return; 
		}

	block->mType 	= htons(EVENTBLOCK_TYPE);
	block->mFrame 	= htonl(mFrame);
	char *ptr = block->mData;

	memcpy(ptr,mAction.data(),action_len+1);
	ptr += action_len+1;

	memcpy(ptr,mTarget.data(),target_len+1);
	ptr += target_len+1;
}

void	
CEventEntry::UDPTransmit(CUDPSocket *socket)
{
	UDPEventEntryBlock_t	block;

	FillInUDPData(&block);
	socket->SendTo((const char *)&block,sizeof(block));
}

CEventEntry *
CEventEntry::CreateFromUDPData(const UDPEventEntryBlock_t *block,int siz)
{
	if (ntohs(block->mType) != EVENTBLOCK_TYPE)
		return NULL;
	if (siz < sizeof(UDPEventEntryBlock_t))
		return NULL;

	int	frame = ntohl(block->mFrame);
	const char *ptr = block->mData;
	int action_len = strlen(ptr);
	const char *action = ptr;
	ptr += action_len + 1;
	int target_len = strlen(ptr);			
	const char *target = ptr;

	return new CEventEntry(frame,nmr_string(action),nmr_string(target));
}






