/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CMonitorDataPackage.h>
#include	<CTCPSocket.h>

CMonitorDataPackage::CMonitorDataPackage(void)
{
	mQueueStats = NULL;
	mDetachQueueData = 0;
	mQueueSize = 0;
}

CMonitorDataPackage::CMonitorDataPackage(int available,int active,int dead,
		ActiveMovieStatList *q,int qsize)
{
	mAvailable 	= available;
	mActive		= active;
	mDead		= dead;
	mQueueStats	= q;
	mQueueSize	= qsize;
	mDetachQueueData = 0;
}

CMonitorDataPackage::~CMonitorDataPackage(void)
{
	if (!mDetachQueueData)
		delete mQueueStats;
	mQueueStats = NULL;
}

void
CMonitorDataPackage::DetachData(void)
{
	mDetachQueueData = 1;
}

void
CMonitorDataPackage::Read(CTCPSocket &socket)
{
	u_short	u[4];
	socket.Read((char *)u,sizeof(u));

	mAvailable 	= ntohs(u[0]);
	mActive		= ntohs(u[1]);
	mDead		= ntohs(u[2]);
	mQueueSize	= ntohs(u[3]);

	mQueueStats = new ActiveMovieStatList;

	for (int x = 0; x < mQueueSize; ++x)
			{
			CActiveMovieStat	ams;
			ams.Read(socket);
			mQueueStats->push_back(ams);
			}
}

size_t
CMonitorDataPackage::CalculateSize(void)
{
	size_t	total = sizeof(u_short) * 4;
	if (mQueueStats)
		{
		ActiveMovieStatList::iterator x;
		for (x = mQueueStats->begin(); x != mQueueStats->end(); x++)
			{
			total += (*x).CalculateSize();
			}
		}

	return total;
}

size_t CalculateTCPSize(CMonitorDataPackage &d)
{
	return d.CalculateSize();
}

void
CMonitorDataPackage::Write(CTCPSocket &socket)
{
	u_short	u[4];

	u[0] = htons(mAvailable);
	u[1] = htons(mActive);
	u[2] = htons(mDead);
	u[3] = htons(mQueueSize);
	socket.Write((char *)u,sizeof(u));

	ActiveMovieStatList::iterator x;
	for (x = mQueueStats->begin(); x != mQueueStats->end(); x++)
		(*x).Write(socket);
}

CTCPSocket& operator<<(CTCPSocket &socket,CMonitorDataPackage &p)
{
	p.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CMonitorDataPackage &p)
{
	p.Read(socket);
	return socket;
}

const char *
CMonitorDataPackage::Name(void)
{
	return "CMonitorDataPackage";
}

void
CMonitorDataPackage::Dump(ostream &o,const char *indent)
{
	nmr_string	nmr(indent);
	nmr += "   ";

	o << indent << "CMonitorDataPackage: " << endl;
	o << indent << "   mAvailable: " << mAvailable << endl;
	o << indent << "   mActive: " << mActive << endl;
	o << indent << "   mDead: " << mDead << endl;
	o << indent << "   mQueueStats: " << endl;

	ActiveMovieStatList::iterator x;
	for (x = mQueueStats->begin(); x != mQueueStats->end(); x++)
		(*x).Dump(o,nmr);
}


ostream& operator<<(ostream &o,CMonitorDataPackage &p)
{
	p.Dump(o,"");
	return o;
}

ostream& operator<<(ostream &o,CMonitorDataPackage *p)
{
	if (p)
		p->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

