/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CStreamData.h>
#ifdef NETHFB
#include	<CTCPSocket.h>
#endif

/*************************************************************************
CStreamData

	An encapsulation of the data sent for a frame of video or
	chunk of audio
*************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// CStreamData
//
// Constructor
///////////////////////////////////////////////////////////////////////////////
CStreamData::CStreamData(void)
{
	mStreamData 	= NULL;
	mDataAttached	= 1;
}

///////////////////////////////////////////////////////////////////////////////
// CStreamData
//
// Constructor with values
//
// INPUT:
//	s_type		- stream type that this data belongs to (DUK_STREAM)
//	s_num		- stream number within the type that the data belongs to
//	i			- Index from the server associated with this data
//	l			- length of the data
//	acc			- data position accumulator which indicates how much
//					raw data up to this point has passed. For video
//					this is just a frame count. For audio it's a
//					byte count
// is_key_frame	- If video, is this a key frame
// d			- The data itself
///////////////////////////////////////////////////////////////////////////////
CStreamData::CStreamData(int s_type,int s_num,long i,long l,long acc,
									long is_key_frame,char *d)
{
	mStreamType		= s_type;
	mStreamNumber	= s_num;
	mIndex			= i;
	mLen			= l;
	mStreamData		= d;
	mDataAttached	= 1;
	mDataPosAccumulator = acc;
	mIsKeyFrame = is_key_frame;
}


///////////////////////////////////////////////////////////////////////////////
// ~CStreamData
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CStreamData::~CStreamData(void)
{
	if (mDataAttached)
		delete [] mStreamData;
	mStreamData = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// Mark the data so it will NOT be disposed when this object is deleted
///////////////////////////////////////////////////////////////////////////////
void
CStreamData::DetachData(void)
{
	mDataAttached = 0;
}

///////////////////////////////////////////////////////////////////////////////
// ReAttachData
//
// Mark the data so it WILL be disposed when this object is deleted
///////////////////////////////////////////////////////////////////////////////
void
CStreamData::ReAttachData(void)
{
	mDataAttached = 1;
}

///// The remaining methods are just Get/Set accessors

int
CStreamData::GetStreamType(void)
{
	return mStreamType;
}

int
CStreamData::GetStreamNumber(void)
{
	return mStreamNumber;
}

long
CStreamData::GetIndex(void)
{
	return mIndex;
}

long
CStreamData::GetLength(void)
{
	return mLen;
}

const char *
CStreamData::GetStreamData(void)
{
	return mStreamData;
}

long
CStreamData::GetDataPosAccumulator(void)
{
	return mDataPosAccumulator;
}

long
CStreamData::GetIsKeyFrame(void)
{
	return mIsKeyFrame;
}

#ifdef NETHFB
void
CStreamData::Read(CTCPSocket &socket)
{
	u_long	u[6];

	socket.Read((char *)&u,sizeof(u));
	mStreamType				= ntohl(u[0]);
	mStreamNumber			= ntohl(u[1]);
	mIndex					= ntohl(u[2]);
	mDataPosAccumulator		= ntohl(u[3]);
	mIsKeyFrame				= ntohl(u[4]);
	mLen					= ntohl(u[5]);
	
	if (mLen)
		{
		mStreamData = new char[mLen];
		socket.Read(mStreamData,mLen);
		}
	else
		{
		mStreamData = NULL;
		}
}

size_t
CStreamData::CalculateSize(void)
{
	return (sizeof(u_long) * 6) + mLen;
}

void
CStreamData::Write(CTCPSocket &socket)
{
	u_long	u[6];

	u[0] = htonl((long)mStreamType);
	u[1] = htonl((long)mStreamNumber);
	u[2] = htonl((long)mIndex);
	u[3] = htonl((long)mDataPosAccumulator);
	u[4] = htonl((long)mIsKeyFrame);
	u[5] = htonl((long)mLen);

	socket.Write((char *)u,sizeof(u));

	if (mLen > 0)
		socket.Write(mStreamData,mLen);
}
#endif

const char *
CStreamData::Name(void)
{
	return "CStreamData";
}

void
CStreamData::Dump(ostream &o,const char *indent)
{
	o << indent << "CStreamData: " << endl;
	o << indent << "   mStreamType: " << mStreamType << endl;
	o << indent << "   mStreamNumber: " << mStreamNumber << endl;
	o << indent << "   mIndex: " << mIndex << endl;
	o << indent << "   mDataPosAccumulator: " << mDataPosAccumulator << endl;
	o << indent << "   mIsKeyFrame: " << mIsKeyFrame << endl;
	o << indent << "   mLen: " << mLen << endl;
	o << indent << "   mStreamData: " << ((void *)mStreamData) << endl; 
}

#ifdef NETHFB
ostream& operator<<(ostream &o,CStreamData *sd)
{
	if (sd)
		sd->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

ostream& operator<<(ostream &o,CStreamData &sd)
{
	sd.Dump(o,"");
	return o;
}

CTCPSocket& operator<<(CTCPSocket &s,CStreamData &d)
{
	d.Write(s);
	return s;
}

CTCPSocket& operator>>(CTCPSocket &s,CStreamData &d)
{
	d.Read(s);
	return s;
}

size_t CalculateTCPSize(CStreamData &d)
{
	return d.CalculateSize();
}
#endif

