/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CStreamParameters.h>
#include	<CVideoInfo.h>
#include	<CAudioInfo.h>
#ifdef NETHFB
#include	<CUDPSocket.h>
#endif
#include	<nmr_string.h>

CStreamParameters::CStreamParameters(long datarate,int height,int width,
	int num_aud_streams,int num_vid_streams,CAudioInfo *ainfo,
	int own_audio_info,CVideoInfo *vinfo,int own_video_info)
{
#ifdef NETHFB
	assert(sizeof(UDPStreamParameterBlock_t) ==(16 + 40  + 60) );
#endif
	mDatarate = datarate;
	mHeight = height;
	mWidth = width;
	mNumAudioStreams = num_aud_streams;
	mNumVideoStreams = num_vid_streams;
	mAudioInfo = ainfo;
	mVideoInfo = vinfo;
	mOwnAudioInfo = own_audio_info;
	mOwnVideoInfo = own_video_info;
}

CStreamParameters::~CStreamParameters(void)
{
	if (mOwnAudioInfo)
		delete mAudioInfo;
	mAudioInfo = NULL;
	if (mOwnVideoInfo)
		delete mVideoInfo;
	mVideoInfo = NULL;
}

void
CStreamParameters::DetachStreamInfoObjects(void)
{
	mOwnAudioInfo = 0;
	mOwnVideoInfo = 0;
}

#ifdef NETHFB
CStreamParameters *
CStreamParameters::CreateFromUDPData(const UDPStreamParameterBlock_t *block,
																	int siz)
{
	if (ntohs(block->mType) != STREAMPARAMETERBLOCK_TYPE)
		return NULL;
	if (siz < sizeof(UDPStreamParameterBlock_t))
		return NULL;

	int datarate 		= ntohl(block->mDatarate);
	int height 			= ntohs(block->mHeight);
	int width 			= ntohs(block->mWidth);
	int audiostreams  	= ntohs(block->mNumAudioStreams);
	int videostreams 	= ntohs(block->mNumVideoStreams);
	CAudioInfo 	*ainfo 	= CAudioInfo::CreateFromUDPData(&(block->mAudioInfo),
												sizeof(UDPAudioInfoBlock_t));
	CVideoInfo	*vinfo 	= CVideoInfo::CreateFromUDPData(&(block->mVideoInfo),
												sizeof(UDPVideoInfoBlock_t));

	return new CStreamParameters(datarate,height,width,audiostreams,
		videostreams,ainfo,1,vinfo,1);
}

void
CStreamParameters::FillInUDPData(UDPStreamParameterBlock_t *block)
{
	block->mType 			= htons(STREAMPARAMETERBLOCK_TYPE);
	block->mDatarate 		= htonl(mDatarate);
	block->mHeight 			= htons(mHeight);
	block->mWidth 			= htons(mWidth);
	block->mNumAudioStreams = htons(mNumAudioStreams);
	block->mNumVideoStreams = htons(mNumVideoStreams);
	if (mAudioInfo)
		mAudioInfo->FillInUDPData(&(block->mAudioInfo));
	if (mVideoInfo)
		mVideoInfo->FillInUDPData(&(block->mVideoInfo));
}


void
CStreamParameters::UDPTransmit(CUDPSocket *socket)
{
	UDPStreamParameterBlock_t	block;

	FillInUDPData(&block);
	socket->SendTo((const char *)&block,sizeof(block));
}
#endif



const char *
CStreamParameters::Name(void)
{
	return "CStreamParameters";
}

void
CStreamParameters::Dump(ostream &o,const char *indent)
{
	nmr_string	nmr(indent);
	nmr += "   ";

	o << indent << "CStreamParameters: " << endl;
	o << indent << "   mDataRate: " << mDatarate << endl;
	o << indent << "   mHeight: " << mHeight << endl;
	o << indent << "   mWidth: " << mWidth << endl;
	o << indent << "   mNumAudioStreams: " << mNumAudioStreams << endl;
	o << indent << "   mNumVideoStream: " << mNumVideoStreams << endl;
	o << indent << "   mOwnVideoInfo: " << mOwnVideoInfo << endl;
	o << indent << "   mOwnAudioInfo: " << mOwnAudioInfo << endl;
	if (mAudioInfo)
		mAudioInfo->Dump(o,nmr);
	else
		o << nmr << " NULL " << endl;

	if (mVideoInfo)
		mVideoInfo->Dump(o,nmr);
	else o << nmr << " NULL " << endl;
}


ostream&
operator<<(ostream& o,CStreamParameters &sp)
{
	sp.Dump(o,"");
	return o;
}

ostream&
operator<<(ostream& o,CStreamParameters *sp)
{
	if (sp)
		sp->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}
