/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CVideoInfo.h>
#include	<CTCPSocket.h>

CVideoInfo::CVideoInfo(void)
{
#ifdef NETHFB
	assert(sizeof(UDPVideoInfoBlock_t) == 40);
#endif
	mFCCHandler = 0;
	mStreamNum = 0;
	mSamplesPerFrame = 0;
	mInitialFrames = 0;
	mFrameRate = 0;
}

CVideoInfo::~CVideoInfo(void)
{
}

void	
CVideoInfo::ExtractFromStream(HFB_STREAM_HANDLE primary_audio,
											HFB_STREAM_HANDLE strm,int s_num)
{
	int			tmp;
	int			frame_rate;

	mStreamNum    	= s_num;
	mSamplesPerFrame= HFB_GetSamplesPerFrame(strm,primary_audio);
	mInitialFrames	= HFB_GetInitialFrames(strm,primary_audio);
	HFB_GetFrameRates(strm,primary_audio,&frame_rate,&tmp);

	long rate,scale;
	HFB_GetStreamRateAndScale(strm,&rate,&scale);
	nmr_fraction<I32> f(rate,scale);
	HFB_GetStreamFCCs(strm,NULL,&mFCCHandler);
	mFrameRate = f;
	mDKBitmap = HFB_GetStreamInfo(strm)->a.BitmapInfo;
}


double
CVideoInfo::GetTimingFactor(void)
{
	double f = (double)mFrameRate.Numerator();
	f /= (double)mFrameRate.Denominator();
	return f;
}

nmr_fraction<I32>
CVideoInfo::GetTimingFactorAsFraction(void)
{
	return mFrameRate;
}

unsigned long
CVideoInfo::GetFCCHandler(void)
{
	return mFCCHandler;
}

#ifdef NETHFB
void	
CVideoInfo::FillInUDPData(UDPVideoInfoBlock_t *block)
{
	block->mType 					= htons(VIDEOINFOBLOCK_TYPE);
	block->mStreamNum 				= htonl(mStreamNum);
	block->mSamplesPerFrame			= htonl(mSamplesPerFrame);
	block->mInitialFrames 			= htonl(mInitialFrames);
	block->mFrameRateNumerator 		= htonl(mFrameRate.Numerator());
	block->mFrameRateDenominator	= htonl(mFrameRate.Denominator());
	block->mFCCHandler				= htonl(mFCCHandler);
	block->usFormatTag				= htons(mDKBitmap.usFormatTag);
	block->usHeight					= htons(mDKBitmap.usHeight);
	block->usWidth					= htons(mDKBitmap.usWidth);
	block->usDepth					= htons(mDKBitmap.usDepth);
	block->usHandler				= htons(mDKBitmap.usHandler);
}

CVideoInfo *
CVideoInfo::CreateFromUDPData(const UDPVideoInfoBlock_t *block,int siz)
{
	int numerator,denominator;

	if (siz < sizeof(UDPVideoInfoBlock_t))
		return NULL;
	if (ntohs(block->mType) != VIDEOINFOBLOCK_TYPE)
		return NULL;

	CVideoInfo	*vinfo = new CVideoInfo;

	vinfo->mDKBitmap.usFormatTag	= ntohs(block->usFormatTag);
	vinfo->mDKBitmap.usHeight		= ntohs(block->usHeight);
	vinfo->mDKBitmap.usWidth		= ntohs(block->usWidth);
	vinfo->mDKBitmap.usDepth		= ntohs(block->usDepth);
	vinfo->mDKBitmap.usHandler		= ntohs(block->usHandler);
	vinfo->mStreamNum 			= ntohl(block->mStreamNum);
	vinfo->mSamplesPerFrame 	= ntohl(block->mSamplesPerFrame);
	vinfo->mInitialFrames 		= ntohl(block->mInitialFrames);
	numerator					= ntohl(block->mFrameRateNumerator);
	denominator					= ntohl(block->mFrameRateDenominator);
	vinfo->mFCCHandler			= ntohl(block->mFCCHandler);
	nmr_fraction<I32> f(numerator,denominator);
	vinfo->mFrameRate = f;

	return vinfo;
}


void
CVideoInfo::Read(CTCPSocket &socket)
{
	int numerator,denominator;

	u_long 	data[6];
	u_short ds[5];

	socket.Read((char *)data,sizeof(data));
	socket.Read((char *)ds,sizeof(ds));

	mDKBitmap.usFormatTag	= ntohs(ds[0]);
	mDKBitmap.usHeight		= ntohs(ds[1]);
	mDKBitmap.usWidth		= ntohs(ds[2]);
	mDKBitmap.usDepth		= ntohs(ds[3]);
	mDKBitmap.usHandler		= ntohs(ds[4]);
	
	mStreamNum 				= ntohl(data[0]);
	mSamplesPerFrame 		= ntohl(data[1]);
	mInitialFrames 			= ntohl(data[2]);
	numerator				= ntohl(data[3]);
	denominator 			= ntohl(data[4]);
	mFCCHandler				= ntohl(data[5]);

	nmr_fraction<I32> f(numerator,denominator);
	mFrameRate = f;
}

size_t
CVideoInfo::CalculateSize(void)
{
	return (sizeof(u_long) * 5);
}

void
CVideoInfo::Write(CTCPSocket &socket)
{
	u_long	data[6];
	u_short ds[5];

	ds[0]	= htons(mDKBitmap.usFormatTag);
	ds[1]	= htons(mDKBitmap.usHeight);
	ds[2]	= htons(mDKBitmap.usWidth);
	ds[3] 	= htons(mDKBitmap.usDepth);
	ds[4]	= htons(mDKBitmap.usHandler);

	data[0] = htonl(mStreamNum);
	data[1] = htonl(mSamplesPerFrame);
	data[2] = htonl(mInitialFrames);
	data[3] = htonl(mFrameRate.Numerator());
	data[4] = htonl(mFrameRate.Denominator());
	data[5] = htonl(mFCCHandler);

	socket.Write((char *)data,sizeof(data));
	socket.Write((char *)ds,sizeof(ds));
}

#endif

double
CVideoInfo::ChunkTimeSize(long len)
{
	// a chunk is always one frame
	return 1.0;
}

const char *
CVideoInfo::Name(void)
{
	return "CVideoInfo";
}

void
CVideoInfo::Dump(ostream &o,const char *indent)
{
	o << indent << Name() << ": " << endl;
	o << indent << "   mStreamNum: " << mStreamNum << endl;
	o << indent << "   mSamplesPerFrame: " << mSamplesPerFrame << endl;
	o << indent << "   mInitialFrames: " << mInitialFrames << endl;
	o << indent << "   mFrameRate: " << 
		(((float)mFrameRate.Numerator()) / ((float)mFrameRate.Denominator())) 
		<< endl;
	o << indent << "   mFCCHandler: " << mFCCHandler << endl;
}


ostream& operator<<(ostream&o,CVideoInfo *vi)
{
	if (vi)
		vi->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

ostream& operator<<(ostream&o,CVideoInfo &vi)
{
	vi.Dump(o,"");
	return o;
}


#ifdef NETHFB
CTCPSocket& operator<<(CTCPSocket &socket,CVideoInfo &vi)
{
	vi.Write(socket);
	return socket;
}

CTCPSocket& operator>>(CTCPSocket &socket,CVideoInfo &vi)
{
	vi.Read(socket);
	return socket;
}

size_t CalculateTCPSize(CVideoInfo &vi)
{
	return vi.CalculateSize();
}
#endif




