/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	TCPPacketDefinition_H_
#define TCPPacketDefinition_H_

#include	<MegaHeaders.h>
#include	<nmr_string.h>
#include	<CDataList.h>
#include	<CAudioInfo.h>
#include	<CVideoInfo.h>
#include	<CStreamData.h>
#include	<CIndexData.h>
#include	<CEventEntry.h>
#include	<CPacketSimple.h>
#include	<CPacketType.h>
#include	<CMonitorDataPackage.h>
#include	<CConfiguration.h>

#define PACKET_TYPE_STREAMDATA		PACKET_TYPE_FIRST_USER
#define PACKET_TYPE_AUDIOINFO		PACKET_TYPE_FIRST_USER+1
#define	PACKET_TYPE_VIDEOINFO		PACKET_TYPE_FIRST_USER+2
#define PACKET_TYPE_INDEXDATA		PACKET_TYPE_FIRST_USER+3
#define PACKET_TYPE_CONFIGURATION	PACKET_TYPE_FIRST_USER+4
#define PACKET_TYPE_EVENTENTRY		PACKET_TYPE_FIRST_USER+5

////////////// SUPERVISOR PROTOCOL ///////////////////////
#define PROTOCOL_SUPERVISOR	0

#define	PS_VERSION				50	// version transmit
typedef CPacketType<I16>		PS_VERSION_TYPE;
#define	PS_VERSION_SUBTYPE		PACKET_TYPE_I16

///////////////// MOVIE PROTOCOL ///////////////////////
#define PROTOCOL_MOVIE1		1

#define	PS_FILENAME				50	// filename transmit
typedef CPacketType<nmr_string>	PS_FILENAME_TYPE;
#define	PS_FILENAME_SUBTYPE		PACKET_TYPE_STRING

#define	PS_FILENAMEOK			51 // filename approved
typedef	CPacketSimple			PS_FILENAMEOK_TYPE;
#define	PS_FILENAMEOK_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_FILENAMEBAD			52 // filename rejected
typedef	CPacketSimple			PS_FILENAMEBAD_TYPE;
#define	PS_FILENAMEBAD_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_INITBUFFER			53	// initialize HFB
typedef	 CPacketType<I32>		PS_INITBUFFER_TYPE;
#define	PS_INITBUFFER_SUBTYPE	PACKET_TYPE_I32

#define	PS_STARTSTREAM			54	// start sending stream
typedef	CPacketType<CDataList<I16> >	PS_STARTSTREAM_TYPE;
#define	PS_STARTSTREAM_SUBTYPE	PACKET_TYPE_I16LIST
#define	PS_STARTSTREAM_FIELD_STREAMTYPE		0
#define PS_STARTSTREAM_FIELD_STREAMNUMBER	1

#define	PS_STOPSTREAM			55	// stop sending stream
typedef	CPacketType<CDataList<I16> >	PS_STOPSTREAM_TYPE;
#define	PS_STOPSTREAM_SUBTYPE	PACKET_TYPE_I16LIST
#define	PS_STOPSTREAM_FIELD_STREAMTYPE		0
#define PS_STOPSTREAM_FIELD_STREAMNUMBER	1

#define	PS_STREAMDATA			56	// packet of stream data
typedef	CPacketType< CStreamData >	PS_STREAMDATA_TYPE;
#define	PS_STREAMDATA_SUBTYPE	PACKET_TYPE_STREAMDATA

#define PS_AUDIOINFO			57	// info about audio stream
typedef CPacketType< CAudioInfo >	PS_AUDIOINFO_TYPE;
#define	PS_AUDIOINFO_SUBTYPE	PACKET_TYPE_AUDIOINFO

#define	PS_VIDEOINFO			58	// info about a video stream
typedef	CPacketType< CVideoInfo > 	PS_VIDEOINFO_TYPE;
#define	PS_VIDEOINFO_SUBTYPE	PACKET_TYPE_VIDEOINFO

#define	PS_DONE					59	// all done
typedef CPacketSimple				PS_DONE_TYPE;
#define	PS_DONE_SUBTYPE			PACKET_TYPE_SIMPLE

#define	PS_READDATA				60	// request for unbuffered read
typedef	CPacketType<CDataList<I32> >	PS_READDATA_TYPE;
#define	PS_READDATA_SUBTYPE				PACKET_TYPE_I32LIST
#define	PS_READDATA_FIELD_STREAMTYPE	0
#define	PS_READDATA_FIELD_STREAMNUMBER	1
#define	PS_READDATA_FIELD_MAXLENGTH		2
#define	PS_READDATA_FIELD_DIRECTION		3
#define	PS_READDATA_FIELD_COUNT			4

#define	PS_READDATARESPONSE		61	// response to read data request
typedef	 CPacketType< CStreamData >		PS_READDATARESPONSE_TYPE;
#define	PS_READDATARESPONSE_SUBTYPE		PACKET_TYPE_STREAMDATA

#define	PS_STOPSTREAMACK		62	// stop sending stream acknowledge
typedef	CPacketType<CDataList<I16> >	PS_STOPSTREAMACK_TYPE;
#define	PS_STOPSTREAMACK_SUBTYPE	PACKET_TYPE_I16LIST
#define	PS_STOPSTREAMACK_FIELD_STREAMTYPE	0
#define PS_STOPSTREAMACK_FIELD_STREAMNUMBER	1

#define	PS_INDEXREQUEST			63	// request index info for stream
typedef CPacketType<CDataList<I16> >	PS_INDEXREQUEST_TYPE;
#define	PS_INDEXREQUEST_SUBTYPE		PACKET_TYPE_I16LIST
#define	PS_INDEXREQUEST_FIELD_STREAMTYPE	0
#define	PS_INDEXREQUEST_FIELD_STREAMNUMBER	1

#define	PS_INDEX				64	// index for a stream
typedef	CPacketType<CIndexData>		PS_INDEX_TYPE;
#define	PS_INDEX_SUBTYPE			PACKET_TYPE_INDEXDATA
#define	PS_INDEX_FIELD_STREAMTYPE		0
#define	PS_INDEX_FIELD_STREAMNUMBER		1
#define	PS_INDEX_FIELD_NUMRECORDS		2
#define	PS_INDEX_FIELD_DATASTART		3
#define	PS_INDEX_SUBFIELD_ISKEY			0
#define	PS_INDEX_SUBFIELD_SERVERCHUNK	1
#define	PS_INDEX_NUMBER_OF_SUBFIELD_ENTRIES	2

////////////// MOVIE 2 PROTOCOL (includes elements from MOVIE 1)/////////////

#define	PROTOCOL_MOVIE2	2

#define	PS_TRANSFERMODE	65	// what type of mode to use
typedef	 CPacketType<I16>		PS_TRANSFERMODE_TYPE;
#define	PS_TRANSFERMODE_SUBTYPE	PACKET_TYPE_I16
#define	PS_TRANSFERMODE_TCP_SERVER	0	// tcp with server control
#define	PS_TRANSFERMODE_TCP_CLIENT	1	// tcp with client control
#define	PS_TRANSFERMODE_UDP_CLIENT	2	// udp with client control
#define PS_TRANSFERMODE_MULTICAST	3	// multicast udp
#define PS_TRANSFERMODE_MULTICAST_NOCONNECT	4 // no server connect multicast

#define	PS_TRANSFERMODEREPLY	66	// transfer mode supported or not
typedef	CPacketType<I16>		PS_TRANSFERMODEREPLY_TYPE;
#define	PS_TRANSFERMODEREPLY_SUBTYPE	PACKET_TYPE_I16

#define PS_STREAMCOUNTREQUEST	67	// give me the stream count
typedef CPacketSimple			PS_STREAMCOUNTREQUEST_TYPE;
#define	PS_STREAMCOUNTREQUEST_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_STREAMCOUNTREPLY		68	// the stream count list
typedef	CPacketType<CDataList<I16> >	PS_STREAMCOUNTREPLY_TYPE;
#define	PS_STREAMCOUNTREPLY_SUBTYPE		PACKET_TYPE_I16LIST
#define	PS_STREAMCOUNTREPLY_FIELD_STREAMTYPE		0
#define PS_STREAMCOUNTREPLY_FIELD_STREAMCOUNT	1

#define PS_DATARATEREQUEST		69 // raw file data rate
typedef CPacketSimple			PS_DATARATEREQUEST_TYPE;
#define PS_DATARATEREQUEST_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_DATARATEREPLY		70 // the raw data rate and ping lag
typedef CPacketType<CDataList<I32> >		PS_DATARATEREPLY_TYPE;
#define PS_DATARATEREPLY_SUBTYPE	PACKET_TYPE_I32LIST
#define	PS_DATARATEREPLY_FIELD_DATARATE			0
#define	PS_DATARATEREPLY_FIELD_LAG				1
#define	PS_DATARATEREPLY_FIELD_HEIGHT			2
#define	PS_DATARATEREPLY_FIELD_WIDTH			3
#define PS_DATARATEREPLY_FIELD_MULTICASTPORT	4

#define	PS_STREAMBEHIND			71	// the stream is behind
typedef	CPacketType<CDataList<I16> >	PS_STREAMBEHIND_TYPE;
#define	PS_STREAMBEHIND_SUBTYPE			PACKET_TYPE_I16LIST
#define	PS_STREAMBEHIND_FIELD_STREAMTYPE	0
#define	PS_STREAMBEHIND_FIELD_STREAMNUMBER	1
#define PS_STREAMBEHIND_FIELD_VALUE			2 
// for video it reports the current frame rate. for audio
// it is unused

#define	PS_STREAMCAUGHTUP		72	// attempt data rate increase
typedef	 CPacketType<CDataList<I16> >	PS_STREAMCAUGHTUP_TYPE;
#define	PS_STREAMCAUGHTUP_SUBTYPE		PACKET_TYPE_I16LIST
#define	PS_STREAMCAUGHTUP_FIELD_STREAMTYPE		0
#define	PS_STREAMCAUGHTUP_FIELD_STREAMNUMBER	1
#define PS_STREAMCAUGHTUP_FIELD_VALUE			2	 
// for video it reports the current frame rate, for audio
// it is unused

#define	PS_UDPPORT				73	// client sends server udp port
typedef	CPacketType<I32>		PS_UDPPORT_TYPE;
#define	PS_UDPPORT_SUBTYPE		PACKET_TYPE_I32

#define PS_EVENTENTRY			74	// server sends client event
typedef	CPacketType<CEventEntry>	PS_EVENTENTRY_TYPE;
#define PS_EVENTENTRY_SUBTYPE		PACKET_TYPE_EVENTENTRY


	// uni-directional bandwidth estimation

#define PS_DATARATEREQUEST2		75 // dr request followed by udp bandwidth
typedef CPacketSimple			PS_DATARATEREQUEST2_TYPE;
#define PS_DATARATEREQUEST2_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_CDRRUDPPORT			76	// client sends server udp port
typedef	CPacketType<I32>		PS_CDRRUDPPORT_TYPE;
#define	PS_CDRRUDPPORT_SUBTYPE		PACKET_TYPE_I32

#define	PS_CLIENTDATARATEREPORT	77
typedef CPacketType<I32>		PS_CLIENTDATARATEREPORT_TYPE;
#define PS_CLIENTDATARATEREPORT_SUBTYPE	PACKET_TYPE_I32

#define	PS_DRR2REPLY			78	// unidirect bandwidth ok/not
typedef	CPacketType<I16>		PS_DRR2REPLY_TYPE;
#define	PS_DRR2REPLY_SUBTYPE	PACKET_TYPE_I16




/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

// this conflicts with the ones above,but we have to leave it
// for compatibility

#define	PACKET_TYPE_MONITORDATA	PACKET_TYPE_FIRST_USER+3

//////////////////////////MONITOR PROTOCOL///////////////////////////
#define	PROTOCOL_MONITOR1	0

#define	PS_UPDATE			25 // request server state
typedef	CPacketSimple		PS_UPDATE_TYPE;
#define	PS_UPDATE_SUBTYPE	PACKET_TYPE_SIMPLE

#define	PS_SERVERSTATE					26	// the stuff
typedef	CPacketType<CMonitorDataPackage>	PS_SERVERSTATE_TYPE;
#define	PS_SERVERSTATE_SUBTYPE				PACKET_TYPE_MONITORDATA

#define PS_REQUESTCONFIG				27
typedef CPacketSimple					PS_REQUESTCONFIG_TYPE;
#define	PS_REQUESTCONFIG_SUBTYPE		PACKET_TYPE_SIMPLE

#define	PS_CONFIG						28
typedef CPacketType<CConfiguration>		PS_CONFIG_TYPE;
#define	PS_CONFIG_SUBTYPE				PACKET_TYPE_CONFIGURATION

#define	PS_SERVERTERMINATE				29
typedef CPacketSimple					PS_SERVERTERMINATE_TYPE;
#define	PS_SERVERTERMINATE_SUBTYPE		PACKET_TYPE_SIMPLE

#define PS_MOVIETERMINATE				30
typedef CPacketType<I32 >				PS_MOVIETERMINATE_TYPE;
#define PS_MOVIETERMINATE_SUBTYPE		PACKET_TYPE_I32

#define PS_CAPTURECHANGE				31
typedef CPacketType<CDataList<I32> >	PS_CAPTURECHANGE_TYPE;
#define PS_CAPTURECHANGE_SUBTYPE		PACKET_TYPE_I32LIST
#define PS_CAPTURECHANGE_FIELD_THREADID			0
#define PS_CAPTURECHANGE_FIELD_VIDSOURCE		1
#define PS_CAPTURECHANGE_FIELD_VIDCHANNEL		2
#define PS_CAPTURECHANGE_FIELD_VIDBRIGHTNESS	3
#define PS_CAPTURECHANGE_FIELD_VIDCONTRAST		4
#define PS_CAPTURECHANGE_FIELD_VIDHUE			5
#define PS_CAPTURECHANGE_FIELD_VIDSATURATION	6
#define PS_CAPTURECHANGE_NUMFIELDS				7

#define	PS_AVICAPTUREOFF				32
typedef CPacketType<I32 >				PS_AVICAPTUREOFF_TYPE;
#define PS_AVICAPTUREOFF_SUBTYPE		PACKET_TYPE_I32

#define PS_AVICAPTUREON					33
typedef CPacketType<nmr_string>			PS_AVICAPTUREON_TYPE;
#define	PS_AVICAPTUREON_SUBTYPE			PACKET_TYPE_STRING

#define PS_CONFIGHUP					34
typedef CPacketSimple					PS_CONFIGHUP_TYPE;
#define PS_CONFIGHUP_SUBTYPE			PACKET_TYPE_SIMPLE

// config that requires an ack
#define PS_REQUESTCONFIG2				35
typedef CPacketSimple					PS_REQUESTCONFIG2_TYPE;
#define	PS_REQUESTCONFIG2_SUBTYPE		PACKET_TYPE_SIMPLE

#define	PS_REQUESTCONFIGACK				36
typedef	CPacketSimple					PS_REQUESTCONFIGACK_TYPE;
#define	PS_REQUESTCONFIGACK_SUBTYPE		PACKET_TYPE_SIMPLE

#endif



