/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	UDPPACKETDEFINITION_H_
#define UDPPACKETDEFINITION_H_

#include	<MegaHeaders.h>

#define VIDEOINFOBLOCK_TYPE			96
#define	AUDIOINFOBLOCK_TYPE			97
#define	STREAMPARAMETERBLOCK_TYPE	98
#define EVENTBLOCK_TYPE				99
#define ERRORBLOCK_TYPE				255	// invalid

// Using these assures cross platform rollover at the same points //
typedef u_long	sequence_t;
typedef	u_long	datacounter_t;

#define	DATACOUNTER_MASK	0xffffffff
#define	SEQUENCENUMBER_MASK	0xffffffff

#define	ADVANCE_SEQUENCE(s,amount)\
	((((sequence_t) s) + ((sequence_t)amount)) & SEQUENCENUMBER_MASK)

#define ADVANCE_DATACOUNTER(d,amount)\
	((((datacounter_t) d) + ((datacounter_t)amount)) & DATACOUNTER_MASK)

#define	DIFF_SEQUENCE(hi,lo) \
	((hi >= lo ? hi - lo : SEQUENCENUMBER_MASK - lo + hi + 1) & \
														SEQUENCENUMBER_MASK)

#define	DIFF_DATACOUNTER(hi,lo) \
	((hi >= lo ? hi - lo : DATACOUNTER_MASK - lo + hi + 1) & \
														DATACOUNTER_MASK)

////////////////////////////////////////////////////////////////////

/***************************************************
UDP Packet Definition
****************************************************/
typedef struct {
	u_short		mStreamType;
	u_short		mStreamNumber;
	sequence_t	mSequenceNumber;	// what chunk (zero based)
								// two byte rollover
	u_short	mSequenceSize;		// number of packets in sequence
	u_short	mPacketNumber;		// packet in sequence (zero based)
	u_long	mChunkDataSize;		// total size of data in all packets of chunk

	// data sent as header with CStreamData
	u_long			mIndex;
	u_long			mAcc;
	u_short			mIsKeyFrame;
	u_short			mPad;
	datacounter_t	mDataCounter;// amount of data before me since initbuff
								 // 4 byte rollover
	u_long	mDataLen;			// length in this packet beyond this point
} UDPPacketDefHeader_t;


typedef struct {
	UDPPacketDefHeader_t	mHeader;

	u_char	mData[1];			// actually much larger

} UDPPacketDefinition_t;

////////////////////////////////////////////////////////
typedef struct {
	u_short	mType;
	u_short	mPad;
	u_long	mNumChannels;
	u_long mSamplesPerSec;
	u_long mBytesPerSec;
	u_long mFormat;
	u_long mBytesPerBlock;
	u_long wi_mFormatTag;
	u_long wi_mChannels;
	u_long wi_mSamplesPerSec;
	u_long wi_mAvgBytesPerSec;
	u_long wi_mBlockAlign;
	u_long wi_mBitsPerSample;
	u_long wi_mCbSize;
	u_long wi_mSamplesPerBlock;
	u_long	mFCCHandler;
} UDPAudioInfoBlock_t;

typedef struct {
	u_short	mType;
	u_short mPad;
	u_long	mStreamNum;
	u_long	mSamplesPerFrame;
	u_long	mInitialFrames;
	u_long	mFrameRateNumerator;
	u_long	mFrameRateDenominator;
	u_long	mFCCHandler;
	u_short	usFormatTag;
	u_short	usHeight;
	u_short	usWidth;
	u_short	usDepth;
	u_short	usHandler;
	u_short	mPad2;
} UDPVideoInfoBlock_t;

typedef struct {
	u_short	mType;
	u_short	mPad;
	u_long	mDatarate;
	u_short	mHeight;
	u_short	mWidth;
	u_short mNumAudioStreams;
	u_short mNumVideoStreams;
	UDPAudioInfoBlock_t	mAudioInfo;
	UDPVideoInfoBlock_t	mVideoInfo;

} UDPStreamParameterBlock_t;

typedef struct {
	u_short mType;
	u_short	mPad;
	u_long	mFrame;
	char	mData[256*2];
} UDPEventEntryBlock_t;


/****************************************************/

#define	MAX_MTU	4196

// this is temporary until we make this value dynamic
#define	TEMP_MTU	1024
#endif
