/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//==========================================================================
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1999 - 2001  On2 Technologies Inc. All Rights Reserved.
//
//--------------------------------------------------------------------------


#ifndef vp3d_h
#define vp3d_h 1

// Interface between vp3d.dll and Albany's DXV adaptor/blitter.


// The main object "defined" here.

struct VP3decompressor;


// Some conveniences.

typedef unsigned char uchar;
typedef unsigned int uint;
typedef unsigned long ulong;


// FourCC codes.  Should agree with microsoft's definition
// sans their stupid types and include files.

typedef ulong FourCC;

#define MakeFourCC( a, b, c, d) ( \
	(ulong) (uchar) a \
	| (ulong) (uchar) b << 8 \
	| (ulong) (uchar) c << 16 \
	| (ulong) (uchar) d << 24 \
)

// The actual fourCC for now; similar remarks apply.

#define VP30 1


// Array of fourCC codes, has length _and_ is null-terminated.
// As Donald Knuth once said,
// "Some people occasionally like a little extra redundancy sometimes."

typedef struct { const FourCC * codes;  uint numCodes;}  FourCClist;


// YUV buffer configuration.

typedef struct {

	ulong Ywidth, Yheight, UVwidth, UVheight;

	long Ystride, UVstride;

	const uchar *Ybuf, *Ubuf, *Vbuf;

} YUVbufferLayout;


#if __cplusplus
#	define Decompressor VP3decompressor
	extern "C" {
#else
#	define Decompressor struct VP3decompressor
#endif

#if defined(MACPPC)
#define _stdcall 
#endif

// Return array of fourCC codes supported.

const FourCClist * _stdcall VP3DfourCClist();


// Create a decompressor for a particular supported stream type.
// Returns 0 on failure.

Decompressor * _stdcall VP3DcreateDecompressor( FourCC streamType);

void _stdcall VP3DdestroyDecompressor( Decompressor *);


// Advance to next frame, returning reference to updated YUV buffer.

const YUVbufferLayout * _stdcall VP3DnextFrame
( 
	Decompressor *, const uchar * CXdata, ulong CXdataLengthInBytes
);

void _stdcall VP3DblitBGR(
	const Decompressor *, uchar * outRGB, long outStride, long outHeight
);


#if __cplusplus
	}
#endif

#undef Decompressor

#endif	// vp3d_h
