/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//==========================================================================
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1999 - 2001  On2 Technologies Inc. All Rights Reserved.
//
//--------------------------------------------------------------------------


#include <windows.h>
#include <stdarg.h>
#include "cpuidlib.h"
#include "cidasm.h"
#include <process.h>
#include <stdio.h>

extern int	WillametteNewInstructionOSSupport();
extern int	WillametteNewInstructionHWSupport();


/*
 * **-DoesOSSupportXMM
 *
 * This function will check to see if the operating supports the XMM (Pentium III) instructions
 * The XMM functionality adds 8 128-bit registers to the pentium II register set.  With the addition
 * of the new registers the OS needs to preserve and restore the registers on task switches.
 *
 * Inputs:
 *  None
 *
 * Outputs:
 *  True returned if the OS supports the XMM instructions.
 *  False returned if the OS does not suppor the XMM instructions.
 */
int DoesOSSupportXMM( void )
{
   OSVERSIONINFO OSInformation;           // Data structure where OS version will be filled in
   int           ReturnValue = FALSE;     // Preload to fail

   return TRUE;								///	dbm -- CR0 bug -- cannot be 
											///	called in Linux (or theoretically
											///	in NT so it may break eventually
											///	This function just says that the OS
											///	supports XMM, there is an independent
											///	processor check.  Returning TRUE may
											///	cause problems on old versions of 95.

   // need to initilize size of OS info structure before calling GetVersionEx
   OSInformation.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   
   if( !GetVersionEx( &OSInformation ) )  // Get OS information
   {
      /*
       * having trouble getting OS information
       * to be safe will return that we do not support XMM
       * instructions
       */
      // ReturnValue = FALSE;
   }

   if( OSInformation.dwPlatformId == VER_PLATFORM_WIN32_NT )
   //   if( 1 )
   {
      /*
       * If we are on a windows NT system we cannot directly
       * read the control registers to see if the OS supports
       * the XMM instructions.  We will just check to see if
       * service pack 4 is installed.
       */
       int ServicePackNumber;
       
       if( strcmp(OSInformation.szCSDVersion, "" ) != 0 ) // is there a service pack installed?
       {
           // Yes, get service pack revision
           char Junk[132], Junk2[132];

           sscanf( OSInformation.szCSDVersion, "%s %s %d", Junk, Junk2, &ServicePackNumber );
       }
       else
       {
           ServicePackNumber = 0;
       }

       if( OSInformation.dwMajorVersion == 4 && // must be versio 4 or greater
           ServicePackNumber >= 4 ||              // must have service pack 4 or greater
			OSInformation.dwMajorVersion >=5)
       {
           ReturnValue = TRUE;
       }
       else
       {
           // ReturnValue = FALSE;
       }
           
#if 0
       // some handy debugging info if you are desperate
       printf("OS Major Revision %d\n", OSInformation.dwMajorVersion );
       printf("OS Minor REvision %d\n", OSInformation.dwMinorVersion  );
       printf("Service Pack Number %d\n", ServicePackNumber );
#endif
   }
   else
   {
      /*
       * we are on a Windows 9x system.
       */
      if( Does9xOSSupportXMM())         // does the Windows 9x support the XMM instructions?
      {
         ReturnValue = TRUE;            // yup
      }
      else
      {
         //ReturnValue = FALSE;           // Nope, don't support XMM instructions
      }
   }

   return( ReturnValue );
}

/*
 * **-findCPUId
 *
 * See cpuidlib.h for a detailed description of this function
 */
PROCTYPE findCPUId( void )
{
   PROCTYPE CpuType;
// return (PII);         // drop to next lowest type of CPU which should be the Pentium II processor

   CpuType = getCPUType();      // Get version of processor

   // The code to check whether willammete instructions are called attempts to run 
   // an illegal instruction.  Under 98 mplayer crashes the os as soon as the illegal 
   // instruction is called, so I've disabled it.  

   if( CpuType == XMM )         // If the CPU supports XMM (Pentium III) instructions
   {
      if( DoesOSSupportXMM())   // need to check to see if the OS supports the XMM instructions
      {
		  
		 if( WillametteNewInstructionHWSupport()&&
		  WillametteNewInstructionOSSupport())
		  {
					CpuType = WMT;					
		  }
      }
      else
      {
         // os does not support the XMM instructions
         CpuType = PII;         // drop to next lowest type of CPU which should be the Pentium II processor
      }
   }
   return( CpuType );
}


