/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
 *	TrueMotion installable compressor for Video for Windows.
 *  Not to be reproduced in any form.
 *	Copyright (c) 1994  The Duck Corporation.  All Rights Reserved.
*/

#ifndef _cduck_h
#define _cduck_h

#if defined(__cplusplus)
extern "C" {
#endif

#define PLAYONLY 			0
#define CHECKCHECKSUM		0
#define LIMITFRAMES			0
#define RESTRICT_VERSION	1
#define METATYPE			3

/*
//METATYPE == 0:		pre-pixel fixup (horde)
//METATYPE == 1:		pixel fixups
//METATYPE == 2:		Frameinfo field in header
//						control word in header
//						interframe supported
//METATYPE == 3:		SPRITES!
*/

#define CTL_DUCK			1		//on=DUCK, off=HTI
#define CTL_WINDOWS			2
#define CTL_DOS				4
#define CTL_MAC				8
#define CTL_3DO				16
#define CTL_SATURN			32

#define SEGA_DEV_CTL		(CTL_DUCK|CTL_SATURN)
/*#define DUCK_CONTROL		SEGA_DEV_CTL*/
#define DUCK_CONTROL		0xFF

/*Frameinfo bits*/
#define IS_SPRITE			32
#define IS_KEYFRAME			16
#define IS_INTERFRAME		8
#define	IS_INTERPOLATED		4

/*block types */
#define BLK_IF		0
#define BLK_TM		1
#define BLK_AL		2

typedef struct compStruct {
	U8 DXEnabled[METATYPE+1];
	U8 Algorithm;
	U8 NumDeltas;
	U8 HRes,VRes;
} COMPSTRUCT;

typedef struct duckini {
	I16 Algorithm;
	I16 Gamma[8];
	I16 Crop[8];
	I16 Clamp[8];
	I16 ImageDither[4];		/*use [0] for lower, [1] for cross, [2] for upper*/
	I16 BlackDither[4];		/*unused*/
	I16 DitherThresh;		/*unused*/
	I16 ChromaRamp[4];
	I16 Ack24,dummy1,dummy2,dummy3;
	I16 LookAhead;
	I16 Dimensions[6];
	I32 MaxFrameSize;
	I32 FrameNum;
	U32 ScreenAddr;
	U16 ScreenDim[3];
	I16 Saturation;
	I16 Contrast;
	I16 Brightness; 
	I16 HorFilter;  
	I16 Detail; 
	I16 Reserved[8];

	/*KeyRate, Quality, KeyIncrement, KeyMax, KeyColThresh, KeyBigFrames,*/
	/*FramesPerSec, DataratePerSec*/
	I16 IF[8];
	
	char DeltaStr[64];
	char CompareStr[64];
	char CDeltaStr[64];
	char CCompareStr[64];
	char GammaStr[64];
	char KeyStr[128];
	U8	DeltaSet;
	U8	ColorResolution;
	U8	Scaling; 
	U8	Cropping;
	U8	CVersion;
	U8	CompLevel;
	U8	Deinterlace;
	U8	SmoothChroma;
	U8	MatchDim;
	U8	KeyFrameFlag;
	U8	CompMode;
} DUCKINI;

/*METATYPE 0 header*/
typedef struct oldframeheader {
	U32 Type;
	U32 NumBits;
	U16 Ysize;
	U16	Xsize;
	I16 deltas[8];
	I16 Cdeltas[8];
} OLDFRAMEHEADER;

/*METATYPE 1 header*/
typedef struct frameheader0 {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
} FRAMEHEADER0;

/*METATYPE 2 header*/
typedef struct frameheader2 {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
	U8	Frameinfo;
	U8	Control;
} FRAMEHEADER2;

/*METATYPE 3 header (sprites only)*/
typedef struct frameheader {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
	U8	Frameinfo;
	U8	Control;
	U16 xoff,yoff,width,height;
} FRAMEHEADER;

/* Algorithms*/
enum algorithms {
	RGB16V=1,
	RGB16H,
	RGB24H,
	MAX_ALGORITHMS
};

enum comp_styles {
	SMOOTH=0,
	DITHERED,
	COMPRESSION_STYLES
};

enum comp_modes {
	CM_INTRAFRAME=0,
	CM_INTERFRAME,
	CM_SPRITE,
	COMPRESSION_MODES
};

enum color_resolutions {
	COLOR_RES_4x4=0,
	COLOR_RES_4x2,
	COLOR_RES_2x4,
	COLOR_RES_2x2,
	COLOR_RESOLUTIONS
};

enum table_nums {
	PC_TBL=0,
	PC_TBL2,
	PC_TBL3,
#if defined(NEW24)
	PC_TBLX,
#endif
	TBL_MAX };

enum clevel_nums {
	CLEV_NORM=0,
	CLEV_HIGH,
	CLEV_MAX };

enum cmp_types {
	CTYP_UNKNOWN=0,
	CTYP_V16_4x4,
	CTYP_H16_4x4, 
	CTYP_V16_4x2,
	CTYP_H16_4x2,
	CTYP_V16_2x4,
	CTYP_H16_2x4,
	CTYP_V16_2x2,
	CTYP_H16_2x2,
	CTYP_V24_4x4,
	CTYP_H24_4x4, 
	CTYP_V24_4x2,
	CTYP_H24_4x2,
	CTYP_V24_2x4,
	CTYP_H24_2x4,
	CTYP_V24_2x2,
	CTYP_H24_2x2,
	COMP_TYPES
};

/*#define CUSTOM_8DELTA	0xFF*/

#define DUCK_COPYRIGHT "\2522::5Evdl\257"

#define RED565   0xF800L
#define GREEN565 0x07E0L
#define BLUE565  0x001FL

int readini(DUCKINI *dini);

#define DUCK_INTERNAL_OLD	0
#define SEGA_1				1
#define PUBLIC_VERSION		2		/*played by any coduck*/
#define DUCK_INTERNAL		3
#define RESERVED			4

#define		___		0x0L
#define		X__		0x00000ffffL
#define		__X		0x0ffff0000L
#define		X_X		0x0ffffffffL

#if defined(DEBUG)
#define DIAGNOSTIC(x) x
#else
#define DIAGNOSTIC(x)
#endif

typedef struct DXINFOSTRUCT{                     
	int imwidth;
	int imheight; 
	int blockpower;
	int lpbmione;
	int block2x;
	unsigned char *vectbl;
	int hinterp;
	int interframe;
	int iskeyframe;
	int sprite;
	int bitcnt;
	int hdrSize; 
	int drawing;
	int fmt;          
	FRAMEHEADER f;
	int algorithm;
} dxInfoStruct;

#if defined(__cplusplus)
}
#endif 
#endif
