/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// VESA.H -- symbolic constants for VESA BIOS function calls

// VESA info block structure
typedef _Packed struct
//typedef struct
{
  char              VbeSignature[4];    /* 'VESA' 4 byte signature */
  short             VbeVersion;         /*  VBE version number */
  char              *OemStringPtr;      /* Pointer to OEM string */
  unsigned long     Capabilities;       /* Capabilities of video card */
  unsigned short    *VideoModePtr;      /* Pointer to supported modes */
  short             TotalMemory;        /* Number of 64kb memory blocks */

  short             OemSoftwareRev;     /* OEM Software revision number */
  char              *OemVendorNamePtr;  /* Pointer to Vendor Name string */
  char              *OemProductNamePtr; /* Pointer to Product Name string */
  char              *OemProductRevPtr;  /* Pointer to Product Revision str */
  char              reserved[222];      /* Pad to 256 byte block size */
  char              OemDATA[256];       /* Scratch pad for OEM data */

} VESAINFOBLOCK;


#define	VBE_CAPS_8BITDAC	(1<<0)	// DAC can be 8 bit
#define	VBE_CAPS_NONVGA	 	(1<<1)	// Non-vga compatible controller
#define	VBE_CAPS_BLANKRAMDAC 	(1<<2)	// Programmed DAC with blank bit


// VESA mode info block structure
typedef _Packed struct
{
  // Mandatory information for all VBE revisions:

  unsigned short  ModeAttributes;        // mode attributes
  unsigned char	  WinAAttributes;        // window A attributes
  unsigned char   WinBAttributes;        // window B attributes
  unsigned short  WinGranularity;        // window granularity
  unsigned short  WinSize;               // window size
  unsigned short  WinASegment;           // window A start segment
  unsigned short  WinBSegment;           // window B start segment
  unsigned long	  WinFuncPtr;            // pointer to window function
  unsigned short  BytesPerScanLine;      // bytes per scan line

  // Mandatory information for VBE 1.2 and above:

  unsigned short  XResolution;           // horizontal resolution in pixels or chars
  unsigned short  YResolution;           // vertical resolution in pixels or chars
  unsigned char   XCharSize;             // character cell width in pixels
  unsigned char	  YCharSize;             // character cell height in pixels
  unsigned char	  NumberOfPlanes;        // number of memory planes
  unsigned char	  BitsPerPixel;          // bits per pixel
  unsigned char	  NumberOfBanks;         // number of banks
  unsigned char	  MemoryModel;           // memory model type
  unsigned char	  BankSize;              // bank size in KB
  unsigned char	  NumberOfImagePages;    // number of images
  unsigned char	  _Reserved;             // reserved for page function

  // Direct Color fields (required for direct/6 and YUV/7 memory models)

  unsigned char	  RedMaskSize;           // size of direct color red mask in bits
  unsigned char	  RedFieldPosition;      // bit position of lsb of red mask
  unsigned char	  GreenMaskSize;         // size of direct color green mask in bits
  unsigned char	  GreenFieldPosition;    // bit position of lsb of green mask
  unsigned char	  BlueMaskSize;          // size of direct color blue mask in bits
  unsigned char	  BlueFieldPosition;     // bit position of lsb of blue mask
  unsigned char	  RsvdMaskSize;          // size of direct color reserved mask in bits
  unsigned char	  RsvdFieldPosition;     // bit position of lsb of reserved mask
  unsigned char	  DirectColorModeInfo;   // direct color mode attributes

  // Mandatory information for VBE 2.0 and above:

  unsigned long	  PhysBasePtr;           // physical address for flat frame buffer
  unsigned long	  OffScreenMemOffset;    // pointer to start of off screen memory
  unsigned short  OffScreenMemSize;      // amount of off screen memory in 1k units
  char		  __Reserved[206];

} MODEINFOBLOCK;


/* Flags to combine with mode number */

#define	VBE_MODEFLAG_NOCLEAR	0x8000	/* Dont clear screen on mode set *
#define	VBE_MODEFLAG_LFB	0x4000	    /* Request LFB */

/* Mode attributes flags */
#define VBEMODE_CAPS_AVAILABLE  (1<<0)
#define	VBEMODE_CAPS_TTYOUTPUT  (1<<2)
#define	VBEMODE_CAPS_COLORMODE  (1<<3)
#define	VBEMODE_CAPS_GRAPHMODE  (1<<4)
#define	VBEMODE_CAPS_NONVGA     (1<<5)	
#define	VBEMODE_CAPS_NONBANKED  (1<<6)
#define	VBEMODE_CAPS_LINEAR     (1<<7)	

/* Mode memory models */
#define	VBEMODE_MODEL_TEXT      (0)	
#define	VBEMODE_MODEL_CGA       (1)	
#define	VBEMODE_MODEL_HERCULES  (2)
#define	VBEMODE_MODEL_PLANAR    (3)
#define	VBEMODE_MODEL_PACKED    (4)
#define	VBEMODE_MODEL_MODEX     (5)
#define	VBEMODE_MODEL_RGB       (6)
#define	VBEMODE_MODEL_YUV       (7)
