/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _duck_hfb_h
#define _duck_hfb_h

#if defined(__cplusplus)
extern "C" {
#endif

#include "duck_bmp.h"
#include "duck_wav.h"

enum dukDirect {             /* direction to move within stream */
    DUK_BACKWARD=-1,         /* move backward from current position */
    DUK_ABSOLUTE=0,          /* move to an absolute position */
    DUK_FORWARD=1,            /* move forward from current position */
    DUK_RESET=2              /* reset stream according to file position */
};

enum HFB_FileType {          /* type of file opened */
    DUK_NULFILE = 0,         /* not a TrueMotion "S" A/V file */
    DUK_DUKFILE = 1,         /* TrueMotion "S" duk file */
    DUK_AVIFILE = 2,         /* TrueMotion "S" Video for Windows AVI compatible file */ 
    DUK_QTFILE = 3           /* TrueMotion "S" Quicktime MOV compatible file */
};

enum HFB_StreamType {        /* stream types */ 
    DUK_UNDEFINED = 0,       /* indeterminate or uninitialized stream */
    DUK_AUDSTREAM = 1,       /* audio stream */
    DUK_VIDSTREAM = 2,       /* video stream */
    DUK_TXTSTREAM = 3        /* text stream */
};

enum HFB_Modes {             /* HFB buffering modes */
    HFBMODE_NORMAL = 0,      /* normal play once mode */
    HFBMODE_FORWARDLOOP = 1, /* forward looping mode (loop back to start) */
    HFBMODE_REVERSE = 2,     /* reverse play once mode (not yet implemented)*/
    HFBMODE_REVERSELOOP = 3, /* reverse looping mode (not yet implemented)*/
    HFBMODE_QUEUE = 4        /* file queue mode (not yet implemented)*/
};

enum mv_returns { DUK_END = 0, DUK_FULL, DUK_MAXREAD, DUK_INCOMPLETE };

#define HFB_USE_DEFAULT		0L
#define HFB_RESET_COUNT		-1L
#define HFB_IGNORE_COUNT	-2L

#define HFB_DATA_READ		0x01
#define HFB_DATA_RELEASED	0x02
#define HFB_DATA_KEYFRAME	0x08

enum FTYPE { 
	HFB_FRAMENUM = 0, 
	HFB_INDEXNUM = 1
};

typedef struct tHFB_STREAM_INFO{
	enum HFB_StreamType streamType;
	int streamNum;
	long lSampleRate;
	long lStart;
	long lLength;
	char szName[24];
	union { 
		DKWAVEFORM WaveformInfo; 
		DKBITMAP BitmapInfo;
	} a;
} HFB_STREAM_INFO, MFP_STREAM_INFO;

typedef struct tHFB_FILE_INFO{
	long lFileSize;
	long lNumStreams;
	char sName[96];
	long lNumFrames;
} HFB_FILE_INFO, MFP_FILE_INFO;

typedef struct tHFBFile *HFB_FILE_HANDLE;
typedef struct tHFBStream *HFB_STREAM_HANDLE;
typedef struct tHFBBuffer *HFB_BUFFER_HANDLE;

typedef int HFB_DATA_HANDLE;

#define DCK_DEFAULT_READSIZE 0
#define DCK_DEFAULT_BUFFSIZE 0

#define HFB_GetStreamLength(strmh) HFB_GetStreamInfo(strmh)->lLength
#define HFB_GetStreamName(strmh) HFB_GetStreamInfo(strmh)->szName

DKWAVEFORM *HFB_GetAudioInfo(HFB_STREAM_HANDLE ,int *,int *,int *,int *);
int HFB_GetInitialFrames(HFB_STREAM_HANDLE ,HFB_STREAM_HANDLE );
int HFB_GetSamplesPerFrame(HFB_STREAM_HANDLE ,HFB_STREAM_HANDLE );
void HFB_GetFrameRates(HFB_STREAM_HANDLE ,HFB_STREAM_HANDLE ,int *,int *);
void HFB_LoadIndex(HFB_FILE_HANDLE ,HFB_BUFFER_HANDLE );
int HFB_ParseFile(HFB_FILE_HANDLE ,HFB_BUFFER_HANDLE );

int HFB_Init(int ,int ,int );
void HFB_Exit(void);

HFB_FILE_HANDLE HFB_FindFile(const char *);
HFB_FILE_HANDLE HFB_OpenFile(const char *,HFB_BUFFER_HANDLE );
void HFB_CloseFile(HFB_FILE_HANDLE );
HFB_FILE_INFO *HFB_GetFileInfo(HFB_FILE_HANDLE );
void HFB_ResetStreams(HFB_BUFFER_HANDLE );
HFB_STREAM_HANDLE HFB_GetStream(HFB_FILE_HANDLE ,const char *,int ,unsigned long );
void HFB_ReleaseStream(HFB_STREAM_HANDLE );
HFB_STREAM_INFO *HFB_GetStreamInfo(HFB_STREAM_HANDLE );
HFB_BUFFER_HANDLE HFB_CreateBuffer(long ,long );
void HFB_DestroyBuffer(HFB_BUFFER_HANDLE );
int HFB_ResetBuffer(HFB_BUFFER_HANDLE ,long ,long );
int HFB_SetBufferMode(HFB_BUFFER_HANDLE ,enum HFB_Modes );
HFB_FILE_HANDLE HFB_QueueOpenFile(const char *,HFB_BUFFER_HANDLE );

long HFB_GetBufferPerCentFull(HFB_BUFFER_HANDLE );
long HFB_GetmovieSize(HFB_BUFFER_HANDLE );
void HFB_InitBuffer(HFB_BUFFER_HANDLE ,HFB_FILE_HANDLE ,int ,int );
long HFB_GetBufferSpace(HFB_BUFFER_HANDLE );
int HFB_FillBuffer(HFB_BUFFER_HANDLE ,long ,long );
int HFB_getBufferStatus(HFB_BUFFER_HANDLE );

HFB_DATA_HANDLE HFB_GetStreamingData(HFB_STREAM_HANDLE ,void **,long *,enum dukDirect ,int );
void HFB_ReleaseStreamingData(HFB_BUFFER_HANDLE ,HFB_DATA_HANDLE );

int HFB_ReadData(HFB_STREAM_HANDLE ,void *,long *,enum dukDirect ,int );

int HFB_FramestoNextKeyFrame(HFB_STREAM_HANDLE ,int ,int *);
int HFB_FrameToChunk(HFB_STREAM_HANDLE ,int );
int HFB_PreviousKeyFrame(HFB_STREAM_HANDLE ,int );
int HFB_GetIndexFlags(HFB_STREAM_HANDLE ,enum FTYPE ,int );

long HFB_GetDataPosition(HFB_STREAM_HANDLE ,HFB_DATA_HANDLE );

///	dbm -- note caps! was: long HFB_getDataPosition(HFB_STREAM_HANDLE ,HFB_DATA_HANDLE );

int HFB_WhatsAhead(HFB_STREAM_HANDLE ,int ,int *);
int HFB_ConditionBuffer(HFB_BUFFER_HANDLE ,long ,long );

HFB_STREAM_INFO *HFB_GetStreamInfo(HFB_STREAM_HANDLE );
HFB_BUFFER_HANDLE HFB_GetStreamParentBuffer(HFB_STREAM_HANDLE );
HFB_FILE_HANDLE HFB_GetStreamParentFile(HFB_STREAM_HANDLE);

void HFB_GetStreamRateAndScale(HFB_STREAM_HANDLE xStream,long *rate, long *scale);
unsigned long HFB_GetStreamFCCs(HFB_STREAM_HANDLE,unsigned long *, unsigned long *);
int HFB_GetStreamSampleSize(HFB_STREAM_HANDLE xStream);

/* for backward compatibility */
#define MFP_Init HFB_Init
#define MFP_Exit HFB_Exit
#define MFP_OpenFile HFB_OpenFile

#define MFP_FindFile HFB_FindFile
#define MFP_ParseFile HFB_ParseFile
#define MFP_LoadIndex HFB_LoadIndex

#define MFP_CloseFile HFB_CloseFile
#define MFP_ReleaseStream HFB_ReleaseStream
#define MFP_GetStream HFB_GetStream
#define MFP_GetSamplesPerFrame HFB_GetSamplesPerFrame
#define MFP_GetAudioInfo HFB_GetAudioInfo
#define MFP_GetInitialFrames HFB_GetInitialFrames
#define MFP_GetFrameRates HFB_GetFrameRates
#define MFP_GetStreamInfo HFB_GetStreamInfo
#define MFP_GetFileInfo HFB_GetFileInfo
#define MFP_ReadData HFB_ReadData

typedef struct tHFBFile *MFP_FILE_HANDLE;
typedef struct tHFBStream *MFP_STREAM_HANDLE;
typedef struct tHFBBuffer *MFP_BUFFER_HANDLE;

typedef int MFP_DATA_HANDLE;

/* windows 95 dll system abstraction functions */

#include <stddef.h>

void HFB_Setmalloc(void *(*ptr)(size_t));
void HFB_Setcalloc(void *(*ptr)(size_t, size_t));
void HFB_Setfree(void (*ptr)(void *));

void HFB_Setopen(int (*ptr)(const char *, int,...));
void HFB_Setclose(int (*ptr)(int));
void HFB_Setread(int (*ptr)(int,void *, unsigned int));
void HFB_Setseek(long (*ptr)(int,long,int));

#if defined(__cplusplus)
}
#endif


#endif 
