/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _blt_itrp_h
#define _blt_itrp_h

#define HOT_MASK 0x7BDF7BDF
#define HOT_FUDGE 0x04200420 

#define HOT_MASK16 0x7BDF
#define HOT_FUDGE16 0x0420

/* take 2 qty of 16bit pixels = 32bits and interpolate using hot fudge factor */
#define INTERPOLATE32(src1,src2) \
			(*(src1) & *(src2) & HOT_FUDGE) + \
		     (  	(*(src1)  & HOT_MASK) +   \
							(*(src2) & HOT_MASK ))/2; 
							
							
/* take 1 qty of 16bit pixels = 16bits and interpolate using hot fudge factor */							
#define INTERPOLATE16(src1,src2) \
			((*(src1) & *(src2) & HOT_FUDGE16) + \
		     (  	(*(src1)  & HOT_MASK16) +   \
							(*(src2) & HOT_MASK16 ))/2); 
							



#define STRETCH_16BITPIXEL_32BITPTR(dxSrc,dest32,temp) \
			{ \
			temp = *dxSrc++; \
			dest32[1] = ((U16)temp + (temp << sixteen)); \
			temp &= 0xffff0000; \
			dest32[0] = (temp + (temp >> sixteen)); \
			}
			

#define STRETCH_16BITPIXEL_32BITPTR_WITHVINTERP(dxSrc,dxLast,dest32,temp) \
			{ \
			temp =  INTERPOLATE32(dxSrc,dxLast); \
			dxSrc++; \
			dxLast++; \
			dest32[1] = ((U16)temp + (temp << sixteen)); \
			temp &= 0xffff0000; \
			dest32[0] = (temp + (temp >> sixteen)); \
			}
			
			
/* This stretches the first pixel by three in order to avoid the */
/* read-ahead wrap around inherent in the following sequence ... */
/*   0, 0, (0 + 1)/2 , 1, 1, (1 + 2)/2, ...                      */
/*                                                               */
/*  see how writing the first 6 pixels looks at the first 3      */
/*  source pixels. That's bad, cause it means you will wrap      */
/*  wrap around.                                                 */
/*  The logical alternative is to treat the first some dest      */
/*  pixels separately, then blit the rest via formula shown above*/ 

#define BLIT_SOME_DESTPIX(dest16,dxSrc)  \
   *dest16++ = *dest16++ = *dest16++ = *dxSrc; \
   BLIT_REST_DESTPIX(dest16,dxSrc);\
   BLIT_REST_DESTPIX(dest16,dxSrc);\
   BLIT_REST_DESTPIX(dest16,dxSrc);

  
#define BLIT_REST_DESTPIX(dest16,dxSrc) \
      *dest16++ = INTERPOLATE16(dxSrc,(dxSrc + 1)); \
	  dxSrc++; \
	  *dest16++ = *dest16++ = *dxSrc;
 
#define BLIT_REST_DESTPIX_USING_LAST(dest16,dxSrc,dxLast,temp) \
      temp = INTERPOLATE16(dxSrc,(dxSrc + 1));\
	  temp2 = INTERPOLATE16(dxLast,(dxLast + 1));\
      *dest16++ = INTERPOLATE16(&temp,&temp2);\
	  dxSrc++;\
	  dxLast++;\
	  dest16[0] = INTERPOLATE16(dxSrc,dxLast);\
	  dest16[1] = dest16[0];\
	  dest16 += 2;

#define BLIT_SOME_DESTPIX_USING_LAST(dest16,dxSrc,dxLast,temp)  \
   *dest16++ = *dest16++ = *dest16++ = INTERPOLATE16(dxSrc,dxLast); \
   BLIT_REST_DESTPIX_USING_LAST(dest16,dxSrc,dxLast,temp);\
   BLIT_REST_DESTPIX_USING_LAST(dest16,dxSrc,dxLast,temp);\
   BLIT_REST_DESTPIX_USING_LAST(dest16,dxSrc,dxLast,temp);




#endif					
