;/***********************************************\
;??? v_interp.ash
; contains various vertical interp related macros
;\***********************************************/ 

EXTRN __5X5_MASK:QWORD
EXTRN __5X5_SAVEBITS:QWORD

;************************************************
LOAD_5X5MASKS MACRO
    lea         edx,__5X5_MASK
    movq        mm7,[edx]
    lea         edx,__5X5_SAVEBITS
    movq        mm6,[edx]
ENDM

;************************************************
; write 4 pixel pairs from dxLast, interpolate with
; 4 pixel pairs from dxDest and write interped pixels
;
; mm7/mm6 contains 5x5mask/5x5savebit patterns
;************************************************
_4_PIXEL_PAIRS_555 MACRO
    movq        mm0,[esi]           ;4 pixels
    movq        mm1,mm0
    punpcklwd   mm0,mm1             ;unpack low pixels
    movq        [edi],mm0           ;2 stretched pixels
    movq        mm0,mm1             ;ready for high pixels
    punpckhwd   mm0,mm1             ;unpack high pixels
    movq        [edi+8],mm0         ;2 stretched pixels

    movq        mm2,[ebp]
    movq        mm3,mm2
    pand        mm3,mm1
    pand        mm3,mm6             ;save fudge factor

    pand        mm1,mm7
    pand        mm2,mm7
    paddw       mm2,mm1
;----------------------------------------------------------------------
;problem will occur with 565 here
;going to loose a bit
;----------------------------------------------------------------------
    psrlw       mm2,1               ;last+current/2
    paddw       mm2,mm3             ;add in fudge factor

    movq        mm1,mm2
    punpcklwd   mm2,mm1             ;unpack low pixels
    movq        [edi+ecx],mm2
    movq        mm2,mm1
    punpckhwd   mm2,mm1             ;unpack high pixels
    movq        [edi+ecx+8],mm2
ENDM

;************************************************
; write 4 pixel pairs from dxLast, interpolate with
; 4 pixel pairs from dxDest and write interped pixels
;
; mm7/mm6 contains 5x5mask/5x5savebit patterns
;************************************************
_4_PIXEL_PAIRS_565 MACRO
    movq        mm0,[esi]           ;4 pixels
    movq        mm1,mm0
    punpcklwd   mm0,mm1             ;unpack low pixels
    movq        [edi],mm0           ;2 stretched pixels
    movq        mm0,mm1             ;ready for high pixels
    punpckhwd   mm0,mm1             ;unpack high pixels
    movq        [edi+8],mm0         ;2 stretched pixels

    movq        mm2,[ebp]
    movq        mm3,mm2
    pand        mm3,mm1
    pand        mm3,mm6             ;save fudge factor

    pand        mm1,mm7
    pand        mm2,mm7

    punpcklwd   mm4,mm1
    punpckhwd   mm5,mm2


    paddw       mm2,mm1
;----------------------------------------------------------------------
;problem will occur with 565 here
;going to loose a bit
;----------------------------------------------------------------------
    psrlw       mm2,1               ;last+current/2
    paddw       mm2,mm3             ;add in fudge factor

    movq        mm1,mm2
    punpcklwd   mm2,mm1             ;unpack low pixels
    movq        [edi+ecx],mm2
    movq        mm2,mm1
    punpckhwd   mm2,mm1             ;unpack high pixels
    movq        [edi+ecx+8],mm2
ENDM

if 0
;************************************************
; (1) 16 bit pixel read from source
_1_16BIT_PIXEL_PAIR MACRO _5x5_savebits, _5x5_mask           
    mov     ebx,[esi]                   ;get last line pixel pair 
    mov     eax,ebx                     ;save last line pixel pair
    shl     ebx,16
    mov     bx,ax
    mov     [edi],ebx
    mov     ebx,[ebp]                   ;get current line pixel pair
    mov     edx,ebx                     ;save current line pixel pair
    and     ebx,eax
    and     ebx,_5x5_savebits           ;save fudge factor

    and     edx,_5x5_mask
    and     eax,_5x5_mask
    add     eax,edx
;    shr     eax,1                       ;last+current/2
    ;for 565
    rcr     eax,1                       ;last+current/2
    add     eax,ebx                     ;add in fudge factor
    mov     ebx,eax
    shl     eax,16
    mov     ax,bx
    mov     [edi+ecx],eax               ;write out stretched vinterped pixel
    add     esi,2  
    add     ebp,2
    add     di,4
ENDM

;************************************************
WR_REMAINING_PIXEL_PAIRS MACRO _5x5_savebits, _5x5_mask
LOCAL wr_one_pixel, no_remaining_pixels

    cmp     DWORD PTR[esp+PIXELS_LEFT],0h         ;check for remaining pixels
    jz      no_remaining_pixels
wr_one_pixel:                                       ;occurs when page does not align with
    _1_16BIT_PIXEL_PAIR _5x5_savebits, _5x5_mask        ;4 pixel boundary
    dec     DWORD PTR[esp+PIXELS_LEFT]
    jnz     wr_one_pixel
no_remaining_pixels:        
ENDM

;************************************************
DO_SMART_LINE MACRO _5x5_savebits, _5x5_mask, destAddr_reg
LOCAL do_block, LessThanEqual32, blcopyloop, blskipcopy, black_out, block_exit
do_block:
    cmp     DWORD PTR[esp+BLKLOOPCNT],32
    jle     LessThanEqual32
    mov     DWORD PTR[esp+BLKLOOPCNT],32
LessThanEqual32:
    mov     ebx,[esp+BLKLOOPCNT]
    sub     [esp+BLKWIDTH],ebx

    mov     ebx,[esp+UBLOC] 
    mov     eax,[ebx]
    mov     [esp+UB_DATA],eax
    add     ebx,4
    mov     [esp+UBLOC],ebx             

    mov     ebx,[esp+UBLOC_LAST] 
    mov     eax,[ebx]
    mov     [esp+UB_DATA_LAST],eax
    add     ebx,4
    mov     [esp+UBLOC_LAST],ebx            
    
blcopyloop:                                     ; this copies a block, if we want to
    rcr     DWORD PTR[esp+UB_DATA],1
    setb    al
    rcr     DWORD PTR[esp+UB_DATA_LAST],1
    setb    ah
    and     al,ah
    jnz     black_out

    _4_PIXEL_PAIRS
;jmp blskipcopy

black_out:
;    mov DWORD PTR[edi],0
;    mov DWORD PTR[edi+4],0
;    mov DWORD PTR[edi+ecx],0
;    mov DWORD PTR[edi+ecx+4],0
;    mov DWORD PTR[edi+8],0
;    mov DWORD PTR[edi+12],0
;    mov DWORD PTR[edi+ecx+8],0
;    mov DWORD PTR[edi+ecx+12],0

blskipcopy:
    add     esi,8  
    add     ebp,8
    add     destAddr_reg,16 
    dec     DWORD PTR[esp+BLKLOOPCNT]
    jnz     blcopyloop

    cmp     DWORD PTR[esp+BLKWIDTH],0
    jle     block_exit

    mov     ebx,[esp+BLKWIDTH]              ;ready to do some more
    mov     [esp+BLKLOOPCNT],ebx

    jmp     do_block
block_exit:
ENDM
endif
