/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "rename.h"
#include "dkpltfrm.h"

#include "duktypes.h"
#include "cduck.h"       

extern short dt[8];
extern short cdt[8];
extern short fdt[8];
extern short fcdt[8];
					    
/*
 *
 * This program,is PROPRIETARY and CONFIDENTIAL material of
 * The Duck Corporation!
 *
 * void maketbl (short *dts,short *cdts,char *dvfile)
 * 
 * fills in dxtbl[] for dx32.asm to use, getting pertinent facts from:
 * dts[], an array of Y deltas
 * cdts[], an array of color deltas
 * dvfile, a filename for the .DAT file of vectors used to compress
 *
 */

#define BITSET   0x00000001L
#define BITCLEAR 0xFFFFFFFEL
#define BITSHIFT 1L

#define VECCNT  256
#define VECLEN  15
#define TBL_LEN 4096*4

typedef uchar (*tbltype)[VECLEN+1];

uchar vectbl[TBL_LEN];

static signed short protodt[8]={0,-1,2,-3,4,-6,6,0};     /* WATCOM unsigned default! */
static signed short protocdt[8]={0,-1,1,-2,3,-4,5,0};
static signed short protofdt[8]={0,-1,1,-2,3,-4,5,0};
static signed short protofcdt[8]={0,-1,1,-2,3,-4,5,0};

/* some free space for dx:*/
unsigned long blockSpace[8];

extern int is565;

#ifdef SUPPORT565
static short GREENSHIFT = 5;
static short REDSHIFT = 10;
#else
#define GREENSHIFT 5
#define REDSHIFT 10
#endif

#if !SWAPENDS
static long proto2dt(long p1,long p2) {
#else
static long proto2dt(long p2,long p1) {
#endif
	long lo,hi;

	lo=protodt[p1];
	lo=lo + (lo<< GREENSHIFT) + (lo<<REDSHIFT);
	hi=protodt[p2];
	hi=hi + (hi<< GREENSHIFT) + (hi<<REDSHIFT);

	return((lo+(hi<<16))<<BITSHIFT);
}


#if (COLORORDER == RGBORDER)
static long protoC2dt(long p1,long p2) {
#else
static long protoC2dt(long p2,long p1) {
#endif
	long r,b,lo;

	b=protocdt[p2];
	r=protocdt[p1]<<REDSHIFT;

	lo=b+r;
		return((lo+(lo<<16))<<BITSHIFT);
}

static long proto2dt24_2(long p2,long p1) {
	long lo,hi;

	lo=protodt[p1];         
	hi=protodt[p2];         
	return((lo+(hi<<8))<<BITSHIFT);
}

#if !SWAPENDS
static long proto2alpha(long p1,long p2) {
#else
static long proto2alpha(long p2,long p1) {
#endif
	return((p1+(p2*5))<<BITSHIFT);
}

#if (COLORORDER==RGBORDER)
static long protoC2dt24(long p1,long p2) {
#else
static long protoC2dt24(long p2,long p1) {
#endif
	long r,b;

	b=protocdt[p2];
	r=protocdt[p1]<<16;

	return ((b+r)<<BITSHIFT);
}

static long proto2dt24_2fx(long p2,long p1) {
	long lo,hi;
	/*
	lo=(p1) ? protodt[p1]*5 : 0;
	hi=(p2) ? protodt[p2]*5 : 0;
    */
	lo=protofdt[p1];
	hi=protofdt[p2];

	return((lo+(hi<<8))<<BITSHIFT);
}

#if (COLORORDER==RGBORDER)
static long protoC2dt24fx(long p1,long p2) {
#else
static long protoC2dt24fx(long p2,long p1) {
#endif
	long r,b;
    
    /*
	b=(p2) ? protocdt[p2]*5 : 0;
	r=(p1) ? (protocdt[p1]*5)<<16 : 0;
    */
	b=protofcdt[p2];
	r=protofcdt[p1]<<16;

	return ((b+r)<<BITSHIFT);
}
#if CENTER_TABLE
static void centerTbl(void)
{
	int i;
	unsigned long saveVec;
	register unsigned long *dvTblPtr = (unsigned long *)vectbl;
	
	for(i = 0; i < 512; i++, dvTblPtr++){
		saveVec = dvTblPtr[0];
		dvTblPtr[0] = dvTblPtr[512];
		dvTblPtr[512] = saveVec; 
		
		saveVec = dvTblPtr[1024];
		dvTblPtr[1024] = dvTblPtr[512+1024];
		dvTblPtr[1024+512] = saveVec;
	
		saveVec = dvTblPtr[2048];
		dvTblPtr[2048] = dvTblPtr[512+2048];
		dvTblPtr[2048+512] = saveVec;         

		saveVec = dvTblPtr[3072];
		dvTblPtr[3072] = dvTblPtr[512+3072];
		dvTblPtr[3072+512] = saveVec;
	} 
}
#endif

#include "pctbl4c.h"

void maketbl (signed short *dts,signed short *cdts,unsigned char *lvectbl)
{
    unsigned int i,j;
	int length;
	long temp;
	register unsigned long *deltavectbl = (unsigned long *)vectbl;
	
#ifdef SUPPORT565
	if (is565){
		GREENSHIFT = 6;
		REDSHIFT = 11;
	}else{
		GREENSHIFT = 5;
		REDSHIFT = 10;
	}
#endif

	for(i=0;i<8;i++) {
		protodt[i]=dts[i];
		protocdt[i]=cdts[i];
	}
    
	for(i=0; i<1024; i+=4){
		length = *lvectbl/2;
		lvectbl++;
			
		for(j=0;(short) j<length; j++){
			temp = *lvectbl++;
			deltavectbl[i+j] =
				BITCLEAR & (proto2dt ((long)temp>>4,(long)(temp & 0x0f)));
			deltavectbl[i+j+1024] =
				BITCLEAR & (protoC2dt((long)temp>>4,(long)(temp & 0x0f))); 
			deltavectbl[i+j+2048] =
				BITCLEAR & (proto2alpha((long)temp>>4,(long)(temp & 0x0f)));
		}
        deltavectbl[i+(j-1)] |= BITSET;
		deltavectbl[(i+(j-1))+1024] |= BITSET;
		deltavectbl[(i+(j-1))+2048] |= BITSET;
	}
#if CENTER_TABLE
    centerTbl();
#endif
}

void maketbl24_2 (signed short *dts,signed short *cdts,
	signed short *fdts,signed short *fcdts,unsigned char *lvectbl)
{
	register unsigned long *deltavectbl = (unsigned long *)vectbl;
    unsigned int i,j,length;
	unsigned short dtemp = (unsigned short)dts[0];
	unsigned long dt0 = dtemp;
	long temp;
	void dxSetdts24(long);

    dxSetdts24((long) dt0);

	for(i=0;i<8;i++) {
		protodt[i]=dts[i];
		protocdt[i]=cdts[i];
		protofdt[i]=fdts[i];
		protofcdt[i]=fcdts[i];
	}
	
	for(i=0; i<1024; i +=4){   
		length = *lvectbl/2;
		lvectbl++;
			
		for(j=0;j<length; j++){
			temp = *lvectbl++;

			deltavectbl[i+j] =
				BITCLEAR & (proto2dt24_2((long)temp>>4,(long)(temp & 0x0f))); 
			deltavectbl[i+j+1024] =
				BITCLEAR & (protoC2dt24((long)temp>>4,(long)(temp & 0x0f)));
			deltavectbl[i+j+2048] =
				BITCLEAR & (proto2dt24_2fx((long)temp>>4,(long)(temp & 0x0f)));
			deltavectbl[i+j+3072] =
				BITCLEAR & (protoC2dt24fx((long)temp>>4,(long)(temp & 0x0f)));
		}
        deltavectbl[i+(j-1)] |= BITSET;
		deltavectbl[(i+(j-1))+1024] |= BITSET;
		deltavectbl[(i+(j-1))+2048] |= BITSET;
		deltavectbl[(i+(j-1))+3072] |= BITSET;
	}
#if CENTER_TABLE
    centerTbl();
#endif
}

void
setup_dxdts2(I16 *dd, I16 *cd, I16 *fd, I16 *fcd)
{
	I16 i;
	
	for(i=0; i<8; i++) {
		*dd &= 0xFFFE;          /*DSS*/
		dt[i] = *dd++/2;
		cdt[i] = *cd++;
		fdt[i] = *fd++;
		fcdt[i] = *fcd++;
	}
}

int setup_dxdts(I16 *dd, I16 *cd, I16 *fd, I16 *fcd)
{
	I16 i, ret = 0;
	
	for(i=0; i<8; i++,dd++,cd++,fd++,fcd++) {
		*dd &= 0xFFFE;          /*DSS*/
		if ((dt[i] != (*dd/2)) || (cdt[i] != *cd)){
			dt[i] = *dd/2;
			cdt[i] = *cd;
			fdt[i] = *fd;
			fcdt[i] = *fcd;
			ret++;
		}
	}
	return ret;
}

