/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>

#include "rename.h"
#include "dkpltfrm.h"

#include "duktypes.h"
#include "cduck.h"

#include "duck_mem.h"
#include "dxl_main.h"                                            

unsigned long work1K[320*2];

int DXL_GetXImageType(DXL_XIMAGE_HANDLE src)
{
    validate(src);

    return ((DXL_XIMAGE_1HANDLE)src)->imType;
}

int DXL_AlterXImageSpriteModes(DXL_XIMAGE_HANDLE srcx, int sprMode, int sprColor)
{
	DXL_XIMAGE_1HANDLE src = (DXL_XIMAGE_1HANDLE) srcx;

	validate(src);

	src->sprMode = (enum BGMODE )sprMode;
	src->sprColor = (short) sprColor;

    return DXL_OK;
}   

unsigned short *DXL_GetXImageFrameBuffer(DXL_XIMAGE_HANDLE srcx)
{
    DXL_XIMAGE_1HANDLE src = (DXL_XIMAGE_1HANDLE)srcx;

	if (src){
        if (src->dkFlags.inUse){
            if (src->colorDepth == RGB555)
                return (unsigned short *)(src->lineBuffer + (src->imWidth*2));
        }
    }
    return NULL;
}

#if 0

int tm1_Init(int lmaxImages){ return 0; }
int tm1_Exit(void){ return 0; }

#else


static int tm1_xImageDestroy(DXL_XIMAGE_1HANDLE xThis)
{
    if (xThis->chromaBuffer) 
		duck_free(xThis->chromaBuffer);
    else if (xThis->lineBuffer){
        if (xThis->lineBuffer != (unsigned char *)work1K)
            duck_free(xThis->lineBuffer);
    }
    
    xThis->chromaBuffer = NULL;
    xThis->lineBuffer = NULL;

    xThis->dkFlags.inUse = 0;
    xThis->addr = NULL;

	duck_memset(xThis,0,sizeof(DXL_XIMAGE_1));

	return DXL_OK;
}
        
DXL_XIMAGE_HANDLE populateXImage(DXL_XIMAGE_HANDLE vImage)
{
    int byteDepth;
    DXL_XIMAGE_1HANDLE nImage = (DXL_XIMAGE_1HANDLE)vImage;

    nImage->dkFlags.inUse = 1;

    nImage->sprMode = NORM;
    nImage->sprColor = (short)0x8000;
    
    nImage->dxCount = 0;
    nImage->x = 0;
    nImage->y = 0;
    nImage->imWidth = (short) nImage->dxInfo.imwidth;               
    nImage->imHeight = nImage->h = (short) nImage->dxInfo.imheight;         

    nImage->chromaBufferSize = 0;
    nImage->chromaBuffer = (unsigned long *)NULL;
    

    nImage->lw = nImage->imWidth;
    nImage->lh = (nImage->dxInfo.interframe) ? nImage->imHeight : 0;

    if ((nImage->dxInfo.f.Type >= CTYP_V24_4x4) && (nImage->dxInfo.f.Type <= CTYP_H24_2x2)){
        
        nImage->lh = nImage->imHeight;

        byteDepth = 4;
        nImage->imType =(nImage->dxInfo.interframe) ? DXL_INTERFRAME : DXL_INTRAFRAME;
        
        nImage->lineBufferSize = nImage->imWidth*(nImage->imHeight+1);

        nImage->chromaBufferSize = nImage->imWidth/2;
        nImage->chromaBufferSize *= (nImage->imHeight/2)+1;
        
        nImage->chromaBuffer = (unsigned long *)duck_malloc(
            (nImage->chromaBufferSize*4) + nImage->lineBufferSize,DMEM_FRAMEBUFF);  
    
        if (nImage->chromaBuffer != NULL){
            duck_memset(nImage->chromaBuffer,0,(nImage->chromaBufferSize*4) + nImage->lineBufferSize);
            nImage->lineBuffer = (unsigned char *)(nImage->chromaBuffer + nImage->chromaBufferSize);
        }else
            nImage->lineBuffer = NULL;
    }else{
        byteDepth = 2;
        
        if (nImage->dxInfo.interframe){
            nImage->lineBuffer = (unsigned char *)duck_malloc(
                nImage->imWidth*(nImage->imHeight+1)*byteDepth,DMEM_FRAMEBUFF);
            nImage->lineBufferSize = nImage->imWidth*(nImage->imHeight+1)*byteDepth;
            nImage->imType = DXL_INTERFRAME;
        }else{
            /*
              create enough space for 2 lines 
               this way we can interpolate vertically as needed 
            */
            if ((nImage->imWidth*byteDepth*2) <= 1280){
                nImage->lineBuffer = (unsigned char *)work1K;
                nImage->lineBufferSize = 1280;
            }else{
                nImage->lineBuffer = (unsigned char *)duck_malloc(nImage->imWidth*byteDepth*2,DMEM_FRAMEBUFF);
                nImage->lineBufferSize = nImage->imWidth*byteDepth*2;
            }
            nImage->imType = (nImage->dxInfo.sprite) ? DXL_SPRITE : DXL_INTRAFRAME;
        }
    }
    nImage->w = (short)((nImage->dxInfo.hinterp) ? (nImage->dxInfo.imwidth*3)/2 : nImage->dxInfo.imwidth);  
    
    if (nImage->lineBuffer == NULL){
        DXL_DestroyXImage((DXL_XIMAGE_HANDLE)nImage);
        return NULL;
    }            
    return (DXL_XIMAGE_HANDLE)nImage;
}
 
static int tm1_xImageAlterData(DXL_XIMAGE_1HANDLE xThis)
{
	int ret = decodeHeader(xThis->addr,&(xThis->dxInfo));

	if (ret < 0) return DXL_BAD_DATA;

	xThis->dkFlags.nullFrame = (ret == 0);

	xThis->imWidth = (short) xThis->dxInfo.imwidth;               
	xThis->imHeight = (short) xThis->dxInfo.imheight;         

    if (xThis->dxInfo.sprite){
        xThis->w = xThis->dxInfo.f.Xsize;
        xThis->h = xThis->dxInfo.f.Ysize;
    }else{
		xThis->w = (short)((xThis->dxInfo.hinterp) ? 
			(xThis->dxInfo.imwidth*3)/2 : xThis->dxInfo.imwidth);
		xThis->h = (short) xThis->dxInfo.imheight;
	}

	if (xThis->dxInfo.interframe)
		xThis->imType = DXL_INTERFRAME;
	else
		xThis->imType = (xThis->dxInfo.sprite) ? DXL_SPRITE : DXL_INTRAFRAME;

    if ((xThis->dxInfo.f.metaType > 1) && (xThis->dxInfo.interframe))
        xThis->dkFlags.keyFrame = xThis->dxInfo.iskeyframe ? 1: 0;
    else
        xThis->dkFlags.keyFrame = 1;

    if (xThis->dxInfo.algorithm == RGB24H)
        xThis->colorDepth = RGB888;
	else
		xThis->colorDepth = (xThis->dxInfo.sprite) ? RGB555A : RGB555;

	return DXL_OK;
}

static DXL_INTERNAL_FORMAT tm1_GetXImageInternalFormat(DXL_XIMAGE_HANDLE xImage,
												DXL_VSCREEN_HANDLE vScreen)
{
    if (((((DXL_XIMAGE_1HANDLE)xImage)->dxInfo.f.Type >= CTYP_V24_4x4) && 
		(((DXL_XIMAGE_1HANDLE)xImage)->dxInfo.f.Type <= CTYP_H24_2x2)))
		return TM1_24;

	if (((DXL_XIMAGE_1HANDLE)xImage)->dxInfo.sprite){
		return DXL_LINE16spr;
	}else if (((DXL_XIMAGE_1HANDLE)xImage)->dxInfo.hinterp){
		return DXL_LINE16hi;
	}else if ((!DXL_IsXImageKeyFrame(xImage)) && (!xImage->dkFlags.doCompleteBlit)){
		return DXL_LINE16i;
	}

	return DXL_LINE16;
}

static DXL_XIMAGE_1HANDLE xImages;
static int maxImages;
    
static DXL_XIMAGE_1HANDLE tm1_xImageCreate(char *data)
{
	extern int tm1_xImageBlit(DXL_XIMAGE_1HANDLE,DXL_VSCREEN_HANDLE);
	extern DXL_XIMAGE_HANDLE tm1_AlterXImage(DXL_XIMAGE_HANDLE,void *,int,int,int,int);
	
	extern int tm1_xImageDx(DXL_XIMAGE_1HANDLE,DXL_VSCREEN_HANDLE);
	DXL_XIMAGE_1HANDLE nImage = xImages;
	int i;

	for(i=0; i < maxImages; i++,nImage++)
		if (!nImage->dkFlags.inUse) break;
    
	if ((i >= maxImages) || !nImage) return NULL;
    
	duck_memset(nImage,0,sizeof(DXL_XIMAGE_1));

	nImage->recreate = (DXL_XIMAGE_HANDLE (*)(DXL_XIMAGE_HANDLE,void *,int,int,int,int)) tm1_AlterXImage;
	nImage->destroy = (int (*)(DXL_XIMAGE_HANDLE)) tm1_xImageDestroy;
	nImage->seedData = (int (*)(DXL_XIMAGE_HANDLE)) tm1_xImageAlterData;
	nImage->blit = (int (*)(DXL_XIMAGE_HANDLE, DXL_VSCREEN_HANDLE)) tm1_xImageBlit;
	nImage->dx = (int (*)(DXL_XIMAGE_HANDLE, DXL_VSCREEN_HANDLE)) tm1_xImageDx;
    nImage->addr = (unsigned char *) data;

    nImage->internalFormat = (int (*)(DXL_XIMAGE_HANDLE, DXL_VSCREEN_HANDLE)) tm1_GetXImageInternalFormat; 

    nImage->dkFlags.inUse = 1;
    
    if (data != NULL){
        if (DXL_AlterXImageData((DXL_XIMAGE_HANDLE) nImage,(unsigned char *) data) == DXL_OK){
			if (!nImage->dkFlags.nullFrame){
				if(nImage->dxInfo.f.Type == COMP_TYPES && nImage->dxInfo.f.metaType == 5)
				{
					DXL_DestroyXImage((DXL_XIMAGE_HANDLE)nImage);
					return NULL;
				}else
					return (DXL_XIMAGE_1HANDLE ) populateXImage((DXL_XIMAGE_HANDLE) nImage);
			}
		}
        DXL_DestroyXImage((DXL_XIMAGE_HANDLE) nImage);
        return NULL;
    }
    return nImage;
}


void resetHdr(void); /* strict CodeWarrior reqmt. */
void tm1SetBlit(void);

int tm1_Init(int lmaxImages)
{
    tm1SetBlit();

    xImages = (DXL_XIMAGE_1HANDLE) duck_calloc(maxImages = lmaxImages,sizeof(DXL_XIMAGE_1),DMEM_GENERAL);
    if (xImages == NULL)
		return -1;

    DXL_RegisterXImage((DXL_XIMAGE_HANDLE (*)(unsigned char *)) tm1_xImageCreate,DXL_MKFOURCC('D','U','C','K'));

	resetHdr();

	return 0;
}

int tm1_Exit(void)
{
    int i;

	if (xImages){
        for(i = 0; i < maxImages; i++)
            DXL_DestroyXImage((DXL_XIMAGE_HANDLE ) &xImages[i]);
        duck_free(xImages);        
    }

    xImages = (DXL_XIMAGE_1HANDLE)NULL;

	return 0;

}

#endif
