/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#define COMP_GAMMA			1
#define DECOMP_GAMMA		1

#define MAX_PITCH			1280
#define MAX_DELTAS			8
#define CHECKCHECKSUM		0
#define RESTRICT_VERSION	0
#define METATYPE			3

//METATYPE == 0:		pre-pixel fixup (horde)
//METATYPE == 1:		pixel fixups
//METATYPE == 2:		Frameinfo field in header
//						control word in header
//						interframe supported
//METATYPE == 3:		SPRITES!
//METATYPE == 4:		4x2 sprites support BLK_NC blocks
//						interframe frames contain FrameNum in header
//						24 bit 4x2 interframe frames support new block types
//							BLK_NC, BLK_UP, changes to BLK_IF
//						24 bit intraframe and interframe use gamma correction
//						METATYPE 2 will still be used for 16 bit intraframe


//possible bit values for Control word in FrameHeader
#define CTL_DUCK			1
#define CTL_HTI				2
#define CTL_PC				4
#define CTL_MAC				8
#define CTL_3DO				16
#define CTL_SATURN			32

//Frameinfo bits
#define IS_SPRITE			32
#define IS_KEYFRAME			16
#define IS_INTERFRAME		8
#define	IS_INTERPOLATED		4

//block types
#define BLK_IF		0		//interframe (transparent) block
#define BLK_TM		1
#define BLK_TM44	1		//TrueMotion block types
#define BLK_TM22	2
#define BLK_LR44	3
#define BLK_LR22	4
#define BLK_AL		5		//alpha channel block (sprite)
#define BLK_UP		6		//update block (24 bit interframe)
#define BLK_NC		7		//no color block (24 bit interframe)
#define MAX_BLK		8

/*
 *	TrueMotion installable compressor for Video for Windows.
 *  Not to be reproduced in any form.
 *	Copyright (c) 1994  The Duck Corporation.  All Rights Reserved.
*/

typedef struct compStruct {
	U8 DXEnabled[METATYPE+1];
	U8 Algorithm;
	U8 NumDeltas;
	U8 HRes,VRes;
} COMPSTRUCT;

typedef struct duckini {
	I32	Magic,Version;
	I16 Algorithm;
	I16 Crop[8];
	I16 Clamp[4];
	I16 DUKGamma[2];
	I16 dummy1,dummy2;
	I16 ImageDither[4];		//use [0] for lower, [1] for cross, [2] for upper
	I16 ChromaRamp[4];
	I16 Ack24,YGamma,UVGamma,dummy3;
	I16 LookAhead;
	I16 Dimensions[6];
	I32 MaxFrameSize;
	I32 FrameNum;
	U32 ScreenAddr;
	U16 ScreenDim[3];
	I16 Saturation;
	I16 Contrast;
	I16 Brightness; 
	I16 HorFilter;  
	I16 Detail; 
	I16 Reserved[8];

	//KeyRate, Quality, KeyIncrement, KeyMax, KeyColThresh, KeyBigFrames,
	//FramesPerSec, DataratePerSec
	I16 IF[8];
	
	char KeyStr[128];
	U8	DeltaSet;
	U8	ColorResolution;
	U8	Cropping;
	U8	CVersion;
	U8	CompLevel;
	U8	Deinterlace;
	U8	SmoothChroma;
	U8	Resize;
	U8	KeyFrameFlag;
	U8	CompMode;
	U8	didcompress;
	U8	NoKeyFrames;
	U8	FrameDrop;
	U8	KeyRows;
	U8	DataRateControl;
	U8	AlphaBits;
	U8	InvertedAlpha;
	U8	displayflag;
	U8	PatternDither;
	U8	pad[7];
} DUCKINI;

//METATYPE 0 header
typedef struct oldframeheader {
	U32 Type;
	U32 NumBits;
	U16 Ysize;
	U16	Xsize;
	I16 deltas[8];
	I16 Cdeltas[8];
} OLDFRAMEHEADER;

//METATYPE 1 header
typedef struct frameheader0 {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
} FRAMEHEADER0;

//METATYPE 2 header
typedef struct frameheader2 {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
	U8	Frameinfo;
	U8	Control;
} FRAMEHEADER2;

//METATYPE 3 header (sprites only)
typedef struct frameheader_spr {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
	U8	Frameinfo;
	U8	Control;
	U16 xoff,yoff,width,height;
} FRAMEHEADER_SPR;

//METATYPE 4 header (non-sprite)
typedef struct frameheader {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
	U16	CheckSum;
	U8	CVersion;
	U8	metaType;
	U8	Frameinfo;
	U8	Control;
	U16 Gamma,Mult;
	U32 FrameNum;
} FRAMEHEADER;

// Algorithms
enum algorithms {
	RGB16V=1,
	RGB16H,
	RGB24H,
	MAX_ALGORITHMS
};

enum comp_styles {
	SMOOTH=0,
	DITHERED,
	COMPRESSION_STYLES
};

enum alpha_bits {
	EIGHT_BIT=0,
	ONE_BIT,
	ALPHA_BITS
};

enum comp_modes {
	CM_INTRAFRAME=0,
	CM_INTERFRAME,
	CM_SPRITE,
	COMPRESSION_MODES
};

enum color_resolutions {
	COLOR_RES_4x4=0,
	COLOR_RES_4x2,
	COLOR_RES_2x4,
	COLOR_RES_2x2,
	COLOR_RESOLUTIONS
};

enum table_nums {
	PC_TBL=0,
	PC_TBL2,
	PC_TBL3,
	PC_TBL4,
	TBL_MAX };

enum clevel_nums {
	CLEV_NORM=0,
	CLEV_HIGH,
	CLEV_MAX };

enum cmp_types {
	CTYP_UNKNOWN=0,
	CTYP_V16_4x4,
	CTYP_H16_4x4, 
	CTYP_V16_4x2,
	CTYP_H16_4x2,
	CTYP_V16_2x4,
	CTYP_H16_2x4,
	CTYP_V16_2x2,
	CTYP_H16_2x2,
	CTYP_V24_4x4,
	CTYP_H24_4x4, 
	CTYP_V24_4x2,
	CTYP_H24_4x2,
	CTYP_V24_2x4,
	CTYP_H24_2x4,
	CTYP_V24_2x2,
	CTYP_H24_2x2,
	COMP_TYPES
};

#define	TIGER_BETA	COMP_TYPES

//#define CUSTOM_8DELTA	0xFF

#define DUCK_COPYRIGHT "\2522::5Evdl\257"

#define RED565   0xF800L
#define GREEN565 0x07E0L
#define BLUE565  0x001FL

int readini(DUCKINI *dini);

#define DUCK_INTERNAL_OLD	0
#define SEGA_1				1
#define PUBLIC_VERSION		2		//played by any coduck
#define DUCK_INTERNAL		3
#define RESERVED			4

#define		___		0x0L
#define		X__		0x00000ffffL
#define		__X		0x0ffff0000L
#define		X_X		0x0ffffffffL

#if defined(DEBUG)
#define DIAGNOSTIC(x) x
#else
#define DIAGNOSTIC(x)
#endif
