/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*****************************************************************
******************************************************************
this call initializes real time decompression

	void dxinit(int 565mode);

Args:
	565mode			flag: 0=555, 1=565

Note:
	You can call dxinit() before each call to dx.  If the colormode
	has not changed, it will just return immediately.


******************************************************************
this is the main real time decompression call

	void dx(U8 far *dst_addr, U32 nada, U8 far *compbuf, U32 wh, U32 dstpitch);

Args:
	dst_addr		points to where decompressed image should go
	nada			flag:  0=normal, 1=stretch
	compbuf			pointer to compressed data (AFTER HEADER)
	wh				a longword containing width in the low word 
					and height in the high word, i.e. (width | (height<<16))
	dstpitch		destination framebuffer pitch in bytes


******************************************************************
there is no cleanup call



******************************************************************
reading the header

TMRT files use the TMRT FOURCC code.

The header is similar to the normal truemotion header but contains
minimal useful information.

Assuming xxdata is a pointer to a chunk of compressed data from a
video stream, comprising one frame, this code is all that is necessary
to completely decode the header:

typedef struct {
	U8	hdrSize;
	U8	Type;
	U8	DeltaSet;
	U8	Table;
	U16 Ysize;
	U16 Xsize;
} FRAMEHEADER_SHORT;

	int i;
	FRAMEHEADER_SHORT *frmh;
	char buffer[128];
	int hdrSize;

	// get the header size from the first byte
	frmh = (FRAMEHEADER_SHORT *)xxdata;
	hdrSize = (U8)((frmh->hdrSize >> 5) | (frmh->hdrSize << 3)) & 0x7F;
        
	//decrypt the rest of the header
	for (i = 1; i < hdrSize; i++) buffer[i] = xxdata[i] ^ xxdata[i+1];

	//now point frmh at the decrypted header in the temporary buffer
	frmh = (FRAMEHEADER *)buffer;

	//width, height and type are now available
	imwidth = frmh->Xsize;
	imheight = frmh->Ysize;
	type = frmh->Type;

This code is already in the existing DecodeHeader() routine, I just want
to show how little is really necessary.

The "Type" field will be set to "5" for TMRT files.

******************************************************************
*****************************************************************/

#include <stdio.h>
#include "dkpltfrm.h"
#include "duktypes.h"

//I found this name mangling to be necessary
#define _clear_linebuf	clear_linebuf_
#define	_dxy			dxy_
#define	_dxu			dxu_
#define	_dxv			dxv_
#define	_dxy2x2			dxy2x2_

#define MAXWIDTH	640
#define	NUMDELTAS	16
#define	VUBYTES		(160*120*2)

//local storage
static I32		deltas[NUMDELTAS]=
{ 1,-1, 2,-3,8,-8,18,-18,36,-36,54,-54,96,-96,144,-144 };


#ifdef __cplusplus  
	extern "C" {               
#endif   //__cplusplus__  

void dx(U8 far *dst_addr, U32 nada, U8 *compbuf, U32 wh, U32 dstpitch);

//external asm routines from dx.asm
extern void _dxy(U8 *dst_addr, U8 *compbuf, U32 wh);
extern void _dxy2x2(U8 *dst_addr, U8 * compbuf, U32 wh);
extern void _dxu(U8 *dst_addr, U8 * compbuf, U32 wh);
extern void _dxv(U8 *dst_addr, U8 * compbuf, U32 wh);
extern void _clear_linebuf(I32 width, I32 setval);
extern long pitch;

//the following globals are referenced by asm code in dx.asm:
U32		dxtbl[16];
U16 	yvuclut[0x80000];
U8		linebuf[MAXWIDTH];
U8		vubuf[VUBYTES];

#ifdef __cplusplus  
	}               
#endif   //__cplusplus__  


static int yvu_to_r(int y,int v,int u)
{
	int r;

	r = (int)((float)y + 1.370705 * (float)v - 191.45029);
	if(r < 0) return(0);
	else if(r > 255) return(255);
	else return(r);
}

static int yvu_to_g(int y,int v,int u)
{
	int r;

	r = (int)((float)y - 0.698001 * (float)v - 0.337633 * (float)u + 116.56116);
	if(r < 0) return(0);
	else if(r > 255) return(255);
	else return(r);
}

static int yvu_to_b(int y,int v,int u)
{
	int r;

	r = (int)((float)y + 1.732446 * (float)u - 237.7534);
	if(r < 0) return(0);
	else if(r > 255) return(255);
	else return(r);
}

static void build_yvuclut(int mode565) {
	I32	y,v,u,r,g,b,rgb,i;

	for(y=0;y<64;y++) {
		for(v=0;v<64;v++) {
			for(u=0;u<64;u++) {
				i=y+(v<<12)+(u<<6);
					r=yvu_to_r(y<<2,v<<2,u<<2);
					g=yvu_to_g(y<<2,v<<2,u<<2);
					b=yvu_to_b(y<<2,v<<2,u<<2);
					if(mode565) {
						r=(r+4)>>3;
						b=(b+4)>>3;
						g=(g+2)>>2;
						rgb=(r<<11) + (g<<5) + b;
					} else {
						r>>=3; g>>=3; b>>=3;
						rgb=(r<<10) + (g<<5) + b;
					}
					yvuclut[i]=rgb;
				}
		}
	}
}

static void setline(I32 width, I32 setval) {
	_clear_linebuf(width,setval);
}

void dxinit(int mode565) {

	I32 i,s,dvq,t;
	static int savemode = -1;

	if(mode565 == savemode) return;
	for(s=0;s<16;s++) {
		dvq=deltas[s];
		dxtbl[s]=dvq;
	}
	build_yvuclut(mode565);
	savemode = mode565;
}


void dx(U8 far *dst_addr, U32 nada, U8 *compbuf, U32 wh, U32 dstpitch) {
	U32 width=wh & 0xffff;
	U32 height=wh>>16,voff,uoff;
	
	pitch = dstpitch;

	voff = (height*width)>>1;
	uoff = voff + (((height>>2) * (width>>2)) >>1);

// V & U planes:

	wh=((height>>2)<<16)+(width>>2);
	setline(width,128);
	_dxv(vubuf, compbuf+voff, wh);
	setline(width,128);
	_dxu(vubuf+1, compbuf+uoff, wh);

// Y plane:

	setline(width,0);
	wh=(height<<16)+width;
	if(nada) {		// set to decompress and stretch together
		_dxy2x2(dst_addr, compbuf, wh);
	} else {
		_dxy(dst_addr, compbuf, wh);
	}
}
