/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "tinker.h"
#include "huffman.h"

/* build table for speeding up the huffman decode process */

unsigned char buildTableFromTree(DXL_STREAM_HANDLE strmPtr)
{
	unsigned char *tree = (unsigned char *) strmPtr->htree;
	unsigned char value[(1+TBL_BITS)],nodes[(1+TBL_BITS)],valid[(1+TBL_BITS)];
	unsigned char *val = valid;
	int numbits = 1,index,i,v=0;
	unsigned char nodeptr = 0;
	unsigned char *table;

	nodeptr = nodes[0] = 0;

	for (table = (unsigned char *) strmPtr->htable,index = (0x01 << TBL_BITS); 
		index < (0x01 << (1+TBL_BITS));table+=TOK_SLEN){
		
		/* this loop will always travel down the left, 
			starting at nodeptr,
			it keeps track of what nodes we pass 
			and how many bits were valid when we passed
			(along with a list of the values)

			TBL_BITS-numbits is the number of bits we 
			need to get (number of branches worth following)
		*/
		if (v < (TOK_SLEN-1)){
			for (i = numbits; i < (1+TBL_BITS); i++){
				nodeptr = tree[nodeptr];
				if (nodeptr & 1){ /* we've reached the end of the code/branch */
					value[v] = nodeptr & 0xfe;
					valid[v++] = i; /* mark the new number of valid bits */
					nodeptr = 0; /* start back at the top of the tree */
				}
				nodes[i] = nodeptr;
			}
		}
		if (v == 0){ /* no values found, save the node "pointer" */
#if 0
			((long *)table)[0] = nodeptr;
			((long *)table)[1] = 0;
#else
			table[0] = 1;
			table[1] = nodeptr;
#endif
		}else{
#if 0
			/* some number of values found, write'em out */
			if ((unsigned long)table & 0x02){ /* not ok to read/write an int32 */
				((unsigned short *)table)[0] = ((unsigned short *)value)[0];
				if (v > 2) ((unsigned short *)table)[1] = ((unsigned short *)value)[1];
			}else
				*(unsigned long *)table = ((unsigned long *)value)[0];

			if (v > 4) /* write out last short */
				((unsigned short *)table)[2] = ((unsigned short *)value)[2];

#endif			

#if 0
			if (v > (TOK_SLEN-2)) /* we limit to 5 valid codes in table */ 
				v = (TOK_SLEN-2);

			*(long *)table = *(long *)value;

			if (v > 4){
				table[6] = value[4];
				table[7] = value[5];
			}

			table[4] = v;
			table[5] = valid[v-1];
#else
			if (v > (TOK_SLEN-1)) /* we limit to 5 valid codes in table */ 
				v = (TOK_SLEN-1);
	
			for(i = 0; i < v; i++) table[i] = value[i];

			table[v] = (valid[v-1]<<1) | 0x01; 
#endif
		}

		numbits = TBL_BITS;
		{	/* back-tracking operation */
			int temp;

			temp = ++index;

			while(!(temp & 1)){
				numbits--;
				temp >>= 1;
			}
			if (v > 0)
				while(valid[v-1] >= numbits) 
					if (--v <= 0) break;
		}

		/* move to the right in next higher node of this branch */
		nodeptr = nodes[numbits-1]+1;
	}


	return 0;
}
