/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dkpltfrm.h"
#include "tinker.h"
#include "huffman.h"

/* 
	my latest inclination is to use a 256x6 byte token lookup table
	
	  t0, t1, t2, t3, t4, end
	  or
	  end node# nc nc nc nc 

	looking up TBL_BITS bits into (maximum) 5 bytes estimates less than 2 bits/token
*/

extern unsigned long mask[];

int decodeTokens(DXL_STREAM_HANDLE strmPtr)
{
	register int tCount = strmPtr->tokenCount;
	register unsigned char *table;
	register unsigned char *tBuffer = (unsigned char *) strmPtr->tokenPtr;
	unsigned char value;
	unsigned char *htree;
	unsigned char *htable; 
	unsigned short *shortBuffer;
	unsigned char numBits = 32;
	unsigned long bitBuffer;
	DXL_BITSOURCE_HANDLE bs=&(strmPtr->bitSource);

	htree = (unsigned char *) strmPtr->htree;
	htable = (unsigned char *) strmPtr->htable; 
	shortBuffer = bs->shortBuffer;
 
#if defined(_bigend_h) && 0
	bitBuffer = *((unsigned long *) shortBuffer);
	shortBuffer += 2;
#else
	bitBuffer = ((unsigned long)*shortBuffer++) << 16;
	bitBuffer |= (unsigned long)*shortBuffer++;	
#endif

	/* get the first table entry */
	table = htable + (((bitBuffer & mask[numBits]) >> (numBits - TBL_BITS))*TOK_SLEN);

	while (!(*table & 1)){
		*tBuffer++ = *table++ >> 1;
		if (!--tCount){
			bs->shortBuffer = shortBuffer;
			return 0;
		}
	}
#if 0
	if (strmPtr->maxCodeLength <= TBL_BITS){
		while(1){
			value  = *table++ >> 1;

			if ((numBits -= value) < 17) {
				// need more bits ! shift the low 16 up and get 16 fresh bits
				bitBuffer <<= 16;
				bitBuffer += 
					(unsigned short)*shortBuffer++;
				numBits += 16;
			}

			// figure out a new starting point in the table 
			
			table = htable + (((bitBuffer & mask[numBits]) >> (numBits - TBL_BITS))*TOK_SLEN);

			/* read a "vector" */
			while (!(*table & 1)){
				*tBuffer++ = *table++ >> 1;
				if (!--tCount){
					bs->shortBuffer = shortBuffer;
					return 0;
				}
			}
		}
		return 0;
	}
#endif
	while(1){
		if ((value = *table++) == 1){
			// no used bits so we need to go to tree
#if 0
			bs->bitBuffer = bitBuffer;
			bs->numBits = numBits;
			bs->shortBuffer = shortBuffer;
			
			*tBuffer++ = searchTree(bs,htree,*table);
			
			/* bitBuffer, buffBitCount need to be returned by searchTree */
			bitBuffer = bs->bitBuffer;
			numBits = bs->numBits;
			shortBuffer = bs->shortBuffer;
#else
			{
				int nodeptr = *table;

				numBits -= TBL_BITS;
				bitBuffer <<= 32-numBits;

				do{
					while(numBits--){

						/* code chooses left/right byte entry */
						nodeptr += (bitBuffer & 0x80000000) ? 1 : 0; 
						bitBuffer <<= 1; /* shift out the used bit */

						nodeptr = htree[nodeptr]; /* get the value/next-mode */
						
						/* we've reached the end of the code/branch */

						if (nodeptr & 1){ 
							/* shift unused bits all the way to the right */
							bitBuffer >>= (32 - numBits);
							*tBuffer++ = (nodeptr >> 1); /* return value w/o flag */
							break;
						}
						/* shift out flag, nodeptr is now a multiple of 2 */
						//nodeptr &= 0xfffe; 
					}
					if (numBits) break;
					/* may as well get 32 */
					numBits = 32;
#if defined(_bigend_h) && 0
					bitBuffer = *((unsigned long *) shortBuffer);
					shortBuffer += 2;
#else
					bitBuffer = ((unsigned long)*shortBuffer++) << 16;
					bitBuffer |= (unsigned long)*shortBuffer++;	
#endif

				}while(1);
			}
#endif
			if (!--tCount){
				bs->shortBuffer = shortBuffer;
				return 0;
			}
			value = 0;
		}

		value >>= 1;

		if ((numBits -= value) < 17) {
			// need more bits ! shift the low 16 up and get 16 fresh bits
			bitBuffer <<= 16;
			bitBuffer += 
				(unsigned short)*shortBuffer++;
			numBits += 16;
		}

		// figure out a new starting point in the table 
		
		table = htable + (((bitBuffer & mask[numBits]) >> (numBits - TBL_BITS))*TOK_SLEN);

		/* read a "vector" */
		while (!(*table & 1)){
			*tBuffer++ = *table++ >> 1;
			if (!--tCount){
				bs->shortBuffer = shortBuffer;
				return 0;
			}
		}
	}
	return 0;
}

#if 0

unsigned char searchTree(DXL_BITSOURCE_HANDLE bs,unsigned char *tree,int nodeptr)
{
	bs->numBits -= TBL_BITS;
	bs->bitBuffer <<= 32-bs->numBits;

	do{
		while(bs->numBits--){
			nodeptr += (bs->bitBuffer & 0x80000000) ? 1 : 0; /* code chooses left/right byte entry */
			bs->bitBuffer <<= 1; /* shift out the used bit */

			nodeptr = tree[nodeptr]; /* get the value/next-mode */
			
			if (nodeptr & 1){ /* we've reached the end of the code/branch */
				/* shift unused bits all the way to the right */
				bs->bitBuffer >>= (32 - bs->numBits);
				return (nodeptr >> 1); /* return value w/o flag */
			}
			//nodeptr &= 0xfffe; /* shift out flag, nodeptr is now a multiple of 2 */
		}
		/* may as well get 32 */
		bs->numBits = 32;
		bs->bitBuffer = ((unsigned long)*bs->shortBuffer++) << 16;
		bs->bitBuffer |= (unsigned long)*bs->shortBuffer++;

	}while(1);

	return 0; /* shouldn't get here */
}
#endif
