/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "dxl_main.h"
#include "tinker.h"

extern unsigned long tm2Table[(8*MAXDELTAS)+6+1024];

#define delta tm2Table

#define vphold ((long *)(delta+vpOffset))
#define cvphold ((long *)(delta+vpOffset)+4)

#define FASTINTERP 0

#define TMPPITCH 0
#define OFFSETSIZE 1

#if COLORORDER == BGRORDER
#define REDSHIFT(x) x
#define BLUESHIFT(x) (x <<16)
#else
#define REDSHIFT(x) (x << 16)
#define BLUESHIFT(x) x
#endif

long offsetTable[4][4] = {
	{ 0,OFFSETSIZE,OFFSETSIZE,OFFSETSIZE },
	{ 0,OFFSETSIZE,OFFSETSIZE,TMPPITCH+OFFSETSIZE }, 
	{ 0,OFFSETSIZE,TMPPITCH+OFFSETSIZE,OFFSETSIZE }, 
	{ 0,TMPPITCH+OFFSETSIZE,OFFSETSIZE,OFFSETSIZE }
};

long tinkPitch;

void setupOffsetTable(int blockwidth)
{
	tinkPitch = blockwidth*8; /* 8 longs (32 bytes) per block */
	offsetTable[1][3] = offsetTable[2][2] = offsetTable[3][1] = (tinkPitch - 3)*OFFSETSIZE;
}


/* This STATIC_INLINE define is a trick to allow easy and complete disassembly and subsequent */
/* assembler level optimizations and assembler MACRO snappiness (on the PowerPC at least).    */
//#define STATIC_INLINE static inline
#define STATIC_INLINE static 

STATIC_INLINE void dxHiRes(DXL_TM2X_HANDLE srcPtr,
	unsigned long *dest,
	unsigned long *last4ptr
	)
{
	register long vp;
	register unsigned char *ordinal = srcPtr->chromaHi.tokenCPtr;

	/********** high chroma **********/
	{
		register long last2a = last4ptr[0];
		register long last2b = last4ptr[1];

		vp = cvphold[0] += REDSHIFT((long)delta[chromaHiOffset + *ordinal++]);
		vp += BLUESHIFT((long)delta[chromaHiOffset + *ordinal++]);
		dest[0] = last2a += vp;
		
		vp += REDSHIFT((long)delta[chromaHiOffset + *ordinal++]);
		cvphold[0] = vp += BLUESHIFT((long)delta[chromaHiOffset + *ordinal++]);
		dest[1] = last2b += vp;

		vp = cvphold[1] += REDSHIFT((long)delta[chromaHiOffset + *ordinal++]);
		vp += BLUESHIFT((long)delta[chromaHiOffset + *ordinal++]);
		last4ptr[0] = dest[2] = last2a += vp;
		
		vp += REDSHIFT((long)delta[chromaHiOffset + *ordinal++]);
		cvphold[1] = vp += BLUESHIFT((long)delta[chromaHiOffset + *ordinal++]);
		last4ptr[1] = dest[3] = last2b += vp;

		srcPtr->chromaHi.tokenCPtr += 8;
	}	
	/********** high luma **********/
	{
		register long last4 = last4ptr[2];
		register long vpx;
		ordinal = srcPtr->LumaHi.tokenCPtr;

		vp = vphold[0];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 0]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 1]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 2]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[ 3]]; 
		vphold[0] = vp;
		dest[4] = last4 += vpx;

		vp = vphold[1];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 4]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 5]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 6]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[ 7]]; 
		vphold[1] = vp;
		dest[5] = last4 += vpx;

		vp = vphold[2];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 8]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 9]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[10]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[11]]; 
		vphold[2] = vp;
		dest[6] = last4 += vpx;

		vp = vphold[3];
		vpx =  vp += delta[LumaHiOffset + ordinal[12]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[13]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[14]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[15]]; 
		vphold[3] = vp;
		last4ptr[2] = dest[7] = last4 += vpx;

		srcPtr->LumaHi.tokenCPtr += 16;
	}
}

STATIC_INLINE void dxMedRes(DXL_TM2X_HANDLE srcPtr,
				unsigned long *dest,
				unsigned long *last4ptr
				)
{
	register long vp;
	{
		/**********		 low chroma 		**********/
		register long last2a = last4ptr[-2];
		register long last2b = last4ptr[1];
		{
			register long tvp;

			tvp = vp = cvphold[0] + cvphold[1];
			last2a -= vp;
			last2a += last2b;
#if FASTINTERP
			last2a += 0x00008000;	/* if lower word is neg.  adds one to upper */
			last2a >>=1;			/* divide by 2 */
			last2a &= 0xffff3fff;	/* mask out div 2 remainder and added bit */

			if (last2a & 0x00000800) /* if lower word neg. fix sign and upper word */
				last2a += 0xffffc000;

			vp += 0x00008000;
			vp >>=1;
			vp &= 0xffff3fff;

			if (vp & 0x00000800)
				vp += 0xffffc000;

#else
			{
				register short temp;
				temp = 0;
				if ((short)last2a < 0){ 
					last2a -= temp = (short)last2a; 
					temp >>= 1;
				}

				last2a &= 0xfffeffff; 
				last2a >>= 1;
				last2a += temp;
				temp = 0;
				if ((short)vp < 0){
					vp -= temp = (short)vp;	temp >>= 1;
				}

				vp &= 0xfffeffff; vp >>= 1; vp += temp;
			}
#endif

			cvphold[1] = tvp - vp;
		}
		{
			register unsigned char *ordinal = srcPtr->chromaLo.tokenCPtr;

			vp += REDSHIFT((long)delta[chromaLoOffset + ordinal[0]]);
			vp += BLUESHIFT((long)delta[chromaLoOffset + ordinal[1]]);
		}
		dest[0] = last2a += cvphold[0] = vp;
		dest[1] = last2b += vp;
		
		last4ptr[0] = dest[2] = last2a += vp = cvphold[1];
		last4ptr[1] = dest[3] = last2b += vp;
		
		srcPtr->chromaLo.tokenCPtr += 2;

	}
	/********** high luma **********/
	{
		register long last4 = last4ptr[2];
		register long vpx;
		register unsigned char *ordinal = srcPtr->LumaHi.tokenCPtr;

		vp = vphold[0];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 0]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 1]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 2]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[ 3]]; 
		vphold[0] = vp; 
		dest[4] = last4 += vpx;

		vp = vphold[1];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 4]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 5]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 6]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[ 7]]; 
		vphold[1] = vp; 
		dest[5] = last4 += vpx;

		vp = vphold[2];
		vpx =  vp += delta[LumaHiOffset + ordinal[ 8]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[ 9]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[10]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[11]]; 
		vphold[2] = vp; 
		dest[6] = last4 += vpx;

		vp = vphold[3];
		vpx =  vp += delta[LumaHiOffset + ordinal[12]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[13]]; vpx <<= 8;
		vpx += vp += delta[LumaHiOffset + ordinal[14]]; vpx <<= 8;		
		vpx += vp += delta[LumaHiOffset + ordinal[15]]; 
		vphold[3] = vp; 
		last4ptr[2] = dest[7] = last4 += vpx;

		srcPtr->LumaHi.tokenCPtr += 16;
	}
}

STATIC_INLINE void dxLowRes(DXL_TM2X_HANDLE srcPtr,
						unsigned long *dest,
						unsigned long *last4ptr
						)
{
	register long vp;
	{
		/**********		 low chroma 		**********/
		register long last2a = last4ptr[-2];
		register long last2b = last4ptr[1];
		{
			register long tvp;

			tvp = vp = cvphold[0] + cvphold[1];
			last2a -= vp;
			last2a += last2b;
#if FASTINTERP
			last2a += 0x00008000;	/* if lower word is neg.  adds one to upper */
			last2a >>=1;			/* divide by 2 */
			last2a &= 0xffff3fff;	/* mask out div 2 remainder and added bit */

			if (last2a & 0x00000800) /* if lower word neg. fix sign and upper word */
				last2a += 0xffffc000;

			vp += 0x00008000;
			vp >>=1;
			vp &= 0xffff3fff;

			if (vp & 0x00000800)
				vp += 0xffffc000;
#else
			{
				register short temp;
				temp = 0;
				if ((short)last2a < 0){ 
					last2a -= temp = (short)last2a; temp >>= 1;
				}

				last2a &= 0xfffeffff; last2a >>= 1; last2a += temp;

				temp = 0;
				if ((short)vp < 0){
					vp -= temp = (short)vp;	temp >>= 1;
				}

				vp &= 0xfffeffff; vp >>= 1; vp += temp;
			}
#endif
			cvphold[1] = tvp - vp;
		}
		{
			register unsigned char *ordinal = srcPtr->chromaLo.tokenCPtr;

			vp += REDSHIFT((long)delta[chromaLoOffset + ordinal[0]]);
			vp += BLUESHIFT((long)delta[chromaLoOffset + ordinal[1]]);
		}
		dest[0] = last2a += cvphold[0] = vp;
		dest[1] = last2b += vp;
		
		last4ptr[0] = dest[2] = last2a += vp = cvphold[1];
		last4ptr[1] = dest[3] = last2b += vp;
		
		srcPtr->chromaLo.tokenCPtr += 2;

	}
	/********** low luma **********/
	{
		register long last4;
		register long vpx;

		last4 = last4ptr[2];
		vp = vphold[0]+vphold[1];

		{
			unsigned char lastPix;

			lastPix = ((unsigned char)(last4ptr[2-3])-
				(unsigned char)(vp+vphold[2]+vphold[3]));

			vpx = ((unsigned char)(last4>>16) + lastPix)>>1;
		}

		last4 &= 0x00ff00ff;
		last4 |= (unsigned char)vpx << 24;

		vpx = ((unsigned char)last4 + (unsigned char)(last4 >> 16))>>1;
		last4 |= (unsigned char)vpx << 8;

		/* first eight */

		vphold[1] = vp-(vp>>1); /* vp1 = vpTotal - vpHalf */
		vp >>= 1; /* vp = vpHalf */

		{
			register unsigned char *ordinal = srcPtr->LumaLo.tokenCPtr;

			vpx = vp += delta[LumaLoOffset + ordinal[0]];	vpx <<= 8;
			vpx += vp;						vpx <<= 8;
			vpx += vp += delta[LumaLoOffset + ordinal[1]];	vpx <<= 8;
			vpx += vp;

			dest[4] = last4 += vpx;
			vphold[0] = vp; /* save off the vp for next block */

			vpx = vphold[1] * 0x01010101;
			dest[5] = last4 += vpx;

			/* second eight */

			vp = (vphold[2]+vphold[3])>>1; /* calculate interpolated vp */		
			vphold[3] = (vphold[2]+vphold[3])-vp; /* adjust next vp */

			vpx = vp += delta[LumaLoOffset + ordinal[2]];	vpx <<= 8;
			vpx += vp;						vpx <<= 8;
			vpx += vp += delta[LumaLoOffset + ordinal[3]];	vpx <<= 8;
			vpx += vp;
		}

		dest[6] = last4 += vpx;
		vphold[2] = vp; /* save off the vp for next block */

		vpx = vphold[3] * 0x01010101;
		last4ptr[2] = dest[7] = last4 += vpx;

		/***************/

		srcPtr->LumaLo.tokenCPtr += 4;
		
	} 
}


STATIC_INLINE void dxNullRes(DXL_TM2X_HANDLE srcPtr,
						unsigned long *dest,
						unsigned long *last4ptr
						)
{
	{
		/********** null chroma **********/
		register long last2a = last4ptr[-2];
		register long last2b = last4ptr[1];
		register long vp;
		long tvp;

		tvp = vp = cvphold[0] + cvphold[1];
		last2a -= vp;
		last2a += last2b;

#if FASTINTERP
		last2a += 0x00008000;	/* if lower word is neg.  adds one to upper */
		last2a >>=1;			/* divide by 2 */
		last2a &= 0xffff3fff;	/* mask out div 2 remainder and added bit */

		if (last2a & 0x00000800) /* if lower word neg. fix sign and upper word */
			last2a += 0xffffc000;

		vp += 0x00008000;
		vp >>=1;
		vp &= 0xffff3fff;

		if (vp & 0x00000800)
			vp += 0xffffc000;

#else
		{
			register short temp = 0;
			
			if ((short)last2a < 0){ 
				last2a -= temp = (short)last2a; temp >>= 1;
			}

			last2a &= 0xfffeffff; last2a >>= 1; last2a += temp;

			temp = 0;
			if ((short)vp < 0){
				vp -= temp = (short)vp; temp >>= 1;
			}

			vp &= 0xfffeffff; vp >>= 1; vp += temp;
		}
#endif
		cvphold[1] = tvp - vp;
		
		dest[0] = last2a += cvphold[0] = vp;
		dest[1] = last2b += vp;
		
		last4ptr[0] = dest[2] = last2a += vp = cvphold[1];
		last4ptr[1] = dest[3] = last2b += vp;
	}
	{

		/*********	 null luma 	*********/

		/* vp does not change */
		register long last4;
		register unsigned char leftPix,rightPix;
		register long diff,vpt,qvpt,hvpt;

		vpt = (vphold[0]+vphold[1]+vphold[2]+vphold[3]);

		leftPix = (unsigned char)(last4ptr[2-3]-vpt);
		rightPix = (unsigned char)last4ptr[2];

		diff = rightPix - leftPix;

		last4 = rightPix | 
			(((unsigned char)(rightPix - (diff>>2))) << 8) |
			(((unsigned char)(leftPix + (diff>>1))) << 16) |
			(((unsigned char)(leftPix + (diff>>2))) << 24);

#if 0
		{
			unsigned char tPix = leftPix;

			leftPix += vphold[0] = (tPix + (vpt>>2)) - leftPix;
			leftPix += vphold[1] = (tPix + (vpt>>1)) - leftPix;	
			leftPix += vphold[2] = ((tPix + vpt) - (vpt>>2)) - leftPix;
			vphold[3] = (tPix + vpt) - leftPix;
		}

		dest[4] = last4 += (vphold[0] * 0x01010101);
		dest[5] = last4 += (vphold[1] * 0x01010101);
		dest[6] = last4 += (vphold[2] * 0x01010101);
		last4ptr[2] = dest[7] = last4 += (vphold[3] * 0x01010101);
#else
		vphold[3] = vphold[0] = qvpt = vpt>>2;
		vphold[1] = (hvpt = vpt>>1) - qvpt;
		vphold[2] = vpt - qvpt - hvpt;

		qvpt *= 0x01010101;

		dest[4] = last4 += qvpt;
		dest[5] = last4 += (vphold[1] * 0x01010101);
		dest[6] = last4 += (vphold[2] * 0x01010101);
		last4ptr[2] = dest[7] = last4 += qvpt;
#endif
	}
}

STATIC_INLINE void dxStillBlock(DXL_TM2X_HANDLE srcPtr,
						unsigned long *dst,
						unsigned long *last4ptr,
						unsigned long *src
						)
{
	/**********
	 still block
	**********/

	/* recalculate the vp's */
	cvphold[0] = src[1] - last4ptr[1];
	cvphold[1] = src[3] - src[1];
	vphold[0] = (long)(unsigned char)src[4] - (long)(unsigned char)last4ptr[2];
	vphold[1] = (long)(unsigned char)src[5] - (long)(unsigned char)src[4];
	vphold[2] = (long)(unsigned char)src[6] - (long)(unsigned char)src[5];
	vphold[3] = (long)(unsigned char)src[7] - (long)(unsigned char)src[6];

	/* only necessary when sourcePtr and destPtr 
	are different, i.e. when using a "motion buffer" */

	dst[0] = src[0];
	dst[1] = src[1];
	last4ptr[0] = dst[2] = src[2];
	last4ptr[1] = dst[3] = src[3];

	dst[4] = src[4];
	dst[5] = src[5];
	dst[6] = src[6];
	last4ptr[2] = dst[7] = src[7];

}

STATIC_INLINE void dxMotionBlock(DXL_TM2X_HANDLE srcPtr,
						unsigned long *dst,
						unsigned long *last4ptr,
						unsigned long *src
						)
{
	long *offset;
	int xoff,yoff;

	xoff = delta[motionOffset + *srcPtr->motion.tokenCPtr++];
	yoff = delta[motionOffset + *srcPtr->motion.tokenCPtr++];

#if 0
	/*	adjust the source address according to 
		the coarse (block) offset in the y direction */
	while (yoff < 0){ 
		src -= tinkPitch; 
		yoff += 4; 
	}
	while (yoff > 3){ 
		src += tinkPitch; 
		yoff -= 4; 
	}
#else
	src += (tinkPitch * (yoff>>2));
	yoff &= 3;
#endif

	/*	adjust the source according to the 
		coarse (block) offset in the x direction */
	src += ((xoff>>2) * 8);

	if (yoff & 2){
		if (xoff & 2){
			dst[0] = src[3];
			dst[1] = src[2+8];
			dst[2] = src[1+tinkPitch];
			dst[3] = src[0+8+tinkPitch];
		}else{
			dst[0] = src[2];
			dst[1] = src[3];
			dst[2] = src[0+tinkPitch];
			dst[3] = src[1+tinkPitch];
		}
	}else{
		if (xoff & 2){
			dst[0] = src[1];
			dst[1] = src[0+8];
			dst[2] = src[3];
			dst[3] = src[2+8];
		}else{
			dst[0] = src[0];
			dst[1] = src[1];
			dst[2] = src[2];
			dst[3] = src[3];
		}
	}
	cvphold[0] = dst[1] - last4ptr[1];
	cvphold[1] = dst[3] - dst[1];
	
	last4ptr[0] = dst[2];
	last4ptr[1] = dst[3];

	/* add in the line offset within the block */
	src += (4+yoff);

	/*	init the offset table pointer */
	offset = &offsetTable[yoff&3][0];

	/* now copy the pixels */
	switch(xoff&3){
		case 0:
			src += offset[0];
			dst[4] = *src;
			src += offset[1];
			dst[5] = *src;
			src += offset[2];
			dst[6] = *src;
			src += offset[3];
			dst[7] = *src;
			break;
		case 1:
			src += offset[0];
			dst[4] = (*src <<  8) | (src[8] >> 24);
			src += offset[1];
			dst[5] = (*src <<  8) | (src[8] >> 24);
			src += offset[2];
			dst[6] = (*src <<  8) | (src[8] >> 24);
			src += offset[3];
			dst[7] = (*src <<  8) | (src[8] >> 24);
			break;
		case 2:
			src += offset[0];
			dst[4] = (*src << 16) | (src[8] >> 16);
			src += offset[1];
			dst[5] = (*src << 16) | (src[8] >> 16);
			src += offset[2];
			dst[6] = (*src << 16) | (src[8] >> 16);
			src += offset[3];
			dst[7] = (*src << 16) | (src[8] >> 16);
			break;
		case 3:
			src += offset[0];
			dst[4] = (*src << 24) | (src[8] >>  8);
			src += offset[1];
			dst[5] = (*src << 24) | (src[8] >>  8);
			src += offset[2];
			dst[6] = (*src << 24) | (src[8] >>  8);
			src += offset[3];
			dst[7] = (*src << 24) | (src[8] >>  8);
			break;
	}
	vphold[0] = (long)(unsigned char)dst[4] - (long)(unsigned char)last4ptr[2];
	vphold[1] = (long)(unsigned char)dst[5] - (long)(unsigned char)dst[4];
	vphold[2] = (long)(unsigned char)dst[6] - (long)(unsigned char)dst[5];
	vphold[3] = (long)(unsigned char)dst[7] - (long)(unsigned char)dst[6];
	last4ptr[2] = dst[7];
}

STATIC_INLINE void dxUpdateBlock(DXL_TM2X_HANDLE srcPtr,
						unsigned long *dest,
						unsigned long *last4ptr,
						unsigned long *src
						)
{
	register unsigned char *ordinal = srcPtr->update.tokenCPtr;
	register long vp,last;

	{
		/********** update chroma **********/
		
		vp = REDSHIFT((long)delta[updateOffset + ordinal[0]]);
		vp += BLUESHIFT((long)delta[updateOffset + ordinal[1]]);

		dest[0] = src[0] + vp;

		vp = REDSHIFT((long)delta[updateOffset + ordinal[2]]);
		vp += BLUESHIFT((long)delta[updateOffset + ordinal[3]]);

		last = vp = dest[1] = src[1] + vp;

		cvphold[0] = vp - last4ptr[1];

		vp = REDSHIFT((long)delta[updateOffset + ordinal[4]]);
		vp += BLUESHIFT((long)delta[updateOffset + ordinal[5]]);

		last4ptr[0] = dest[2] = src[2] + vp;

		vp = REDSHIFT((long)delta[updateOffset + ordinal[6]]);
		vp += BLUESHIFT((long)delta[updateOffset + ordinal[7]]);

		last4ptr[1] = vp = dest[3] = src[3] + vp;

		cvphold[1] = vp - last;
	}
	{
		/**********	 update luma	**********/

		last = last4ptr[2];

		vp =  delta[updateOffset + ordinal[ 8]] << 8;
		vp += delta[updateOffset + ordinal[ 9]];vp <<= 8;
		vp += delta[updateOffset + ordinal[10]];vp <<= 8;
		vp += delta[updateOffset + ordinal[11]];
		dest[4] = vp = src[4] + vp;
		vphold[0] = (unsigned char)vp - (unsigned char)last;
		last = vp;

		vp =  delta[updateOffset + ordinal[12]] << 8;
		vp += delta[updateOffset + ordinal[13]];vp <<= 8;
		vp += delta[updateOffset + ordinal[14]];vp <<= 8;
		vp += delta[updateOffset + ordinal[15]];
		dest[5] = vp = src[5] + vp;
		vphold[1] = (unsigned char)vp - (unsigned char)last;
		last = vp;

		vp =  delta[updateOffset + ordinal[16]] << 8;
		vp += delta[updateOffset + ordinal[17]];vp <<= 8;
		vp += delta[updateOffset + ordinal[18]];vp <<= 8;
		vp += delta[updateOffset + ordinal[19]];
		dest[6] = vp = src[6] + vp;
		vphold[2] = (unsigned char)vp - (unsigned char)last;
		last = vp;

		vp =  delta[updateOffset + ordinal[20]] << 8;
		vp += delta[updateOffset + ordinal[21]];vp <<= 8;
		vp += delta[updateOffset + ordinal[22]];vp <<= 8;
		vp += delta[updateOffset + ordinal[23]];
		last4ptr[2] = dest[7] = vp = src[7] + vp;
		vphold[3] = (unsigned char)vp - (unsigned char)last;

		srcPtr->update.tokenCPtr += 24;
	}
}

#if 0

void dxHiRes(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxMedRes(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxLowRes(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxNullRes(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxStillBlock(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxUpdateBlock(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);
void dxMotionBlock(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *);

typedef void ((*funcPtr)(DXL_TM2X_HANDLE,unsigned long *,unsigned long *,unsigned long *));

funcPtr TransformBlock[] = { 
    dxHiRes, dxMedRes, dxLowRes, dxNullRes,
    dxUpdateBlock, dxStillBlock, dxMotionBlock 
};
#endif
  




#include "tm2blit.h"

typedef void (*tm2BlitBlockFunc)(unsigned long *lcDest,unsigned long **hDest, 
				PitchType scrnPitch);

static void nullBlit(
	unsigned long *lcDest,
	unsigned long **hDest,
	PitchType scrnPitch)
{}

#if 1
void dxTM2Blocks(
				 DXL_TM2X_HANDLE src,
				 DXL_VSCREEN_HANDLE vScreen)
{    
	long *vph;
	unsigned char *blockInfoPtr;
	unsigned long *lcDest,*lcSource;
	unsigned long *lastLine;
	int i;
	int numBlkRows = src->imHeight/4;
	int numBlkCols = src->imWidth/4;
	register unsigned long *dst;
	register unsigned char *ldst;
	tm2BlitBlockFunc  blockBlitter = nullBlit;
	int scrnPitch;
	  
	if (vScreen) {
		int x,y;

		y = vScreen->viewY + src->y;
		x = vScreen->viewX + src->x;
		
		if (vScreen->bq == DXBLIT_STRETCH || vScreen->bq == DXBLIT_STRETCH_BRIGHT) 
            scrnPitch = vScreen->pitch<<1;
        else
            scrnPitch = vScreen->pitch;

		ldst = (unsigned char *) vScreen->addr;
		ldst += x * DXL_GetVScreenSizeOfPixel(vScreen) + y * vScreen->pitch;
		dst = (unsigned long *)ldst;

		blockBlitter = (tm2BlitBlockFunc) vScreen->blitter;
		if (blockBlitter == (tm2BlitBlockFunc)-1L)
			return;
	}	
	
#if DX_TIMING
	UnsignedWide w1,w2;
	Microseconds(&w1);
#endif	
	  
	vph = src->deltas + (MAXDELTAS*8);
    blockInfoPtr = (unsigned char *) src->blockType.tokenPtr;
    lcDest = src->frontBuffer;
    lcSource = src->backBuffer;

	do{
        lastLine = src->lineBuff+3; /* restart the last line */

		/* reset all the vps */
        vph[0] = vph[1] = vph[2] = vph[3] = vph[4] = vph[5] = 0;

        i = numBlkCols;

		do {
            
			switch (*blockInfoPtr++){
				case 0:
					dxHiRes(src,lcDest,lastLine);
					break;
				case 1:
					dxMedRes(src,lcDest,lastLine);
					break;
				case 2:
					dxLowRes(src,lcDest,lastLine);
					break;
				case 3:
					dxNullRes(src,lcDest,lastLine);
					break;
				case 4:
					dxUpdateBlock(src,lcDest,lastLine,lcSource);
					break;
				case 5:
					dxStillBlock(src,lcDest,lastLine,lcSource);
					break;
				case 6:
					dxMotionBlock(src,lcDest,lastLine,lcSource);
					break;
			} 


            blockBlitter(lcDest, &dst ,scrnPitch);
			
			lastLine += 3;
       		lcDest += 8;
        	lcSource += 8;	
		} while (--i);

        dst = (unsigned long *)(ldst += (scrnPitch<<2));

    }while(--numBlkRows);
 
 
#if DX_TIMING
    Microseconds(&w2);
	gDXTime32 += w2.lo - w1.lo;
#endif
    
}
#endif
