/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dkpltfrm.h"
#include "tinker.h"
#include "huffman.h"
 
unsigned long mask[] = {
	0x00000000,0x00000001,0x00000003,0x00000007,
	0x0000000f,0x0000001f,0x0000003f,0x0000007f,
	0x000000ff,0x000001ff,0x000003ff,0x000007ff,
	0x00000fff,0x00001fff,0x00003fff,0x00007fff,
	0x0000ffff,0x0001ffff,0x0003ffff,0x0007ffff,
	0x000fffff,0x001fffff,0x003fffff,0x007fffff,
	0x00ffffff,0x01ffffff,0x03ffffff,0x07ffffff,
	0x0fffffff,0x1fffffff,0x3fffffff,0x7fffffff,
	0xffffffff
	};


getBits(DXL_BITSOURCE_HANDLE srcPtr,int nBits)
{
	unsigned long temp = 0,retVal;

	if (srcPtr->numBits <= nBits){
		nBits -= srcPtr->numBits;
		temp = (srcPtr->bitBuffer & mask[srcPtr->numBits]) << nBits;
#ifndef _bigend_h
		srcPtr->bitBuffer = *srcPtr->shortBuffer++;
		srcPtr->bitBuffer |= (long)*srcPtr->shortBuffer++ << 16;
#else
		{
			unsigned long temp2;

			srcPtr->bitBuffer = *srcPtr->shortBuffer++;
			srcPtr->bitBuffer = invert2(srcPtr->bitBuffer); 
			temp2 = (long ) *srcPtr->shortBuffer++;
			temp2 = invert4(temp2);
			srcPtr->bitBuffer |= temp2;
		}
#endif
		srcPtr->numBits = 32;
		if (!nBits)
			return temp;
	}

	retVal = mask[srcPtr->numBits];
	retVal &= srcPtr->bitBuffer ;
	retVal >>= (srcPtr->numBits - nBits);

	srcPtr->numBits -= nBits;

	return retVal|temp;
}

int getSignedBits(DXL_BITSOURCE_HANDLE srcPtr,int nBits)
{
	int val = getBits(srcPtr,nBits);

	if (val & (1 << (nBits -1)))
		val -= 1 << nBits;

	return val;
}

/* read huffman tree */

int nnode = 0;
		
/*
	32bit word 

	xxxxxyyyyy0000000 x valbits 0->32 5
						y maximum codelength 5 
						z min code length 5
						node count 17


	a					0 value
						1 two trees
*/

void readTree(DXL_STREAM_HANDLE strm)
{
	int ret;

	strm->valueLength   = getBits(&(strm->bitSource),5);
	strm->maxCodeLength = getBits(&(strm->bitSource),5);
	strm->minCodeLength = getBits(&(strm->bitSource),5);
	strm->nodeCount     = getBits(&(strm->bitSource),17);

	ret = subTree(&(strm->bitSource),(unsigned char *) strm->htree,nnode = 0,strm->valueLength);

	if (ret & 1){
		strm->htree[0] = ret;
		strm->htree[1] = 0;
	}
}

unsigned char subTree(DXL_BITSOURCE_HANDLE src,unsigned char *nodeArray,int node,int vLen) 
{
	if( !getBits(src,1)) {
		return (getBits(src,vLen)<<1) | 0x01;
	} else {
		node = nnode++;
		nodeArray[(node*2)] = subTree(src,nodeArray,nnode,vLen);
		nodeArray[(node*2)+1] = subTree(src,nodeArray,nnode,vLen);
		return node<<1;
	}
}


