/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _tinker_h
#define _tinker_h


#include <dxl_main.h>

#ifndef invert4
#define invert4(x) (x)
#endif

#define MAXDELTAS 32
#define MONOCHROME 1

enum BLOCKTYPES {
	STRM_CHROMAHI,STRM_CHROMALO,STRM_LUMAHI,STRM_LUMALOW,
	STRM_UPDATE,STRM_MOTION,STRM_BLOCKTYPE };

#define	chromaHiOffset	0
#define chromaLoOffset	(MAXDELTAS)
#define LumaHiOffset	(MAXDELTAS*2)
#define LumaLoOffset	(MAXDELTAS*3)
#define updateOffset	(MAXDELTAS*4)
#define motionOffset	(MAXDELTAS*5)
#define blockTypeOffset	(MAXDELTAS*6)

#define vpOffset (MAXDELTAS*8)

typedef struct bitSrc{

	unsigned short *shortBuffer;
	unsigned long bitBuffer;
	char numBits;
} *DXL_BITSOURCE_HANDLE, DXL_BITSOURCE;

#define TBL_BITS 9
#define TBL_SIZE (1<<TBL_BITS)
#define TOK_SLEN 8

typedef struct stream {
	int		numDeltas;
	long	*deltas;		/* upto 32 deltas (128bytes) */
	char	*htree;			/* huffman tree (64bytes) */
	char	*htable;		/* huffman decoder table (1536bytes) */
							/* total "decoder" size 2176 bytes */
	int tokenCount;
	char *tokenPtr;
	unsigned char *tokenCPtr;

	DXL_BITSOURCE bitSource;
	
	int valueLength;
	int maxCodeLength;
	int minCodeLength;
	int nodeCount;

	//struct stream *nextStream;
	//int streamNum;

} *DXL_STREAM_HANDLE, DXL_STREAM;

typedef struct TM2X {
	xImageBaseStruct;

	//DXL_STREAM_HANDLE stream;
	DXL_STREAM chromaHi;
	DXL_STREAM chromaLo;
	DXL_STREAM LumaHi;
	DXL_STREAM LumaLo;
	DXL_STREAM update;
	DXL_STREAM motion;
	DXL_STREAM blockType;

	unsigned long type;
	unsigned long depth;
	unsigned long dataSize;
	unsigned long compressFlags;

	unsigned long formatSize;
	unsigned long *cData;

	unsigned char *hBuffer;
	int hBufferTop;
	int hBufferSize;

	unsigned long *lineBuff;
	unsigned long *pBuffer1;
	unsigned long *pBuffer2;

	long *deltas;

	unsigned long *frontBuffer;
	unsigned long *backBuffer;
} *DXL_TM2X_HANDLE, DXL_TM2X;

typedef struct TM2Xptrs {
    /* WARNING: changes made in here must be made in tinker.ash */
	char *chromaHiTptr;
	char *chromaLoTptr;
	char *LumaHiTptr;
	char *LumaLoTptr;
	char *updateTptr;
	char *motionTptr;
	char *blockTypeTptr;
    long *deltas;
    unsigned char *blockInfoPtr;
    unsigned long *lastLine;
    unsigned long *lcDest;
    unsigned long *lcSource;
    long *vph;
	DXL_TM2X_HANDLE src;
} *DXL_TM2Xptrs_HANDLE, DXL_TM2Xptrs;

typedef void *BLIT_FUNC;

void dxTM2Blocks(DXL_TM2X_HANDLE , DXL_VSCREEN_HANDLE);

void setupOffsetTable(int blockwidth);
void tm2_createGamma162Table(void);
void tm2_createGammaTable(void);


#ifdef _X86PROFILING
enum PROFILESECTION { LOSSLESSDX, PLANARDX, BLITME };
void tscStart(enum PROFILESECTION);
void tscEnd(enum PROFILESECTION);
void tscInit(void);
void tscUninit(void);
#endif


#endif /* include guards */
