/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "dkpltfrm.h"
#include "duck_mem.h"
#include "dxl_main.h"

#include "tinker.h"
#include "huffman.h"
#include "tm2.h"
#include <stdlib.h>
#include <string.h>

unsigned long tm2Table[(8*MAXDELTAS)+6+1024];

int readNewTable(DXL_STREAM_HANDLE strmPtr)
{
    int maxBits,i;

    strmPtr->numDeltas = getBits(&(strmPtr->bitSource),9);
    maxBits = getBits(&(strmPtr->bitSource),5);

    for(i = 0; i < strmPtr->numDeltas; i++)
        strmPtr->deltas[i] = getSignedBits(&(strmPtr->bitSource),maxBits);

    return 0;
}

/*  ulong type;
    ulong width;
    ulong height;
    ulong depth;
    ulong headerSize;   // in archived bytes (4 per long word)
    ulong dataSize;
    ulong compressFlags; // 000vvhh0
    ulong reserved;
*/

#if defined(_bigend_h)
#define getLong(src) ((((short *)src->cData)[0] << 16) + ((unsigned short *)src->cData)[1]);  *src->cData++;
#else
#define getLong(src) *src->cData++;
#endif

static void initBitSource(DXL_TM2X_HANDLE src,
						  DXL_BITSOURCE *bitSource)
{
	bitSource->bitBuffer = getLong(src);
    bitSource->bitBuffer = invert4(bitSource->bitBuffer);
    bitSource->shortBuffer = (unsigned short *) src->cData;
    bitSource->numBits = 32;
}

int readHeader(DXL_TM2X_HANDLE src)
{
    unsigned long *data = (unsigned long *)src->addr;
    int hsize,vrad,hrad;

    src->cData = data;

    /* BE */
    src->type   = getLong(src);
    src->type = i4(src->type);

	/* BE: 1x4 bytes, format/length ffffllll llllllll llllllll llllllll */
	src->formatSize = getLong(src);
	src->formatSize = i4(src->formatSize);
    
	if (src->type != 0x00000100){
		DXL_BITSOURCE bitSource;

        initBitSource(src,&bitSource);

		src->dataSize = getBits(&bitSource,31);
		src->imHeight = (short)getBits(&bitSource,15);
		src->imWidth = (short)getBits(&bitSource,15);
		src->compressFlags = getBits(&bitSource,31);
		vrad = getBits(&bitSource,9);
		hrad= getBits(&bitSource,9);

        src->cData += 4;

		/*
        src->cData = (unsigned long *)bitSource.shortBuffer;
        if (bitSource.numBits == 32)
                src->cData--;
		*/
	}else{
		long temp;

		hsize   = getLong(src);
		hsize   = i4(hsize);
    
		src->dataSize = getLong(src);
		src->dataSize = invert4(src->dataSize);
    
		temp = getLong(src);
		src->imHeight = (short)invert4(temp);
    
		temp = getLong(src);
		src->imWidth = (short)invert4(temp);
    
		src->compressFlags = getLong(src);
		src->compressFlags = invert4(src->compressFlags);
    
		vrad = getLong(src);
		vrad = invert4(vrad);
    
		hrad = getLong(src);
		hrad = invert4(hrad);
	}
	src->w = src->imWidth;
	src->h = src->imHeight;
    return 0;
}


#define TABLE_ESCAPE 0x80000000


/* really simple "one-shot" allocater 
   once all space is used, all space is freed */
unsigned char *allocateTokenSpace(DXL_TM2X_HANDLE srcPtr,int tokenCount)
{

    if (srcPtr->hBufferTop >= tokenCount){
        srcPtr->hBufferTop -= tokenCount;
        return (srcPtr->hBuffer + srcPtr->hBufferTop);
    }else
        return (unsigned char *)0L;
}

int updateStream(DXL_TM2X_HANDLE srcPtr, int strmNum, DXL_STREAM_HANDLE strmPtr)
{
    long length;
    long *nextData;

    length = getLong(srcPtr);
    length = invert4(length);

    if (!length) return 0;

    nextData = (long *) srcPtr->cData + length;

    length = getLong(srcPtr);
    length = invert4(length);

    strmPtr->tokenCount = length >>1; /* number of tokens */
    
    strmPtr->tokenCPtr = (unsigned char *) (strmPtr->tokenPtr = (char *)
        allocateTokenSpace(srcPtr, strmPtr->tokenCount+8+(TBL_SIZE*TOK_SLEN)+96));
        
    strmPtr->htable = strmPtr->tokenPtr+strmPtr->tokenCount+8+32;
    strmPtr->htable = (char *)((unsigned long)strmPtr->htable & 0xfffffff0);

    strmPtr->tokenCPtr = (unsigned char *) (strmPtr->tokenPtr = strmPtr->htable - (strmPtr->tokenCount+8));

    strmPtr->htree = strmPtr->htable + (TBL_SIZE*TOK_SLEN);

    if (length & 1){ /* has a delta table */
        length = getLong(srcPtr);
        length = invert4(length);
        
		if (length == TABLE_ESCAPE){ /* allow for "forget" events */
            //resetAll(srcPtr);
            length = getLong(srcPtr);
            length = invert4(length);
        }

        if (length > 0){

			initBitSource(srcPtr,&(strmPtr->bitSource));

            readNewTable(strmPtr);
            srcPtr->cData = (unsigned long *) strmPtr->bitSource.shortBuffer;
            if (strmPtr->bitSource.numBits == 32)
                srcPtr->cData--;
        }
    }

    {
        unsigned long decoder;

        length = getLong(srcPtr);
        length = invert4(length);

		if (length == TABLE_ESCAPE){ /* allow for "forget" events */
			length = getLong(srcPtr);
			length = invert4(length);
		}

        decoder = getLong(srcPtr); /* get the decoder type/algorithm */
        decoder = invert4(decoder);
    }

	initBitSource(srcPtr,&(strmPtr->bitSource));

    readTree(strmPtr);

	/* redundancy alert ! */

	/* get position based on length of block */
    srcPtr->cData += (length-2) & 0x0fffffff;

	/* get position based on relative location (post bit-by-bit) */
	srcPtr->cData = (unsigned long *) strmPtr->bitSource.shortBuffer;
	if (strmPtr->bitSource.numBits == 32)
		srcPtr->cData--;

	/* end redundancy alert */

    length = getLong(srcPtr);
	length = invert4(length);

    if (length > 0){
        strmPtr->bitSource.shortBuffer = (unsigned short *) srcPtr->cData;
        reorder_cData(srcPtr->cData, length);
        buildTableFromTree(strmPtr);
        decodeTokens(strmPtr);
    }else
        duck_memset(strmPtr->tokenPtr,strmPtr->htree[0]>>1,strmPtr->tokenCount);

    srcPtr->cData += length & 0x0fffffff;
    srcPtr->cData = (unsigned long *) strmPtr->bitSource.shortBuffer;
    srcPtr->cData = (unsigned long *) nextData;

    return 0;
}

int tm2_xImageDestroy(DXL_TM2X_HANDLE src)
{
    duck_free(src->pBuffer1);	/* planar buffers */
	duck_free(src->hBuffer);	/* lossless heap */
    duck_free(src);				/* xImage */
    return 0;
}

int DXL_AlterTM2XData(DXL_TM2X_HANDLE src, char *data)
{
    src->addr = (unsigned char *) data;
    return 0;
}


int DXL_dTM2XToVScreen(DXL_TM2X_HANDLE src, DXL_VSCREEN_HANDLE vScreen)
{   
	if(!(src->addr))
	        return 0;

#ifdef _X86PROFILING
    tscStart(LOSSLESSDX);
#endif    

    /* read the 32 byte header */
    readHeader(src);    /* LE: 8x4 bytes */

    /* first we decompress from huffman to ordinals/deltas */
    /* reset the lossless heap */
    src->hBufferTop = src->hBufferSize;

    updateStream(src,STRM_CHROMAHI,&(src->chromaHi));
    updateStream(src,STRM_CHROMALO,&(src->chromaLo));
    updateStream(src,STRM_LUMAHI,&(src->LumaHi));
    updateStream(src,STRM_LUMALOW,&(src->LumaLo));
    updateStream(src,STRM_UPDATE,&(src->update));
    updateStream(src,STRM_MOTION,&(src->motion)); 
    updateStream(src,STRM_BLOCKTYPE,&(src->blockType));

#ifdef _X86PROFILING
    tscEnd(LOSSLESSDX);
#endif    

    if (src->frontBuffer != src->pBuffer2){
        src->frontBuffer = src->pBuffer2;
        src->backBuffer = src->pBuffer2+((src->imHeight*src->imWidth)*2)/4;
    }else{
        src->frontBuffer = src->pBuffer2+((src->imHeight*src->imWidth)*2)/4;
        src->backBuffer = src->pBuffer2;
    }
    
    duck_memset((unsigned char *)src->lineBuff,0,(src->imWidth*3)+12);

    {
        setupOffsetTable(src->imWidth/4);

        /* setting up for dx functions */
#ifdef _X86PROFILING
    tscStart(PLANARDX);
#endif    

        dxTM2Blocks(src, vScreen);

#ifdef _X86PROFILING
    tscEnd(PLANARDX);
#endif    
    }

    return DXL_OK; /* all OK? */
}

DXL_INTERNAL_FORMAT tm2_GetXImageInternalFormat(DXL_XIMAGE_HANDLE xImage,
												DXL_VSCREEN_HANDLE vScreen)
{
	return TM2_BLOCK24;
}

DXL_TM2X_HANDLE tm2_xImageReCreate(DXL_TM2X_HANDLE src,unsigned char *data,
	int type,enum BITDEPTH bitDepth,int w,int h);

DXL_TM2X_HANDLE tm2_xImageCreate(unsigned char *data)
{
	return tm2_xImageReCreate(NULL,data,0,0,0,0);
}

DXL_TM2X_HANDLE tm2_xImageReCreate(DXL_TM2X_HANDLE src,unsigned char *data,
	int type,enum BITDEPTH bitDepth,int w,int h)
{
    int area;
  
    if (data){
        long type = i4(*(long *)data);

        if ((type != 0x00000100) && (type != 0x00000101))
            return NULL;
    }

	if (src == NULL)
		src = (DXL_TM2X_HANDLE)duck_calloc(1,sizeof(DXL_TM2X),DMEM_GENERAL);

    if (src == NULL) return NULL;

    tm2_createGammaTable();
    tm2_createGamma162Table();

    src->dx = (int (*)(DXL_XIMAGE_HANDLE, DXL_VSCREEN_HANDLE)) DXL_dTM2XToVScreen;
    src->destroy = (int (*)(DXL_XIMAGE_HANDLE)) tm2_xImageDestroy;
    src->create =  (DXL_XIMAGE_HANDLE (*)(void *)) tm2_xImageCreate;
    src->recreate =  (DXL_XIMAGE_HANDLE (*)(DXL_XIMAGE_HANDLE,void *,int,int,int,int)) tm2_xImageReCreate;

    src->internalFormat = (int (*)(DXL_XIMAGE_HANDLE, DXL_VSCREEN_HANDLE)) tm2_GetXImageInternalFormat; 

    if (data){
        src->addr = data;

        readHeader(src); /* LE: 8x4 bytes */

        w = src->imWidth;
        h = src->imHeight;
    }else{
        src->imWidth = w;
        src->imHeight = h;
	}
    
    {
        long *vph;

        src->deltas = (long *)tm2Table;
        vph = src->deltas + MAXDELTAS*8;
        src->lineBuff = (unsigned long *) (vph + 6) ;
    }

    src->chromaHi.deltas    = src->deltas;
    src->chromaLo.deltas    = src->chromaHi.deltas  +MAXDELTAS;
    src->LumaHi.deltas      = src->chromaLo.deltas  +MAXDELTAS;
    src->LumaLo.deltas      = src->LumaHi.deltas    +MAXDELTAS;
    src->update.deltas      = src->LumaLo.deltas    +MAXDELTAS;
    src->motion.deltas      = src->update.deltas    +MAXDELTAS;
    src->blockType.deltas   = src->motion.deltas    +MAXDELTAS;

    area = w*h;

	if (src->pBuffer1){
		duck_free(src->pBuffer1);
		src->pBuffer2 = src->pBuffer1   = (unsigned long *)NULL;
	}

	if (src->hBuffer){
		duck_free(src->hBuffer);
		src->hBufferTop = src->hBufferSize = 0;
		src->hBuffer = (unsigned char *)NULL;
	}

	if (area){
		/* planar buffers */

		src->pBuffer1   = (unsigned long *) duck_calloc(1,(area*4)+32,DMEM_GENERAL);

		if (src->pBuffer1 == NULL){
			duck_free(src);
			return NULL;
		}
		src->pBuffer2 = (unsigned long *)((((unsigned long)src->pBuffer1)+31) & 0xffffffe0);

		/* lossless heap */
		src->hBufferTop = src->hBufferSize = 
			(area*3)/2 + (area/16)+(8*8) + ((TBL_SIZE*TOK_SLEN)+96)*8;

		src->hBuffer = (unsigned char *) duck_calloc(1,src->hBufferSize,DMEM_GENERAL);

		if (src->hBuffer == NULL){
			duck_free(src->pBuffer1);
			duck_free(src);
			return NULL;
		}
	}

    return src;
}

void tm2SetBlit(void);

int tm2_Init(int lmaxImages)
{
    DXL_RegisterXImage((DXL_XIMAGE_HANDLE (*)(unsigned char *)) tm2_xImageCreate,DXL_MKFOURCC('T','M','2','0'));

    tm2SetBlit();

#ifdef _X86PROFILING
    tscInit();
#endif
    return 0;
}

int tm2_Exit(void)
{
#ifdef _X86PROFILING
    tscUninit();
#endif
    return 0;
}

