;   Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.
;
;   This file is part of VpVision.
;
;   VpVision is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License ver. 2,
;   as published by the Free Software Foundation (www.fsf.org).
;
;   Foobar is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with VpVision; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
        .586
        .MODEL  flat, SYSCALL, os_dos
 
        .DATA 
        ALIGN 4  


; modified 10/97 by sjl to use eric's version of tm2
;
; 
; Blit from planar to a variety of output formats.  Timothy S. Murphy 10/12/96.
;
; UYVY and YUY2 blit.

; In the output, 2 pixels = one word.

; For UYVY, the format is U Y1 V Y2 in memory, Y2 V Y1 U in a register.
; For YUY2, the format is Y1 U Y2 V in memory, V Y2 U Y1 in a register.

; Per John, the right transform to use is:

; Y =  .299*R + .587*G + .114*B
; V =  .500*R - .419*G - .081*B + 128
; U = -.169*R - .331*G + .500*B + 128

; Note that the sum of the coefficients appearing in Y is 1
; and the the sums for V and U are 0.

; Our planar intermediate format stores G at full resolution,
; r = R-G  and  b = B-G at half resolution.  So

; U  = -.169*r + .500*b + 128
; Y1 =  .299*r + .114*b + G1
; V  =  .500*r - .081*b + 128
; Y2 =  .299*r + .114*b + G2

; We do this arithmetic in parallel (4-bytes at a time in 32-bit words).

; Each of the three addends is produced by shifting each (sign-extended)
; component to its proper position and adding all 4 of them together

; The resulting sum gives the correct 4 unsigned byte values
; provided each of those values stays in the range 0 to 255.

; The addends corresponding to r and b are simply looked up from tables,
; (which also include the +128 bias); green is handled manually.

; Note that we need r and b to be _signed_ here but that our intermediate
; planar format only stores their values mod 256.  We infer the (9th) sign bit
; by adding to (any one of) the corresponding green (luma) values; the sign
; bit equals the carry produced by that add.  This is a little tricky but it's
; better than trying to use the full R, G, B as in the first matrix because:

; (1) It saves a table lookup on green (we already have 4K worth of tables).
; (2) It gives truly full resolution in Y.

; These tables are indexed by a 9-bit index I (0 <= I <= 511) corresponding
; to an r = R - G  or  b = B - G value; these values range from -255 to +255.
; An index I with 0 <= I <= 255  corresponds to the value I.
; An index I with 257 <= I <= 511  corresponds to the value 512 - I.
; Note that -256 is an impossible index.


UYVYredTbl		dd	512 dup(?)
UYVYblueTbl		dd	512 dup(?)

YUY2blueTbl		dd	512 dup(?)
YUY2redTbl		dd	512 dup(?)
				dd	128 dup(?)

.CODE

NAME x86yuv

BYTEOFF2 EQU 32
INCLUDE locvars.ash

PUBLIC _YUVBlitInit
PUBLIC YUVBlitInit_

PUBLIC blitYUY2_
PUBLIC _blitYUY2

PUBLIC blitYVYU_
PUBLIC _blitYVYU

EXTRN jmp_dxBlock

; Initialize the tables used above.

Yquot   EQU 0
UVquot  EQU 4

YVUtblReset MACRO
	xor		edi, edi		; eax edi = 2-part accumulator 
	xor		eax, eax		; for U or V = -(uvVAL/1000) * index

	xor		esi, esi		; ebx esi = 2-part accumulator
	xor		ebx, ebx		; for Y =  (yVAL/1000) * index

	xor		edx, edx		; index runs from 0 to 255
ENDM


YVUtblInit	MACRO	yVAL, uvVAL
	mov		edx, yVAL
	xor		eax, eax
	mov		ebx, 1000
	div		ebx
	mov		Yquot[esp], eax		; (yVAL/1000) * (2<<32)

	mov		edx, uvVAL
	xor		eax, eax
	mov		ebx, 1000
	div		ebx
	mov		UVquot[esp], eax	; (uvVAL/1000) * (2<<32)

	YVUtblReset
ENDM


YVUtblBeg	MACRO	TBL, LAB
	mov		TBL[edx*4], ebp		; 0th entry is simple
	inc		dl
LAB:
	sub		edi, UVquot[esp]
	sbb		eax, 0
	add		esi, Yquot[esp]
	adc		ebx, 0
ENDM


YVUtblEnd	MACRO	TBL, LAB
	add		ecx, ebp		; add in bias
	mov		TBL[edx*4], ecx	; entry for I (1 <= I <= 255)

	neg		edx
	neg		ecx				; entry for -I = -(entry for I)
	lea		ecx, [ecx+2*ebp]	; except bias is fixed (so add twice)
	mov		TBL+(512*4)[edx*4], ecx

	neg		edx
	inc		dl
	jne		LAB
ENDM


	; Call this routine once at startup.

_YUVBlitInit:
YUVBlitInit_:
	pushad
	sub		esp, 8

	; First build red tables

	YVUtblInit	299, 169

	mov		ebp, 128*65536	; bias the V component of r's vector

	YVUtblBeg	UYVYredTbl, __r1

	mov		ch, bl			; byte 3 = Y2 = Y = bl
	mov		cl, dl			; byte 2 = V = .500 * r (>0 here)
	shr		cl, 1
	shl		ecx, 16
	mov		ch, bl			; byte 1 = Y1 = Y = bl (>0 here)
	add		ecx, eax		; add in byte 0 = U = eax (<0 here)

	YVUtblEnd	UYVYredTbl, __r1
	

	YVUtblReset

	mov		ebp, 32768*65536	; bias the V component of r's vector

	YVUtblBeg	YUY2redTbl, __r2

	mov		ch, dl			; byte 3 = V = .500 * r
	shr		ch, 1
	mov		cl, bl			; byte 2 = Y2 = Y = bl (>0 here)
	shl		ecx, 8
	add		ecx, eax		; add in byte 1 = U = eax (<0 here)
	shl		ecx, 8
	mov		cl, bl			; byte 0 = Y1 = Y = bl (>0 here)

	YVUtblEnd	YUY2redTbl, __r2
	

	; Blue tables are similar.

	YVUtblInit	114, 81

	mov		ebp, 128		; bias the U component of b's vector

	YVUtblBeg	UYVYblueTbl, __b1

	mov		ecx, eax		; byte 2 = V = eax (<0 here)
	add		ch, bl			; byte 3 = Y2 = Y = bl
	shl		ecx, 16
	mov		ch, bl			; byte 1 = Y1 = Y = ebx (>0 here)
	mov		cl, dl			; byte 0 = U = .500 * b (>0 here)
	shr		cl, 1

	YVUtblEnd	UYVYblueTbl, __b1
	

	YVUtblReset

	mov		ebp, 32768		; bias the U component of b's vector

	YVUtblBeg	YUY2blueTbl, __b2

	mov		ch, al			; byte 3 = V = al (<0 here but on top)
	mov		cl, bl			; byte 2 = Y2 = Y = bl (>0 here)
	shl		ecx, 16
	mov		ch, dl			; byte 1 = U = .500 * b (>0 here)
	shr		ch, 1
	mov		cl, bl			; byte 0 = Y1 = Y = bl (>0 here)

	YVUtblEnd	YUY2blueTbl, __b2
	
	add		esp, 8
	popad
	ret

PURGE	YVUtblReset, YVUtblInit, YVUtblBeg, YVUtblEnd

;------------------------------------------------
; writes 4 yuv pixels
;------------------------------------------------
byuvPix MACRO scrnLoc, chroma, luma, TYPE, BSHF, BREG, BMASK
    mov     bl,BYTE PTR luma[esi+3]
    mov     ecx,[esi+(chroma+0)]                ;rb rb rb rb b b b b

    movsx   eax,cx

    mov     bh,BYTE PTR luma[esi+2]
    sub     ecx,eax                             ;sub out blue diff

    shr     ecx,16
    and     eax,0ffh

    mov     ch,al
    mov     al, bl

    shl     ebx, BSHF                           ;ebx will be G2 0 G1 0 or  0 G2 0 G1
    and     ecx,0ffffh

    mov     edx, eax
    add     al, cl                              ;eax will be 9-bit signed r1 = R1 - G1

    adc     ah, ah
    mov     BREG, dl

    mov     al, cl
    add     dl, ch                              ;edx will be 9-bit signed b1 = B1 - G1

    adc     dh,dh
    and     ebx,BMASK

    mov     eax,TYPE&redTbl[eax*4]
    mov     dl,ch

    add     eax,ebx
nop

    mov     bl,BYTE PTR luma[esi+1]             ;g1   
    mov     edx,TYPE&blueTbl[edx*4]

    mov     ecx,[esi+(chroma+4)]                ;rb rb rb rb b b b b
    add     eax,edx

    mov     bh,BYTE PTR luma[esi+0]             ;g0   
    mov     [scrnLoc], eax                      ;write first 2 pixels

    movsx   eax,cx

    sub     ecx,eax                             ;sub out blue diff
nop

    shr     ecx,16
    and     eax,0ffh

    mov     ch,al
    mov     al, bl

    mov     edx,ecx                             ;edx = 0 0 b2 r2
    mov     ecx,eax                             ;ecx will be 9-bit signed b2

    shl     ebx, BSHF
    and     edx,0ffffh

    add     al, dl                              ;eax will be 9-bit signed r2
    mov     BREG, cl

    adc     ah, ah
    and     ebx, BMASK                          ;ebx = G4 0 G3 0 or  0 G4 0 G3 

    mov     al, dl
    add     cl, dh

    adc     ch, ch
    mov     eax, TYPE&redTbl[eax*4]

    mov     cl, dh
    add     eax, ebx

    mov     ecx, TYPE&blueTbl[ecx*4]
nop

    add     eax, ecx
nop

    mov     4[scrnLoc], eax
ENDM
;------------------------------------------------
blitYUY2_:
_blitYUY2:
    mov     esi,L_lcDest[esp]
nop

    mov     edi,L_vScreenAddr[esp] 
    mov     ebp,L_vScreenPitch[esp]

    byuvPix edi, %0-BYTEOFF2, %16+0-BYTEOFF2, YUY2, 8, bl, 0ff00ffh 
    byuvPix edi+ebp, %0-BYTEOFF2, %16+4-BYTEOFF2, YUY2, 8, bl, 0ff00ffh 
    byuvPix edi+ebp*2, %8-BYTEOFF2, %16+8-BYTEOFF2, YUY2, 8, bl, 0ff00ffh 
    mov     ebp,L_vScreenPitch3[esp]

    byuvPix edi+ebp, %8-BYTEOFF2, %16+12-BYTEOFF2, YUY2, 8, bl, 0ff00ffh 

    add     edi,8
    mov     eax,L_blockInfoPtr[esp]

    mov     L_vScreenAddr[esp],edi
    jmp     [jmp_dxBlock]
;------------------------------------------------
blitYVYU_:
_blitYVYU:
    mov     esi,L_lcDest[esp]
nop

    mov     edi,L_vScreenAddr[esp] 
    mov     ebp,L_vScreenPitch[esp]

    byuvPix edi, %0-BYTEOFF2, %16+0-BYTEOFF2, UYVY, 16, bh, 0ff00ff00h 
    byuvPix edi+ebp, %0-BYTEOFF2, %16+4-BYTEOFF2, UYVY, 16, bh, 0ff00ff00h
    byuvPix edi+ebp*2, %8-BYTEOFF2, %16+8-BYTEOFF2, UYVY, 16, bh, 0ff00ff00h
    mov     ebp,L_vScreenPitch3[esp]

    byuvPix edi+ebp, %8-BYTEOFF2, %16+12-BYTEOFF2, UYVY, 16, bh, 0ff00ff00h

    add     edi,8
    mov     eax,L_blockInfoPtr[esp]

    mov     L_vScreenAddr[esp],edi
    jmp     [jmp_dxBlock]
    
;************************************************
    END
