/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "dkpltfrm.h"
#include <string.h>
#include <stdlib.h>

#include "duktypes.h"
//#include "cduck.h"       

#include <duck_wav.h>                     
#include <duck_mem.h>                       
#include "dxl_main.h"
#include "dxl_audi.h"
#include "dxaprof.h"

static DXL_XAUDIOSRC_HANDLE xAudioSrcs;
static DXL_AUDIODST_HANDLE AudioDsts;
static int maxSrcs,maxDsts;

/* initialize the audio portion of the DXL */
int DXL_InitAudio(int lmaxSrcs,int lmaxDsts)
{
#ifdef _X86PROFILING
    tscAudioInit();
#endif

        xAudioSrcs = (DXL_XAUDIOSRC_HANDLE)duck_calloc(maxSrcs = lmaxSrcs,sizeof(DXL_XAUDIOSRC),DMEM_GENERAL);
	if (xAudioSrcs == NULL) return -1;
	
        AudioDsts = (DXL_AUDIODST_HANDLE)duck_calloc(maxDsts = lmaxDsts,sizeof(DXL_AUDIODST),DMEM_GENERAL);
	
	if (AudioDsts == NULL){
		duck_free(xAudioSrcs);
		xAudioSrcs = (DXL_XAUDIOSRC_HANDLE)NULL;
		return -1;
	}else
		return 0;	
}

/* close down the audio portion of the DXL */
void DXL_ExitAudio(void)
{                                     
#ifdef _X86PROFILING
    tscAudioUninit();
#endif

	if (xAudioSrcs) duck_free(xAudioSrcs);
	if (AudioDsts) duck_free(AudioDsts);        
	                                    
	xAudioSrcs = (DXL_XAUDIOSRC_HANDLE)NULL;
	AudioDsts = (DXL_AUDIODST_HANDLE)NULL;
}	

/* change the data element of the data source */
void DXL_AlterXAudioData(DXL_XAUDIOSRC_HANDLE xSource,unsigned char *addr,int length)
{
	if (xSource){
		if (xSource->numChannels){
			xSource->addr = addr;
			xSource->length = length;
			xSource->finiBlock = 1;
			xSource->samplesRead = xSource->totalPos = 0;
			
			if (xSource->wv.wFormatTag == 1)
				xSource->samplesPerBlock = 
					length/(xSource->wv.nChannels *(xSource->wv.wBitsPerSample/8));
					
			/*if (addr == NULL) xSource->samplesPerBlock = length;*/
		}
	}
}

/* create an audio source */		
DXL_XAUDIOSRC_HANDLE DXL_CreateXAudioSrc(DKWAVEFORM *wv, unsigned char *addr,int length)
{
	int i;
	DXL_XAUDIOSRC_HANDLE xSource = xAudioSrcs;

	if (wv == NULL) return NULL;
	
	if (wv->nChannels == 0) return NULL;
	
	for(i=0; i < maxSrcs; i++,xSource++)
		if (xSource->numChannels == 0) break;
		
	if (i >= maxSrcs) return NULL;
	                                     
	xSource->numChannels = wv->nChannels;
	xSource->samplesPerBlock = wv->wSamplesPerBlock;
	
	xSource->wv.wFormatTag = wv->wFormatTag;			/* format type */
	xSource->wv.nChannels = wv->nChannels;				/* number of channels (i.e. mono, stereo...) */
	xSource->wv.nSamplesPerSec = wv->nSamplesPerSec;	/* sample rate */
	xSource->wv.nAvgBytesPerSec = wv->nAvgBytesPerSec;	/* for buffer estimation */
	xSource->wv.nBlockAlign = wv->nBlockAlign;			/* block size of data */
	xSource->wv.wBitsPerSample = wv->wBitsPerSample;	/* Number of bits per sample of mono data */
	xSource->wv.cbSize = wv->cbSize;					/* The count in bytes of the size of
														extra information (after cbSize) */
	xSource->wv.wSamplesPerBlock = wv->wSamplesPerBlock;
	
	DXL_AlterXAudioData(xSource,addr,length);
	return xSource;
}

/* destory an audio source */
void DXL_DestroyXAudioSrc(DXL_XAUDIOSRC_HANDLE xSource)
{
	if (xSource) xSource->numChannels = 0;
}
void DXL_AlterAudioDst(
	DXL_AUDIODST_HANDLE dest,
	void *addrL,void *addrR,
	int length,int bitDepth,
	int numChannels,
	int sampleRate)
{
	if (dest)
		if (dest->numChannels){
			if (numChannels <= 0) numChannels = dest->numChannels;
			
			if (addrL != NULL) dest->addrL = (unsigned char *) addrL;
			if ((addrR != NULL) || (numChannels != dest->numChannels))
				dest->addrR = (numChannels == 2) ? (unsigned char *) addrR : dest->addrL;
			
			dest->index = 0;
			if (length > 0) dest->length = length;
			if (bitDepth > 0) dest->bitDepth = bitDepth;
			if (numChannels > 0) dest->numChannels = numChannels;
			if (sampleRate > 0) dest->SampleRate = sampleRate;
		}
}

/* create/destory an audio destination AUDIODST */		
DXL_AUDIODST_HANDLE DXL_CreateAudioDst(
	void *addrL,void *addrR,
	int length,int bitDepth,
	int numChannels,
	int sampleRate)
{
	int i;
	DXL_AUDIODST_HANDLE dest = AudioDsts;
	
	if (!numChannels) return NULL;
	
	for(i=0; i < maxDsts; i++,dest++)
		if (dest->numChannels == 0) break;
		
	if (i >= maxDsts) return NULL;
	
	dest->numChannels = numChannels;
	
	DXL_AlterAudioDst(dest,addrL,addrR,length,bitDepth,
		numChannels,sampleRate);

	return dest;
}

/* destory an audio destination */
void DXL_DestroyAudioDst(DXL_AUDIODST_HANDLE dst)
{
	if (dst) dst->numChannels = 0;
}
