/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/***********************************************\
??? dxa.c
? profiling functions
? also see perf.asm and pentium.asm
\***********************************************/
#include "duck_mem.h"
//#include "..\src\dxaprof.h"

#define MAX_TIMES 1024*16

typedef struct profStruct {
    unsigned long pkc1;
    unsigned long pkc2;
    unsigned long numTimes;
    unsigned long *Times;
    unsigned long *firstTime;
}PSECTION;

static PSECTION DK4audio;
static PSECTION DK3audio;
static PSECTION PCMaudio;

static int profStarted = 0;

enum AUDIOSECTION { DK4AUDIO, DK3AUDIO, PCMAUDIO };
 
extern void addTSC(unsigned long *, unsigned long, unsigned long *);
unsigned long pentiumKiloCycles(void);
void DUCK_cli(void);
void DUCK_sti(void);

#if 1
/***********************************************/
void tscAudioStart(enum AUDIOSECTION pSection) 
{
    if(profStarted) {
        switch(pSection) {
            case DK4AUDIO:
                DK4audio.pkc1 = pentiumKiloCycles();
            break;
            case DK3AUDIO:
                DK3audio.pkc1 = pentiumKiloCycles();
            break;
            case PCMAUDIO:
                PCMaudio.pkc1 = pentiumKiloCycles();
            break;
        }
    
//        DUCK_cli();
    }        
}

/***********************************************/
void tscAudioEnd(enum AUDIOSECTION pSection) 
{
    if(profStarted) {
        switch(pSection) {
            case DK4AUDIO:
                DK4audio.pkc2 = pentiumKiloCycles();
                DK4audio.pkc2 = (DK4audio.pkc2 - DK4audio.pkc1);
                DK4audio.numTimes += 1;
                *DK4audio.Times++ = DK4audio.pkc2;    
            break;
            case DK3AUDIO:
                DK3audio.pkc2 = pentiumKiloCycles();
                DK3audio.pkc2 = (DK3audio.pkc2 - DK3audio.pkc1);
                DK3audio.numTimes += 1;
                *DK3audio.Times++ = DK3audio.pkc2;    
            break;
            case PCMAUDIO:
                PCMaudio.pkc2 = pentiumKiloCycles();
                PCMaudio.pkc2 = (PCMaudio.pkc2 - PCMaudio.pkc1);
                PCMaudio.numTimes += 1;
                *PCMaudio.Times++ = PCMaudio.pkc2;    
            break;
        }

//        DUCK_sti();
    }
}

/***********************************************/
void tscAudioInit() 
{
    DK4audio.firstTime = DK4audio.Times = 
        (unsigned long *)duck_calloc(MAX_TIMES, sizeof(long), DMEM_GENERAL);
    DK3audio.firstTime = DK3audio.Times = 
        (unsigned long *)duck_calloc(MAX_TIMES, sizeof(long), DMEM_GENERAL);
    PCMaudio.firstTime = PCMaudio.Times = 
        (unsigned long *)duck_calloc(MAX_TIMES, sizeof(long), DMEM_GENERAL);
    profStarted = 1;
}

/***********************************************/
void tscAudioUninit() 
{
    if(profStarted) {
        duck_free(DK4audio.firstTime);
        duck_free(DK3audio.firstTime);
        duck_free(PCMaudio.firstTime);
    }
    profStarted = 0;
}

/***********************************************/
unsigned long tscAudioProcessCounts(unsigned long *cnt, enum AUDIOSECTION pSection) 
{
    unsigned long rv = 0;

    *cnt = 0;
    if(profStarted) {
        switch(pSection) {
            case DK4AUDIO:
                DK4audio.pkc1 = pentiumKiloCycles();
                if(DK4audio.numTimes) {
                    addTSC(DK4audio.firstTime, DK4audio.numTimes, &rv);
                    *cnt = DK4audio.numTimes;
                }
                DK4audio.numTimes = 0L;
                DK4audio.Times = DK4audio.firstTime;
            break;
            case DK3AUDIO:
                DK3audio.pkc1 = pentiumKiloCycles();
                if(DK3audio.numTimes) {
                    addTSC(DK3audio.firstTime, DK3audio.numTimes, &rv);
                    *cnt = DK3audio.numTimes;
                }
                DK3audio.numTimes = 0L;
                DK3audio.Times = DK3audio.firstTime;
            break;
            case PCMAUDIO:
                PCMaudio.pkc1 = pentiumKiloCycles();
                if(PCMaudio.numTimes) {
                    addTSC(PCMaudio.firstTime, PCMaudio.numTimes, &rv);
                    *cnt = PCMaudio.numTimes;
                }
                PCMaudio.numTimes = 0L;
                PCMaudio.Times = PCMaudio.firstTime;
            break;
        }
    }
    return (rv);
}
#endif

