/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "dkpltfrm.h"
#include <stdlib.h>

#include "duktypes.h"
#include "cduck.h"       
                     
#include <duck_wav.h>                       
#include "dxl_main.h"
#include "dxl_audi.h"
#include "duck_mem.h"
#include "dxaprof.h"

#if AUDIOINTERLEAVED
#define outData8Right outData8Left
#define outData16Right outData16Left
#endif
                                                           
int DXL_dxPCMAudio(DXL_XAUDIOSRC_HANDLE src,DXL_AUDIODST_HANDLE dst,int length)
{	
	int j,numGotten = 0;
	register int numToGet;
	register unsigned char *data = src->addr;
	
	if (data != NULL){
		if (length > src->samplesPerBlock) length = src->samplesPerBlock;
		src->samplesPerBlock -= length;
	}
	
	numGotten = length;            
	
	if (!length) return 0;
			
	j = dst->index % dst->length; 
	
	if (src->wv.wBitsPerSample == 8){
		register unsigned char *outData8Left = (unsigned char *)dst->addrL;
#if !AUDIOINTERLEAVED
		register unsigned char *outData8Right = (unsigned char *)dst->addrR;
#endif
		outData8Left += j;	 
				
		if (dst->numChannels == 2)
			outData8Right += j;

		do{
			if (length + j > dst->length)
				numToGet = dst->length - j;
			else
				numToGet = length;			
			
			length -= numToGet;	

			switch(src->numChannels){
				case 1:	/* 8 bit mono to 8 bit mono */
					if (data != NULL) {
#if 1
                        duck_memcpy(outData8Left,data,numToGet);
                        data += numToGet;
                        numToGet = 0;
#else
						while(numToGet--) *outData8Left++ = *data++; /* - 128; */
#endif
					} else {
#if 1
                        duck_memset(outData8Left,128,numToGet);
//                        duck_memset(outData8Left,0,numToGet);
                        data += numToGet;
                        numToGet = 0;
#else
						while(numToGet--) *outData8Left++  = 0;
#endif
                    }

					break;
				case 2: /* 8 bit stereo to 8 bit stereo */
					if (data != NULL){
						while(numToGet--){      
							*outData8Left++ = *data++; // - 128;
							*outData8Right++ = *data++; // - 128;
						}
					} else {
#if 1
                        duck_memset(outData8Right,128,numToGet);
                        duck_memset(outData8Left,128,numToGet);
                        numToGet = 0;
#else
						while(numToGet--) *outData8Right++ = *outData8Left++  = 0; 
#endif
                    }
					break;
			}
			outData8Left = (unsigned char *)dst->addrL; 
#if !AUDIOINTERLEAVED
			outData8Right = (unsigned char *)dst->addrR;
#endif
			j=0;
		}while(length > 0);
	}else{ /* Do the 16 bit version */
		register unsigned short *outData16Left = (unsigned short *)dst->addrL;
#if !AUDIOINTERLEAVED
		register unsigned short *outData16Right = (unsigned short *)dst->addrR;
#endif
#if 0
		register unsigned short temp;
#endif
	    
		outData16Left += j;
		
		if (dst->numChannels == 2)
			outData16Right += j;
			
		do{
			if (length + j > dst->length)
				numToGet = dst->length - j;
			else
				numToGet = length;			

			length -= numToGet;	
			
			switch(src->numChannels){                                 
			    case 1: /* 16 bit mono to 16 bit stereo */
					if (data != NULL){
#if 1
                        duck_memcpy(outData16Left,data,numToGet*2);
                        data += numToGet*2;
                        numToGet = 0;
#else
						while(numToGet--){               
							temp = (unsigned short)*data++;
							temp |= (unsigned short)*data++ << 8; 
							*outData16Left++ = temp;
						}
#endif
					}else
						while(numToGet--) *outData16Left++  = 0x8000;						
					break;
				case 2: /* pure 16 bit stereo */
					if (data != NULL){ 
#if 1
                        duck_memcpy(outData16Left,data,numToGet*4);
                        data += numToGet*4;
                        numToGet = 0;
#else
						while(numToGet--){                
							temp = (unsigned short)*data++;
							temp |= (unsigned short)*data++ << 8; 	
							*outData16Left++ = temp;
							temp = (unsigned short)*data++;     
							temp |= (unsigned short)*data++ << 8;
							*outData16Right++ = temp;
						}
#endif
					}else
						while(numToGet--) *outData16Right++ = *outData16Left++  = 0x8000;						
					break;
			}                                               
			outData16Left = (unsigned short *)dst->addrL; outData16Right = (unsigned short *)dst->addrR;
			j=0;
		}while(length > 0);
	}
	dst->index += numGotten;	/* update starting point for next time */
	src->addr = data;
	return numGotten;	
}

#undef outData16Right
#undef outData8Right

/* decompress a number of samples from src to dst */
int DXL_dxAudio(DXL_XAUDIOSRC_HANDLE src,DXL_AUDIODST_HANDLE dst,int length)
{
	int j,numToGet,numGotten = 0,nDecoded = 0;
	unsigned short *outData16Left = (unsigned short *)dst->addrL;
	unsigned short *outData16Right = (unsigned short *)dst->addrR;
    int rv;
        
    if ((src->addr != NULL) && (src->length == 0)) return 0;
    
    if ((src->wv.wFormatTag == 1) || (src->addr == NULL)) {
#ifdef _X86PROFILING
    tscAudioStart(PCMAUDIO);
#endif
    	rv = DXL_dxPCMAudio(src,dst,length);
#ifdef _X86PROFILING
    tscAudioEnd(PCMAUDIO);
#endif
        return rv;
    }
    	
	j = dst->index % dst->length; 
	
	outData16Left += j;	 
		
#if AUDIOINTERLEAVED
		if (dst->numChannels == 2) outData16Left += j;
		outData16Right = (unsigned short *)-1L;
#else
		outData16Right += j;
#endif

	do{
		if (length + j > dst->length)
			numToGet = dst->length - j;
		else
			numToGet = length;			
		
		length -= numToGet;	
        
        if (src->wv.wBitsPerSample != 3){
	       	if (dst->numChannels != 2){
	       		outData16Right = NULL;
			}
#ifdef _X86PROFILING
    tscAudioStart(DK4AUDIO);
#endif
			numGotten += (nDecoded = dxDK4(src,(short *) outData16Left,(short *) outData16Right,numToGet));
#ifdef _X86PROFILING
    tscAudioEnd(DK4AUDIO);
#endif
		}else{
#ifdef _X86PROFILING
    tscAudioStart(DK3AUDIO);
#endif
			numGotten += (nDecoded = dxDK3S(src,(short *) outData16Left,(short *) outData16Right,numToGet));
#ifdef _X86PROFILING
    tscAudioEnd(DK3AUDIO);
#endif
		}	
		outData16Left = (unsigned short *)dst->addrL; 
#if !AUDIOINTERLEAVED
		outData16Right = (unsigned short *)dst->addrR;
#endif
		j=0;
		
	}while((length > 0) && (nDecoded == numToGet));
	
	dst->index += numGotten;	/* update starting point for next time */

	return numGotten;	
}
