/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CAppServices.h>
#include	<stack_aids.h>

/******************************************************************************
CAppServices

A class that encapsulates some general purpose stuff
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// CAppServices
//
// Constructor
///////////////////////////////////////////////////////////////////////////////
CAppServices::CAppServices(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// ~CAppServices
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CAppServices::~CAppServices(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// Daemonize
//
// Convert this process into a deamon via fork
///////////////////////////////////////////////////////////////////////////////
void
CAppServices::Daemonize(void)
{
	// not yet supported under windows or Mac

#if ((!defined WIN32) && (!defined __POWERPC__))

	pid_t	pid;
	if ((pid = fork()) < 0)
		throw DaemonizeException("CAppServices::Daemonize",pid);
	if (pid != 0)
		exit(0);

	setsid();
	chdir("/");
	umask(0);
	close(0);
	close(1);
	close(2);

#endif
}

///////////////////////////////////////////////////////////////////////////////
// ExtractArgs
//
// Build an internal data structure from the argc and argv unix inputs
// for later retrieval via HasArg() and GetArgAsXXXX()
//
// We assume arguments are unix like
// for example:
//
//	-f filename
//
// would be an example. In this case "-f" is the argument and "filename"
// is the value. It is assumed that the arguments come in pairs though
// this will still work if the last argument only has no value.
//
// INPUT:
//	argc	- as passed into main()
//	argv	- as passed into main()
///////////////////////////////////////////////////////////////////////////////
void
CAppServices::ExtractArgs(int argc,char *argv[])
{
	stack_MutexLock sml(&mArgTableMutex);

	nmr_string	k,v;

	if (argc) 
		mAppName = argv[0];

	for (int x = 1; x < argc; x++)
		{
		k = argv[x];
		v = "";		
		
		if ((x + 1 < argc) && (argv[x+1][0] != '-'))
			v = argv[++x];

		mArgTable[k] = v;
		}
}

// WIN32 style arguments version
void
CAppServices::ExtractArgs(char *arg)
{
	stack_MutexLock sml(&mArgTableMutex);

	nmr_string	k,v;

	mAppName = "server";

	char *ptr;

	ptr = strtok(arg," \t\n");

	while (ptr)
		{
		k = ptr;
		v = "";

		ptr = strtok(NULL," \t\n");		
		
		if (ptr && (ptr[0] != '-'))
			{
			v = ptr;
			ptr = strtok(NULL," \t\n");
			}
		mArgTable[k] = v;
		}
}

///////////////////////////////////////////////////////////////////////////////
// HasArg
//
// Return true if the argument exists. 
//
// INPUT:
//	arg - the string we are checking for
//
// OUTPUT:
//	true if it exists
///////////////////////////////////////////////////////////////////////////////
int
CAppServices::HasArg(const char *arg)
{
	stack_MutexLock sml(&mArgTableMutex);

	return  (mArgTable.find(nmr_string(arg)) != mArgTable.end());
}

//////////////////////////////////////////////////////////////////////////////
// GetArgAsString
//
// Return the argument value as a string
//
// INPUT:
//	arg - the argument
//
// OUTPUT:
// 	the value of the argument
///////////////////////////////////////////////////////////////////////////////
nmr_string
CAppServices::GetArgAsString(const char *arg)
{
	stack_MutexLock sml(&mArgTableMutex);

	return mArgTable[arg];
}

//////////////////////////////////////////////////////////////////////////////
// GetArgAsInt
//
// Return the argument value as an integer
//
// INPUT:
//	arg - the argument
//
// OUTPUT:
// 	the value of the argument as an int
///////////////////////////////////////////////////////////////////////////////
int
CAppServices::GetArgAsInt(const char *arg)
{
	stack_MutexLock sml(&mArgTableMutex);

	return atoi(mArgTable[arg]);
}

unsigned long
CAppServices::GetArgAsUnsignedLong(const char *arg)
{
	stack_MutexLock sml(&mArgTableMutex);

	return strtoul(mArgTable[arg],NULL,10);
}

///////////////////////////////////////////////////////////////////////////////
// GetAppName
//
// Returns the application name as appearing in argv[0]
//
// OUTPUT:
//	the application name
///////////////////////////////////////////////////////////////////////////////
nmr_string
CAppServices::GetAppName(void)
{
	return mAppName;
}

