/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CCMutex.h>
#include	<nmr_string.h>
#include	<system_ticks.h>
#include	<milli_sleep.h>
#ifdef	__POWERPC__
#include	<CCThread.h>
#endif

/********************************************************
CCMutex

A trivial encapsulation of pthread mutex calls. Exceptions
are thrown instead of errors being returned.
***********************************************************/


CCMutex::CCMutex(void)
{

#ifdef	WIN32
	mMutex = CreateMutex(NULL,FALSE,NULL);
	if (mMutex == NULL)
		throw CtorException("CCMutex::CCMutex",(int)GetLastError());

#elif __POWERPC__
	mMutex = 0;
#else
	int	err;

	if (err=pthread_mutex_init(&mMutex,NULL))
		throw CtorException("CCMutex::CCMutex",err);
#endif
}

CCMutex::~CCMutex(void)
{

#ifdef WIN32
	if (mMutex != NULL)
		CloseHandle(mMutex);

#elif __POWERPC__
#else
	int err;

	if (err=pthread_mutex_destroy(&mMutex))
		throw DtorException("CCMutex::~CCMutex",err);
#endif
}

int				
CCMutex::TryLock(void)
{
#ifdef WIN32
	DWORD	err;

	err = WaitForSingleObject(mMutex,0);
	if (err == WAIT_TIMEOUT)
		return 0;
	if (err == WAIT_FAILED)
		throw TryLockException("CCMutex::TryLock",(int)GetLastError());
	return 1;

#elif __POWERPC__
	if (mMutex) return 0;
	mMutex = 1;
	return 1;
#else
	int err;
	err = pthread_mutex_trylock(&mMutex);
	if (err == EBUSY)
		return 0;
	if (!err)
		return 1;
	throw TryLockException("CCMutex::TryLock",err);
#endif
}

int				
CCMutex::TryLock(int milli_timeout)
{
#ifdef WIN32
	DWORD	err;

	err = WaitForSingleObject(mMutex,milli_timeout);
	if (err == WAIT_TIMEOUT)
		return 0;
	if (err == WAIT_FAILED)
		throw TryLockException("CCMutex::TryLock",(int)GetLastError());
	return 1;

#elif __POWERPC__
	U32	start_time = timeGetTime();
	
	while (mMutex)
		{
		U32 elapsed = timeGetTime() - start_time;
		if (elapsed > milli_timeout) return 0;
		CCThread::Yield();
		}
	mMutex = 1;
	return 1;
#else
	int err;
	int	wait_left = milli_timeout / 10;	
	// round time so we don't thrash like crazy. The time really
	// doesn't need to be that accurate

	do 
		{
		err = pthread_mutex_trylock(&mMutex);
		if (!err)
			return 1;
		if (err != EBUSY)
			throw TryLockException("CCMutex::TryLock",err);
		milli_sleep(10);
		wait_left--;
		}
	while (wait_left > 0);
	return 0;

#endif
}

void
CCMutex::Lock(void)
{

#ifdef	WIN32
	DWORD	err;

	err = WaitForSingleObject(mMutex,INFINITE);
	if (err == WAIT_FAILED)
		throw LockException("CCMutex::Lock",(int)GetLastError());

#elif __POWERPC__
	while (mMutex) CCThread::Yield();
	mMutex = 1;
#else
	int err;

	if (err=pthread_mutex_lock(&mMutex))
		throw LockException("CCMutex::Lock",err);
#endif
}

void
CCMutex::Unlock(void)
{
#ifdef	WIN32
	if (!ReleaseMutex(mMutex))
		throw UnlockException("CCMutex::Unlock",(int)GetLastError());

#elif __POWERPC__
	mMutex = 0;
	CCThread::Yield();
#else
	int err;

	if (err=pthread_mutex_unlock(&mMutex))
		throw UnlockException("CCMutex::Unlock",err);
#endif
}

void
CCMutex::Dump(ostream &o,const char *indent) const
{
	o << indent << "CCMutex" << endl
		<< indent << "   mMutex: " << (&mMutex) << endl;
}

ostream& operator<<(ostream &o,CCMutex &m)
{
	m.Dump(o,"");
	return o;
}

ostream& operator<<(ostream &o,CCMutex *m)
{
	if (m)
		m->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}


