/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CCTHREAD_H_
#define CCTHREAD_H_

#include	<MegaHeaders.h>

#ifdef	WIN32
#undef	Yield
typedef	unsigned long	CCThread_ID_Type;

#elif __POWERPC__
#include <Threads.h>
typedef ThreadID	CCThread_ID_Type;

#else
typedef pthread_t	CCThread_ID_Type;

#endif

#include	<CError.h>

#define	JOIN_DELETE_CLEAR(x) \
	if (x) \
		{ \
		x->Join(); \
		delete x;\
		x = NULL;\
		}


class CCThread
{
	CLASS_NOCOPY(CCThread)

protected:
	int					mRunning;
	CCThread_ID_Type	mThreadID;

	void			_Go(void);
	virtual	void	Run(void) = 0; // your code goes here!!
	static void		ForceExit(void);

public:
	CCThread(void);
	virtual ~CCThread(void);
	virtual	void	Start(void);
	virtual	void	Join(void);
	virtual	void	Cancel(void);
	virtual	void	Kill(int signal);
	virtual	int		IsRunning(void) const;

	static void 				Yield(void);
	static CCThread_ID_Type		Self(void);

	virtual	CCThread_ID_Type	ReturnNativePtr(void) const;
	virtual	void	Dump(ostream &o,const char *ident);
	virtual const char *Name(void);

	CERROR_SUBCLASS(CtorException);
	CERROR_SUBCLASS(DtorException);
	CERROR_SUBCLASS(StartException);
	CERROR_SUBCLASS(JoinException);
	CERROR_SUBCLASS(YieldException);
	CERROR_SUBCLASS(CancelException);

#ifdef	WIN32
	friend unsigned int __stdcall thread_start(void *arg);
#elif __POWERPC__
	friend pascal void *thread_start(void *arg);
#else
	friend void *thread_start(void *arg);
#endif
};

ostream& operator << (ostream &o,CCThread &t);
ostream& operator << (ostream &o,CCThread *t);

#ifdef __POWERPC__

#define	NMR_MAIN \
int nmr_main(int argc,char *argv[]); \
\
int main(int argc,char *argv[])\
{\
	MaxApplZone();  \
	argc = ccommand(&argv);\
	printf("\n");\
	return nmr_main(argc,argv);\
}\
int nmr_main(int argc,char *argv[])

#else
#define	NMR_MAIN	int main(int argc,char *argv[])
#endif


#endif


