/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CDATALIST_H_
#define CDATALIST_H_

#include	<MegaHeaders.h>
#include	<CTCPSocket.h>
#include	<CError.h>

template <class T>
class CDataList
{

protected:
	int	mSize;
	T	*mData;

public:
	CERROR_SUBCLASS(RangeException);



// These first two are here just to shut up microsoft
private:
	CDataList(const CDataList<T>& t)
{
	assert(0);
}

CDataList& 
operator=(const CDataList<T> &)
{
	assert(0);
	return *this;
}

public:

CDataList(void)
{
	mSize = 0;
	mData = NULL;
}

CDataList(int siz)
{
	mSize = siz;
	mData = NULL;
	mData = new T[siz];
}

~CDataList(void)
{
	delete [] mData;
	mData = NULL;
}

int
GetSize(void)
{
	return mSize;
}

void
Resize(int siz)
{
	delete [] mData;
	mSize = siz;
	mData = new T[siz];
}

T&
operator[](int index)
{
	assert(index < mSize);
	if (index >= mSize)
		throw RangeException("CDataList::operator[] range error",index);
	return mData[index];
}

void
Dump(ostream &o,const char *indent)
{
	o << indent << "CDataList<T>" << ":" << endl;
	o << indent << "   mSize: " << mSize << endl;
	o << indent << "   mData:" << endl;
	for (int x = 0; x < mSize; ++x)
		o << indent << "   " << x << ")" << mData[x];
}


};

template <class T>
ostream & operator<<(ostream &o,CDataList<T> &cd)
{
	cd.Dump(o,"");
	return o;
}


template <class T>
ostream & operator<<(ostream &o,CDataList<T> *cd)
{
	if (cd)
		cd->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

template <class T>
size_t
CalculateTCPSize(CDataList<T> &l)
{
	size_t	siz;

	// backward compat kludge
	if (sizeof(T) == sizeof(U16))
		{
		siz = sizeof(u_short) + 
			(l.GetSize() * sizeof(u_short));
		}
	else if (sizeof(T) == sizeof(U32))
		{
		siz = sizeof(u_long) + 
			(l.GetSize() * sizeof(u_long));
		}
	else
		{
		siz = sizeof(u_long);
		int	lsiz = l.GetSize();
		for (int x = 0; x < lsiz; x++)
			siz += CalculateTCPSize(l[x]);
		}
	return siz;
}

template <class T>
CTCPSocket& operator<<(CTCPSocket &s,CDataList<T> &l)
{
	// backward compat kludge
	if (sizeof(T) == sizeof(I16))
		{
		U16 datasize = l.GetSize();
		s << datasize;

		if (datasize)
			{
			u_short	*ua = new u_short[datasize];
			stack_ArrayDeleter<u_short> sad(ua);

			for (int x = 0; x < datasize; ++x)
				ua[x] = htons(l[x]);

			s.Write((char *)ua,sizeof(u_short) * datasize);
			}
		}
	else if (sizeof(T) == sizeof(I32))
		{
		U32	datasize = l.GetSize();
		s << datasize;

		if (datasize)
			{
			u_long	*ua = new u_long[datasize];
			stack_ArrayDeleter<u_long> sad(ua);

			for (int x = 0; x < datasize; ++x)
				ua[x] = htonl(l[x]);

			s.Write((char *)ua,sizeof(u_long) * datasize);
			}
		}
	else
		{
		U32	datasize = l.GetSize();
		s << datasize;

		for (int x = 0; x < datasize; x++)
			s << l[x];
		}
	return s;
}

template <class T>
CTCPSocket& operator>>(CTCPSocket &s,CDataList<T> &l)
{
	// backward compat kludge
	if (sizeof(T) == sizeof(I16))
		{
		U16	datasize;
		s >> datasize;
		
		l.Resize(datasize);
		if (datasize)
			{
			u_short	*ua = new u_short[datasize];
			stack_ArrayDeleter<u_short> sad(ua);

			s.Read((char *)ua,sizeof(u_short) * datasize);
			for (int x = 0; x < datasize; ++x)
				l[x] = ntohs(ua[x]);
			}
		}
	else if (sizeof(T) == sizeof(I32))
		{
		U32	datasize;
		s >> datasize;

		l.Resize(datasize);
		if (datasize)
			{
			u_long	*ua = new u_long[datasize];
			stack_ArrayDeleter<u_long> sad(ua);

			s.Read((char *)ua,sizeof(u_long) * datasize);
			for (int x = 0; x < datasize; ++x)
				l[x] = ntohl(ua[x]);
			}
		}
	else
		{
		U32	datasize;
		s >> datasize;

		l.Resize(datasize);
		for (int x = 0; x < datasize; x++)
			s >> l[x];
		}
	return s;
}


#endif
