/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CEchoService.h>
#include	<CTCPSocket.h>
#include	<nmr_string.h>

/*****************************************************************************
CEchoService
	A demonstration CTcpService that provides a funky bracketed echo
******************************************************************************/

MD(CEchoService,	Init,		CMOS_SYS_INIT_OK);
MD(CEchoService,	INT_Error,	CMOS_SYS_INTERNAL_ERROR);
MD(CEchoService,	IO_Error,	CMOS_SYS_IO_ERROR);
MD(CEchoService,	Echo,		CMOS_SYS_FIRST_USER_EVENT);
MD(CEchoService,	Done,		CMOS_SYS_FIRST_USER_EVENT+1);
MD(CEchoService,	Open,		CMOS_SYS_FIRST_USER_EVENT+2);

/*------------------------- callbacks --------------------------------------*/

///////////////////////////////////////////////////////////////////////////////
// CB_Init
//
// Called when initialized
///////////////////////////////////////////////////////////////////////////////
void
CEchoService::CB_Init(CMosMessage *message)
{
	TraceCallback("CB_Init");

	mFileDescriptor = -1;
}

//////////////////////////////////////////////////////////////////////////////
// CB_Open
//
// Called when there is a connection
///////////////////////////////////////////////////////////////////////////////
void
#ifdef	WIN32
CEchoService::CB_Open(CMosMessage *mess)
#else
CEchoService::CB_Open(CMosMessageType<SOCKET> *message)
#endif
{
	TraceCallback("CB_Open");

#ifdef	WIN32
	CMosMessageType<SOCKET> *message = (CMosMessageType<SOCKET> *)mess;
#endif

	mDone = 0;
	mFileDescriptor = *(message->GetData());
	mSocket = new CTCPSocket(mFileDescriptor);
	SEND_MESS_Echo();
}

///////////////////////////////////////////////////////////////////////////////
// CB_Echo
//
// Begin echo and continue until an error occurs
///////////////////////////////////////////////////////////////////////////////
void
CEchoService::CB_Echo(CMosMessage *message)
{
	TraceCallback("CB_Echo");

	while (!mDone)
		{
		char cc[4] = "< >";
		try 
			{
			mSocket->Read(&(cc[1]),1);
			if (isprint(cc[1]))
				mSocket->Write(cc,3);
			}
		catch(...)
			{
			mDone = 1;
			}
		}

	SEND_MESS_Done();
}

//////////////////////////////////////////////////////////////////////////////
// CB_Close
//
// Close the socket and make me available for further connections
///////////////////////////////////////////////////////////////////////////////
void
CEchoService::CB_Close(CMosMessage *message)
{
	TraceCallback("CB_Close");

	delete mSocket;
	mSocket = NULL;
	mFileDescriptor = -1;
	MakeAvailable(); // make me available to the server again
}

#define	STATE_INITIAL	0
#define	STATE_READY		1
#define	STATE_OPEN		2


CFsm::state_table_entry_t CEchoService::sStateTable[] = 
{
{STATE_INITIAL,	MESS_Init,		STATE_READY,	FCT(CEchoService::CB_Init)},

{STATE_READY,	MESS_Open,		STATE_OPEN,		FCT(CEchoService::CB_Open)},

{STATE_OPEN,	MESS_Echo,		STATE_OPEN,		FCT(CEchoService::CB_Echo)},
{STATE_OPEN,	MESS_INT_Error,	STATE_READY,	FCT(CEchoService::CB_Close)},
{STATE_OPEN,	MESS_IO_Error,	STATE_READY,	FCT(CEchoService::CB_Close)},
{STATE_OPEN,	MESS_Done,		STATE_READY,	FCT(CEchoService::CB_Close)}
};

///////////////////////////////////////////////////////////////////////////////
// CEchoService
//
// Constructor
//
// INPUT:
//	queue_size	- size of message queue
//	server		- the CTcpServer managing me
///////////////////////////////////////////////////////////////////////////////
CEchoService::CEchoService(int queue_size,CTcpServer *server)
	: CFsmTcpService(server,queue_size)
{
	AttachStateTable(sStateTable,sizeof(sStateTable) / 
			sizeof(state_table_entry_t));
	mFileDescriptor = -1;
	mSocket = NULL;
	mDone = 0;
}

///////////////////////////////////////////////////////////////////////////////
// ~CEchoService
//
// Destructor. Close the socket if it's still open
///////////////////////////////////////////////////////////////////////////////
CEchoService::~CEchoService(void)
{
	delete mSocket;
	mFileDescriptor = -1;
}

void
CEchoService::Terminate(void)
{
	mDone = 1;
	CFsmTcpService::Terminate();

	if (mSocket)
		mSocket->BreakBlockingIO(this);
}

const char *
CEchoService::Name(void)
{
	return "CEchoService";
}

void
CEchoService::Dump(ostream &o,const char *indent)
{
	nmr_string	nmr(indent);
	nmr += "   ";

	o << indent << "CEchoService: " << endl;
	o << indent << "   mFileDescriptor: " << mFileDescriptor << endl;
	CTcpService::Dump(o,nmr);
}
