/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CError.h>
#include	<string.h>
///////////////////////////////////////////////////////////////////////////////
//
// CError
//
// Base of all exception classes. Contains a short string of data that can
// be used to hold an error message.
//
///////////////////////////////////////////////////////////////////////////////

CError::CError(void)
{
	mData[0] = '\0';
	mCode = 0;
}

CError::CError(const CError &err,int code)
{
	strcpy(mData,err.mData);
	mCode = code;
}

CError::CError(const char *s,int code)
{
	mCode = code;
	mData[0] = '\0';
	if (s)
		{
		strncpy(mData,s,MAX_CERROR_MESSAGE_SIZE);
		mData[MAX_CERROR_MESSAGE_SIZE] = '\0';
		}
}

CError::~CError(void)
{
}

CError&
CError::operator=(int code)
{
	mCode = code;
	return *this;
}

CError&
CError::operator=(const CError &err)
{
	strcpy(mData,err.mData);
	return *this;
}

CError&
CError::operator=(const char *s)
{
	if (s)
		{
		strncpy(mData,s,MAX_CERROR_MESSAGE_SIZE);
		mData[MAX_CERROR_MESSAGE_SIZE] = '\0';
		}
	return *this;
}

void
CError::Dump(ostream &o,const char *indent) const
{
	o << indent << "CError " << endl
		<< indent << "   mData: " << mData << endl
		<< indent << "   mCode: " << mCode << endl;
}

int
CError::code(void) const
{
	return mCode;
}

int
CError::length(void) const
{
	return strlen(mData);
}

const char *
CError::data(void) const
{
	return mData;
}


ostream& operator<<(ostream &o,const CError &err)
{
	err.Dump(o,"");
	return o;
}

ostream& operator<<(ostream &o,const CError *err)
{
	if (err)
		err->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}

