/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if 0
#include	<CFifo.h>
#include	<CError.h>
#include	<stack_aids.h>

///////////////////////////////////////////////////////
// CFifo
//
// Constructor. Build a fifo. The max number of entries
// is passed in.
//
// INPUT:
//	limit	- max number of fifo entries
////////////////////////////////////////////////////////

template<class T> 
CFifo<T>::CFifo(int limit)
{
	mData = new T[limit];
	mLimit=limit;
	mCurrent = 0;
	mGetIndex = mPutIndex = 0;
}


//////////////////////////////////////////////////////
// ~CFifo
//
// Destructor.
//////////////////////////////////////////////////////
template<class T> 
CFifo<T>::~CFifo(void)
{
	delete [] mData;
}

template<class T>
void
CFifo<T>::Clear(void)
{
	stack_MutexLock sml(&mMutex);

	delete [] mData;
	mData = new T[mLimit];
	mCurrent = 0;
	mGetIndex = mPutIndex = 0;
	mLessData.Signal();
}

//////////////////////////////////////////////////
// Put
//
// Puts an object pointer into the queue. Will
// block if the queue is full
//
// INPUT:
//	o - the object pointer to put into the queue
////////////////////////////////////////////////////
template<class T> 
void
CFifo<T>::Put(T o)
{
	stack_MutexLock sml(&mMutex);

	while (mCurrent == mLimit)
		mLessData.Wait(mMutex);
	mData[mPutIndex++] = o;
	if (mPutIndex == mLimit)
		mPutIndex = 0;
	mCurrent++;
	mMoreData.Signal();
}

///////////////////////////////////////////////////////////////////////////////
// PutIfSpaceElseBlock
//
// Block until free space in fifo drops below percent
//
// INPUT:
//	o 		- object to put in queue
//  percent	- percent threshold
//
// OUTPUT:
//	1 if a block occured else zero
///////////////////////////////////////////////////////////////////////////////
template<class T>
int
CFifo<T>::PutIfSpaceElseBlock(T o,int percent)
{
	stack_MutexLock sml(&mMutex);

	int	block = 0;
	while (((mCurrent*100)/mLimit) >= percent)
		{
		block = 1;
		mLessData.Wait(mMutex);
		}
	mData[mPutIndex++] = o;
	if (mPutIndex == mLimit)
		mPutIndex = 0;
	mCurrent++;
	mMoreData.Signal();
	return block;
}

template<class T>
int
CFifo<T>::PutIfSpaceElseFail(T o,int percent)
{
	stack_MutexLock sml(&mMutex);

	int	result = 0;
	if (((mCurrent*100)/mLimit) < percent)
		{
		mData[mPutIndex++] = o;
		if (mPutIndex == mLimit)
			mPutIndex = 0;
		mCurrent++;
		mMoreData.Signal();
		result = 1;
		}
	return result;
}

//////////////////////////////////////////////////
// PutCanFail
//
// Like Put() except it will not block if the queue
// is full. Instead, it will return 0.
//
// INPUT:
//	o - the object to add to the queue
//
// OUTPUT:
//	0 - queue was full. Operation failed
//	1 - operation succesful.
///////////////////////////////////////////////////////
template<class T>
int
CFifo<T>::PutCanFail(T o)
{
	stack_MutexLock sml(&mMutex);

	int	result = 0;
	if (mCurrent != mLimit)
		{
		mData[mPutIndex++] = o;
		if (mPutIndex == mLimit)
			mPutIndex = 0;
		mCurrent++;
		mMoreData.Signal();
		result = 1;
		}

	return result;
}

//////////////////////////////////////////////
// Get
//
// Retrieves the next item from the queue.
// Will block if the queue is empty
//
// OUTPUT:
//	Pointer to the next item in the queue
////////////////////////////////////////////////
template<class T>
T
CFifo<T>::Get(void)
{
	stack_MutexLock sml(&mMutex);

	T	result;
	while (mCurrent == 0)
		mMoreData.Wait(mMutex);
	result = mData[mGetIndex++];
	if (mGetIndex == mLimit)
		mGetIndex = 0;
	mCurrent--;
	mLessData.Signal();
	return result;
}

/////////////////////////////////////////////
// GetCanFail
//
// Like Get() except it will not block if the
// queue is empty
//
// OUTPUT:
//	0 in fail if the queue is empty otherwise the
//  next object in the queue.
///////////////////////////////////////////////
template<class T>
T
CFifo<T>::GetCanFail(int &fail)
{
	stack_MutexLock sml(&mMutex);

	T	result;
	fail = 0;

	if (mCurrent != 0)
		{
		result = mData[mGetIndex++];
		if (mGetIndex == mLimit)
			mGetIndex = 0;
		mCurrent--;
		mLessData.Signal();
		fail = 1;
		}

	return result;		
}

template <class T>
int
CFifo<T>::PercentFull(void) const
{
	return ((mCurrent*100)/mLimit);
}

template <class T>
int
CFifo<T>::NumObjectsInQueue(void) const
{
	return mCurrent;
}

template <class T>
void
CFifo<T>::Dump(ostream &o,const char *indent)
{
	stack_MutexLock sml(&mMutex);

	o << indent << "CFifo:" << endl;
	if (!mCurrent)
		{
		o << indent << "   <empty>" << endl;
		}
	else
		{
		int x = mCurrent;
		int i = mGetIndex;
		do
			{
			o << indent << "   " << x << ")" << mData[i++];
			if (i == mLimit) i = 0;
			}
		while (--x);
		}
}

template <class T>
ostream &operator<<(ostream &o,CFifo<T> &f)
{
	f.Dump(o,"");
	return o;
}

template <class T>
ostream &operator<<(ostream &o,CFifo<T> *f)
{
	if (f)
		f->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}


#endif