/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CIOUData.h>
#include	<stack_aids.h>

///////////////////////////////////////////////////////////////////////////////
// CIOUData
//
// Implements an IOU data container model.
//
// A thread can retrieve the data from an IOU by calling Retrieve. The
// retrieving thread now owns that data. Retrieve blocks until another
// thread fulfills the request by calling fulfill. The owner ship of
// the data belongs to
//
// before fulfill: fulfilling thread
// after fulfill, before redeem : the IOU
// after redeem : the redeeming thread
//
// The owner thread must properly dispose of the data.
// Redeem can be called multiple times (though the same data pointer will
// be returned on each call), but fulfill can only be called once.
///////////////////////////////////////////////////////////////////////////////

template <class T>
CIOUData<T>::CIOUData(void)
{
	mDataReady 		= 0;
	mDataAttached 	= 0;
	mData 			= NULL;
}

// These next are needed just to shut up microsoft
template <class T>
CIOUData<T>::CIOUData(const CIOUData<T> &data)
{
	assert(0);
}

template <class T>
CIOUData<T>& 
CIOUData<T>::operator=(const CIOUData<T> &data)
{
	assert(0);
	return *this;
}


template <class T>
CIOUData<T>::CIOUData(T *data)
{
	mDataReady		= 1;
	mDataAttached	= 1;
	mData			= data;
}

template <class T>
CIOUData<T>::~CIOUData(void)
{
	if (mDataAttached)
		delete mData;
}

template <class T>
T*
CIOUData<T>::Redeem(void)
{
	stack_MutexLock sml(&mMutex);

	while (!mDataReady)
		mWait.Wait(mMutex);

	mDataAttached = 0;
	return mData;
}

template <class T>
T*
CIOUData<T>::Redeem(int interval)
{
	stack_MutexLock sml(&mMutex);
	while (!mDataReady)
		mWait.WaitInterval(mMutex,interval);

	mDataAttached = 0;
	return mData;
}

template <class T>
void
CIOUData<T>::Fulfill(T* data)
{
	stack_MutexLock sml(&mMutex);

	assert(!mDataAttached);

	mData = data;
	mDataAttached = 1;
	mDataReady = 1;
	mWait.Signal();
}

