/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CLogger.h>
#include	<CMosMessage.h>
#include	<CMosMessageType.h>
#include	<safe_ostrstream.h>
#include	<sduplicate.h>
#include	<stack_aids.h>
#include	<time.h>
#include	<string.h>
#include	<nmr_string.h>

/******************************************************************************
CLogger
	class used for logging. Takes strings and writes them to files or
	system loggers
******************************************************************************/
static int s_code = CMOS_SYS_FIRST_USER_EVENT;

// in case logs are being written to the same file
CCMutex	CLogger::sLogMutex;

#define	MESSAGE_TYPE	CMosMessageType<nmr_string>

///////////////////////////////////////////////////////////////////////////////
// CLogger
//
// Constructor
//
// INPUT:
//	maxmessages	- size of message queue
//	log			- should we actually log, or just ignore it all
//	filedes		- file descriptor of file to write log to. If it is -1
//					then we use system daemon logger.
//
//	ident		- application id string. Used to destinguish this application
//					from others in a Unix log.
//
///////////////////////////////////////////////////////////////////////////////
CLogger::CLogger(int maxmessages,int log,int filedes,int syslog_priority,
		const char *identfr)
	: CMos(maxmessages)
{
	mLog 				= log;
	mFileDes 			= filedes;
	mSyslogPriority 	= syslog_priority;
	mIdent				= identfr;
}

///////////////////////////////////////////////////////////////////////////////
// ~CLogger
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CLogger::~CLogger(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// SendMessage
//
// Special SendMessage variant used to report errors. Appends date and
// time to the message.
//
// INPUT:
//	message - the message to log
///////////////////////////////////////////////////////////////////////////////
void
CLogger::SendMessage(const char *message)
{
	time_t	t;

	if (mLog)
		{
#ifdef	__POWERPC__
		stack_MutexLock sml(&mTimeLock);
#endif
		safe_ostrstream	o;

		time(&t);
#if (defined WIN32 || defined __POWERPC__)
		char *date = ctime(&t);
#elif ((defined SOLARIS) && (!defined __GNUC__))
		char	date[100]="";
		ctime_r(&t,date,100);
#else
		char	date[100]="";
		ctime_r(&t,date);
#endif
		o << "[" << mIdent << "] " << ((long)CCThread::Self());
		// system logs provide their own date-time
		if (mFileDes != -1)
			o << " " << date;
		o << " " << message << ends;
		CMos::SendMessageIfSpaceElseFail(new MESSAGE_TYPE(s_code,
				new nmr_string(o.str())),75);
		}
}

///////////////////////////////////////////////////////////////////////////////
// MesageHandler
//
// Overriden message handler. writes message to the log.
//
// INPUT:
//	message - the message
///////////////////////////////////////////////////////////////////////////////
void
CLogger::MessageHandler(CMosMessage *message)
{
	if (message->GetCode() == s_code)
		{
		MESSAGE_TYPE	*mms = (MESSAGE_TYPE *)message;
		nmr_string *d = mms->GetData();
		if (mLog)
			{
			stack_MutexLock sml(&sLogMutex);
#if ((!defined WIN32) && (!defined __POWERPC__))
			if (mFileDes == -1)
				{
#ifndef SOLARIS
				syslog(mSyslogPriority,(*d).data());
#endif
				}
			else
#endif
				{
#ifdef	__POWERPC__
				// huge kludge
				if (mFileDes == STDOUT_FILENO)
					{
					char	*dd = new char[d->length() + 1];
					strcpy(dd,*d);
					char*	c;
					int 	x;
					int 	len = d->length();
					for (x = 0,c = dd; x < len; ++x,++c)
						if ((*c) == '\n') *c = '\r';

					write(mFileDes,dd,len);
					write(mFileDes,"\r",1);
					delete [] dd;
					}
				else
					{
					IO_write(mFileDes,*d,d->length());
					IO_write(mFileDes,"\n",1);
					}
#else
				IO_write(mFileDes,*d,d->length());
				IO_write(mFileDes,"\n",1);
#endif
				}
			}
		}
}

const char *
CLogger::Name(void)
{
	return "CLogger";
}

void
CLogger::Dump(ostream &o,const char *indent)
{
	o 	<< indent << "CLogger: " << endl
		<< indent << "   mLog: " << mLog << endl
		<< indent << "   mFileDes: " << mFileDes << endl
		<< indent << "   mSyslogPriority: " << mSyslogPriority << endl
		<< indent << "   mIdent: " << mIdent << endl;
	nmr_string	 i2(indent);
	i2 += "   ";
	CMos::Dump(o,i2);
}



