/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CMessageReflectContainer.h>
#include	<CMos.h>
#include	<CMosMessageType.h>
#include	<nmr_string.h>

/***************************************************************************
CMessageReflectContainer

An object that is a container for a CMos message and a target (a CMos).
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// CMessageReflectContainer
//
// Construct the reflector object. You must indicate whether or not this
// object owns the data.
//
// INPUT:
//	message		- the message
//	destination	- the destination
//	attach_data	- if true then this object owns the message and destination
//					and will dispose of them when it is destroyed.
///////////////////////////////////////////////////////////////////////////////
CMessageReflectContainer::CMessageReflectContainer(CMosMessage *message,
										CMos *destination,int attach_data)
{
	mMessage 	= message;
	mDestination= destination;
	mAttached 	= attach_data;
}

///////////////////////////////////////////////////////////////////////////////
// ~CMessageReflectContainer
//
// Destructor. Will dispose of it's mMessage and mDestination fields if the
// data is attached.
///////////////////////////////////////////////////////////////////////////////
CMessageReflectContainer::~CMessageReflectContainer(void)
{
	if (mAttached)
		{
		delete mMessage;
		delete mDestination;
		}
}

///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// Set the data fields as detached. The object will not dispose of them when
// it is destroyed
///////////////////////////////////////////////////////////////////////////////
void
CMessageReflectContainer::DetachData(void)
{
	mAttached = 0;
}

///////////////////////////////////////////////////////////////////////////////
// AttachData
//
// Set the data fields as attached. The object will dispose of them when
// it is destroyed
///////////////////////////////////////////////////////////////////////////////
void
CMessageReflectContainer::AttachData(void)
{
	mAttached = 1;
}

///////////////////////////////////////////////////////////////////////////////
// GetMessage
//
// Return value of message field
///////////////////////////////////////////////////////////////////////////////
CMosMessage	*
CMessageReflectContainer::GetMessage(void)
{
	return mMessage;
}

///////////////////////////////////////////////////////////////////////////////
// GetDestination
//
// Return value of destination field
///////////////////////////////////////////////////////////////////////////////
CMos*
CMessageReflectContainer::GetDestination(void)
{
	return mDestination;
}


const char *
CMessageReflectContainer::Name(void) const
{
	return "CMessageReflectContainer";
}

void
CMessageReflectContainer::Dump(ostream &o,const char *indent)
{
	nmr_string	nmr(indent);
	nmr += "   ";

	o << indent << "CMessageReflectContainer: " << endl;
	o << indent << "   mAttached: " << mAttached << endl;
	o << indent << "   mMessage: ";
	if (mMessage)
		{
		o << endl;
		mMessage->Dump(o,nmr);
		}
	else
		{
		o << " NULL " << endl;
		}
	o << indent << "   mDestination: " << endl;
	if (mDestination)
		{
		o << endl;
		mDestination->Dump(o,nmr);
		}
	else
		{
		o << " NULL " << endl;
		}
}


ostream& operator<<(ostream &o,CMessageReflectContainer &m)
{
	m.Dump(o,"");
	return o;
}

ostream& operator<<(ostream &o,CMessageReflectContainer *m)
{
	if (m)
		m->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}
