/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CMilliTimer.h>
#include	<CMosMessage.h>
#include	<milli_sleep.h>
#include	<nmr_string.h>

#define TIME_IT	CMOS_SYS_FIRST_USER_EVENT

CMilliTimer::CMilliTimer(CMos *target,int mess_code,int time,int autotrigger)
			: CMos(10)
{
	mTarget 		= target;
	mMessageCode	= mess_code;
	mTime 			= time;
	mAutoTrigger	= autotrigger;
	mInterrupt		= 0;
}

CMilliTimer::~CMilliTimer(void)
{
}

void
CMilliTimer::Terminate(void)
{
	mInterrupt = 1;
	CMos::Terminate();

	// force a break out of any blocked IO
#if ((defined IRIX) || (defined SOLARIS))
	Kill(DUCKSIGNAL);
#endif
}

void
CMilliTimer::SetTime(int time)
{
	mTime = time;
}

int
CMilliTimer::GetTime(void)
{
	return mTime;
}

void
CMilliTimer::Start(void)
{
	CMos::Start();
	if (mAutoTrigger)
		BeginCountdown();
}

void
CMilliTimer::BeginCountdown(void)
{
	SendMessage(new CMosMessage(TIME_IT));
}

#define	MAX_SLEEP_CHUNK	100


void
CMilliTimer::MessageHandler(CMosMessage *mess)
{
	if (mess->GetCode() != TIME_IT)
		return;

#if ((defined IRIX) || (defined SOLARIS))
	milli_sleep(mTime);
#else
	int t = mTime;
	while (t > 0 && (!mInterrupt))
		{
		if (t <= MAX_SLEEP_CHUNK)
			{
			milli_sleep(t);
			t = 0;
			}
		else
			{
			milli_sleep(MAX_SLEEP_CHUNK);
			t -= MAX_SLEEP_CHUNK;
			}
		}
#endif

	if (!mInterrupt)
		{
		mTarget->SendMessageIfSpaceElseFail(new CMosMessage(mMessageCode),75);
		if (mAutoTrigger)
			SendMessage(new CMosMessage(TIME_IT));
		}
}

const char *
CMilliTimer::Name(void)
{
	return "CMilliTimer";
}

void
CMilliTimer::Dump(ostream &o,const char *indent)
{
	nmr_string	nmr(indent);
	nmr += "   ";

	o << indent << "CMilliTimer: " << endl;
	o << indent << "   mMessageCode: " << mMessageCode << endl;
	o << indent << "   mTime: " << mTime << endl;
	o << indent << "   mAutoTrigger: " << mAutoTrigger << endl;
	o << indent << "   mInterrupt: " << mInterrupt << endl;
	o << indent << "   mTarget: " << ((void *)mTarget) << endl;
	CMos::Dump(o,indent);
}
