/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CMosMessage.h>
#include	<CLogger.h>

/*****************************************************************************
CMosMessage
	The base class for messages sent by a CMos. The base only has a code
	representing the message type. To attach data, use the subclasses
	provided by the CMosMessageType template. Has a custom allocator
	for speed.
******************************************************************************/
#define BLOCKS_PER_FAILURE 100

#include	<static_allocator.h>

static_allocation(CMosMessage)

void *
CMosMessage::operator new(size_t t)
{
	return do_new_CMosMessage(t);
}

void
CMosMessage::operator delete(void *d)
{
	do_delete_CMosMessage(d);
}

///////////////////////////////////////////////////////////////////////////////
// CMosMessage
//
// Constructor
//
// INPUT:
//	code - the message code
///////////////////////////////////////////////////////////////////////////////
CMosMessage::CMosMessage(int code)
{
	mCode = code;
}

///////////////////////////////////////////////////////////////////////////////
// ~CMosMessage
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CMosMessage::~CMosMessage(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// GetCode
//
// Return the message code
//
// OUTPUT:
//	the message code
///////////////////////////////////////////////////////////////////////////////
int
CMosMessage::GetCode(void)
{
	return mCode;
}

///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// does nothing. Is needed by CMosMessageType subclasses with data
///////////////////////////////////////////////////////////////////////////////
void
CMosMessage::DetachData(void)
{
	// do nothing, used in subclasses
}

void
CMosMessage::Dump(ostream &o,const char *indent) const
{
	o << indent << "CMosMessage: " << endl
		<< indent << "   mCode: " << mCode << endl;
}

ostream& operator<<(ostream &o,CMosMessage &m)
{
	m.Dump(o,"");
	return o;
}

ostream& operator<<(ostream &o,CMosMessage *m)
{
	if (m)
		m->Dump(o,"");
	else
		o << " NULL " << endl;
	return o;
}




