/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if 0
#include	<CMosMessageType.h>
#include	<nmr_string.h>

/*****************************************************************************
CMosMessageType
	Template for CMosMessage subclasses with data. It is assumed that the
	data can be deleted when the message is destroyed. If not, you must
	call DetachData() before the message is destroyed
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// new/delete
//
// The base class has custom new delete operators which, unfortunately, get
// inherited. In order to avoid a crisis we must redefine to the global
// default.
//
///////////////////////////////////////////////////////////////////////////////
template <class T>
void *
CMosMessageType<T>::operator new(size_t t)
{
	return ::operator new(t);
}

template <class T>
void	
CMosMessageType<T>::operator delete(void *d)
{
	::operator delete(d);
}

///////////////////////////////////////////////////////////////////////////////
// CMosMessageType
//
// Constructor. 
//
// INPUT:
//	code - the message code
//	data - the data for the message
///////////////////////////////////////////////////////////////////////////////
template <class T>
CMosMessageType<T>::CMosMessageType(int code,T* data)
	: CMosMessage(code)
{
	mData = data;
	mDetached = 0;
}

///////////////////////////////////////////////////////////////////////////////
// ~CMosMessageType
//
// Destructor. Deletes the data associated with the message unless
// DetachData() has been called
///////////////////////////////////////////////////////////////////////////////
template <class T>
CMosMessageType<T>::~CMosMessageType(void)
{
	if (!mDetached) 
		delete mData;
}

///////////////////////////////////////////////////////////////////////////////
// GetData
//
// Return the data associated with the message
//
// OUTPUT:
//	the associated data
///////////////////////////////////////////////////////////////////////////////
template <class T>
T*
CMosMessageType<T>::GetData(void)
{
	return mData;
}


///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// Mark the data in the mos message as "undeleteable" so it is not deleted
// when the message is destroyed
///////////////////////////////////////////////////////////////////////////////
template <class T>
void
CMosMessageType<T>::DetachData(void)
{
	mDetached = 1;
}


template <class T>
void
CMosMessageType<T>::Dump(ostream &o,const char *indent) const
{
	o << indent << "CMosMessageType: " << endl
		<< indent << "   mDetached: " << mDetached << endl
		<< indent << "   mData: ";
	if (mData)
		o << (*mData) << endl;
	else
		o << " NULL " << endl;
	nmr_string i2(indent);
	i2 += "   ";
	CMosMessage::Dump(o,i2);
}

#endif