/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CMOSMESSAGETYPE_H_
#define CMOSMESSAGETYPE_H_

#include	<MegaHeaders.h>
#include	<CMosMessage.h>

template <class T> 
class CMosMessageType: public CMosMessage
{
private:

	// NOT ALLOWED
	// using the CLASS_NOCOPY macro doesnt work on mac
	CMosMessageType(const CMosMessageType<T> &m);
	CMosMessageType<T>& operator=(const CMosMessageType<T> &m);

protected:
	T	*mData;
	int	mDetached;

public:

	
///////////////////////////////////////////////////////////////////////////////
// new/delete
//
// The base class has custom new delete operators which, unfortunately, get
// inherited. In order to avoid a crisis we must redefine to the global
// default.
//
///////////////////////////////////////////////////////////////////////////////
static void *
operator new(size_t t)
{
	return ::operator new(t);
}

static void	
operator delete(void *d)
{
	::operator delete(d);
}

///////////////////////////////////////////////////////////////////////////////
// CMosMessageType
//
// Constructor. 
//
// INPUT:
//	code - the message code
//	data - the data for the message
///////////////////////////////////////////////////////////////////////////////
CMosMessageType(int code,T* data)
	: CMosMessage(code)
{
	mData = data;
	mDetached = 0;
}

///////////////////////////////////////////////////////////////////////////////
// ~CMosMessageType
//
// Destructor. Deletes the data associated with the message unless
// DetachData() has been called
///////////////////////////////////////////////////////////////////////////////
~CMosMessageType(void)
{
	if (!mDetached) 
		delete mData;
}

///////////////////////////////////////////////////////////////////////////////
// GetData
//
// Return the data associated with the message
//
// OUTPUT:
//	the associated data
///////////////////////////////////////////////////////////////////////////////
T*
GetData(void)
{
	return mData;
}


///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// Mark the data in the mos message as "undeleteable" so it is not deleted
// when the message is destroyed
///////////////////////////////////////////////////////////////////////////////
void
DetachData(void)
{
	mDetached = 1;
}


void
Dump(ostream &o,const char *indent) const
{
	o << indent << "CMosMessageType: " << endl
		<< indent << "   mDetached: " << mDetached << endl
		<< indent << "   mData: ";
	if (mData)
		o << (*mData) << endl;
	else
		o << " NULL " << endl;
	nmr_string i2(indent);
	i2 += "   ";
	CMosMessage::Dump(o,i2);
}
	
	
	
};


#endif
