/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CPACKET_H_
#define CPACKET_H_

#include	<MegaHeaders.h>
#include	<CCMutex.h>

class CPacket;
class CTCPSocket;

#define	PACKET_TYPE_SIMPLE 		0
#define PACKET_TYPE_I16			1
#define PACKET_TYPE_STRING		2
#define	PACKET_TYPE_CINETSPEC	3 // no longer used
#define	PACKET_TYPE_I16LIST		4
#define	PACKET_TYPE_I32LIST		5
#define PACKET_TYPE_I32			6

#define PACKET_TYPE_FIRST_USER	20

typedef CPacket *(*packet_init_func_t)
	(int code,int packet_type,int protocol_code);


class CPacket
{
	CLASS_NOCOPY(CPacket)

protected:

	static 	packet_init_func_t	*sPacketInitTable;
	static	int			sPacketTableInitialized;
	static	int			sPacketTableMax;
	static	CCMutex		sPacketTableMutex;

	int			mCode;
	int			mType;
	int			mProtocolCode;

	virtual	void	_Read(CTCPSocket &socket) = 0;
	virtual	void	_Write(CTCPSocket &socket) = 0;

public:

	static void	RegisterPacketType(int,packet_init_func_t);
	static int	InitializeSystemPackets(void);
	static void DestroySystemPackets(void);

	static CPacket*	Receive(CTCPSocket &socket);
			void	Send(CTCPSocket &socket);
	
	CPacket(int packet_code,int packet_type,int protocol_code);
	virtual	~CPacket(void);

	int		GetCode(void);
	int		GetType(void);
	int		GetProtocol(void);

	virtual	void	DetachData(void);
	virtual	size_t	CalculateSize(void);

	virtual const char *Name(void);
	virtual	void	Dump(ostream &o,const char *indent);

	CERROR_SUBCLASS(PacketTypeException);
};

ostream& operator<<(ostream &o,CPacket &p);
ostream& operator<<(ostream &o,CPacket *p);


#endif
