/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CPacketType.h>
#include	<CTCPSocket.h>

/******************************************************************************
CPacketType
	Template for CPacket subclasses with data. It is assumed that the
	associated data can be deleted along with the packet. If this is not
	the case, then call DetachData() before deleting the packet
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// _Creator
//
// static function that creates a packet of type CPacketType. Used when
// registering the packet with the packet subsystem
//
// INPUT:
//	code 			- the message code
//	packet_type 	- the packet type
//	protocol_code 	- the protocol sending this packet
///////////////////////////////////////////////////////////////////////////////
template <class T>
CPacket* 
CPacketType<T>::_Creator(int code,int packet_type,int protocol_code)
{
	return new CPacketType<T>(code,packet_type,protocol_code);
}

///////////////////////////////////////////////////////////////////////////////
// CPacketType
//
// Constructor
//
// INPUT:
//	code 			- the message code
//	packet_type 	- the packet type
//	protocol_code 	- the protocol sending this packet
///////////////////////////////////////////////////////////////////////////////
template <class T>
CPacketType<T>::CPacketType(int code,int packet_type,int protocol_code)
	: CPacket(code,packet_type,protocol_code)
{
	mDetached = 0;
	mData = 0;
}

///////////////////////////////////////////////////////////////////////////////
// CPacketType
//
// Constructor
//
// INPUT:
//	code 			- the message code
//	packet_type 	- the packet type
//	protocol_code 	- the protocol sending this packet
//	data			- the initial data
///////////////////////////////////////////////////////////////////////////////
template <class T>
CPacketType<T>::CPacketType(int code,int packet_type,int protocol_code,T* data)
	: CPacket(code,packet_type,protocol_code)
{
	mData = data;
	mDetached = 0;
}

///////////////////////////////////////////////////////////////////////////////
// ~CPacketSimple
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
template <class T>
CPacketType<T>::~CPacketType(void)
{
	if (!mDetached) 
		delete mData;
}

//////////////////////////////////////////////////////////////////////////////
// SetData
//
// Set the packet's data
//
// INPUT:
//	t - the data to associate with the packet
///////////////////////////////////////////////////////////////////////////////
template <class T>
void
CPacketType<T>::SetData(T* t)
{
	mData = t;
}

///////////////////////////////////////////////////////////////////////////////
// GetData
//
// gets the data associated with the packet
//
// OUTPUT:
//	the associated data
///////////////////////////////////////////////////////////////////////////////
template <class T>
T*
CPacketType<T>::GetData(void)
{
	return mData;
}

///////////////////////////////////////////////////////////////////////////////
// DetachData
//
// detach the packet's data so it is not deleted with the packet is destroyed
///////////////////////////////////////////////////////////////////////////////
template <class T>
void
CPacketType<T>::DetachData(void)
{
	mDetached = 1;
}

template <class T>
void
CPacketType<T>::Dump(ostream &o,const char *indent)
{
	o << indent << Name() << ": " << endl;
	o << indent << "   mDetached: " << mDetached << endl;
	o << indent << "   mData: " << endl;
	o << indent << "   -------------" << endl;
	if (mData)
		o << (*mData) << endl;
	else
		o << " NULL " << endl;
	o << indent << "   -------------" << endl;
}

template <class T>
void
CPacketType<T>::_Read(CTCPSocket &socket)
{
	delete mData;
	mData = new T;
	socket >> (*mData);
}

template <class T>
void
CPacketType<T>::_Write(CTCPSocket &socket)
{
	socket << (*mData);
}

template <class T>
size_t
CPacketType<T>::CalculateSize(void)
{
	return CPacket::CalculateSize() + CalculateTCPSize(*mData);
}

///////////////////////////////////////////////////////////////////////////////
// const char *Name(void)
//
///////////////////////////////////////////////////////////////////////////////
