/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CPing.h>
#include	<CCThread.h>
#include	<milli_sleep.h>

/***************************************************************************
CPing

A class that uses a ping to deduce lag and bandwidth from a client.
You create a CPing class and call it's Ping method to gather the 
statistics.

Unfortunately many platforms do not support the raw socket structures
needed to do a ping. They exists in the operating system, but no
header files are provided. The is very much the case under windows and
less so under linux. Hence, the huge blocks of platform specific
crap.
*****************************************************************************/

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/

#if (defined __POWERPC__)
// can't do ping yet

CPing::CPing(void)
{
}

CPing::~CPing(void)
{
}

int	
CPing::Ping(unsigned long addr,int id1,int id2,
				unsigned int &lag, unsigned int &fourktime)
{
	lag = 1000;
	fourktime = 4;
	return 1;
}

#elif (defined	WIN32)

// Windows is totally different
extern "C" {
#include	<ipexport.h>
#include	<icmpapi.h>
};

CPing::CPing(void)
{
}

CPing::~CPing(void)
{
}

int	
CPing::Ping(unsigned long addr,int id1,int id2,
				unsigned int &lag, unsigned int &fourktime)
{
	int		result = 0;
	HANDLE	icmp_handle = NULL;
	int		err;

	try 
		{
		addr = htonl(addr);

		icmp_handle = IcmpCreateFile();
		if (icmp_handle == NULL)
			throw 0;

		char out_data[1024];
		char reply_buffer[2048]; // big just to be safe

		// calculate data rate
		//////////////////////////

		if (!IcmpSendEcho(icmp_handle,addr,out_data,1024,NULL,
			reply_buffer,2048,2000)) throw 0;
		// ignore the first one for DNS lookup

		unsigned long	t;
		t = timeGetTime();
		if (!IcmpSendEcho(icmp_handle,addr,out_data,1024,NULL,
			reply_buffer,2048,2000)) throw 0;
		if (!IcmpSendEcho(icmp_handle,addr,out_data,1024,NULL,
			reply_buffer,2048,2000)) throw 0;
		t = timeGetTime() - t;
		fourktime = t;

		// calculate lag
		///////////////////////////
		t = timeGetTime();
		if (!IcmpSendEcho(icmp_handle,addr,out_data,0,NULL,
			reply_buffer,2048,2000)) throw 0;
		if (!IcmpSendEcho(icmp_handle,addr,out_data,0,NULL,
			reply_buffer,2048,2000)) throw 0;
		t = timeGetTime() - t;
		lag = t / 2;
		result = 1;
		}
	catch(...)
		{
		err = GetLastError(); // so we can debug it
		}

	if (icmp_handle)
		IcmpCloseHandle(icmp_handle);

	return result;
}


#else

#if (defined LINUX)

// Linux has wierd names for the structure fields. We ifdef them
// so they are the same as other Unix'es

#define	icmp	icmphdr
#define	icmp_type	type
#define	icmp_code	code
#define	icmp_cksum	checksum
#define icmp_id		un.echo.id
#define	icmp_seq	un.echo.sequence
#define	ICMP_MINLEN	2	// dunno
#define ip			iphdr
#define	ip_hl		ihl


#endif

// I wish someone would explain why I can NEVER get a prototype
// for bzero

#ifndef LINUX
extern "C" {
void bzero(void *s,size_t n);
};
#endif

#define	SIZE_ICMP_HDR	8	// size of icmp header

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/

#define	MAX_PING_DATA 4096

///////////////////////////////////////////////////////////////////////////////
// CPing
//
// Constructor
///////////////////////////////////////////////////////////////////////////////
CPing::CPing(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// ~CPing
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CPing::~CPing(void)
{
}

/////////////////////////////////////////////////////////////////////////////
// Checksum
//
// Calculate checksum for IP family headers. From Unix Network Programming
//	(Stevens) page 454
//
// INPUT:
//	ptr	- the data to checksum
//	nbytes - the number of bytes to checksum
//
// OUTPUT:
//	the checksum
/////////////////////////////////////////////////////////////////////////////
int	
CPing::Checksum(u_short *ptr,int nbytes)
{
	long	sum;
	u_short	oddbyte;
	u_short	answer;

/*
 * Our algorithm is simple, using a 32-bit accumulator (sum),
 * we add sequential 16-bit words to it, and at the end, fold back
 * all the carry bits from the top 16 bits into the lower 16 bits.
 */

	sum	 = 0;
	while (nbytes > 1)
		{
		sum 	+= *ptr++;
		nbytes 	-= 2;
		}

	// mop up an odd byte if necessary

	if (nbytes == 1)
		{
		oddbyte = 0;	// make sure top half is zero
		*((u_char *)&oddbyte) = *(u_char *)ptr; // one byte only
		sum += oddbyte;
		}

/*
 * Add back carry outs from top 16 bits to low 16 bits
 */
	sum = (sum >> 16) + (sum & 0xffff);	// add high-16 to low-16
	sum += (sum >> 16);					// add carry
	answer = ~sum;	// ones-complement, then truncate to 16 bits

	return answer;
}

//////////////////////////////////////////////////////////////////////////////
// SendPacket
//
// Send ping packet with a specific amount of bogus data. Remember that raw
// sockets see EVERYTHING that comes by. We use the process id and thread
// id to distinguish packets for this thread from others.
//
// INPUT:
//	sockfd		- open socket descriptor
//	dest		- pointer to packet destination
//	id1			- process id identifier
//	id2			- thread identifier
//	amount_of_data	- how much bogus data to send
//
// OUTPUT:
//	1 if ok else 0
///////////////////////////////////////////////////////////////////////////////
int
CPing::SendPacket(int sockfd,struct sockaddr_in *dest,int id1,int id2,
					int amount_of_data)
{
	struct icmp			*icp;
	u_char				sendpacket[MAX_PING_DATA];	// output packet
	int					packetsize;
	int					err;

	if (amount_of_data > MAX_PING_DATA)
		return 0;

	icp	= (struct icmp *)sendpacket;// pointer to ICMP header 
	icp->icmp_type 	= ICMP_ECHO;
	icp->icmp_code 	= 0;
	icp->icmp_cksum	= 0;			// zero then call in_cksum()
	icp->icmp_id 	= id1;			// our pid, to identify on return
	icp->icmp_seq 	= 0;			// sequence number

	// copy time and thread id into packet
	gettimeofday(&mErrorTime,NULL);
	memcpy((void *)&sendpacket[SIZE_ICMP_HDR],&mErrorTime,
													sizeof(mErrorTime));
//	gettimeofday((struct timeval *)&sendpacket[SIZE_ICMP_HDR],NULL);
	memcpy((void *)(&sendpacket[SIZE_ICMP_HDR]+ sizeof(struct timeval)),
			&id2,sizeof(id2));
	
	// compute checksum
	packetsize = SIZE_ICMP_HDR + sizeof(struct timeval) + 
												sizeof(id2) + amount_of_data;
	icp->icmp_cksum = Checksum((ushort *)icp,packetsize);
		
	// send packet
	do 
		{
		err = sendto(sockfd,(const char *)sendpacket,packetsize,0,
									(struct sockaddr *)dest,sizeof(*dest));
		if (err == -1)
			sleep(1);
		}
	while (err == -1 && errno == EWOULDBLOCK);

	return (err < 0 || err != packetsize ? 0 : 1);
}

///////////////////////////////////////////////////////////////////////////////
// ReceivePacket
//
// Waits to receive the echoed packet from the SendPacket method.
//
// INPUT:
//	sockfd		- the socket descriptor
//	id1			- the process id
//	id2			- the thread id
//	amount		- amount of data we're expecting
//
// OUTPUT:
//	t	- the time that was loaded into the packet. This is the time
//			set in the SendPacket method.
//	return 1 if everything went ok, else zero
//
///////////////////////////////////////////////////////////////////////////////
int
CPing::ReceivePacket(int sockfd,int id1,int id2,int amount,struct timeval &t)
{
	int 				err;
	u_char				recpacket[MAX_PING_DATA];
	int					retries = 20;
	struct	ip			*ip;
	struct icmp			*icp;
	int					result = 0;
	int					ip_header_len;
	int					eno;
	int					id22;
//	int					local_retry;
	struct timeval		local_time;
try {
foo:
	if (retries-- <= 0) 
		throw 0;

//	local_retry = 3000;
	gettimeofday(&local_time,NULL);
	do 
		{
		struct sockaddr_in	from;
		int 			fromlen = sizeof(from);
		err = recvfrom(sockfd,(char *)recpacket,sizeof(recpacket),0,
				(struct sockaddr *)&from, &fromlen);
		if (err == -1)
			{
			struct timeval lt;
			gettimeofday(&lt,NULL);
			if (lt.tv_sec - local_time.tv_sec > 4) break;
			eno = errno;
			CCThread::Yield();//milli_sleep(1);
			}
		}
//	while ((--local_retry) && retries && (err == -1 && eno == EWOULDBLOCK));
	while (retries && (err == -1 && eno == EWOULDBLOCK));

	if (err < 0)
		{
		t = mErrorTime;
		return 1;
		}

	ip = (struct ip *)recpacket;
	ip_header_len = ip->ip_hl << 2;	// convert 32-bit words to bytes

	// block was somehow truncated. Try again
	if (err < ip_header_len + ICMP_MINLEN)
		goto foo;

	err -= ip_header_len;
	icp = (struct icmp *)(recpacket + ip_header_len);

	// packet was not an echo reply. Try again
	if (icp->icmp_type != ICMP_ECHOREPLY)
		goto foo;

	// packet does not belong to me, try again
	if (icp->icmp_id != id1)
		goto foo;

	memcpy((void *)&id22,(void *) 
	((&((char *)icp)[SIZE_ICMP_HDR]) + sizeof(struct timeval)),sizeof(id2));
	
	// packet does not belong to me, try again
	if (id22 != id2) 
		goto foo;

	// copy the time data out of the packet
	memcpy((void *)&t,(void *)&((char *)icp)[SIZE_ICMP_HDR],
													sizeof(struct timeval));
	result = 1;
	}
catch(...)
	{
	}
	return result;

}

///////////////////////////////////////////////////////////////////////////////
// Ping
//
// This is where we actually do the ping. We send three of everything
// but only use the last two. This is because the first ping might cause
// a DNS lookup which will take extra time and cause the ping to be
// unusually long.
//
// INPUT:
//	address - internet address in native byte format
//	id1		- packet identifier (getpid())
//	id2		- a thread identifier if needed
//
// OUTPUT:
// 	lag - lag in milliseconds
//	fourktime - time in milliseconds to transfer 4k of data
//	Returns 1 if everything ran ok, else 0
///////////////////////////////////////////////////////////////////////////////
int
CPing::Ping(unsigned long address,int id1,int id2,
				unsigned int &lag,unsigned int &fourktime)
{
	int result = 0;
	int	sockfd = -1;

try
	{
	int					x;
	struct sockaddr_in	dest;
	struct protoent		*proto;
	struct timeval		t,tp;

	///////////////////////////////////////////////////
	// setup socket
	/////////////////////////////////////////////////////
	lag = fourktime = 0;
	bzero((char *)&dest,sizeof(dest));

	dest.sin_family 	= AF_INET;
	dest.sin_addr.s_addr= htonl(address);
//#ifdef	LINUX
	// linux is screwed up (pthreads) do it manually
//	sockfd				= socket(AF_INET, SOCK_RAW, 1);
//#else
	proto				= getprotobyname("icmp");
	if (proto == NULL) 
			throw 0;
	sockfd				= socket(AF_INET, SOCK_RAW, proto->p_proto);
//#endif
	if (sockfd < 0)
			throw 0;

	{
	// non-blocking
	int flags;
	flags = fcntl(sockfd,F_GETFL);
	if (flags == -1) 
		throw 0;
	if (fcntl(sockfd,F_SETFL,flags | O_NONBLOCK) < 0)
		throw 0;
	}

	//////////////////////////////////////////////////////
	// send three pings of minimal data to calculate lag
	/////////////////////////////////////////////////////
	for (x = 0; x < 3; ++x)
		{
		if (!SendPacket(sockfd,&dest,id1,id2,0))
			throw 0;

		if (!ReceivePacket(sockfd,id1,id2,0,tp))
			throw 0;

		if (x != 0)	// ignore the first because of DNS lookup
			{
			gettimeofday(&t,NULL);
			lag += (t.tv_sec - tp.tv_sec) * 1000;
			lag += (t.tv_usec - tp.tv_usec) / 1000;
			}
		}
	lag /= 2;
	/////////////////////////////////////////////////////
	// send three pings of 1024 to calculate datarate
	// this is 4k because 1k + 1k = 2k, then double it for
	// the echo.
	/////////////////////////////////////////////////////
	for (x = 0; x < 3; ++x)
		{
		if (!SendPacket(sockfd,&dest,id1,id2,1024))
			throw 0;

		if (!ReceivePacket(sockfd,id1,id2,1024,tp))
			throw 0;

		if (x != 0)
			{
			int	delay;
			gettimeofday(&t,NULL);
			fourktime += (t.tv_sec - tp.tv_sec) * 1000;
			fourktime += (t.tv_usec - tp.tv_usec) / 1000;
			}
		}
	result = 1;
	}
catch(...)
	{
	}
	/////////////////////////////////////////////////////
	// cleanup
	/////////////////////////////////////////////////////

	if (sockfd >= 0)
		close(sockfd);

	return result;
}

#endif

