/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CProtocol.h>
#include	<CMosMessage.h>
#include	<CPortMonitor.h>
#include	<CPortTransmitter.h>
#include	<nmr_string.h>

/***********************************************************************
 CProtocol
	A CProtocol is an object that uses CPortMonitor and CPortTransmtiter
	it also has a number that identifies the protocol. This number is
	sent in transmitted packets to allow the recieving side to 
	dispatch the packets appropriately
**********************************************************************/

///////////////////////////////////////////////////////////////////////////////
// CProtocol
//
// Constructor
//
// INPUT:
//	pcode - the protcol number
///////////////////////////////////////////////////////////////////////////////
CProtocol::CProtocol(int pcode)
{
	mProtocolCode = pcode;
	mPortMonitor = NULL;
	mPortTransmitter = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// ~CProtocol
//
// Destructor
///////////////////////////////////////////////////////////////////////////////
CProtocol::~CProtocol(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// GetProtocolCode
//
// Returns the protocol code
//
// OUTPUT:
//	the code
///////////////////////////////////////////////////////////////////////////////
int
CProtocol::GetProtocolCode(void)
{
	return mProtocolCode;
}

///////////////////////////////////////////////////////////////////////////////
// SetPortMonitor
//
// Set the CPortmonitor for this protocol object. The object will register
// itself with the port monitor
//
// INPUT:
//	port_monitor	- the port monitor
///////////////////////////////////////////////////////////////////////////////
void
CProtocol::SetPortMonitor(CPortMonitor *port_monitor)
{
	mPortMonitor = port_monitor;
	mPortMonitor->RegisterProtocol(this);
}

///////////////////////////////////////////////////////////////////////////////
// SetPortTransmitter
//
// Set the port transmitter for this protocol object.
//
// INPUT:
//	port_transmitter - the port transmitter
///////////////////////////////////////////////////////////////////////////////
void
CProtocol::SetPortTransmitter(CPortTransmitter *port_transmitter)
{
	mPortTransmitter = port_transmitter;
}

///////////////////////////////////////////////////////////////////////////////
// MessageMe
//
// the port transmitter and monitor use this method to send feedback to
// the protocol (ie error messages). CMosProtocol and CFsmProtocol override
// this to automatically feed this to their SendMessage() methods.
//
// INPUT:
//	mess - the message
///////////////////////////////////////////////////////////////////////////////
void
CProtocol::MessageMe(CMosMessage *mess)
{
}

const char *
CProtocol::Name(void)
{
	return "CProtocol";
}

void
CProtocol::Dump(ostream &o,const char *indent)
{
	nmr_string nmr(indent);
	nmr += "   ";

	o << indent << "CProtocol: " << endl;
	o << indent << "   mProtocolCode: " << mProtocolCode << endl;
	o << indent << "   mPortMonitor: ";
	if (mPortMonitor) 
		{
		o << endl;
		mPortMonitor->Dump(o,nmr);
		}
	else
		{
		o << "NULL" << endl;
		}
	o << indent << "   mPortTransmitter: ";
	if (mPortTransmitter) 
		{
		o << endl;
		mPortTransmitter->Dump(o,nmr);
		}
	else
		{
		o << "NULL" << endl;
		}
}
