/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CSOCKET_H_
#define CSOCKET_H_

#include	<MegaHeaders.h>
#include	<CCMutex.h>
#include	<CError.h>

#if	((!defined WIN32) && (!defined __POWERPC__))
#define	SOCKET	int
#endif

#if (defined __POWERPC__)
#define	SOCKET_close		th_socket_close
#define	SOCKET_socket		th_socket_socket
#define	SOCKET_listen		th_socket_listen
#define	SOCKET_accept		th_socket_accept
#define	SOCKET_connect		th_socket_connect
#define	SOCKET_getpeername	th_socket_getpeername
#define	SOCKET_send			th_socket_send
#define	SOCKET_recv			th_socket_recv
#define	SOCKET_bind			th_socket_bind
#define	SOCKET_setsockopt	th_socket_setsockopt
#define	SOCKET_getsockopt	th_socket_getsockopt
#define	SOCKET_gethostbyname	th_socket_gethostbyname
#define	SOCKET_getsockname	th_socket_getsockname
#define	SOCKET_recvfrom		th_socket_recvfrom
#define	SOCKET_sendto		th_socket_sendto
#define	SOCKET_inet_addr	th_socket_inet_addr
#define	FETCH_SOCKET_ERROR	th_socket_errno()

#elif (defined WIN32)
#define	SOCKET_close		closesocket
#define	SOCKET_socket		socket
#define	SOCKET_listen		listen
#define	SOCKET_accept		accept
#define	SOCKET_connect		connect
#define	SOCKET_getpeername	getpeername
#define	SOCKET_send			send
#define	SOCKET_recv			recv
#define	SOCKET_bind			bind
#define	SOCKET_setsockopt	setsockopt
#define	SOCKET_getsockopt	getsockopt
#define	SOCKET_gethostbyname	gethostbyname
#define	SOCKET_getsockname	getsockname
#define	SOCKET_recvfrom		recvfrom
#define	SOCKET_sendto		sendto
#define	SOCKET_inet_addr	inet_addr
#define	FETCH_SOCKET_ERROR	WSAGetLastError()

#else
#define	SOCKET_close		close
#define	SOCKET_socket		socket
#define	SOCKET_listen		listen
#define	SOCKET_accept		accept
#define	SOCKET_connect		connect
#define	SOCKET_getpeername	getpeername
#define	SOCKET_send			send
#define	SOCKET_recv			recv
#define	SOCKET_bind			bind
#define	SOCKET_setsockopt	setsockopt
#define	SOCKET_getsockopt	getsockopt
#define	SOCKET_gethostbyname	gethostbyname
#define	SOCKET_getsockname	getsockname
#define	SOCKET_recvfrom		recvfrom
#define	SOCKET_sendto		sendto
#define	SOCKET_inet_addr	inet_addr
#define	FETCH_SOCKET_ERROR	errno

#endif

class CCThread;

class CSocket
{
	CLASS_NOCOPY(CSocket)

protected:
	static CCMutex	sGetHostByNameMutex;

	int		mSocketClosed;
	SOCKET	mSocketDescriptor;
	int		mLastBoundPort;

							CSocket(void);
							CSocket(SOCKET socket_descriptor);

public:
	static const unsigned long	AnyAddress;

	virtual					~CSocket(void);
	virtual	void			Close(void);
	virtual	void			BreakBlockingIO(CCThread *t);
	virtual	void			Bind(unsigned long addr,int port);

			unsigned long	GetHostByName(const char *name);
			void			GetSocketInfo(unsigned long &addr,int &port);
	virtual	void			SetSockOpt(int level,int optname,
												const void *optval,int optlen);
	virtual	void			GetSockOpt(int level,int optname,void *optval,
																int *optlen);

	virtual	I16				GetLastBoundPort(void);
			void			SetOption_ReuseAddr(bool b);
			void			SetOption_ReceiveBuffer(int siz);
			void			SetOption_SendBuffer(int siz);

	virtual	const char *	Name(void);
	virtual	void			Dump(ostream &o,const char *indent);

	CERROR_SUBCLASS(BindException);
	CERROR_SUBCLASS(CloseException);
	CERROR_SUBCLASS(SetSockOptException);
	CERROR_SUBCLASS(GetSockOptException);
	CERROR_SUBCLASS(SocketInfoException);
};

#define	PRINTNETADDRESS(addr) \
		   (int)((addr & 0xFF000000) >> 24) << "." \
		<< (int)((addr & 0x00FF0000) >> 16) << "." \
		<< (int)((addr & 0x0000FF00) >> 8) << "." \
		<< (int)((addr & 0x000000FF) >> 0) << "." 

#endif
