/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CTCPSocket.h>
#include	<nmr_string.h>
#include	<stack_aids.h>

CTCPSocket::CTCPSocket(void)
	: CSocket()
{
	int one = 1;
	mSocketClosed = 1;
	mSocketDescriptor = SOCKET_socket(AF_INET,SOCK_STREAM,0);
#ifdef	WIN32
	if (mSocketDescriptor == INVALID_SOCKET)
#else
	if (mSocketDescriptor < 0)
#endif
		throw CtorException("CTCPSocket::CTCPSocket",FETCH_SOCKET_ERROR);
	mSocketClosed = 0;
	SetOption_ReuseAddr(true);
}

CTCPSocket::CTCPSocket(SOCKET socket_descriptor)
	: CSocket(socket_descriptor)
{
}

CTCPSocket::~CTCPSocket(void)
{
}

///////////////////////////////////////////////////////////////////////////////
// Listen
//
// listen on the socket.
//
// INPUT:
//	backlog - max unserved connections allowed on socket
///////////////////////////////////////////////////////////////////////////////
void	
CTCPSocket::Listen(int backlog)
{

	int result = SOCKET_listen(mSocketDescriptor,backlog);

#ifdef	WIN32
	if (result)
#else
	if (result < 0)
#endif
		throw ListenException("CSocket::Listen",FETCH_SOCKET_ERROR);

}

///////////////////////////////////////////////////////////////////////////////
// Accept
//
// Block until a connection is available then return the file descriptor
// for the new connection
//
// OUTPUT:
//	the file descriptor for the new connection
///////////////////////////////////////////////////////////////////////////////
SOCKET
CTCPSocket::Accept(void)
{
	struct sockaddr_in	cli_addr;
	int					clilen;
	SOCKET				fd;

	clilen = sizeof(cli_addr);
	fd = SOCKET_accept(mSocketDescriptor,(struct sockaddr *)&cli_addr,&clilen);

#ifdef	WIN32
	if (fd == INVALID_SOCKET)
#else
	if (fd < 0)
#endif
		throw AcceptException("CSocket::Accept",FETCH_SOCKET_ERROR);
	return fd;
}

///////////////////////////////////////////////////////////////////////////////
// Connect
//
// Client side tcp connect
//
// INPUT:
//	host_addr 	- the address of the host to connect to in 4 byte binary form
//	port		- the port to connect to
///////////////////////////////////////////////////////////////////////////////
void
CTCPSocket::Connect(unsigned long host_addr,int port)
{
	struct sockaddr_in serv_addr;
	memset(&serv_addr,0,sizeof(serv_addr));
	serv_addr.sin_family		= AF_INET;
	serv_addr.sin_addr.s_addr	= htonl(host_addr);
	serv_addr.sin_port			= htons(port);
	int result =SOCKET_connect(mSocketDescriptor,(struct sockaddr *)&serv_addr,
													sizeof(serv_addr));
#ifdef	WIN32
	if (result)
#else
	if (result < 0)
#endif
		throw ConnectException("CSocket::Connect",FETCH_SOCKET_ERROR);
}

///////////////////////////////////////////////////////////////////////////////
// Connect
//
// client side connect
//
// INPUT:
//	host_addr	- host address as text. CAN ONLY BE NUMERIC, no names.
//	port		- the port to connect to
///////////////////////////////////////////////////////////////////////////////
void
CTCPSocket::Connect(const char *host_addr,int port)
{
	struct sockaddr_in serv_addr;
	memset(&serv_addr,0,sizeof(serv_addr));
	serv_addr.sin_family		= AF_INET;
	serv_addr.sin_addr.s_addr	= SOCKET_inet_addr(host_addr);
	serv_addr.sin_port			= htons(port);

	int result =SOCKET_connect(mSocketDescriptor,(struct sockaddr *)&serv_addr,
														sizeof(serv_addr));
#ifdef	WIN32
	if (result)
#else
	if (result < 0)
#endif
		throw ConnectException("CSocket::Connect",FETCH_SOCKET_ERROR);
}

unsigned long
CTCPSocket::GetPeerAddress(void)
{
	struct sockaddr_in 	d;
	int	len = sizeof(d);
	int result = SOCKET_getpeername(mSocketDescriptor,(sockaddr *)&d,&len);

#ifdef	WIN32
	if (result)
#else
	if (result < 0)
#endif
		throw GetPeerException("CSocket::GetPeerAddress",FETCH_SOCKET_ERROR);

	return ntohl(d.sin_addr.s_addr);
}

void
CTCPSocket::Write(const char *buf,int count)
{
	while (count)
		{
		int result = SOCKET_send(mSocketDescriptor,buf,count,0);
		if (result < 0)
			throw WriteException("CTCPSocket::Write",result);
		buf += result;
		count -= result;
		}
}

void
CTCPSocket::Read(char *buf,int count)
{
	while (count)
		{
		int result = SOCKET_recv(mSocketDescriptor,buf,count,0);
		// under WIN32 - a <= 0 result is an error
		// This is less clear under Unix, but the IRIX client
		// will not close unless zero is marked as an error
		if (result <= 0)
			throw ReadException("CTCPSocket::Read",result);
		buf += result;
		count -= result;
		}
}

void			
CTCPSocket::SetOption_Linger(bool onoff,int duration)
{
	struct linger l;
	l.l_onoff = (onoff ? 1 : 0);
	l.l_linger = duration;
	SetSockOpt(SOL_SOCKET,SO_LINGER,&l,sizeof(l));
}

void			
CTCPSocket::SetOption_NoDelay(bool b)
{
	int     nodelay = (b ? 1 : 0);
	SetSockOpt(IPPROTO_TCP,TCP_NODELAY,(char *)&nodelay,sizeof(nodelay));
}

const char *
CTCPSocket::Name(void)
{
	return "CTCPSocket";
}

void
CTCPSocket::Dump(ostream &o,const char *indent)
{
	o << indent << "CTCPSocket:" << endl;

	nmr_string	nmr(indent);
	nmr += "   ";
	CSocket::Dump(o,nmr);
}

///////////////////////////

CTCPSocket &operator <<(CTCPSocket &s,I32 l)
{
	u_long	u = htonl(l);
	s.Write((char *)&u,sizeof(u));
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,U32 l)
{
	u_long	u = htonl(l);
	s.Write((char *)&u,sizeof(u));
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,I16 ss)
{
	u_short u = htons(ss);
	s.Write((char *)&u,sizeof(u));
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,U16 ss)
{
	u_short u = htons(ss);
	s.Write((char *)&u,sizeof(u));
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,I8 c)
{
	s.Write((char *)&c,1);
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,U8 c)
{
	s.Write((char *)&c,1);
	return s;
}

CTCPSocket &operator <<(CTCPSocket &s,nmr_string c)
{
	U16		len = c.length();
	s << len;
	s.Write(c.data(),len);
	return s;
}


//////////////////////////
CTCPSocket &operator >>(CTCPSocket &s,I32 &l)
{
	u_long	u;
	s.Read((char *)&u,sizeof(u));
	l = ntohl(u);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,U32 &l)
{
	u_long	u;
	s.Read((char *)&u,sizeof(u));
	l = ntohl(u);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,I16 &ss)
{
	u_short	 u;
	s.Read((char *)&u,sizeof(u));
	ss = ntohs(u);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,U16 &ss)
{
	u_short	 u;
	s.Read((char *)&u,sizeof(u));
	ss = ntohs(u);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,I8 &c)
{
	s.Read((char *)&c,1);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,U8 &c)
{
	s.Read((char *)&c,1);
	return s;
}

CTCPSocket &operator >>(CTCPSocket &s,nmr_string &c)
{
	U16 len;
	s >> len;
	c = "";
	if (len)
		{
		char *buf = new char[len + 1];
		stack_ArrayDeleter<char> sad(buf);

		s.Read(buf,len);
		buf[len] = '\0';
		c = buf;
		}
	return s;
}

size_t	CalculateTCPSize(I8 &i)
{
	return 1;
}

size_t	CalculateTCPSize(U8 &i)
{
	return 1;
}

size_t	CalculateTCPSize(I16 &i)
{
	return sizeof(u_short);
}

size_t	CalculateTCPSize(U16 &i)
{
	return sizeof(u_short);
}

size_t	CalculateTCPSize(I32 &i)
{
	return sizeof(u_long);
}

size_t	CalculateTCPSize(U32 &i)
{
	return sizeof(u_long);
}

size_t	CalculateTCPSize(nmr_string &i)
{
	return sizeof(u_short) + i.length();
}

