/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	CTCPSERVER_H_
#define CTCPSERVER_H_

#include	<CFsm.h>
#include	<CCMutex.h>
#include	<CTCPSocket.h>

class CTcpService;

class CTcpServer: public CFsm
{
	CLASS_NOCOPY(CTcpServer)

protected:
	int		mIgnoreAcceptError;
	int		mAcceptingConnections;
	CCMutex	mAcceptingMutex;

	typedef	CTcpService	*service_table_t;

	// active services
	service_table_t			*mServices;
	int						mServiceTableSize;
	int						mServiceTableMaxSize;
	CCMutex					mServiceMutex;

	// all services (used for shutdown)
	service_table_t			*mAllServices;
	int						mAllServicesTableSize;
	int						mAllServicesTableMaxSize;

	CTCPSocket				*mSocket;
	int						mPort;
	int						mPortBacklog;

	void	CB_Init(CMosMessage *mess);
	void	CB_Error(CMosMessage *mess);
	void	CB_Open(CMosMessage *mess);

	static state_table_entry_t	sTable[];

	virtual	CTcpService	*CreateTcpService(int x) = 0;

	virtual	void	BuildTcpServices(void);
	virtual	void	HandleConnection(SOCKET fd);
	virtual	void	SetSocketOptions(CTCPSocket *s) = 0;
	virtual	void	RegisterForShutdownTcpService(CTcpService *service);

public:
	CTcpServer(int queue_size,int max_services,int port,int port_backlog);
	virtual ~CTcpServer(void);

	virtual	void			AddTcpService(CTcpService *service);
	virtual	CTcpService*	RemoveTcpService(void);
	virtual	void	Terminate(void);
	virtual	const char *	Name(void);
	virtual	void			Dump(ostream &o,const char *indent);

	MEP(Open);
	MEP(Init);
	MEP(IO_Error);
	MEP(INT_Error);
};

#endif


