/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<CUDPSocket.h>
#include	<nmr_string.h>


CUDPSocket::CUDPSocket(void)
{
	mSocketClosed = 1;
	_ClearPeers();
	mSocketDescriptor = SOCKET_socket(AF_INET,SOCK_DGRAM,0);
#ifdef	WIN32
	if (mSocketDescriptor == INVALID_SOCKET)
#else
	if (mSocketDescriptor < 0)
#endif
		throw CtorException("CUDPSocket::CUDPSocket",FETCH_SOCKET_ERROR);

	mSocketClosed = 0;
	SetOption_ReuseAddr(true);
}


CUDPSocket::CUDPSocket(SOCKET socket_descriptor)
	: CSocket(socket_descriptor)
{
	_ClearPeers();
}

void
CUDPSocket::_ClearPeers(void)
{
	memset(&mSendPeer,0,sizeof(mSendPeer));
	memset(&mReceivePeer,0,sizeof(mReceivePeer));
}

CUDPSocket::~CUDPSocket(void)
{
}

void
CUDPSocket::_SetPeer(struct sockaddr_in &s,unsigned long addr,int port)
{
	s.sin_family = AF_INET;
	s.sin_addr.s_addr = htonl(addr);
	s.sin_port = htons(port);
}

void			
CUDPSocket::SetSendPeer(unsigned long addr,int port)
{
	_SetPeer(mSendPeer,addr,port);
}

void			
CUDPSocket::SetReceivePeer(unsigned long addr,int port)
{
	_SetPeer(mReceivePeer,addr,port);
}

void			
CUDPSocket::GetSendPeer(unsigned long &addr,int &port)
{
	addr = ntohl(mSendPeer.sin_addr.s_addr);
	port = ntohs(mSendPeer.sin_port);
}

void			
CUDPSocket::GetReceivePeer(unsigned long &addr,int &port)
{
	addr = ntohl(mReceivePeer.sin_addr.s_addr);
	port = ntohs(mReceivePeer.sin_port);
}

int			
CUDPSocket::RecvFrom(char *data,int len)
{
	int plen = sizeof(mReceivePeer);
	int	result = SOCKET_recvfrom(mSocketDescriptor,data,len,0,
				(struct sockaddr*)&mReceivePeer,&plen);
#ifdef	WIN32
	if (result == SOCKET_ERROR)
#else
	if (result < 0)
#endif
		throw RecvException("CUDPSocket::RecvFrom",result);
	return result;
}

void			
CUDPSocket::SendTo(const char *data,int len)
{
	int	result = SOCKET_sendto(mSocketDescriptor,data,len,0,
					(struct sockaddr*)&mSendPeer,sizeof(mSendPeer));
#ifdef	WIN32
	if (result == SOCKET_ERROR)
#else
	if (result < 0)
#endif
		throw SendException("CUDPSocket::SendTo",result);
}



const char *	
CUDPSocket::Name(void)
{
	return "CUDPSocket";
}

void			
CUDPSocket::Dump(ostream &o,const char *indent)
{
	o << hex << indent << "CUDPSocket: " << endl
		<< indent << "   mSendPeer: " << endl
		<< indent << "      addr: " << mSendPeer.sin_addr.s_addr << endl
		<< indent << "      port: " << mSendPeer.sin_port << endl
		<< indent << "   mReceivePeer: " << endl
		<< indent << "      addr: " << mReceivePeer.sin_addr.s_addr << endl
		<< indent << "      port: " << mReceivePeer.sin_port << endl;
	o << dec;
	nmr_string	nmr(indent);
	nmr += "   ";
	CSocket::Dump(o,nmr);
}


