/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef	__POWERPC__
#include	<Mac_IO.h>
#include	<CCThread.h>
#include	<CCMutex.h>
#include	<map.h>
#include	<stack_aids.h>

typedef CCThread_ID_Type errno_key_t;

typedef map<errno_key_t,int, less<errno_key_t>, allocator<int> > errno_map_t;
static errno_map_t	gsErrnoMap;
static CCMutex	gsErrnoMutex;

int th_errno(void)
{
	stack_MutexLock	sml(&gsErrnoMutex);

	CCThread_ID_Type id;
	::GetCurrentThread(&id);
	return gsErrnoMap[id];
}

int th_io_creat(const char *filename)
{
	HParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);
	Str255				fname;
	int					len;

	len = strlen(filename);
	if (len > 255) len = 255;
	fname[0] = len;
	strncpy((char *)&(fname[1]),filename,len);
	fname[255] = '\0';

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioNamePtr = fname;
	pb.ioParam.ioVRefNum = 0;
	pb.fileParam.ioDirID = 0;

	(void)::PBHCreateAsync(&pb);
	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;
	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return (err == noErr ? 0 : -1);
}

int th_io_open(const char *filename,int perm)
{
	return th_io_open(filename,perm,0);
}

int th_io_open(const char *filename,int perm,int mode)
{
	HParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);
	Str255				fname;
	int					len;

	if (perm & O_CREAT)
		{
		int err = th_io_creat(filename);
		gsErrnoMutex.Lock();
		int err2 = gsErrnoMap[th];
		gsErrnoMutex.Unlock();

		if (err == -1 && err2 != dupFNErr)
			return -1;
		}

	if (perm & O_RDWR)
		perm = fsRdWrPerm;
	else if (perm & O_WRONLY)
		perm = fsWrPerm;
	else if (perm & O_RDONLY)
		perm = fsRdPerm;

	len = strlen(filename);
	if (len > 255) len = 255;
	fname[0] = len;
	strncpy((char *)&(fname[1]),filename,len);
	fname[255] = '\0';

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioNamePtr = fname;
	pb.ioParam.ioVRefNum = 0;
	pb.ioParam.ioPermssn = perm;
	pb.fileParam.ioDirID = 0;

	(void)::PBHOpenDFAsync(&pb);

	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;

	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return (err == noErr ? pb.ioParam.ioRefNum : -1);
}

int th_io_close(int fd)
{
	ParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioRefNum = fd;

	(void)::PBCloseAsync(&pb);

	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;
	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return (err == noErr ? 0 : -1);
}

int th_io_read(int fd,char *buf,int siz)
{
	ParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioRefNum = fd;
	pb.ioParam.ioBuffer = (Ptr)buf;
	pb.ioParam.ioReqCount = siz;
	pb.ioParam.ioPosMode = fsAtMark;
	pb.ioParam.ioPosOffset = 0;
	pb.ioParam.ioActCount = 0;

	(void)::PBReadAsync(&pb);

	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;
	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return ((err == noErr || err == eofErr) 
				? pb.ioParam.ioActCount : -1);
}

int th_io_write(int fd,const char *buf,int siz)
{
	ParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioRefNum = fd;
	pb.ioParam.ioBuffer = (Ptr)buf;
	pb.ioParam.ioReqCount = siz;
	pb.ioParam.ioPosMode = fsAtMark;
	pb.ioParam.ioPosOffset = 0;
	pb.ioParam.ioActCount = 0;

	(void)::PBWriteAsync(&pb);

	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;
	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return (err == noErr ? pb.ioParam.ioActCount : -1);
}


int th_io_seek(int fd,long amt,int orig)
{
	ParamBlockRec		pb;
	CCThread_ID_Type	th;
	::GetCurrentThread(&th);

	// convert from unix
	switch(orig)
		{
		case SEEK_SET:
			orig = fsFromStart;
			break;
		case SEEK_CUR:
			orig = fsFromMark;
			break;
		case SEEK_END:
			orig = fsFromLEOF;
			break;
		}

	pb.ioParam.ioCompletion = NULL;
	pb.ioParam.ioRefNum = fd;
	pb.ioParam.ioPosMode = orig;
	pb.ioParam.ioPosOffset = amt;

	(void)::PBSetFPosAsync(&pb);

	while (pb.ioParam.ioResult >= 1) CCThread::Yield();

	int err = pb.ioParam.ioResult;

	gsErrnoMutex.Lock();
	gsErrnoMap[th] = err;
	gsErrnoMutex.Unlock();
	return (err == noErr ? pb.ioParam.ioPosOffset : -1);
}



#endif









