/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef __POWERPC__
#include	<MegaHeaders.h>
#include	<CCMutex.h>
#include	<Mac_Sockets.h>
#include	<map.h>
#include	<milli_sleep.h>
#include	<CCThread.h>
#include	<stack_aids.h>

typedef CCThread_ID_Type  errno_key_t;

typedef map<errno_key_t,int, less<errno_key_t>, allocator<int> > errno_map_t;
static errno_map_t	gsSocketErrnoMap;
static CCMutex	gsSocketErrnoMutex;

#define	MACTCP_MIN_REC_BUFFER	(4 * 1024)
#define	MACTCP_MAX_REC_BUFFER	(128 * 1024)

////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void SetError(int e);
void SetError(int e)
{
	stack_MutexLock	sml(&gsSocketErrnoMutex);

	CCThread_ID_Type	th;
	::GetCurrentThread(&th);
	gsSocketErrnoMap[th] = e;
}

class ForceTcpStartup
{

public:
	int		mTcpIsUp;
	int		err;

	ForceTcpStartup(void)
		{
		mTcpIsUp = 0;
		if ((err = InitOpenTransport()) != kOTNoError)
			{
			cout << "Could not initialize TCP Layer. Error = " 
					<< err << endl;
			exit(-1);
			}
		else
			{
			mTcpIsUp = 1;
			}
		};

	~ForceTcpStartup(void)
		{
		if (mTcpIsUp)
			CloseOpenTransport();
		};
};

static ForceTcpStartup	gFTS;

//////////////////////////////////////////////////
/////////////////////////////////////////////////////////

class Private_Socket
{
public:
	int				mValidObject;
protected:
	int				mOccupied;
	int				mLockout;
	int				mAsyncInProgress;
	OTEventCode		mEventCode;
	OTResult		mResult;
	EndpointRef		mEndpoint;
	
	int				mLinger;
	int 			mTCPNoDelay;
	int				mTCP;	// if 1 then tcp else udp
	int 			mOpened;	// have we actually opened it
	InetHost		mPeerAddress;
	InetPort		mPeerPort;

	// TCP stuff
	InetHost 		mBindAddress;
	InetPort		mBindPort;

	// UDP stuff

private:
	Private_Socket(Private_Socket &ps)
		{
		assert(0);
		};


public:
	Private_Socket(int tcp)
		{
		mLockout		= 0;
		mOccupied		= 0;
		mValidObject	= 97531;
		mLinger 		= 0;
		mTCPNoDelay 	= 0;
		mTCP 			= tcp;
		mOpened 		= 0;
		mPeerAddress	= 0;
		mPeerPort		= 0;
		mBindAddress	= 0;
		mBindPort		= 0;
		};

	~Private_Socket(void)
		{ 
		mValidObject = 0;
		};


	void			AsyncPrepare(void) { if (mLockout) mAsyncInProgress = 0; else mAsyncInProgress = 1; mResult = kOTNoError;}
	OTResult		AsyncWait(void) 
						{ 
						while (mAsyncInProgress && !mLockout) 
							CCThread::Yield(); 
						if (mLockout) return -1;
						return mResult;
						}
	void			SetAsyncDone(void) { mAsyncInProgress = 0; }
	void			SetEndpoint(EndpointRef e) { mEndpoint = e; }
	void			SetResult(OTResult r) { mResult = r; }
	void			SetEventCode(OTEventCode e) { mEventCode = e; }
	EndpointRef		GetEndpoint(void) { return mEndpoint; }
	OTResult		GetResult(void) { return mResult; }
	OTEventCode		GetEventCode(void) { return mEventCode; } 
	void			SetLockout(void) { mLockout = 1; }
	int				GetLockout(void) { return mLockout; }
	void			SetOccupied(int o) { mOccupied = o; }
	int				GetOccupied(void) { return mOccupied; }
	
	int				IsOpen(void) { return mOpened; };
	int 			IsTCP(void) { return mTCP; };
	int				GetLinger(void) { return mLinger; };
	unsigned long 	GetBindAddress(void) { return mBindAddress; };
	int				GetBindPort(void) { return mBindPort; };	
	int				GetTCPNoDelay(void) { return mTCPNoDelay; };
	unsigned long	GetPeerAddress(void) { return mPeerAddress; }
	int				GetPeerPort(void) { return mPeerPort; }

	void	SetPeerAddress(unsigned long addr) { mPeerAddress = addr; }
	void	SetPeerPort(int port) { mPeerPort = port; }
	void	SetLinger(int duration) 
				{ assert(duration >= 0); mLinger = duration; };
	void	SetOpen(int o) { mOpened = o; };
	void	SetTCPNoDelay(int d) { mTCPNoDelay = d; };

	void SetBindAddress(unsigned long addr) { mBindAddress = addr; };
	void SetBindPort(int port) { mBindPort = port; };

};

#define	TCP_CHECK(x)	if (!(x)->IsTCP()) return -1
#define	UDP_CHECK(x)	if ((x)->IsTCP()) return -1
#define	CLOSED_CHECK(x)	if ((x)->IsOpen()) return -1
#define OPEN_CHECK(x)	if (!(x)->IsOpen()) return -1

pascal void NotifierCallback(void *contextPtr,OTEventCode code,
	OTResult result,void *cookie);
pascal void NotifierCallback(void *contextPtr,OTEventCode code,
	OTResult result,void *cookie)
{
	Private_Socket *ps = (Private_Socket *)contextPtr;
	assert(ps->mValidObject == 97531);
	ps->SetEventCode(code);
	ps->SetResult(result);
	ps->SetAsyncDone();
	switch(code)
		{
		case T_OPENCOMPLETE:
			ps->SetEndpoint((EndpointRef)cookie);
			break;
		case T_BINDCOMPLETE:
			// cookie has bound TBind address, but dont need it
			break;
		}
}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

SOCKET th_socket_socket(int domain,int type,int protocol)
{
	SOCKET			result = -1;
	Private_Socket	*ps;
	OSStatus		err;
	//EndpointRef		ep;

	ps = new Private_Socket(type == SOCK_STREAM);
	ps->SetOccupied(1);
	ps->AsyncPrepare();
	err = OTAsyncOpenEndpoint(
		OTCreateConfiguration(type == SOCK_STREAM ? kTCPName : kUDPName),0,NULL, 
		NotifierCallback,(void *)ps);
	SetError(err);
	if (err != kOTNoError)
		{
		delete ps;
		}
	else
		{
		err = ps->AsyncWait();
		SetError(err);
		if (err != kOTNoError)
			delete ps;
		else
			result = (SOCKET)ps;
		}

	ps->SetOccupied(0);
	return result;
}

int th_socket_bind(SOCKET s,struct sockaddr *sa,int len)
{
	int result = -1;
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	TBind	bind_request,bind_result;
	InetAddress request_address,result_address;
	
	OTInitInetAddress(&request_address,(InetPort)sa-> sin_port,(InetHost)sa-> sin_addr.s_addr);
	memset(&bind_request,0,sizeof(bind_request));
	memset(&bind_result,0,sizeof(bind_result));
	bind_request.addr.len = sizeof(struct InetAddress);
	bind_request.addr.buf = (unsigned char *)&request_address;
	bind_request.qlen = 5;
	bind_result.addr.maxlen = sizeof(struct InetAddress);
	bind_result.addr.buf = (unsigned char *)&result_address;
	
	ps->AsyncPrepare();
	OSStatus err = OTBind(ps->GetEndpoint(),&bind_request,&bind_result);
	SetError(err);
	if (err == kOTNoError)
		{
		err = ps->AsyncWait();
		SetError(err);
		if (err == kOTNoError)
			result = 0;
		}
	ps->SetBindAddress(result_address.fHost);
	ps->SetBindPort(result_address.fPort);

	ps->SetOccupied(0);
	return result;
}

int	th_socket_close(SOCKET &s)
{
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return 0;
	
	ps->SetLockout();
	while (ps->GetOccupied()) CCThread::Yield();
	ps->SetOccupied(1);
	
	OTSetSynchronous(ps->GetEndpoint());
	OSStatus err = OTSndOrderlyDisconnect(ps->GetEndpoint());
	err = OTRcvOrderlyDisconnect(ps->GetEndpoint());
	err = OTUnbind(ps->GetEndpoint());
	OTRemoveNotifier(ps->GetEndpoint());
	err = OTCloseProvider(ps->GetEndpoint());

	delete ps;
	s = NULL;

	return 0;
}


int th_socket_listen(SOCKET s,int backlog)
{
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);

	TCP_CHECK(ps);	// must be tcp

	ps->SetOccupied(0);
	return 0;
}

SOCKET th_socket_accept(SOCKET s,struct sockaddr *,int *len)
{
	SOCKET result = (SOCKET)-1;
	
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	TCP_CHECK(ps); // must be tcp

	TCall 		remote;
	InetAddress	remote_address;
	
	while (1)
		{
		memset(&remote_address,0,sizeof(remote_address));
		memset(&remote,0,sizeof(remote));
		remote.addr.maxlen = sizeof(struct InetAddress);
		remote.addr.len = sizeof(struct InetAddress);
		remote.addr.buf = (unsigned char *)&remote_address;

		OSStatus err = OTListen(ps->GetEndpoint(),&remote);
		SetError(err);
		if (ps->GetLockout())
			break;
		if (err != kOTNoError && err != kOTNoDataErr)
			break;
		if (err == kOTNoError)
			{
			Private_Socket *newps = (Private_Socket *)th_socket_socket(0,SOCK_STREAM,0);
			if (newps == NULL || newps == ((Private_Socket *)(-1)))
				break;
			if (ps->GetLockout())
				{
				th_socket_close((SOCKET)newps);
				break;
				}
			ps->AsyncPrepare();
			newps->AsyncPrepare();
			err = OTAccept(ps->GetEndpoint(),newps->GetEndpoint(),&remote);
			if (err != kOTNoError || ps->GetLockout() || newps->GetLockout())
				{
				SetError(err);
				th_socket_close((SOCKET)newps);
				break;
				}
			OSStatus err1 = ps->AsyncWait();
			OSStatus err2 = newps->AsyncWait();
			if (err1 != kOTNoError || err2 != kOTNoError || ps->GetLockout() || newps->GetLockout())
				{
				SetError(err1 == kOTNoError ? err2 : err1);
				th_socket_close((SOCKET)newps);
				break;
				}
			newps->SetPeerAddress(remote_address.fHost);
			newps->SetPeerPort(remote_address.fPort);
			result = (SOCKET)newps;
			break;
			}
		CCThread::Yield();
		}
			
	ps->SetOccupied(0);
	return result;
}

int th_socket_connect(SOCKET s,struct sockaddr *addr,int len)
{
	int result = -1;
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);

	TCP_CHECK(ps); // must be tcp

	ps->AsyncPrepare();
	OSStatus err = OTBind(ps->GetEndpoint(),nil,nil);
	SetError(err);
	if (err != kOTNoError) 
		{
		ps->SetOccupied(0);
		return -1;
		}
	err = ps->AsyncWait();
	SetError(err);
	if (err != kOTNoError) 
		{
		ps->SetOccupied(0);
		return -1;
		}
		
	TCall in_call,out_call;
	InetAddress in_addr,out_addr;
	
	memset(&in_call,0,sizeof(in_call));
	memset(&out_call,0,sizeof(out_call));
	memset(&in_addr,0,sizeof(in_addr));
	memset(&out_addr,0,sizeof(out_addr));
	
	in_call.addr.len = sizeof(in_addr);
	in_call.addr.buf = (unsigned char *)&in_addr;
	out_call.addr.maxlen = sizeof(out_addr);
	out_call.addr.buf = (unsigned char *)&out_addr;
	OTInitInetAddress(&in_addr,(InetPort)addr-> sin_port,(InetHost)addr-> sin_addr.s_addr);

	ps->AsyncPrepare();
	err = OTConnect(ps->GetEndpoint(),&in_call,&out_call);
	SetError(err);
	if (err == kOTNoError || err == kOTNoDataErr)
		{
		err =ps->AsyncWait();
		SetError(err);
		if (err == kOTNoError || err == kOTNoDataErr)
			{
			ps->AsyncPrepare();
			memset(&out_addr,0,sizeof(out_addr));	
			memset(&out_call,0,sizeof(out_call));
			out_call.addr.maxlen = sizeof(out_addr);
			out_call.addr.buf = (unsigned char *)&out_addr;
			err = OTRcvConnect(ps->GetEndpoint(),&out_call);
			SetError(err);
			if (err == kOTNoError)
				{
				result = 0;
				}
			else if (err == kOTNoDataErr)
				{
				err = ps->AsyncWait();
				SetError(err);
				result = 0;
				}
			}
		}
		
	ps->SetPeerAddress(out_addr.fHost);
	ps->SetPeerPort(out_addr.fPort);

	ps->SetOccupied(0);
	return result;
}

int th_socket_setsockopt(SOCKET s,int level,int name,char *val,int len)
{
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	int	result = 0;
	struct linger	*linger_ptr;
	struct ip_mreq	*mreq;
	int	intval;

	switch(name)
		{
		case SO_RCVBUF:
			{
			int					buffsize = *(int *)val;
			UInt8				optBuffer[kOTFourByteOptionSize];
			TOption*			opt = (TOption*)optBuffer;
			opt->len = kOTFourByteOptionSize;
			opt->level = XTI_GENERIC;
			opt->name = XTI_RCVBUF;
			opt->status = 0;
			opt->value[0] = buffsize;
			
			TOptMgmt			optReq;
			optReq.flags = T_NEGOTIATE;
			optReq.opt.len = sizeof(optBuffer);
			optReq.opt.buf = optBuffer;

			ps->AsyncPrepare();
			OSStatus err = OTOptionManagement(ps->GetEndpoint(),&optReq,&optReq);
			SetError(err);
			if (err == kOTNoError)
				{
				err = ps->AsyncWait();
				SetError(err);
				}
			if (err != kOTNoError)
				result = -1;
			}
			break;
		case SO_SNDBUF:
			{
			int					buffsize = *(int *)val;
			UInt8				optBuffer[kOTFourByteOptionSize];
			TOption*			opt = (TOption*)optBuffer;
			opt->len = kOTFourByteOptionSize;
			opt->level = XTI_GENERIC;
			opt->name = XTI_SNDBUF;
			opt->status = 0;
			opt->value[0] = buffsize;
			
			TOptMgmt			optReq;
			optReq.flags = T_NEGOTIATE;
			optReq.opt.len = sizeof(optBuffer);
			optReq.opt.buf = optBuffer;

			ps->AsyncPrepare();
			OSStatus err = OTOptionManagement(ps->GetEndpoint(),&optReq,&optReq);
			SetError(err);
			if (err == kOTNoError)
				{
				err = ps->AsyncWait();
				SetError(err);
				}
			if (err != kOTNoError)
				result = -1;
			}
			break;
		case SO_REUSEADDR:
			break;
		case SO_LINGER:
//			linger_ptr = (struct linger *)val;
//			ps->SetLinger(linger_ptr->l_onoff ? linger_ptr->l_linger : 0);
			break;
		case TCP_NODELAY:
//			ps -> SetTCPNoDelay(*((int *)val));
			break;
		case IP_ADD_MEMBERSHIP:
			{
			struct ip_mreq *mreq	= (struct ip_mreq *)val;
			TOptMgmt			optReq;
			UInt8				optBuffer[kOTOptionHeaderSize + sizeof(TIPAddMulticast)];
			TOption*			opt = (TOption*)optBuffer;
			TIPAddMulticast*	addopt = (TIPAddMulticast*)opt->value;
			
			optReq.flags = T_NEGOTIATE;
			optReq.opt.len = sizeof(optBuffer);
			optReq.opt.buf = optBuffer;
			
			opt->level = INET_IP;
			opt->name = IP_ADD_MEMBERSHIP;
			opt->len = sizeof(optBuffer);
			
			addopt->multicastGroupAddress = mreq->imr_multiaddr.s_addr;
			addopt->interfaceAddress = kOTAnyInetAddress;
			
			ps->AsyncPrepare();
			OSStatus err = OTOptionManagement(ps->GetEndpoint(),&optReq,&optReq);
			SetError(err);
			if (err == kOTNoError)
				{
				err = ps->AsyncWait();
				SetError(err);
				}
			if (err != kOTNoError)
				result = -1;
			}
			// no documented support for this on Mac
			break;
		default:
//			result = -1;
			break;
		}

	ps->SetOccupied(0);
	return result;
}

int th_socket_getsockopt(SOCKET s,int level,int name,char *val,int *len)
{
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;

	int	result = 0;
	struct linger *linger_ptr;

	switch(name)
		{
		case SO_RCVBUF:
//			*((int *)val) = ps -> GetRecBufferSize();
			break;
		case SO_SNDBUF:
//			*((int *)val) = ps -> GetSndBufferSize();
			break;
		case SO_REUSEADDR:
			break;
		case SO_LINGER:
//			linger_ptr = (struct linger *)val;

//			linger_ptr->l_linger = ps -> GetTCPNoDelay();
//			linger_ptr->l_onoff = linger_ptr->l_linger ? 1 : 0;			
			break;
		case TCP_NODELAY:
//			*((int *)val) = ps -> GetTCPNoDelay();
			break;
		default:
//			result = -1;
			break;
		}

	return result;
}

int th_socket_send(SOCKET s,const char *buf,int len,int flags)
{
	int result = -1;
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	TCP_CHECK(ps); // must be tcp

	OTFlags f = 0;
	int	amt_sent = len;
	
	while (amt_sent)
		{
		if (ps->GetLockout())
			{
			result = -1;
			break;
			}
		OTResult err = OTSnd(ps->GetEndpoint(),(char *)buf,amt_sent,f);
		SetError(err);
		if (err == amt_sent) 
			{
			result = len;
			break;
			}
		if (err < 0) 
			break;
		buf += err;
		amt_sent -= err;
		CCThread::Yield();
		}
	ps->SetOccupied(0);
	return result;
}

int th_socket_recv(SOCKET s,char *buf,int len,int flags)
{
	int result = -1;
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	TCP_CHECK(ps);
	while (1)
		{
		OTFlags f = 0;
		if (ps->GetLockout())
			{
			result = -1;
			break;
			}
		OTResult err = OTRcv(ps->GetEndpoint(),buf,len,&f);
		SetError(err);
		if (err != kOTNoDataErr)
			{
			result = err;
			if (err > 0) break;
			result = -1;
			break;
			}
		CCThread::Yield();
		}
	ps->SetOccupied(0);
	return result;
}

int th_socket_sendto(SOCKET s,const char *buf,int len,int flags,
						struct sockaddr *addr,int slen)
{
	int result = -1;
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);
	
	UDP_CHECK(ps);

	TUnitData	ud;
	InetAddress	tmpsin;
	OTInitInetAddress(&tmpsin,addr->sin_port,addr->sin_addr.s_addr);
	memset(&ud,0,sizeof(ud));
	ud.addr.len = sizeof(struct InetAddress);
	ud.addr.buf = (UInt8*)&tmpsin;
	ud.opt.len = 0;
	ud.opt.buf = 0;
	ud.udata.len = len;
	ud.udata.buf = (UInt8*)buf;
	
	OSStatus err = OTSndUData(ps->GetEndpoint(),&ud);

	ps->SetOccupied(0);
	if (err == kOTNoError)
		return len;
		
	return result;
}

int th_socket_recvfrom(SOCKET s,char *buf,int len,int flags,
						struct sockaddr *addr,int *slen)
{
	int result = -1;
	Private_Socket	*ps = (Private_Socket *)s;
	if (!ps) return -1;
	assert(ps->GetEndpoint());
	if (ps->GetLockout()) return -1;
	ps->SetOccupied(1);

	UDP_CHECK(ps);

	TUnitData ud;
	InetAddress tmpsin;
	ud.addr.maxlen = sizeof(struct InetAddress);
	ud.opt.maxlen = 0;
	ud.opt.buf = 0;
	ud.udata.maxlen = len;
	ud.udata.buf = (unsigned char *)buf;
	ud.addr.buf = (UInt8*)&tmpsin;
	OTFlags f;
	
	while (1)
		{
		if (ps->GetLockout())
			{
			result = -1;
			break;;
			}
		OSStatus err = OTRcvUData(ps->GetEndpoint(),&ud,&f);
		if (err == kOTNoError)
			{
			result = ud.udata.len;
			addr->sin_port = tmpsin.fPort;
			addr->sin_addr.s_addr = tmpsin.fHost;
			break;
			}
		if (err != kOTNoDataErr)
			break;
		CCThread::Yield();
		}
		
	ps->SetOccupied(0);
	return result;
}


// intentionally unsafe, like Unix
struct hostent *th_socket_gethostbyname(const char *name)
{
	return NULL;
#if 0
	static struct hostent	he;
	he.h_addr = &(he.addr);
	hostInfo	hi;

	int err = StrToAddr((char *)name,&hi,NULL,NULL);

	CCThread_ID_Type	th;
	::GetCurrentThread(&th);

	gsSocketErrnoMutex.Lock();
	gsSocketErrnoMap[th] = err;
	gsSocketErrnoMutex.Unlock();

	he.addr = hi.addr[0];

	return (err == noErr ? &he : NULL);
#endif
}

unsigned long th_socket_inet_addr(const char *name)
{
	int a,b,c,d;
	char ch;

	sscanf((char *)name,"%d%c%d%c%d%c%d",&a,&ch,&b,&ch,&c,&ch,&d);
	return ((a << 24) + (b << 16) + (c << 8) + d);
}

int th_socket_getsockname(SOCKET s,struct sockaddr *addr,int *len)
{
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;

	addr->sin_addr.s_addr = ps->GetBindAddress();
	addr->sin_port = ps->GetBindPort();

	return 0;
}

int th_socket_getpeername(SOCKET s,struct sockaddr *peer,int *len)
{
	Private_Socket *ps = (Private_Socket *)s;
	if (!ps) return -1;

	TCP_CHECK(ps); // must be tcp

	peer->sin_addr.s_addr 	= ps->GetPeerAddress();
	peer->sin_port 			= ps->GetPeerPort();
	*len = sizeof(struct sockaddr);

	return 0;
}


int th_socket_errno(void)
{
	stack_MutexLock	sml(&gsSocketErrnoMutex);

	CCThread_ID_Type	th;
	::GetCurrentThread(&th);
	return gsSocketErrnoMap[th];
}


#endif
