/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include	<MegaHeaders.h>

/******************************************************************************
Two functions that do some interesting IP stuff for Unix

FindACloseIPAddress - Given our IP address, it searches the /etc/hosts table
	for another machine that is "close". Close is defined as a machine whose
	address differs only by the low byte. It returns the address if found
	else zero

FindMyIPAddress - Given our machine name, it searches the /etc/hosts
	table for our address.
******************************************************************************/


///////////////////////////////////////////////////////////////////////////////
// FindACloseIPAddress
//
// INPUT:
//	addr	- our address in native numeric form
//
// OUTPUT:
//	a "close" machine address in native numeric form if one is found, else
//	zero
///////////////////////////////////////////////////////////////////////////////
unsigned long FindACloseIPAddress(unsigned long addr)
{
	char	buf[1024],*ptr;
	unsigned long	result = 0;

	FILE *f = fopen("/etc/hosts","r");
	if (f == NULL)
		return result;
	do
		{
		if (!fgets(buf,1024,f))
			return result;
		// is the line a comment
		if (buf[0] != '#')
			{
			// no, parse out the address
			result = strtoul(ptr = strtok(buf,"."),NULL,10) << 24;
			result += strtoul(ptr = strtok(NULL,"."),NULL,10) << 16;
			result += strtoul(ptr = strtok(NULL,"."),NULL,10) << 8;
			result += strtoul(ptr = strtok(NULL," "),NULL,10);			
			if ((addr != result) && (addr & 0xffffff00) == (result & 0xffffff00))
				return result;
			}
		}
	while (1);
	fclose(f);
	return result;
}

///////////////////////////////////////////////////////////////////////////////
// FindMyIPAddress
//
// Given my machine name, search the /etc/hosts file for my address
//
// INPUT:
//	name	- my machine name
//	
// OUTPUT:
//	The address of my machine in native numeric form if found, else zero
///////////////////////////////////////////////////////////////////////////////
unsigned long FindMyIPAddress(const char *name)
{
#ifdef WIN32
	struct hostent *h = gethostbyname(name);
	if (!h) return 0;

	return ntohl(*((unsigned long *)h->h_addr_list[0]));
#else
	char	buf[1024],*ptr;
	unsigned long	result = 0;

	FILE *f = fopen("/etc/hosts","r");
	if (f == NULL) 
		return result;
	do
		{
		if (!fgets(buf,1024,f))
			return result;
		// is it a comment?
		if (buf[0] != '#')
			{
			// no, parse out the address
			if (strstr(buf,name))
				{
				result = strtoul(ptr = strtok(buf,"."),NULL,10) << 24;
				result += strtoul(ptr = strtok(NULL,"."),NULL,10) << 16;
				result += strtoul(ptr = strtok(NULL,"."),NULL,10) << 8;
				result += strtoul(ptr = strtok(NULL," "),NULL,10);
				break;
				}
			}
		}
	while (1);
	fclose(f);
	return result;
#endif
}




