/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	NMR_FRACTION_H_
#define NMR_FRACTION_H_

#include	<iostream>
using namespace std;

template <class base_t>
class nmr_fraction
{
protected:

	base_t	mNumerator;
	base_t	mDenominator;

public:

	//// CONSTRUCTORS

nmr_fraction(void)
		{
		mNumerator = 0;
		mDenominator = 1;
		}

nmr_fraction(const nmr_fraction &f)
		{
		mNumerator = f.mNumerator;
		mDenominator = f.mDenominator;
		}

nmr_fraction(const base_t numerator,const base_t denominator)
		{
		mNumerator = numerator;
		mDenominator = denominator;
		}

nmr_fraction(const base_t l)
		{
		mNumerator = l;
		mDenominator = 1;
		}

	///// ASSIGN

nmr_fraction&	
operator=(const nmr_fraction &f)
		{
		mNumerator = f.mNumerator;
		mDenominator = f.mDenominator;
		return *this;
		}

nmr_fraction&	
operator=(const base_t l)
		{
		mNumerator = l;
		mDenominator = 1;
		return *this;
		}

	///// MATH

nmr_fraction	
operator+(const nmr_fraction &f)
		{
		base_t	n1,n2;

		if (f.mDenominator == mDenominator)
			return nmr_fraction(mNumerator + f.mNumerator,mDenominator);
		
		n1 = mNumerator * f.mDenominator;
		n2 = f.mNumerator * mDenominator;
		return nmr_fraction(n1+n2,mDenominator * f.mDenominator);
		}

nmr_fraction	
operator+(base_t l)
		{
		return nmr_fraction(mNumerator+(((base_t)l) * mDenominator),
															mDenominator);
		}


nmr_fraction&	
operator+=(const nmr_fraction &f)
		{
		if (f.mDenominator == mDenominator)
			{
			mNumerator += f.mNumerator;
			}
		else
			{
			mNumerator = (mNumerator * f.mDenominator) + 
						(f.mNumerator * mDenominator);
			mDenominator *= f.mDenominator;
			}
		return *this;
		}

nmr_fraction&	
operator+=(base_t l)
		{
		mNumerator += (l * mDenominator);
		return *this;
		}



nmr_fraction	
operator-(const nmr_fraction &f)
		{
		base_t	n1,n2;

		if (f.mDenominator == mDenominator)
			return nmr_fraction(mNumerator - f.mNumerator,mDenominator);

		n1 = mNumerator * f.mDenominator;
		n2 = f.mNumerator * mDenominator;
		return nmr_fraction(n1-n2,mDenominator * f.mDenominator);
		}

nmr_fraction	
operator-(base_t l)
		{
		return nmr_fraction(mNumerator-(((base_t)l) * mDenominator),
															mDenominator);
		}



nmr_fraction&	
operator-=(const nmr_fraction &f)
		{
		if (f.mDenominator == mDenominator)
			{
			mNumerator -= f.mNumerator;
			}
		else
			{
			mNumerator = (mNumerator * f.mDenominator) -
						(f.mNumerator * mDenominator);
			mDenominator *= f.mDenominator;
			}
		return *this;
		}

nmr_fraction&	
operator-=(base_t l)
		{
		mNumerator -= (l * mDenominator);
		return *this;
		}


nmr_fraction	
operator*(const nmr_fraction &f)
		{
		return nmr_fraction(mNumerator * f.mNumerator,
						mDenominator * f.mDenominator);
		}

nmr_fraction	
operator*(base_t l)
		{
		return nmr_fraction(mNumerator * ((base_t)l),mDenominator);
		}



nmr_fraction&	
operator*=(const nmr_fraction &f)
		{
		mNumerator *= f.mNumerator;
		mDenominator *= f.mDenominator;
		return *this;
		}

nmr_fraction&	
operator*=(base_t l)
		{
		mNumerator *= l;
		return *this;
		}



nmr_fraction	
operator/(const nmr_fraction &f)
		{
		return nmr_fraction(mNumerator * f.mDenominator,
						mDenominator * f.mNumerator);
		}

nmr_fraction	
operator/(base_t l)
		{
		return nmr_fraction(mNumerator,mDenominator * ((base_t)l));
		}


nmr_fraction&	
operator/=(const nmr_fraction &f)
		{
		mNumerator *= f.mDenominator;
		mDenominator *= f.mNumerator;
		return *this;
		}

nmr_fraction&	
operator/=(base_t l)
		{
		mDenominator *= l;
		return *this;
		}


nmr_fraction	
operator-(void)
		{
		return nmr_fraction(-mNumerator,mDenominator);
		}

nmr_fraction	
operator~(void)
		{
		return nmr_fraction(mDenominator,mNumerator);
		}


	///// EQUALITY


int		
operator==(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) == (f.mNumerator*mDenominator));
		}

int		
operator==(base_t l) const
		{
		return (mNumerator == (l * mDenominator));
		}



int		
operator>(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) > (f.mNumerator*mDenominator));
		}

int		
operator>(base_t l) const
		{
		return (mNumerator > (l * mDenominator));
		}


int		
operator<(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) < (f.mNumerator*mDenominator));
		}

int		
operator<(base_t l) const
		{
		return (mNumerator < (l * mDenominator));
		}


int		
operator>=(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) >= (f.mNumerator*mDenominator));
		}

int		
operator>=(base_t l) const
		{
		return (mNumerator >= (l * mDenominator));
		}


int		
operator<=(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) <= (f.mNumerator*mDenominator));
		}

int		
operator<=(base_t l) const
		{
		return (mNumerator <= (l * mDenominator));
		}


int		
operator!=(const nmr_fraction &f) const
		{
		return ((mNumerator*f.mDenominator) != (f.mNumerator*mDenominator));
		}

int		
operator!=(base_t l) const
		{
		return (mNumerator != (l * mDenominator));
		}


	//// CASTS

base_t	
Numerator(void) const
		{
		return mNumerator;
		}

base_t	
Denominator(void) const
		{
		return mDenominator;
		}

};

template <class base_t>
std::ostream& operator<<(std::ostream &o,const nmr_fraction<base_t> &f)
{
	o << f.Numerator() << "/" << f.Denominator();
	return o;
}


#endif
