/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <MegaHeaders.h>
#include <nmr_string.h>
#include <stack_aids.h>

///////////////////////////////////////////////////////////////////////////////
//
// nmr_string
//
// A string class. Every platform seems to have it's own variant. It
// can get frustrating trying to resolve the differences, so I just
// wrote my own, giving it an appropriately wierd name to avoid conflicts.
// Should do all the basic stuff. Uses a reference count approach to
// speed handling of duplicate strings. No substring support.
//
///////////////////////////////////////////////////////////////////////////////

nmr_string::nmr_string(void)
{
#ifndef NDEBUG
	strcpy(mClassName,"nmr_string");
#endif
	mData = new string_ref;
}

nmr_string::nmr_string(const nmr_string &s)
{
#ifndef NDEBUG
	strcpy(mClassName,"nmr_string");
#endif
	mData = s.mData;
	mData->AddReference();
}

nmr_string::nmr_string(const char *s)
{
#ifndef NDEBUG
	strcpy(mClassName,"nmr_string");
#endif
	mData = new string_ref(s);
}

nmr_string::~nmr_string(void)
{
	assert(mData);
#ifndef NDEBUG
	strcpy(mClassName,"shit_str");
#endif

	if (!mData->DeleteReference())
		delete mData;
	mData = NULL;
}

char
nmr_string::operator[](int i) const
{
	assert(mData);
	assert(mData->data());
	return (mData->data())[i];
}

nmr_string&
nmr_string::operator=(const nmr_string &s)
{
	nmr_string::string_ref	*tmp;

	tmp = mData;
	mData = s.mData;
	mData->AddReference();
	if (!tmp->DeleteReference())
		delete tmp;

	return *this;
}

nmr_string& 
nmr_string::operator=(const char *s)
{
	if (!mData->DeleteReference())
		delete mData;
	mData = NULL;
	mData = new nmr_string::string_ref(s);

	return *this;
}

int	
nmr_string::compare(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);
	return strcmp(mData->data(),s.mData->data());
}

//////
int 
nmr_string::operator==(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);
	return (strcmp(mData->data(),s.mData->data()) == 0);
}

int
nmr_string::operator==(const char *s) const
{
	if (!s) return 0;

	assert(mData);
	return (strcmp(mData->data(),s) == 0);
}

int 
nmr_string::operator!=(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);
	return (strcmp(mData->data(),s.mData->data()) != 0);
}

int
nmr_string::operator!=(const char *s) const
{
	if (!s) return 0;

	assert(mData);
	return (strcmp(mData->data(),s) != 0);
}

int
operator==(const char *s,const nmr_string &nmr)
{
	if (!s) return 0;

	assert(nmr.mData);
	return (strcmp(nmr.mData->data(),s) == 0);
}

//////
int 
nmr_string::operator<(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);

	return (strcmp(mData->data(),s.mData->data()) < 0);
}

int
nmr_string::operator<(const char *s) const
{
	if (!s) return 0;

	assert(mData);
	return (strcmp(mData->data(),s) < 0);
}

int operator<(const char *s,const nmr_string &nmr)
{
	if (!s) return 1;

	assert(nmr.mData);
	return (strcmp(s,nmr.mData->data()) < 0);
}
//////

int 
nmr_string::operator>(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);

	return (strcmp(mData->data(),s.mData->data()) > 0);
}

int
nmr_string::operator>(const char *s) const
{
	if (!s) return 1;
	assert(mData);

	return (strcmp(mData->data(),s) > 0);
}

int operator>(const char *s,const nmr_string &nmr)
{
	if (!s) return 0;

	assert(nmr.mData);
	return (strcmp(s,nmr.mData->data()) > 0);
}

//////
int 
nmr_string::operator<=(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);

	return (strcmp(mData->data(),s.mData->data()) <= 0);
}

int
nmr_string::operator<=(const char *s) const
{
	if (!s) return 0;
	assert(mData);

	return (strcmp(mData->data(),s) <= 0);
}

int operator<=(const char *s,const nmr_string &nmr)
{
	if (!s) return 1;
	assert(nmr.mData);

	return (strcmp(s,nmr.mData->data()) <= 0);
}

//////
int 
nmr_string::operator>=(const nmr_string &s) const
{
	assert(mData);
	assert(s.mData);

	return (strcmp(mData->data(),s.mData->data()) >= 0);
}

int
nmr_string::operator>=(const char *s) const
{
	if (!s) return 1;
	assert(mData);
	return (strcmp(mData->data(),s) >= 0);
}

int operator>=(const char *s,const nmr_string &nmr)
{
	if (!s) return 0;
	assert(nmr.mData);

	return (strcmp(s,nmr.mData->data()));
}
//////
nmr_string
nmr_string::operator+(const nmr_string &s) const
{
	int len;
	char *tmp = NULL;
	nmr_string nmr_tmp;

try {
	assert(mData);
	assert(s.mData);

	len = mData->length() + s.mData->length();
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	strcat(tmp,s.mData->data());
	nmr_tmp = tmp;
	delete [] tmp;
	return nmr_tmp;
	}
catch(...)
	{
	delete [] tmp;
	throw;
	}
}

nmr_string
nmr_string::operator+(const char *s) const
{
	int len;
	char *tmp = NULL;
	nmr_string nmr_tmp;

try {
	assert(mData);

	len = mData->length() + (s ? strlen(s) : 0);
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	if (s)
		strcat(tmp,s);
	nmr_tmp = tmp;
	delete [] tmp;
	return nmr_tmp;
	}
catch(...)
	{
	delete [] tmp;
	throw;
	}
}

nmr_string
nmr_string::operator+(int i) const
{
	int len;
	char *tmp = NULL;
	nmr_string nmr_tmp;
	char itmp[128];

try {
	assert(mData);

	sprintf(itmp,"%d",i);
	len = mData->length() + strlen(itmp);
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	strcat(tmp,itmp);
	nmr_tmp = tmp;
	delete [] tmp;
	return nmr_tmp;
	}
catch(...)
	{
	delete [] tmp;
	throw;
	}
}


//////
nmr_string&
nmr_string::operator+=(const nmr_string &s)
{
	int		len;
	char*	tmp = NULL;
	nmr_string::string_ref	*ref = NULL;

try {
	assert(mData);
	assert(s.mData);

	len = mData->length() + s.mData->length();
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	strcat(tmp,s.mData->data());

	ref = new nmr_string::string_ref(tmp);
	delete [] tmp;

	if (!mData->DeleteReference())
		delete mData;
	mData = ref;
	return *this;
	}
catch(...)
	{
	delete ref;
	delete [] tmp;
	throw;
	}
}

nmr_string&
nmr_string::operator+=(const char *s)
{
	int		len;
	char*	tmp = NULL;
	nmr_string::string_ref	*ref = NULL;

try {
	assert(mData);

	len = mData->length() + (s ? strlen(s) : 0);
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	if (s)
		strcat(tmp,s);

	ref = new nmr_string::string_ref(tmp);
	delete [] tmp;

	if (!mData->DeleteReference())
		delete mData;
	mData = ref;
	return *this;
	}
catch(...)
	{
	delete ref;
	delete [] tmp;
	throw;
	}
}

nmr_string&
nmr_string::operator+=(int i)
{
	int		len;
	char*	tmp = NULL;
	nmr_string::string_ref	*ref = NULL;
	char	itmp[128];

try {
	assert(mData);

	sprintf(itmp,"%d",i);
	len = mData->length() + strlen(itmp);
	tmp = new char[len+1];
	strcpy(tmp,mData->data());
	strcat(tmp,itmp);

	ref = new nmr_string::string_ref(tmp);
	delete [] tmp;

	if (!mData->DeleteReference())
		delete mData;
	mData = ref;
	return *this;
	}
catch(...)
	{
	delete ref;
	delete [] tmp;
	throw;
	}
}

//////
int	
nmr_string::length(void) const
{
	assert(mData);
	return mData->length();
}

const char *
nmr_string::data(void) const
{
	assert(mData);
	return mData->data();
}

nmr_string::operator const char *() const
{
	assert(mData);
	return mData->data();
}

///////////////////////////////////////////////////////////////////////////////


nmr_string::string_ref::string_ref(void)
{
	stack_MutexLock sml(&mMutex);
#ifndef NDEBUG
	strcpy(mClassName,"string_ref");
#endif
	mData = new char[1];
	mData[0] = '\0';
	mRefCount = 1;
}

nmr_string::string_ref::string_ref(const char *s)
{
	stack_MutexLock sml(&mMutex);

#ifndef NDEBUG
	strcpy(mClassName,"string_ref");
#endif
	mRefCount = 1;
	if (s)
		{
		mData = new char[strlen(s) + 1];
		strcpy(mData,s);
		}
	else
		{
		mData = new char[1];
		mData[0] = '\0';
		}
}

nmr_string::string_ref::~string_ref(void)
{
	stack_MutexLock sml(&mMutex);

	assert(mRefCount == 0);
	assert(mData);
#ifndef	NDEBUG
	strcpy(mClassName,"shit_ref");
	mData[0] = '\0';
#endif
	delete [] mData;
}

int
nmr_string::string_ref::AddReference(void)
{
	stack_MutexLock sml(&mMutex);

	return ++mRefCount;
}

int
nmr_string::string_ref::DeleteReference(void)
{
	stack_MutexLock sml(&mMutex);

	return --mRefCount;
}

int
nmr_string::string_ref::length(void)
{
	stack_MutexLock sml(&mMutex);

	assert(mData);
	return strlen(mData);
}
	
const char *
nmr_string::string_ref::data(void)
{
	stack_MutexLock sml(&mMutex);

	assert(mData);
	return  mData;
}
///////////////////////////////////////////////////////////////////////////////

ostream& operator<<(ostream &o,const nmr_string &s)
{
	o << s.data();
	return o;
}

ostream& operator<<(ostream &o,const nmr_string *s)
{
	o << s->data();
	return o;
}

istream& operator>>(istream &i,nmr_string &s)
{
	char buf[1024];
	i.getline(buf,1024,'\n');
	s = buf;
	return i;
}
