/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef NMRSTRING_H_
#define NMRSTRING_H_

#include	<MegaHeaders.h>

#include	<CCMutex.h>

class nmr_string
{
protected:
	
	class string_ref
		{
		CLASS_NOCOPY(string_ref)

		protected:
#ifndef NDEBUG
			char	mClassName[11];
#endif
			char *	mData;
			int		mRefCount;
			CCMutex	mMutex;
		public:
			string_ref(void);
			string_ref(const char *s);
			~string_ref(void);
			int		AddReference(void);
			int		DeleteReference(void);
			int		length(void);
			const char *data(void);
		};

#ifndef NDEBUG
	char	mClassName[11];
#endif
	string_ref *mData;
	string_ref *_GetStringRef(void) const;

public:
	nmr_string(void);
	nmr_string(const char *);
	nmr_string(const nmr_string &s);
	~nmr_string(void);

	int	compare(const nmr_string &s) const;
	nmr_string& operator=(const char *);
	nmr_string& operator=(const nmr_string &);
	int operator==(const nmr_string &s) const;
	int operator==(const char *) const;
	int operator!=(const nmr_string &s) const;
	int operator!=(const char *) const;

	int operator<(const nmr_string &s) const;
	int operator<(const char *s) const;

	int operator>(const nmr_string &s) const;
	int operator>(const char *s) const;

	int operator<=(const nmr_string &s) const;
	int operator<=(const char *s) const;

	int operator>=(const nmr_string &s) const;
	int operator>=(const char *s) const;

	nmr_string operator+(const nmr_string &s) const;
	nmr_string operator+(const char *s) const;
	nmr_string operator+(int i) const;
	nmr_string& operator+=(const nmr_string &s);
	nmr_string& operator+=(const char *s);
	nmr_string& operator+=(int i);

	char	operator[](int i) const;
	int	length(void) const;
	const char *data(void) const;
	operator const char *() const;

	friend int operator==(const char *s,const nmr_string &nmr);
	friend int operator<(const char *s,const nmr_string &nmr);
	friend int operator>(const char *s,const nmr_string &nmr);
	friend int operator<=(const char *s,const nmr_string &nmr);
	friend int operator>=(const char *s,const nmr_string &nmr);
	
};

ostream& operator<<(ostream &o,const nmr_string &s);
ostream& operator<<(ostream &o,const nmr_string *s);
istream& operator>>(istream &i,nmr_string &s);

#endif
