/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef	STATIC_ALLOCATOR_H_
#define STATIC_ALLOCATOR_H_

////////////// include in cpp file.

#include	<MegaHeaders.h>

// allow this to be overridden
#ifndef	BLOCKS_PER_FAILURE
#define	BLOCKS_PER_FAILURE	10
#endif

#define	static_allocation(CLASS_NAME) \
typedef void	*CLASSNAME ## block_hunk_t; \
\
static CLASSNAME ## block_hunk_t	*gs_staticalloc_ ## CLASS_NAME = NULL;\
static int			gs_staticalloc_capacity_ ## CLASS_NAME;\
static int			gs_staticalloc_nextavailable_ ## CLASS_NAME;\
\
static CCMutex	gs_staticalloc_Mutex_ ## CLASS_NAME;\
\
static void increase_gs_staticalloc_ ## CLASS_NAME(void)\
{\
	int		x;\
	size_t	siz = sizeof(CLASS_NAME);\
\
	if (gs_staticalloc_ ## CLASS_NAME)\
		delete [] gs_staticalloc_ ## CLASS_NAME;\
	gs_staticalloc_capacity_ ## CLASS_NAME += BLOCKS_PER_FAILURE;\
	gs_staticalloc_ ## CLASS_NAME = \
		new CLASSNAME ## block_hunk_t[gs_staticalloc_capacity_ ## CLASS_NAME];\
	for (x = 0; x < BLOCKS_PER_FAILURE; ++x)\
		{\
		gs_staticalloc_ ## CLASS_NAME\
			[++gs_staticalloc_nextavailable_ ## CLASS_NAME] = \
			::operator new(siz);\
		}\
}\
\
static int init_gs_staticalloc_ ## CLASS_NAME(void)\
{\
\
	gs_staticalloc_capacity_ ## CLASS_NAME = 0;\
	gs_staticalloc_nextavailable_ ## CLASS_NAME = -1;\
	increase_gs_staticalloc_ ## CLASS_NAME();\
\
	return 1;\
}\
\
static void destroy_gs_staticalloc_ ## CLASS_NAME(void)\
{\
	int x;\
\
	if (!gs_staticalloc_ ## CLASS_NAME)\
		return;\
\
	for (x = 0; x < gs_staticalloc_capacity_ ## CLASS_NAME; ++x)\
		{\
		::operator delete(gs_staticalloc_ ## CLASS_NAME[x]);\
		}\
	delete [] gs_staticalloc_ ## CLASS_NAME;\
}\
\
static void *do_new_ ## CLASS_NAME(size_t t)\
{\
	void	*tmp;\
\
	gs_staticalloc_Mutex_ ## CLASS_NAME.Lock();\
\
	assert(t == sizeof(CLASS_NAME));\
\
	if (gs_staticalloc_nextavailable_ ## CLASS_NAME == -1)\
		{\
		gTraceLog->SendMessage(#CLASS_NAME " STATIC ALLOCATOR ENLARGED");\
		increase_gs_staticalloc_ ## CLASS_NAME();\
		}\
	tmp = gs_staticalloc_ ## CLASS_NAME[gs_staticalloc_nextavailable_ ## CLASS_NAME--];\
\
	gs_staticalloc_Mutex_ ## CLASS_NAME.Unlock();\
	return tmp;\
}\
\
static void do_delete_ ## CLASS_NAME(void *d)\
{\
	gs_staticalloc_Mutex_ ## CLASS_NAME.Lock();\
\
	gs_staticalloc_ ## CLASS_NAME[++gs_staticalloc_nextavailable_ ## CLASS_NAME] = d;\
\
	assert(gs_staticalloc_nextavailable_ ## CLASS_NAME < gs_staticalloc_capacity_ ## CLASS_NAME);\
\
	gs_staticalloc_Mutex_ ## CLASS_NAME.Unlock();\
}\
\
class allocation_instigator_ ## CLASS_NAME\
{\
public:\
	allocation_instigator_ ## CLASS_NAME(void) { init_gs_staticalloc_ ## CLASS_NAME(); }\
	~allocation_instigator_ ## CLASS_NAME(void) { destroy_gs_staticalloc_ ## CLASS_NAME(); }\
};\
\
static allocation_instigator_ ## CLASS_NAME dummy_ ## CLASS_NAME;\

#endif
