/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _duckwalk_h
#define _duckwalk_h

#ifdef __cplusplus
extern "C" {
#endif

struct tHFBFile;   			/* pre-define for compiler*/
struct tHFBBuffer;
struct tHFBStream;

#ifdef __cplusplus
}
#endif

#include "duckvfw.h"
#include "nethfb.h"

#ifdef __cplusplus
extern "C" {
#endif

	/*
	at 12 bytes per entry:
	12*30*2 = 720 bytes/second on index *300sec(5 min) = 216K

	at 8 bytes per entry:
	8*30*2  = 480 bytes/second on index *300sec(5 min) = 144K
	*/

#if defined(LARGECHUNKS)
#if !defined(NETHFB)
#undef LARGECHUNKS
#pragma message( "!!!!!!!!!!!!!! Undefining LARGECHUNKS !!!!!!!!!!!!!!" )
#endif
#endif

typedef struct
{
	unsigned long	dwChunkOffset;     /* Position of chunk */
#ifdef LARGECHUNKS
	unsigned long	dwChunkLength;     /* Length of chunk */
#else
	unsigned short	dwChunkLength;     /* Length of chunk */
#endif
	unsigned char	dckID;             /* The stream associated with this chunk */
	unsigned char	dckFlags;
} DCKINDEXENTRY;

#define dukfHandle int

#if defined(SEGA)
#define fpos_t	long         
#else
#if defined(SONY)
#define fpos_t	long
#endif

#endif

#define MAXSTREAMS 16

/**************************
 *
 * Stream information
 *
 *************************/

/* private information */    

/* 24 bytes */
typedef struct tHFBStream{    
	enum HFB_StreamType streamType;
	struct tHFBStreamX* parent;
	struct tHFBStream* nextStream;
	long getOffset;
	int lastSample;
	int lastFrame;
} HFB_STREAM;

typedef struct tHFBStreamX{ /* 100 bytes */      
	enum HFB_StreamType streamType;
	int streamNum;
	long lSampleRate;
	long lStart;
	long lLength;
	char szName[24];
	union { 
		DKWAVEFORM WaveformInfo; /* 20 bytes */ 
		DKBITMAP BitmapInfo;	 /* 6 bytes */
	} a;                  
	struct tHFBFile* fHnd;
	struct tHFBBuffer* bfHnd;
	long getOffset;
	
	int lastSample;
	int sampleSize;
	long rate,scale;
	struct tHFBStream* nextStream;
	int numRef;
	int lastFrame;
	unsigned long fccType,fccHandler;
} HFB_STREAM_X, *HFB_STREAM_HANDLE_X;

/**************************
 *
 * TrueMotion/S A/V File information
 *
 *************************/

/* private information */
typedef struct tHFBFile{                                       
	long lFileSize;
	long lNumStreams;
	char sName[96];
    long lNumFrames;
	
	dukfHandle aviFile;				/* file with which we are associated */
	enum HFB_FileType fileType;
	MainAVIHeader header;			/* mainavi header for the opened file */
	DCKINDEXENTRY* indx;			/* pointer to the AVI index */
	unsigned short* findx;			/* pointer to the frame only index */
	int dfAudioStream;				/* stream number for first audio stream */
	int dfVideoStream;				/*  stream number for first video stream */
	int GlobalFrameRate;
	struct tHFBBuffer* bfHnd;
	long  pos;					/* offset of movie chunk within file */
    long indexSize;
	int streamCount;
	HFB_STREAM_X heads[MAXSTREAMS];	/* pointer to the stream headers */
} HFB_FILE;

#define NEW_MEMBERS	1

typedef struct tHFBBuffer{                                       
	HFB_FILE_HANDLE	fHnd,qfHnd;
    
    unsigned char* buffer;	/* pointer to the raw buffer*/
	long rawSize;
	enum HFB_Modes mode;
	
	unsigned long movieSize;			/* size of the movie buffer window */
	unsigned long bfrSize;			/* total size of the movie buffer */
	unsigned char* movie;	/* pointer to the movie buffer */   
    unsigned long movieOffset;   
    unsigned long releaseOffset;
    
	unsigned long movieSize2;		/* size of the movie buffer window */
	unsigned long bfrSize2;			/* total size of the movie buffer */
	unsigned char* movie2;	/* pointer to the movie buffer */   
	unsigned long movieOffset2;
    
	DCKINDEXENTRY* indx;	/* pointer to the AVI index */
	long minIndx;
	
	unsigned short* findx;	/* pointer to the frame only index */
	int indxCount;			/* number of index entries */
	int nextIndex;			/* next chunk to be read from index */
	int lastReleased;		/* last chunk released by app */
	
	unsigned int maxChunkSize;		/* max bytes per read */
	int recAverage;			/* average bytes per frame */
	int maxToRead;			/* max bytes per read */
	int modToRead;			/* how often to modify reads */
	unsigned int nextRead;			/* position within movie to perform next read into */
	unsigned int readIndex;			/* last byte read from buffer */
	unsigned int lastByteOffset;		/* offset of last byte read from file */
	
	int GlobalFrameRate;
	int bytesPerSec;
	
	int frameCount;
	int subSec;
	int readTable[60];
	int readTotal;
	int extrabuff;
	int hwBuffer;          
	int fcount;				/* frame counter for fill buffer */
	int delay;
	int bytesReadLast;
	void* NetworkContext;

#ifdef NEW_MEMBERS
	unsigned long	fly_frameTotal;  /* total bytes of data processed */
	unsigned long	fly_videoFrameCount;  /* total number of video frames processed */
	AVIINDEXENTRY	fly_aviRecWrapper; 
	int				fly_bAviRecWrapperSet;
	int				fly_oldLindxOffset;
	int				fly_aviIndxCount;
	int				fly_chunkType;
#endif
	int frameToggle;
	long m_indxSize;	/* size needed for avi index */
	long targetSize;	/* desired buffer size (not including index) */
} HFB_BUFFER;

#define DCKCHUNK_CLEAR      0x00
#define DCKCHUNK_READ       0x01
#define DCKCHUNK_RELEASED   0x02
#define DCKCHUNK_UPPER      0x04
#define DCKCHUNK_KEYFRAME	0x08

#define QVAL 2048

#define ROUND_UP2K(x) ((((x)+2047)>>11)<<11)
#define ROUND_DOWN2K(x) (((x)>>11)<<11)

#define fcc2chars(x) (BYTE)x,(BYTE)(x>> 8),(BYTE)(x>>16),(BYTE)(x>>24)

#define duck_isdigit(x) ((x >= 0x30 && x <= 0x39) ? 1 : 0)

#define DEBUG 0

#if DEBUG
#define DIAGNOSTIC(x) x
#else 
#define DIAGNOSTIC(x)
#endif

void AVIFileClose(HFB_FILE_HANDLE dckPtr);
void buildIndex(HFB_BUFFER_HANDLE dckPtr, void* data, int size);  

/* FWG 9-13-2000 */
int initIndex(HFB_BUFFER_HANDLE dckPtr, void* data, int size);
void parseIndex(HFB_BUFFER_HANDLE dckPtr, void* data, int size);
int readMoreData(HFB_BUFFER_HANDLE dckPtr, int size);
/* FWG 9-13-2000 */

#ifdef __cplusplus
}
#endif

#endif
