/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "duck_hfb.h"
#include "duck_mem.h"
#include "duckwalk.h"


HFB_STREAM_HANDLE* HFB_GetStreamArray(
	HFB_FILE_HANDLE FileHandle,     /* the file we will be looking in for our streams */
	unsigned long   StreamType,     /* the type of stream we are looking for */
	unsigned int*   numberOfStreams)  /* a pointer to the number of matching streams */
{
	unsigned int       streamIndex    = 0;
	HFB_STREAM_HANDLE* StreamArray    = (HFB_STREAM_HANDLE*) NULL;
	HFB_STREAM*        RawStreamArray = (HFB_STREAM*) NULL;

	/* Check the passed-in parameters */
	if (!FileHandle || !numberOfStreams)
	{
		return (HFB_STREAM_HANDLE*) NULL;
	}

	/* Before allocating the array, we need to know how many elements will be in the array */
	*numberOfStreams = 0;
	if (StreamType == DUK_UNDEFINED)
	{
		/* if the requested stream type is "undefined", we will return all arrays */
		*numberOfStreams = (int) FileHandle->lNumStreams;
	}
	else
	{
		/* we are only looking for arrays of a particular type */
		for (streamIndex = 0;  (int) streamIndex < FileHandle->lNumStreams; streamIndex++)
		{
			/* if this stream is of the correct type, increment our counter */
			if (FileHandle->heads[streamIndex].streamType == (long)StreamType)
			{
				*numberOfStreams += 1;
			}
		}
	}

	/* in the case of no matching streams, return a NULL array */
	if (*numberOfStreams == 0)
	{
		return (HFB_STREAM_HANDLE*) NULL;
	}

	/* Allocate the array of HFB stream handles */
	StreamArray = (HFB_STREAM_HANDLE*) duck_calloc(*numberOfStreams, sizeof(HFB_STREAM_HANDLE), DMEM_GENERAL);
	if (!StreamArray)
	{
		return (HFB_STREAM_HANDLE*) NULL;
	}

	/* Now fill the array */
	for (streamIndex = 0;  streamIndex < *numberOfStreams; streamIndex++)
	{
		StreamArray[streamIndex] = HFB_GetStream(FileHandle, NULL, streamIndex + 1, StreamType);
	}

	return StreamArray;
}


void HFB_ReleaseStreamArray(
	HFB_STREAM_HANDLE* StreamArray,     /* the array of streams we want to release */
	unsigned int       numberOfStreams) /* number of streams in the array */
{
	/* We only do this if we're given a valid array */
	if (StreamArray)
	{
		/* Release each stream */
		while (numberOfStreams > 0)
		{
			numberOfStreams--;
			/* release this stream if it hasn't already been released */
			if (StreamArray[numberOfStreams])
			{
				HFB_ReleaseStream(StreamArray[numberOfStreams]);
				StreamArray[numberOfStreams] = (HFB_STREAM_HANDLE) NULL;
			}
		}

		/* Release the array itself */
		duck_free(StreamArray);
		StreamArray = (HFB_STREAM_HANDLE*) NULL;
	}
	return;
}

