/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************\
*                                                                               *
* hfb_avi.c -   multi-platform AVI Streaming library                            *
*               Copyright (c) 1994-99, On2.Com All rights reserved.             * 
*
*                 Edited by:    Eric Ameres                                     *
*                 Version:      1.4
*                 Edited:       09/28/99
                    - added stronger parameter checking

*                 Edited by:    Eric Ameres                                     *
*                 Version:      .97                                             *
*                 Edited:       01/25/94                                        *
                    - added adaptive datarate fill buffer routine
                    - removed unnecessary fillbuffering
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95												*
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>  
#include <string.h>
#include <assert.h>

#include "dkpltfrm.h"

#include "duckwalk.h"

#include "duck_mem.h"
#include "duck_io.h"
#include "danbug.h"

#if defined(DISPLAYDIB)
#if DISPLAYDIB
extern int globalIsDIB,globalDIBWidth,globalDIBHeight;
#endif
#endif
/*************************************************************************
    local defines
*************************************************************************/

#define fcc2chars(x) (BYTE)x,(BYTE)(x>> 8),(BYTE)(x>>16),(BYTE)(x>>24)

/*************************************************************************
    function prototypes
*************************************************************************/

static void getList(HFB_FILE_HANDLE dckPtr, unsigned char *buffer, long length);
static int interpretChunk(HFB_FILE_HANDLE dckPtr, DWORD chunkType, unsigned char *data, long size);
static long AVIChunkGet(HFB_FILE_HANDLE dckPtr, DWORD *chunkType, unsigned char *buffer,int bSize);  
    
/*************************************************************************
    static and global variables
*************************************************************************/

int GlobalDIBWidth,GlobalDIBHeight,GlobalIsDIB;

/*************************************************************************
    functions   
*************************************************************************/
                    
DKWAVEFORM *lHFB_GetAudioInfo(HFB_STREAM_HANDLE nStream, int *lNumChannels, int *lSamplesPerSec, int *lBytesPerSec, int *lwFormat)
{   
    assert(lNumChannels != NULL);
	assert(lSamplesPerSec != NULL);
	assert(lBytesPerSec != NULL);
	assert(lwFormat != NULL);
	
	if (nStream != NULL)
	{
		if (nStream->parent != NULL)
		{
			*lNumChannels = nStream->parent->a.WaveformInfo.nChannels;
			*lSamplesPerSec = (int)nStream->parent->a.WaveformInfo.nSamplesPerSec;
			*lwFormat = nStream->parent->a.WaveformInfo.wFormatTag;
			if ((*lwFormat) != 1)                               
				*lBytesPerSec = (*lSamplesPerSec)*2*(*lNumChannels);
			else
				*lBytesPerSec = (int)nStream->parent->a.WaveformInfo.nAvgBytesPerSec;

			return &(nStream->parent->a.WaveformInfo);
		}
	}

    *lNumChannels = 1;
    *lSamplesPerSec = 22050;
    *lBytesPerSec = 22050;                      
    *lwFormat = 1;

    return NULL;
}

int rrate,sscale;

/*DSS 1/96, NEW FUNCTION (place before HFB_GetFrameRates())*/
int lHFB_GetInitialFrames(HFB_STREAM_HANDLE vStream,HFB_STREAM_HANDLE aStream)
{
    if (aStream != NULL){
		if (aStream->parent)
			if (aStream->parent->fHnd)
    			return((int)aStream->parent->fHnd->header.dwInitialFrames);
    }
	else
	{
		/* just to keep compiler from complaining about unused variable vStream */
		if (vStream) return 0; 
	}
	return 0;
}

int lHFB_GetSamplesPerFrame(HFB_STREAM_HANDLE vStream,HFB_STREAM_HANDLE aStream)
{
    int SamplesPerSec = 22050;

   	if (vStream)
	{
		if (vStream->parent)
		{
			if (aStream && aStream->parent)
    			SamplesPerSec = aStream->parent->a.WaveformInfo.nSamplesPerSec;

			return (int)SamplesPerSec/vStream->parent->lSampleRate;
		}
	}
	return 0;
}

void lHFB_GetStreamRateAndScale(HFB_STREAM_HANDLE xStream,long *rate, long *scale)
{   
	if (rate) *rate = 0L;
	if (scale) *scale = 1L; /* set to 1 to avoid accidental div by 0s */

	if (xStream && xStream->parent)
	{
		if (rate) *rate = xStream->parent->rate;    
		if (scale) *scale = xStream->parent->scale;
	}
}

unsigned long lHFB_GetStreamFCCs(HFB_STREAM_HANDLE xStream,unsigned long *type, unsigned long *handler)
{   
	if (type) *type = 0L;
	if (handler) *handler = 0L;

	if (xStream && xStream->parent)
	{		  
		if (handler)
			*handler = xStream->parent->fccHandler;
		if (type)
			*type = xStream->parent->fccType;  
		return xStream->parent->fccType;  
	}
	return 0;
}

int lHFB_GetStreamSampleSize(HFB_STREAM_HANDLE xStream)
{   
	if (xStream && xStream->parent)
		return (xStream->parent->sampleSize);

	return 0;
}

void lHFB_GetFrameRates(HFB_STREAM_HANDLE vStream,HFB_STREAM_HANDLE aStream,int *lGlobalFrameRate, int *prebuffer)
{   
	if (vStream && vStream->parent)
	{
		*lGlobalFrameRate = vStream->parent->lSampleRate;    

		if (aStream == NULL) 
			*prebuffer = 0;
		else
		{
			if (aStream->parent && aStream->parent->fHnd && aStream->parent->fHnd->header.dwInitialFrames)
				*prebuffer = HFB_GetSamplesPerFrame(vStream,aStream) * aStream->parent->fHnd->header.dwInitialFrames;
			else
				*prebuffer = HFB_GetSamplesPerFrame(vStream,aStream) * (((*lGlobalFrameRate*3)/4) + 1);
		}
	}
} 

static int interpretChunk(HFB_FILE_HANDLE dckPtr, DWORD chunkType, unsigned char *data, long size)
{
    DIAGNOSTIC(DBG_Printf("interp'ing chunkType,%X\n",chunkType);)
 dbgout("HFB_interpretChunk\n");
        
    if (duck_isdigit((unsigned char)(chunkType)) || (size < 1)) return 1;
    
    switch (chunkType){
        case MKFOURCC('L','I','S','T'): /* list chunks read as a single block containing sub-chunks*/           
            data += 4;
            size -= 4;
        case MKFOURCC('s','t','r','l'): /* stream list */
        case MKFOURCC('h','d','r','l'): /* header info list */
        case MKFOURCC('r','e','c',' '): /* record list */
            getList(dckPtr,data, size);
            return 0;
        case MKFOURCC('a','v','i','h'): /* "avih"  AVI FILE HEADER*/
        case MKFOURCC('v','p','3','h'): /* "vp3h"  VP3 FILE HEADER*/
			{
				MainAVIHeader m,*mptr;

	            duck_memcpy(&m, data, sizeof(MainAVIHeader));

				mptr = &(dckPtr->header);
				
				mptr->dwMicroSecPerFrame = invert4(m.dwMicroSecPerFrame);
				mptr->dwMaxBytesPerSec = invert4(m.dwMaxBytesPerSec);
				mptr->dwPaddingGranularity = invert4(m.dwPaddingGranularity);
				mptr->dwFlags = invert4(m.dwFlags);
				dckPtr->lNumFrames	=mptr->dwTotalFrames = invert4(m.dwTotalFrames);
				mptr->dwInitialFrames = invert4(m.dwInitialFrames);
				dckPtr->lNumStreams	= mptr->dwStreams = invert4(m.dwStreams);
				mptr->dwSuggestedBufferSize = invert4(m.dwSuggestedBufferSize);
				
				mptr->dwWidth = invert4(m.dwWidth);
				mptr->dwHeight = invert4(m.dwHeight);
				dckPtr->streamCount = 0;

		    	DIAGNOSTIC(DBG_Printf("Got the MainAVIHeader, total %d, initial %d, numStreams %d\n",
					mptr->dwTotalFrames,mptr->dwInitialFrames,mptr->dwStreams);)

	            if (!(mptr->dwFlags & AVIF_HASINDEX)){ dckPtr->indx = NULL; return 1; }
			}
        	
            return 0;
        case MKFOURCC('s','t','r','h'): /* "strh"  STREAM HEADER*/
			{
				AVIStreamHeader trout;
				HFB_STREAM_X* hptr;
							        		
				duck_memcpy(&trout,data,sizeof(AVIStreamHeader));
				hptr = &(dckPtr->heads[dckPtr->streamCount]);

				/* initialize the child stream pointer to NULL (there are no children yet) */
				hptr->nextStream = (HFB_STREAM*) NULL;

				hptr->scale = trout.dwScale = invert4(trout.dwScale);
				hptr->rate = trout.dwRate = invert4(trout.dwRate);        
				trout.fccHandler = invert4(trout.fccHandler);
				trout.fccType = invert4(trout.fccType);
				hptr->sampleSize = trout.dwSampleSize = invert4(trout.dwSampleSize);
				hptr->lLength = trout.dwLength = invert4(trout.dwLength);
				hptr->streamNum = dckPtr->streamCount;
				hptr->fHnd = dckPtr;

				/* FWG 1-10-2000 */
				hptr->lStart = trout.dwStart = invert4(trout.dwStart);

				if (dckPtr)	hptr->bfHnd = dckPtr->bfHnd;

				if (trout.fccType == MKFOURCC('v','i','d','s')){
#if 0
			        if (
						/* TrueMotion 1.0 (aka "S") */
						(trout.fccHandler != MKFOURCC('D','U','C','K')) && 
			            (trout.fccHandler != MKFOURCC('d','u','c','k')) &&
			            /* dib support */
						(trout.fccHandler != MKFOURCC('D','I','B',' ')) &&
			            /* realtime CAPTURE */
						(trout.fccHandler != MKFOURCC('D','K','R','T')) &&
			            (trout.fccHandler != MKFOURCC('d','k','r','t')) &&
						/* tiger beta (aka realtime COMPRESSION) */
						(trout.fccHandler != MKFOURCC('t','i','g','b')) &&
						(trout.fccHandler != MKFOURCC('T','I','G','B')) &&

						(trout.fccHandler != MKFOURCC('t','m','2','0')) &&
						(trout.fccHandler != MKFOURCC('T','M','2','0')) &&

						(trout.fccHandler != MKFOURCC('t','o','r','q')) &&
						(trout.fccHandler != MKFOURCC('T','O','R','Q')) &&

						(trout.fccHandler != MKFOURCC('T','M','2','X')) &&
						(trout.fccHandler != MKFOURCC('t','m','2','x')) &&

						(trout.fccHandler != MKFOURCC('t','r','2','0')) &&
						(trout.fccHandler != MKFOURCC('T','R','2','0')) )
					{
						return 0;
					}
#endif
					hptr->fccType = trout.fccType;
					hptr->fccHandler = trout.fccHandler;

					hptr->rate = trout.dwRate;
					hptr->scale = trout.dwScale;

					if (trout.dwRate < 1000000)
					{
						 int multi;
						 multi = 1000000/trout.dwRate;
						 trout.dwRate = 1000000;
						 trout.dwScale = multi * trout.dwScale;
					}
								                    
			        if (trout.dwScale > 1) trout.dwScale--;
							                    
			        hptr->lSampleRate = 
			        	dckPtr->GlobalFrameRate = trout.dwRate / trout.dwScale;
								                    
			        if (dckPtr->dfVideoStream == -1)
			        	dckPtr->dfVideoStream = dckPtr->streamCount;
													
					hptr->streamType = DUK_VIDSTREAM;
				}
				else if (trout.fccType == MKFOURCC('a','u','d','s'))
				{
				        if (trout.fccHandler != MKFOURCC('\0','\0','\0','\0'))
				        	return 0;
				        
				        if (dckPtr->dfAudioStream == -1)
				        	dckPtr->dfAudioStream = dckPtr->streamCount;
								
						hptr->streamType = DUK_AUDSTREAM;
				}         
            	dckPtr->streamCount++;
			}             
            return 0;
        case MKFOURCC('s','t','r','f'):
            /*
             "strf" stream format chunk could be for a "vids" or "auds" stream
              assumes that the stream header most recently read references the 
              same stream
            */
            switch(dckPtr->heads[dckPtr->streamCount-1].streamType)
			{
                case DUK_VIDSTREAM:
                	{
	                	BITMAPINFO bmp;
	                	duck_memcpy(&bmp,data,sizeof(BITMAPINFO));
#if defined(DISPLAYDIB)
#if DISPLAYDIB		                
		                globalDIBWidth = invert4(bmp.bmiHeader.biWidth);
                    	globalDIBHeight = invert4(bmp.bmiHeader.biHeight);
						globalIsDIB = (bmp.bmiHeader.biCompression == MKFOURCC(0,0,0,0));					
#endif
#endif
	                    dckPtr->heads[dckPtr->streamCount-1].a.BitmapInfo.usWidth = 
	                    	(unsigned short)invert4(bmp.bmiHeader.biWidth);
	                    dckPtr->heads[dckPtr->streamCount-1].a.BitmapInfo.usHeight = 
	                    	(unsigned short)invert4(bmp.bmiHeader.biHeight);
#if defined(LARGECHUNKS) || 1
	                    dckPtr->heads[dckPtr->streamCount-1].a.BitmapInfo.usDepth = 
	                    	(unsigned short)invert4(bmp.bmiHeader.biBitCount);
#endif
						dckPtr->heads[dckPtr->streamCount-1].a.BitmapInfo.ulFormatTag = 
							(invert4(bmp.bmiHeader.biCompression) != MKFOURCC(0,0,0,0));					
#if defined(LARGECHUNKS) || 1
						dckPtr->heads[dckPtr->streamCount-1].a.BitmapInfo.ulHandler = 
							(invert4(bmp.bmiHeader.biCompression) != MKFOURCC(0,0,0,0)) ? 
							invert4(bmp.bmiHeader.biCompression) : 
							MKFOURCC('D','I','B',' ');
#endif
                    }
                    break;
                case DUK_AUDSTREAM:
		{
                    	DKWAVEFORM dkw,*dptr;
 dbgout("HFB_interpretChunk B\n");
                    	duck_memcpy(&dkw,data,min(size,sizeof(DKWAVEFORM)));
	                    
						dptr = &(dckPtr->heads[dckPtr->streamCount-1].a.WaveformInfo);
	                    
						dptr->nBlockAlign = invert2(dkw.nBlockAlign);
						dptr->wBitsPerSample= invert2(dkw.wBitsPerSample);
						dptr->wFormatTag = invert2(dkw.wFormatTag);
						dptr->nSamplesPerSec = dckPtr->heads[dckPtr->streamCount-1].lSampleRate = 
						invert4(dkw.nSamplesPerSec);
	                    				
						dptr->nChannels = invert2(dkw.nChannels);
						dptr->nAvgBytesPerSec = invert4(dkw.nAvgBytesPerSec);            
						dptr->cbSize = invert2(dkw.cbSize);

 dbgout("HFB_interpretChunk B1\n");
						/* tags 97 and 98 may be voxware or dk3/dk4 */
						if ((dptr->wFormatTag == 98) || (dptr->wFormatTag == 97))
							if (dptr->cbSize != 2) /* cbsize for dk3/dk4 would be 2 */
								dptr->wFormatTag = 117; /* this is NOT dk3/dk4, it's voxware */

 dbgout("HFB_interpretChunk B2\n");
						if (dptr->cbSize)
						{
							if (dptr->wFormatTag == 117) /* for voxware, calculate samplesPerBlock */
								dptr->wSamplesPerBlock = dptr->nBlockAlign * 8;
							else
								dptr->wSamplesPerBlock = invert2(dkw.wSamplesPerBlock);
						}

 dbgint1("HFB_interpretChunk B3: cbSize=",dptr->cbSize);
						/* copy any userdata after wSamplesPerBlock */
						if(dptr->wFormatTag > 1 && dptr->cbSize>0)		///	dbm 3/02 -- PCM has no cbSize param, old struct
																		///	3/26/02 -- added  cbSize thing -- zero is illegal!
							duck_memcpy(dptr->userData,dkw.userData,dptr->cbSize-2);				
				
 dbgout("HFB_interpretChunk C\n");
			break;
		}
		default:
                    break;
            }                                                                            
            return 0;
        case MKFOURCC('i','d','d','x'): /* "iddx" AVI compliant duck pre-index*/
        case MKFOURCC('i','d','x','1'): /* "idx1" AVI FILE INDEX CHUNK*/
       		gooseCD(dckPtr->aviFile,size); /* let the CD start pre-buffering (if applicable)*/
            /*buildIndex(dckPtr->bfHnd,data,size); */
            return 1;                                                            
        case MKFOURCC('s','t','r','n'): /* "strn" STREAM NAME*/
        	duck_memcpy(&(dckPtr->heads[dckPtr->streamCount-1].szName),data,24);
        	return 0;


		/* FWG 9-13-2000 */
		/* This just drops out of interpert chunk while returning avilen. Used in Pseudo_Async mode. */
		/* "JRTR" == Just Return */
		case MKFOURCC('J','R','T','R'):
			return size;
#if 0
        case MKFOURCC('s','t','r','d'): /* "strd" STREAM CODEC STATE INFORMATION*/
        case MKFOURCC('J','U','N','K'):
        case MKFOURCC('C','R','A','P'):
            return 0;

		

        default:
            break;
#endif
    }
    return 0;
}

/* parse out and interpret chunks within a list chunk */

static void getList(HFB_FILE_HANDLE dckPtr, unsigned char *buffer, long length)
{        
    long chunkLength;
    unsigned long chunkType;

    while (length > 8 )
	{
        duck_memcpy(&chunkLength,buffer+4,sizeof(long));
        duck_memcpy(&chunkType,buffer,sizeof(long));
        
        chunkLength = invert4(chunkLength);

        interpretChunk(dckPtr,invert4(chunkType), buffer + 8, chunkLength);
    
        length -= (chunkLength +8);
        buffer += (chunkLength +8);
    }
}

typedef struct tchunk
{
	unsigned long chunkType;
	long chunkLength;
} chunkhead;

static long AVIChunkGet(HFB_FILE_HANDLE dckPtr, DWORD* chunkType, unsigned char* buffer, int bSize)
{   
    long avilen;
    chunkhead chunkHead;
    int res;

	res = duck_read(dckPtr->aviFile, (BYTE *)&chunkHead, sizeof(chunkhead));
	    
    if (res != sizeof(chunkhead)){ if (res < 0) return res; else return 0;}
    
    *chunkType = invert4(chunkHead.chunkType);
    avilen = invert4(chunkHead.chunkLength);
    
    /*          
    // if the chunk is a list, we treat it a little special 
    // (i.e. don't necessarily read the whole thing at once 
    // (unless, of course, it's a "rec " record chunk in which 
    // case it seems like we have to read it as a whole and parse it later))
    */
    if (*chunkType == MKFOURCC('L','I','S','T')){

		res = duck_read(dckPtr->aviFile, (BYTE*)chunkType, sizeof(DWORD));        
        if (res != sizeof(DWORD)){ if (res < 0) return res; else return 0;}
        
        *chunkType = invert4(*chunkType);
	    
	    avilen -= 4;
	    /*  
	    // to use the index, we have to get the position of the movi list
	    // this is because despite what MicroSoft may say in the documentation,
	    // the index entry offsets are from the beginning of the movi list,
	    // not the beginning of the AVI file.
	    */
        if (*chunkType == MKFOURCC('m','o','v','i'))
		{
			dckPtr->header.dwReserved[0] = avilen;

            if (dckPtr->header.dwFlags & AVIF_WASCAPTUREFILE)
				dckPtr->pos = 4;
			else
	            dckPtr->pos = duck_seek(dckPtr->aviFile,0,SEEK_CUR);
            
			 /* adjust to the start of the first data chunk */
			/* dckPtr->pos += 4; */

			/* FWG 6-23-99 In HFBMODE_FLYINDEX we don't need to 
				seek to the end of the file. */ 
			/* Return that we are idx1 Chunk with a false positive
			value for the length. */
			/* FWG 7-7-99 we have two modes that can use Fly-Index. */
			if ((dckPtr->bfHnd->mode == HFBMODE_FLYINDEX) || 
				(dckPtr->bfHnd->mode == HFBMODE_ASYNC_FLYINDEX) )
			{
				*chunkType = MKFOURCC('i','d','x','1');
				return 4;
			}
			/* FWG 6-23-99 */
            

			/* FWG 9-13-2000 */
			if (dckPtr->bfHnd->mode == HFBMODE_PSEUDO_ASYNC)
			{
				/* Just Return */
				*chunkType = MKFOURCC('J','R','T','R');
				return avilen;
			}
			/* FWG 9-13-2000 */

            if (dckPtr->bfHnd->indxCount == 0) 
			{
            	/* if we haven't already got an index, skip past the movi */
				/* list to get the next chunk,  which should be the index */
            	duck_seek(dckPtr->aviFile, avilen, SEEK_CUR);

            	return AVIChunkGet(dckPtr,chunkType, buffer, bSize);
            }
			else
			{
        	   	return 0;
            }
        } 
    }
        
    if (*chunkType == MKFOURCC('i','d','x','1')) return avilen;
    if (*chunkType == MKFOURCC('i','d','d','x')) return avilen;

    if ((*chunkType == MKFOURCC('J','U','N','K')) || (*chunkType == MKFOURCC('C','R','A','P')))
	{
        /*if (avilen > 64*1024) 
        	duck_seek(dckPtr->aviFile, avilen, SEEK_CUR);
        else{
        */
        	res = duck_read(dckPtr->aviFile, buffer, (size_t)avilen);
         	if (res != avilen){ if (res < 0) return res; else return 0;}
    	/*}*/
        return AVIChunkGet(dckPtr,chunkType, buffer, bSize); 
    }                          

    if (avilen > bSize) /* is the buffer big enough? */
    	return 0;
        	
    /* OK, now try to read the data */
    res = duck_read(dckPtr->aviFile, (BYTE *)buffer, (size_t)avilen);
    
    if (res != avilen){ if (res < 0) return res; else return 0;}
    
    return avilen;
}

typedef struct tAVIhd
{
	unsigned long RIFF;
	long fileLength;
	unsigned long AVI;
} avihead;

void lHFB_LoadIndex(HFB_FILE_HANDLE dckPtr, HFB_BUFFER_HANDLE bfHnd)
{
	unsigned long chunkType;
	long avilen;
    void *buffer = (void *)bfHnd->buffer;

	assert((dckPtr != NULL) && (bfHnd != NULL));

	if (!dckPtr && !bfHnd) return;

	if (!dckPtr)
		dckPtr = bfHnd->fHnd;
	else
	{
		if (!bfHnd)
			bfHnd = dckPtr->bfHnd;
	}

	if (!bfHnd || !dckPtr) return;

	dckPtr->bfHnd = bfHnd;
	bfHnd->fHnd = dckPtr;

	if ( (dckPtr->indexSize / sizeof(AVIINDEXENTRY))
		 > (bfHnd->m_indxSize / sizeof(DCKINDEXENTRY)) )
	{
		duck_free(bfHnd->buffer); /* free the old, small buffer */

		bfHnd->m_indxSize = (dckPtr->indexSize / sizeof(AVIINDEXENTRY)) 
							* sizeof(DCKINDEXENTRY);
		bfHnd->rawSize = bfHnd->targetSize + bfHnd->m_indxSize;

		/* allocate the new, larger buffer */
		bfHnd->buffer = (unsigned char*) duck_malloc(bfHnd->rawSize, DMEM_CDBUFF);
	}
	
	/* ((unsigned long)bfHnd->buffer + (bfHnd->minIndx * sizeof(DCKINDEXENTRY))); */

	if (dckPtr->bfHnd->indxCount == 0)
	{
     	buildIndex(dckPtr->bfHnd, buffer, dckPtr->indexSize);
	}
    if (!dckPtr->pos)
	{
	    do {
	        avilen = AVIChunkGet(dckPtr, &chunkType, (unsigned char*) buffer, 16L * 1024L);
	        if (!avilen) break;
	        /* pass the chunk to the interpreter until we get past the index*/
	    }while (!interpretChunk(dckPtr, chunkType, (unsigned char*) buffer, avilen));
	}

	/* 
		added this callculation based on the avi header
		this replaces the manual version done in buildIndex
	 */
	if (!dckPtr->header.dwTotalFrames)
		dckPtr->header.dwTotalFrames = 1;

	dckPtr->bfHnd->recAverage = ROUND_UP2K(dckPtr->header.dwReserved[0] / dckPtr->header.dwTotalFrames );

	dckPtr->bfHnd->maxToRead = dckPtr->bfHnd->recAverage; 
    dckPtr->bfHnd->modToRead = 0;
}

int lHFB_ParseFile(HFB_FILE_HANDLE fHnd, HFB_BUFFER_HANDLE bfHnd)
{
	unsigned long chunkType;
	long avilen;
    void *buffer;
    avihead AVIhead;
    
	assert(fHnd != NULL);

 dbgout("HFB_ParseFile A\n");
	if (!fHnd) return -3;

 dbgout("HFB_ParseFile B\n");
	if (duck_read(fHnd->aviFile, (BYTE*) &AVIhead, sizeof(avihead)) != sizeof(avihead))
		return -3;

 dbgout("HFB_ParseFile C\n");
    if (invert4(AVIhead.RIFF) != MKFOURCC('R','I','F','F'))
		if (invert4(AVIhead.RIFF) != MKFOURCC('D','U','C','K'))
			if (invert4(AVIhead.RIFF) != MKFOURCC('O','N','2',' '))
    			return -5;

 dbgout("HFB_ParseFile D\n");
    if (invert4(AVIhead.AVI) != MKFOURCC('A','V','I',' '))
		if (invert4(AVIhead.AVI) != MKFOURCC('V','P','3',' '))
    		return -7;
    
 dbgout("HFB_ParseFile E\n");
    fHnd->dfAudioStream = fHnd->dfVideoStream = -1;
 dbgout("HFB_ParseFile F\n");
	fHnd->lFileSize = invert4(AVIhead.fileLength);

 dbgout("HFB_ParseFile G\n");
   	if (bfHnd == NULL)
	{
   		buffer = (void *)duck_malloc(16L*1024L,DMEM_TEMP);
	}
   	else
	{
	   	fHnd->bfHnd = bfHnd;
		bfHnd->fHnd = fHnd;
   		buffer = bfHnd->buffer;
   	}
   	if (buffer == NULL)
   		return -8;
	
 dbgout("HFB_ParseFile H\n");
    do {
 dbgout("HFB_ParseFile H1\n");
        avilen = AVIChunkGet(fHnd,&chunkType, (unsigned char *) buffer, 16L*1024L);
        if (avilen <= 0) break;
        /* pass the chunk to the interpreter until we get to the START of the index*/
    }while (!interpretChunk(fHnd,chunkType, (unsigned char *) buffer, avilen));    

    fHnd->indexSize = avilen;


#if 0 /* This has been moved to HFB_LoadIndex (above) */
	/* check to see that there is enough room for the index and the buffer */
	if ( (avilen / sizeof(AVIINDEXENTRY)) > (bfHnd->m_indxSize / sizeof(DCKINDEXENTRY)) )
	{
		duck_free(bfHnd->buffer); /* free the old, small buffer */

		bfHnd->m_indxSize = (avilen/sizeof(AVIINDEXENTRY))*sizeof(DCKINDEXENTRY);
		bfHnd->rawSize = bfHnd->targetSize+bfHnd->m_indxSize;

		/* allocate the new, larger buffer */
		bfHnd->buffer = (unsigned char *)duck_malloc(bfHnd->rawSize,DMEM_CDBUFF);
	}
#endif


 dbgout("HFB_ParseFile I\n");
   	if (bfHnd == NULL) 
		duck_free(buffer);
	else if (!bfHnd->buffer)
		return -1; /* error, no buffer allocated */

   	if (avilen < 0)
   		return avilen;
   	else
		return 0;
}

void AVIFileClose(HFB_FILE_HANDLE dckPtr)
{   
    assert(dckPtr != NULL);

	if(dckPtr != NULL)
	{
		duck_close(dckPtr->aviFile);
		dckPtr->aviFile = 0;
        if(dckPtr->indx != NULL) 
			duck_free(dckPtr->indx);                            
        if(dckPtr->findx != NULL) 
			duck_free(dckPtr->findx);                            
    }
}
