/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************\
*                                                                               *
* hfb_buff.c -   multi-platform AVI Streaming library                           *
*               Copyright (c) 1994-99, On2.com All rights reserved.             * 
*
*                 Edited by:    Eric Ameres                                     *
*                 Version:      1.4
*                 Edited:       09/28/99
                    - added stronger parameter checking

*                 Edited by:    Eric Ameres                                     *
*                 Version:      .97                                             *
*                 Edited:       01/25/94                                        *
					- added adaptive datarate fill buffer routine
					- removed unnecessary fillbuffering
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dkpltfrm.h"
#include "duck_mem.h"
#include "duck_io.h"

#include "duckwalk.h"
#include "hfb_buff.h"

#ifndef max         /* ?? NOMINMAX apparently defined in Windows.h (watcom\h\win\win16.h*/
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

/*************************************************************************
	local defines
*************************************************************************/

#define NORMALIZE(x)    ((x) % dckPtr->movieSize)
#define FIXOFFSET(x)    ((x) + dckPtr->movieOffset)

/* all reads are in blocks of multiple sectors (a sector is usually 2K)*/

#define MAX_DATA_RATE 300

/*************************************************************************
	function prototypes
*************************************************************************/
static int fillBufferDirect(register HFB_BUFFER_HANDLE dckPtr, int maxToRead);
int chunkInReferencedStream(HFB_BUFFER_HANDLE bufferHandle, int index);
//int duck_readFinished(int, int); /* FWG 7-9-99 */
//int duck_readFinished(int temp1, int temp2) {return 0;};

/*************************************************************************
	functions   
*************************************************************************/

static void hfb_reclaim(HFB_BUFFER_HANDLE dckPtr)
{
	register DCKINDEXENTRY* lindx = (DCKINDEXENTRY*) NULL;
	
	if (!dckPtr) return;

	/* make sure we don't start past the end of the file */
	if (dckPtr->lastReleased < dckPtr->indxCount)
	{
		/* start looking for chunks to free at the last released chunk */
		lindx = &(dckPtr->indx[dckPtr->lastReleased]); 

		/* keep looking through chunks marked for release until we find one that isn't */
		while( ((lindx->dckFlags & DCKCHUNK_RELEASED) == DCKCHUNK_RELEASED)
			   || (chunkInReferencedStream(dckPtr, dckPtr->lastReleased) == 0) )
		{
			++dckPtr->lastReleased;
			/* make sure we don't skip past the end of the file */
			if (dckPtr->lastReleased >= dckPtr->indxCount)
			{
				break;
			}
			/* try the next chunk */
			lindx++;        
		}
	}

	dckPtr->readIndex = dckPtr->releaseOffset + 
		dckPtr->indx[dckPtr->lastReleased].dwChunkOffset - 16;
}


int chunkInReferencedStream(HFB_BUFFER_HANDLE bufferHandle, int indexNumber)
{
	HFB_FILE_HANDLE fileHandle         = (HFB_FILE_HANDLE) NULL;
	HFB_STREAM_HANDLE_X parentStreamHandle   = (HFB_STREAM_HANDLE_X)   NULL;
	register DCKINDEXENTRY* indexEntry = (DCKINDEXENTRY*)  NULL;

	/* make sure we were given a valid buffer handle */
	if (!bufferHandle) return 0;

	/* make sure this buffer is associated with a file */
	fileHandle = bufferHandle->fHnd;
	if (!fileHandle) return 0;

	/* make sure we don't start past the end of the file */
	if (indexNumber < bufferHandle->indxCount)
	{
		/* we can find the stream number by looking at the chunk's index entry */
		indexEntry = &(bufferHandle->indx[indexNumber]); 
		/* find out if the parent of this stream is referenced */
		parentStreamHandle = &(fileHandle->heads[indexEntry->dckID]);
		if (parentStreamHandle->numRef > 0)
		{
			/* this index is in a stream that is referenced at least once */
			return 1;
		}
	}
	/* This index is not in a referenced stream */
	return 0;
}


/* returns percent of CD buffer filled with data */
long lHFB_GetBufferPerCentFull(HFB_BUFFER_HANDLE dckPtr)
{
	if (!dckPtr) return 0;

	hfb_reclaim(dckPtr);

	if (dckPtr->movieSize <= 0)
		return 0;

	return(((dckPtr->lastByteOffset-dckPtr->readIndex)*100)/dckPtr->movieSize);
}

long lHFB_GetmovieSize(HFB_BUFFER_HANDLE dckPtr)
{
	if (!dckPtr) return -1;
	return (dckPtr->movieSize);
}

/* compute maxToRead and modToRead based on datarate of i/o device */
static void computeMaxToRead(HFB_BUFFER_HANDLE dckPtr,int deviceRate)
{
	/* make sure we were passed a valid buffer handle */
	if (!dckPtr) return;
	if (!dckPtr->fHnd) return;

	/* deviceRate is only used if HW_CD_BUFFER is defined  */
	/* This will get rid of the compiler warning otherwise */
	(void) deviceRate;

	dckPtr->maxToRead = ROUND_UP2K(dckPtr->recAverage);
	dckPtr->modToRead = 0;
    
#if defined(HW_CD_BUFFER)
#if HW_CD_BUFFER
	{
		int framesPerSecond = 0;
		long fileRate 0L;

		/* compute frames per second */
		framesPerSecond = 1000000 / (dckPtr->fHnd->header.dwMicroSecPerFrame - 1);

		fileRate = (framesPerSecond * dckPtr->maxToRead)/1024;
		/* use the device's approximate datarate */

		if (fileRate < deviceRate)
		{
			/* get a coarse value */
			dckPtr->maxToRead = ROUND_DOWN2K((deviceRate*1024)/framesPerSecond);
			fileRate = (framesPerSecond * dckPtr->maxToRead)/1024;

			/* modToRead fine tunes the reads to be closer to the data rate */
			if (deviceRate > fileRate)
				dckPtr->modToRead = (framesPerSecond/((deviceRate-fileRate)/2))-1;
		}
	}
#endif
#endif
}

#define offsetRoundedUp (offsetRoundedDown + QVAL)

static void firstRead(register HFB_BUFFER_HANDLE dckPtr, int frame, unsigned long forceOffset)
{
	register DCKINDEXENTRY* lindx;
	int offsetRoundedDown, i;

	if (!dckPtr) return;

	(void)forceOffset;

	/* clear out the hardware buffering tables */
	dckPtr->subSec = dckPtr->readTotal = dckPtr->extrabuff = 0;    
	for(i = 0; i < 60; i++) dckPtr->readTable[i] = 0;

	/* first, lets get the first chunk from the index */
	dckPtr->nextIndex = frame;
	lindx = &(dckPtr->indx[dckPtr->nextIndex]);
	
	/* now figure out where it is in the file and seek to it */
	/* 
		We are actually seeking to the 2K boundary before the first
		data record specified 
	*/

	offsetRoundedDown = ROUND_DOWN2K(dckPtr->fHnd->pos+lindx->dwChunkOffset);

	duck_seek(dckPtr->fHnd->aviFile, offsetRoundedDown, SEEK_SET);

	dckPtr->releaseOffset = dckPtr->movieOffset = dckPtr->fHnd->pos + 4;

	dckPtr->lastByteOffset = offsetRoundedDown;

	dckPtr->nextRead = NORMALIZE(dckPtr->lastByteOffset);   

	/* adjust for high reads */
	if ((NORMALIZE(FIXOFFSET(lindx->dwChunkOffset)) + lindx->dwChunkLength) > dckPtr->movieSize)
	{
		if (dckPtr->nextRead  < NORMALIZE(dckPtr->lastByteOffset))
			dckPtr->nextRead += dckPtr->movieSize;
	}
}

void lHFB_ResetStreams(HFB_BUFFER_HANDLE bufferHandle);

void lHFB_InitBuffer(register HFB_BUFFER_HANDLE dckPtr, HFB_FILE_HANDLE fHnd, int frame,int preload)
{
	register DCKINDEXENTRY *lindx;
	register int i;
	int mFrame = -1;

	if (!dckPtr && !fHnd) return;

	if (!dckPtr)
	{
		dckPtr = fHnd->bfHnd;
	}
	else
	{
		if (!fHnd)
			fHnd = dckPtr->fHnd;
	}

	if (!fHnd || !dckPtr) return;

	dckPtr->fHnd = fHnd;
	fHnd->bfHnd = dckPtr;

	dckPtr->bytesReadLast = 0;
	dckPtr->bfrSize = dckPtr->rawSize;
	dckPtr->lastReleased = 0;
	
	dckPtr->frameCount = dckPtr->fHnd->lNumFrames - frame;

	/* calculate the size and address of the movie based on the space used by the index */
	dckPtr->movie = dckPtr->buffer;
	
	if (dckPtr->minIndx == 0) 
		dckPtr->minIndx = dckPtr->indxCount;

	/* FWG 6-23-99 */
	/* FWG 7-7-99 we have two modes that can use Fly-Index. */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
		(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
	{
		int iCount, tempStreams, temp2XVideo;

		/* This should be set to 0. We need this to happen in case we
		call InitBuffer() more than once in a row without calling HFB_OpenFile() inbetween. */
		dckPtr->indxCount = 0;

		/* We will have more indexs than we will need. but I don't
		know how to compute how many DCKINDX's we will need from 
		indexSize. */
		/* FWG 6-23-99 I don't read the indexSize anymore I will compute DCKINDX's
		by dckPtr->fHnd->lNumFrames * dckPtr->fHnd->lNumStreams */

		//dckPtr->fly_aviRecWrapper.dwFlags = 4; /* setup for first time adding to lindxOffset */
		dckPtr->fly_bAviRecWrapperSet = 0;	/* We haven't parsed any rec chunks yet */
		dckPtr->fly_frameTotal = 0;			/* total bytes of data processed */
		dckPtr->fly_videoFrameCount = 0;	/* total number of indexes to video frames */
	
		/* finding total number of indexes is such a pain becuase AVI 
		files can lie to you. There not very nice. FWG 6-23-99 */
		dckPtr->fly_aviIndxCount = 0;
		tempStreams = 0;
		temp2XVideo = 0;
		for (iCount=0; iCount<dckPtr->fHnd->lNumStreams; iCount++)
		{
			tempStreams += dckPtr->fHnd->heads[iCount].lLength;

			if (dckPtr->fHnd->heads[iCount].streamType == DUK_VIDSTREAM)
			{
				temp2XVideo = dckPtr->fHnd->heads[iCount].lLength * dckPtr->fHnd->lNumStreams;
			}
		}

		/* take lesser of values */
		if (tempStreams < temp2XVideo)
		{
			dckPtr->fly_aviIndxCount = tempStreams;
		}
		else
		{
			dckPtr->fly_aviIndxCount = temp2XVideo;
		}

		dckPtr->movie += (dckPtr->fly_aviIndxCount) * sizeof(DCKINDEXENTRY);	/* move past the index */
		dckPtr->bfrSize -= (dckPtr->fly_aviIndxCount) * sizeof(DCKINDEXENTRY);	/* subtract index from size */
	}
	else
	{
		dckPtr->movie += (dckPtr->minIndx * sizeof(DCKINDEXENTRY)); /* move past the index */
		dckPtr->bfrSize -= (dckPtr->minIndx * sizeof(DCKINDEXENTRY)); /* subtract index from size */
	}

	/* align the buffer address on a 32 byte boundary */
	dckPtr->bfrSize -= 32;
	dckPtr->movie = (unsigned char *)(((unsigned long)(dckPtr->movie)+31) & 0xffffffe0);

	/* Since I don't know the maximum Chunk Size, I have to guess */
	/* FWG 6-23-99 */
	/* FWG 7-7-99 we have two modes that can use Fly-Index. */
	if ( ((dckPtr->mode == HFBMODE_FLYINDEX) || 
		 (dckPtr->mode == HFBMODE_ASYNC_FLYINDEX)) &&
		 (dckPtr->maxChunkSize == 0) )
	{
		int tempMaxChunkSize;

		/* Get average frame size to guess at maximum chunk size */
		if (dckPtr->fHnd->lNumStreams == 2)
		{
			/* assuming audio & video */
			int totalSize;
			totalSize = dckPtr->fHnd->lFileSize;
			totalSize -= (dckPtr->fHnd->lNumFrames * 0x800); /* get rid of audio chunks */
			tempMaxChunkSize =  totalSize / dckPtr->fHnd->lNumFrames;
		}
		else
		{
			tempMaxChunkSize = dckPtr->fHnd->lFileSize / (dckPtr->fHnd->lNumStreams * dckPtr->fHnd->lNumFrames);
		}
		
		/* Since I don't know the maximum Chunk Size, I have to guess */
		tempMaxChunkSize *= 8;

		/* FWG 6-25-99 */
		/* videdit does not have any data restrictions on Key Frames */
		/* The largest frame will not be bigger than width * height * bitDepth. Jimb said */
		if ( (dckPtr->fHnd->header.dwWidth * dckPtr->fHnd->header.dwHeight * 2) < tempMaxChunkSize)
		{
			dckPtr->maxChunkSize = dckPtr->fHnd->header.dwWidth * dckPtr->fHnd->header.dwHeight * 2;
		}
		else
		{
			dckPtr->maxChunkSize = tempMaxChunkSize; 
		}

		/* FWG 6-25-99 */
		/* IF this happens then it is not good */
		if (dckPtr->maxChunkSize > dckPtr->rawSize)
		{
			/* we need at least this much for buffer lets hope it works */
			dckPtr->maxChunkSize = (dckPtr->rawSize/2) - 0x800;
		}
	}

	/* shrink the movie to allow for the largest frame to hang over */
	dckPtr->movieSize = (dckPtr->bfrSize & 0xffffffe0) - (ROUND_UP2K(dckPtr->maxChunkSize) + 2048);


	/* if frame == 0, reset all index flags to not read and not released*/

	if (!frame){
		lindx = &(dckPtr->indx[0]);
		for(i=0; i < fHnd->lNumStreams; i++){
			fHnd->heads[i].lastSample = -1;
			fHnd->heads[i].lastFrame = -1;
		}
		
		/* FWG 6-23-99 */
		/* FWG 7-7-99 we have two modes that can use Fly-Index. */
		if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
			(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
		{
			/* This should work in special Fly index mode and regular mode but
			I might as well use the preprocessor defines like everywhere else.
			In FI mode zero out all of our DCKINDEXENTRY. */
			/* FWG 5-3-99 */
			if (dckPtr->indxCount)
			{
				for(i=0; i < dckPtr->indxCount; i++,lindx++)
					lindx->dckFlags &= ~(DCKCHUNK_READ|DCKCHUNK_RELEASED);  
			}
			else
			{
				duck_memset(lindx,0,(dckPtr->fly_aviIndxCount) * sizeof(DCKINDEXENTRY));
			}
			/* FWG 5-3-99 */	
		}
		else
		{
			for(i=0; i < dckPtr->indxCount; i++,lindx++)
				lindx->dckFlags &= ~(DCKCHUNK_READ|DCKCHUNK_RELEASED); 
		}

	}
	else
	{
		register int nStream;                        
		register DCKINDEXENTRY *lindx; 
		register HFB_STREAM_HANDLE_X strm = fHnd->heads;
		int counts[32],bcounts[32];

		duck_memset(counts,0,sizeof(counts));
		duck_memset(bcounts,0,sizeof(bcounts));
		/***** first thing to be done, reset last samples ****/
		/* these numbers are actually local since stream objects
		   contain an independent last gotten sample,  the last 
		   samples used here are mostly for local use  */
		
		for(i = 0; i < fHnd->lNumStreams; i++){
		    strm[i].lastSample = -1;
			strm[i].lastFrame = frame-1;
			
		    if (strm[i].streamType == DUK_AUDSTREAM){
				if (fHnd->dfVideoStream > -1)
					counts[i] = (int)(((((double)frame * (double)strm[i].lSampleRate)*
					(double)strm[fHnd->dfVideoStream].scale)
					/(double)strm[fHnd->dfVideoStream].rate));
				else
					counts[i] = frame;
					
				if (strm[i].a.WaveformInfo.wFormatTag != 1) {
 dbgint1("wSamplesPerBlock = ",strm[i].a.WaveformInfo.wSamplesPerBlock);
					if(strm[i].a.WaveformInfo.wSamplesPerBlock)					///	dbm -- 3/27/02 stupidity protection
						counts[i] /= strm[i].a.WaveformInfo.wSamplesPerBlock;
					else {
						strm[i].a.WaveformInfo.wSamplesPerBlock = 200;
						counts[i] /= strm[i].a.WaveformInfo.wSamplesPerBlock;						
					}
				}
		    }
		}
	    /**************************************************/
	    /* now figure out where we wind up, frame wise */
	    lindx = &(dckPtr->indx[0]);
													 
	    for(i=0; i < dckPtr->indxCount; i++,lindx++){       
		    lindx->dckFlags &= 0xf8;       /* reset released and read flags */
			
			nStream = lindx->dckID;
			
			if (strm[nStream].lastSample == -1){
				if (strm[nStream].streamType == DUK_AUDSTREAM){
					if (bcounts[nStream] >= counts[nStream])
						continue;
					
					if (strm[nStream].sampleSize)
						bcounts[nStream] += lindx->dwChunkLength/strm[nStream].sampleSize;
					else
						bcounts[nStream] ++;

					if (bcounts[nStream] >= counts[nStream]){
						strm[nStream].lastSample = i -1;
						continue;
					}
					lindx->dckFlags |= DCKCHUNK_READ|DCKCHUNK_RELEASED;
				}else{
					lindx->dckFlags |= DCKCHUNK_READ|DCKCHUNK_RELEASED;
						if (++counts[nStream] >= frame){
							strm[nStream].lastSample = i;
							continue;
					}
				}
			}
		}
		
		frame = dckPtr->indxCount;
		mFrame = -1;
			
		for(i = 0; i < fHnd->lNumStreams; i++){
		    if (strm[i].lastSample > -1){
				frame = min(frame,strm[i].lastSample);
				mFrame = max(mFrame,strm[i].lastSample);
			}else
				frame = 0;
		}
    }
	
	/* reset the size of the CD hardware buffer */
	dckPtr->hwBuffer = HW_CD_BUFFER<<10;

	firstRead(dckPtr,frame,0);	/* kick off the first seek */

	lHFB_ResetStreams(dckPtr);	/* reset the streams */

	if (preload == -3) return;

	/* fill the buffer as necessary */    
	DIAGNOSTIC(printf("filling buffer, nextRead %d\n",dckPtr->nextRead);)
	{
		int amtRead = 0;

		switch (preload){
			case -1:
				preload = (dckPtr->movieSize*3)/4;
				break;
			case -2:
				preload = dckPtr->movieSize-QVAL;
				break;
		}
		preload = ROUND_DOWN2K(preload);
		
		DIAGNOSTIC(printf("Filling MovieSize %d bufferSize %d preload %d...\n",dckPtr->movieSize,dckPtr->bfrSize,preload);)
		 
			if (preload != 0){
				amtRead = fillBufferDirect(dckPtr,preload);

				/* FWG 7-7-99 There are two different async modes. */
				if ((dckPtr->mode == HFBMODE_ASYNC) || 
					(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
				{
					/* FWG 7-7-99 Tell duck_io.c to block until the async read is finished. */
					duck_readFinished(dckPtr->fHnd->aviFile, 1);
				}

				fillBufferDirect(dckPtr,0); /* once to process the data */
			}
		
		computeMaxToRead(dckPtr,MAX_DATA_RATE);
	}
    
	/* if multiple streams, clean up any skewed chunks */
    if (mFrame > 0){
		if (frame < 0) frame = 0;

		lindx = &(dckPtr->indx[frame]);
			
	    for(i=frame; i <= mFrame; i++,lindx++)
			if (i <= fHnd->heads[lindx->dckID].lastSample)
				lindx->dckFlags |= DCKCHUNK_READ|DCKCHUNK_RELEASED;
    }

	DIAGNOSTIC(printf("finished filling to %d\n",dckPtr->nextIndex);)
}

static int fillBufferDirect(register HFB_BUFFER_HANDLE dckPtr, int maxToRead)
{
	unsigned long lastEnd;
	long bytesToRead;
	register DCKINDEXENTRY *lindx;
	
	/* Used by FLY_INDEX FWG 7-7-99 */
	int dfVideoStream;
	
	if (!dckPtr) return -1;

	hfb_reclaim(dckPtr);

	/* Used by FLY_INDEX FWG 7-7-99 */
	dfVideoStream = dckPtr->fHnd->dfVideoStream;

	if (dckPtr->delay){
		return 0;
	}
	
	/**************** figure maxToRead *********************/
	if (maxToRead < 2048){
		if (maxToRead != HFB_USE_DEFAULT) return 0;
		maxToRead = dckPtr->maxToRead;
		
		if (dckPtr->modToRead){
			if (dckPtr->frameToggle++ >= dckPtr->modToRead){
				maxToRead += QVAL;
				dckPtr->frameToggle = 0;
			}
		}
	}
	else if(maxToRead & 0x400)
	{
       	static int flop = 0;
		maxToRead -= 1024;
        if (dckPtr->frameToggle++ & 1)
			maxToRead += 1024;
		else maxToRead -= 1024;
    }

	/* ela - 10/09/2000 : temp fix to avoid reading past end of file */
	/* bfs - 12/06/2000 : modified to handle Fly Index modes */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || (dckPtr->mode == HFBMODE_ASYNC_FLYINDEX))
	{
		if (dckPtr->nextIndex >= dckPtr->fly_aviIndxCount)
		{
			return 0;
		}
	}
	else if (dckPtr->nextIndex >= dckPtr->indxCount)
	{
		return 0;
	}

	if (bytesToRead = dckPtr->bytesReadLast)
	{
		int localIndexCount;

		/* FWG 5-3-99 */
		int lindxOffset;
		AVIINDEXENTRY *tempAviIndx, tAviIndx;
		DCKINDEXENTRY oldLindx; /* FWG 7-6-99 */

		/* 10-4-2000 */
		dckPtr->bytesReadLast = 0;

		/* FWG 6-23-99 */
		/* FWG 7-7-99 we have two modes that can use Fly-Index. */
		if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
			(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
		{
			tempAviIndx = &tAviIndx;
			
			/* We haven't read through any chunks yet. */
			if (dckPtr->nextIndex == 0)
			{
				lindxOffset = 0;
			}
		}
		/* FWG 5-3-99 */

		/* add in the new bytes read to the last byte offset and amount read */
		dckPtr->lastByteOffset += bytesToRead;
			
		if ((dckPtr->nextRead + bytesToRead) > dckPtr->movieSize)
			lastEnd = dckPtr->lastByteOffset - NORMALIZE(dckPtr->lastByteOffset);
		else
			lastEnd = 0;
		
		dckPtr->nextRead = NORMALIZE(dckPtr->lastByteOffset);
		
		/* FWG 6-23-99 */
		/* FWG 7-7-99 we have two modes that can use Fly-Index. */
		if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
			(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
		{
			/* FWG 7-6-99 */
			oldLindx.dwChunkOffset = 0;
			oldLindx.dwChunkLength = 0;
			localIndexCount = dckPtr->fly_aviIndxCount;
		}
		else
		{
			localIndexCount = dckPtr->indxCount;
		}

		/* FWG 6-23-99 */
		for (lindx = &(dckPtr->indx[dckPtr->nextIndex]); (unsigned long)lindx <= (unsigned long)&(dckPtr->indx[localIndexCount]) ;  lindx++)
		{
			int extraHeaderInfo; /* FWG 6-23-99 */
					
			/*lindx->dckFlags &= 0xf8;*/       /* reset released and read flags */
			
			/* FWG 5-3-99 */
			/* FWG 7-7-99 we have two modes that can use Fly-Index. */
			if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
				(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
			{
				/* If we are not on a good chunk we are on chunks 12 bytes long "LIST    rec " */
				if ( (lindx->dwChunkOffset == 0) && (lindx->dwChunkLength == 0) )
				{
					unsigned char * chunkBegin;
					
					/* FWG 7-6-99 */
					if ( (oldLindx.dwChunkOffset != 0) && (oldLindx.dwChunkLength != 0) )
					{
						/* FWG 6-25-99 */
						/* add four becasue during memcpy we dec by 4 */
						lindxOffset = oldLindx.dwChunkOffset + oldLindx.dwChunkLength + (oldLindx.dwChunkLength % 2) + 4;
					}
					else if ( (oldLindx.dwChunkOffset != 0) && (oldLindx.dwChunkLength == 0) )
					{
						/* FWG 8-14-2000 */
						/* Adds support for NULL frames. */
						lindxOffset = oldLindx.dwChunkOffset + 4;
					}

					/* FWG 6-23-99 */
					/* This is probably redundant */
					if (dckPtr->fly_aviIndxCount == dckPtr->indxCount)
					{
						/* we reached the end of the data */
						dckPtr->fly_aviIndxCount = dckPtr->indxCount;
						
						/* Inc nextIndex past indexCount so we don't come into this loop anymore */
						dckPtr->nextIndex++;
						return 0;
					}
					
					do {

						if (lastEnd  && (FIXOFFSET(lindxOffset) >= lastEnd))
						{
							/* I have to do this because the next chunk header I have to 
							parse is in the buffer hangover. */
							//chunkBegin = dckPtr->movie + dckPtr->movieOffset + lindxOffset;
							chunkBegin = dckPtr->movie + (NORMALIZE(FIXOFFSET(lindxOffset))) + dckPtr->movieSize; 
						}
						else
						{
							//chunkBegin = dckPtr->movie + dckPtr->movieOffset + lindxOffset;
							chunkBegin = dckPtr->movie + (NORMALIZE(FIXOFFSET(lindxOffset))); 
						}
						
						
						/* dest, src, count */
						memcpy(tempAviIndx, chunkBegin-4, sizeof(AVIINDEXENTRY));
						
					
						/* FWG 6-24-99 */
						/* (tempAviIndx->dwFlags == "rec ") */
						if (invert4(tempAviIndx->dwChunkOffset) == 0x20636572)
						{
							if (dckPtr->fly_bAviRecWrapperSet)
							{
								/* The size in bytes of empty rec header chunks */
								lindxOffset = dckPtr->fly_oldLindxOffset + invert4(dckPtr->fly_aviRecWrapper.dwFlags) + 8;
								
								/* dest, src, count */
								dckPtr->fly_bAviRecWrapperSet = 0;
							}
							else
							{
								/* dest, src, count */
								memcpy(&dckPtr->fly_aviRecWrapper, tempAviIndx, sizeof(AVIINDEXENTRY));
								dckPtr->fly_bAviRecWrapperSet = 1;
								
								/* The first time should have dckPtr->fly_aviRecWrapper.dwFlags = 0 */
								dckPtr->fly_oldLindxOffset = lindxOffset;
								//lindxOffset += 8; 
								lindxOffset += 12; 
							}
							/* This needs to be in whatever endian the machine is for the macros lowByte, etc. */
							dckPtr->fly_chunkType = tempAviIndx->dwChunkOffset; 
						}
						else if (duck_isdigit(lowByte(tempAviIndx->ckid)))
						{
							/* endian swap for offset and length if necessary */
							/*
							lindx->dwChunkOffset = invert4(tempAviIndx->dwChunkOffset);
							lindx->dwChunkLength = invert4(tempAviIndx->dwChunkLength);
							*/
							/* skip over Chunk Offset */
							lindx->dwChunkLength = invert4(tempAviIndx->dwFlags);
							//lindx->dwChunkLength = invert4(tempAviIndx->dwChunkLength);
							
							/* Add 4 bytes to skip record type */ 
							if (lindx->dwChunkOffset == 0)
							{
								/* not set by rec wrapper */
								lindx->dwChunkOffset = lindxOffset + 4;
							}
							
							if (lindx->dwChunkLength > dckPtr->maxChunkSize) 
								dckPtr->maxChunkSize = lindx->dwChunkLength;
							
							/* generate a integer stream number from the chunkId */
							lindx->dckID = ((lowByte(tempAviIndx->ckid)-'0')*10) + (mid1Byte(tempAviIndx->ckid)-'0');

							/* we are of type 01wb or 00dc */
							dckPtr->fly_chunkType = tempAviIndx->ckid;
#ifdef LIMITREADS
#if 1
							/* Do I need these? FWG 5-3-99 */
							dckPtr->fly_frameTotal += lindx->dwChunkLength;	/* keep a running size for the movie */
							
							/* keep track of the number of frames */
							//if (dfVideoStream == lindx->dckID) frameCount++;
							if (dfVideoStream == lindx->dckID) dckPtr->fly_videoFrameCount++;
#endif
#endif
						}
						else
						{
							/* We hit a chunk that we do not know how to decode skip over it for now */
							lindx->dwChunkLength = invert4(tempAviIndx->dwFlags);
							
							lindx->dwChunkOffset = lindxOffset + 4;
							
							dckPtr->fly_chunkType = tempAviIndx->ckid;
						}
					/* probably should use a MMFOURCC macro here */
					} while (invert4(dckPtr->fly_chunkType) == 0x20636572); /* "rec " */

					/*lindx++;*/ /* update the temporary index pointer, done in for loop */
					if (duck_isdigit(lowByte(dckPtr->fly_chunkType)))
					{
						dckPtr->indxCount++;
					}
				}
			} /* (dckPtr->mode == HFBMODE_FLYINDEX) FWG 6-23-99 */
			/* FWG 5-3-99 */

			if (FIXOFFSET(lindx->dwChunkOffset) > dckPtr->lastByteOffset)
				break; /* this chunk not touched yet*/

			/******** complete chunks ********/
			/* We want to make sure that we have at least this chunk and the header 
			of another chunk in buffer before we process it. Got an error when I tried
			to process header chunk which only half was there. */
			/* FWG 6-23-99 */
			/* We want to subtract the size of a index in the data so we wont run into the problem
			of processing and incomplete index chunk. This means we will put off processing this chunk
			till next call to fill_buffer. number to subtract 12(size of index chunk) + 8() + 2(to acount for
			odd data chunks)*/
			/* FWG 7-7-99 we have two modes that can use Fly-Index. */
			if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
				(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
			{
				extraHeaderInfo = 22;
			}
			else
			{
				/* we already know where all the indexes end and begin. */
				extraHeaderInfo = 0;
			}

			if ((FIXOFFSET(lindx->dwChunkOffset) + lindx->dwChunkLength) < (dckPtr->lastByteOffset-extraHeaderInfo))
			{
				/* FWG 7-7-99 we have two modes that can use Fly-Index. */
				if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
					(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
				{
					if (duck_isdigit(lowByte(dckPtr->fly_chunkType)))
					{
						/* Chunk is of type audio or video. */
						
						if (lindx->dckID == dckPtr->fHnd->dfVideoStream){
							
							int newMax;
							
							if (dckPtr->frameCount == 0) 
								dckPtr->frameCount = 1;
							
							newMax = ROUND_UP2K((dckPtr->fHnd->lFileSize - lindx->dwChunkOffset)
								/dckPtr->frameCount--);
							
							if (newMax > dckPtr->maxToRead)
								dckPtr->maxToRead = newMax;
						}
						
						lindx->dckFlags |= DCKCHUNK_READ; 
						if (lastEnd && (FIXOFFSET(lindx->dwChunkOffset) >= lastEnd)) 
							lindx->dckFlags |= DCKCHUNK_UPPER;
						
						
						dckPtr->nextIndex++;
						
						/* FWG 6-23-99 */
						oldLindx.dwChunkOffset = lindx->dwChunkOffset;
						oldLindx.dwChunkLength = lindx->dwChunkLength;
						/* FWG 5-3-99 */
					}
					else
					{
						/* This chunk was a chunk of a type we didn't know how to handle.
						Do not put this into our index. In the future this will have to be
						changed so HFB can handle user defined chunks. ie 3D data. FWG 7-7-99 */
						oldLindx.dwChunkOffset = lindx->dwChunkOffset;
						oldLindx.dwChunkLength = lindx->dwChunkLength;
						lindx->dwChunkLength = 0;	
						lindx->dwChunkOffset = 0;
						lindx--;
					}
				}
				else
				{
					/* NORMAL MODE */
					if (lindx->dckID == dckPtr->fHnd->dfVideoStream){
						
						int newMax;
						
						if (dckPtr->frameCount == 0) 
							dckPtr->frameCount = 1;
						
						newMax = ROUND_UP2K((dckPtr->fHnd->lFileSize - lindx->dwChunkOffset)
							/dckPtr->frameCount--);
						
						if (newMax > dckPtr->maxToRead)
							dckPtr->maxToRead = newMax;
					}
					
					lindx->dckFlags |= DCKCHUNK_READ; 
					if (lastEnd && (FIXOFFSET(lindx->dwChunkOffset) >= lastEnd)) 
						lindx->dckFlags |= DCKCHUNK_UPPER;
					
					
					dckPtr->nextIndex++;
				}
				
				
				/*
				if (!(dckPtr->fHnd->heads[lindx->dckID].numRef))
					lindx->dckFlags |= DCKCHUNK_RELEASED;
				*/
				continue;
			}else{
				
				/******** partial chunks ********/ 
				if ((NORMALIZE(FIXOFFSET(lindx->dwChunkOffset)) + lindx->dwChunkLength) >= dckPtr->movieSize) {
					if ((NORMALIZE(FIXOFFSET(lindx->dwChunkOffset)) + 
						(dckPtr->lastByteOffset - FIXOFFSET(lindx->dwChunkOffset)))  >= dckPtr->movieSize){
						dckPtr->nextRead += dckPtr->movieSize;
					}
					break;
				}
				
				if (lastEnd  && (FIXOFFSET(lindx->dwChunkOffset) >= lastEnd)){
					/* chunks that started outside the buffer and should be moved down */
					duck_memcpy(dckPtr->movie + NORMALIZE(FIXOFFSET(lindx->dwChunkOffset))-8,
						dckPtr->movie + NORMALIZE(FIXOFFSET(lindx->dwChunkOffset)) + dckPtr->movieSize-8,
						dckPtr->lastByteOffset - FIXOFFSET(lindx->dwChunkOffset)+8); 
					break;
				}
				
				/* all other partial chunks falling entirely within buffer but not yet finished */
				break;
			}
		}
		/* if we've hit the end of the buffer, 
		check the buffer mode to see if we want to loop, preload, etc... */
		if (dckPtr->nextIndex >= dckPtr->indxCount)
		{
			/* FWG 7-7-99 We only want to do this when lastChunkType == audio or
			video. In HFB_normal mode we don't have to worry about this. 7-8-99 */
			/* FWG 7-7-99 we have two modes that can use Fly-Index. */
			if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
				(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
			{
				if (duck_isdigit(lowByte(dckPtr->fly_chunkType)))
				{
					/* FWG 7-8-99 Right now if were in HFBMODE_FLYINDEX then we can't
					be in HFBMODE_FORWARDLOOP. So don't bother checking. */
					return 0;
				}
			}
			else
			{
				if (dckPtr->mode == HFBMODE_FORWARDLOOP)
				{
					int offsetRoundedDown;
					
					/* now figure out where the beginning of the file is, and seek to it*/
					offsetRoundedDown = ROUND_DOWN2K(dckPtr->fHnd->pos+dckPtr->indx[0].dwChunkOffset);
					
					duck_seek(dckPtr->fHnd->aviFile, offsetRoundedDown, SEEK_SET);
					dckPtr->delay = 1;
				}
				return 0;
			}
		}
	} /* if (bytesToRead = dckPtr->bytesReadLast) */

	/* FWG 6-23-99 */
	/* Recompute the maximum length to read in one call. */
	/* FWG 7-7-99 we have two modes that can use Fly-Index. */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
		(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
	{
		if (dckPtr->fly_videoFrameCount > 0)
		{
			dckPtr->recAverage = ROUND_UP2K(dckPtr->fly_frameTotal / dckPtr->fly_videoFrameCount);
		}
		dckPtr->maxToRead = dckPtr->recAverage; 
	}

	/* first figure out the most we can read */
	bytesToRead = dckPtr->movieSize - (dckPtr->lastByteOffset - dckPtr->readIndex);
	
	/* now, keep it from blowing out of buffer */
	if ((bytesToRead + dckPtr->nextRead) > dckPtr->bfrSize)
		bytesToRead = dckPtr->bfrSize - dckPtr->nextRead;
		
	/* now limit amount to read to max to read and round down to 2K */          
	if (bytesToRead > maxToRead) 
		bytesToRead = maxToRead;
	else
		bytesToRead = (bytesToRead >> 11) << 11;
	
	/* remember to account for first (previous) pass */
	if (bytesToRead > maxToRead) 
		bytesToRead = maxToRead;                                              
	
	/* if, after all that, we can read something, do it! */ 
	if (bytesToRead > 0){
		if ((dckPtr->nextRead + bytesToRead) <= dckPtr->bfrSize)
		{
			/* FWG 5-20-99 */
			/* Why did I put this here? I think it is here just to make sure there
			are no async reads in process. I think this should never, ever, ever,
			ever happen. FWG 7-7-99 */
			if ((dckPtr->mode == HFBMODE_ASYNC) || 
				(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
			{
				/* FWG 7-7-99 Tell duck_io.c to block until the async read is finished. */
				duck_readFinished(dckPtr->fHnd->aviFile, 1);
			}

			bytesToRead = duck_read(dckPtr->fHnd->aviFile,(unsigned char *) dckPtr->movie+dckPtr->nextRead, bytesToRead);
			if (bytesToRead < 0) 
				return bytesToRead;
		}
		else
			return -99;
	}
	
	return (dckPtr->bytesReadLast = bytesToRead);
}                                                                            

/*DSS 1/96, NEW FUNCTION (place before HFB_FillBuffer())*/
long lHFB_GetBufferSpace(HFB_BUFFER_HANDLE dckPtr)
{
	return(dckPtr->movieSize - (dckPtr->lastByteOffset - dckPtr->readIndex));
}

int lHFB_FillBuffer(HFB_BUFFER_HANDLE dckPtr,long maxToRead, long elapsed)
{
	int fillErr,filled = 0;

	if (!dckPtr) return -1;

	if (elapsed != HFB_IGNORE_COUNT)
	{
		if (dckPtr->fcount >= elapsed)
			return 0;
	
		dckPtr->fcount++;
	}
	else if (elapsed == HFB_RESET_COUNT) dckPtr->fcount = 0;

	if (maxToRead > 0) return fillBufferDirect(dckPtr,maxToRead); 
	
	if ((fillErr = fillBufferDirect(dckPtr,0)) < 0) return fillErr;
	
	if ((!dckPtr->hwBuffer) || dckPtr->delay)
		return fillErr;
					
	/****************************************
	 figure the running datarate of the file
	****************************************/
	dckPtr->readTotal -= dckPtr->readTable[dckPtr->subSec];
	dckPtr->readTotal += dckPtr->readTable[dckPtr->subSec] = fillErr;   
			
	if (++dckPtr->subSec >= dckPtr->GlobalFrameRate)
		dckPtr->subSec = 0;
	
	if (dckPtr->subSec >= 30) dckPtr->subSec = 0;
	
	/****************************************
		 once we're running for a second,
		 make additional reads based on 
		 the assumed hardware buffer
	****************************************/ 
	if (++dckPtr->frameCount > dckPtr->GlobalFrameRate){
		/*******************************************
			the amount added to the extra (hardware) 
			buffer is the difference between 
			the amount read into RAM in 1 sec and the 
			max amount read by the CD-ROM during 1 sec
			( spread out across the next second )
		********************************************/
		dckPtr->extrabuff += ((MAX_DATA_RATE<<10) - dckPtr->readTotal)/dckPtr->GlobalFrameRate;
							
		if (dckPtr->extrabuff > dckPtr->hwBuffer) dckPtr->extrabuff = dckPtr->hwBuffer;
				
		if (dckPtr->extrabuff > QVAL)
		{
			filled = fillBufferDirect(dckPtr,ROUND_DOWN2K(dckPtr->extrabuff));
			if (filled < 0) return filled;
			dckPtr->extrabuff -= filled;
		}
	}
	return filled + fillErr;                                            
}

hfbBufferStatus lHFB_GetBufferStatus(HFB_BUFFER_HANDLE dckPtr)
{
	if (!dckPtr) return ( hfbBufferStatus ) 0;

	if (dckPtr->nextIndex >= dckPtr->indxCount)
		return ( hfbBufferStatus ) 1;
	
	return ( hfbBufferStatus ) 0;   
}                                                                               
