/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************\
*                                                                               *
* hfb_indx.c -   multi-platform AVI library                        *
*               Copyright (c) 1994, The Duck Corp. All rights reserved.         * 
*
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/
#include <assert.h>
#include "dkpltfrm.h"

#include "duckwalk.h"
#include "duck_mem.h"
#include "duck_io.h"

/* all reads are in blocks of multiple sectors (a sector is usually 2K)*/

#define READCHUNK   (QVAL*2)
#define MAX_DATA_RATE 300

/*************************************************************************
    function prototypes
*************************************************************************/
//int duck_readFinished(int, int); /* FWG 7-9-99 */
//int duck_readFinished(int temp1, int temp2) {return 0;};

int initIndex(HFB_BUFFER_HANDLE dckPtr, void *data, int size)
{
	if (!dckPtr)
		return 0;

    /* reset the index count, only do this once */                                    
    dckPtr->indxCount = 0;
    dckPtr->maxChunkSize = 0L;
	
    dckPtr->indx = (DCKINDEXENTRY *)dckPtr->buffer; /* explicitly point at buffer */
	
	if (data == NULL)
		data = (void *)&(dckPtr->indx[1]);

	/* FWG 6-23-99 */
	/* FWG 7-7-99 we have two modes that can use Fly-Index. */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || 
		(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
	{
		return 0; /* just set duck->indx to beginning of buffer */
	}

	/* if this is a continuation of an index read, 
	   put the new index entries at the end */
	
	return (duck_read(dckPtr->fHnd->aviFile,data, size));
}

int readMoreData(HFB_BUFFER_HANDLE dckPtr,int size)
{
	return (duck_read(dckPtr->fHnd->aviFile,&(dckPtr->indx[dckPtr->indxCount+1]), size));
}

void parseIndex(HFB_BUFFER_HANDLE dckPtr, void *data, int size)
{   
    register AVIINDEXENTRY *tempIndx;
    register DCKINDEXENTRY *lindx;
    int i;
	int blockSize;
    
	assert(dckPtr != NULL);
	if (!dckPtr)
		return;

	if (data == NULL)
		data = (void *)&(dckPtr->indx[dckPtr->indxCount+1]);

	tempIndx = (AVIINDEXENTRY *)data;

	/* if this is a continuation of an index read, 
	   put the new index entries at the end */

    lindx = &(dckPtr->indx[dckPtr->indxCount]);

	/* FWG 7-7-99 We have two modes async can be in. We should
	never get here if we are in HFBMODE_ASYNC_FLYINDEX. */
	if ((dckPtr->mode == HFBMODE_ASYNC) || 
		(dckPtr->mode == HFBMODE_ASYNC_FLYINDEX) )
	{
		/* FWG 7-7-99 Tell duck_io.c to block until the async read is finished. */
		duck_readFinished(dckPtr->fHnd->aviFile, 1);
	}
	
    blockSize = size/sizeof(AVIINDEXENTRY);

    for(i = 0; i < blockSize; i++,tempIndx++){
	    /* if this is a stream chunk */    
        if (duck_isdigit(lowByte(tempIndx->ckid))){
            /* endian swap for offset and length if necessary */
			lindx->dwChunkOffset = invert4(tempIndx->dwChunkOffset);
            lindx->dwChunkLength = invert4(tempIndx->dwChunkLength);
	        
	        if (invert4(tempIndx->dwFlags) & AVIIF_KEYFRAME)
	            lindx->dckFlags = DCKCHUNK_KEYFRAME;
			else
	            lindx->dckFlags = DCKCHUNK_CLEAR;
				
	        if (lindx->dwChunkLength > dckPtr->maxChunkSize) 
	            dckPtr->maxChunkSize = lindx->dwChunkLength;

			/* generate a integer stream number from the chunkId */
            lindx->dckID = ((lowByte(tempIndx->ckid)-'0')*10) + (mid1Byte(tempIndx->ckid)-'0');
	        lindx++; /* update the temporary index pointer */
        }
    }

    dckPtr->indxCount = 
      	((unsigned long)lindx - (unsigned long)dckPtr->indx)/sizeof(DCKINDEXENTRY);

}

/*****************************************************************************
                        buildIndex()                                                                              
*****************************************************************************/

void buildIndex(HFB_BUFFER_HANDLE dckPtr, void *data, int size)
{   
    int amountRead;
	int k32kbytes = 32 * 1024;

	/* initialize the Index */
	amountRead = initIndex(dckPtr,(void *)NULL,0);

	/* Read and parse the index 32k at a time */
	while (size >= k32kbytes)
	{
		amountRead = readMoreData(dckPtr, k32kbytes);
		parseIndex(dckPtr, (void*)NULL, amountRead);
		size -= k32kbytes;
	}

	/* if the index size is not a multiple of 32k, handle the remainder */
	amountRead = readMoreData(dckPtr, size);
	parseIndex(dckPtr, (void*)NULL, amountRead);
}
/************************ END buildIndex() *************************************/                                                                               
