/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************\
*                                                                               *
* hfb_key.c -   multi-platform AVI library                        *
*               Copyright (c) 1994-99, The Duck Corp. All rights reserved.         * 
*
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/
#include <assert.h>
#include "dkpltfrm.h"
#include "duckwalk.h"

/* find number of frames to next Keyframe forward, from here
   assumes ChunkIndex is pointing at a video frame */
int lHFB_FramestoNextKeyFrame(HFB_STREAM_HANDLE sPtr,int ChunkIndex, int *pNewChunkIndex)
{
    HFB_BUFFER_HANDLE dckPtr;
    register int i = ChunkIndex;
	int NumFrames = 0;
	int flags; 

	if (!pNewChunkIndex)
		return -1;

	/* initialize the index to an error code */
	*pNewChunkIndex = -1;

	/* in order for this to work we need a valid stream handle */
	if (sPtr)
	{
		if (!sPtr->parent)
			return -1;

		dckPtr = sPtr->parent->bfHnd;
		if (!dckPtr) 
			return -1;

		/* this is not a valid operation in fly index mode */
		if ((dckPtr->mode == HFBMODE_FLYINDEX) || (dckPtr->mode == HFBMODE_ASYNC_FLYINDEX))
			return -1;

		if (dckPtr && dckPtr->indx)
		{
			/* dckPtr->indxCount is total number of index entries...*/
			if ((ChunkIndex > -1) && (ChunkIndex < dckPtr->indxCount))
			{
	 			register DCKINDEXENTRY *lindx = &(dckPtr->indx[ChunkIndex + 1]);
	 				
	 			for(i = ChunkIndex + 1; i < dckPtr->indxCount; i++,lindx++)	  {
		   			if (lindx->dckID == sPtr->parent->streamNum)	{	/* same stream -- gotta be video */
						NumFrames++;
						flags = (int)dckPtr->indx[i].dckFlags;
						if (flags & HFB_DATA_KEYFRAME) {
							*pNewChunkIndex = i;		/* chunk index of new frame  */
							return NumFrames;
						}
					}
				}

				/* off end of file, but call the end a keyframe for skipping purposes*/
				*pNewChunkIndex = -2;
				return NumFrames + 1; 
 			}
		}
	}
 	return -1;    /* bad call */
}

int lHFB_FrameToChunk(HFB_STREAM_HANDLE sPtr, int FrameNum)
{
    HFB_BUFFER_HANDLE dckPtr;
    int i;
	int frame = FrameNum;
	DCKINDEXENTRY *lindx;

	if (!sPtr)
		return -1;
	
	if (!sPtr->parent) 
		return -1;

	dckPtr = sPtr->parent->bfHnd;
	if (!dckPtr) 
		return -1;

	/* this is not a valid operation in fly index mode */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || (dckPtr->mode == HFBMODE_ASYNC_FLYINDEX))
		return -1;

	lindx = dckPtr->indx;

	if (!lindx)
		return -1;

	for(i = 0; i < dckPtr->indxCount; i++,lindx++)	/* find Index corresponding to frame */
    	if (lindx->dckID == sPtr->parent->streamNum)
            if (--frame < 0) 
            	break;

	if (i >= dckPtr->indxCount)
		return -2;	/* off end of file*/

	return i;
}
/*
// Returns Frame and ChunkIndex of previous key frame before this frame.
// Used in Seek and Cue to ensure that we start decompressing on a keyframe.
// assumes ChunkIndex is pointing at a video frame
// Counts back frames from this Chunk Index and returns the starting Frame Number
*/
int lHFB_PreviousKeyFrame(HFB_STREAM_HANDLE sPtr, int FrameNum)
{
    HFB_BUFFER_HANDLE dckPtr;
    int i;
	int flags; 
	int frame = FrameNum;

	if (!sPtr)
		return -1;

	if (!sPtr->parent)
		return -1;

    dckPtr = sPtr->parent->bfHnd;
	if (!dckPtr)
		return -1;

	/* this is not a valid operation in fly index mode */
	if ((dckPtr->mode == HFBMODE_FLYINDEX) || (dckPtr->mode == HFBMODE_ASYNC_FLYINDEX))
		return -1;

	if ((FrameNum < 0) || (FrameNum >= dckPtr->indxCount)) 
		return -1;

	if (!dckPtr->indx)
		return -1;

    {
		register DCKINDEXENTRY *lindx = dckPtr->indx;		

		for(i = 0; i < dckPtr->indxCount; i++,lindx++)
	    	if (lindx->dckID == sPtr->parent->streamNum)
	            if (--frame < 0) 
	            	break;
	
		if (i >= dckPtr->indxCount)	
			return -2;
	    /*
		// i now Chunk Index, lindx points for flags for this frame
	    // dckPtr->indxCount is total number of index entries...
	 	*/	
	 	for( ; i >= 0; i--,lindx--)	  {
		   	if (lindx->dckID == sPtr->parent->streamNum)
			{
				flags = (int)dckPtr->indx[i].dckFlags;
				if (flags & HFB_DATA_KEYFRAME) 
					return FrameNum;
				FrameNum--;				/* not this one, back up some more*/
			}
	    }
    }
	return -2;  /* off end of file*/
}
