/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <assert.h>
#include "dkpltfrm.h"
#include "duckwalk.h"
#include "duck_mem.h"
#include "duck_io.h"
#include "stdlib.h"
#include "stdio.h"

#define DANBUGGIN 0
#if DANBUGGIN

void dbgout(char *str)
{
	FILE *fh;

    fh = fopen("C:\\dan.log","a+");
	if(fh != NULL) {
		fputs(str,fh);
		fclose(fh);
    } 
}
void dbgint(char *str, int numargs, int *args)
{
    char ach[256];
	int x;
	for(x=0; x<255; x++) {
		ach[x] = str[x];
		if(str[x]==0) break;
	}
	dbgout(ach);
	for(x=0; x<numargs; x++) {
		itoa(args[x], ach, 10);
		dbgout(ach);
		ach[0]=0x20;				//	' '
		ach[1]=0;
		dbgout(ach);
	}
	ach[0]=0x0a;				//	'\'
	ach[1]=0;
	dbgout(ach);
}

void dbgint1(char *str, int arg) {
	dbgint(str, 1, &arg);
}

void dbgint2(char *str, int arg1, int arg2) {
	int args[2];
	args[0]=arg1;
	args[1]=arg2;
	dbgint(str, 2, args);
}

void dbgint3(char *str, int arg1, int arg2, int arg3) {
	int args[3];
	args[0]=arg1;
	args[1]=arg2;
	args[2]=arg3;
	dbgint(str, 3, args);
}

#else
void dbgout(char *str) { }
void dbgint1(char *a, int b) { }
void dbgint2(char *a, int b, int c) { }
void dbgint3(char *a, int b, int c, int d) { }

#endif
/************************************************************************
 *
 * Multi-Format Parser functions
 *
 ************************************************************************/
typedef struct tHFBContext {
	HFB_FILE_HANDLE hFiles;
	HFB_STREAM_HANDLE hStreams;
	HFB_BUFFER_HANDLE hBuffers;
	int maxStreams;
	int maxFiles;
	int maxBuffers;
}HFB_CONTEXT, *HFB_CONTEXT_HANDLE;

#if PRE_ALLOCATE
static HFB_CONTEXT_HANDLE defContext = (HFB_CONTEXT_HANDLE)NULL;
#endif

/* initialize Multi-Format Parser */
int lHFB_Init(int MaxOpenFiles,int MaxOpenStreams,int MaxOpenBuffers)
{
#if PRE_ALLOCATE
	int total;

	defContext = (HFB_CONTEXT_HANDLE)duck_calloc(1,sizeof(HFB_CONTEXT),DMEM_GENERAL);

	if (!defContext) return -1;

	total = MaxOpenFiles*sizeof(HFB_FILE)+
		MaxOpenStreams*sizeof(HFB_STREAM)+
		MaxOpenBuffers*sizeof(HFB_BUFFER);

	if (!total){
		defContext->maxStreams = MaxOpenStreams;
		defContext->maxFiles = MaxOpenFiles;
		defContext->maxBuffers = MaxOpenBuffers;

		return 0;
	}

	defContext->hFiles = (HFB_FILE_HANDLE)duck_malloc(total,DMEM_GENERAL);

	if (defContext->hFiles){
		duck_memset(defContext->hFiles,0,total);
		defContext->hStreams = (HFB_STREAM_HANDLE)(defContext->hFiles + MaxOpenFiles);
		defContext->hBuffers = (HFB_BUFFER_HANDLE)(defContext->hStreams + MaxOpenStreams);

		defContext->maxStreams = MaxOpenStreams;
		defContext->maxFiles = MaxOpenFiles;
		defContext->maxBuffers = MaxOpenBuffers;

		return 0;
	}else{
		defContext->maxStreams = defContext->maxFiles = defContext->maxBuffers = 0;
		duck_free(defContext);
		defContext = NULL;
	}
#endif
	return -1;
}

/* exit Multi-Format Parser */

void lHFB_Exit()
{
#if PRE_ALLOCATE
	if (defContext){
		if (defContext->hFiles)
			duck_free(defContext->hFiles);
		duck_free(defContext);
	}
	defContext = NULL;
#endif
}


HFB_FILE_HANDLE lHFB_FindFile(const char *FileName, unsigned long userData)
{
    HFB_FILE_HANDLE fHnd;

#if PRE_ALLOCATE
	if (defContext && defContext->hFiles){
	    int i;

		for (fHnd = defContext->hFiles, i = 0; i < defContext->maxFiles; i++,fHnd++)
			if (fHnd->fileType == DUK_NULFILE) break;

		if (i >= defContext->maxFiles) return NULL;
    }else
#endif
	{
		fHnd = (HFB_FILE_HANDLE ) duck_calloc(1,sizeof(HFB_FILE),DMEM_GENERAL);
		if (!fHnd)
			return fHnd;
	}
	duck_memset(fHnd,0,sizeof(HFB_FILE));

	if((long)(fHnd->aviFile = duck_open(FileName, userData)) < 0){
#if PRE_ALLOCATE
		if (!(defContext && defContext->hFiles))
#endif
			duck_free(fHnd);
		return (HFB_FILE_HANDLE)NULL;
    }

    fHnd->fileType = DUK_AVIFILE;

	return fHnd;
}

/* close a TrueMotion/S A/V file */
void lHFB_CloseFile(HFB_FILE_HANDLE fHnd)
{
	if (!fHnd) return;

	if (fHnd->fileType == DUK_AVIFILE)
		AVIFileClose(fHnd);

#if PRE_ALLOCATE
	if (defContext && defContext->hFiles)
		duck_memset(fHnd,0,sizeof(HFB_FILE));
	else
#endif
		duck_free(fHnd);
}


int lHFB_ParseFile(
	HFB_FILE_HANDLE fileHandle,
	HFB_BUFFER_HANDLE bufferHandle
	);

void lHFB_LoadIndex(
	HFB_FILE_HANDLE fileHandle,
	HFB_BUFFER_HANDLE bufferHandle
	);


/* open a TrueMotion/S A/V file, if bfHnd is NULL no buffering is used */
HFB_FILE_HANDLE lHFB_OpenFile(const char *FileName, HFB_BUFFER_HANDLE bfHnd, unsigned long userData)
{
    HFB_FILE_HANDLE fHnd;
	int foo;
 dbgout("trying to open this damn file::: ");
 dbgout(FileName);
 dbgout("\n");
	if (FileName)
	{
		/* open and preprocess the avi file */
		if ((fHnd = lHFB_FindFile(FileName, userData))){
			dbgout("HFB_OpenFile found it, let's parse it\n");
    		if ((foo = lHFB_ParseFile(fHnd, bfHnd)) < 0){
    			lHFB_CloseFile(fHnd);
				dbgout("HFB_OpenFile return path A\n");
    			return (HFB_FILE_HANDLE)NULL;
    		}

			/* FWG 9-13-2000 */
			if (bfHnd->mode == HFBMODE_PSEUDO_ASYNC)
			{
				dbgout("HFB_OpenFile return path B\n");
				return fHnd;
			}
			/* FWG 9-13-2000 */
    		if (bfHnd) lHFB_LoadIndex(fHnd,bfHnd);
			dbgout("HFB_OpenFile return path C\n");
			return fHnd;
		}
	}
 dbgout("well, that went OK--not!\n");
	return (HFB_FILE_HANDLE)NULL;
}

#ifndef SEEK_CUR
#define SEEK_CUR    1
#endif

/*SJM 10-26-00 */
int lHFB_ReadMoreIndex(HFB_BUFFER_HANDLE dckPtr,int size)
{
	int q = readMoreData(dckPtr,size);

	return q;
}

int lHFB_ParseBigIndex(HFB_BUFFER_HANDLE dckPtr, void *data, int size)
{
	parseIndex(dckPtr,data,size);

	return 0;
}


/* FWG 9-13-2000 */
int lHFB_SeekToIndex(HFB_FILE_HANDLE FileHandle)
{
	int avilen;

	avilen = FileHandle->header.dwReserved[0];

	/* if we haven't already gotten an index,
	skip past the movi list to get the next
	chunk,  which should be the index */

	duck_seek(FileHandle->aviFile, avilen, SEEK_CUR);

	return 0;
}

int lHFB_BeginLoadIndex(HFB_BUFFER_HANDLE dckPtr, int size)
{
	int amountRead;

	amountRead = initIndex(dckPtr,(void *)NULL,size);

	return amountRead;
}

int lHFB_FinishLoadIndex(HFB_BUFFER_HANDLE dckPtr, void *data, int size)
{
	parseIndex(dckPtr, data, size);

		/*
		added this callculation based on the avi header
		this replaces the manual version done in buildIndex
	 */
	if (!dckPtr->fHnd->header.dwTotalFrames)
		dckPtr->fHnd->header.dwTotalFrames = 1;

	dckPtr->recAverage = ROUND_UP2K(dckPtr->fHnd->header.dwReserved[0] / dckPtr->fHnd->header.dwTotalFrames );

	dckPtr->maxToRead = dckPtr->recAverage;
    dckPtr->modToRead = 0;

	return 0;
}
/* FWG 9-13-2000 */

/*
int lHFB_SeekToData(HFB_FILE_HANDLE FileHandle)
{
}
*/

/* get information about a TrueMotion/S A/V file */
HFB_FILE_INFO *lHFB_GetFileInfo(HFB_FILE_HANDLE FileHandle)
{
	return (HFB_FILE_INFO *)FileHandle;
}

static void syncStreamAndParent(HFB_STREAM_HANDLE cStream)
{
	if (cStream)
	{
		assert(cStream->parent != NULL);
		if (cStream->parent)
		{
			cStream->lastSample = cStream->parent->lastSample;
			cStream->lastFrame = cStream->parent->lastFrame;
			if (cStream->parent->bfHnd)
				cStream->getOffset = cStream->parent->bfHnd->movieOffset;
		}
	}
}

void lHFB_ResetStreams(HFB_BUFFER_HANDLE nBuffer)
{
	int i = 0;
	register HFB_STREAM_HANDLE_X nStream;
	register HFB_STREAM_HANDLE cStream;

	if (nBuffer)
	{
		assert(nBuffer->fHnd != NULL);
		if (nBuffer->fHnd)
		{
			for(nStream = nBuffer->fHnd->heads; i < nBuffer->fHnd->lNumStreams; i++,nStream++)
			{
				cStream = nStream->nextStream;
				while(cStream)
				{
					syncStreamAndParent(cStream);
					cStream = cStream->nextStream;
				}
			}
		}
	}
}

/* get a handle to a stream within a TrueMotion/S A/V file by name, number or type */
/* Note: If the caller provides *both* a stream name and a stream number, we will  */
/*       search by name, and ignore the stream number.                             */
HFB_STREAM_HANDLE lHFB_GetStream(HFB_FILE_HANDLE fHnd,const char *StreamName,int StreamNum, unsigned long lSType)
{
	HFB_STREAM_HANDLE nStream = (HFB_STREAM_HANDLE) NULL;
	HFB_STREAM_HANDLE_X foundStream = (HFB_STREAM_HANDLE_X) NULL;
	/* We need to recast the given long into an enumeration */
	enum HFB_StreamType SType = (enum HFB_StreamType) lSType;

	/* make sure we've got a valid file handle */
	if (fHnd == (HFB_FILE_HANDLE) NULL)
	{
		return (HFB_STREAM_HANDLE) NULL;
	}
	
#if PRE_ALLOCATE
	if (defContext && defContext->hStreams)
	{
		int i;

		for(nStream = defContext->hStreams, i = 0;
			i < defContext->maxStreams;
			i++, nStream++)
		{
			if (nStream->streamType == DUK_UNDEFINED) break;
		}
		if (i == defContext->maxStreams) return NULL;
	}
	else
#endif

	{
		/* allocate a new stream handle, and verify that the allocation worked */
		nStream = (HFB_STREAM_HANDLE) duck_calloc(1, sizeof(HFB_STREAM), DMEM_GENERAL);
		if (nStream == (HFB_STREAM_HANDLE) NULL)
		{
			return (HFB_STREAM_HANDLE) NULL;
		}
	}

	/* If we were given a stream name, search for a matching stream */
	if (StreamName != NULL)
	{
		int i;

		/* check each stream associated with the file handle */
		for(i = 0; i < fHnd->lNumStreams; i++)
		{
			/* if we found a match (comparison returns 0), note the stream */
			/*   and stop looping.                                         */
			if ( duck_strcmp(fHnd->heads[i].szName, StreamName) == 0 )
			{
				foundStream = &(fHnd->heads[i]);
				break;
			}
		}
	}

	/* If we were given a stream number, and the stream type */
	/* is undefined, just return the ith stream              */
	else if ((StreamNum > 0) && (SType == DUK_UNDEFINED))
	{
		/* make sure the requested stream exists */
		if (StreamNum <= fHnd->lNumStreams)
		{
			foundStream = &(fHnd->heads[StreamNum - 1]);
		}
	}

	/* the caller is requesting a stream of a certain type, */
	/* so look for the ith stream of that type              */
	else if (StreamNum > 0)
	{
		int i = 0;
		int matchingStreamCount = 0;
		
		/* look for a match until there are no more streams */
		while (i < fHnd->lNumStreams)
		{
			/* we only care about streams matching the requested type */
			if (fHnd->heads[i].streamType == (long)SType)
			{
				/* increment the counter and see if this is the stream the caller wants */
				matchingStreamCount++;
				if (matchingStreamCount == StreamNum)
				{
					/* we found the stream, so we can stop looking */
					foundStream = &(fHnd->heads[i]);
					break;
				}
			}
			/* either the stream type or number didn't match.  Check the next stream */
			i++;
		}
	}

	/* if we couldn't find a matching stream, clean up and bail out */ 
	if (foundStream == NULL)
	{
#if PRE_ALLOCATE
		if (defContext && defContext->hStreams)
			return (HFB_STREAM_HANDLE) NULL;
#endif
		duck_free(nStream);
		return (HFB_STREAM_HANDLE) NULL;
    }

	/* if we made it this far, we found a matching stream, */
	/* so fill in that new stream handel we created        */ 
	nStream->parent = foundStream;
	nStream->streamType = foundStream->streamType;
	nStream->nextStream = foundStream->nextStream;

	syncStreamAndParent(nStream);

	/* update the parent stream */
	foundStream->nextStream = nStream;
    foundStream->numRef += 1;

    return nStream;
}


void lHFB_ReleaseStream(HFB_STREAM_HANDLE StreamHandle)
{
	if (StreamHandle)
	{
		if (StreamHandle->parent)
		{
			if (StreamHandle->parent->nextStream == StreamHandle){
				StreamHandle->parent->nextStream = StreamHandle->nextStream;
			}else{
				HFB_STREAM_HANDLE nStream = StreamHandle->parent->nextStream;

				while(nStream){
					if (nStream->nextStream == StreamHandle)
						nStream->nextStream = nStream->nextStream->nextStream;

					nStream = nStream->nextStream;
				}
			}
			StreamHandle->parent->numRef--;

#if PRE_ALLOCATE
			if (defContext && defContext->hStreams){
				duck_memset(StreamHandle,0,sizeof(HFB_STREAM));
				StreamHandle->streamType = DUK_UNDEFINED;
			}else
#endif
				duck_free(StreamHandle);
		}
	}
}
#if 0
/* release a handle to a stream */
void lHFB_ReleaseStream(HFB_STREAM_HANDLE StreamHandle)
{
	HFB_STREAM_HANDLE tempStreamHandle = (HFB_STREAM_HANDLE) NULL;

	/* make sure we were passed a valid stream */
	if (StreamHandle == (HFB_STREAM_HANDLE) NULL)
		return;
	if (StreamHandle->parent == (HFB_STREAM_HANDLE_X) NULL)
		return;


	/* look for this stream in the parent's list of children */
	tempStreamHandle = StreamHandle->parent->nextStream;

	while( (tempStreamHandle != StreamHandle)
		   && (tempStreamHandle != (HFB_STREAM_HANDLE) NULL) )
	{
		/* If we found a match, remove this stream from the linked list */
		if (tempStreamHandle->nextStream == StreamHandle)
		{
			tempStreamHandle->nextStream = tempStreamHandle->nextStream->nextStream;
			break;
		}
		/* We haven't found a match yet. Try the next stream */
		tempStreamHandle = tempStreamHandle->nextStream;
	}

	/* decrement the parent's reference count */
	StreamHandle->parent->numRef -= 1;

#if PRE_ALLOCATE
	if (defContext && defContext->hStreams)
	{
		duck_memset(StreamHandle, 0, sizeof(HFB_STREAM));
		StreamHandle->streamType = DUK_UNDEFINED;
	}
	else
#endif

	/* free the memeory we allocated for this child stream */
		duck_free(StreamHandle);
}
#endif
/* get information about a stream */
HFB_STREAM_INFO *lHFB_GetStreamInfo(HFB_STREAM_HANDLE StreamHandle)
{
	if (StreamHandle)
	{
		assert(StreamHandle->parent != NULL);
		return (HFB_STREAM_INFO *)(StreamHandle->parent);
	}
	return (HFB_STREAM_INFO *)NULL;
}

HFB_BUFFER_HANDLE HFB_GetStreamParentBuffer(HFB_STREAM_HANDLE StreamHandle)
{
	if (StreamHandle)
	{
		assert(StreamHandle->parent != NULL);
		if (StreamHandle->parent)
		{
			assert(StreamHandle->parent->fHnd != NULL);
			return StreamHandle->parent->bfHnd;
		}
	}
	return NULL;
}

HFB_FILE_HANDLE HFB_GetStreamParentFile(HFB_STREAM_HANDLE StreamHandle)
{
	if (StreamHandle)
	{
		assert(StreamHandle->parent != NULL);
		if (StreamHandle->parent)
			return StreamHandle->parent->fHnd;
	}
	return NULL;
}

/* FWG 6-23-99 */
int HFB_GetStreamParentFileHandle(HFB_STREAM_HANDLE StreamHandle)
{
	if (StreamHandle)
	{
		if (StreamHandle->parent)
		{
			if (StreamHandle->parent->fHnd)
				return StreamHandle->parent->fHnd->aviFile;
		}
	}
	return -1;
}

/* FWG 7-7-99 */
int HFB_SetMaxFrameSize(HFB_BUFFER_HANDLE Buffer, int maxFrameSize)
{
	if (Buffer)
		Buffer->maxChunkSize = maxFrameSize;

	return 0;
}

/* FWG 7-8-99 */
hfbBufferMode HFB_GetBufferMode(HFB_BUFFER_HANDLE Buffer)
{
	if (Buffer)
		return Buffer->mode;
	return 0;
}

/************************************************************************
 *
 * High-speed File Buffering functions
 *
 ************************************************************************/

HFB_BUFFER_HANDLE lHFB_CreateBuffer(long bSize,long minIndexFrames)
{
	HFB_BUFFER_HANDLE nBuffer;

	if (bSize < 32)
		return NULL;

#if PRE_ALLOCATE
	if (defContext && defContext->hBuffers)
	{
		int i;
		for (nBuffer = defContext->hBuffers, i = 0; i < defContext->maxBuffers; i++,nBuffer++)
			if (nBuffer->rawSize == 0) break;

		if (i >= defContext->maxBuffers) return NULL;
	}
	else
#endif
	{
		nBuffer = (HFB_BUFFER_HANDLE) duck_calloc(1,sizeof(HFB_BUFFER),DMEM_GENERAL);
		if (!nBuffer)
			return nBuffer;
	}

	nBuffer->targetSize = bSize;
	nBuffer->m_indxSize = minIndexFrames ? (minIndexFrames+32)*sizeof(DCKINDEXENTRY) : bSize/4;
	nBuffer->rawSize = nBuffer->targetSize+nBuffer->m_indxSize;

	nBuffer->buffer = (unsigned char *)
		duck_malloc(nBuffer->rawSize,DMEM_CDBUFF);

	if (nBuffer->buffer == NULL)
	{
#if PRE_ALLOCATE
		if (defContext && defContext->hBuffers)
			return NULL;
#endif
		duck_free(nBuffer);
		return NULL;
	}

	nBuffer->indx = (DCKINDEXENTRY *)(nBuffer->buffer);
	nBuffer->minIndx = 0;

	nBuffer->releaseOffset = nBuffer->movieOffset = 2048 + 16;

	return nBuffer;
}
void lHFB_DestroyBuffer(HFB_BUFFER_HANDLE Buffer)
{
	if (Buffer)
	{
		if (Buffer->buffer)
			duck_free(Buffer->buffer);
#if PRE_ALLOCATE
		if (defContext && defContext->hBuffers){
			duck_memset(Buffer,0,sizeof(HFB_BUFFER));
		}
		else
#endif
		{
			duck_free(Buffer);
		}
	}
}

int lHFB_SetBufferMode(HFB_BUFFER_HANDLE Buffer,enum HFB_Modes newMode)
{
	enum HFB_Modes oldMode;

	if (!Buffer) return -1;

	oldMode = Buffer->mode;
	Buffer->mode = newMode;

	return oldMode;
}

