/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************\
*                                                                               *
* windwalk.c -   multi-platform TrueMotion/S AVI library                        *
*               Copyright (c) 1994, The Duck Corp. All rights reserved.         * 
*
*                 Edited by:    Eric Ameres                                     *
*                 Version:      .97                                             *
*                 Edited:       01/25/94                                        *
                    - added adaptive datarate fill buffer routine
                    - removed unnecessary fillbuffering
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "dkpltfrm.h"
#include "duckwalk.h"

/*************************************************************************
    local defines
*************************************************************************/

#define GETOFFSET(x)    (x + dckPtr->getOffset)

/* all reads are in blocks of multiple sectors (a sector is usually 2K)*/

HFB_DATA_HANDLE lHFB_GetStreamingData(HFB_STREAM_HANDLE dckPtr, void** data, long* length, enum dukDirect direction, int count)
{   
	int chunkIndex, count2, target, streamNum;
	HFB_BUFFER_HANDLE bfr;
	register DCKINDEXENTRY* lindx; 
	int updateLast = 1;

	assert( (direction == DUK_RESET) || 
		    (direction == DUK_FORWARD) || 
	        (direction == DUK_BACKWARD) ||
		    (direction == DUK_ABSOLUTE) );

	if (count == 0)
	{
		if ((direction == DUK_FORWARD) || (direction == DUK_BACKWARD))
		{
			count = 1;
			updateLast = 0;
		}
	}

	assert(dckPtr != NULL);
	if (!dckPtr)
		return -6;

	assert(dckPtr->parent != NULL);
	if (!dckPtr->parent)
		return -6;

	assert(dckPtr->parent->bfHnd != NULL);
	if (!(bfr = dckPtr->parent->bfHnd))
		return -6;

	assert(bfr->indx != NULL);
	if (!bfr->indx)
		return -6;

	streamNum = dckPtr->parent->streamNum;

	if (direction == DUK_RESET)
	{
		dckPtr->lastSample = dckPtr->parent->lastSample;
		dckPtr->lastFrame = dckPtr->parent->lastFrame;
		return dckPtr->lastSample;
	}
	else if (direction == DUK_ABSOLUTE)
	{
		if (count < 0) return -1; /* invalid frame number */

		if (dckPtr->lastFrame < count)
		{
			count -= dckPtr->lastFrame;
			direction = DUK_FORWARD;
		}
		else if (dckPtr->lastFrame > count)
		{
			count = dckPtr->lastFrame - count;
			direction = DUK_BACKWARD;
		}
		else count = 0;
	}

	if (dckPtr->lastSample+direction < 0)
		return -5;

	chunkIndex = dckPtr->lastSample+direction;

	if (chunkIndex > bfr->indxCount)
		return -5;

	assert(data != NULL);
	assert(length != NULL);

	if (!data || !length) return -5;

	*data = NULL;
	*length = 0;	

	lindx = &(bfr->indx[chunkIndex]);

	/* limit the search to length of index */
	if ((direction == 1) || (direction == 0))
		count2 = bfr->indxCount - chunkIndex;
	else
		count2 = chunkIndex+1;
	
	target = dckPtr->lastFrame + (count * direction);
 dbgint1("**** HFB_GetStreamingData: target = ",target);
 dbgout("\n");
	/* search the index */	

	while(count2-- > 0)
	{
		if (lindx->dckID == streamNum) 
		{
			if (--count <= 0)
				break;
		}
		lindx += direction;
		chunkIndex += direction;
	}

    /* if we passed the end of buffer... */
	if ( (chunkIndex >= bfr->indxCount) || 
		 (lindx->dckID != streamNum) )
	{ 
		/*
		if (bfr->mode == HFBMODE_FORWARDLOOP)
		{
			dckPtr->getOffset = bfr->movieOffset;
			dckPtr->lastSample = 0;
			return -3;
		}
		*/
		return -1;
	}

	if (chunkIndex > -1) 
	{
		*length = lindx->dwChunkLength;

		if (lindx->dckFlags & DCKCHUNK_RELEASED) 
		{
			return -4;
		} 
		else if (!(lindx->dckFlags & DCKCHUNK_READ)) 
		{
			return -2;
		}
		
		/* update last sample/frame */
		if (updateLast)
		{
			dckPtr->lastSample = chunkIndex;
			dckPtr->lastFrame = target;
		}
		if (*length)
		{
			*data = bfr->movie + ((GETOFFSET(lindx->dwChunkOffset)) % bfr->movieSize);
	    
			/* if the chunk has upper memory flag set, offset is past buffer limit */
			if (lindx->dckFlags & DCKCHUNK_UPPER)
				*data = (void*)((DWORD)(*data) + bfr->movieSize);
		}
	}
 dbgint1("**** HFB_GetStreamingData: chunkIndex = ",chunkIndex);
 dbgout("\n");
	return chunkIndex;
}

static _inline int checkHandle(HFB_BUFFER_HANDLE bfHnd, HFB_DATA_HANDLE DataHandle)
{
	assert(bfHnd != NULL);
    if (bfHnd)
	{
		assert(bfHnd->indx != NULL);
		if (bfHnd->indx)
		{
			assert (DataHandle > -1);
			assert (DataHandle < bfHnd->indxCount);
			if ((DataHandle > -1) && (DataHandle < bfHnd->indxCount))
				return 0;
		}
	}
	return -1;
}

void lHFB_ReleaseStreamingData(HFB_BUFFER_HANDLE bfHnd, HFB_DATA_HANDLE DataHandle)
{   
	if (!checkHandle(bfHnd,DataHandle))
		bfHnd->indx[DataHandle].dckFlags |= DCKCHUNK_RELEASED; /* mark the chunk released*/
}

int lHFB_GetDataSize(HFB_BUFFER_HANDLE bfHnd, HFB_DATA_HANDLE DataHandle)
{   
	if (!checkHandle(bfHnd,DataHandle))
		return bfHnd->indx[DataHandle].dwChunkLength;

	return 0;
}

static int byteRate(HFB_STREAM_HANDLE_X strm, int amount, int frames)
{
	double temp;

	if (strm->sampleSize) // fixed bitrate, just calculate the rate
	{
		temp = strm->sampleSize;
		temp *= strm->rate;
		temp /= strm->scale;
	}
	else
	{
		if (!frames)
			return 0;

		temp = amount;

		temp *= strm->rate;
		temp /= (strm->scale*frames);
	}

	return (int)temp;
}				

int HFB_NextKeyFrame(HFB_STREAM_HANDLE sPtr, int startChunk,int *amountAhead)
{
	HFB_BUFFER_HANDLE dckPtr = sPtr->parent->bfHnd;

	++startChunk;  /*start with next chunk in the buffer*/

	if (startChunk > dckPtr->indxCount)	
		return 0;

	{
		int streamNum = sPtr->parent->streamNum;
		int frames = 0,amtAhead = 0;
	
		DCKINDEXENTRY *lindx = &dckPtr->indx[startChunk];
		DCKINDEXENTRY *endIndx = &dckPtr->indx[dckPtr->indxCount];
		
		for(; lindx < endIndx; lindx++)	
		{ 
			if (lindx->dckID == streamNum)
			{
	    		if (lindx->dckFlags & DCKCHUNK_KEYFRAME)
					break;

				frames++;
				amtAhead += lindx->dwChunkLength;
			}
		}

		if (amountAhead)
			*amountAhead = byteRate(sPtr->parent,amtAhead,frames);

		return (lindx == endIndx) ? -1 : lindx - dckPtr->indx;
	}
}

int HFB_ProjectBitRate(HFB_STREAM_HANDLE sPtr, int startChunk, int *horizon)
{
    HFB_BUFFER_HANDLE dckPtr = sPtr->parent->bfHnd;
	int streamNum = sPtr->parent->streamNum;
	int amountAhead = 0;
    int frame;

	if (!sPtr)
		return -1;

	if (!sPtr->parent)
		return -1;

	if (!sPtr->parent->bfHnd)
		return -1;

	/* this is not a valid operation in fly index mode */
	if ( (sPtr->parent->bfHnd->mode == HFBMODE_FLYINDEX)
		 || (sPtr->parent->bfHnd->mode == HFBMODE_ASYNC_FLYINDEX) )
		return -1;

	if (sPtr->parent->sampleSize) // fixed bitrate, just calculate the rate
	{
		double temp;

		temp = sPtr->parent->sampleSize;
		temp *= sPtr->parent->rate;
		temp /= sPtr->parent->scale;

		return (int)temp;
	}
	
	++startChunk;  /*start with next chunk in the buffer*/

	if (startChunk > dckPtr->indxCount)	
		return 0;

	{
		DCKINDEXENTRY *lindx = &dckPtr->indx[startChunk];
		DCKINDEXENTRY *endIndx = &dckPtr->indx[dckPtr->indxCount];

		if (*horizon == -1) // go to the next keyframe
			*horizon = dckPtr->indxCount;

		frame = *horizon;

		for(; lindx < endIndx; lindx++)	
		{ 
			/* find Index corresponding to frame */
    		if (lindx->dckID == streamNum)
			{
	    		amountAhead += lindx->dwChunkLength;

				if (--frame <= 0)
					break;
			}
		}
	}
	*horizon -= frame;		/* return number examined to build amount
							// could be cut down by end-of-file
							*/
	if (!*horizon)
		return 0;

	return byteRate(sPtr->parent,amountAhead,*horizon);
}


/* Determine the stream to which we can switch that has the highest precedence       */
/* (defined by order in the array), but is below the ceiling specified by the caller */
/* Note: We can only switch to another stream if its next frame is a keyframe.       */
/* Note: If no streams are below the ceiling, we will attempt to switch to the       */
/*       lowest bitrate stream on a keyframe.                                        */
int HFB_SelectBestFitStream(
	HFB_STREAM_HANDLE* streamArray, // array of streams to consider
	int numberOfStreamsInArray,     // number of streams in the array
	int currentStream,              // array index matching currently used stream
	int desiredProjectionSpan,      // distance in frames ahead to project
	int bitRateCeiling)             // we're looking for a stream under this bitrate
{
	int arrayIndex       = 0;
	int bestFitStream    = -1;
	int spanOfProjection = -1;
	int projectedBitRate = 0;
	int frameToTest      = 0;
	long vidlength       = 0L;
	int lowestStreamBitRate    = 0x7fffffff;
	int lowestStreamOnKeyframe = -1;
	unsigned char* data  = (unsigned char*) NULL;
	HFB_STREAM_HANDLE streamToTest = (HFB_STREAM_HANDLE) NULL;
	
	/* if there aren't at least two streams, there's nothing to switch to */
	if(numberOfStreamsInArray < 2)
	{
		return currentStream;
	}

	/* check each stream in the array for key frame */
	/* NOTE: streams are arraged in the array from lowest precedence to highest */
	while(arrayIndex < numberOfStreamsInArray)
	{
		/* get the stream handle from the array, as well as its current frame */
		streamToTest = streamArray[arrayIndex];
		frameToTest = HFB_GetStreamingData(streamToTest,&data,&vidlength,DUK_FORWARD,0);

		/* we only allow switching on a keyframe boundary */
		if (HFB_GetIndexFlags(streamToTest,HFB_INDEXNUM,frameToTest) & HFB_DATA_KEYFRAME)  /*FIXME - eats frames*/
		{
			/* check the bit rate for the next "desiredProjectionSpan" frames */
			spanOfProjection = desiredProjectionSpan;
			/* the returned projected bit rate is actually in bytes, so adjust */
			projectedBitRate = HFB_ProjectBitRate(streamToTest, frameToTest, &spanOfProjection) << 3;
			/* if we are under the bit rate ceiling we'll use this stream */
			if(projectedBitRate < bitRateCeiling)
			{
				bestFitStream = arrayIndex;
			}
			
			/* if this stream has the lowest bit rate of any stream on a keyframe, */
			/* we'll store it away, in case we get into trouble later.             */
			if (projectedBitRate < lowestStreamBitRate)
			{
				lowestStreamBitRate = projectedBitRate;
				lowestStreamOnKeyframe = arrayIndex;
			}
		}
		/* check the next stream */
		arrayIndex++;
	}

	/* if we can't switch streams, (we haven't found any on a keyframe), we're done */
	if (lowestStreamBitRate == 0x7fffffff)
	{
		return currentStream;
	}

	/* The best fit stream is either the current stream, or one with higher precedence */
	/* NOTE: streams are arraged in the array from lowest precedence to highest */
	if (bestFitStream >= currentStream)
	{
		return bestFitStream;
	}

	/* We've only checked streams currently on a key-frame.  The currently selected */
	/* stream may be the best bet, but may not be on a key-frame.  If the currently */
	/* selected stream is the best, we won't have to switch streams, so we don't    */
	/* care if we're on a key-frame.  So give the current stream a chance to        */
	/* override even though it may not be on a keyframe.                            */

	/* get the stream handle from the array, as well as its current frame */
	streamToTest = streamArray[currentStream];
	frameToTest = HFB_GetStreamingData(streamToTest,&data,&vidlength,DUK_FORWARD,0);

	/* check the bit rate for the next "desiredProjectionSpan" frames */
	spanOfProjection = desiredProjectionSpan;
	/* the returned projected bit rate is actually in bytes, so adjust */
	projectedBitRate = HFB_ProjectBitRate(streamToTest,frameToTest,&spanOfProjection) << 3;

	/* if the current stream is under the bit rate ceiling we'll use this stream. */
	/* If there is a best fit stream, the current stream has higher precedence.   */
	if(projectedBitRate < bitRateCeiling)
	{
		return currentStream;
	}

	/* the current stream is above the ceiling, but we found a stream that is below - switch */
	if (bestFitStream >= 0)
	{
		return bestFitStream;
	}

	/* Nuts!  The current stream is above the ceiling, and so are the streams that */
	/* are on a keyframe.  So we don't have ANY valid streams.  Let's try to       */
	/* minimize the damage by selecting the lowest bit rate stream we can use.     */
	if (projectedBitRate < lowestStreamBitRate)
	{
		return currentStream;
	}
	return lowestStreamOnKeyframe;
}
