#if !defined(VPSTREAMDATA_HPP)
#define VPSTREAMDATA_HPP
//______________________________________________________________________________
//
//  VPStreamData.hpp
//  

//______________________________________________________________________________
//  Include Files and Forward Declarations

#include <iosfwd>

namespace on2vp
{

//______________________________________________________________________________
//  Macro, Enumeration, and Constant Definitions

//______________________________________________________________________________
//  Type, Struct, and Class Definitions

    //--------------------------------------
    class StreamData
    {
        friend std::ostream& operator<<(std::ostream& os, const StreamData& sd);

    public:
        enum
        {
            Null = -1
        };

        StreamData();
        StreamData(unsigned long ulSize, const unsigned char* const pData);
        StreamData(const StreamData& sd);
        ~StreamData();

        StreamData& operator=(const StreamData& sd);

        unsigned long sizeOrig() const;
        unsigned long versionOrig() const;
        unsigned long size() const;
        unsigned long version() const;
        const unsigned char* data() const;

        int data(unsigned long ulSize, const unsigned char* pData);

        //  VP specific member access

        long playerBitRate() const;
        long playerBufferMax() const;
        long playerPrebufferMax() const;
        long playerPrebufferRequired() const;

        long playerFrameWidth() const;
        long playerFrameHeight() const;

        long& playerBitRate();
        long& playerBufferMax();
        long& playerPrebufferMax();
        long& playerPrebufferRequired();

        void playerBuffer(long lPlayerBitRate, long lPlayerBufferMax, long lPlayerPrebufferMax);
        void playerFrameSize(long lPlayerFrameWidth, long lPlayerFrameHeight);

    private:
        enum
        {
            SizeMin = 4,
            SizeV1 = 20,
            SizeV2 = 28
        };

        enum
        {
            SizeCurrent = SizeV2
        };

        enum
        {
            VersionMax = 2
        };

        int extract_(unsigned long ulSize, const unsigned char* pData);
        void update_();

        unsigned long m_ulSizeOrig;
        unsigned long m_ulVersionOrig;

        unsigned char* m_pData;

        //  VP specific members

        long m_lPlayerBitRate;
        long m_lPlayerBufferMax;
        long m_lPlayerPrebufferMax;
        long m_lPlayerPrebufferRequired;

        long m_lPlayerFrameWidth;
        long m_lPlayerFrameHeight;
    };

//______________________________________________________________________________
//  Object and Function Declarations

//______________________________________________________________________________
//  Object and Function Definitions

}  //  namespace on2vp

#endif  //  VPSTREAMDATA_HPP
