/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "IVAconvert.h"
#ifdef WIN32
#include <stdexcept>
using namespace std;

BITMAPINFO&			to_BITMAPINFO(BITMAPINFO &w,const pmsio_BITMAPINFO &iva)
{
	if (w.bmiHeader.biSize != iva.bmiHeader.biSize)
		throw runtime_error("biSize field mismatch converting pmsio_BITMAPINFO to preallocated BITMAPINFO");

	to_BITMAPINFOHEADER(w.bmiHeader,iva.bmiHeader);

	size_t palette_bytes = iva.bmiHeader.biSize - sizeof(BITMAPINFOHEADER);
	if (palette_bytes)
		{
		size_t palette_entries = palette_bytes / sizeof(RGBQUAD);
		if (palette_entries)
			{
			for (size_t x = 0; x < palette_entries; ++x)
				{
				w.bmiColors[x].rgbBlue	= iva.bmiColors[x].rgbBlue;
				w.bmiColors[x].rgbGreen	= iva.bmiColors[x].rgbGreen;
				w.bmiColors[x].rgbRed		= iva.bmiColors[x].rgbRed;
				w.bmiColors[x].rgbReserved= iva.bmiColors[x].rgbReserved;
				}
			}
		}

	return w;
}

BITMAPINFOHEADER&	to_BITMAPINFOHEADER(BITMAPINFOHEADER &w,const pmsio_BITMAPINFOHEADER &iva)
{
	w.biSize			= iva.biSize;
	w.biWidth			= iva.biWidth;
	w.biHeight			= iva.biHeight;
	w.biPlanes			= iva.biPlanes;
	w.biBitCount		= iva.biBitCount;
	w.biCompression		= iva.biCompression;
	w.biSizeImage		= iva.biSizeImage;
	w.biXPelsPerMeter	= iva.biXPelsPerMeter;
	w.biYPelsPerMeter	= iva.biYPelsPerMeter;
	w.biClrUsed			= iva.biClrUsed;
	w.biClrImportant	= iva.biClrImportant;

	return w;
}

WAVEFORMATEX&		to_WAVEFORMATEX(WAVEFORMATEX &w,const pmsio_DKWAVEFORM &iva)
{
	if (w.cbSize != iva.cbSize)
		throw runtime_error("Cannot convert pmsio_DKWAVEFORM to preallocated WAVEFORMATEX with different cbSize field");

	w.wFormatTag		= iva.wFormatTag;
	w.nChannels		= iva.nChannels;
	w.nSamplesPerSec	= iva.nSamplesPerSec;
	w.nAvgBytesPerSec	= iva.nAvgBytesPerSec;
	w.nBlockAlign		= iva.nBlockAlign;
	w.wBitsPerSample	= iva.wBitsPerSample;
	w.cbSize			= iva.cbSize;

	if (iva.cbSize)
		memcpy(((char*)&w) + sizeof(WAVEFORMATEX),iva.mExtraData,iva.cbSize);

	return w;
}


// "news" returned struct. WILL copy dynamic bits. You must delete it when you're done
BITMAPINFO*			to_BITMAPINFO(const pmsio_BITMAPINFO &iva)
{
	BITMAPINFO* result = reinterpret_cast<BITMAPINFO*>(new char[iva.bmiHeader.biSize]);

	to_BITMAPINFOHEADER(result->bmiHeader,iva.bmiHeader);

	size_t palette_bytes = iva.bmiHeader.biSize - sizeof(BITMAPINFOHEADER);
	if (palette_bytes)
		{
		size_t palette_entries = palette_bytes / sizeof(RGBQUAD);
		if (palette_entries)
			{
			for (size_t x = 0; x < palette_entries; ++x)
				{
				result->bmiColors[x].rgbBlue	= iva.bmiColors[x].rgbBlue;
				result->bmiColors[x].rgbGreen	= iva.bmiColors[x].rgbGreen;
				result->bmiColors[x].rgbRed		= iva.bmiColors[x].rgbRed;
				result->bmiColors[x].rgbReserved= iva.bmiColors[x].rgbReserved;
				}
			}
		}

	return result;
}

BITMAPINFOHEADER*	to_BITMAPINFOHEADER(const pmsio_BITMAPINFOHEADER &iva)
{
	BITMAPINFOHEADER *result = reinterpret_cast<BITMAPINFOHEADER*>(new char[sizeof(BITMAPINFOHEADER)]);

	result->biSize			= iva.biSize;
	result->biWidth			= iva.biWidth;
	result->biHeight		= iva.biHeight;
	result->biPlanes		= iva.biPlanes;
	result->biBitCount		= iva.biBitCount;
	result->biCompression	= iva.biCompression;
	result->biSizeImage		= iva.biSizeImage;
	result->biXPelsPerMeter = iva.biXPelsPerMeter;
	result->biYPelsPerMeter = iva.biYPelsPerMeter;
	result->biClrUsed		= iva.biClrUsed;
	result->biClrImportant	= iva.biClrImportant;

	return result;
}

WAVEFORMATEX*		to_WAVEFORMATEX(const pmsio_DKWAVEFORM &iva)
{
	WAVEFORMATEX *result = 
		reinterpret_cast<WAVEFORMATEX*>(new char[sizeof(WAVEFORMATEX) + iva.cbSize]);

	result->wFormatTag		= iva.wFormatTag;
	result->nChannels		= iva.nChannels;
	result->nSamplesPerSec	= iva.nSamplesPerSec;
	result->nAvgBytesPerSec	= iva.nAvgBytesPerSec;
	result->nBlockAlign		= iva.nBlockAlign;
	result->wBitsPerSample	= iva.wBitsPerSample;
	result->cbSize			= iva.cbSize;

	if (iva.cbSize)
		memcpy(((char*)result) + sizeof(WAVEFORMATEX),iva.mExtraData,iva.cbSize);

	return result;
}


// WILL convert dynamic bits
pmsio_BITMAPINFO&		to_pmsio_BITMAPINFO(pmsio_BITMAPINFO &iva,const BITMAPINFO &w)
{
	to_pmsio_BITMAPINFOHEADER(iva.bmiHeader,w.bmiHeader);

	size_t palette_bytes = w.bmiHeader.biSize - sizeof(BITMAPINFOHEADER);
	if (palette_bytes)
		{
		size_t palette_entries = palette_bytes / sizeof(RGBQUAD);
		if (palette_entries)
			{
			iva.bmiColors = new pmsio_RGBQUAD[palette_entries];
			for (size_t x = 0; x < palette_entries; ++x)
				{
				iva.bmiColors[x].rgbBlue	= w.bmiColors[x].rgbBlue;
				iva.bmiColors[x].rgbGreen	= w.bmiColors[x].rgbGreen;
				iva.bmiColors[x].rgbRed		= w.bmiColors[x].rgbRed;
				iva.bmiColors[x].rgbReserved= w.bmiColors[x].rgbReserved;
				}
			}
		}

	return iva;
}

pmsio_BITMAPINFOHEADER&	to_pmsio_BITMAPINFOHEADER(pmsio_BITMAPINFOHEADER &iva,const BITMAPINFOHEADER &w)
{
	iva.biSize			= w.biSize;
	iva.biWidth			= w.biWidth;
	iva.biHeight		= w.biHeight;
	iva.biPlanes		= w.biPlanes;
	iva.biBitCount		= w.biBitCount;
	iva.biCompression	= w.biCompression;
	iva.biSizeImage		= w.biSizeImage;
	iva.biXPelsPerMeter = w.biXPelsPerMeter;
	iva.biYPelsPerMeter = w.biYPelsPerMeter;
	iva.biClrUsed		= w.biClrUsed;
	iva.biClrImportant	= w.biClrImportant;

	return iva;
}

pmsio_DKWAVEFORM&		to_pmsio_DKWAVEFORM(pmsio_DKWAVEFORM &iva,const WAVEFORMATEX &w)
{
	delete [] iva.mExtraData;

	iva.wFormatTag		= w.wFormatTag;
	iva.nChannels		= w.nChannels;
	iva.nSamplesPerSec	= w.nSamplesPerSec;
	iva.nAvgBytesPerSec	= w.nAvgBytesPerSec;
	iva.nBlockAlign		= w.nBlockAlign;
	iva.wBitsPerSample	= w.wBitsPerSample;
	iva.cbSize			= w.cbSize;
	iva.mExtraData		= 0;

	if (w.cbSize)
		{
		iva.mExtraData = new char[w.cbSize];
		memcpy(iva.mExtraData,((char *)(&w)) + sizeof(WAVEFORMATEX),w.cbSize);
		}
	
	return iva;
}


// "news" returned struct. You must delete it when you're done with it.
pmsio_BITMAPINFO*		to_pmsio_BITMAPINFO(const BITMAPINFO &w)
{
	pmsio_BITMAPINFO* result = new pmsio_BITMAPINFO;

	to_pmsio_BITMAPINFOHEADER(result->bmiHeader,w.bmiHeader);

	size_t palette_bytes = w.bmiHeader.biSize - sizeof(BITMAPINFOHEADER);
	if (palette_bytes)
		{
		size_t palette_entries = palette_bytes / sizeof(RGBQUAD);
		if (palette_entries)
			{
			result->bmiColors = new pmsio_RGBQUAD[palette_entries];
			for (size_t x = 0; x < palette_entries; ++x)
				{
				result->bmiColors[x].rgbBlue	= w.bmiColors[x].rgbBlue;
				result->bmiColors[x].rgbGreen	= w.bmiColors[x].rgbGreen;
				result->bmiColors[x].rgbRed		= w.bmiColors[x].rgbRed;
				result->bmiColors[x].rgbReserved= w.bmiColors[x].rgbReserved;
				}
			}
		}

	return result;
}

pmsio_BITMAPINFOHEADER*	to_pmsio_BITMAPINFOHEADER(const BITMAPINFOHEADER &w)
{
	pmsio_BITMAPINFOHEADER *result = new pmsio_BITMAPINFOHEADER;

	result->biSize			= w.biSize;
	result->biWidth			= w.biWidth;
	result->biHeight		= w.biHeight;
	result->biPlanes		= w.biPlanes;
	result->biBitCount		= w.biBitCount;
	result->biCompression	= w.biCompression;
	result->biSizeImage		= w.biSizeImage;
	result->biXPelsPerMeter = w.biXPelsPerMeter;
	result->biYPelsPerMeter = w.biYPelsPerMeter;
	result->biClrUsed		= w.biClrUsed;
	result->biClrImportant	= w.biClrImportant;

	return result;
}

pmsio_DKWAVEFORM*		to_pmsio_DKWAVEFORM(const WAVEFORMATEX &w)
{
	pmsio_DKWAVEFORM *result = new pmsio_DKWAVEFORM;

	result->wFormatTag		= w.wFormatTag;
	result->nChannels		= w.nChannels;
	result->nSamplesPerSec	= w.nSamplesPerSec;
	result->nAvgBytesPerSec	= w.nAvgBytesPerSec;
	result->nBlockAlign		= w.nBlockAlign;
	result->wBitsPerSample	= w.wBitsPerSample;
	result->cbSize			= w.cbSize;
	result->mExtraData		= 0;

	if (w.cbSize)
		{
		result->mExtraData = new char[w.cbSize];
		memcpy(result->mExtraData,((char *)(&w)) + sizeof(WAVEFORMATEX),w.cbSize);
		}
	
	return result;
}

#endif
