/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "intelFil.h"
#include "ffirChunk.h"
#include "iva_defines.h"
#include "ivaIO.h"
//#include "log.h"

using namespace std;

int
ffirChunkHeader::read(ivaIO &f, ffirChunkHeader &h)
{
	if (! h._FCC.read(f)) return 0;
	uchar buffer[4];
	if (f.Read(buffer, sizeof(buffer)) != sizeof(buffer)) return 0;
	intelReader reader(buffer);
	reader >> h._size;
	if (h._size % 2) ++h._size; // ...riff chunks are always word-aligned
	return 1;
}

int
ffirChunkHeader::write(ivaIO &f) const
{
	if (! _FCC.write(f)) return 0;
	uchar buffer[4];
	intelWriter writer(buffer);
	writer << _size;
	if (f.Write(buffer, sizeof(buffer)) != sizeof(buffer)) return 0;
	return 1;
}

int
ffirListChunkHeader::write(ivaIO &f) const
{
	if (! ffirChunkHeader::write(f)) return 0;
	if (! _list_FCC.write(f)) return 0;
	return 1;
}

int
ffirParser::fail(const char *method, const char *message) const
{
	cerr << "ffirParser::" << method << ": " << message << endl;
	return 0;
}

int
ffirParser::parse_list(const ffirChunkHeader &h, uint level) const
{
	const char *me = "parse_list";
	// read the list 4cc...
	ffirFCC list_FCC;
	if (!list_FCC.read(_f)) return fail(me, "read failed");
	size_t chunk_bytes = h.size() - 4; 
		// ...the first 4 data bytes were the list 4cc
	int skip=0;
	if (_handler) {
		ivaIO::FilePosition_t fpos;
		if ((fpos = _f.Getpos()) == ivaIO::sFilePositionError) 
			return fail(me, "getpos failed");
		if (!_handler->handle_list_chunk(_f, h, list_FCC, level, skip)) 
			return fail(me, "handler failed");
		if (_f.Seek(fpos,ivaIO::SET) == ivaIO::sSeekOffsetError) 
			return fail(me, "fsetpos failed");
	}
	if (skip) {
		if (_f.Seek(chunk_bytes,ivaIO::CUR) == ivaIO::sSeekOffsetError) 
			return fail(me, "seek failed");
		return 1;
	}
	++level;
	while (chunk_bytes > 0) {
		ffirChunkHeader sh; // ...subchunk header
		if (!ffirChunkHeader::read(_f, sh)) return fail(me, "read failed");
		if (!parse(sh, level)) return fail(me, "subchunk read failed");
		chunk_bytes -= sh.size() + 8; // ...including subchunk's header
	}
	return 1;
}

int
ffirParser::parse(const ffirChunkHeader &h, uint level) const
{
	const char *me = "parse";
	if (h.FCC().is_list()) return parse_list(h, level);
	// it's not a list...
	if (_handler) {
		ivaIO::FilePosition_t fpos;
		if ((fpos = _f.Getpos()) == ivaIO::sFilePositionError) 
			return fail(me, "getpos failed");
		if (!_handler->handle_chunk(_f, h, level))
			return fail(me, "handler failed");
		if (_f.Seek(fpos,ivaIO::SET) == ivaIO::sSeekOffsetError) 
			return fail(me, "seek failed");
	}
	long seek_width = h.size();
	if (_f.Seek(seek_width,ivaIO::CUR) == ivaIO::sSeekOffsetError) 
		return fail(me, "seek failed");
	return 1;
}

int
ffirParser::parse() const
{
	const char *me = "parse";
	_f.Seek(0,ivaIO::SET);
	ffirChunkHeader h;
	if (!ffirChunkHeader::read(_f, h)) 
		return fail(me, "read failed");
	return parse(h, 0);
}
