/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "ffirFCC.h"
#include "intelFil.h"
#include "ivaIO.h"
#include <stdio.h>
#include <stdlib.h>


#include <ctype.h>

ffirTCC *ffirTCC::_p_wb = 0;
ffirTCC *ffirTCC::_p_db = 0;
ffirTCC *ffirTCC::_p_dc = 0;


void
ffirTCC::startup()
{
	_p_wb = new ffirTCC("wb");
	_p_db = new ffirTCC("db");
	_p_dc = new ffirTCC("dc");

}

void 
ffirTCC::cleanup()
{
	delete _p_wb;
	delete _p_db;
	delete _p_dc;

}

ffirTCC::ffirTCC(ffirFCC fcc)
{
	strncpy(_id, fcc._id + 2, 2);
}

InitializorC(ffirTCCStatic, ffirTCC::startup();)
InitializorD(ffirTCCStatic, ffirTCC::cleanup();)

ffirTCC::ffirTCC(const char *id)
{
	memcpy(_id, id, sizeof(_id));
}

ffirFCC *ffirFCC::_p_RIFF = 0;
ffirFCC *ffirFCC::_p_LIST = 0;
ffirFCC *ffirFCC::_p_AVI  = 0;
ffirFCC *ffirFCC::_p_avih = 0;
ffirFCC *ffirFCC::_p_hdrl = 0;
ffirFCC *ffirFCC::_p_strl = 0;
ffirFCC *ffirFCC::_p_strf = 0;
ffirFCC *ffirFCC::_p_strh = 0;
ffirFCC *ffirFCC::_p_strd = 0;
ffirFCC *ffirFCC::_p_movi = 0;
ffirFCC *ffirFCC::_p_rec  = 0;
ffirFCC *ffirFCC::_p_idx1 = 0;
ffirFCC *ffirFCC::_p_auds = 0;
ffirFCC *ffirFCC::_p_vids = 0;
ffirFCC *ffirFCC::_p_JUNK = 0;


void
ffirFCC::startup()
{
	_p_RIFF = new ffirFCC("RIFF");
	_p_LIST = new ffirFCC("LIST");
	_p_AVI  = new ffirFCC("AVI ");
	_p_avih = new ffirFCC("avih");
	_p_hdrl = new ffirFCC("hdrl");
	_p_strl = new ffirFCC("strl");
	_p_strf = new ffirFCC("strf");
	_p_strh = new ffirFCC("strh");
	_p_strd = new ffirFCC("strd");
	_p_movi = new ffirFCC("movi");
	_p_rec  = new ffirFCC("rec ");
	_p_idx1 = new ffirFCC("idx1");
	_p_auds = new ffirFCC("auds");
	_p_vids = new ffirFCC("vids");
	_p_JUNK = new ffirFCC("JUNK");

}

void 
ffirFCC::cleanup()
{
	delete _p_RIFF;
	delete _p_LIST;
	delete _p_AVI ;
	delete _p_avih;
	delete _p_hdrl;
	delete _p_strl;
	delete _p_strf;
	delete _p_strh;
	delete _p_strd;
	delete _p_movi;
	delete _p_rec ;
	delete _p_idx1;
	delete _p_auds;
	delete _p_vids;
	delete _p_JUNK;

}

InitializorC(ffirFCCStatic, ffirFCC::startup();)
InitializorD(ffirFCCStatic, ffirFCC::cleanup();)

bool
ffirFCC::read(ivaIO &f)
{
	return f.Read(_id, sizeof(_id)) == sizeof(_id);
}

bool
ffirFCC::write(ivaIO &f) const
{
	return f.Write((void *)_id, sizeof(_id)) == sizeof(_id);
}

bool
ffirFCC::write(char *s) const
{
	memcpy(s,_id,sizeof(_id));
	return 1;
}

ffirFCC::ffirFCC(const char *id)
{
	memcpy(_id, id, sizeof(_id));
}

ffirFCC::ffirFCC(ulong id)
{
	intelWriter w((uchar*) &_id);
	w << id;
}

#if(0)
ffirFCC::ffirFCC(int stream_number, const char *twocc)
{
	sprintf(_id, "%02d", stream_number);
	strncpy(_id + 2, twocc, 2);
}
#endif

ffirFCC::ffirFCC(int stream_number, ffirTCC twocc)
{
	sprintf(_id, "%02d", stream_number);
	strncpy(_id + 2, twocc._id, 2);
}

ffirFCC::operator ulong() const
{
	intelReader r((uchar*) &_id);
	ulong u;
	r >> u;
	return u;
}

int
ffirFCC::extract_stream_number(ulong &n) const
{
	if (!isdigit(_id[0]) || !isdigit(_id[1])) return 0;
	n = atoi(_id);
	return 1;
}

int 
ffirFCC::is_list() const 
{
	return (*this == "LIST") || (*this == "RIFF");
}

