/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ffirFCC_included
#define ffirFCC_included

/*
	ffirFCC: 4 character code for riff chunks identifiers
	ffirTCC: 2 character code for subpart of riff chunks
*/

#include <string.h>
#include <iostream>

#include "iva_timstyle.h"
class ivaIO;

#ifdef WIN32 //busted VC compiler
class ffirTCC;
std::ostream& operator<<(std::ostream &,const ffirTCC&); // ...convenience
int operator==(ffirTCC, ffirTCC);
#endif

class ffirTCC {
public:
	// methods...
	friend class ffirTCCStatic;
	friend class ffirFCC;
	ffirTCC(const char *);
	ffirTCC(ffirFCC);
	ffirTCC() {memcpy(_id, "??", sizeof(_id));}
	friend int operator==(ffirTCC, ffirTCC);
	friend int operator!=(ffirTCC, ffirTCC);
	friend std::ostream& operator<<(std::ostream &,const ffirTCC&); // ...convenience
	void export_data(char *d) const { memcpy(d,_id,id_size()); }
public:
	// class methods...
	static unsigned long id_size() {return 2;}
	static ffirTCC &wb() {return *_p_wb;}
	static ffirTCC &db() {return *_p_db;}
	static ffirTCC &dc() {return *_p_dc;}

private:
	// members...
	char _id[2];
private:
	// class methods...
	static void startup();
	static void cleanup();
	// class members...
	static ffirTCC *_p_wb;
	static ffirTCC *_p_db;
	static ffirTCC *_p_dc;

};
Initializor(ffirTCCStatic)

inline std::ostream& operator<<(std::ostream &os,const ffirTCC &id) {
	os.write(id._id, sizeof(id._id));
	return os;
}

#ifdef WIN32
class ffirFCC;
std::ostream& operator<<(std::ostream &,const ffirFCC&); // ...convenience
int operator==(ffirFCC, ffirFCC);
#endif

class ffirFCC {
public:
	// methods...
	bool read(ivaIO&);
	bool write(ivaIO&) const;
	bool write(char *) const;
	inline void write(unsigned long *l) const
	{ *l = *(unsigned long *)(&_id); }
	friend class ffirFCCStatic;
	friend class ffirTCC;
	inline friend int operator==(ffirFCC, ffirFCC);
	friend std::ostream& operator<<(std::ostream &,const ffirFCC&); // ...convenience
	int is_list() const;
	ffirFCC(const char *);
	ffirFCC(int, ffirTCC); // ...constructer from stream number and twocc
	ffirFCC(ulong);
	ffirFCC() {memcpy(_id, "????", sizeof(_id));}
	operator ulong() const;
	int extract_stream_number(ulong &) const;

public:
	// class methods...
	static unsigned long id_size() {return 4;}
	static ffirFCC &RIFF() {return *_p_RIFF;}
	static ffirFCC &LIST() {return *_p_LIST;}
	static ffirFCC &AVI () {return *_p_AVI ;}
	static ffirFCC &avih() {return *_p_avih;}
	static ffirFCC &hdrl() {return *_p_hdrl;}
	static ffirFCC &strl() {return *_p_strl;}
	static ffirFCC &strf() {return *_p_strf;}
	static ffirFCC &strh() {return *_p_strh;}
	static ffirFCC &strd() {return *_p_strd;}
	static ffirFCC &movi() {return *_p_movi;}
	static ffirFCC &rec () {return *_p_rec ;}
	static ffirFCC &idx1() {return *_p_idx1;}
	static ffirFCC &auds() {return *_p_auds;}
	static ffirFCC &vids() {return *_p_vids;}
	static ffirFCC &JUNK() {return *_p_JUNK;}

private:
	// members...
	char _id[4];
private:
	// class methods...
	static void startup();
	static void cleanup();
	// class members...
	static ffirFCC *_p_RIFF;
	static ffirFCC *_p_LIST;
	static ffirFCC *_p_AVI ;
	static ffirFCC *_p_avih;
	static ffirFCC *_p_hdrl;
	static ffirFCC *_p_strl;
	static ffirFCC *_p_strf;
	static ffirFCC *_p_strh;
	static ffirFCC *_p_strd;
	static ffirFCC *_p_movi;
	static ffirFCC *_p_rec ;
	static ffirFCC *_p_idx1;
	static ffirFCC *_p_auds;
	static ffirFCC *_p_vids;
	static ffirFCC *_p_JUNK;

};
Initializor(ffirFCCStatic)

inline int operator==(ffirFCC cc1, ffirFCC cc2) {
	return strncmp(cc1._id, cc2._id, 4) == 0;
}
inline int operator!=(ffirFCC cc1, ffirFCC cc2) {
	return !(cc1 == cc2);
}

inline int operator==(ffirTCC cc1, ffirTCC cc2) {
	return strncmp(cc1._id, cc2._id, 2) == 0;
}
inline int operator!=(ffirTCC cc1, ffirTCC cc2) {
	return !(cc1 == cc2);
}

inline std::ostream& operator<<(std::ostream &os,const ffirFCC &id) {
	os.write(id._id, sizeof(id._id));
	return os;
}


#endif // ffirFCC_included
