/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef savi_included
#define savi_included

// $Id: iva.h,v 1.2 2001/12/12 21:03:24 nradisch Exp $
// Abstract interface to avi file services.
// An avi file is modelled as a collection of audio, video, and other streams.
//
// Alan S. Rojer
// The Duck Corp.
// 1 Sept 1997

#include "MegaHeaders.h"
#include "pmsio.h"
#include "pmsio_BITMAPINFO.h"
#include "ffirFCC.h"
#include "ivaFrame.h"
#include "iva_defines.h"

// what we've got here...
class ivaFile;
class ivaReader;
class ivaReadStreamBase;
template <class StreamFormat> class ivaReadStream;
class ivaStreamReader;
class ivaWriter;
class ivaWriteStream;

typedef pmsio_MainAVIHeader ivaHeader;
typedef pmsio_AVIStreamHeader ivaStreamHeader;
typedef pmsio_BITMAPINFO ivaVideoHeader;
typedef pmsio_DKWAVEFORM ivaAudioHeader;
typedef ivaFrame ivaOtherHeader;
typedef ivaFrame ivaDriverData;

typedef ivaReadStream<ivaVideoHeader> ivaVideoReadStream;
typedef ivaReadStream<ivaAudioHeader> ivaAudioReadStream;
typedef ivaReadStream<ivaOtherHeader> ivaOtherReadStream;

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;

typedef unsigned int uint;

class ivaFile {
public:
	virtual ~ivaFile () {}
	virtual bool open(const char *fname) = 0;
	virtual int is_open() const = 0;
	virtual int close(int recalc_framerate = 0) = 0;
	virtual const ivaHeader* header() const = 0;
};

class ivaReader : public ivaFile {
public:
	virtual ~ivaReader() {}
	virtual const ivaVideoReadStream* video_stream(size_t i=0) const = 0;
	virtual const ivaAudioReadStream* audio_stream(size_t i=0) const = 0;
	virtual const ivaOtherReadStream* other_stream(size_t i=0) const = 0;
public:
	// factory methods for reference implementation...
	// caller must delete returned objects...
	//static ivaReader *reference();
};

class ivaReadStreamBase {
public:
	virtual ~ivaReadStreamBase() {}
	virtual const ivaStreamHeader* header() const = 0;
	// this will return 0 unless there was driver data...
	virtual const ivaDriverData *driver_data() const = 0;

	// file reader we're drawn from...
	virtual const ivaReader *reader() const = 0;

	// iterater to supply frames...
	// caller must delete the stream reader...
	virtual ivaStreamReader *stream_reader(ulong start=0, ulong end=-1) const = 0;
	
	// information about the frames
	virtual	U32		indicies(void)  const = 0;
	virtual	U32		total_data(void) const = 0;
	virtual	U32		chunk_size(U32 index) const = 0;
};

template <class StreamFormat>
class ivaReadStream : public ivaReadStreamBase {
public:
	virtual ~ivaReadStream() {}
	virtual const StreamFormat* stream_format() const = 0;
};

class ivaStreamReader {
public:
	virtual ~ivaStreamReader() {}
	virtual const ivaReadStreamBase *stream() const = 0;
	virtual int read(ivaFrame &, int &is_key, ffirTCC &twocc) = 0;
		// the caller supplies the frame, which must contain 
		// a sufficiently sized buffer
		// ...returns 0 on end or error, 1 otherwise
	virtual	int	read(U32 index,ivaFrame &,int &is_key, ffirTCC &twocc) = 0;
};

class ivaReaderFactory {
public:
	virtual ivaReader *make(void) = 0;
};


class ivaWriter : public ivaFile {
public:
	virtual ~ivaWriter() {}
	virtual ivaWriteStream* create_stream(
		const ivaStreamHeader *,
		const ivaVideoHeader *,
		const ivaDriverData *p_dd = 0)= 0;
	virtual ivaWriteStream* create_stream(
		const ivaStreamHeader *,
		const ivaAudioHeader *,
		const ivaDriverData *p_dd = 0)= 0;
	virtual ivaWriteStream* create_stream(
		const ivaStreamHeader *,
		const ivaOtherHeader *,
		const ivaDriverData *p_dd = 0)= 0;
public:
	// factory methods for reference implementation...
	// caller must delete returned objects...
	//static ivaWriter *reference(const ivaHeader&, 
	//	size_t header_reserved_size = 2048,
	//	size_t chunk_alignment = 1); 
};

class ivaWriteStream {
public:
	virtual ~ivaWriteStream() {}
	virtual int write(const ivaFrame &, int is_key, ffirTCC twocc) = 0;
	virtual ivaFrame *write_getframe(size_t siz) = 0;
	virtual	int		write_returnframe(ivaFrame *f,int is_key, ffirTCC twocc) = 0;
};

class ivaWriterFactory {
public:
	virtual ivaWriter *make(const ivaHeader&,
		size_t header_reserved_size = 2048,
		size_t chunk_alignment = 1) = 0;
};

#endif // savi_included
