/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ivaFilter_included
#define ivaFilter_included

// $Id: ivaFilter.h,v 1.1 2001/12/12 18:47:44 nradisch Exp $
// Wrapper class to ease provision of avi filters.  Here, a filter 
// is a console app that reads, processes, and writes an avi file.
//
// Alan S. Rojer
// The Duck Corp.
// Mon Sep 22 08:39:00 EDT 1997

#include "iva.h"

class ivaFilterDriver;

class ivaFileFilter {
public:
	virtual int open(ivaFilterDriver &,
		const ivaHeader &read_header, 
		ivaHeader &write_header) = 0;
			// ...open must also register user stream filters against driver
			// ...open must make any changes to write header also
			// (which is initilized to read_header)
	virtual int close() = 0;
public:
	virtual ~ivaFileFilter() {}
};

class ivaStreamFilterBase {
public:
	virtual int apply(const ivaFrame &domain, ivaFrame &range) = 0;
		// ...caller (FilterDriver) populates the domain frame
		// ...StreamFilter must populate the range frame
	// open() resides in derived (templatized) classes
	virtual int close() = 0;
public:
	virtual ~ivaStreamFilterBase() {}
};

template <class StreamFormat>
class ivaStreamFilter : public ivaStreamFilterBase {
public:
	virtual int open(ivaFilterDriver&, 
		// supplied to the stream filter...
		ivaStreamHeader *, StreamFormat*, ivaDriverData*,
		// stream filter must provide...
		ivaWriteStream **, ivaFrame **p_domain, ivaFrame **p_range) = 0;
	// ...handler must create write stream, read/write frames
	// ...the memory header/driver is transient; 
	// make a copy if you need to refer to it after initialization.
public:
	virtual ~ivaStreamFilter() {}
};

typedef ivaStreamFilter<ivaVideoHeader> ivaVideoFilter;
typedef ivaStreamFilter<ivaAudioHeader> ivaAudioFilter;
typedef ivaStreamFilter<ivaOtherHeader> ivaOtherFilter;

class ivaFilterDriver {
public:
	// registration...
	virtual int register_filter(ivaVideoFilter &) = 0;
	// ...if there's more than video filter, register in stream order!
	// processing audio & other streams unsupported for now...any demand?
	// audio & other streams will be passed through the filter untouched.
public:
	// control flow...
	int open(int arg, char **argv, int argc); // ...convenience; uses open() below
	virtual int open(const char *input_fname, const char *output_fname) = 0;
		// ...calls the filter opens
		// ...arg should point past the options in argv...
		// ...expects two file names at argv[arg+1], argv[arg+2]...
	virtual int apply() = 0;
	virtual int close() = 0;
public:
	// access to members...
	virtual ivaWriter *writer() = 0;
public:
	// factory...
	//static ivaFilterDriver *reference_driver(ivaFileFilter&);
	~ivaFilterDriver() {}
private:
	int fail(const char *method, const char *message) const;
};

#endif // ivaFilter_included
