/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ivaFrame_included
#define ivaFrame_included

// $Id: ivaFrame.h,v 1.1 2001/12/12 18:47:45 nradisch Exp $
//
// Alan S. Rojer
// The Duck Corp.
// Tue Sep 16 13:49:17 EDT 1997

#include <stdio.h>

class ivaIO;

typedef unsigned char uchar;

class ivaFrame {
public:
	virtual ~ivaFrame() {}
	virtual uchar *buffer() const = 0;
	virtual size_t buffer_size() const = 0;
	virtual size_t &length() = 0;
	virtual const size_t &length() const = 0;
	virtual ivaFrame *clone() const = 0;
	// for compatibility w/pmsio stuff...
	int write(ivaIO&) const; 
};

class ivaTrivialFrame : public ivaFrame {
public:
	ivaTrivialFrame(uchar * buffer, size_t buffer_size)
	: _buffer(buffer), _buffer_size(buffer_size), _length(0) {}
	ivaTrivialFrame(void) {}
	virtual ~ivaTrivialFrame() {}
public:
	uchar * buffer() const {return _buffer;}
	size_t buffer_size() const {return _buffer_size;}
	size_t &length() {return _length;}
	const size_t &length() const {return _length;}
	virtual ivaFrame *clone() const;
	void	setbuffer(uchar *buffer,size_t siz) {_buffer = buffer; _buffer_size = siz; }
private:
	uchar * _buffer;
	size_t _buffer_size;
	size_t _length;
};

class ivaOwnerFrame : public ivaTrivialFrame {
public:
	ivaOwnerFrame(uchar * buffer, size_t buffer_size)
	: ivaTrivialFrame(buffer, buffer_size) {}
	ivaOwnerFrame(size_t buffer_size)
	: ivaTrivialFrame(new uchar [buffer_size], buffer_size) {}
	~ivaOwnerFrame() {delete [] buffer();}
	ivaFrame *clone() const;
private:
	// copy and assignment prohibited...use clone
	ivaOwnerFrame& operator=(const ivaOwnerFrame&);
	ivaOwnerFrame(const ivaOwnerFrame&);
};

#endif // ivaFrame_included
