/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __POWERPC__
// doesnt compile on mac yet

#include "ivaIOLog.h"

// standard io stuff, but also logs the calls
#include <sys/types.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <fcntl.h>      
#ifdef WIN32
#include <io.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <fstream>
#include <iostream>
#include <assert.h>

#ifndef WIN32
#define	_open		open
#define _read		read
#define	_write		write
#define	_lseek		lseek
#define _close		close
#define _O_CREAT	O_CREAT
#define _O_RDONLY	O_RDONLY
#define _O_WRONLY	O_WRONLY
#ifdef __POWERPC__
#define _O_BINARY	O_BINARY
#else
#define _O_BINARY	0
#endif
#endif

using namespace std;

int	
ivaIOLog::Read(void *buffer,size_t amt)
{
	assert(mFileDes != -1);
	cout << "Read " << amt << endl;
	return _read(mFileDes,buffer,amt);
}

int	
ivaIOLog::Write(void *buffer,size_t amt)
{
	assert(mFileDes != -1);
	cout << "Write " << amt << endl;
	return _write(mFileDes,buffer,amt);
}

bool
ivaIOLog::IsOpen(void) const 
{
	cout << "IsOpen" << endl;
	return (mFileDes != -1);
}

bool	
ivaIOLog::Open(const char *filename,OpenMode_t mode)
{
	assert(mFileDes == -1);
	cout << "Open " << filename << " for "
		<< (mode == ivaIO::READ ? "reading" : "writing") << endl;
	switch(mode)
		{
		case ivaIO::READ:
			mFileDes = _open(filename,_O_RDONLY | _O_BINARY);
			break;
		case ivaIO::WRITE:
#ifdef WIN32
			mFileDes = _open(filename,_O_WRONLY | _O_BINARY | _O_CREAT);
#else
			mFileDes = _open(filename,_O_WRONLY | _O_BINARY | _O_CREAT,0666);
#endif
			break;
		default:
			assert(0);
			break;
		}

	return (mFileDes != -1);
}

bool	
ivaIOLog::Close(void)
{
	assert(mFileDes != -1);
	cout << "Close" << endl;
	int	tmp = mFileDes;
	mFileDes = -1;
	return (_close(tmp) != -1);
}

ivaIO::SeekOffset_t
ivaIOLog::Seek(ivaIO::SeekOffset_t offset,Seek_t seek_type)
{
	assert(mFileDes != -1);
	cout << "Seek to " << offset << " " <<
		( seek_type == ivaIO::SET ? "SET" : 
			(seek_type == ivaIO::CUR ? "CUR" : "END")) << endl;
	int whence;
	switch(seek_type)
		{
		case ivaIO::SET:
			whence = SEEK_SET;
			break;
		case ivaIO::CUR:
			whence = SEEK_CUR;
			break;
		case ivaIO::END:
			whence = SEEK_END;
			break;
		}
	return _lseek(mFileDes,offset,whence);
}


ivaIO::FilePosition_t 
ivaIOLog::Getpos(void)
{
	ivaIO::FilePosition_t res = _lseek(mFileDes,0,SEEK_CUR);
	cout << "Getpos " << res << endl;
	return res;
}


#endif
