/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef iva_DefaultFilter_included
#define iva_DefaultFilter_included

// $Id: iva_DefaultFilter.h,v 1.1 2001/12/12 18:47:49 nradisch Exp $
//
// Alan S. Rojer
// The Duck Corp.
// Fri Sep 26 13:19:56 EDT 1997

#include "ivaFilter.h"

template <class StreamFormat>
class iva_DefaultFilter: public ivaStreamFilter<StreamFormat> {
public:
	int open(
		ivaFilterDriver &driver,
		ivaStreamHeader *header,
		StreamFormat *stream_format, 
		ivaDriverData *driver_data,
		ivaWriteStream **p_write_stream,
		ivaFrame **p_domain,
		ivaFrame **p_range)
	{
		*p_write_stream = driver.writer()->create_stream(header, 
			stream_format, driver_data);
		size_t buffer_size = header->dwSuggestedBufferSize
			? header->dwSuggestedBufferSize
			: 0x800000; // ...a big honking buffer! (about 8 meg)
		uchar *buffer = new uchar [buffer_size];
		*p_domain = new ivaOwnerFrame(buffer, buffer_size);
		*p_range = new ivaTrivialFrame(buffer, buffer_size);
			// ...range and domain share the same buffer; domain owns it
		return buffer && *p_write_stream && *p_domain && *p_range;
	}
	int apply(const ivaFrame &domain, ivaFrame &range)
	{
		range.length() = domain.length();
			// ...BEWARE; EFFICIENCY HACK HERE...
			// domain and range must share the same buffer!
			// derived classes must ensure this! (cf iva_DefaultFilter::open())
		return 1;
	}
	int close() {return 1;}
public:
	// factory...
	iva_DefaultFilter() {}
	~iva_DefaultFilter() {}
};

typedef iva_DefaultFilter<ivaVideoHeader> iva_DefaultVideoFilter;
typedef iva_DefaultFilter<ivaAudioHeader> iva_DefaultAudioFilter;
typedef iva_DefaultFilter<ivaOtherHeader> iva_DefaultOtherFilter;

#endif // iva_DefaultFilter_included
