/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __POWERPC__
// I haven't ported this to the mac yet

//	$Id: iva_Filter.cxx,v 1.1 2001/12/12 18:47:50 nradisch Exp $
//
// Alan S. Rojer
// The Duck Corp.
// Mon Sep 22 09:16:05 EDT 1997

#include "iva_Filter.h"
#include "ivaIO.h"

typedef iva_FilterDriverStreamBundle Bundle;

int
iva_FilterDriver::fail(const char *method, const char *message)
{
	const char *class_name = "iva_FilterDriver";
	cerr << class_name << "::" << method << ": " << message << endl;
	return 0;
}

iva_FilterDriver::iva_FilterDriver(ivaFileFilter &filter,
								   ivaIO &inf,ivaIO &outf,
								   ivaReaderFactory &rf,
								   ivaWriterFactory &wf)
: 
	_in_file(inf),
	_out_file(outf),
	_reader_factory(rf),
	_writer_factory(wf),
	_filter(&filter), 
	_writer(0),
	_streams_accepted(0), 
	_video_streams_accepted(0)
{
}

iva_FilterDriver::~iva_FilterDriver()
{
	vector<ivaStreamFilterBase*>::iterator ii = _allocated_filters.begin();
	while (ii != _allocated_filters.end()) {
		delete (*ii);
		++ii;
	}
	delete _writer;
}

int
iva_FilterDriver::register_filter(ivaVideoFilter &filter)
{
	const char *me = "register_filter";
	_video_filters.push_back(&filter);
	return 1;
}

// clone for audio, other?

int
iva_FilterDriver::accept_header(ivaHeader *header)
{
	const char *me = "accept_header";
	ivaHeader write_header(*header);
	if (!_filter->open(*this, *header, write_header))
		return fail(me, "filter open failed");
	write_header.dwFlags 
		= AVIF_HASINDEX  // ...we're going to make one!
		| AVIF_WASCAPTUREFILE // ...mysteriously, this flag cures ActiveMovie 
			// crashing on exit!
		;
	_writer = _writer_factory.make(write_header);
	if (!_writer) return fail(me, "writer could not be created");
	if (!_writer->open(_out_fname)) return fail(me, "couldn't open output file");
	delete header;
	return 1;
}

int
iva_FilterDriver::accept_index_entry(const ivaIndexEntry &ie)
{
	const char *me = "accept_index_entry";
	_index.push_back(ie);
	return 1;
}


int
iva_FilterDriver::open(const char *in_fname, const char *out_fname)
{
	const char *me = "open";
	// open the input file...
	_in_fname = in_fname;
	_out_fname = out_fname;
	if (!_in_file.Open(_in_fname,ivaIO::READ))
		 return fail(me, "couldn't open input file");

	// parse the file... this will get the streams opened, and build the index
	iva_Parser parser(_in_file, this);
	if (!parser.parse()) return fail(me, "input parse failed");
	return 1;
}

int
iva_FilterDriver::apply()
{
	const char *me = "apply";
	vector<ivaIndexEntry>::const_iterator ii = _index.begin();
	while (ii != _index.end()) {
		ffirFCC FCC((*ii).ckid);
		ulong offset = (*ii).dwChunkOffset;
		ulong length = (*ii).dwChunkLength;
		int is_key = ((*ii).dwFlags & AVIIF_KEYFRAME); 
			// should be determined by apply?

		// is there a filter for this stream?
		ulong n;
		if (!FCC.extract_stream_number(n)) continue; // ...ie, ignore it
		Bundle bundle;
		if (n < _bundles.size()) bundle = _bundles[n];
		else continue;
		if (!bundle._filter) continue; // ...nothing to do

		// there's a filter for this stream...

		// read the domain frame...
		ivaFrame *domain = bundle._domain;
		if (length > domain->buffer_size())
			return fail(me, "frame buffer too small");
		if (_in_file.Seek(offset,ivaIO::SET) == ivaIO::sSeekOffsetError) return fail(me, "seek failed");
		ffirChunkHeader h;
		if (!ffirChunkHeader::read(_in_file, h))
			return fail(me, "header read failed");
		if (length > 0) {
			if (_in_file.Read(domain->buffer(), length) != length)
				return fail(me, "fread failed");
			domain->length() = length;
		}
		else domain->length() = 0;

		// apply the filter...
		ivaFrame *range = bundle._range;
		if (!bundle._filter->apply(*domain, *range)) 
			return fail(me, "filter failed");

		// write the range frame...
		if (!bundle._write_stream->write(*range, is_key, FCC))
			return fail(me, "write failed");
	}
	return 1;
}	

int
iva_FilterDriver::close()
{
	const char *me = "close";
	if (!_writer->close()) return fail(me, "output close failed");
	vector<Bundle>::const_iterator ii = _bundles.begin();
	while (ii != _bundles.end()) {
		if (!(*ii)._filter->close()) return fail(me, "stream filter close failed");
	}
	if (!_filter->close()) return fail(me, "filter close failed");
	return 1;
}

int
iva_FilterDriver::accept_video_header(
	ivaStreamHeader *stream_header, 
	ivaVideoHeader *video_header,
	ivaDriverData *driver_data)
{
	ivaVideoFilter *filter=0;
	if (_video_streams_accepted < _video_filters.size()) {
		filter = _video_filters[_video_streams_accepted];
	}
	else {
		filter = new iva_DefaultVideoFilter;
		_allocated_filters.push_back(filter);
	}
	Bundle bundle;
	bundle._filter = filter;
	if (!filter->open(*this, stream_header, video_header, driver_data,
			&bundle._write_stream, &bundle._domain, &bundle._range))
		return 0;
	_bundles.push_back(bundle);
	++ _video_streams_accepted;
	++ _streams_accepted;
	delete stream_header;
	delete video_header;
	delete driver_data;
	return 1;
}

// accept_audio & accept_other are clones of accept_video...

int
iva_FilterDriver::accept_audio_header(
	ivaStreamHeader *stream_header, 
	ivaAudioHeader *audio_header,
	ivaDriverData *driver_data)
{
	ivaAudioFilter *filter=0;
	if (_audio_streams_accepted < _audio_filters.size()) {
		filter = _audio_filters[_audio_streams_accepted];
	}
	else {
		filter = new iva_DefaultAudioFilter;
		_allocated_filters.push_back(filter);
	}
	Bundle bundle;
	bundle._filter = filter;
	if (!filter->open(*this, stream_header, audio_header, driver_data,
			&bundle._write_stream, &bundle._domain, &bundle._range))
		return 0;
	_bundles.push_back(bundle);
	++ _audio_streams_accepted;
	++ _streams_accepted;
	delete stream_header;
	delete audio_header;
	delete driver_data;
	return 1;
}

int
iva_FilterDriver::accept_other_header(
	ivaStreamHeader *stream_header, 
	ivaOtherHeader *other_header,
	ivaDriverData *driver_data)
{
	ivaOtherFilter *filter=0;
	if (_other_streams_accepted < _other_filters.size()) {
		filter = _other_filters[_other_streams_accepted];
	}
	else {
		filter = new iva_DefaultOtherFilter;
		_allocated_filters.push_back(filter);
	}
	Bundle bundle;
	bundle._filter = filter;
	if (!filter->open(*this, stream_header, other_header, driver_data,
			&bundle._write_stream, &bundle._domain, &bundle._range))
		return 0;
	_bundles.push_back(bundle);
	++ _other_streams_accepted;
	++ _streams_accepted;
	delete stream_header;
	delete other_header;
	delete driver_data;
	return 1;
}
#endif
