/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef iva_Filter_included
#define iva_Filter_included

// $Id: iva_Filter.h,v 1.1 2001/12/12 18:47:50 nradisch Exp $
//
// Reference implementation of iva filter driver...
//
// Alan S. Rojer
// The Duck Corp.
// Mon Sep 22 09:16:05 EDT 1997

#include "ivaFilter.h"
#include "iva_Parser.h"
#include "iva_DefaultFilter.h"
#include <vector>
using namespace std;

// for internal use only...
struct iva_FilterDriverStreamBundle {
	ivaWriteStream *_write_stream;
	ivaFrame *_domain;
	ivaFrame *_range;
	ivaStreamFilterBase *_filter;
	iva_FilterDriverStreamBundle()
		:_write_stream(0), _domain(0), _range(0), _filter(0) {}
};

class iva_FilterDriver : public ivaFilterDriver, private iva_Handler {
public:
	// factory...
	iva_FilterDriver(ivaFileFilter &,ivaIO &inf,ivaIO &outf,
		ivaReaderFactory &rf,ivaWriterFactory &wf);
	~iva_FilterDriver();
public:
	// control flow...
	int open(const char *in_fname, const char *out_fname);
	int apply();
	int close();
public:
	// access methods from ivaFilter...
	ivaWriter *writer() {return _writer;}
public:
	// registration methods from ivaFilter...
	int register_filter(ivaVideoFilter &);
	//int register_filter(ivaAudioFilter &);
	//int register_filter(ivaOtherFilter &);
private:
	// handler methods...
	// remember, handler must accept memory in the header callbacks...
	int accept_header(ivaHeader *);
	int accept_audio_header(ivaStreamHeader *, ivaAudioHeader*, ivaDriverData*);
	int accept_video_header(ivaStreamHeader *, ivaVideoHeader*, ivaDriverData*);
	int accept_other_header(ivaStreamHeader *, ivaOtherHeader*, ivaDriverData*);
	int accept_movi_offset(long) {return 1;}
	int accept_index_entry(const ivaIndexEntry&);
private:
	// members...
	ivaIO	&_in_file;
	ivaIO	&_out_file;
	ivaWriterFactory	&_writer_factory;
	ivaReaderFactory	&_reader_factory;
	const char *_in_fname;
	const char *_out_fname;
	ivaFileFilter *_filter; // ...referenced
	ivaWriter *_writer; // ...owned
	vector<iva_FilterDriverStreamBundle> _bundles;
	// filters are all referenced...
	vector<ivaVideoFilter*> _video_filters;
	vector<ivaAudioFilter*> _audio_filters;
	vector<ivaOtherFilter*> _other_filters;
	// ...except for these, which were internally allocated...
	vector<ivaStreamFilterBase*> _allocated_filters;
	uint _streams_accepted;
	uint _video_streams_accepted, _audio_streams_accepted;
	uint _other_streams_accepted;
	// file index...
	vector<ivaIndexEntry> _index;
private:
	// error handler...
	int fail(const char *method, const char *message);
		// ...always returns 0
};

#endif // iva_Filter_included
