/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef iva_Parser_included
#define iva_Parser_included

// $Id: iva_Parser.h,v 1.1 2001/12/12 18:47:51 nradisch Exp $
//
// Alan S. Rojer
// The Duck Corp.
// Mon Sep  8 08:26:34 EDT 1997

#include "ffirChunk.h"
#include "iva.h"
#include "ivaIO.h"
#include "iva_defines.h" // ...index flags

typedef pmsio_AVIINDEXENTRY ivaIndexEntry;

class iva_Handler {
public:
	virtual ~iva_Handler() {}
	// handler must return nonzero on success...
	// handler must accept memory in the header callbacks...
	virtual int accept_header(ivaHeader *) = 0; 
	virtual int accept_audio_header(ivaStreamHeader *, 
		ivaAudioHeader*, ivaDriverData*) = 0;
	virtual int accept_video_header(ivaStreamHeader *, 
		ivaVideoHeader*, ivaDriverData*) = 0;
	virtual int accept_other_header(ivaStreamHeader *, 
		ivaFrame*, ivaDriverData*) = 0;
	virtual int accept_movi_offset(long) = 0;
		// ...absolute file offset to start of movi list data in the file;
		// ie, 12 bytes after start of movi chunk
		// ie, LIST length 'movi' *HERE*
		// ie, absolute offset of first movi chunk.
	virtual int accept_index_entry(const ivaIndexEntry&) = 0;
		// ...dwChunkOffset is standardized to be always absolute in the file
};

class iva_Parser : private ffirHandler {
public:
	iva_Parser(ivaIO&, iva_Handler*);
	virtual ~iva_Parser();
	int parse() const;
	ivaIO::FilePosition_t	last_header_mark(void) const { return _last_header_mark; }
	ivaIO::FilePosition_t	last_format_mark(void) const { return _last_format_mark; }
protected:
	// ffir handler methods...
	virtual int handle_chunk(ivaIO&, const ffirChunkHeader&, uint level);
		// ...file position is immediately after chunk header (at start of data)
	virtual int handle_list_chunk(ivaIO&, const ffirChunkHeader&, ffirFCC, 
		uint level, int &skip);
	// used by the handlers...
	virtual int handle_avih(ivaIO&, const ffirChunkHeader&);
	virtual int handle_idx1(ivaIO&, const ffirChunkHeader&);
	virtual int handle_strl(ivaIO&, const ffirChunkHeader&);
	virtual int handle_movi(ivaIO&);
private:
	// error handler...
	int fail(const char *method, const char *message) const;
	// members...
	// Order of declaration is important; handlers must precede parser
	// since the parser is opend with the handlers.
	iva_Handler *_avi_handler;
	ffirParser _chunk_parser;
	long _movi_offset; // ...for index base computation
	ivaIO::FilePosition_t	_last_header_mark; // location of last header read
	ivaIO::FilePosition_t	_last_format_mark; // location of last format read
};

#endif // iva_Parser_included
