/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef iva_timstyle_h
#define iva_timstyle_h 1

// Some convenient abbbreviations.  Timothy S. Murphy 2/8/94.

#include <stddef.h>


#define rg		register
#define	streq	!strcmp

// Commas in macro arguments are a problem.

#define COMMA		,
#define Q(A,B)		A,B
#define Q2(A,B,C)	A,B,C

typedef	signed	 char	schar;
typedef	unsigned char	uchar;
typedef unsigned int	uint;
typedef	unsigned short	ushort;
typedef unsigned long	ulong;


#if __cplusplus

// General compiler nuisance handling.

#if _MSC_VER

/* Note that msvc by default does not inline _anything_ (regardless of the
   setting of inline_depth) and that a command-line option (-Ob1 or -Ob2)
   is required to inline even the smallest functions. */

#pragma warning( disable : 4250) 	// dominance complaints
#pragma inline_depth( 255)			// I mean it when I inline something
#pragma warning( disable : 4284) 	// operator-> in templates

#endif	// _MSC_VER


// Heap allocation stuff.

//#include <new.h>	// Does new.h always supply placement new?

// inline void *operator new( size_t, void *p) { return p;}

// Sometimes char *s are more natural than void *s.

//inline void *operator new( size_t, char *p) { return (void *)p;}

/* Do not change the values in objectDisposition; code sequences such as
   the following (which depend on these values) should be supported:

	class foo {
		objectDisposition od;
		thing *p;
		~foo() { if( od) delete p;}
	};
*/

	enum objectDisposition { leaveAlone =0, deleteWhenDone =1};

// Class definition stuff.

// Prohibit copying/assignment.

#define noCopying( TYPE) \
 private: \
	TYPE( const TYPE&); \
	TYPE& operator=( const TYPE&);


#define noAssign( TYPE) \
 private: \
	TYPE& operator=( const TYPE&);


// Version for abstract base class provides default constructor.

#define noCopyingBC( TYPE) \
	noCopying( TYPE) \
 protected: \
	TYPE() {}


template<class T> class heapTemp {
	T *p;
	noCopying( heapTemp)
 public:
	heapTemp( T *tp): p( tp) {}
	~heapTemp() { delete p;}
	operator T*() const { return p;}
	T *operator->() const { return p;}
	T& operator*() const { return *p;}
	T *operator()() const { return p;}	// for upcasting may need to force
};

/* Prohibit class derivation; to prevent derivation from a class foo,
   simply inherit privately and virtually from seal<foo>:

	class foo :  ... private virtual seal<foo> ... { ... };

   Relies on fact that virtual bases must be initialized by leaves.
   Costs a word of storage in the sealed class. */

//template<class T>
//class seal {
//	typedef T thing;	// friend T; and friend class T; don't work (why?)
//	friend thing;
//	seal() {};			// only T can access private ctor.
//};


/* Static initializors use common usage-counting idiom (see e.g. ARM 3.4).
   Order of include files gives corresponding order of initializors.
   Note that the macro generates data; hence all include files
   using this idiom must prevent multiple inclusion. */

#define InitializorExtra( TYPE, BODY) \
	class TYPE { \
		static uint usecnt; \
		noCopying( TYPE) \
	BODY \
	public: \
		TYPE(); \
		~TYPE(); \
	}; \
	static TYPE TYPE ## StaticObj;

// Put this in a header file that needs initialization.

#define Initializor( TYPE)	InitializorExtra( TYPE, public:)

// Put these in the corresponding C file.

#define InitializorC( TYPE, BODY) \
	uint TYPE::usecnt = 0; \
	TYPE::TYPE() { if( ++usecnt == 1) { BODY}}

#define InitializorD( TYPE, BODY) \
	TYPE::~TYPE() { if( !--usecnt) { BODY}}


/* Optionally drag in tcl commands associated to a header file.

   The header file then says maybeTcl( SYM) for each group of commands
   (associated to SYM) it wishes to drag in (usually SYM = header file name).
   As above, these header files must have include guards.
   Such header files must include this file but typically do _not_ include
   TclWrap.h.  Rather, TclWrap.h is included by someone else before
   including a header with an optional tcl interface.
   
   The C++ file providing the commands requested by SYM includes
   TclWrap.h and this file, then says defineTcl( SYM).

   If building for tcl, the object file implementing the tcl commands must
   be in the same library file as the C++ implementation of the header.
   Also, TclWrap.h must be included before this file and before any of the
   headers whose corresponding commands you wish to drag in.
   If you are not building for tcl, TclWrap.h must _not_ be included. */

#if TclWrap_h
#	define maybeTcl( SYM) \
		Initializor( TclInclude ## SYM)
#	define defineTclWithBodies( SYM, CTOR, DTOR) \
		InitializorC( TclInclude ## SYM, CTOR ;) \
		InitializorD( TclInclude ## SYM, DTOR ;)
#	define defineTcl( SYM)  defineTclWithBodies( SYM, {}, {})
#else
#	define maybeTcl( SYM)
#endif




// For some reason, GNU does not like this externC macro ...

#define externC( A) extern "C" { A}

// ... an alternative

#define Clinkage extern "C" {
#define endClinkage }

inline uint umod( int v, uint m) {
	if( v >= 0)
		return v % m;
	const uint j( -v % m);
	return j? m-j : 0;
}

#else	// not __cplusplus

#define externC( A) A
#define Clinkage
#define endClinkage

#endif	// __cplusplus

#endif	// iva_timstyle_h
