/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "intelFil.h"
#include "sclrFormat.h"
#include "pmsio.h"
#include "ivaIO.h"
#include <string.h>
#include <assert.h>

using namespace std;

pmsio_DKWAVEFORM::pmsio_DKWAVEFORM(void)
{
	mExtraData = 0;
	cbSize = 0;
}

void
pmsio_DKWAVEFORM::copy_to_me(const pmsio_DKWAVEFORM &d)
{
	wFormatTag 		= d.wFormatTag;
	nChannels		= d.nChannels;
	nSamplesPerSec 	= d.nSamplesPerSec;
	nAvgBytesPerSec = d.nAvgBytesPerSec;
	nBlockAlign 	= d.nBlockAlign;
	wBitsPerSample 	= d.wBitsPerSample;
	cbSize 			= d.cbSize;
	delete [] mExtraData;
	mExtraData = 0;
	if (cbSize && wFormatTag != 1)
		{
		mExtraData = new char[cbSize];
		memcpy(mExtraData,d.mExtraData,cbSize);
		}
	else
		{
		cbSize = 0;
		}
}

pmsio_DKWAVEFORM::pmsio_DKWAVEFORM(const pmsio_DKWAVEFORM &d)
{
	mExtraData = 0;
	copy_to_me(d);
}

pmsio_DKWAVEFORM& 
pmsio_DKWAVEFORM::operator=(const pmsio_DKWAVEFORM &p)
{
	copy_to_me(p);
	return *this;
}

pmsio_DKWAVEFORM::~pmsio_DKWAVEFORM()
{
	delete [] mExtraData;
}

pmsio_DKWAVEFORM *
pmsio_DKWAVEFORM::clone() const
{
	return new pmsio_DKWAVEFORM(*this);
}


int
pmsio_DKWAVEFORM::read(ivaIO &f, pmsio_DKWAVEFORM *p)
{ 
	uchar buffer[_base_size];
	if (f.Read(buffer, _base_size) != _base_size) return 0;
	if (!read_from_sub_buffer(buffer, p)) return 0;
	delete [] p->mExtraData;
	p->mExtraData = 0;
	if (p->cbSize)
		{
		p->mExtraData = new char[p->cbSize];
		if (f.Read(p->mExtraData,p->cbSize) != p->cbSize) return 0;
		}
	return 1;
}

int
pmsio_DKWAVEFORM::read_from_sub_buffer(const uchar *buffer, pmsio_DKWAVEFORM *p)
{
	intelReader reader(buffer);
	
	reader >> p->wFormatTag;
	reader >> p->nChannels;
	reader >> p->nSamplesPerSec;
	reader >> p->nAvgBytesPerSec;
	reader >> p->nBlockAlign;
	reader >> p->wBitsPerSample;
	reader >> p->cbSize;
	if (p->wFormatTag == 1)
		p->cbSize = 0;
	return 1;
}

int
pmsio_DKWAVEFORM::read_from_buffer(const uchar *buffer, pmsio_DKWAVEFORM *p)
{
	read_from_sub_buffer(buffer,p);
	delete [] p->mExtraData;
	p->mExtraData = 0;
	if (p->cbSize)
		{
		p->mExtraData = new char[p->cbSize];
		memcpy(p->mExtraData,buffer + _base_size,p->cbSize);
		}
	return 1;
}


int
pmsio_DKWAVEFORM::write(ivaIO &f) const
{ 
	uchar buffer[_base_size];
	if (!write_to_sub_buffer(buffer)) return 0;
	if (f.Write(buffer, _base_size) != _base_size) return 0;
	if (cbSize)
		{
		if (f.Write(mExtraData,cbSize) != cbSize) return 0;
		}
	return 1;
}

int
pmsio_DKWAVEFORM::write_to_sub_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << wFormatTag;
	writer << nChannels;
	writer << nSamplesPerSec;
	writer << nAvgBytesPerSec;
	writer << nBlockAlign;
	writer << wBitsPerSample;
	writer << cbSize;
	return 1;
}


int
pmsio_DKWAVEFORM::write_to_buffer(uchar *buffer) const
{
	write_to_sub_buffer(buffer);
	if (cbSize)
		{
		assert(mExtraData);
		memcpy(buffer + _base_size,mExtraData,cbSize);
		}
	return 1;
}


void
pmsio_DKWAVEFORM::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "DKWAVEFORM:\n";
	++level;
	riffFormat::prefix(os, level); os << "wFormatTag = ";
	riffFormat::a_ushort(os, wFormatTag);
	os << endl;	riffFormat::prefix(os, level); os << "nChannels = ";
	riffFormat::a_ushort(os, nChannels);
	os << endl;	riffFormat::prefix(os, level); os << "nSamplesPerSec = ";
	riffFormat::a_ulong(os, nSamplesPerSec);
	os << endl;	riffFormat::prefix(os, level); os << "nAvgBytesPerSec = ";
	riffFormat::a_ulong(os, nAvgBytesPerSec);
	os << endl;	riffFormat::prefix(os, level); os << "nBlockAlign = ";
	riffFormat::a_ushort(os, nBlockAlign);
	os << endl;	riffFormat::prefix(os, level); os << "wBitsPerSample = ";
	riffFormat::a_ushort(os, wBitsPerSample);
	os << endl;	riffFormat::prefix(os, level); os << "cbSize = ";
	riffFormat::a_ushort(os, cbSize);
	os << endl;}

int
pmsio_RECT::read(ivaIO &f, pmsio_RECT *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_RECT::read_from_buffer(const uchar *buffer, pmsio_RECT *p)
{
	intelReader reader(buffer);
	reader >> p->left;
	reader >> p->top;
	reader >> p->right;
	reader >> p->bottom;
	return 1;
}


int
pmsio_RECT::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_RECT::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << left;
	writer << top;
	writer << right;
	writer << bottom;
	return 1;
}


void
pmsio_RECT::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "RECT:\n";
	++level;
	riffFormat::prefix(os, level); os << "left = ";
	riffFormat::a_ushort(os, left);
	os << endl;	riffFormat::prefix(os, level); os << "top = ";
	riffFormat::a_ushort(os, top);
	os << endl;	riffFormat::prefix(os, level); os << "right = ";
	riffFormat::a_ushort(os, right);
	os << endl;	riffFormat::prefix(os, level); os << "bottom = ";
	riffFormat::a_ushort(os, bottom);
	os << endl;}

int
pmsio_BITMAP::read(ivaIO &f, pmsio_BITMAP *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_BITMAP::read_from_buffer(const uchar *buffer, pmsio_BITMAP *p)
{
	intelReader reader(buffer);
	reader >> p->bmType;
	reader >> p->bmWidth;
	reader >> p->bmHeight;
	reader >> p->bmWidthBytes;
	reader >> p->bmPlanes;
	reader >> p->bmBitsPixel;
	return 1;
}


int
pmsio_BITMAP::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_BITMAP::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << bmType;
	writer << bmWidth;
	writer << bmHeight;
	writer << bmWidthBytes;
	writer << bmPlanes;
	writer << bmBitsPixel;
	return 1;
}


void
pmsio_BITMAP::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAP:\n";
	++level;
	riffFormat::prefix(os, level); os << "bmType = ";
	riffFormat::a_int(os, bmType);
	os << endl;	riffFormat::prefix(os, level); os << "bmWidth = ";
	riffFormat::a_int(os, bmWidth);
	os << endl;	riffFormat::prefix(os, level); os << "bmHeight = ";
	riffFormat::a_int(os, bmHeight);
	os << endl;	riffFormat::prefix(os, level); os << "bmWidthBytes = ";
	riffFormat::a_int(os, bmWidthBytes);
	os << endl;	riffFormat::prefix(os, level); os << "bmPlanes = ";
	riffFormat::a_byte(os, bmPlanes);
	os << endl;	riffFormat::prefix(os, level); os << "bmBitsPixel = ";
	riffFormat::a_byte(os, bmBitsPixel);
	os << endl;}

int
pmsio_RGBTRIPLE::read(ivaIO &f, pmsio_RGBTRIPLE *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_RGBTRIPLE::read_from_buffer(const uchar *buffer, pmsio_RGBTRIPLE *p)
{
	intelReader reader(buffer);
	reader >> p->rgbtBlue;
	reader >> p->rgbtGreen;
	reader >> p->rgbtRed;
	return 1;
}


int
pmsio_RGBTRIPLE::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_RGBTRIPLE::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << rgbtBlue;
	writer << rgbtGreen;
	writer << rgbtRed;
	return 1;
}


void
pmsio_RGBTRIPLE::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "RGBTRIPLE:\n";
	++level;
	riffFormat::prefix(os, level); os << "rgbtBlue = ";
	riffFormat::a_byte(os, rgbtBlue);
	os << endl;	riffFormat::prefix(os, level); os << "rgbtGreen = ";
	riffFormat::a_byte(os, rgbtGreen);
	os << endl;	riffFormat::prefix(os, level); os << "rgbtRed = ";
	riffFormat::a_byte(os, rgbtRed);
	os << endl;}

int
pmsio_RGBQUAD::read(ivaIO &f, pmsio_RGBQUAD *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_RGBQUAD::read_from_buffer(const uchar *buffer, pmsio_RGBQUAD *p)
{
	intelReader reader(buffer);
	reader >> p->rgbBlue;
	reader >> p->rgbGreen;
	reader >> p->rgbRed;
	reader >> p->rgbReserved;
	return 1;
}


int
pmsio_RGBQUAD::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_RGBQUAD::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << rgbBlue;
	writer << rgbGreen;
	writer << rgbRed;
	writer << rgbReserved;
	return 1;
}


void
pmsio_RGBQUAD::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "RGBQUAD:\n";
	++level;
	riffFormat::prefix(os, level); os << "rgbBlue = ";
	riffFormat::a_byte(os, rgbBlue);
	os << endl;	riffFormat::prefix(os, level); os << "rgbGreen = ";
	riffFormat::a_byte(os, rgbGreen);
	os << endl;	riffFormat::prefix(os, level); os << "rgbRed = ";
	riffFormat::a_byte(os, rgbRed);
	os << endl;	riffFormat::prefix(os, level); os << "rgbReserved = ";
	riffFormat::a_byte(os, rgbReserved);
	os << endl;}

int
pmsio_BITMAPCOREHEADER::read(ivaIO &f, pmsio_BITMAPCOREHEADER *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_BITMAPCOREHEADER::read_from_buffer(const uchar *buffer, pmsio_BITMAPCOREHEADER *p)
{
	intelReader reader(buffer);
	reader >> p->bcSize;
	reader >> p->bcWidth;
	reader >> p->bcHeight;
	reader >> p->bcPlanes;
	reader >> p->bcBitCount;
	return 1;
}


int
pmsio_BITMAPCOREHEADER::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_BITMAPCOREHEADER::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << bcSize;
	writer << bcWidth;
	writer << bcHeight;
	writer << bcPlanes;
	writer << bcBitCount;
	return 1;
}


void
pmsio_BITMAPCOREHEADER::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAPCOREHEADER:\n";
	++level;
	riffFormat::prefix(os, level); os << "bcSize = ";
	riffFormat::a_dword(os, bcSize);
	os << endl;	riffFormat::prefix(os, level); os << "bcWidth = ";
	riffFormat::a_short(os, bcWidth);
	os << endl;	riffFormat::prefix(os, level); os << "bcHeight = ";
	riffFormat::a_short(os, bcHeight);
	os << endl;	riffFormat::prefix(os, level); os << "bcPlanes = ";
	riffFormat::a_word(os, bcPlanes);
	os << endl;	riffFormat::prefix(os, level); os << "bcBitCount = ";
	riffFormat::a_word(os, bcBitCount);
	os << endl;}

int
pmsio_BITMAPINFOHEADER::read(ivaIO &f, pmsio_BITMAPINFOHEADER *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_BITMAPINFOHEADER::read_from_buffer(const uchar *buffer, pmsio_BITMAPINFOHEADER *p)
{
	intelReader reader(buffer);
	reader >> p->biSize;
	reader >> p->biWidth;
	reader >> p->biHeight;
	reader >> p->biPlanes;
	reader >> p->biBitCount;
	reader >> p->biCompression;
	reader >> p->biSizeImage;
	reader >> p->biXPelsPerMeter;
	reader >> p->biYPelsPerMeter;
	reader >> p->biClrUsed;
	reader >> p->biClrImportant;
	return 1;
}


int
pmsio_BITMAPINFOHEADER::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_BITMAPINFOHEADER::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << biSize;
	writer << biWidth;
	writer << biHeight;
	writer << biPlanes;
	writer << biBitCount;
	writer << biCompression;
	writer << biSizeImage;
	writer << biXPelsPerMeter;
	writer << biYPelsPerMeter;
	writer << biClrUsed;
	writer << biClrImportant;
	return 1;
}


void
pmsio_BITMAPINFOHEADER::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAPINFOHEADER:\n";
	++level;
	riffFormat::prefix(os, level); os << "biSize = ";
	riffFormat::a_dword(os, biSize);
	os << endl;	riffFormat::prefix(os, level); os << "biWidth = ";
	riffFormat::a_long(os, biWidth);
	os << endl;	riffFormat::prefix(os, level); os << "biHeight = ";
	riffFormat::a_long(os, biHeight);
	os << endl;	riffFormat::prefix(os, level); os << "biPlanes = ";
	riffFormat::a_word(os, biPlanes);
	os << endl;	riffFormat::prefix(os, level); os << "biBitCount = ";
	riffFormat::a_word(os, biBitCount);
	os << endl;	riffFormat::prefix(os, level); os << "biCompression = ";
	riffFormat::a_dword(os, biCompression);
	os << endl;	riffFormat::prefix(os, level); os << "biSizeImage = ";
	riffFormat::a_dword(os, biSizeImage);
	os << endl;	riffFormat::prefix(os, level); os << "biXPelsPerMeter = ";
	riffFormat::a_long(os, biXPelsPerMeter);
	os << endl;	riffFormat::prefix(os, level); os << "biYPelsPerMeter = ";
	riffFormat::a_long(os, biYPelsPerMeter);
	os << endl;	riffFormat::prefix(os, level); os << "biClrUsed = ";
	riffFormat::a_dword(os, biClrUsed);
	os << endl;	riffFormat::prefix(os, level); os << "biClrImportant = ";
	riffFormat::a_dword(os, biClrImportant);
	os << endl;}

int
pmsio_BITMAPCOREINFO::read(ivaIO &f, pmsio_BITMAPCOREINFO *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_BITMAPCOREINFO::read_from_buffer(const uchar *buffer, pmsio_BITMAPCOREINFO *p)
{
	intelReader reader(buffer);
	pmsio_BITMAPCOREHEADER::read_from_buffer(reader, &p->bmciHeader);
	reader += pmsio_BITMAPCOREHEADER::_size;
	{		int i;
		for (i=0; i<1; i++) {
			pmsio_RGBTRIPLE::read_from_buffer(reader, &p->bmciColors[i]);
			reader += pmsio_RGBTRIPLE::_size;
		}
	}	return 1;
}


int
pmsio_BITMAPCOREINFO::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_BITMAPCOREINFO::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	bmciHeader.write_to_buffer(writer);
	buffer += pmsio_BITMAPCOREHEADER::_size;
	{		int i;
		for (i=0; i<1; i++) {
			bmciColors[i].write_to_buffer(writer);
			buffer += pmsio_RGBTRIPLE::_size;
		}
	}	return 1;
}


void
pmsio_BITMAPCOREINFO::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAPCOREINFO:\n";
	++level;
	riffFormat::prefix(os, level); os << "bmciHeader = ";
	bmciHeader.show(os, level);
	os << endl;	riffFormat::prefix(os, level); os << "bmciColors = ";
	{
		int i;
		for (i=0; i<1; i++) {
			bmciColors[i].show(os, level + 1);
		}
	}	os << endl;}

int
pmsio_BITMAPFILEHEADER::read(ivaIO &f, pmsio_BITMAPFILEHEADER *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_BITMAPFILEHEADER::read_from_buffer(const uchar *buffer, pmsio_BITMAPFILEHEADER *p)
{
	intelReader reader(buffer);
	reader >> p->bfType;
	reader >> p->bfSize;
	reader >> p->bfReserved1;
	reader >> p->bfReserved2;
	reader >> p->bfOffBits;
	return 1;
}


int
pmsio_BITMAPFILEHEADER::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_BITMAPFILEHEADER::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << bfType;
	writer << bfSize;
	writer << bfReserved1;
	writer << bfReserved2;
	writer << bfOffBits;
	return 1;
}


void
pmsio_BITMAPFILEHEADER::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAPFILEHEADER:\n";
	++level;
	riffFormat::prefix(os, level); os << "bfType = ";
	riffFormat::a_uint(os, bfType);
	os << endl;	riffFormat::prefix(os, level); os << "bfSize = ";
	riffFormat::a_dword(os, bfSize);
	os << endl;	riffFormat::prefix(os, level); os << "bfReserved1 = ";
	riffFormat::a_uint(os, bfReserved1);
	os << endl;	riffFormat::prefix(os, level); os << "bfReserved2 = ";
	riffFormat::a_uint(os, bfReserved2);
	os << endl;	riffFormat::prefix(os, level); os << "bfOffBits = ";
	riffFormat::a_dword(os, bfOffBits);
	os << endl;}

int
pmsio_PALETTEENTRY::read(ivaIO &f, pmsio_PALETTEENTRY *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_PALETTEENTRY::read_from_buffer(const uchar *buffer, pmsio_PALETTEENTRY *p)
{
	intelReader reader(buffer);
	reader >> p->peRed;
	reader >> p->peGreen;
	reader >> p->peBlue;
	reader >> p->peFlags;
	return 1;
}


int
pmsio_PALETTEENTRY::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_PALETTEENTRY::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << peRed;
	writer << peGreen;
	writer << peBlue;
	writer << peFlags;
	return 1;
}


void
pmsio_PALETTEENTRY::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "PALETTEENTRY:\n";
	++level;
	riffFormat::prefix(os, level); os << "peRed = ";
	riffFormat::a_byte(os, peRed);
	os << endl;	riffFormat::prefix(os, level); os << "peGreen = ";
	riffFormat::a_byte(os, peGreen);
	os << endl;	riffFormat::prefix(os, level); os << "peBlue = ";
	riffFormat::a_byte(os, peBlue);
	os << endl;	riffFormat::prefix(os, level); os << "peFlags = ";
	riffFormat::a_byte(os, peFlags);
	os << endl;}

int
pmsio_MainAVIHeader::read(ivaIO &f, pmsio_MainAVIHeader *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_MainAVIHeader::read_from_buffer(const uchar *buffer, pmsio_MainAVIHeader *p)
{
	intelReader reader(buffer);
	reader >> p->dwMicroSecPerFrame;
	reader >> p->dwMaxBytesPerSec;
	reader >> p->dwPaddingGranularity;
	reader >> p->dwFlags;
	reader >> p->dwTotalFrames;
	reader >> p->dwInitialFrames;
	reader >> p->dwStreams;
	reader >> p->dwSuggestedBufferSize;
	reader >> p->dwWidth;
	reader >> p->dwHeight;
	{int i; for (i=0; i<4; i++) {reader >> p->dwReserved[i];}}
	return 1;
}


int
pmsio_MainAVIHeader::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_MainAVIHeader::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << dwMicroSecPerFrame;
	writer << dwMaxBytesPerSec;
	writer << dwPaddingGranularity;
	writer << dwFlags;
	writer << dwTotalFrames;
	writer << dwInitialFrames;
	writer << dwStreams;
	writer << dwSuggestedBufferSize;
	writer << dwWidth;
	writer << dwHeight;
	{int i; for (i=0; i<4; i++) {writer << dwReserved[i];}}
	return 1;
}


void
pmsio_MainAVIHeader::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "MainAVIHeader:\n";
	++level;
	riffFormat::prefix(os, level); os << "dwMicroSecPerFrame = ";
	riffFormat::a_dword(os, dwMicroSecPerFrame);
	os << endl;	riffFormat::prefix(os, level); os << "dwMaxBytesPerSec = ";
	riffFormat::a_dword(os, dwMaxBytesPerSec);
	os << endl;	riffFormat::prefix(os, level); os << "dwPaddingGranularity = ";
	riffFormat::a_dword(os, dwPaddingGranularity);
	os << endl;	riffFormat::prefix(os, level); os << "dwFlags = ";
	riffFormat::a_dword(os, dwFlags);
	os << endl;	riffFormat::prefix(os, level); os << "dwTotalFrames = ";
	riffFormat::a_dword(os, dwTotalFrames);
	os << endl;	riffFormat::prefix(os, level); os << "dwInitialFrames = ";
	riffFormat::a_dword(os, dwInitialFrames);
	os << endl;	riffFormat::prefix(os, level); os << "dwStreams = ";
	riffFormat::a_dword(os, dwStreams);
	os << endl;	riffFormat::prefix(os, level); os << "dwSuggestedBufferSize = ";
	riffFormat::a_dword(os, dwSuggestedBufferSize);
	os << endl;	riffFormat::prefix(os, level); os << "dwWidth = ";
	riffFormat::a_dword(os, dwWidth);
	os << endl;	riffFormat::prefix(os, level); os << "dwHeight = ";
	riffFormat::a_dword(os, dwHeight);
	os << endl;	riffFormat::prefix(os, level); os << "dwReserved = ";
	{
		int i;
		for (i=0; i<4; i++) {
			if (i>0) os << ", ";
			riffFormat::a_dword(os, dwReserved[i]);
		}
	}
	os << endl;}

int
pmsio_AVIStreamHeader::read(ivaIO &f, pmsio_AVIStreamHeader *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_AVIStreamHeader::read_from_buffer(const uchar *buffer, pmsio_AVIStreamHeader *p)
{
	intelReader reader(buffer);
	reader >> p->fccType;
	reader >> p->fccHandler;
	reader >> p->dwFlags;
	reader >> p->wPriority;
	reader >> p->wLanguage;
	reader >> p->dwInitialFrames;
	reader >> p->dwScale;
	reader >> p->dwRate;
	reader >> p->dwStart;
	reader >> p->dwLength;
	reader >> p->dwSuggestedBufferSize;
	reader >> p->dwQuality;
	reader >> p->dwSampleSize;
	pmsio_RECT::read_from_buffer(reader, &p->rcFrame);
	reader += pmsio_RECT::_size;
	return 1;
}


int
pmsio_AVIStreamHeader::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_AVIStreamHeader::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << fccType;
	writer << fccHandler;
	writer << dwFlags;
	writer << wPriority;
	writer << wLanguage;
	writer << dwInitialFrames;
	writer << dwScale;
	writer << dwRate;
	writer << dwStart;
	writer << dwLength;
	writer << dwSuggestedBufferSize;
	writer << dwQuality;
	writer << dwSampleSize;
	rcFrame.write_to_buffer(writer);
	buffer += pmsio_RECT::_size;
	return 1;
}


void
pmsio_AVIStreamHeader::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "AVIStreamHeader:\n";
	++level;
	riffFormat::prefix(os, level); os << "fccType = ";
	riffFormat::a_FCC(os, fccType);
	os << endl;	riffFormat::prefix(os, level); os << "fccHandler = ";
	riffFormat::a_FCC(os, fccHandler);
	os << endl;	riffFormat::prefix(os, level); os << "dwFlags = ";
	riffFormat::a_dword(os, dwFlags);
	os << endl;	riffFormat::prefix(os, level); os << "wPriority = ";
	riffFormat::a_word(os, wPriority);
	os << endl;	riffFormat::prefix(os, level); os << "wLanguage = ";
	riffFormat::a_word(os, wLanguage);
	os << endl;	riffFormat::prefix(os, level); os << "dwInitialFrames = ";
	riffFormat::a_dword(os, dwInitialFrames);
	os << endl;	riffFormat::prefix(os, level); os << "dwScale = ";
	riffFormat::a_dword(os, dwScale);
	os << endl;	riffFormat::prefix(os, level); os << "dwRate = ";
	riffFormat::a_dword(os, dwRate);
	os << endl;	riffFormat::prefix(os, level); os << "dwStart = ";
	riffFormat::a_dword(os, dwStart);
	os << endl;	riffFormat::prefix(os, level); os << "dwLength = ";
	riffFormat::a_dword(os, dwLength);
	os << endl;	riffFormat::prefix(os, level); os << "dwSuggestedBufferSize = ";
	riffFormat::a_dword(os, dwSuggestedBufferSize);
	os << endl;	riffFormat::prefix(os, level); os << "dwQuality = ";
	riffFormat::a_dword(os, dwQuality);
	os << endl;	riffFormat::prefix(os, level); os << "dwSampleSize = ";
	riffFormat::a_dword(os, dwSampleSize);
	os << endl;	riffFormat::prefix(os, level); os << "rcFrame = ";
	rcFrame.show(os, level);
	os << endl;}

int
pmsio_AVIINDEXENTRY::read(ivaIO &f, pmsio_AVIINDEXENTRY *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_AVIINDEXENTRY::read_from_buffer(const uchar *buffer, pmsio_AVIINDEXENTRY *p)
{
	intelReader reader(buffer);
	reader >> p->ckid;
	reader >> p->dwFlags;
	reader >> p->dwChunkOffset;
	reader >> p->dwChunkLength;
	return 1;
}


int
pmsio_AVIINDEXENTRY::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_AVIINDEXENTRY::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << ckid;
	writer << dwFlags;
	writer << dwChunkOffset;
	writer << dwChunkLength;
	return 1;
}


void
pmsio_AVIINDEXENTRY::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "AVIINDEXENTRY:\n";
	++level;
	riffFormat::prefix(os, level); os << "ckid = ";
	riffFormat::a_dword(os, ckid);
	os << endl;	riffFormat::prefix(os, level); os << "dwFlags = ";
	riffFormat::a_dword(os, dwFlags);
	os << endl;	riffFormat::prefix(os, level); os << "dwChunkOffset = ";
	riffFormat::a_dword(os, dwChunkOffset);
	os << endl;	riffFormat::prefix(os, level); os << "dwChunkLength = ";
	riffFormat::a_dword(os, dwChunkLength);
	os << endl;}

int
pmsio_AVIPALCHANGE::read(ivaIO &f, pmsio_AVIPALCHANGE *p)
{ 
	uchar buffer[_size];
	if (f.Read(buffer, _size) != _size) return 0;
	return read_from_buffer(buffer, p);
}

int
pmsio_AVIPALCHANGE::read_from_buffer(const uchar *buffer, pmsio_AVIPALCHANGE *p)
{
	intelReader reader(buffer);
	reader >> p->bFirstEntry;
	reader >> p->bNumEntries;
	reader >> p->wFlags;
	{		int i;
		for (i=0; i<1; i++) {
			pmsio_PALETTEENTRY::read_from_buffer(reader, &p->peNew[i]);
			reader += pmsio_PALETTEENTRY::_size;
		}
	}	return 1;
}


int
pmsio_AVIPALCHANGE::write(ivaIO &f) const
{ 
	uchar buffer[_size];
	if (!write_to_buffer(buffer)) return 0;
	if (f.Write(buffer, _size) != _size) return 0;
	return 1;
}

int
pmsio_AVIPALCHANGE::write_to_buffer(uchar *buffer) const
{
	intelWriter writer(buffer);
	writer << bFirstEntry;
	writer << bNumEntries;
	writer << wFlags;
	{		int i;
		for (i=0; i<1; i++) {
			peNew[i].write_to_buffer(writer);
			buffer += pmsio_PALETTEENTRY::_size;
		}
	}	return 1;
}


void
pmsio_AVIPALCHANGE::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "AVIPALCHANGE:\n";
	++level;
	riffFormat::prefix(os, level); os << "bFirstEntry = ";
	riffFormat::a_byte(os, bFirstEntry);
	os << endl;	riffFormat::prefix(os, level); os << "bNumEntries = ";
	riffFormat::a_byte(os, bNumEntries);
	os << endl;	riffFormat::prefix(os, level); os << "wFlags = ";
	riffFormat::a_word(os, wFlags);
	os << endl;	riffFormat::prefix(os, level); os << "peNew = ";
	{
		int i;
		for (i=0; i<1; i++) {
			peNew[i].show(os, level + 1);
		}
	}	os << endl;}

