/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// $Id: pmsio_BITMAPINFO.cxx,v 1.1 2001/12/12 18:47:56 nradisch Exp $
#include "intelFil.h"
#include "sclrFormat.h"
#include "pmsio_BITMAPINFO.h"
#include "ivaIO.h"
#include <iostream>

using namespace std;

int
pmsio_BITMAPINFO::read(ivaIO &f, pmsio_BITMAPINFO *p)
{ 
	if (!pmsio_BITMAPINFOHEADER::read(f, &p->bmiHeader)) return 0;
	if (p->bmiHeader.biClrUsed > 0) {
		p->bmiColors = new pmsio_RGBQUAD[p->bmiHeader.biClrUsed];
		if (!p->bmiColors) return 0;
		for (size_t i=0; i < p->bmiHeader.biClrUsed; i++) {
			if (!pmsio_RGBQUAD::read(f, &p->bmiColors[i])) return 0;;
		}
	}
	return 1;
}

int
pmsio_BITMAPINFO::read_from_buffer(const uchar *buffer, pmsio_BITMAPINFO *p)
{
	intelReader reader(buffer);
	pmsio_BITMAPINFOHEADER::read_from_buffer(reader, &p->bmiHeader);
	reader += pmsio_BITMAPINFOHEADER::size();
	if (p->bmiHeader.biClrUsed > 0) {
		p->bmiColors = new pmsio_RGBQUAD[p->bmiHeader.biClrUsed];
		if (!p->bmiColors) return 0;
		for (size_t i=0; i < p->bmiHeader.biClrUsed; i++) {
			pmsio_RGBQUAD::read_from_buffer(reader, &p->bmiColors[i]);
			reader += pmsio_RGBQUAD::size();
		}
	}
	return 1;
}

int
pmsio_BITMAPINFO::write(ivaIO &f) const
{ 
	if (!bmiHeader.write(f)) return 0;
	if (bmiHeader.biClrUsed > 0) {
		for (size_t i=0; i<bmiHeader.biClrUsed; i++) {
			if (!bmiColors[i].write(f)) return 0;
		}
	}
	return 1;
}

void
pmsio_BITMAPINFO::show(std::ostream &os, unsigned int level) const
{ 
	riffFormat::prefix(os, level); os << "BITMAPINFO:\n";
	++level;
	riffFormat::prefix(os, level); os << "bmiHeader = ";
	bmiHeader.show(os, level);
	os << endl;	riffFormat::prefix(os, level); os << "bmiColors = ";
	for (size_t i=0; i<bmiHeader.biClrUsed; i++) {
		bmiColors[i].show(os, level + 1);
	}
	os << endl;}


pmsio_BITMAPINFO::~pmsio_BITMAPINFO()
{
	delete bmiColors;
}

pmsio_BITMAPINFO::pmsio_BITMAPINFO() 
: bmiColors(0)
{
}

pmsio_BITMAPINFO::pmsio_BITMAPINFO(const pmsio_BITMAPINFO &p)
: bmiColors(0)
{
	bmiHeader = p.bmiHeader;
	clone_palette_from(p);
}


pmsio_BITMAPINFO&
pmsio_BITMAPINFO::operator=(const pmsio_BITMAPINFO &p)
{
	bmiHeader = p.bmiHeader;
	clone_palette_from(p);
	return *this;
}

pmsio_BITMAPINFO *pmsio_BITMAPINFO::clone() const 
{
	return new pmsio_BITMAPINFO(*this);
}


void 
pmsio_BITMAPINFO::clone_palette_from(const pmsio_BITMAPINFO &p) 
{
	// ...assumes headers are consistent
	delete bmiColors; bmiColors = 0;
	if (bmiHeader.biClrUsed > 0) {
		bmiColors = new pmsio_RGBQUAD[bmiHeader.biClrUsed]; // ...could fail!
		for (size_t i=0; i<bmiHeader.biClrUsed; i++) 
			bmiColors[i] = p.bmiColors[i];
	}
}
