/*  Copyright (c) 1994-2002 by On2 Technologies, The Duck Corporation Inc.

    This file is part of VpVision.

    VpVision is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License ver. 2,
    as published by the Free Software Foundation (www.fsf.org).

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with VpVision; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "MegaHeaders.h"
#include "ivaWriterFast.h"
#include "intelFil.h"
#include "stack_aids.h"
#include <assert.h>
#if 0
#include <fstream>
ofstream ffff("c:\\snarf.txt");
#endif

using namespace std;

#define	BUFFER_SIZE		(1024 * 1000)
#define	MINIMUM_WRITE	(1024 * 100)
#define	WRITE_BLOCK		2048l
#define WRITE_BLOCK_MASK (~(WRITE_BLOCK - 1))

ivaWriterFast::ivaWriterFast(const ivaHeader &header)
: iva_Writer(mIO,header,2048,0)
{
	mFileHandle = 0;
	mFileName = 0;
	mMasterBuffer = (char *)::VirtualAlloc(NULL,BUFFER_SIZE + (3 * WRITE_BLOCK),MEM_COMMIT,PAGE_NOCACHE | PAGE_READWRITE);
	::VirtualLock(mMasterBuffer,BUFFER_SIZE + (3 * WRITE_BLOCK));

	mBuffer = (char *)(((unsigned long)mMasterBuffer) & WRITE_BLOCK_MASK) + WRITE_BLOCK;

	mEndOfBuffer = mBuffer + BUFFER_SIZE;
	mThread = new ivaWriterThread(*this);
}

ivaWriterFast::~ivaWriterFast()
{
	delete [] mFileName;
	if (mFileHandle)
		::CloseHandle(mFileHandle);
	mFileHandle = 0;
	::VirtualUnlock(mMasterBuffer,BUFFER_SIZE + (3 * WRITE_BLOCK));
	::VirtualFree(mMasterBuffer,0,MEM_RELEASE);
	delete mThread;
}

bool
ivaWriterFast::open(const char *fname)
{
	mAmtWrittenToDisk = 0;
	_index.reserve(10000);
	assert(!mFileHandle);
	delete [] mFileName;
	mFileName = new char[strlen(fname)+ 1];
	strcpy(mFileName,fname);

	mDynamicEndOfBuffer = mBuffer;
	mWritePtr = mReadPtr = mBuffer;

	mFileHandle = ::CreateFile(fname,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL  | FILE_FLAG_NO_BUFFERING,NULL);
	if (!mFileHandle) return 0;
	_movi_bytes = 4; // ...the 'movi' list header
	_idx1_bytes = 0;

	unsigned long ul = ::SetFilePointer(mFileHandle,2048,NULL,FILE_BEGIN);
	mThread->Start();
	return (ul == 0xffffffff ? false : true);
}

int 
ivaWriterFast::is_open() const
{
	return (mFileHandle ? 1 : 0);
}
 
int 
ivaWriterFast::close(int recalc)
{
	mBufferMutex.Lock();
	mThread->mStop = 1;
	mWaitingForData.Signal();
	mBufferMutex.Unlock();
	mThread->Join();
	
	// flush remaining data
	while (1)
		{
		size_t	amt;
		DWORD s;
		char *new_read_ptr;

		if (mWritePtr == mReadPtr) break;
		if (mWritePtr > mReadPtr)
			{
			AddJunk(mWritePtr - mReadPtr,mWritePtr);
			amt = mWritePtr - mReadPtr;
			new_read_ptr = mWritePtr;
			}
		else
			{
			AddJunk(mDynamicEndOfBuffer - mReadPtr,mDynamicEndOfBuffer);
			amt = mDynamicEndOfBuffer - mReadPtr;
			new_read_ptr = mBuffer;
			}
		int err = ::WriteFile(mFileHandle,mReadPtr,amt,&s,NULL);
		assert(err);
		assert(amt == s);
		mAmtWrittenToDisk += amt;
		mReadPtr = new_read_ptr;
		}

	if (mFileHandle)
		::CloseHandle(mFileHandle);
	mFileHandle = 0;

	assert(mAmtWrittenToDisk == _movi_bytes - 4);

	mIO.Open(mFileName,ivaIO::WRITE);
	mIO.Seek(0,ivaIO::END);
	return iva_Writer::close(recalc);
}

int 
ivaWriterFast::put_frame(ffirFCC fcc, const ivaFrame &f, int is_key)
{
	ivaFrame	*dest = get_frame(f.length());
	memcpy(dest->buffer(),f.buffer(),f.length());
	return return_frame(fcc,dest,is_key);
}

void
ivaWriterFast::AddJunk(size_t basis,char* &where)
{
	// pad with junk and write it
	size_t	padding = WRITE_BLOCK - (basis % WRITE_BLOCK);
	if (padding < 8) padding += WRITE_BLOCK;
	*(where++) = 'J';
	*(where++) = 'U';
	*(where++) = 'N';
	*(where++) = 'K';
	intelWriter writer((unsigned char *)where);
	writer << ((long)(padding - 8));
	where += (padding - 4);
	_movi_bytes += padding;
}

void
ivaWriterFast::abort_outstanding_frame()
{
	mOutstandingFrameLock.Unlock();
}

ivaTrivialFrame	sFrame;

ivaFrame *
ivaWriterFast::get_frame(size_t siz)
{
	mOutstandingFrameLock.Lock();

	size_t room = 0;
	size_t	realsize = siz + 8;	// 8 is size of chunk header
	if (realsize & 0x01) realsize += 1; // round to word boundary

	// see if we have room
	stack_MutexLock sml(&mBufferMutex);

	do
		{
		room = 0;
		if (mWritePtr >= mReadPtr)
			{
			room = mEndOfBuffer - mWritePtr;
			if (room - (3 * WRITE_BLOCK) < realsize)
				{
				if (mReadPtr > mBuffer)
					{
					mDynamicEndOfBuffer = mWritePtr;
					mWritePtr = mBuffer;
					room = 0;

					AddJunk(mDynamicEndOfBuffer - mBuffer,mDynamicEndOfBuffer);
					}
				else
					{
					room = 0;
					}
				}
			}
		if (mWritePtr < mReadPtr)
			room = mReadPtr - mWritePtr - 1;

		if (room < realsize)
			mWaitingForSpace.Wait(mBufferMutex);
		}
	while (room < realsize);
	sFrame.setbuffer((uchar *)mWritePtr + 8,realsize - 8);
	sFrame.length() = siz;

	return &sFrame;
}

int		
ivaWriterFast::return_frame(ffirFCC fcc, ivaFrame *f,int is_key)
{
	stack_MutexLock sml(&mBufferMutex);
	// force the use of fast copy
	long	siz = f->length();
	if (siz & 1) siz++;

	fcc.write((unsigned long *)mWritePtr);
	intelWriter iw((unsigned char *)(mWritePtr+4));
	iw << (long)(f->length());
	mWritePtr += siz + 8;
	mWaitingForData.Signal();
	mOutstandingFrameLock.Unlock();

	iva_IndexEntry ie;
	ie.ckid = fcc;
	ie.dwFlags = (is_key ? AVIIF_KEYFRAME : 0);
	ie.dwChunkOffset = _movi_bytes;
	ie.dwChunkLength = f->length();
	_index.push_back(ie);
	_idx1_bytes += ie.size();
	_movi_bytes += siz + 8;
	
	return 1;
}


void
ivaWriterFast::ivaWriterThread::Run()
{
	while (!mStop)
		{
		stack_MutexLock sml(&mWriter.mBufferMutex);

		size_t	data_to_output;
		if (mWriter.mWritePtr >= mWriter.mReadPtr)
			{
			data_to_output = mWriter.mWritePtr - mWriter.mReadPtr;
			if (data_to_output < MINIMUM_WRITE)
				data_to_output = 0;
			}
		else
			{
			data_to_output = mWriter.mDynamicEndOfBuffer - mWriter.mReadPtr;
			assert((data_to_output % WRITE_BLOCK) == 0);
			}

		if (!data_to_output)
			{
			mWriter.mWaitingForData.Wait(mWriter.mBufferMutex);
			}
		else 
			{
			data_to_output &= WRITE_BLOCK_MASK;
			DWORD s;
			assert(data_to_output <= BUFFER_SIZE);
			int err = ::WriteFile(mWriter.mFileHandle,mWriter.mReadPtr,data_to_output,&s,NULL);
			assert(err);
			assert(s == data_to_output);
#if 0
			ffff << data_to_output << " " 
				<< ((long)mWriter.mReadPtr) << " " 
				<< ((long)mWriter.mWritePtr) << " " 
				<< ((long)mWriter.mBuffer) << " "
				<< ((long)mWriter.mDynamicEndOfBuffer) << " "
				<< ((long)mWriter.mEndOfBuffer) << " "
				<< endl;
#endif
			mWriter.mAmtWrittenToDisk += data_to_output;
			mWriter.mReadPtr += data_to_output;
			if (mWriter.mReadPtr == mWriter.mDynamicEndOfBuffer && mWriter.mWritePtr < mWriter.mReadPtr)
				mWriter.mReadPtr = mWriter.mBuffer;
			mWriter.mWaitingForSpace.Signal();
			}
		}
}
